c******************************************************************************
c***             Berechnung der Auslenkungsfelder nach der                  ***
c***                    Projektionsoperator-Methode                         ***
c******************************************************************************

c     Harald Ritter
c     Institut fuer Kristallographie
c     der Universitaet Tuebingen
c     Charlottenstrasse 33
c     Germany
c     Tel: +49(0)7071/29-76389
c     E-mail: harald.ritter@uni-tuebingen.de
c
c     Last modified: 22 Apr 1999
c
c     ----------------------------------------------------------------------
c
c     ### Erklaerung der verwendeten Felder und Parameter:
c
c     *** Allgemein ***
c
c     spgrp          : Raumgruppen-Symbol
c     inpfile        : Name der Eingabedatei
c     outfile1       : Name der Ausgabedatei
c     symfile        : Name der Datei mit den Symmetriekarten
c     titel          : Titel in der Ein- und Ausgabedatei 
c     intaus         : Zentrierung der Zelle (P, I, A, B, C, F, R)
c     laue           : Nummer der Lauegruppe (1-15)
c     clau(15)       : Symbole der Lauegruppe (1-15)
c     x0(3)          : Atomlage in der asymmetrischen Einheit
c     q(3)           : Wellenvektor in Einheiten der Gitterkonstanten
c     NSY            : Maximale Zahl der Symmetrieelemente
c     NATO           : Maximale Zahl der Atome in der primitiven Einheit
c     MAXLINE        : Maximale Laenge der Ausgabezeile
c
c     *** Raumgruppe G0 ***
c
c     num(NSY)       : Nummer des Symmetrieelementes (1-48(24))
c                      in der Reihenfolge der Sym.el. vor dem Ordnen
c     numsy0(NSY)    : Nummer des Symmetrieelementes (1-48(24))
c                      in der Reihenfolge der geordneten Sym.el.
c     csym0(NSY)     : Name des Sym.elementes (z.B.: 2, m, 1 , 1b, ...)
c     symor0(NSY)    : Orientierung des Symmetrieelementes (z.B.: [1,1,0])
c     ism0(3,3,NSY)  : Rotationsanteile der Symmetrieelemente von G0
c     tran0(3,NSY)   : Translationsanteile der Symmetrieelemente von G0
c     trp0(3,NSY)    : Echter Translationsanteil des Sym.el. von G0
c     trs(3,NSY)     : Hilfsfeld der Subroutine symsort
c     nsym0          : Anzahl der Symmetrieelemente der Raumgr. G0
c     natom0         : Zahl der Atome der spez. Lage x0
c     iat0(NSY)      : Nummer des Atoms, erzeugt durch Symmetrieelement n
c                      der speziellen Lage x0 der Raumgruppe G0
c     r0(3,NATO)     : Symmetrieaequivalente Positionen zu x0 
c     tp0(3,NATO)    : Translation zur Berechnung der Phase
c
c     *** Little Group Gq ***
c
c     laue_q         : Nummer der Lauegruppe von Gq
c     numsy(NSY)     : Nummer des Symmetrieelementes
c     csym(NSY)      : Name des Sym.elementes (z.B.: 2, m, 1 , 1b, ...)
c     symor(NSY)     : Orientierung des Symmetrieelementes (z.B.: [1,1,0])
c     ism(3,3,NSY)   : Rotationsanteile der Symmetrieelemente von Gq
c     tran(3,NSY)    : Translationsanteile der Symmetrieelemente von Gq
c     trp(3,NSY)     : Echter Translationsanteil des Sym.el. von Gq
c     nsym           : Anzahl der Symmetrieelemente der Raumgr. Gq
c     natom          : Zahl der Atome der spez. Lage x0 (little Group)
c     iat(NSY,NATO)  : Nummer des Atoms, erzeugt durch Symmetrieelement n
c                      der speziellen Lagen(n) der little group
c     r(3,NATO)      : Symmetrieaequivalente Positionen zu x0 (little group)
c     rs(3,NATO,NSY) : Durch die Symmetrieelemente de little group
c                      aus den Positionen r erzeugte Positionen
c     tp(3,NATO)     : Translation zur Berechnung der Phase (little group)
c     phase_tp(NATO) : Phasen der Atome
c     nlc            : Anzahl der Gitterkomplexe bezuegl. der Little Group
c     xs(3,NATO)     : Atompositionen der Gitterkomplexe in der Asymmetrische 
c                      Einheit bezuegl. der Little Group 
c     neat(NATO)     : Nummer des Gitterkomplexes, falls der Gitterkomplex r0
c                      in mehrere Gitterkomplexe bezgl. Gq zerfaellt
c     nuat(NATO)     : Nummer der Atome in r bezueglich r0
c     iwav(NSY)      : Symmetrieelement gehoert zur little group iwav=1, 
c                      sonst iwav=0
c
c     *** Irreduzible Darstellungen und Basisvektoren ***
c
c     IBS            : Maximale Zahl der Basisvektoren
c     IDM            : Maximale Dimension der irred. Darstellungen
c     IRD            : Maximale Zahl der irred. Darstellungen
c     idar           : Anzahl der irred. Darstellungen 
c     cmod(IRD)      : Name der irred. Darstellung
c     ndim(IRD)      : Dimension der irred. Darstellungen
c     mult(IRD)      : Multiplizitaet der irred. Darstellung
c     nbas(IDM,IRD)  : Anzahl der Basisvektoren pro irred. Darstellung
c     taug(NSY,IDM,IDM,IRD) : Matrixelemente der irred. Darstellungen 
c                      bezueglich der Generatoren
c     tau0(NSY,IDM,IDM,IRD) : Matrixelemente der irred. Darstellungen 
c                      bezueglich der Raumgruppe G0
c     tau(NSY,IDM,IDM,IRD) : Matrixelemente der irred. Darstellungen 
c                      bezuglich der Little Group Gq
c     ipm(NATO,NSY)  : Permutationsmatrix
c     nchi_red(NSY)  : Charakter der reduziblen Darstellungen
c     chi_ird(NSY,IRD) : Charakteren der irred. Darstellungen
c     mt(NATO,NATO,NSY) : Reduzible Matrix
c     p(3*NATO,IBS,IDM,IRD) : Basisvektoren der irred. Darstellungen
c     prj(3*NATO,3*NATO) : Projektionsoperatoren
c     mpt(NSY)       : Zuordnungsschema der Symmetrieelemente zu den
c                      Matrixelementen der irred. Darstellungen
c     c(3*NATO,3*NATO) : Matrix der Subroutine diagonal
c     bvsym(NSY)*144 : Symmetrieverhalten der Basisvektoren

      program promet

      parameter(ke = 2, ka = 3, PI = 3.14159265359) 
      parameter(IRD = 12, NSY = 48, NATO = 48, MAXLINE = 132)
      parameter(IBS = 144, IDM = 3, EPSY = 0.00001)

      external idcheck, laugrp, ctau, blkweg

      logical symmorph

      character*80 inpfile, outfile, symfile
      character spgrp*60, titel*80, intaus*1 
      character bvsym(3*NATO)*144, blkweg*144, bvtext(3*NATO)*9
      character csym(NSY)*3, symor(NSY)*9, cmod(IRD)*4
      character csym0(NSY)*3, symor0(NSY)*9, clau(15)*8, form*20
      character ctau*4, date*9, time*8, sg*3, cpgr*7, line*132

      integer ism0(3,3,NSY), iat0(NSY), numsy0(NSY), num(NSY)
      integer ism(3,3,NSY), iat(NSY,NATO), numsy(NSY)
      integer ipm(NATO,NSY), mt(NATO,NATO,NSY), nchi_red(NSY)
      integer neat(NATO), nuat(NATO), iwav(NSY)
      integer nbas(IDM,IRD), mult(IRD), ndim(IRD), mpt(NSY)

      real x0(3), r0(3,NATO), r(3,NATO), q(3), pr(3), phi(3)
      real tran0(3,NSY), trp0(3,NSY)
      real tran(3,NSY), trp(3,NSY), trs(3,NSY)
      real xs(3,NATO), tp(3,NATO), tp0(3,NATO), rs(3,NATO,NSY)

      complex c(3*NATO,3*NATO), cpp
      complex taug(NSY,IDM,IDM,IRD), tau0(NSY,IDM,IDM,IRD)
      complex tau(NSY,IDM,IDM,IRD), prj(3*NATO,3*NATO), phase_tp(NATO)
      complex chi_ird(NSY,IRD), p(3*NATO,IBS,IDM,IRD)


c --- Lauegruppen:

      data clau(1),clau(2),clau(3),clau(4),clau(5),clau(6),clau(7),
     *     clau(8),clau(9),clau(10),clau(11),clau(12),clau(13),
     *     clau(14),clau(15)
     * /'1b','2/m (C)','m m m','4/m','4/m m m','3b','3b m 1',
     *  '6/m','6/m m m','m 3','m 3b m','2/m (B)','3b (R)','3b m (R)',
     *  '3b 1 m'/


c --- Titel ausgeben:

      write(*,'(15(/2x,a))')
     *'+-----------------------------------------------------------+',
     *'|                                                           |',
     *'|                        P r o M e t                        |',
     *'|                                                           |',
     *'|                   V e r s i o n   1 . 0                   |',
     *'|                                                           |',
     *'|                 Last modified 22 Apr 1999                 |',
     *'|                                                           |',
     *'|                 H. Ritter, J. P. Ihringer                 |',
     *'| Institut fuer Kristallographie der Universitaet Tuebingen |',
     *'|       Charlottenstr. 33, D-72070 Tuebingen, Germany       |',
     *'|                                                           |',
     *'+-----------------------------------------------------------+'


c --- Datum und Uhrzeit ausgeben:

      call datum(date,time)
      write(*,'(/2x,a,4x,a)') date,time

c --- Eingabedatei einlesen:

      write(*,'(/2x,$a)') 'Please type the input filename: '
      read(5,'(a)') inpfile
      write(*,*)
      write(*,*)
      
      open(ke,file=inpfile,status='old',err=1200)
      call einles(ke,jobtyp,titel,spgrp,x0,q,csym0,intaus,icent,laue,
     *            ism0,tran0,nsym0,NSY,cmod,taug,ndim,idar,IDM,IRD)
      close(ke)

c --- Generatoren ausgeben:

      if(jobtyp.eq.2) then

        write(*,'(//2x,a)') 'Generators:'
        write(*,'(/2x,a,i4)') 'Number of generators = ',nsym0
        if(icent.eq.2) then
          write(*,'(2x,a)') 'This space group is centrosymmetric'
        else
          write(*,'(2x,a)') 'This space group is non-centrosymmetric'
        end if
        write(*,'(2x,a,a,4x,a,a)')
     *  'Int. Ausl. : ',intaus,'    Laue-group : ',clau(laue)
        write(*,'(/2x,a/)') 'S y m m e t r y   c a r d s'
        do 5 n=1,nsym0
           write(*,2) csym0(n),(real(ism0(1,i,n)),i=1,3),tran0(1,n),
     *                ((real(ism0(j,i,n)),i=1,3),tran0(j,n),j=2,3)
 2         format(2x,a,' :',8x,3f5.1,8x,f5.2,/,
     *            11x,'R=  ',3f5.1,'    T=  ',f5.2,/,
     *            15x,3f5.1,8x,f5.2,/)
 5      continue

c --- Irreduzible Darstellungen zu den Generatoren ausgeben:

        nv = nsym0/10
        nr = nsym0 - 10*nv
        do 5360 k = 1,nv
           ibeg = (k-1)*10 + 1 
           iend = k*10
           write(*,'(/2x,a,8x,10(3x,a3))') 'IR',(csym0(n),n=ibeg,iend)
           do 5350 m=1,idar
              do 5340 i=1,ndim(m)
                 do 5330 j=1,ndim(m)
                    write(*,'(2x,a4,1x,2i1,5x,10(2x,a4))') 
     *                   cmod(m),i,j,(ctau(taug(n,j,i,m)),n=ibeg,iend)
5330             continue
5340          continue
5350       continue
5360    continue
        if(nr.gt.0) then
          ibeg = nsym0 - nr + 1
          iend = nsym0
          write(*,'(/2x,a,8x,10(3x,a3))') 'IR',(csym0(n),n=ibeg,iend)
          do 5400 m=1,idar
             do 5395 i=1,ndim(m)
                do 5390 j=1,ndim(m)
                   write(*,'(2x,a4,1x,2i1,5x,10(2x,a4))') 
     *                  cmod(m),i,j,(ctau(taug(n,j,i,m)),n=ibeg,iend)
5390            continue
5395         continue
5400      continue
        end if
        write(*,'(/2x,a,1x,a)') 'w = exp(2*PI*i/3),',
     *  's = sin(2*PI/6) = 1/2*sqrt(3), c = cos(2*PI/6) = 1/2'

      end if

c === Die Raumgruppe:

c --- Symmetriekarten bestimmen:

      if(jobtyp.eq.2) then
        call spggen(csym0,ism0,tran0,nsym0,NSY,taug,idar,ndim,IDM,IRD)
      else
        call symint(spgrp,intaus,laue,icent,ism0,tran0,nsym0)
        if(icent.eq.2) then
          do 17 n=1,nsym0
             ns = nsym0 + n
             do 16 i=1,3
                tr = -tran0(i,n)
                nt = int(tr-EPSY)
                if(tr.lt.-EPSY) nt = nt - 1
                tran0(i,ns) = tr - nt
                do 15 j=1,3
                   ism0(j,i,ns) = -ism0(j,i,n)
 15             continue
 16          continue
 17       continue
          nsym0 = 2*nsym0
        end if
      end if

c --- Kristallsystem bestimmen:

      nc = 1
      if(laue.eq.6.or.laue.eq.7.or.laue.eq.8.or.
     *   laue.eq.9.or.laue.eq.15) nc = 2

c --- Symmetrieelemente sortieren und ihnen Symbole zuordnen, 
c     sowie den Translationsanteil trp0 einer Schraubenachse oder
c     Gleitspiegelebene ermitteln:

      call symsort(nc,csym0,symor0,ism0,tran0,trp0,trs,num,numsy0,
     *             nsym0,NSY)

c --- Matrixelemente in der Reihenfolge der Symmetrieelemente sortieren:

      if(jobtyp.eq.2) then
        do 23 k=1,nsym0
           do 22 l=1,nsym0
              if(num(l).eq.numsy0(k)) then
                do 21 m=1,idar
                   do 20 i=1,ndim(m)
                      do 19 j=1,ndim(m)
                         tau0(k,j,i,m) = taug(l,j,i,m)
 19                   continue
 20                continue
 21             continue
              end if
 22        continue
 23     continue
      end if

c --- Feld "bvsym" initialisieren:

      do 25 k=1,3*NATO
         bvsym(k) = ' '
 25   continue

c --- Ausgabe der Symmetrieelemente der Raumgruppe:

      write(*,'(//2x,a)') 'Space group:'
      write(*,'(/2x,a,i4)') 'Number of symmetry elements = ',nsym0
      if(icent.eq.2) then
        write(*,'(2x,a)') 'This space group is centrosymmetric'
      else
        write(*,'(2x,a)') 'This space group is non-centrosymmetric'
      end if
      write(*,'(2x,a,a,4x,a,a)')
     *'Int. Ausl. : ',intaus,'    Laue-group : ',clau(laue)
      write(*,'(/2x,a/)') 'S y m m e t r y   c a r d s'
      do 35 n=1,nsym0
         write(*,32) csym0(n),symor0(n),
     *         (real(ism0(1,i,n)),i=1,3),tran0(1,n),trp0(1,n),
     *        ((real(ism0(j,i,n)),i=1,3),tran0(j,n),trp0(j,n),j=2,3)
32       format(2x,a,1x,a,' :',8x,3f5.1,8x,f5.2,8x,f5.2,/,
     *          21x,'R=  ',3f5.1,'    T=  ',f5.2,'    Tp= ',f5.2,/,
     *          25x,3f5.1,8x,f5.2,8x,f5.2/)
35    continue

c --- Aus x0 symmetrieaequivalente Positionen r0 berechnen:

      call posgen(x0,r0,tp0,iat0,natom0,NATO,intaus,ism0,tran0,
     *            nsym0,NSY)

      write(*,'(/2x,a/)') 'Atoms of the unit cell:'
      write(*,'(2x,a)') 'No.       x/a      y/b      z/c'
      do 36 k=1,natom0
         write(*,'(2x,i3,3x,3(f9.4))') k,(r0(i,k),i=1,3)
36    continue


c === Die Raumgruppe des Wellenvektors:

c --- Symmetrieelemente, die den Wellenvektor in einen aequivalenten
c     Vektor ueberfuehren, bestimmen und die ensprechenden Felder
c     der Raumgruppe des Wellenvektors auffuellen:

      nq = 0
      nons = 0
      symmorph = .true.
      do 50 n=1,nsym0
         call qvekinv(iwav(n),q,ism0(1,1,n),intaus)
         if(iwav(n).eq.1) then
           nq = nq + 1
           csym(nq) = csym0(n)
           symor(nq) = symor0(n)
           numsy(nq) = numsy0(n)
           do 41 i=1,3
              trp(i,nq) = trp0(i,n)
              tran(i,nq) = tran0(i,n)
              if(abs(trp(i,nq)).gt.EPSY) nons = nons + 1
              do 40 j=1,3
                 ism(j,i,nq) = ism0(j,i,n)
40           continue
41         continue
           if(jobtyp.eq.2) then
             do 45 m=1,idar
                do 44 i=1,ndim(m)
                   do 43 j=1,ndim(m)
                      tau(nq,j,i,m) = tau0(n,j,i,m)
43                 continue
44              continue
45           continue
           end if
         end if
50    continue
      nsym = nq
      if(nons.gt.0) symmorph = .false.

c --- Punktgruppe zur Raumgruppe des Wellenvektors:

      call pktgrp(npg,cpgr,laue,numsy,nsym,NSY)

c --- Lauegruppe zur Raumgruppe des Wellenvektors:

      laue_q = laugrp(nc,npg,symor0,NSY)

c --- Die Raumgruppe des Wellenvektors ausgeben:

      write(*,'(///2x,a)') 'Group of the wave vector (little group):'
      write(*,'(/2x,a,a)') 'Point group: ',cpgr
      if(symmorph) then 
        write(*,'(2x,a)') 'This space group is symmorphic'
      else
        write(*,'(2x,a)') 'This space group is non-symmorphic'
      end if
      write(*,'(2x,a,i4)') 'Number of symmetry elements = ',nsym
      write(*,'(2x,a,a,4x,a,a)')
     *'Int. Ausl. : ',intaus,'    Laue-group : ',clau(laue_q)
      write(*,'(/2x,a/)') 'S y m m e t r y   c a r d s'
      do 54 n=1,nsym
         write(*,32) csym(n),symor(n),
     *               (real(ism(1,j,n)),j=1,3),tran(1,n),
     *               trp(1,n),((real(ism(i,j,n)),j=1,3),
     *               tran(i,n),trp(i,n),i=2,3)
54    continue

c --- Positionen r0 in Gitterkomplexe zerlegen:

      call latcmp(xs,nlc,neat,r0,natom0,NATO,intaus,ism,tran,nsym,NSY)


c === Ausgabedateien (SIMREF-Inputfiles) anlegen:

      call outfil(outfile,inpfile,'mod',laenge,0)
      open(ka,file=outfile,status='unknown',err=1300)

c --- Titel schreiben:

      line = ' '
      line = 'TIT> '//titel  
      call writline(ka,line,MAXLINE)

c --- Eingabezeilen schreiben:

      write(ka,'(/a,/a)') 'Base vectors:','BAS>'
      write(ka,'(/a,/a)') 'Parameters:','PAR>'

c --- Wellenvektor:

      write(ka,'(/a,3f8.4)') 'Wave vector: ',(q(i),i=1,3)

c --- Atompositionen schreiben:

      write(ka,'(/a)') 'Positions and phases:'

      if(nlc.gt.1) then
        write(ka,'(5x,a,a)') 'No.    x/a      y/b      z/c',
     *  '        t_x      t_y      t_z      LC'
        do 70 k=1,natom0
           write(ka,'(a,i4,3f9.5,2x,3f9.5,i6)') 
     *     'POS>', k, (r0(i,k),i=1,3), (tp0(i,k),i=1,3), neat(k)
 70     continue
      else
        write(ka,'(5x,a)') 
     *  'No.    x/a      y/b      z/c        t_x      t_y      t_z'
        do 75 k=1,natom0
           write(ka,'(a,i4,3f9.5,2x,3f9.5)') 
     *     'POS>', k, (r0(i,k),i=1,3), (tp0(i,k),i=1,3)
 75     continue   
      end if


c === Schleife ueber die Gitterkomplexe:

      na = 0
      nbv = 0
      do 800 nn=1,nlc 

         if(nlc.gt.1) 
     *     write(*,'(//2x,a,i2,/2x,a)') 'Lattice complex ',nn,
     *                                  '=================='

c --- Aus xs symmetrieaequivalente Positionen r berechnen:

         call posgen(xs(1,nn),r,tp,iat(1,nn),natom,NATO,intaus,ism,
     *               tran,nsym,NSY)

c --- Atompositionen r bezueglich der little group den
c     Positionen r0 der Raumgruppe G0 zuordnen:

         do 85 k=1,natom
            do 80 l=1,natom0
               if(abs(r0(1,l)-r(1,k)).lt.EPSY.and.
     *            abs(r0(2,l)-r(2,k)).lt.EPSY.and.
     *            abs(r0(3,l)-r(3,k)).lt.EPSY) nuat(k) = l
 80         continue
 85      continue

c --- Phasen berechnen:

         do 90 k=1,natom
            arg = 0.
            do 88 i=1,3
               arg = arg + tp(i,k) * q(i)
 88         continue
               arg = 2. * PI * arg
               phase_tp(k) = cmplx(cos(arg),sin(arg))
 90      continue

c --- Positionen und Phasen ausgeben:

         write(*,'(/2x,a/)') 'Positions and phases:'
         write(*,'(2x,a,a)') 
     *'No.       x/a      y/b      z/c       translation',
     *'         phase(T)'
         do 95 k=1,natom
            if(aimag(phase_tp(k)).ge.0.) then
              sg = ' + '
            else
              sg = ' - '
            end if
            write(*,'(2x,i3,3x,3f9.4,3x,3f5.2,3x,f6.3,a,f6.3,a)') 
     *      k,(r(i,k),i=1,3),(tp(i,k),i=1,3),
     *      real(phase_tp(k)),sg,abs(aimag(phase_tp(k))),'*i'
 95      continue


c === Die reduziblen Darstellungen:

         write(*,'(//2x,a)') 
     *        'Permutations and the reducible matrices T:'

c --- Permutationsmatrix bestimmen und ausgeben:

         call permat(ipm,rs,r,natom,NATO,intaus,ism,tran,
     *               nsym,NSY)
         write(*,'(/2x,a)') 
     *   'Symm. element  atom    atom''     position of atom'' '
         do 200 n=1,nsym
            write(*,'(/2x,a3,1x,a9,2x,i3,2x,a,i3,3x,3f8.3)') 
     *      csym(n),symor(n),1,'-->',ipm(1,n),(rs(i,1,n),i=1,3)
            do 190 k=2,natom
               write(*,'(17x,i3,2x,a,i3,3x,3f8.3)') 
     *         k,'-->',ipm(k,n),(rs(i,k,n),i=1,3)
190         continue
200      continue    

c --- Reduzible Darstellungen mt ,deren Character nchi_red und die
c     Phasen in der reduziblen Darstellungen berechnen:

         call redmat(ipm,ism,mt,nchi_red,natom,nsym,NATO,NSY)

c === Die irreduziblen Darstellungen:

         write(*,'(//2x,a)') 'Irreducible representations (IR):'

c --- Die irreduziblen Darstellungen bestimmen:

         if(jobtyp.eq.1)
     *     call irred(npg,cmod,tau,nsym,ndim,idar,mpt,numsy,NSY,IDM,IRD)
         if(idar.eq.0) then
           write(*,'(2x,a)') 'ERROR: No irred. representations'
           stop
         else
           write(*,'(/2x,a,i4)') 
     *          'Number of irred. representations = ',idar
         end if

c --- Orthogonalitaet der Darstellungen ueberpruefen:

         ierr = idcheck(cmod,tau,nsym,ndim,idar,NSY,IDM,IRD)
         if(ierr.ne.0) stop

c --- Charakteren und Multiplizitaeten der irred. Darstellungen bestimmen:

         mlts = 0
         do 250 m=1,idar
            call darmult(cmod(m),tau(1,1,1,m),nchi_red,chi_ird(1,m),
     *                   mult(m),nsym,ndim(m),NSY,IDM)
            mlts = mlts + mult(m) * ndim(m)
250      continue

c --- Ausgabe der irred. Darstellungen:

         nv = nsym/6
         nr = nsym - 6*nv
         do 360 k = 1,nv
            ibeg = (k-1)*6 + 1 
            iend = k*6
            write(*,'(/18x,6(6x,a3,1x))') (csym(n),n=ibeg,iend)
            write(*,'(2x,a,1x,6(a9,1x))') 
     *           'IR     dim mult',(symor(n),n=ibeg,iend)
            do 350 m=1,idar
               if(ndim(m).eq.1) then
                 write(*,'(2x,a4,a,1x,i3,1x,i4,3x,6(5x,a4,1x))') 
     *                cmod(m),'11',ndim(m),mult(m),
     *                (ctau(tau(n,1,1,m)),n=ibeg,iend)
               else
                 write(*,'(2x,a4,3x,i3,1x,i4,1x,6(i9,1x))') 
     *                cmod(m),ndim(m),mult(m),
     *                (nint(real(chi_ird(n,m))),n=ibeg,iend)
                 do 340 i=1,ndim(m)
                    do 330 j=1,ndim(m)
                       write(*,'(2x,a4,2i1,12x,6(5x,a4,1x))') 
     *                 cmod(m),i,j,(ctau(tau(n,j,i,m)),n=ibeg,iend)
330                 continue
340              continue
               end if  
350         continue
            write(*,'(2x,a,10x,6(i9,1x))') 
     *           'chi(T)',(nchi_red(n),n=ibeg,iend)
360      continue

         if(nr.gt.0) then
           ibeg = nsym - nr + 1
           iend = nsym
           write(*,'(/18x,6(6x,a3,1x))') (csym(n),n=ibeg,iend)
           write(*,'(2x,a,1x,6(a9,1x))') 
     *          'IR     dim mult',(symor(n),n=ibeg,iend)
           do 400 m=1,idar
              if(ndim(m).eq.1) then
                write(*,'(2x,a4,a,1x,i3,1x,i4,3x,6(5x,a4,1x))')
     *          cmod(m),'11',ndim(m),mult(m),
     *          (ctau(tau(n,1,1,m)),n=ibeg,iend)
              else
                write(*,'(2x,a4,3x,i3,1x,i4,1x,6(i9,1x))') 
     *          cmod(m),ndim(m),mult(m),
     *          (nint(real(chi_ird(n,m))),n=ibeg,iend)
                do 395 i=1,ndim(m)
                   do 390 j=1,ndim(m)
                      write(*,'(2x,a4,2i1,12x,6(5x,a4,1x))') 
     *                cmod(m),i,j,(ctau(tau(n,j,i,m)),n=ibeg,iend)
390                continue
395             continue
              end if
400        continue
           write(*,'(2x,a,10x,6(i9,1x))') 
     *          'chi(T)',(nchi_red(n),n=ibeg,iend)
         end if

         write(*,'(/2x,a,1x,a)') 'w = exp(2*PI*i/3),',
     *   's = sin(2*PI/6) = 1/2*sqrt(3), c = cos(2*PI/6) = 1/2'

c --- Warnung:

         if(mlts.ne.3*natom) then
           write(*,'(/2x,a,/2x,a)') 
     *     'WARNING: the sum of the multiplicities is not equal to',
     *     'the degrees of freedom!'
         end if

c --- Projektionsoperatoren und Basisvektoren bestimmen:

         write(*,'(//2x,a)') 'Calculation of the base vectors:'
         nb = 0
         write(*,*)
         do 500 m=1,idar
            nbs = 0
            do 490 k=1,ndim(m)
               call proj(prj,mt,natom,NATO,tau(1,k,k,m),ndim(m),
     *                   ism,nsym,NSY)
               call diagonal(prj,c,nbas(k,m),3*natom,3*natom,
     *                       3*NATO,3*NATO)
               do 480 l=1,nbas(k,m)
                  do 470 i=1,3*natom
                     p(i,l,k,m) = prj(i,l)
 470              continue
 480           continue
               write(*,'(2x,a,2i1,a,i4)') 
     *         cmod(m),k,k,': number of base vectors = ',nbas(k,m)
               nb = nb + nbas(k,m)
               nbs = nbs + nbas(k,m)
490         continue
            if(nbs.ne.mult(m)*ndim(m)) then
              write(*,'(2x,a,a)')
     *        'WARNING: the number of base vectors is not equal to',
     *        ' the multiplicity!'
            end if
500      continue
         write(*,'(/2x,a,i4)') 'Total number of base vectors   = ',nb 
         if(nb.ne.3*natom) then
           write(*,'(/2x,a,/2x,a)')
     *     'WARNING: the number of base vectors is not equal to',
     *     'the degrees of freedom!'
         end if 

c --- Basisvektoren ausgeben:

         if(nlc.gt.1) then
           write(ka,'(/a,i2,a)') 
     *     'Base vectors of lattice complex ',nn,':'
         else
           write(ka,'(/a)') 'Base vectors:'
         end if

         if(nb.gt.0) then
           write(*,'(//2x,a)') 'Base vectors:'
           write(*,'(/2x,a)') 
     *  'ID      Nr.  atom       absolute value                  phase' 
           do 700 m=1,idar
              do 650 n=1,ndim(m)
                 do 640 l=1,nbas(n,m)

c --- Ausgabe auf File (ohne Phase):

                    write(ka,'(a,1x,a,2i3,3x,i4,7x,2f10.5)') 
     *              'MOD>',cmod(m),10*n+n,l,nbv+l,0.,0.
                    do 615 k=1,natom
                       do 610 i=1,3
                          call cxphs(p(3*(k-1)+i,l,n,m),pr(i),phi(i))
 610                   continue
                       write(ka,'(a,1x,i3,3(1x,f8.5),1x,3(1x,f7.2))')
     *                 'VEC>',nuat(k),(pr(i),i=1,3),
     *                 (180./PI*phi(i),i=1,3)
 615                continue

c --- Bildschirmausgabe (mit Phase):

                    do 620 i=1,3
                       call cxphs(p(i,l,n,m),pr(i),phi(i))
 620                continue
                    write(*,'(/2x,a,2i1,1x,i3,2x,i4,1x,3(1x,f8.5),
     *              1x,3(1x,f7.2))') cmod(m),n,n,l,1,
     *              (pr(i),i=1,3),(180./PI*phi(i),i=1,3)
                    do 630 k=2,natom
                       do 625 i=1,3
                          cpp = phase_tp(k) * p(3*(k-1)+i,l,n,m) 
                          call cxphs(cpp,pr(i),phi(i))
 625                   continue
                       write(*,'(14x,i4,1x,3(1x,f8.5),1x,3(1x,f7.2))') 
     *                 k,(pr(i),i=1,3),(180./PI*phi(i),i=1,3)
 630                continue

c --- "Ueberlebende" Symmetrieelemente bestimmen:

                    nq = 0
                    do 635 k=1,nsym0
                       if(iwav(k).eq.1) then
                          nq = nq + 1
                          ii = 3 * (nq - 1)
                          if(ctau(tau(nq,n,n,m)).eq.' 1  ') then
                             if(ii.gt.0) bvsym(nbv+l)(ii:ii) = ','
                             write(bvsym(nbv+l)(ii+1:ii+2),'(i2)') k
                          else
                             bvsym(nbv+l)(ii:ii+2) = '    '
                          end if
                       end if
 635                continue
                    bvtext(nbv+l)(1:4) = cmod(m)
                    write(bvtext(nbv+l)(5:9),'(2i1,i3)') n,n,l

 640             continue
                 nbv = nbv + nbas(n,m)
 650          continue
 700       continue
         end if

         na = na + natom

 800  continue

c --- "Ueberlebende" Symmetrieelemente bestimmen:

      write(ka,'(/a)') 'Symmetry of the base vectors:'
      do 850 n=1,nbv
         bvsym(n) = blkweg(bvsym(n),laenge)
         form = '(3a,a000)'
         write(form(6:8),'(i3)') laenge
         write(ka,form) 'BVS  ',bvtext(n),' > ',bvsym(n)
 850  continue

c --- Symmetrieelemente ausgeben:

      write(ka,'(/a)') 'Symmetry elements:'
      do 900 n=1,nsym0
         if(n.lt.10) then
            write(ka,'(a,i1,a,1x,a,1x,a)') 
     *           'SYM  ',n,' >', csym0(n), symor0(n)
         else
            write(ka,'(a,i2,a,1x,a,1x,a)') 
     *           'SYM ',n,' >', csym0(n), symor0(n)
         end if
 900  continue

c --- Name der Ausgabe-Datei ausgeben:

      write(*,*)
      write(*,*)
      line = ' '
      line = '  Output written on file '//outfile
      call writline(6,line,MAXLINE)
      close(ka)


c === Datei mit den Symmetriekarten anlegen:

      call outfil(symfile,inpfile,'sym',laenge,0)
      open(ka,file=symfile,status='unknown',err=1300)

c --- Titel schreiben:

      line = ' '
      line = 'TIT> '//titel  
      call writline(ka,line,MAXLINE)

c --- Integrales Ausloeschungs-Symbol, Nr. der Lauegruppe schreiben

      icent = 1
      write(ka,'(/a)') 'Extin. rule, Laue group, centrosymmetry:'
      write(ka,'(a,2x,a,2x,i2,3x,i1)') 'COD>',intaus, laue, icent

c --- Symmetrieelemente schreiben:

      write(ka,'(/a)') 'Symmetry elements:'
      do 960 n=1,nsym0
         write(ka,'(a,1x,a,1x,3(1x,3i3),1x,3(1x,f5.2))') 
     *   'SYM>',csym0(n),((ism0(j,i,n),j=1,3), i=1,3),
     *   (tran0(i,n),i=1,3)
 960  continue

c --- Name der Ausgabe-Datei ausgeben:

      write(*,*)
      line = ' '
      line = '  Symmetry cards written on file '//symfile
      call writline(6,line,MAXLINE)
      close(ka)


c --- Datum und Uhrzeit ausgeben:

      call datum(date,time)
      write(*,'(/2x,a,4x,a/)') date,time


c --- Fehlermeldungen:

      goto 9999
1200  stop 'Cannot open INPUT file'
1300  stop 'Cannot open OUTPUT file'

c --- Ende:

9999  continue
      end


c******************************************************************************
c***    Bildschirmausgabe eines Matrixelementes bestimmen                   ***
c******************************************************************************

      character*4 function ctau(tau)

      parameter(EPSY = 0.00001, PI = 3.14159265359)
      complex tau

      zr = real(tau)
      zi = aimag(tau)

      dr = sqrt(zr*zr + zi*zi)
      if(abs(zr).lt.EPSY.and.abs(zi).lt.EPSY) then
        idph = 0
      else 
        idph = nint(180./PI*atan2(zi,zr))
      end if
      if(idph.lt.0) idph = idph + 360

      if(abs(dr).lt.EPSY) then
        ctau = ' 0'
      else if(abs(dr-1.).lt.EPSY) then
        if(idph.eq.0) then
          ctau = ' 1'
        else if(idph.eq.60) then
          ctau = '-w* '
        else if(idph.eq.90) then
          ctau = ' i'
        else if(idph.eq.120) then
          ctau = ' w'
        else if(idph.eq.180) then
          ctau = '-1'
        else if(idph.eq.240) then
          ctau = ' w* '
        else if(idph.eq.270) then
          ctau = '-i'
        else if(idph.eq.300) then
          ctau = '-w'
        end if                       
      else if(abs(dr-0.5).lt.EPSY) then
        if(idph.eq.180) then
          ctau = '-c'
        else
          ctau = ' c'
        end if
      else if(abs(dr-0.866025403784).lt.EPSY) then
        if(idph.eq.180) then
          ctau = '-s'
        else
          ctau = ' s'
        end if  
      else
        ctau = '??'
      end if

      return
      end


c******************************************************************************
c***             Betrag und Phase einer komplexen Zahl z bestimmen          ***
c******************************************************************************

      subroutine cxphs(z,r,phi)

      parameter(PI = 3.14159265359, EPSY = 0.00001)

      real r, phi
      complex z

      x = real(z)
      y = imag(z)

      r = sqrt(x*x + y*y)

      if(abs(y).lt.EPSY) then
        if(x.gt.-EPSY) then
          phi = 0.
        else
          phi = PI
        end if
      else
        phi = atan2(y,x)
        if(phi.lt.-EPSY.or.phi.gt.2*PI-EPSY) then
          np = int(phi/2./PI)
          if(phi.lt.-EPSY) np = np - 1
          phi = phi - 2*PI * np
        end if
      end if
      
      return
      end


c******************************************************************************
c***              Blanks aus einem String entfernen                         ***
c******************************************************************************

      character*(*) function blkweg(string,n)

      character*(*) string
      
      n = 0
      do 100 i=1,len(string)
         if(string(i:i).ne.' ') then
            n = n + 1
            blkweg(n:n) = string(i:i)
         end if
 100  continue

      return
      end



c******************************************************************************
c***                    Matrix diagonalisieren                              ***
c******************************************************************************

c --- Harald Ritter, J. Ihringer
c     Version vom 22.07.1996

      subroutine diagonal(a,c,irang,nzeil,nspalt,NZE,NSP)

      parameter(EPSY = 0.00001)
      logical ende
      complex a(NZE,NSP), c(NZE,NSP), temp, div, cmlt


c --- Matrix a nach Matrix c kopieren:

      do 3 ia=1,nzeil
         do 2 ka=1,nspalt
            c(ia,ka) = a(ia,ka)
 2       continue
 3    continue

c --- Matrix c diagonalisieren:

      irang = 0
      ende  = .false.

      do 100 id=1,nspalt

         do 20 i=id,nspalt
            do 10 j=id,nzeil
               if(abs(real(c(j,i))).gt.EPSY.or.
     *            abs(aimag(c(j,i))).gt.EPSY) goto 50
 10         continue
 20      continue
         ende = .true.

 50      if(ende) then

           goto 111

         else

           irang = irang + 1

c --- Spalten id und i der Matizen a und c tauschen:

           if(i.ne.id) then
             do 60 k=1,nzeil
                temp    = c(k,id)
                c(k,id) = c(k,i)
                c(k,i)  = temp
                temp    = a(k,id)
                a(k,id) = a(k,i)
                a(k,i)  = temp
 60          continue
           end if

c --- Zeilen id und j der Matrix c tauschen:

           if(j.ne.id) then
             do 70 k=1,nspalt
                temp    = c(id,k)
                c(id,k) = c(j,k)
                c(j,k)  = temp
 70          continue
           end if

c --- Matrixelement c(id,id) auf eins normieren, d.h. die Zeile id durch
c     das Matrixelement c(id,id) dividieren:

           div = c(id,id)
           do 80 k=1,nspalt
              c(id,k) = c(id,k)/div
 80        continue

c --- Matrixelemente c(in,id) der Spalte id fuer j ungleich id auf Null setzen,
c     d.h. das c(in,id)-fache der Zeile id wird von Zeile in abgezogen,
c     um c(in,id) = 0 zu erzeugen:

           do 95 in=1,nzeil
              if(in.ne.id) then
                if(abs(real(c(in,id))).gt.EPSY.or.
     *             abs(aimag(c(in,id))).gt.EPSY) then
                  cmlt = c(in,id)
                  do 90 k=1,nspalt          
                     c(in,k) = c(in,k) - cmlt * c(id,k)
 90               continue
                end if
              end if
 95        continue

         end if

 100  continue

 111  return
      end
c******************************************************************************
c***    Character und Multiplizitaet einer irred. Darstellung berechnen     ***
c******************************************************************************

      subroutine darmult(cmod,tau,nchi_red,chi_ird,mult,nsym,ndim,
     *                   NSY,IDM)

      parameter(EPSY = 0.00001)
      character cmod*4
      integer nchi_red(NSY)
      complex tau(NSY,IDM,IDM), chi_ird(NSY), ch, cm

      cm = 0.
      do 40 n=1,nsym
         ch = 0.
         do 30 i=1,ndim
               ch = ch + tau(n,i,i)
30       continue
         chi_ird(n) = ch
         cm = cm + nchi_red(n) * conjg(ch)
40    continue
      cm = cm/real(nsym)
      if(abs(aimag(cm)).gt.0.0001) then
        write(*,'(/2x,a,a4,a)')
     *  'ERROR: the multiplicity of irred. Rep. ',cmod,
     *  ' is a complex expression.'
        stop
      else
        if(abs(amod(real(cm),1.)).gt.EPSY) then
          write(*,'(/2x,a,a4,a)')
     *    'ERROR: the multiplicity of irred. Rep. ',cmod,
     *    ' is not an integer expression.'
          stop
        else
          if(nint(real(cm)).lt.0) then
            write(*,'(/2x,a,a4,a)')
     *      'ERROR: the multiplicity of irred. Rep. ',cmod,
     *      ' is negative.'
            stop
          else
            mult = nint(real(cm))
          end if
        end if
      end if

      return
      end
c******************************************************************************
c***          Eine Zeile einer Datei mit oder ohne Code einlesen            ***
c******************************************************************************

      subroutine getinp(ke,code,ecod,werte,char,max,num,nn,ist)

c     ke:      Kanalnummer
c     code:    Zeilencode, z. B. 'TIT'
c              code = '   ': Zeile ohne Code einlesen
c     ecod:    Code, bei dem das Einlesen abgebrochen wird
c     char:    Daten als Character-String abgespeichert
c     werte:   Daten als Realwerte abgespeichert
c     ist:     Eingabe: = 0 : Zeichenkette einlesen --> char
c                       = 1 : Daten einer Zeile als Realwerte (werte) und
c                             zusaetzlich als character (char) abspeichern 
c              Ausgabe: = 11 : Codewort nicht gefunden!
c                       = 13 : Ende beim Einlesen gefunden!
c                       = 15 : Zu viele Werte in der Eingabezeile (num > max)!
c     nn:      Zeilennummer
c     line:    Eingelesene Eingabe-Zeile
c              (einschliesslich aller Fortsetzungszeilen)
c     LMAX     Maximale Anzahl der Zeichen pro Zeile 
c     MAXLINE: Maximale Anzahl der Zeichen pro Eingabe-Zeile 
c              (einschliesslich der Fortsetzungszeilen)
c     max:     Maximale Anzahl der Werte pro Eingabe-Zeile
c     num:     Anzahl der gefundenen Werte pro Eingabe-Zeile


      parameter(MAXLINE = 500, LMAX = 250)
      real werte(max)
      character zeile*(LMAX), code*3, ecod*3, char(max)*132
      character line*(MAXLINE)


c -- Initialisierung:

      ibeg = 1

      do 5 k=1,max
         char(k)  = ' '
         werte(k) = 0.
5     continue

 10   k = 0
      line = ' '
      num = 0

c --- Zeile einlesen:

 11   zeile = ' '
      read(ke,'(a)',end=1100) zeile
c      write(*,'(a,a)') 'Z = ',zeile(1:70)
      nn = nn + 1

c --- Nach Fortsetzungszeilen suchen.

      do 20 i=1,LMAX
         if(zeile(i:i).eq.'\\') then
           if(i.eq.1) then
             goto 11
           else
             if(zeile(i-1:i-1).eq.' ') goto 11
           end if
         end if
         k = k + 1
         if(k.gt.MAXLINE) goto 30
         line(k:k) = zeile(i:i)
 20   continue

c --- Nach der Kennung suchen:

 30   if(code.ne.'   ') then

        if(ecod.ne.'eee'.and.line(1:3).eq.ecod) then
          backspace(ke)
          goto 1300
        end if
        if(line(1:3).ne.code(1:3)) goto 10

c --- ">" suchen:

        do 100 i=1,MAXLINE
           if(line(i:i).eq.'>') then
             ibeg = i + 1
             goto 110
           end if
100     continue
        stop 'ERROR: ">" is missing'

      end if

c --- Anfang suchen:

110   do 120 i=ibeg,MAXLINE
         if(line(i:i).ne.' ') then
           ibeg = i
           goto 130
         end if
120   continue    
      return 

c --- Daten der Zeile abspeichern:

130   if(ist.eq.0) then
        char(1)(1:132) = line(ibeg:ibeg+132)
      else
        ic    = 0
        n     = 0
        num   = 1
        ibl   = 0
        komma = 0
        do 250 i=ibeg,MAXLINE
           if(line(i:i).eq.' ') then
             ibl = ibl + 1
           else if(line(i:i).eq.',') then
             komma = 1
             num = num + 1
             n = 0
           else
             if(ibl.gt.0.and.komma.eq.0) then
               num = num + 1
               n = 0
             end if
             ic = ic + 1
             n = n + 1
             char(num)(n:n) = line(i:i)
             ibl = 0
             komma = 0
           end if
250     continue
        if(num.gt.max) goto 1500
        do 300 l=1,num
           read(char(l)(1:15),'(bn,f15.0)',err=300) werte(l)
300     continue
      end if

      return

c --- Fehlermeldungen und Warnungen zurueckgeben:
 
1100  ist = 11
      return
1300  ist = 13
      return
1500  ist = 15
      return

      end


c******************************************************************************
c***    Orthogonalitaet der irred. Darstellungen ueberpruefen               ***
c******************************************************************************

      integer function idcheck(cmod,tau,nsym,ndim,idar,NSY,IDM,IRD)

      parameter(EPSY = 0.00001)
      integer ndim(IRD)
      character cmod(IRD)*4
      complex tau(NSY,IDM,IDM,IRD), sum

      nerr = 0
      do 500 mu=1,idar
         do 400 nu=1,idar
            ierr = 0
            do 300 i=1,ndim(mu)
               do 200 j=1,ndim(nu)
                  do 180 l=1,ndim(mu)
                     do 150 m=1,ndim(nu)
                        sum = 0.
                        do 100 n=1,nsym
                           x = real(tau(n,m,j,nu))
                           y = aimag(tau(n,m,j,nu))
                           sum = sum + tau(n,l,i,mu) * cmplx(x,-y)
 100                    continue
                        if(abs(aimag(sum)).gt.EPSY) ierr = 1
                        if(mu.eq.nu.and.i.eq.j.and.l.eq.m) then
                          if(nint(real(sum)).ne.nsym/ndim(mu)) ierr = 1
                        else
                          if(nint(real(sum)).ne.0) ierr = 1
                        end if
                        if(ierr.eq.1) nerr = nerr + 1
 150                 continue
 180              continue
 200           continue
 300        continue
            if(ierr.eq.1) then
              write(*,'(2x,5a)') 'ERROR: ',
     *        cmod(mu),' - ',cmod(nu),': matrices not orthogonal'
            end if
 400     continue
 500  continue

      idcheck = nerr
      return
      end
C******************************************************************************
C***            Name der Ausgabe-Datei erzeugen                             ***
C******************************************************************************

      subroutine outfil(name,file,filspe,laenge,nr)

      parameter(MAX = 80)
      character filspe*3, file*80, name*80, form*4


c --- Initialisierung:

      name = ' '

c --- Zehnerpotenz der Zahl NR bestimmen:

      nz = 0
      if(nr.gt.0) nz = int(alog10(real(nr))) + 1

c --- Ende suchen:

      do 50 i = MAX,1,-1
         if(file(i:i).eq.'.') then
           iend = i - 1
           goto 60
         end if
50    continue

c --- Anfang suchen:

60    do 100 i = MAX,1,-1
         if(file(i:i).eq.'/'.or.file(i:i).eq.']'.or.
     *        file(i:i).eq.'\\') then
           ibeg = i + 1
           goto 120
         end if
100   continue
      ibeg = 1

c --- Basisnamen abspeichern:
      
120   n = 0
      do 200 i = ibeg,iend
         if(file(i:i).ne.' ') then
           n = n + 1
           name(n:n) = file(i:i)
         end if
200   continue     
    
c --- Laufende Nummer und Extension anhaengen:

      form = '(i )'
      write(form(3:3),'(i1)') nz
      if(nz.gt.0) write(name(n+1:n+nz),form) nr
      name(n+nz+1:n+nz+1) = '.'
      name(n+nz+2:n+nz+4) = filspe(1:3)
      laenge = n + nz + 4

      return
      end
c******************************************************************************
c***                 Permutationsmatrix bestimmen                           ***
c******************************************************************************

      subroutine permat(ipm,rs,r,natom,NATO,intaus,ism,tran,nsym,NSY)

      parameter(EPSY = 0.00001)
      character intaus*1
      integer ipm(NATO,NSY), ism(3,3,NSY)
      real xp(3), xc(3), r(3,NATO), tran(3,NSY), rs(3,NATO,NSY)


      do 100 k=1,natom

         do 80 n=1,nsym

c --- Symmetrieelement anwenden:

            do 41 i=1,3
               rs(i,k,n) = tran(i,n)
               do 40 j=1,3
                  rs(i,k,n) = rs(i,k,n) + ism(i,j,n) * r(j,k)
40             continue
41          continue

c --- Erzeugte Positionen in der prim. Basis darstellen:

            call transf(intaus,rs(1,k,n),xp,0)

c --- Erzeugte Position xp und zp in die prim. Zelle schieben:

            do 42 i=1,3
               nt = int(xp(i)+EPSY)
               if(xp(i).lt.-EPSY) nt = nt - 1
               xp(i) = xp(i) - nt
42          continue

c --- Erzeugte Position xp ruecktransformieren:

            call transf(intaus,xc,xp,1)

c --- Testen, welche Position erzeugt wurde:

            do 45 m = 1,natom
               if(abs(xc(1)-r(1,m)).lt.EPSY.and.
     *            abs(xc(2)-r(2,m)).lt.EPSY.and.
     *            abs(xc(3)-r(3,m)).lt.EPSY) then
                  ipm(k,n) = m
                  goto 80
               end if
45          continue

80       continue

100   continue


      return
      end
c******************************************************************************
c***          Symmetrieaequivalente Positionen bestimmen                    ***
c******************************************************************************

      subroutine posgen(x0,r,tp,iat,natom,NATO,intaus,ism,tran,nsym,NSY)

      parameter(EPSY = 0.00001)
      character intaus*1
      integer ism(3,3,NSY), iat(NSY)
      real r(3,NATO), tp(3,NATO), tran(3,NSY)
      real x0(3), x(3), xp(3), xc(3), t(3), tc(3), z(3), zp(3)


      natom = 0

      do 50 n=1,nsym

c --- Symmetrieelement auf x0 anwenden:

         do 41 i=1,3
            x(i) = tran(i,n)
            do 40 j=1,3
               x(i) = x(i) + ism(i,j,n) * x0(j)
 40         continue
            z(i) = x(i) - tran(i,n)
 41      continue

c --- Erzeugte Position in der prim. Basis darstellen (itr = 0):

         itr = 0
         call transf(intaus,x,xp,itr)
         call transf(intaus,z,zp,itr)

c --- Erzeugte Position in die prim. Zelle schieben:

         do 44 i=1,3
            nz = int(zp(i)+EPSY)
            if(zp(i).lt.-EPSY) nz = nz - 1
            nt = int(xp(i)+EPSY)
            if(xp(i).lt.-EPSY) nt = nt - 1
            xp(i) = xp(i) - nt
            t(i) = nz - nt
 44      continue

c --- Erzeugte Position ruecktransformieren (itr = 1):

         itr = 1
         call transf(intaus,xc,xp,itr)

c --- Testen, ob erzeugte Position schon vorhanden ist:

         do 45 k=1,natom
            if(abs(xc(1)-r(1,k)).lt.EPSY.and.
     *         abs(xc(2)-r(2,k)).lt.EPSY.and.
     *         abs(xc(3)-r(3,k)).lt.EPSY) then
              iat(n) = k
              goto 50
            end if
 45      continue

c --- Erzeugte Position ruecktransformieren und abspeichern:

         natom = natom + 1
         call transf(intaus,tc,t,1)
         do 46 i=1,3
            r(i,natom) = xc(i)
            iat(n) = natom
 46      continue
         do 47 i=1,3
            tp(i,k) = tran(i,n) + tc(i)
 47      continue

 50   continue


      return
      end
c******************************************************************************
c***               Berechnung des Projektionsoperator                       ***
c******************************************************************************

      subroutine proj(prj,mt,natom,NATO,tau,ndim,ism,nsym,NSY)

      integer ism(3,3,NSY), mt(NATO,NATO,NSY)
      complex tau(NSY), prj(3*NATO,3*NATO)


      nats = 3*natom

c --- Initialisierung:

       do 20 i=1,nats
          do 10 j=1,nats
             prj(j,i) = 0.
 10       continue
 20    continue

c --- Projektionsoperator berechnen:

      do 500 k=1,natom
         do 300 l=1,natom
            do 200 n=1,nsym
               do 150 i=1,3
                  do 130 j=1,3
                     m1 = 3*(k-1) + j
                     m2 = 3*(l-1) + i
                     prj(m2,m1)  = prj(m2,m1) + 
     *               mt(k,l,n) * ism(i,j,n) * conjg(tau(n))
 130              continue
 150           continue
 200        continue
 300     continue
 500  continue

c --- Normierung:

      do 650 i=1,nats
         do 630 j=1,nats
            prj(j,i) = prj(j,i) * ndim/real(nsym)
 630     continue
 650  continue


      return
      end


c******************************************************************************
c***    Invarianz gegenueber dem Wellenvektor ueberpruefen                  ***
c******************************************************************************

      subroutine qvekinv(iwav,q,ism,intaus)

      parameter(EPSY = 0.00001)
      real q(3), d(3), dp(3)
      integer ism(3,3)
      character intaus*1

c --- Vektor d = R*q - q erzeugen:

      do 20 i=1,3
         d(i) = -q(i)
         do 10 j=1,3
            d(i) = d(i) + ism(i,j) * q(j)
10       continue
20    continue

c --- Vektor d in Einheiten der prim. rez. Gittervektoren ausdruecken: 

      itr = 1
      call transf(intaus,dp,d,itr)

c --- Testen, ob Vektor dp ganzzahlig: 

      iwav = 1
      do 30 i=1,3
         if(abs(mod(dp(i),1.)).gt.EPSY) iwav = 0 
30    continue

      return
      end


c******************************************************************************
c***            Eingabe-Datei einlesen                                      ***
c******************************************************************************

      subroutine einles(ke,jobtyp,titel,spgrp,x0,q,csym,intaus,icent,
     *                  laue,ism,tran,nsym,NSY,cmod,tau,ndim,idar,
     *                  IDM,IRD)

      external caplet
      parameter(MAXLINE = 132, MAX = 18, MAXSY = 6, PI = 3.14159265359)
      character intaus*1, line*132, csym(NSY)*3, cmod(IRD)*4
      character char(MAX)*132, ecod*3, titel*80, spgrp*60
      character rgsymb(MAXSY)*10, caplet*1
      real x0(3), werte(MAX), q(3), tran(3,NSY)
      integer ism(3,3,NSY), ndim(IRD)
      complex tau(NSY,IDM,IDM,IRD)

      ecod = 'eee'


c --- Titel einlesen:

      ist = 0
      call getinp(ke,'TIT',ecod,werte,char,MAX,num,nn,ist)
      if(ist.eq.11.or.ist.eq.13) then
        stop 'ERROR: the title is missing'
      else
        write(*,*)
        write(*,*)
        titel(1:80) = char(1)(1:80)
        line = ' '
        line = '  Title: '//titel
        call writline(6,line,MAXLINE)
      end if

c --- Atomlage einlesen:

      ist = 1
      call getinp(ke,'ATO',ecod,werte,char,MAX,num,nn,ist)
      if(ist.eq.11.or.ist.eq.13) then
        stop 'ERROR: the atom position is missing'
      else
        do 10 i=1,3
           x0(i) = werte(i)
 10     continue
        write(*,'(/2x,a,3f8.4)') 
     *       'Position: ',(x0(i),i=1,3)
      end if

c --- Wellenvektor einlesen:

      ist = 1
      call getinp(ke,'WAV',ecod,werte,char,MAX,num,nn,ist)
      if(ist.eq.11.or.ist.eq.13) then
        stop 'ERROR: the wave vector is missing'
      else
        do 20 i=1,3
           q(i) = werte(i)
 20     continue
        write(*,'(/2x,a,3f8.4)') 'Wave vector: ',(q(i),i=1,3)
      end if

c --- Raumgruppen-Symbol einlesen:

      ist = 1
      call getinp(ke,'SPG','COD',werte,char,MAX,num,nn,ist)
      if(ist.eq.11.or.ist.eq.13) then
         jobtyp = 2
      else
         jobtyp = 1
         if(num.gt.MAXSY) then
            write(*,'(2x,2a)') 'Error: Bad space group symbol: ',spgrp
            stop
         else
            nsbl = num
            do 25 i=1,nsbl
               do 24 k=1,10
                  rgsymb(i)(k:k) = caplet(char(i)(k:k))
 24            continue
 25         continue
            call rgtest(rgsymb,nsbl,MAXSY,spgrp,nspg,ncell)
            write(*,'(/2x,a,a40,/2x,a,i3,a,i1)')
     *           'Complete space group symbol: ',spgrp,
     *           'Space group no. ',nspg,',   cell choice ',ncell
         end if
      end if

c --- Folgende Zeilen werden nur dann eingelesen, falls die
c     Raumgruppe generiert werden soll:

      if(jobtyp.eq.2) then

c --- Zentrierung, Zentrosymmetrie, Lauegruppe:

        ist = 1
        call getinp(ke,'COD',ecod,werte,char,MAX,num,nn,ist)
        if(ist.eq.11.or.ist.eq.13) then
          stop 'ERROR: Code is missing'
        end if
        intaus(1:1) = char(1)(1:1)
        icent       = nint(werte(2))
        laue        = nint(werte(3))

c --- Irred. Darstellungen einlesen:

        do 30 m=1,IRD+1
           ist = 1
           call getinp(ke,'REP','GEN',werte,char,MAX,num,nn,ist)
           if(ist.eq.11.or.ist.eq.13) then
             idar = m - 1
             goto 31
           end if
           cmod(m)(1:4) = char(1)(1:4)
           ndim(m)      = nint(werte(2))
 30     continue

c --- Generatoren einlesen:

 31     do 100 n=1,NSY+1
           ist = 1
           call getinp(ke,'GEN',ecod,werte,char,MAX,num,nn,ist)
           if(ist.eq.11.or.ist.eq.13) then
             nsym = n-1
             goto 200
           end if
           ls = 4
           csym(n) = ' '
           do 33 l=3,1,-1
              if(char(1)(l:l).ne.' ') then
                ls = ls - 1
                csym(n)(ls:ls) = char(1)(l:l)
              end if
 33        continue
           do 37 i=1,3
              tran(i,n) = werte(10+i)
              do 36 j=1,3
                 ism(j,i,n) = nint(werte(3*i+j-2))
 36           continue
 37        continue
           do 80 m=1,idar
              ist = 1
              call getinp(ke,'TAU',ecod,werte,char,MAX,num,nn,ist)
              if(ist.eq.11) then
                stop 'ERROR:'
              end if
              do 70 i=1,ndim(m)
                 do 60 j=1,ndim(m)
                    iw = 2 * ndim(m) * (i - 1) + 2*j     
                    r   = werte(iw-1)       
                    iph = nint(werte(iw))
                    zr = r * cos(PI/180.*iph)
                    zi = r * sin(PI/180.*iph)       
                    tau(n,j,i,m)   = cmplx(zr,zi) 
 60              continue
 70           continue
 80        continue
 100    continue

      end if

 200  return
      end
C******************************************************************************
C***               KODIERUNG DER SYMMETRIEKARTEN                            ***
C******************************************************************************

      SUBROUTINE SYMINT(STRING,CZENT,NL,ICENT,ISM,TRAN,NSYM)

      PARAMETER (MAXS = 100)

      INTEGER ISM(3,3,48)
      REAL TRAN(3,48)
 
      CHARACTER HEX(30)*13, TET(85)*14, KUB(42)*14, RHO(32)*14
      CHARACTER ORT(70)*14, MON(60)*13, TRI(2)*13

      COMMON /CHAHEX/ HEX
      COMMON /CHATET/ TET
      COMMON /CHARHO/ RHO
      COMMON /CHAKUB/ KUB
      COMMON /CHAORT/ ORT
      COMMON /CHAMON/ MON
      COMMON /CHATRI/ TRI

      COMMON /INHEX/ IHEX(30,24)
      COMMON /INTET/ ITET(85,16)
      COMMON /INRHO/ IRHO(32,12)
      COMMON /INKUB/ IKUB(42,48)
      COMMON /INORT/ IORT(70,8)
      COMMON /INMON/ IMON(60,4)
      COMMON /INTRI/ ITRI(2,2) 

      CHARACTER CH*3, CZENT*1, C1C*4, C2C*4, C3C*4, CUR*1, STRING*60
      INTEGER LL(0:3), LMAT(3,3), NITR(3), IDUM(MAXS)
      COMMON /CH/ CH, CUR
      COMMON /IENTE/ IH(9)


c --- Initialisierung:

      CZENT(1:1) = ' '
      C1C(1:4)   = '    '
      C2C(1:4)   = '    '
      C3C(1:4)   = '    '
      CUR(1:1)   = ' '

c --- Raumgruppen-Symbol:

      CALL SYMMK(STRING,CZENT,C1C,C2C,C3C,CUR)

C --- Interpretation des Symbols:

      IF(C2C(1:1).EQ.'3') THEN
C        WRITE(*,*) ' Kubische RG.'
	ICO = 43
      ELSE IF(C1C(1:1).EQ.'4') THEN
C        WRITE(*,*) ' Tetragonale RG.'
	ICO = 4
      ELSE IF(C1C(1:1).EQ.'6') THEN
C        WRITE(*,*) ' Hexagonale RG.'
	ICO = 6
      ELSE IF(C1C(1:1).EQ.'3') THEN
C        WRITE(*,*) ' Trigonale RG.'
	ICO = 3
      ELSE IF(C1C(1:1).EQ.'1') THEN
	  IF(C2C(1:1).EQ.' ') THEN
C            WRITE(*,*) ' Trikline RG.'
	    ICO = 1
	  ELSE
C            WRITE(*,*) ' Monokline RG.'
	    ICO = 2
	  END IF
      ELSE IF (C1C(1:1).EQ.'1'.OR.C2C(1:1).EQ.'1'.OR.
     *         C3C(1:1).EQ.'1') THEN
C        WRITE(*,*) ' Monokline RG.'
	ICO = 2
      ELSE
C        WRITE(*,*)' Orthorhombische RG.'
	ICO = 5
      END IF


c --- Tetragonale Raumgruppen:

      IF(ICO.EQ.4) THEN
C        WRITE(*,*)' Tetragonale Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 16
	LSYM  =  0 
	DO 477 K=1,85
	  IF(CUR(1:1).EQ.'2') THEN
	    IF(TET(K)(14:14).NE.'2') GOTO 477
	  END IF
	  IF(TET(K)(1:1).NE.CZENT(1:1)) GOTO 477
	  IF(TET(K)(2:5).NE.C1C(1:4)) GOTO 477
	  IF(TET(K)(6:9).NE.C2C(1:4)) GOTO 477
	  IF(TET(K)(10:13).NE.C3C(1:4)) GOTO 477
	  LSYM = K
	  GOTO 478
477     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (TET)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

478     CONTINUE
	DO 452 L=1,IMAXS
	  IDUM(L) = ITET(LSYM,L)
452     CONTINUE


c --- Hexagonale Raumgruppen:

      ELSE IF (ICO.EQ.6) THEN
C        WRITE(*,*)' Hexagonale Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 24
	LSYM  =  0
	DO 677 K=1,30
	  IF(HEX(K)(1:1).NE.CZENT(1:1)) GOTO 677
	  IF(HEX(K)(2:5).NE.C1C(1:4)) GOTO 677
	  IF(HEX(K)(6:9).NE.C2C(1:4)) GOTO 677
	  IF(HEX(K)(10:13).NE.C3C(1:4)) GOTO 677
	  LSYM = K
	  GOTO 678
677     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (HEX)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

678     CONTINUE
	DO 651 L=1,IMAXS
 	   IDUM(L) = IHEX(LSYM,L)
651     CONTINUE


c --- Trigonale Raumgruppen:

      ELSE IF (ICO.EQ.3) THEN
c        WRITE(*,*)' Rhomboedr. Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
c     *  ' ',CUR

	IMAXS = 12
	LSYM  =  0
	DO 377 K=1,32
	  IF(RHO(K)(1:1).NE.CZENT(1:1)) GOTO 377
	  IF(RHO(K)(2:5).NE.C1C(1:4)) GOTO 377
	  IF(RHO(K)(6:9).NE.C2C(1:4)) GOTO 377
	  IF(RHO(K)(10:13).NE.C3C(1:4)) GOTO 377
	  IF(RHO(K)(14:14).NE.CUR(1:1)) GOTO 377
	  LSYM = K
	  GOTO 378
377     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (RHO)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

378     CONTINUE

C --- BEI RHOMBOEDRISCHEN RAUMGRUPPEN STEHT EIN "R" AN ERSTER STELLE, DAS ALS 
C     INTEGRALE AUSLOESCHUNG INTERPRETIERT WIRD. DESHALB WIRD HIER IN 'CZENT' 
C     EIN "P" REINGESCHRIEBEN.

        if(cur.eq.'R') CZENT(1:1) = 'P'

	DO 351 L=1,IMAXS
	  IDUM(L) = IRHO(LSYM,L)
351     CONTINUE


c --- Kubische Raumgruppen:

      ELSE IF (ICO.EQ.43) THEN
C        WRITE(*,*)' Kubische Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 48
	LSYM  =  0
	DO 4377 K=1,42
	  IF(CUR(1:1).EQ.'2') THEN
	    IF(KUB(K)(14:14).NE.'2') GOTO 4377
	  END IF
	  IF(KUB(K)(1:1).NE.CZENT(1:1)) GOTO 4377
	  IF(KUB(K)(2:5).NE.C1C(1:4)) GOTO 4377
	  IF(KUB(K)(6:9).NE.C2C(1:4)) GOTO 4377
	  IF(KUB(K)(10:13).NE.C3C(1:4)) GOTO 4377
	  LSYM = K
	  GOTO 4378
4377    CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (KUB)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

4378    CONTINUE
	DO 4351 L=1,IMAXS
	  IDUM(L) = IKUB(LSYM,L)
4351    CONTINUE


c --- Trikline Raumgruppen:

      ELSE IF (ICO.EQ.1) THEN
C        WRITE(*,*)' Trikline Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 2
	LSYM  = 0
	DO 177 K=1,2
	  IF(TRI(K)(1:1).NE.CZENT(1:1)) GOTO 177
	  IF(TRI(K)(2:5).NE.C1C(1:4)) GOTO 177
	  IF(TRI(K)(6:9).NE.C2C(1:4)) GOTO 177
	  IF(TRI(K)(10:13).NE.C3C(1:4)) GOTO 177
	  LSYM = K
	  GOTO 178
177     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (TRI)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

178     CONTINUE
	DO 151 L=1,IMAXS
	   IDUM(L) = ITRI(LSYM,L)
151     CONTINUE


c --- Monokline Raumgruppen:

      ELSE IF (ICO.EQ.2) THEN
C        WRITE(*,*)' Monokline Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 4
	LSYM  = 0
	DO 277 K=1,60
	  IF(MON(K)(1:1).NE.CZENT(1:1)) GOTO 277
	  IF(MON(K)(2:5).NE.C1C(1:4)) GOTO 277
	  IF(MON(K)(6:9).NE.C2C(1:4)) GOTO 277
	  IF(MON(K)(10:13).NE.C3C(1:4)) GOTO 277
	  LSYM = K
	  GOTO 278
277     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (MON)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

278     CONTINUE
	DO 251 L=1,IMAXS
	   IDUM(L) = IMON(LSYM,L)
251     CONTINUE


c --- Orthorhombische Raumgruppen:

      ELSE IF (ICO.EQ.5) THEN
C        WRITE(*,*)' Orthor. Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 8
	LSYM  = 0
	DO 577 K=1,70
	  IF(CUR(1:1).EQ.'2') THEN
	    IF(ORT(K)(14:14).NE.'2') GOTO 577
	  END IF
	  IF(ORT(K)(1:1).NE.CZENT(1:1)) GOTO 577
	  IF(ORT(K)(2:5).NE.C1C(1:4)) GOTO 577
	  IF(ORT(K)(6:9).NE.C2C(1:4)) GOTO 577
	  IF(ORT(K)(10:13).NE.C3C(1:4)) GOTO 577
	  LSYM = K
	  GOTO 578
577     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (ORT)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

578     CONTINUE
	DO 551 L=1,IMAXS
	  IDUM(L) = IORT(LSYM,L)
551     CONTINUE

c --- Ungueltiges Raumgruppensymbol:
 
      ELSE

	WRITE(*,*)' This space group is unknown '

      END IF

c --- Schleife ueber die Matrizen:

      ISC = 0
900   CONTINUE

      DO 771 M=1,IMAXS

	 IF(IDUM(M).LE.0) GOTO 771

         ISC = ISC + 1
         ICODE = IDUM(M)
 
c --- Entcodierung:

         CALL ENTCOD(ISC,ICODE,LAUE,ICE,NNENN,LL,NITR)

         DO 500 M1=1,3
	    DO 500 M2=1,3
	       LMAT(M1,M2) = 0
500      CONTINUE

         CALL MATRIX(LL,LMAT)

         IF(ISC.EQ.1) THEN
	   NL = LAUE
	   ICENT = ICE
C           WRITE(*,*)' SYMINT LAUEGRUPPE = ',LAUE
C           WRITE(*,*)' SYMINT ZENTROSY. = ',ICE
         END IF

         IF(NNENN.EQ.0) THEN
	   DO 213 N1=1,3
	      DO 213 N2=1,3
	         ISM(N1,N2,ISC) = LMAT(N1,N2)
213        CONTINUE
C           DO 215 N1=1,3  
C              WRITE(*,*) (ISM(N1,N2,ISC),N2=1,3)
C215        CONTINUE
         ELSE
  	   DO 212 N1=1,3
	      TRANS = REAL(NITR(N1))/REAL(NNENN)
	      TRAN(N1,ISC) = TRANS    
	      DO 212 N2=1,3
	         ISM(N1,N2,ISC) = LMAT(N1,N2)
212        CONTINUE
C           DO 216 N1=1,3  
C              WRITE(*,*) (ISM(N1,N2,ISC),N2=1,3),TRAN(N1,ISC)
C216        CONTINUE
         END IF

C         WRITE(*,*)' DIE ',ISC,' TE MATRIX'

771   CONTINUE

      NSYM = ISC
C      WRITE(*,*)'ISC',ISC,'NSYM=',NSYM
   
      RETURN
      END


C******************************************************************************
C***            Einlesen von Symmetrie-Daten                                ***
C******************************************************************************

      SUBROUTINE SYMMK(STRING,CZENT,C1C,C2C,C3C,CUR)

      PARAMETER (ISTR = 60)

      CHARACTER STRING*60, STRANG*60
      CHARACTER CZENT*1, C1C*4, C2C*4, C3C*4, CUR*1

      INTEGER IKOM(0:20)


c --- Initialisierung:

      do 110 i=0,20
         ikom(i) = 0
 110  continue

C --- Die 'Blanks' werden rausgeschmissen:

      DO 140 ISP=1,ISTR
	 STRANG(ISP:ISP) = ' '
140   CONTINUE

      ISPEN = 0   
210   DO 150 ISP=1,ISTR
	 IF(STRING(ISP:ISP).NE.' ') THEN
	   ISPEN = ISPEN + 1
	   STRANG(ISPEN:ISPEN) = STRING(ISP:ISP)
	 END IF
150   CONTINUE

c      WRITE(*,*) ' SYMM - STRANG ',STRANG

C --- Kommata werden festgelegt:

      IKOMM = 0
      DO 160 JJ=1,ISPEN
	 IF(STRANG(JJ:JJ).EQ.',') THEN
	   IKOMM = IKOMM + 1
	   IKOM(IKOMM) = JJ
	 END IF
160   CONTINUE

      IF(IKOMM.GT.21) THEN
	WRITE(*,*) ' Too many commas in space group symbol'
	STOP
      END IF

      I1  = IKOM(1) - 1
      CZENT(1:1) = STRANG(1:I1)
      I1  = I1 + 2
      I2  = IKOM(2) - 1
      C1C = STRANG(I1:I2)
      I1  = I2 + 2
      I2  = IKOM(3) - 1
      IF(I1.LE.I2) C2C = STRANG(I1:I2)
      I1  = I2 + 2
      IF(IKOM(4).EQ.0) THEN
	C3C = STRANG(I1:I1+4)
	CUR = ' '
      ELSE
	I2  = IKOM(4) - 1
	IF(I1.LE.I2) C3C = STRANG(I1:I2)
	I1  = I2 + 2
	CUR = STRANG(I1:I1)
      END IF

c      WRITE(*,*) ' SYMM: ',CZENT,C1C,C2C,C3C,CUR

9999  RETURN
      END


C******************************************************************************
C***            Entkodierung der Raumgruppen                                ***
C******************************************************************************

      SUBROUTINE ENTCOD(ISC,ICODE,LAUE,ICE,NNENN,LL,NITR)

      INTEGER LL(0:*),NITR(*)

      COMMON/IENTE/IH(9)

      IF(ISC.EQ.1) THEN
	LAUE  = ICODE/IH(9)
	ICODE = ICODE-LAUE*IH(9)
	ICE   = ICODE/IH(8)
	ICODE = ICODE-ICE*IH(8)
      END IF

      LL(0)   = ICODE/IH(7)-4
      ICODE   = ICODE-(LL(0)+4)*IH(7)
      LL(1)   = ICODE/IH(6)-4
      ICODE   = ICODE-(LL(1)+4)*IH(6)
      LL(2)   = ICODE/IH(5)-4
      ICODE   = ICODE-(LL(2)+4)*IH(5)
      LL(3)   = ICODE/IH(4)-4
      ICODE   = ICODE-(LL(3)+4)*IH(4)
      NNENN   = ICODE/IH(3)
      ICODE   = ICODE-NNENN*IH(3)
      NITR(1) = ICODE/IH(2)
      ICODE   = ICODE-NITR(1)*IH(2)
      NITR(2) = ICODE/IH(1)
      ICODE   = ICODE-NITR(2)*IH(1)
      NITR(3) = ICODE

      RETURN
      END    


C******************************************************************************
C***                                                                        ***
C******************************************************************************

      SUBROUTINE MATRIX(KK,IMAT)

      INTEGER KK(0:3),IMAT(3,3)

      IF(KK(0).EQ.0) THEN
	IF(KK(1).EQ. 1) IMAT(1,1) =  1
	IF(KK(1).EQ. 2) IMAT(1,2) =  1
	IF(KK(1).EQ. 3) IMAT(1,3) =  1
	IF(KK(1).EQ.-1) IMAT(1,1) = -1
	IF(KK(1).EQ.-2) IMAT(1,2) = -1
	IF(KK(1).EQ.-3) IMAT(1,3) = -1
	IF(KK(2).EQ. 1) IMAT(2,1) =  1
	IF(KK(2).EQ. 2) IMAT(2,2) =  1
	IF(KK(2).EQ. 3) IMAT(2,3) =  1
	IF(KK(2).EQ.-1) IMAT(2,1) = -1
	IF(KK(2).EQ.-2) IMAT(2,2) = -1
	IF(KK(2).EQ.-3) IMAT(2,3) = -1
	IF(KK(3).EQ. 1) IMAT(3,1) =  1
	IF(KK(3).EQ. 2) IMAT(3,2) =  1
	IF(KK(3).EQ. 3) IMAT(3,3) =  1
	IF(KK(3).EQ.-1) IMAT(3,1) = -1
	IF(KK(3).EQ.-2) IMAT(3,2) = -1
	IF(KK(3).EQ.-3) IMAT(3,3) = -1
      ELSE
	IF(KK(3).EQ. 1) IMAT(3,1) =  1
	IF(KK(3).EQ. 2) IMAT(3,2) =  1
	IF(KK(3).EQ. 3) IMAT(3,3) =  1
	IF(KK(3).EQ.-1) IMAT(3,1) = -1
	IF(KK(3).EQ.-2) IMAT(3,2) = -1
	IF(KK(3).EQ.-3) IMAT(3,3) = -1
	IF(ABS(KK(0)).EQ.1) THEN
	  IF(KK(2).EQ. 1) IMAT(2,1) =  1
	  IF(KK(2).EQ. 2) IMAT(2,2) =  1
	  IF(KK(2).EQ. 3) IMAT(2,3) =  1
	  IF(KK(2).EQ.-1) IMAT(2,1) = -1
	  IF(KK(2).EQ.-2) IMAT(2,2) = -1
	  IF(KK(2).EQ.-3) IMAT(2,3) = -1
	  IF(KK(0).EQ.1) THEN
	    IMAT(1,1) =  1
	    IMAT(1,2) = -1
	  ELSE
	    IMAT(1,1) = -1
	    IMAT(1,2) =  1
	  END IF
	END IF
	IF(ABS(KK(0)).EQ.2) THEN
	  IF(KK(1).EQ. 1) IMAT(1,1) =  1
	  IF(KK(1).EQ. 2) IMAT(1,2) =  1
	  IF(KK(1).EQ. 3) IMAT(1,3) =  1
	  IF(KK(1).EQ.-1) IMAT(1,1) = -1
	  IF(KK(1).EQ.-2) IMAT(1,2) = -1
	  IF(KK(1).EQ.-3) IMAT(1,3) = -1
	  IF(KK(0).EQ.2) THEN
	    IMAT(2,1) =  1
	    IMAT(2,2) = -1
	  ELSE
	    IMAT(2,1) = -1
	    IMAT(2,2) =  1
	  END IF
	END IF
      END IF

      RETURN
      END

c******************************************************************************
c***                  Symmetrieelemente sortieren                           ***
c******************************************************************************

      subroutine symsort(nc,csym,symor,ism,tran,trp,trs,num,numsy,
     *                   nsym,NSY)

      parameter(MAX = 48, EPSY = 0.0001)
      real trp(3,NSY), tran(3,NSY), trs(3,NSY)
      character csym(NSY)*3, symor(NSY)*9, cm*4, so*9
      integer irot(3,3), ism(3,3,NSY)
      integer num(NSY), numsy(NSY)


c --- Nummer des Symmetrieelementes bestimmen:

      do 100 n=1,nsym
         do 50 k=1,MAX
            call rotmat(k,nc,cm,so,irot)
            ng = 0
            do 30 i=1,3
               do 20 j=1,3
                  if(ism(j,i,n).eq.irot(j,i)) ng = ng + 1
 20            continue
 30         continue
            if(ng.eq.9) goto 60
 50      continue
 60      if(ng.eq.9) then
           num(n) = k
           do 70 i=1,3
              trs(i,k) = tran(i,n)
 70        continue           
         else
           write(*,'(/2x,a,i2,a)') 
     *     'ERROR: Symmetry element ',n,' not identified'
           stop
         end if
 100  continue

c --- Symbol bestimmen und Rotationsmatrizen und Translationen nach oben 
c     gegebener Reihenfolge abspeichern:

      ns = 0
      do 200 k=1,MAX
         do 150 n=1,nsym
            if(num(n).eq.k) then

              ns = ns + 1
              numsy(ns) = k
              call rotmat(k,nc,cm,so,irot)
              do 130 i=1,3
                 tran(i,ns) = trs(i,k) 
                 do 120 j=1,3
                    ism(j,i,ns) = irot(j,i)
 120             continue
 130          continue
              csym(ns) = cm
              symor(ns) = so

c --- Pruefen, ob ein Symmetrieelement {ism|tran} phasenschiebend ist
c     und den phasenschiebenden Translationsanteil trp bestimmen
c     (npt gibt an, wie oft man das Symmetrieelement anwenden muss,
c     um wieder die Einheitsmatrix zu erhalten):

              call tranphs(trp(1,ns),npt,ism(1,1,ns),tran(1,ns))

c --- Test auf Gleitspiegelebene/Schraubenachse:

              if(abs(trp(1,ns)).gt.EPSY.or. 
     *           abs(trp(2,ns)).gt.EPSY.or.
     *           abs(trp(3,ns)).gt.EPSY) then
                call transbl(csym(ns),npt,trp(1,ns))
              end if

              goto 200

            end if
 150     continue
 200  continue


      return
      end


c******************************************************************************
c***                 Symmetrieelement bestimmen                             ***
c******************************************************************************

      subroutine rotmat(k,nc,csym,symor,ism)

      parameter(MAX = 48)
      character csym*3, symor*9, cm(MAX,2)*3, so(MAX,2)*9
      integer irot(3,3,MAX,2), ism(3,3)


c --- Rotationsmatrizen fuer das trikline, monokline, orthorhombische,
c     tetragonale, rhomboedrische und kubische System:

      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=1,8)
     *  / '  1',  '         ',   1, 0, 0,   0, 1, 0,   0, 0, 1,  
     *    ' 3+',  '  [1,1,1]',   0, 0, 1,   1, 0, 0,   0, 1, 0,
     *    ' 3+',  '[1,-1,-1]',   0, 0,-1,  -1, 0, 0,   0, 1, 0,
     *    ' 3+',  '[-1,1,-1]',   0, 0, 1,  -1, 0, 0,   0,-1, 0,
     *    ' 3+',  '[-1,-1,1]',   0, 0,-1,   1, 0, 0,   0,-1, 0,
     *    ' 3-',  '  [1,1,1]',   0, 1, 0,   0, 0, 1,   1, 0, 0,
     *    ' 3-',  '[1,-1,-1]',   0,-1, 0,   0, 0, 1,  -1, 0, 0,
     *    ' 3-',  '[-1,1,-1]',   0,-1, 0,   0, 0,-1,   1, 0, 0/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=9,16)
     *  / ' 3-',  '[-1,-1,1]',   0, 1, 0,   0, 0,-1,  -1, 0, 0,
     *    ' 4+',  '  [1,0,0]',   1, 0, 0,   0, 0,-1,   0, 1, 0,
     *    ' 4+',  '  [0,1,0]',   0, 0, 1,   0, 1, 0,  -1, 0, 0,
     *    ' 4+',  '  [0,0,1]',   0,-1, 0,   1, 0, 0,   0, 0, 1,
     *    ' 4-',  '  [1,0,0]',   1, 0, 0,   0, 0, 1,   0,-1, 0,
     *    ' 4-',  '  [0,1,0]',   0, 0,-1,   0, 1, 0,   1, 0, 0,
     *    ' 4-',  '  [0,0,1]',   0, 1, 0,  -1, 0, 0,   0, 0, 1,
     *    '  2',  '  [1,0,0]',   1, 0, 0,   0,-1, 0,   0, 0,-1/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=17,24)
     *  / '  2',  '  [0,1,0]',  -1, 0, 0,   0, 1, 0,   0, 0,-1,
     *    '  2',  '  [0,0,1]',  -1, 0, 0,   0,-1, 0,   0, 0, 1,
     *    '  2',  '  [1,1,0]',   0, 1, 0,   1, 0, 0,   0, 0,-1,
     *    '  2',  '  [1,0,1]',   0, 0, 1,   0,-1, 0,   1, 0, 0,
     *    '  2',  '  [0,1,1]',  -1, 0, 0,   0, 0, 1,   0, 1, 0,
     *    '  2',  ' [1,-1,0]',   0,-1, 0,  -1, 0, 0,   0, 0,-1,
     *    '  2',  ' [-1,1,0]',   0, 0,-1,   0,-1, 0,  -1, 0, 0,
     *    '  2',  ' [-1,0,1]',  -1, 0, 0,   0, 0,-1,   0,-1, 0/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=25,32)
     *  / ' 1b',  '         ',  -1, 0, 0,   0,-1, 0,   0, 0,-1,
     *    '3b+',  '  [1,1,1]',   0, 0,-1,  -1, 0, 0,   0,-1, 0,
     *    '3b+',  '[1,-1,-1]',   0, 0, 1,   1, 0, 0,   0,-1, 0,
     *    '3b+',  '[-1,1,-1]',   0, 0,-1,   1, 0, 0,   0, 1, 0,
     *    '3b+',  '[-1,-1,1]',   0, 0, 1,  -1, 0, 0,   0, 1, 0,
     *    '3b-',  '  [1,1,1]',   0,-1, 0,   0, 0,-1,  -1, 0, 0,
     *    '3b-',  '[1,-1,-1]',   0, 1, 0,   0, 0,-1,   1, 0, 0,
     *    '3b-',  '[-1,1,-1]',   0, 1, 0,   0, 0, 1,  -1, 0, 0/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=33,40)
     *  / '3b-',  '[-1,-1,1]',   0,-1, 0,   0, 0, 1,   1, 0, 0,
     *    '4b+',  '  [1,0,0]',  -1, 0, 0,   0, 0, 1,   0,-1, 0,
     *    '4b+',  '  [0,1,0]',   0, 0,-1,   0,-1, 0,   1, 0, 0,
     *    '4b+',  '  [0,0,1]',   0, 1, 0,  -1, 0, 0,   0, 0,-1,
     *    '4b-',  '  [1,0,0]',  -1, 0, 0,   0, 0,-1,   0, 1, 0,
     *    '4b-',  '  [0,1,0]',   0, 0, 1,   0,-1, 0,  -1, 0, 0,
     *    '4b-',  '  [0,0,1]',   0,-1, 0,   1, 0, 0,   0, 0,-1,
     *    '  m',  '  [1,0,0]',  -1, 0, 0,   0, 1, 0,   0, 0, 1/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=41,48)
     *  / '  m',  '  [0,1,0]',   1, 0, 0,   0,-1, 0,   0, 0, 1,
     *    '  m',  '  [0,0,1]',   1, 0, 0,   0, 1, 0,   0, 0,-1,
     *    '  m',  '  [1,1,0]',   0,-1, 0,  -1, 0, 0,   0, 0, 1,
     *    '  m',  '  [1,0,1]',   0, 0,-1,   0, 1, 0,  -1, 0, 0,
     *    '  m',  '  [0,1,1]',   1, 0, 0,   0, 0,-1,   0,-1, 0,
     *    '  m',  ' [1,-1,0]',   0, 1, 0,   1, 0, 0,   0, 0, 1,
     *    '  m',  ' [-1,0,1]',   0, 0, 1,   0, 1, 0,   1, 0, 0,
     *    '  m',  ' [0,1,-1]',   1, 0, 0,   0, 0, 1,   0, 1, 0/


c --- Rotationmatrizen fuer das hexagonale System:

      data (cm(n,2),so(n,2),((irot(j,i,n,2),i=1,3),j=1,3),n=1,8)
     *  / '  1',  '         ',   1, 0, 0,   0, 1, 0,   0, 0, 1,
     *    ' 3+',  '  [0,0,1]',   0,-1, 0,   1,-1, 0,   0, 0, 1,
     *    ' 3-',  '  [0,0,1]',  -1, 1, 0,  -1, 0, 0,   0, 0, 1,
     *    ' 6+',  '  [0,0,1]',   1,-1, 0,   1, 0, 0,   0, 0, 1,
     *    ' 6-',  '  [0,0,1]',   0, 1, 0,  -1, 1, 0,   0, 0, 1, 
     *    '  2',  '  [0,0,1]',  -1, 0, 0,   0,-1, 0,   0, 0, 1,
     *    '  2',  '  [1,0,0]',   1,-1, 0,   0,-1, 0,   0, 0,-1,
     *    '  2',  '  [0,1,0]',  -1, 0, 0,  -1, 1, 0,   0, 0,-1/
      data (cm(n,2),so(n,2),((irot(j,i,n,2),i=1,3),j=1,3),n=9,16)
     *  / '  2',  '  [1,1,0]',   0, 1, 0,   1, 0, 0,   0, 0,-1,
     *    '  2',  ' [1,-1,0]',   0,-1, 0,  -1, 0, 0,   0, 0,-1,
     *    '  2',  '  [1,2,0]',  -1, 1, 0,   0, 1, 0,   0, 0,-1,
     *    '  2',  '  [2,1,0]',   1, 0, 0,   1,-1, 0,   0, 0,-1,
     *    ' 1b',  '         ',  -1, 0, 0,   0,-1, 0,   0, 0,-1,
     *    '3b+',  '  [0,0,1]',   0, 1, 0,  -1, 1, 0,   0, 0,-1,
     *    '3b-',  '  [0,0,1]',   1,-1, 0,   1, 0, 0,   0, 0,-1,
     *    '6b+',  '  [0,0,1]',  -1, 1, 0,  -1, 0, 0,   0, 0,-1/
      data (cm(n,2),so(n,2),((irot(j,i,n,2),i=1,3),j=1,3),n=17,24)
     *  / '6b-',  '  [0,0,1]',   0,-1, 0,   1,-1, 0,   0, 0,-1,
     *    '  m',  '  [0,0,1]',   1, 0, 0,   0, 1, 0,   0, 0,-1,
     *    '  m',  '  [1,0,0]',  -1, 1, 0,   0, 1, 0,   0, 0, 1,
     *    '  m',  '  [0,1,0]',   1, 0, 0,   1,-1, 0,   0, 0, 1,
     *    '  m',  '  [1,1,0]',   0,-1, 0,  -1, 0, 0,   0, 0, 1,
     *    '  m',  ' [1,-1,0]',   0, 1, 0,   1, 0, 0,   0, 0, 1,
     *    '  m',  '  [1,2,0]',   1,-1, 0,   0,-1, 0,   0, 0, 1,
     *    '  m',  '  [2,1,0]',  -1, 0, 0,  -1, 1, 0,   0, 0, 1/

      csym  = cm(k,nc)
      symor = so(k,nc)
      do 20 i=1,3
         do 10 j=1,3
            ism(j,i) = irot(j,i,k,nc)
 10      continue
 20   continue

      return
      end


c******************************************************************************
c***        Test auf Gleitspiegelebene/Schraubenachse                       ***
c******************************************************************************

      subroutine transbl(csym,npt,trp)

      parameter(NGP = 23, NSA = 16, EPSY = 0.0001)
      real trp(3), tgp(3,NGP), tsa(3,NSA)
      character csym*3, chr*1, gp(NGP)*3, sa(2,NSA)*3

c --- Gleitspiegelebenen:

      data (gp(k),(tgp(i,k),i=1,3),k=1,NGP)
     *   /'  a',  0.5,  0.,   0.,   '  b',  0.,   0.50, 0.,
     *    '  c',  0.,   0.,   0.50, '  n',  0.50, 0.50, 0.,
     *    '  n',  0.,   0.50, 0.50, '  n',  0.50, 0.  , 0.50,
     *    '  n',  0.50, 0.50, 0.50, '  n', -0.50, 0.50, 0.50,
     *    '  n',  0.50,-0.50, 0.50, '  n',  0.50, 0.50,-0.50,
     *    '  d',  0.25, 0.25, 0.,   '  d',  0.25,-0.25, 0.,
     *    '  d',  0.  , 0.25, 0.25, '  d',  0.  , 0.25,-0.25,
     *    '  d',  0.25, 0.  , 0.25, '  d', -0.25, 0.  , 0.25,
     *    '  d',  0.25, 0.25, 0.25, '  d', -0.25, 0.25, 0.25,
     *    '  d',  0.25,-0.25, 0.25, '  d',  0.25, 0.25,-0.25,
     *    '  d', -0.25,-0.25, 0.25, '  d', -0.25, 0.25,-0.25,
     *    '  d',  0.25,-0.25,-0.25/

c --- Schraubenachsen:

      data (sa(1,k),sa(2,k),(tsa(i,k),i=1,3),k=1,NSA)
     *   /'31+','32-', 0.,   0.,   0.33333333,
     *    '32+','31-', 0.,   0.,   0.66666667,
     *    '41+','43-', 0.,   0.,   0.25,
     *    '42+','42-', 0.,   0.,   0.50,
     *    '43+','41-', 0.,   0.,   0.75,
     *    '41+','43-', 0.,   0.25, 0.,
     *    '42+','42-', 0.,   0.50, 0.,
     *    '43+','41-', 0.,   0.75, 0.,
     *    '41+','43-', 0.25, 0.,   0.,
     *    '42+','42-', 0.50, 0.,   0.,
     *    '43+','41-', 0.75, 0.,   0.,
     *    '61+','65-', 0.,   0.,   0.16666667,
     *    '62+','64-', 0.,   0.,   0.33333333,
     *    '63+','63-', 0.,   0.,   0.50,
     *    '64+','62-', 0.,   0.,   0.66666667,
     *    '65+','61-', 0.,   0.,   0.83333333/     


c --- Test auf a,b,c,n,d-Gleitspiegelebene:

      if(index(csym,'m').gt.0) then

        do 245 l=1,NGP
           if(abs(trp(1)-tgp(1,l)).lt.EPSY.and.
     *        abs(trp(2)-tgp(2,l)).lt.EPSY.and.
     *        abs(trp(3)-tgp(3,l)).lt.EPSY) then
             csym = gp(l)
             goto 250
           end if
 245    continue
        do 247 j=1,3
           xx = npt * trp(j)
           if(abs(amod(xx,1.)).gt.EPSY) then
             csym = '  d'
             goto 250
           end if
 247    continue
 250    continue

c --- Test auf Schraubenachse: 

      else

        do 400 n=1,6
           write(chr,'(i1)') n
           na = index(csym,chr)
           if(na.gt.0) then
             if(n.eq.2) then
               csym = ' 21'
             else
               do 350 k=1,NSA
                  if(abs(trp(1)-tsa(1,k)).lt.EPSY.and.
     *               abs(trp(2)-tsa(2,k)).lt.EPSY.and.
     *               abs(trp(3)-tsa(3,k)).lt.EPSY.and.
     *               csym(2:2).eq.sa(1,k)(1:1)) then
                    if(csym(3:3).eq.'+') then
                      csym = sa(1,k)
                    else
                      csym = sa(2,k)
                    end if
                  end if
 350           continue
             end if
             goto 450
           end if
 400    continue
 450    continue

      end if

      return
      end
c******************************************************************************
c***  Bestimmung der Phasenschiebende Translation der Symmetrieelemente     ***
c******************************************************************************

      subroutine tranphs(trp,ns,ism,tran)

      parameter (EPSY = 0.0001)
      integer m(3,3), mp(3,3), me(3,3), ism(3,3)
      real t(3), tp(3), tran(3), trp(3)

      data me / 1,0,0,  0,1,0,  0,0,1 /


c --- Initialisierung:

      do 3 i=1,3
         trp(i) = 0.
 3    continue


      if(abs(tran(1)).gt.EPSY.or.abs(tran(2)).gt.EPSY.or.
     *   abs(tran(3)).gt.EPSY) then
         
c --- Die Matrix m und den Vektor t initialisieren:

        ns = 1
        do 5 i=1,3
           t(i) = tran(i)
           do 4 j=1,3
              m(i,j) = ism(i,j)
 4         continue
 5      continue

c --- Die Produktmatrix {mp|tp} = {ism|tran} * {m|t} berechnen:

 7      ns = ns + 1
        call symprd(ism,tran,m,t,mp,tp)

c --- Testen, ob erzeugte die Produktmatrix mp die Einheitsmatrix me ist
c     und ob die dazugehoerige Translation tp der Einheitsvektor ist.
c     Falls dies der Fall ist, ist das Symmetrieelement n nicht 
c     phasenschiebend: 

        iw = 0
        do 45 i=1,3
           do 44 j=1,3
              if(mp(i,j).ne.me(i,j)) iw = 1
44         continue
45      continue

        if(iw.eq.0) then

c --- Testen, ob nach ns-facher Anwendung des Symmetrieelementes {ism|tran}
c     der Translationsanteil tp von Null verschieden ist. Wenn ja, wird
c     der Translationsanteil pro Anwendung berechnet:

          if(abs(tp(1)).gt.EPSY.or.abs(tp(2)).gt.EPSY.or.
     *       abs(tp(3)).gt.EPSY) then
            do 46 i=1,3
               trp(i) = tp(i)/real(ns)
46          continue
          end if

        else

c --- Matrix m und den Vektor t auffuellen:

47        do 60 i=1,3
             t(i) = tp(i)
             do 50 j=1,3
                m(i,j) = mp(i,j)
50           continue
60        continue
          goto 7

        end if

      end if


      return
      end


c******************************************************************************
c***            Produkt zweier Symmetrieelemente                            ***
c******************************************************************************

c     {mc|tc} = {ma|ta} * {mb|tb} = {ma * mb| ma*tb + ta}

      subroutine symprd(ma,ta,mb,tb,mc,tc)

      integer ma(3,3), mb(3,3), mc(3,3)
      real ta(3), tb(3), tc(3)

      do 50 i=1,3
         tc(i) = ta(i)
         do 40 j=1,3
            tc(i) = tc(i) + ma(j,i) * tb(j)
            mc(j,i) = 0
            do 30 k=1,3
               mc(j,i) = mc(j,i) + ma(k,i) * mb(j,k)   
30          continue
40       continue
50    continue

      return
      end


c******************************************************************************
c***  Koordinaten der Position x bezuegl. der prim. Basis berechnen         ***
c******************************************************************************

      subroutine transf(intaus,x,xp,itr)

      character intaus*1
      real x(3), xp(3), pm(3,3,7), qm(3,3,7) 

c --- P-Matrizen in der Reihenfolge: P, I, F, A, B, C, R
c     (Transformation von der zentrierten in die primitive Zelle)

      data pm / 1.0, 0.0, 0.0,    0.0, 1.0, 0.0,    0.0, 0.0, 1.0,
     *         -0.5, 0.5, 0.5,    0.5,-0.5, 0.5,    0.5, 0.5,-0.5,
     *          0.0, 0.5, 0.5,    0.5, 0.0, 0.5,    0.5, 0.5, 0.0,
     *          1.0, 0.0, 0.0,    0.0, 0.5, 0.5,    0.0,-0.5, 0.5,
     *          0.5, 0.0,-0.5,    0.0, 1.0, 0.0,    0.5, 0.0, 0.5,
     *          0.5, 0.5, 0.0,   -0.5, 0.5, 0.0,    0.0, 0.0, 1.0,
     *          0.6666666667,-0.3333333333,-0.3333333333,
     *          0.3333333333, 0.3333333333,-0.6666666667,
     *          0.3333333333, 0.3333333333, 0.3333333333 /

c --- Q-Matrizen (Q = P^(-1)) in der Reihenfolge: P, I, F, A, B, C, R
c     (Transformation von der primitiven in die zentrierte Zelle)

      data qm / 1.0, 0.0, 0.0,    0.0, 1.0, 0.0,    0.0, 0.0, 1.0,
     *          0.0, 1.0, 1.0,    1.0, 0.0, 1.0,    1.0, 1.0, 0.0,
     *         -1.0, 1.0, 1.0,    1.0,-1.0, 1.0,    1.0, 1.0,-1.0,
     *          1.0, 0.0, 0.0,    0.0, 1.0,-1.0,    0.0, 1.0, 1.0,
     *          1.0, 0.0, 1.0,    0.0, 1.0, 0.0,   -1.0, 0.0, 1.0,
     *          1.0,-1.0, 0.0,    1.0, 1.0, 0.0,    0.0, 0.0, 1.0,
     *          1.0, 0.0, 1.0,   -1.0, 1.0, 1.0,    0.0,-1.0, 1.0 /

      if(intaus.eq.'P') then
        nc = 1
      else if(intaus.eq.'I') then
        nc = 2
      else if(intaus.eq.'F') then
        nc = 3
      else if(intaus.eq.'A') then
        nc = 4
      else if(intaus.eq.'B') then
        nc = 5
      else if(intaus.eq.'C') then
        nc = 6
      else if(intaus.eq.'R'.or.intaus.eq.'H') then
        nc = 7
      end if


c --- Koordinaten bezuegl. der primitiven Basis erzeugen:

      if(itr.eq.0) then
        do 11 i=1,3
           xp(i) = 0.0
           do 10 j=1,3
              xp(i) = xp(i) + qm(j,i,nc) * x(j)    
10         continue
11      continue

c --- Koordinaten bezueglich der zentrierten Basis erzeugen:

      else
        do 21 i=1,3
           x(i) = 0.0
           do 20 j=1,3
              x(i) = x(i) + pm(j,i,nc) * xp(j)    
20         continue
21      continue
      end if

      return
      end
c******************************************************************************
c***                Eine Zeile ins INPUT-File schreiben                     ***
c******************************************************************************

      subroutine writline(ka,line,maxline)

      character line*132, form*6

      form = '(a000)'

      do 10 n=0,maxline-1
         if(line(maxline-n:maxline-n).ne.' ') then
           npos = n
           goto 12
         end if
10    continue
      npos = maxline - 1

12    write(form(3:5),'(i3)') maxline - npos
      write(ka,fmt=form) line

      return
      end


c****************************************************************************
c***             Punktgruppe des Wellenvektors bestimmen                  ***
c****************************************************************************

      subroutine pktgrp(npg,cpgr,laue,numsy,nsym,NSY)

      integer numsy(NSY), npgr(48,32), maxg(32), icod(48)
      character cg(32)*7, cpgr*7

      data (cg(n),n=1,32)
     * /'1      ','1b     ','2      ','m      ','2/m    ','2 2 2  ',
     *  'm m 2  ','m m m  ','4      ','4b     ','4/m    ','4 2 2  ',
     *  '4 m m  ','4b 2 m ','4/m m m','3      ','3b     ','3 2    ',
     *  '3 m    ','3b m   ','6      ','6b     ','6/m    ','6 2 2  ',
     *  '6 m m  ','6b m 2 ','6/m m m','2 3    ','m 3    ','4b 3 m ',
     *  '4 3 2  ','m 3b m '/

      data maxg /1,2, 2,2,4, 4,4,8, 4,4,8,8,8,8,16, 3,6,6,6,12,
     *           6,6,12,12,12,12,24, 12,24,24,24,48/

      data  npgr(1, 1)         /1/
      data (npgr(k, 2),k=1, 2) /1,-1/
      data (npgr(k, 3),k=1, 2) /1,2/
      data (npgr(k, 4),k=1, 2) /1,-2/
      data (npgr(k, 5),k=1, 4) /1,2,-1,-2/
      data (npgr(k, 6),k=1, 4) /1,2,2,2/
      data (npgr(k, 7),k=1, 4) /1,2,-2,-2/
      data (npgr(k, 8),k=1, 8) /1,2,2,2,-1,-2,-2,-2/
      data (npgr(k, 9),k=1, 4) /1,4,43,2/
      data (npgr(k,10),k=1, 4) /1,2,-4,-43/
      data (npgr(k,11),k=1, 8) /1,4,43,2,-1,-4,-43,-2/
      data (npgr(k,12),k=1, 8) /1,4,43,2,2,2,2,2/
      data (npgr(k,13),k=1, 8) /1,4,43,2,-2,-2,-2,-2/
      data (npgr(k,14),k=1, 8) /1,2,2,2,-4,-43,-2,-2/
      data (npgr(k,15),k=1,16) /1,4,43,2,2,2,2,2,
     *                         -1,-4,-43,-2,-2,-2,-2,-2/
      data (npgr(k,16),k=1, 3) /1,3,32/
      data (npgr(k,17),k=1, 6) /1,3,32,-1,-3,-32/
      data (npgr(k,18),k=1, 6) /1,3,32,2,2,2/
      data (npgr(k,19),k=1, 6) /1,3,32,-2,-2,-2/
      data (npgr(k,20),k=1,12) /1,3,32,2,2,2,-1,-3,-32,-2,-2,-2/
      data (npgr(k,21),k=1, 6) /1,3,32,6,65,2/
      data (npgr(k,22),k=1, 6) /1,3,32,-6,-65,-2/
      data (npgr(k,23),k=1,12) /1,3,32,6,65,2,-1,-3,-32,-6,-65,-2/
      data (npgr(k,24),k=1,12) /1,3,32,6,65,2,2,2,2,2,2,2/
      data (npgr(k,25),k=1,12) /1,3,32,6,65,2,-2,-2,-2,-2,-2,-2/
      data (npgr(k,26),k=1,12) /1,3,32,2,2,2,-6,-65,-2,-2,-2,-2/
      data (npgr(k,27),k=1,24) /1,3,32,6,65,2,2,2,2,2,2,2,
     *                         -1,-3,-32,-6,-65,-2,-2,-2,-2,-2,-2,-2/
      data (npgr(k,28),k=1,12) /1,3,3,3,3,32,32,32,32,2,2,2/
      data (npgr(k,29),k=1,24) /1,3,3,3,3,32,32,32,32,2,2,2,
     *                         -1,-3,-3,-3,-3,-32,-32,-32,-32,
     *                         -2,-2,-2/
      data (npgr(k,30),k=1,24) /1,3,3,3,3,32,32,32,32,2,2,2,
     *                         -4,-4,-4,-43,-43,-43,
     *                         -2,-2,-2,-2,-2,-2/
      data (npgr(k,31),k=1,24) /1,3,3,3,3,32,32,32,32,4,4,4,
     *                         43,43,43,2,2,2,2,2,2,2,2,2/
      data (npgr(k,32),k=1,48) /1,3,3,3,3,32,32,32,32,4,4,4,
     *                         43,43,43,2,2,2,2,2,2,2,2,2,
     *                         -1,-3,-3,-3,-3,-32,-32,-32,-32,
     *                         -4,-4,-4,-43,-43,-43,-2,-2,-2,
     *                         -2,-2,-2,-2,-2,-2/

      if(laue.eq.6.or.laue.eq.7.or.laue.eq.8.or.
     *  laue.eq.9.or.laue.eq.15) then
        do 20 n=1,nsym
           if(numsy(n).eq.1) then
             icod(n) = 1
           else if(numsy(n).eq.2) then
             icod(n) = 3
           else if(numsy(n).eq.3) then
             icod(n) = 32
           else if(numsy(n).eq.4) then
             icod(n) = 6
           else if(numsy(n).eq.5) then
             icod(n) = 65
           else if(numsy(n).ge.6.and.numsy(n).le.12) then
             icod(n) = 2
           else if(numsy(n).eq.13) then
             icod(n) = -1
           else if(numsy(n).eq.14) then
             icod(n) = -3
           else if(numsy(n).eq.15) then
             icod(n) = -32
           else if(numsy(n).eq.16) then
             icod(n) = -6
           else if(numsy(n).eq.17) then
             icod(n) = -65
           else if(numsy(n).ge.18.and.numsy(n).le.24) then
             icod(n) = -2
           end if
20      continue
      else
        do 50 n=1,nsym
           if(numsy(n).eq.1) then
             icod(n) = 1
           else if(numsy(n).ge.2.and.numsy(n).le.5) then
             icod(n) = 3 
           else if(numsy(n).ge.6.and.numsy(n).le.9) then
             icod(n) = 32 
           else if(numsy(n).ge.10.and.numsy(n).le.12) then
             icod(n) = 4 
           else if(numsy(n).ge.13.and.numsy(n).le.15) then
             icod(n) = 43 
           else if(numsy(n).ge.16.and.numsy(n).le.24) then
             icod(n) = 2 
           else if(numsy(n).eq.25) then
             icod(n) = -1 
           else if(numsy(n).ge.26.and.numsy(n).le.29) then
             icod(n) = -3 
           else if(numsy(n).ge.30.and.numsy(n).le.33) then
             icod(n) = -32 
           else if(numsy(n).ge.34.and.numsy(n).le.36) then
             icod(n) = -4 
           else if(numsy(n).ge.37.and.numsy(n).le.39) then
             icod(n) = -43 
           else if(numsy(n).ge.40.and.numsy(n).le.48) then
             icod(n) = -2 
           end if
50      continue
      end if

      do 100 k=1,32 
         if(maxg(k).eq.nsym) then
           do 80 n=1,nsym
              if(icod(n).ne.npgr(n,k)) goto 100
80         continue
           npg = k
           cpgr = cg(k)
           return
         end if
100   continue

      write(*,'(2x,a)') 'ERROR: Point group not identified'
      stop

      end


******************************************************************************
c***         Irreduzible Darstellungen der Punktgruppen                     ***
c******************************************************************************

      subroutine irred(npgr,cmod,tau,nsym,ndim,idar,mpt,numsy,
     *                 NSY,IDM,IRD)
      
c --- c = cos(2*PI/6), s = sin(2*PI/6)

      parameter(MAX = 32)
      parameter(s = 0.866025403784 , c = 0.5, PI = 3.14159265359)

      character*4 cid(12,MAX), cmod(IRD)
      integer mpt(NSY), numsy(NSY)
      integer ldim(12,MAX), maxid(MAX), ndim(IRD), idph(3,3,48,12,MAX)
      real dr(3,3,48,12,MAX)
      complex tau(NSY,IDM,IDM,IRD)


c --- Punktgruppe 1 (C_1):

      data maxid(1) /1/
      data ldim(1,1) /1/
      data cid(1,1) /'A   '/  
      data dr(1,1,1,1,1),idph(1,1,1,1,1) 
     * /1,  0/

c --- Punktgruppe 1b (S_2):

      data maxid(2) /2/
      data (ldim(k,2),k=1,2) /1,1/
      data (cid(k,2),k=1,2) /'Ag  ','Au  '/
      data ((dr(1,1,n,k,2),idph(1,1,n,k,2),n=1,2),k=1,2) 
     * /1,  0,1,  0,
     *  1,  0,1,180/

c --- Punktgruppe 2 (C_2):

      data maxid(3) /2/
      data (ldim(k,3),k=1,2) /1,1/
      data (cid(k,3),k=1,2) /'A   ','B   '/
      data ((dr(1,1,n,k,3),idph(1,1,n,k,3),n=1,2),k=1,2) 
     * /1,  0,1,  0,
     *  1,  0,1,180/
    
c --- Punktgruppe m (C_1h): 

      data maxid(4) /2/
      data (ldim(k,4),k=1,2) /1,1/
      data (cid(k,4),k=1,2) /'A''  ','A'''''/
      data ((dr(1,1,n,k,4),idph(1,1,n,k,4),n=1,2),k=1,2) 
     * /1,  0,1,  0,
     *  1,  0,1,180/

c --- Punktgruppe 2/m (C_2h):

      data maxid(5) /4/
      data (ldim(k,5),k=1,4) /1,1,1,1/
      data (cid(k,5),k=1,4) /'Ag  ','Bg  ','Au  ','Bu  '/
      data ((dr(1,1,n,k,5),idph(1,1,n,k,5),n=1,4),k=1,4) 
     * /1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,180,1,  0,1,180,
     *  1,  0,1,  0,1,180,1,180,
     *  1,  0,1,180,1,180,1,  0/

c --- Punktgruppe 2 2 2 (D_2):

      data maxid(6) /4/
      data (ldim(k,6),k=1,4) /1,1,1,1/
      data (cid(k,6),k=1,4) /'A   ','B1  ','B2  ','B3  '/
      data ((dr(1,1,n,k,6),idph(1,1,n,k,6),n=1,4),k=1,4) 
     * /1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,180,1,180,
     *  1,  0,1,180,1,  0,1,180,
     *  1,  0,1,180,1,180,1,  0/

c --- Punktgruppe m m 2 (C_2v):

      data maxid(7) /4/
      data (ldim(k,7),k=1,4) /1,1,1,1/
      data (cid(k,7),k=1,4) /'A1  ','A2  ','B1  ','B2  '/
      data ((dr(1,1,n,k,7),idph(1,1,n,k,7),n=1,4),k=1,4) 
     * /1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,180,1,180,
     *  1,  0,1,180,1,  0,1,180,
     *  1,  0,1,180,1,180,1,  0/

c --- Punktgruppe m m m (D_2h):

      data maxid(8) /8/
      data (ldim(k,8),k=1,8) /1,1,1,1,1,1,1,1/
      data (cid(k,8),k=1,8) 
     * /'Ag  ','B1g ','B2g ','B3g ','Au  ','B1u ','B2u ','B3u '/
      data ((dr(1,1,n,k,8),idph(1,1,n,k,8),n=1,8),k=1,8) 
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,180,1,180,1,  0,1,  0,1,180,1,180,
     *  1,  0,1,180,1,  0,1,180,1,  0,1,180,1,  0,1,180,
     *  1,  0,1,180,1,180,1,  0,1,  0,1,180,1,180,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,
     *  1,  0,1,  0,1,180,1,180,1,180,1,180,1,  0,1,  0,
     *  1,  0,1,180,1,  0,1,180,1,180,1,  0,1,180,1,  0,
     *  1,  0,1,180,1,180,1,  0,1,180,1,  0,1,  0,1,180/

c --- Punktgruppe 4 (C_4):

      data maxid(9) /4/
      data (ldim(k,9),k=1,4) /1,1,1,1/
      data (cid(k,9),k=1,4) /'A   ','B   ','E1  ','E2  '/
      data ((dr(1,1,n,k,9),idph(1,1,n,k,9),n=1,4),k=1,4)
     * /1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,  0,
     *  1,  0,1, 90,1,270,1,180,
     *  1,  0,1,270,1, 90,1,180/

c --- Punktgruppe 4b (S_4):

      data maxid(10) /4/
      data (ldim(k,10),k=1,4) /1,1,1,1/
      data (cid(k,10),k=1,4) /'A   ','B   ','E1  ','E2  '/
      data ((dr(1,1,n,k,10),idph(1,1,n,k,10),n=1,4),k=1,4)
     * /1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,180,1,180, 
     *  1,  0,1,180,1, 90,1,270,
     *  1,  0,1,180,1,270,1, 90/

c --- Punktgruppe 4/m (C_4h):

      data maxid(11) /8/
      data (ldim(k,11),k=1,8) /1,1,1,1,1,1,1,1/
      data (cid(k,11),k=1,8)
     * /'Ag  ','Bg  ','E1g ','E2g ','Au  ','Bu  ','E1u ','E2u '/ 
      data ((dr(1,1,n,k,11),idph(1,1,n,k,11),n=1,8),k=1,8)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,  0,1,  0,1,180,1,180,1,  0, 
     *  1,  0,1, 90,1,270,1,180,1,  0,1, 90,1,270,1,180,
     *  1,  0,1,270,1, 90,1,180,1,  0,1,270,1, 90,1,180,
     *  1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,
     *  1,  0,1,180,1,180,1,  0,1,180,1,  0,1,  0,1,180,
     *  1,  0,1, 90,1,270,1,180,1,180,1,270,1, 90,1,  0,
     *  1,  0,1,270,1, 90,1,180,1,180,1, 90,1,270,1,  0/

c     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
c     *  1,  0,1,180,1,180,1,  0,1,  0,1,180,1,180,1,  0, 
c     *  1,  0,1, 90,1,270,1,180,1,  0,1,270,1, 90,1,180,
c     *  1,  0,1,270,1, 90,1,180,1,  0,1, 90,1,270,1,180,
c     *  1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,
c     *  1,  0,1,180,1,180,1,  0,1,180,1,  0,1,  0,1,180,
c     *  1,  0,1, 90,1,270,1,180,1,180,1, 90,1,270,1,  0,
c     *  1,  0,1,270,1, 90,1,180,1,180,1,270,1, 90,1,  0/

c --- Punktgruppe 4 2 2 (D_4):

      data maxid(12) /5/
      data (ldim(k,12),k=1,5) /1,1,1,1,2/
      data (cid(k,12),k=1,5) /'A1  ','A2  ','B1  ','B2  ','E   '/
      data ((dr(1,1,n,k,12),idph(1,1,n,k,12),n=1,8),k=1,4)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,180,1,180,1,  0,1,180,1,180,
     *  1,  0,1,180,1,180,1,  0,1,  0,1,  0,1,180,1,180,
     *  1,  0,1,180,1,180,1,180,1,180,1,  0,1,  0,1,  0/
      data (((dr(j,i,n,5,12),idph(j,i,n,5,12),n=1,8),j=1,2),i=1,2)
     * /1,  0,0,  0,0,  0,1,  0,1,180,1,180,0,  0,0,  0,
     *  0,  0,1,180,1,  0,0,  0,0,  0,0,  0,1,  0,1,180,
     *  0,  0,1,  0,1,180,0,  0,0,  0,0,  0,1,  0,1,180,
     *  1,  0,0,  0,0,  0,1,180,1,  0,1,180,0,  0,0,  0/

c --- Punktgruppe 4 m m (C_4v):

      data maxid(13) /5/
      data (ldim(k,13),k=1,5) /1,1,1,1,2/
      data (cid(k,13),k=1,5) /'A1  ','A2  ','B1  ','B2  ','E   '/
      data ((dr(1,1,n,k,13),idph(1,1,n,k,13),n=1,8),k=1,4)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,
     *  1,  0,1,180,1,180,1,  0,1,  0,1,  0,1,180,1,180,
     *  1,  0,1,180,1,180,1,  0,1,180,1,180,1,  0,1,  0/
      data (((dr(j,i,n,5,13),idph(j,i,n,5,13),n=1,8),j=1,2),i=1,2)
     * /1,  0,0,  0,0,  0,1,180,1,  0,1,180,0,  0,0,  0,
     *  0,  0,1,180,1,  0,0,  0,0,  0,0,  0,1,  0,1,180,
     *  0,  0,1,  0,1,180,0,  0,0,  0,0,  0,1,  0,1,180,
     *  1,  0,0,  0,0,  0,1,180,1,180,1,  0,0,  0,0,  0/

c --- Punktgruppe 4b 2 m (D_2d):

      data maxid(14) /5/
      data (ldim(k,14),k=1,5) /1,1,1,1,2/
      data (cid(k,14),k=1,5) /'A1  ','A2  ','B1  ','B2  ','E   '/
      data ((dr(1,1,n,k,14),idph(1,1,n,k,14),n=1,8),k=1,4)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,  0,1,  0,1,  0,1,180,1,180,
     *  1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,
     *  1,  0,1,180,1,180,1,  0,1,180,1,180,1,  0,1,  0/
      data (((dr(j,i,n,5,14),idph(j,i,n,5,14),n=1,8),j=1,2),i=1,2)
     * /1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,1,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,1,  0,1,180,1,  0,
     *  1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,0,  0/

c --- Punktgruppe 4/m m m (D_4h):

      data maxid(15) /10/
      data (ldim(k,15),k=1,10) /1,1,1,1,2,1,1,1,1,2/
      data (cid(k,15),k=1,10) 
     *     /'A1g ','A2g ','B1g ','B2g ','Eg  ',
     *      'A1u ','A2u ','B1u ','B2u ','Eu  '/

      data (dr(1,1,n,1,15),idph(1,1,n,1,15),n=1,16)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0/

      data (dr(1,1,n,2,15),idph(1,1,n,2,15),n=1,16)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,  0,1,180,1,180,
     *  1,  0,1,  0,1,  0,1,180,1,180,1,  0,1,180,1,180/

      data (dr(1,1,n,3,15),idph(1,1,n,3,15),n=1,16)
     * /1,  0,1,180,1,180,1,  0,1,  0,1,  0,1,180,1,180,
     *  1,  0,1,180,1,180,1,  0,1,  0,1,  0,1,180,1,180/

      data (dr(1,1,n,4,15),idph(1,1,n,4,15),n=1,16)
     * /1,  0,1,180,1,180,1,180,1,180,1,  0,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,180,1,180,1,  0,1,  0,1,  0/

      data (dr(1,1,n,5,15),idph(1,1,n,5,15),n=1,16)
     * /1,  0,0,  0,0,  0,1,180,1,  0,1,180,0,  0,0,  0,
     *  1,  0,0,  0,0,  0,1,180,1,  0,1,180,0,  0,0,  0/
      data (dr(2,1,n,5,15),idph(2,1,n,5,15),n=1,16)
     * /0,  0,1,180,1,  0,0,  0,0,  0,0,  0,1,180,1,  0,
     *  0,  0,1,180,1,  0,0,  0,0,  0,0,  0,1,180,1,  0/
      data (dr(1,2,n,5,15),idph(1,2,n,5,15),n=1,16)
     * /0,  0,1,  0,1,180,0,  0,0,  0,0,  0,1,180,1,  0,
     *  0,  0,1,  0,1,180,0,  0,0,  0,0,  0,1,180,1,  0/
      data (dr(2,2,n,5,15),idph(2,2,n,5,15),n=1,16)
     * /1,  0,0,  0,0,  0,1,  0,1,180,1,180,0,  0,0,  0,
     *  1,  0,0,  0,0,  0,1,  0,1,180,1,180,0,  0,0,  0/

      data (dr(1,1,n,6,15),idph(1,1,n,6,15),n=1,16)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,180,1,180,1,180,1,180,1,180,1,180,1,180,1,180/

      data (dr(1,1,n,7,15),idph(1,1,n,7,15),n=1,16)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,  0,1,180,1,180,
     *  1,180,1,180,1,180,1,  0,1,  0,1,180,1,  0,1,  0/

      data (dr(1,1,n,8,15),idph(1,1,n,8,15),n=1,16)
     * /1,  0,1,180,1,180,1,  0,1,  0,1,  0,1,180,1,180,
     *  1,180,1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0/

      data (dr(1,1,n,9,15),idph(1,1,n,9,15),n=1,16)
     * /1,  0,1,180,1,180,1,180,1,180,1,  0,1,  0,1,  0,
     *  1,180,1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180/

      data (dr(1,1,n,10,15),idph(1,1,n,10,15),n=1,16)
     * /1,  0,0,  0,0,  0,1,180,1,  0,1,180,0,  0,0,  0,
     *  1,180,0,  0,0,  0,1,  0,1,180,1,  0,0,  0,0,  0/
      data (dr(2,1,n,10,15),idph(2,1,n,10,15),n=1,16)
     * /0,  0,1,180,1,  0,0,  0,0,  0,0,  0,1,180,1,  0,
     *  0,  0,1,  0,1,180,0,  0,0,  0,0,  0,1,  0,1,180/
      data (dr(1,2,n,10,15),idph(1,2,n,10,15),n=1,16)
     * /0,  0,1,  0,1,180,0,  0,0,  0,0,  0,1,180,1,  0,
     *  0,  0,1,180,1,  0,0,  0,0,  0,0,  0,1,  0,1,180/
      data (dr(2,2,n,10,15),idph(2,2,n,10,15),n=1,16)
     * /1,  0,0,  0,0,  0,1,  0,1,180,1,180,0,  0,0,  0,
     *  1,180,0,  0,0,  0,1,180,1,  0,1,  0,0,  0,0,  0/

c --- Punktgruppe 3 (C_3): 

      data maxid(16) /3/
      data (ldim(k,16),k=1,3) /1,1,1/
      data (cid(k,16),k=1,3) /'A   ','E11 ','E22 '/
      data ((dr(1,1,n,k,16),idph(1,1,n,k,16),n=1,3),k=1,3)
     * /1,  0,1,  0,1,  0,
     *  1,  0,1,120,1,240,
     *  1,  0,1,240,1,120/

c --- Punktgruppe 3b (S_6):

      data maxid(17) /6/
      data (ldim(k,17),k=1,6) /1,1,1,1,1,1/
      data (cid(k,17),k=1,6) /'Ag  ','Eg1 ','Eg2 ','Au  ','Eu1 ','Eu2 '/
      data ((dr(1,1,n,k,17),idph(1,1,n,k,17),n=1,6),k=1,6)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,120,1,240,1,  0,1,120,1,240,
     *  1,  0,1,240,1,120,1,  0,1,240,1,120,
     *  1,  0,1,  0,1,  0,1,180,1,180,1,180,
     *  1,  0,1,120,1,240,1,180,1,300,1, 60,
     *  1,  0,1,240,1,120,1,180,1, 60,1,300/

c --- Punktgruppe 3 2 (D_3):

      data maxid(18) /3/
      data (ldim(k,18),k=1,3) /1,1,2/
      data (cid(k,18),k=1,3) /'A1  ','A2  ','E   '/
      data ((dr(1,1,n,k,18),idph(1,1,n,k,18),n=1,6),k=1,2)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,180,1,180,1,180/
      data (((dr(j,i,n,3,18),idph(j,i,n,3,18),n=1,6),j=1,2),i=1,2)
     * /1,  0,c,180,c,180, 1,  0, c,180,c,180,
     *  0,  0,s,180,s,  0, 0,  0, s,180,s,  0,
     *  0,  0,s,  0,s,180, 0,  0, s,180,s,  0, 
     *  1,  0,c,180,c,180, 1,180, c,  0,c,  0/

c --- Punktgruppe 3 m (C_3v):

      data maxid(19) /3/
      data (ldim(k,19),k=1,3) /1,1,2/
      data (cid(k,19),k=1,3) /'A1  ','A2  ','E   '/
      data ((dr(1,1,n,k,19),idph(1,1,n,k,19),n=1,6),k=1,2)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,180,1,180,1,180/
      data (((dr(j,i,n,3,19),idph(j,i,n,3,19),n=1,6),j=1,2),i=1,2)
     * /1,  0,c,180,c,180,1,  0,c,180,c,180,
     *  0,  0,s,180,s,  0,0,  0,s,180,s,  0,
     *  0,  0,s,  0,s,180,0,  0,s,180,s,  0,
     *  1,  0,c,180,c,180,1,180,c,  0,c,  0/

c --- Punktgruppe 3b m (D_3h oder D_3d):

      data maxid(20) /6/
      data (ldim(k,20),k=1,6) /1,1,2,1,1,2/
      data (cid(k,20),k=1,6) /'A1g ','A2g ','Eg  ','A1u ','A2u ','Eu  '/
      data (dr(1,1,n,1,20),idph(1,1,n,1,20),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,2,20),idph(1,1,n,2,20),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0,1,  0,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,3,20),idph(1,1,n,3,20),n=1,12)
     * /1,  0,c,180,c,180,1,  0,c,180,c,180,1,  0,c,180,c,180,
     *  1,  0,c,180,c,180/
      data (dr(2,1,n,3,20),idph(2,1,n,3,20),n=1,12)
     * /0,  0,s,180,s,  0,0,  0,s,180,s,  0,0,  0,s,180,s,  0,
     *  0,  0,s,180,s,  0/
      data (dr(1,2,n,3,20),idph(1,2,n,3,20),n=1,12)
     * /0,  0,s,  0,s,180,0,  0,s,180,s,  0,0,  0,s,  0,s,180,
     *  0,  0,s,180,s,  0/
      data (dr(2,2,n,3,20),idph(2,2,n,3,20),n=1,12)
     * /1,  0,c,180,c,180,1,180,c,  0,c,  0,1,  0,c,180,c,180,
     *  1,180,c,  0,c,  0/
      data (dr(1,1,n,4,20),idph(1,1,n,4,20),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,5,20),idph(1,1,n,5,20),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,6,20),idph(1,1,n,6,20),n=1,12)
     * /1,  0,c,180,c,180,1,  0,c,180,c,180,1,180,c,  0,c,  0,
     *  1,180,c,  0,c,  0/
      data (dr(2,1,n,6,20),idph(2,1,n,6,20),n=1,12)
     * /0,  0,s,180,s,  0,0,  0,s,180,s,  0,0,  0,s,  0,s,180,
     *  0,180,s,  0,s,180/
      data (dr(1,2,n,6,20),idph(1,2,n,6,20),n=1,12)
     * /0,  0,s,  0,s,180,0,  0,s,180,s,  0,0,  0,s,180,s,  0,
     *  0,180,s,  0,s,180/
      data (dr(2,2,n,6,20),idph(2,2,n,6,20),n=1,12)
     * /1,  0,c,180,c,180,1,180,c,  0,c,  0,1,180,c,  0,c,  0,
     *  1,  0,c,180,c,180/

c --- Punktgruppe 6 (C_6):

      data maxid(21) /6/
      data (ldim(k,21),k=1,6) /1,1,1,1,1,1/
      data (cid(k,21),k=1,6) /'A   ','B   ','E11 ','E12 ','E21 ','E22 '/
      data ((dr(1,1,n,k,21),idph(1,1,n,k,21),n=1,6),k=1,6)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,180,1,180,1,180,
     *  1,  0,1,120,1,240,1, 60,1,300,1,180,
     *  1,  0,1,240,1,120,1,300,1, 60,1,180,
     *  1,  0,1,120,1,240,1,240,1,120,1,  0,
     *  1,  0,1,240,1,120,1,120,1,240,1,  0/

c --- Punktgruppe 6b (C_3h):

      data maxid(22) /6/
      data (ldim(k,22),k=1,6) /1,1,1,1,1,1/
      data (cid(k,22),k=1,6) 
     * /'A'' ','A'''' ','E''''1','E''''2','E''1 ','E''2 '/
      data ((dr(1,1,n,k,22),idph(1,1,n,k,22),n=1,6),k=1,6)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,180,1,180,1,180,
     *  1,  0,1,120,1,240,1, 60,1,300,1,180,
     *  1,  0,1,240,1,120,1,300,1, 60,1,180,
     *  1,  0,1,120,1,240,1,240,1,120,1,  0,
     *  1,  0,1,240,1,120,1,120,1,240,1,  0/

c --- Punktgruppe 6/m (C_6h):

      data maxid(23) /12/
      data (ldim(k,23),k=1,12) /1,1,1,1,1,1,1,1,1,1,1,1/
      data (cid(k,23),k=1,12) 
     * /'Ag  ','Bg  ','Eg11','Eg12','Eg21','Eg22',
     *  'Au  ','Bu  ','Eu11','Eu12','Eu21','Eu22'/
      data (dr(1,1,n,1,23),idph(1,1,n,1,23),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,2,23),idph(1,1,n,2,23),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0,1,  0,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,3,23),idph(1,1,n,3,23),n=1,12)
     * /1,  0,1,120,1,240,1, 60,1,300,1,180,1,  0,1,120,1,240,
     *  1, 60,1,300,1,180/
      data (dr(1,1,n,4,23),idph(1,1,n,4,23),n=1,12)
     * /1,  0,1,240,1,120,1,300,1, 60,1,180,1,  0,1,240,1,120,
     *  1,300,1, 60,1,180/
      data (dr(1,1,n,5,23),idph(1,1,n,5,23),n=1,12)
     * /1,  0,1,120,1,240,1,240,1,120,1,  0,1,  0,1,120,1,240,
     *  1,240,1,120,1,  0/
      data (dr(1,1,n,6,23),idph(1,1,n,6,23),n=1,12)
     * /1,  0,1,240,1,120,1,120,1,240,1,  0,1,  0,1,240,1,120,
     *  1,120,1,240,1,  0/
      data (dr(1,1,n,7,23),idph(1,1,n,7,23),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,8,23),idph(1,1,n,8,23),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,9,23),idph(1,1,n,9,23),n=1,12)
     * /1,  0,1,120,1,240,1, 60,1,300,1,180,1,180,1,300,1, 60,
     *  1,240,1,120,1,  0/
      data (dr(1,1,n,10,23),idph(1,1,n,10,23),n=1,12)
     * /1,  0,1,240,1,120,1,300,1, 60,1,180,1,180,1, 60,1,300,
     *  1,120,1,240,1,  0/
      data (dr(1,1,n,11,23),idph(1,1,n,11,23),n=1,12)
     * /1,  0,1,120,1,240,1,240,1,120,1,  0,1,180,1,300,1, 60,
     *  1, 60,1,300,1,180/
      data (dr(1,1,n,12,23),idph(1,1,n,12,23),n=1,12)
     * /1,  0,1,240,1,120,1,120,1,240,1,  0,1,180,1, 60,1,300,
     *  1,300,1, 60,1,180/

c --- Punktgruppe 6 2 2 (D_6):

      data maxid(24) /6/
      data (ldim(k,24),k=1,6) /1,1,1,1,2,2/
      data (cid(k,24),k=1,6) /'A1  ','A2  ','B1  ','B2  ','E1  ','E2  '/

      data (dr(1,1,n,1,24),idph(1,1,n,1,24),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,2,24),idph(1,1,n,2,24),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,3,24),idph(1,1,n,3,24),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0,1,  0,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,4,24),idph(1,1,n,4,24),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,  0,1,  0,1,  0/

      data (dr(1,1,n,5,24),idph(1,1,n,5,24),n=1,12)
     * /1,  0,c,180,c,180,c,  0,c,  0,1,180,1,  0,c,180,c,180,
     *  c,  0,1,180,c,  0/
      data (dr(2,1,n,5,24),idph(2,1,n,5,24),n=1,12)
     * /0,  0,s,180,s,  0,s,180,s,  0,0,  0,0,  0,s,180,s,  0,
     *  s,180,0,  0,s,  0/
      data (dr(1,2,n,5,24),idph(1,2,n,5,24),n=1,12)
     * /0,  0,s,  0,s,180,s,  0,s,180,0,  0,0,  0,s,180,s,  0,
     *  s,180,0,  0,s,  0/
      data (dr(2,2,n,5,24),idph(2,2,n,5,24),n=1,12)
     * /1,  0,c,180,c,180,c,  0,c,  0,1,180,1,180,c,  0,c,  0,
     *  c,180,1,  0,c,180/    

      data (dr(1,1,n,6,24),idph(1,1,n,6,24),n=1,12)
     * /1,  0,c,180,c,180,c,180,c,180,1,  0,1,  0,c,180,c,180,
     *  c,180,1,  0,c,180/
      data (dr(2,1,n,6,24),idph(2,1,n,6,24),n=1,12)
     * /0,  0,s,180,s,  0,s,  0,s,180,0,  0,0,  0,s,180,s,  0,
     *  s,  0,0,  0,s,180/
      data (dr(1,2,n,6,24),idph(1,2,n,6,24),n=1,12)
     * /0,  0,s,  0,s,180,s,180,s,  0,0,  0,0,  0,s,180,s,  0,
     *  s,  0,0,  0,s,180/ 
      data (dr(2,2,n,6,24),idph(2,2,n,6,24),n=1,12)
     * /1,  0,c,180,c,180,c,180,c,180,1,  0,1,180,c,  0,c,  0,
     *  c,  0,1,180,c,  0/

c --- Punktgruppe 6 m m (C_6v):

      data maxid(25) /6/
      data (ldim(k,25),k=1,6) /1,1,1,1,2,2/
      data (cid(k,25),k=1,6) /'A1  ','A2  ','B1  ','B2  ','E1  ','E2  '/

      data (dr(1,1,n,1,25),idph(1,1,n,1,25),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,2,25),idph(1,1,n,2,25),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,3,25),idph(1,1,n,3,25),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0,1,  0,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,4,25),idph(1,1,n,4,25),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,  0,1,  0,1,  0/

      data (dr(1,1,n,5,25),idph(1,1,n,5,25),n=1,12)
     * /1,  0,c,180,c,180,c,  0,c,  0,1,180,1,  0,c,180,c,180,
     *  c,  0,1,180,c,  0/
      data (dr(2,1,n,5,25),idph(2,1,n,5,25),n=1,12)
     * /0,  0,s,180,s,  0,s,180,s,  0,0,  0,0,  0,s,180,s,  0,
     *  s,180,0,  0,s,  0/
      data (dr(1,2,n,5,25),idph(1,2,n,5,25),n=1,12)
     * /0,  0,s,  0,s,180,s,  0,s,180,0,  0,0,  0,s,180,s,  0,
     *  s,180,0,  0,s,  0/
      data (dr(2,2,n,5,25),idph(2,2,n,5,25),n=1,12)
     * /1,  0,c,180,c,180,c,  0,c,  0,1,180,1,180,c,  0,c,  0,
     *  c,180,1,  0,c,180/    

      data (dr(1,1,n,6,25),idph(1,1,n,6,25),n=1,12)
     * /1,  0,c,180,c,180,c,180,c,180,1,  0,1,  0,c,180,c,180,
     *  c,180,1,  0,c,180/
      data (dr(2,1,n,6,25),idph(2,1,n,6,25),n=1,12)
     * /0,  0,s,180,s,  0,s,  0,s,180,0,  0,0,  0,s,180,s,  0,
     *  s,  0,0,  0,s,180/
      data (dr(1,2,n,6,25),idph(1,2,n,6,25),n=1,12)
     * /0,  0,s,  0,s,180,s,180,s,  0,0,  0,0,  0,s,180,s,  0,
     *  s,  0,0,  0,s,180/ 
      data (dr(2,2,n,6,25),idph(2,2,n,6,25),n=1,12)
     * /1,  0,c,180,c,180,c,180,c,180,1,  0,1,180,c,  0,c,  0,
     *  c,  0,1,180,c,  0/

c --- Punktgruppe 6b m 2 (D_3h):

      data maxid(26) /6/
      data (ldim(k,26),k=1,6) /1,1,2,1,1,2/
      data (cid(k,26),k=1,6) 
     * /'A1'' ','A2'' ','E''  ','A1''''','A2''''','E'''' '/
      data (dr(1,1,n,1,26),idph(1,1,n,1,26),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,2,26),idph(1,1,n,2,26),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0,1,  0,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,3,26),idph(1,1,n,3,26),n=1,12)
     * /1,  0,c,180,c,180,1,  0,c,180,c,180,c,180,c,180,1,  0,
     *  c,180,c,180,1,  0/
      data (dr(2,1,n,3,26),idph(2,1,n,3,26),n=1,12)
     * /0,  0,s,180,s,  0,0,  0,s,180,s,  0,s,  0,s,180,0,  0,
     *  s,180,s,  0,0,  0/
      data (dr(1,2,n,3,26),idph(1,2,n,3,26),n=1,12)
     * /0,  0,s,  0,s,180,0,  0,s,180,s,  0,s,180,s,  0,0,  0,
     *  s,180,s,  0,0,  0/
      data (dr(2,2,n,3,26),idph(2,2,n,3,26),n=1,12)
     * /1,  0,c,180,c,180,1,180,c,  0,c,  0,c,180,c,180,1,  0,
     *  c,  0,c,  0,1,180/
      data (dr(1,1,n,4,26),idph(1,1,n,4,26),n=1,12)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,5,26),idph(1,1,n,5,26),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,
     *  1,180,1,180,1,180/
      data (dr(1,1,n,6,26),idph(1,1,n,6,26),n=1,12)
     * /1,  0,c,180,c,180,1,180,c,  0,c,  0,c,  0,c,  0,1,180,
     *  c,180,c,180,1,  0/
      data (dr(2,1,n,6,26),idph(2,1,n,6,26),n=1,12)
     * /0,  0,s,180,s,  0,0,  0,s,  0,s,180,s,180,s,  0,0,  0,
     *  s,180,s,  0,0,  0/
      data (dr(1,2,n,6,26),idph(1,2,n,6,26),n=1,12)
     * /0,  0,s,  0,s,180,0,  0,s,  0,s,180,s,  0,s,180,0,  0,
     *  s,180,s,  0,0,  0/
      data (dr(2,2,n,6,26),idph(2,2,n,6,26),n=1,12)
     * /1,  0,c,180,c,180,1,  0,c,180,c,180,c,  0,c,  0,1,180,
     *  c,  0,c,  0,1,180/

c --- Punktgruppe 6/m m m (D_6h):

      data maxid(27) /12/
      data (ldim(k,27),k=1,12) /1,1,1,1,2,2,1,1,1,1,2,2/
      data (cid(k,27),k=1,12) 
     * /'A1g ','A2g ','B1g ','B2g ','E1g ','E2g ',
     *  'A1u ','A2u ','B1u ','B2u ','E1u ','E2u '/

      data (dr(1,1,n,1,27),idph(1,1,n,1,27),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0/      
      data (dr(1,1,n,2,27),idph(1,1,n,2,27),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,1,180,
     *  1,180,1,180,1,180,1,180,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180/    
      data (dr(1,1,n,3,27),idph(1,1,n,3,27),n=1,24)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,180,1,  0,1,  0,1,  0,1,180,
     *  1,180,1,180,1,  0,1,  0,1,  0,1,180,1,180,1,180/
      data (dr(1,1,n,4,27),idph(1,1,n,4,27),n=1,24)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,
     *  1,180,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,
     *  1,180,1,180,1,180,1,180,1,180,1,  0,1,  0,1,  0/

      data (dr(1,1,n,5,27),idph(1,1,n,5,27),n=1,24)
     * /1,  0,c,180,c,180,c,  0,c,  0,1,180,1,  0,c,180, 
     *  c,180,c,  0,1,180,c,  0,1,  0,c,180,c,180,c,  0,
     *  c,  0,1,180,1,  0,c,180,c,180,c,  0,1,180,c,  0/  
      data (dr(2,1,n,5,27),idph(2,1,n,5,27),n=1,24)
     * /0,  0,s,180,s,  0,s,180,s,  0,0,  0,0,  0,s,180,
     *  s,  0,s,180,0,  0,s,  0,0,  0,s,180,s,  0,s,180,
     *  s,  0,0,  0,0,  0,s,180,s,  0,s,180,0,  0,s,  0/
      data (dr(1,2,n,5,27),idph(1,2,n,5,27),n=1,24)
     * /0,  0,s,  0,s,180,s,  0,s,180,0,  0,0,  0,s,180,
     *  s,  0,s,180,0,  0,s,  0,0,  0,s,  0,s,180,s,  0,
     *  s,180,0,  0,0,  0,s,180,s,  0,s,180,0,  0,s,  0/
      data (dr(2,2,n,5,27),idph(2,2,n,5,27),n=1,24)
     * /1,  0,c,180,c,180,c,  0,c,  0,1,180,1,180,c,  0,
     *  c,  0,c,180,1,  0,c,180,1,  0,c,180,c,180,c,  0,
     *  c,  0,1,180,1,180,c,  0,c,  0,c,180,1,  0,c,180/

      data (dr(1,1,n,6,27),idph(1,1,n,6,27),n=1,24)
     * /1,  0,c,180,c,180,c,180,c,180,1,  0,1,  0,c,180,
     *  c,180,c,180,1,  0,c,180,1,  0,c,180,c,180,c,180,
     *  c,180,1,  0,1,  0,c,180,c,180,c,180,1,  0,c,180/
      data (dr(2,1,n,6,27),idph(2,1,n,6,27),n=1,24)
     * /0,  0,s,180,s,  0,s,  0,s,180,0,  0,0,  0,s,180,
     *  s,  0,s,  0,0,  0,s,180,0,  0,s,180,s,  0,s,  0,
     *  s,180,0,  0,0,  0,s,180,s,  0,s,  0,0,  0,s,180/
      data (dr(1,2,n,6,27),idph(1,2,n,6,27),n=1,24)
     * /0,  0,s,  0,s,180,s,180,s,  0,0,  0,0,  0,s,180,
     *  s,  0,s,  0,0,  0,s,180,0,  0,s,  0,s,180,s,180,
     *  s,  0,0,  0,0,  0,s,180,s,  0,s,  0,0,  0,s,180/
      data (dr(2,2,n,6,27),idph(2,2,n,6,27),n=1,24)
     * /1,  0,c,180,c,180,c,180,c,180,1,  0,1,180,c,  0,
     *  c,  0,c,  0,1,180,c,  0,1,  0,c,180,c,180,c,180,
     *  c,180,1,  0,1,180,c,  0,c,  0,c,  0,1,180,c,  0/

      data (dr(1,1,n,7,27),idph(1,1,n,7,27),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,
     *  1,180,1,180,1,180,1,180,1,180,1,180,1,180,1,180/
      data (dr(1,1,n,8,27),idph(1,1,n,8,27),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,1,180,
     *  1,180,1,180,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,180,1,180,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0/
      data (dr(1,1,n,9,27),idph(1,1,n,9,27),n=1,24)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,180,1,180,1,180,1,180,1,  0,
     *  1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0,1,  0/ 
      data (dr(1,1,n,10,27),idph(1,1,n,10,27),n=1,24)
     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,
     *  1,180,1,  0,1,  0,1,  0,1,180,1,180,1,180,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180/

      data (dr(1,1,n,11,27),idph(1,1,n,11,27),n=1,24)
     * /1,  0,c,180,c,180,c,  0,c,  0,1,180,1,  0,c,180,
     *  c,180,c,  0,1,180,c,  0,1,180,c,  0,c,  0,c,180,
     *  c,180,1,  0,1,180,c,  0,c,  0,c,180,1,  0,c,180/
      data (dr(2,1,n,11,27),idph(2,1,n,11,27),n=1,24)
     * /0,  0,s,180,s,  0,s,180,s,  0,0,  0,0,  0,s,180,
     *  s,  0,s,180,0,  0,s,  0,0,  0,s,  0,s,180,s,  0,
     *  s,180,0,  0,0,  0,s,  0,s,180,s,  0,0,  0,s,180/
      data (dr(1,2,n,11,27),idph(1,2,n,11,27),n=1,24)
     * /0,  0,s,  0,s,180,s,  0,s,180,0,  0,0,  0,s,180,
     *  s,  0,s,180,0,  0,s,  0,0,  0,s,180,s,  0,s,180,
     *  s,  0,0,  0,0,  0,s,  0,s,180,s,  0,0,  0,s,180/
      data (dr(2,2,n,11,27),idph(2,2,n,11,27),n=1,24)
     * /1,  0,c,180,c,180,c,  0,c,  0,1,180,1,180,c,  0,
     *  c,  0,c,180,1,  0,c,180,1,180,c,  0,c,  0,c,180,
     *  c,180,1,  0,1,  0,c,180,c,180,c,  0,1,180,c,  0/

      data (dr(1,1,n,12,27),idph(1,1,n,12,27),n=1,24)
     * /1,  0,c,180,c,180,c,180,c,180,1,  0,1,  0,c,180,
     *  c,180,c,180,1,  0,c,180,1,180,c,  0,c,  0,c,  0,
     *  c,  0,1,180,1,180,c,  0,c,  0,c,  0,1,180,c,  0/
      data (dr(2,1,n,12,27),idph(2,1,n,12,27),n=1,24)
     * /0,  0,s,180,s,  0,s,  0,s,180,0,  0,0,  0,s,180,
     *  s,  0,s,  0,0,  0,s,180,0,  0,s,  0,s,180,s,180,
     *  s,  0,0,  0,0,  0,s,  0,s,180,s,180,0,  0,s,  0/
      data (dr(1,2,n,12,27),idph(1,2,n,12,27),n=1,24)
     * /0,  0,s,  0,s,180,s,180,s,  0,0,  0,0,  0,s,180,
     *  s,  0,s,  0,0,  0,s,180,0,  0,s,180,s,  0,s,  0,
     *  s,180,0,  0,0,  0,s,  0,s,180,s,180,0,  0,s,  0/
      data (dr(2,2,n,12,27),idph(2,2,n,12,27),n=1,24)
     * /1,  0,c,180,c,180,c,180,c,180,1,  0,1,180,c,  0,
     *  c,  0,c,  0,1,180,c,  0,1,180,c,  0,c,  0,c,  0,
     *  c,  0,1,180,1,  0,c,180,c,180,c,180,1,  0,c,180/

c --- Punktgruppe 23 (T):

      data maxid(28) /4/
      data (ldim(k,28),k=1,4) /1,1,1,3/
      data (cid(k,28),k=1,4) /'A   ','E11 ','E22 ','T   '/
      data (dr(1,1,n,1,28),idph(1,1,n,1,28),n=1,12)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,2,28),idph(1,1,n,2,28),n=1,12)
     * /1,  0,1,120,1,120,1,120,1,120,1,240,1,240,1,240,1,240,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,3,28),idph(1,1,n,3,28),n=1,12)
     * /1,  0,1,240,1,240,1,240,1,240,1,120,1,120,1,120,1,120,
     *  1,  0,1,  0,1,  0/
      data (dr(1,1,n,4,28),idph(1,1,n,4,28),n=1,12)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  1,  0,1,180,1,180/
      data (dr(2,1,n,4,28),idph(2,1,n,4,28),n=1,12)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,1,  0,
     *  0,  0,0,  0,0,  0/
      data (dr(3,1,n,4,28),idph(3,1,n,4,28),n=1,12)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0/
      data (dr(1,2,n,4,28),idph(1,2,n,4,28),n=1,12)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0/
      data (dr(2,2,n,4,28),idph(2,2,n,4,28),n=1,12)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  1,180,1,  0,1,180/
      data (dr(3,2,n,4,28),idph(3,2,n,4,28),n=1,12)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,1,180,
     *  0,  0,0,  0,0,  0/
      data (dr(1,3,n,4,28),idph(1,3,n,4,28),n=1,12)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,1,180,
     *  0,  0,0,  0,0,  0/
      data (dr(2,3,n,4,28),idph(2,3,n,4,28),n=1,12)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0/
      data (dr(3,3,n,4,28),idph(3,3,n,4,28),n=1,12)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  1,180,1,180,1,  0/

c --- Punktgruppe m 3b (T_h):

      data maxid(29) /8/
      data (ldim(k,29),k=1,8) /1,1,1,3,1,1,1,3/
      data (cid(k,29),k=1,8) 
     * /'Ag  ','Eg1 ','Eg2 ','Tg  ','Au  ','Eu1 ','Eu2 ','Tu  '/
      data (dr(1,1,n,1,29),idph(1,1,n,1,29),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0/
      data (dr(1,1,n,2,29),idph(1,1,n,2,29),n=1,24)
     * /1,  0,1,120,1,120,1,120,1,120,1,240,1,240,1,240,
     *  1,240,1,  0,1,  0,1,  0,1,  0,1,120,1,120,1,120,
     *  1,120,1,240,1,240,1,240,1,240,1,  0,1,  0,1,  0/
      data (dr(1,1,n,3,29),idph(1,1,n,3,29),n=1,24)
     * /1,  0,1,240,1,240,1,240,1,240,1,120,1,120,1,120,
     *  1,120,1,  0,1,  0,1,  0,1,  0,1,240,1,240,1,240,
     *  1,240,1,120,1,120,1,120,1,120,1,  0,1,  0,1,  0/

      data (dr(1,1,n,4,29),idph(1,1,n,4,29),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180/
      data (dr(2,1,n,4,29),idph(2,1,n,4,29),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0/
      data (dr(3,1,n,4,29),idph(3,1,n,4,29),n=1,24)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0/
      data (dr(1,2,n,4,29),idph(1,2,n,4,29),n=1,24)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0/
      data (dr(2,2,n,4,29),idph(2,2,n,4,29),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,  0,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,  0,1,180/
      data (dr(3,2,n,4,29),idph(3,2,n,4,29),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0/
      data (dr(1,3,n,4,29),idph(1,3,n,4,29),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0/
      data (dr(2,3,n,4,29),idph(2,3,n,4,29),n=1,24)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0/
      data (dr(3,3,n,4,29),idph(3,3,n,4,29),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,180,1,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,180,1,  0/

      data (dr(1,1,n,5,29),idph(1,1,n,5,29),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,
     *  1,180,1,180,1,180,1,180,1,180,1,180,1,180,1,180/
      data (dr(1,1,n,6,29),idph(1,1,n,6,29),n=1,24)
     * /1,  0,1,120,1,120,1,120,1,120,1,240,1,240,1,240,
     *  1,240,1,  0,1,  0,1,  0,1,180,1,300,1,300,1,300,
     *  1,300,1, 60,1, 60,1, 60,1, 60,1,180,1,180,1,180/
      data (dr(1,1,n,7,29),idph(1,1,n,7,29),n=1,24)
     * /1,  0,1,240,1,240,1,240,1,240,1,120,1,120,1,120,
     *  1,120,1,  0,1,  0,1,  0,1,180,1, 60,1, 60,1, 60,
     *  1, 60,1,300,1,300,1,300,1,300,1,180,1,180,1,180/

      data (dr(1,1,n,8,29),idph(1,1,n,8,29),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,1,180,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,  0,1,  0/
      data (dr(2,1,n,8,29),idph(2,1,n,8,29),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,  0,1,  0,1,180,0,  0,0,  0,0,  0/
      data (dr(3,1,n,8,29),idph(3,1,n,8,29),n=1,24)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,  0,1,180,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0/
      data (dr(1,2,n,8,29),idph(1,2,n,8,29),n=1,24)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,  0,1,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0/
      data (dr(2,2,n,8,29),idph(2,2,n,8,29),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0/
      data (dr(3,2,n,8,29),idph(3,2,n,8,29),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,180,1,  0,1,  0,0,  0,0,  0,0,  0/
      data (dr(1,3,n,8,29),idph(1,3,n,8,29),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,  0,1,180,1,  0,0,  0,0,  0,0,  0/
      data (dr(2,3,n,8,29),idph(2,3,n,8,29),n=1,24)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,180,1,  0,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0/
      data (dr(3,3,n,8,29),idph(3,3,n,8,29),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180/

c --- Punktgruppe 4b 3 m (T_d):

      data maxid(30) / 5/
      data (ldim(k,30),k=1, 5) /1,1,2,3,3/
      data (cid(k,30),k=1, 5) /'A1  ','A2  ','E   ','T1  ','T2  '/

      data (dr(1,1,n, 1,30),idph(1,1,n, 1,30),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0/

      data (dr(1,1,n, 2,30),idph(1,1,n, 2,30),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,
     *  1,180,1,180,1,180,1,180,1,180,1,180,1,180,1,180/

      data (dr(1,1,n, 3,30),idph(1,1,n, 3,30),n=1,24)
     * /1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,1,  0,1,  0,1,  0,c,180,c,180,1,  0,c,180,
     *  c,180,1,  0,1,  0,c,180,c,180,1,  0,c,180,c,180/
      data (dr(2,1,n, 3,30),idph(2,1,n, 3,30),n=1,24)
     * /0,  0,s,180,s,180,s,180,s,180,s,  0,s,  0,s,  0,
     *  s,  0,0,  0,0,  0,0,  0,s,180,s,  0,0,  0,s,180,
     *  s,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180/
      data (dr(1,2,n, 3,30),idph(1,2,n, 3,30),n=1,24)
     * /0,  0,s,  0,s,  0,s,  0,s,  0,s,180,s,180,s,180,
     *  s,180,0,  0,0,  0,0,  0,s,180,s,  0,0,  0,s,180,
     *  s,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180/
      data (dr(2,2,n, 3,30),idph(2,2,n, 3,30),n=1,24)
     * /1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,1,  0,1,  0,1,  0,c,  0,c,  0,1,180,c,  0,
     *  c,  0,1,180,1,180,c,  0,c,  0,1,180,c,  0,c,  0/

      data (dr(1,1,n, 4,30),idph(1,1,n, 4,30),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,1,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,180/
      data (dr(2,1,n, 4,30),idph(2,1,n, 4,30),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,1,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0/
      data (dr(3,1,n, 4,30),idph(3,1,n, 4,30),n=1,24)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  1,180,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0/
      data (dr(1,2,n, 4,30),idph(1,2,n, 4,30),n=1,24)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,1,180,1,  0,0,  0,0,  0,1,180,0,  0,0,  0/
      data (dr(2,2,n, 4,30),idph(2,2,n, 4,30),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,  0,1,180,0,  0,1,  0,0,  0,0,  0,
     *  1,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,180,0,  0/
      data (dr(3,2,n, 4,30),idph(3,2,n, 4,30),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180/
      data (dr(1,3,n, 4,30),idph(1,3,n, 4,30),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0/
      data (dr(2,3,n, 4,30),idph(2,3,n, 4,30),n=1,24)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180/
      data (dr(3,3,n, 4,30),idph(3,3,n, 4,30),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,180,1,  0,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,1,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0/

      data (dr(1,1,n, 5,30),idph(1,1,n, 5,30),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,1,180,1,180,1,180,0,  0,0,  0,1,180,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0/
      data (dr(2,1,n, 5,30),idph(2,1,n, 5,30),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,1,180,1,180,0,  0,0,  0,1,  0,0,  0,0,  0/
      data (dr(3,1,n, 5,30),idph(3,1,n, 5,30),n=1,24)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  1,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0/
      data (dr(1,2,n, 5,30),idph(1,2,n, 5,30),n=1,24)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,1,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0/
      data (dr(2,2,n, 5,30),idph(2,2,n, 5,30),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,  0,1,180,0,  0,1,180,0,  0,0,  0,
     *  1,180,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0/
      data (dr(3,2,n, 5,30),idph(3,2,n, 5,30),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0/
      data (dr(1,3,n, 5,30),idph(1,3,n, 5,30),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0/
      data (dr(2,3,n, 5,30),idph(2,3,n, 5,30),n=1,24)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0/
      data (dr(3,3,n, 5,30),idph(3,3,n, 5,30),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,1,180,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,1,180,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0/

c --- Punktgruppe 4 3 2 (O):

      data maxid(31) / 5/
      data (ldim(k,31),k=1, 5) /1,1,2,3,3/
      data (cid(k,31),k=1, 5) /'A1  ','A2  ','E   ','T1  ','T2  '/

      data (dr(1,1,n, 1,31),idph(1,1,n, 1,31),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0/

      data (dr(1,1,n, 2,31),idph(1,1,n, 2,31),n=1,24)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,180,1,180,1,180,1,180,1,  0,
     *  1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180/

      data (dr(1,1,n, 3,31),idph(1,1,n, 3,31),n=1,24)
     * /1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,c,180,c,180,1,  0,c,180,c,180,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,c,180,c,180,1,  0,c,180,c,180/
      data (dr(2,1,n, 3,31),idph(2,1,n, 3,31),n=1,24)
     * /0,  0,s,180,s,180,s,180,s,180,s,  0,s,  0,s,  0,
     *  s,  0,s,180,s,  0,0,  0,s,180,s,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180/
      data (dr(1,2,n, 3,31),idph(1,2,n, 3,31),n=1,24)
     * /0,  0,s,  0,s,  0,s,  0,s,  0,s,180,s,180,s,180,
     *  s,180,s,180,s,  0,0,  0,s,180,s,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180/
      data (dr(2,2,n, 3,31),idph(2,2,n, 3,31),n=1,24)
     * /1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,c,  0,c,  0,1,180,c,  0,c,  0,1,180,1,  0,
     *  1,  0,1,  0,1,180,c,  0,c,  0,1,180,c,  0,c,  0/

      data (dr(1,1,n, 4,31),idph(1,1,n, 4,31),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,
     *  1,180,1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,180/
      data (dr(2,1,n, 4,31),idph(2,1,n, 4,31),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0/
      data (dr(3,1,n, 4,31),idph(3,1,n, 4,31),n=1,24)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0/
      data (dr(1,2,n, 4,31),idph(1,2,n, 4,31),n=1,24)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0/
      data (dr(2,2,n, 4,31),idph(2,2,n, 4,31),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,1,180,
     *  1,  0,1,180,0,  0,1,180,0,  0,0,  0,1,180,0,  0/
      data (dr(3,2,n, 4,31),idph(3,2,n, 4,31),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180/
      data (dr(1,3,n, 4,31),idph(1,3,n, 4,31),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0/
      data (dr(2,3,n, 4,31),idph(2,3,n, 4,31),n=1,24)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180/
      data (dr(3,3,n, 4,31),idph(3,3,n, 4,31),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,1,180,
     *  1,180,1,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0/

      data (dr(1,1,n, 5,31),idph(1,1,n, 5,31),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  1,180,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0/
      data (dr(2,1,n, 5,31),idph(2,1,n, 5,31),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0/
      data (dr(3,1,n, 5,31),idph(3,1,n, 5,31),n=1,24)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0/
      data (dr(1,2,n, 5,31),idph(1,2,n, 5,31),n=1,24)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0/
      data (dr(2,2,n, 5,31),idph(2,2,n, 5,31),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,180,0,  0,1,180,
     *  1,  0,1,180,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0/
      data (dr(3,2,n, 5,31),idph(3,2,n, 5,31),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0/
      data (dr(1,3,n, 5,31),idph(1,3,n, 5,31),n=1,24)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0/
      data (dr(2,3,n, 5,31),idph(2,3,n, 5,31),n=1,24)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0/
      data (dr(3,3,n, 5,31),idph(3,3,n, 5,31),n=1,24)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,180,1,180,
     *  1,180,1,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0/

c --- Punktgruppe m 3b m (O_h):

      data maxid(32) /10/
      data (ldim(k,32),k=1,10) /1,1,2,3,3,1,1,2,3,3/
      data (cid(k,32),k=1,10) /'A1g ','A2g ','Eg  ','T1g ','T2g ',
     *                         'A1u ','A2u ','Eu  ','T1u ','T2u '/

      data (dr(1,1,n, 1,32),idph(1,1,n, 1,32),n=1,48)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0/

      data (dr(1,1,n, 2,32),idph(1,1,n, 2,32),n=1,48)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,180,1,180,1,180,1,180,1,  0,
     *  1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,180,1,180,1,180,1,180,1,  0,
     *  1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180/

      data (dr(1,1,n, 3,32),idph(1,1,n, 3,32),n=1,48)
     * /1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,c,180,c,180,1,  0,c,180,c,180,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,c,180,c,180,1,  0,c,180,c,180,
     *  1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,c,180,c,180,1,  0,c,180,c,180,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,c,180,c,180,1,  0,c,180,c,180/
      data (dr(2,1,n, 3,32),idph(2,1,n, 3,32),n=1,48)
     * /0,  0,s,180,s,180,s,180,s,180,s,  0,s,  0,s,  0,
     *  s,  0,s,180,s,  0,0,  0,s,180,s,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180,
     *  0,  0,s,180,s,180,s,180,s,180,s,  0,s,  0,s,  0,
     *  s,  0,s,180,s,  0,0,  0,s,180,s,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180/
      data (dr(1,2,n, 3,32),idph(1,2,n, 3,32),n=1,48)
     * /0,  0,s,  0,s,  0,s,  0,s,  0,s,180,s,180,s,180,
     *  s,180,s,180,s,  0,0,  0,s,180,s,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180,
     *  0,  0,s,  0,s,  0,s,  0,s,  0,s,180,s,180,s,180,
     *  s,180,s,180,s,  0,0,  0,s,180,s,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180/
      data (dr(2,2,n, 3,32),idph(2,2,n, 3,32),n=1,48)
     * /1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,c,  0,c,  0,1,180,c,  0,c,  0,1,180,1,  0,
     *  1,  0,1,  0,1,180,c,  0,c,  0,1,180,c,  0,c,  0,
     *  1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,c,  0,c,  0,1,180,c,  0,c,  0,1,180,1,  0,
     *  1,  0,1,  0,1,180,c,  0,c,  0,1,180,c,  0,c,  0/

      data (dr(1,1,n, 4,32),idph(1,1,n, 4,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,
     *  1,180,1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,180,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,
     *  1,180,1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,180/
      data (dr(2,1,n, 4,32),idph(2,1,n, 4,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0/
      data (dr(3,1,n, 4,32),idph(3,1,n, 4,32),n=1,48)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0/
      data (dr(1,2,n, 4,32),idph(1,2,n, 4,32),n=1,48)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0/
      data (dr(2,2,n, 4,32),idph(2,2,n, 4,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,1,180,
     *  1,  0,1,180,0,  0,1,180,0,  0,0,  0,1,180,0,  0,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,1,180,
     *  1,  0,1,180,0,  0,1,180,0,  0,0,  0,1,180,0,  0/
      data (dr(3,2,n, 4,32),idph(3,2,n, 4,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180/
      data (dr(1,3,n, 4,32),idph(1,3,n, 4,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0/
      data (dr(2,3,n, 4,32),idph(2,3,n, 4,32),n=1,48)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,
     *  0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180/
      data (dr(3,3,n, 4,32),idph(3,3,n, 4,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,1,180,
     *  1,180,1,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,1,180,
     *  1,180,1,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0/

      data (dr(1,1,n, 5,32),idph(1,1,n, 5,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  1,180,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  1,180,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0/
      data (dr(2,1,n, 5,32),idph(2,1,n, 5,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0/
      data (dr(3,1,n, 5,32),idph(3,1,n, 5,32),n=1,48)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0/
      data (dr(1,2,n, 5,32),idph(1,2,n, 5,32),n=1,48)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0/
      data (dr(2,2,n, 5,32),idph(2,2,n, 5,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,180,0,  0,1,180,
     *  1,  0,1,180,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,180,0,  0,1,180,
     *  1,  0,1,180,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0/
      data (dr(3,2,n, 5,32),idph(3,2,n, 5,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0/
      data (dr(1,3,n, 5,32),idph(1,3,n, 5,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0/
      data (dr(2,3,n, 5,32),idph(2,3,n, 5,32),n=1,48)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0/
      data (dr(3,3,n, 5,32),idph(3,3,n, 5,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,180,1,180,
     *  1,180,1,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,180,1,180,
     *  1,180,1,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0/

      data (dr(1,1,n, 6,32),idph(1,1,n, 6,32),n=1,48)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,180,1,180,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,180,1,180,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,180,1,180,1,180,1,180,1,180,1,180,1,180,1,180/

      data (dr(1,1,n, 7,32),idph(1,1,n, 7,32),n=1,48)
     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
     *  1,  0,1,180,1,180,1,180,1,180,1,180,1,180,1,  0,
     *  1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,180,1,180,1,180,1,180,1,180,1,180,1,180,1,180,
     *  1,180,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,
     *  1,180,1,180,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0/

      data (dr(1,1,n, 8,32),idph(1,1,n, 8,32),n=1,48)
     * /1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,c,180,c,180,1,  0,c,180,c,180,1,  0,1,  0,
     *  1,  0,1,  0,1,  0,c,180,c,180,1,  0,c,180,c,180,
     *  1,180,c,  0,c,  0,c,  0,c,  0,c,  0,c,  0,c,  0,
     *  c,  0,c,  0,c,  0,1,180,c,  0,c,  0,1,180,1,180,
     *  1,180,1,180,1,180,c,  0,c,  0,1,180,c,  0,c,  0/
      data (dr(2,1,n, 8,32),idph(2,1,n, 8,32),n=1,48)
     * /0,  0,s,180,s,180,s,180,s,180,s,  0,s,  0,s,  0,
     *  s,  0,s,180,s,  0,0,  0,s,180,s,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180,
     *  0,  0,s,  0,s,  0,s,  0,s,  0,s,180,s,180,s,180,
     *  s,180,s,  0,s,180,0,  0,s,  0,s,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,180,s,  0,0,  0,s,180,s,  0/
      data (dr(1,2,n, 8,32),idph(1,2,n, 8,32),n=1,48)
     * /0,  0,s,  0,s,  0,s,  0,s,  0,s,180,s,180,s,180,
     *  s,180,s,180,s,  0,0,  0,s,180,s,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,  0,s,180,0,  0,s,  0,s,180,
     *  0,  0,s,180,s,180,s,180,s,180,s,  0,s,  0,s,  0,
     *  s,  0,s,  0,s,180,0,  0,s,  0,s,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,s,180,s,  0,0,  0,s,180,s,  0/
      data (dr(2,2,n, 8,32),idph(2,2,n, 8,32),n=1,48)
     * /1,  0,c,180,c,180,c,180,c,180,c,180,c,180,c,180,
     *  c,180,c,  0,c,  0,1,180,c,  0,c,  0,1,180,1,  0,
     *  1,  0,1,  0,1,180,c,  0,c,  0,1,180,c,  0,c,  0,
     *  1,180,c,  0,c,  0,c,  0,c,  0,c,  0,c,  0,c,  0,
     *  c,  0,c,180,c,180,1,  0,c,180,c,180,1,  0,1,180,
     *  1,180,1,180,1,  0,c,180,c,180,1,  0,c,180,c,180/

      data (dr(1,1,n, 9,32),idph(1,1,n, 9,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,
     *  1,180,1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,180,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,180,
     *  1,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0/
      data (dr(2,1,n, 9,32),idph(2,1,n, 9,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,  0,1,  0,
     *  1,180,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0/
      data (dr(3,1,n, 9,32),idph(3,1,n, 9,32),n=1,48)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,1,180,1,  0,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0/
      data (dr(1,2,n, 9,32),idph(1,2,n, 9,32),n=1,48)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,1,180,1,  0,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0/
      data (dr(2,2,n, 9,32),idph(2,2,n, 9,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,1,180,
     *  1,  0,1,180,0,  0,1,180,0,  0,0,  0,1,180,0,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,180,0,  0,1,  0,
     *  1,180,1,  0,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0/
      data (dr(3,2,n, 9,32),idph(3,2,n, 9,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,180,1,  0,
     *  1,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0/
      data (dr(1,3,n, 9,32),idph(1,3,n, 9,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,  0,1,180,
     *  1,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0/
      data (dr(2,3,n, 9,32),idph(2,3,n, 9,32),n=1,48)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,
     *  0,  0,1,180,1,180,1,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0/
      data (dr(3,3,n, 9,32),idph(3,3,n, 9,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,1,180,
     *  1,180,1,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,180,1,  0,
     *  1,  0,1,180,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0/

      data (dr(1,1,n,10,32),idph(1,1,n,10,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  1,180,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,
     *  1,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,180/
      data (dr(2,1,n,10,32),idph(2,1,n,10,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,180,
     *  1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,  0,1,  0,
     *  1,180,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0/
      data (dr(3,1,n,10,32),idph(3,1,n,10,32),n=1,48)
     * /0,  0,1,  0,1,180,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,1,180,1,  0,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0/
      data (dr(1,2,n,10,32),idph(1,2,n,10,32),n=1,48)
     * /0,  0,1,  0,1,180,1,180,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,1,180,1,  0,1,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0/
      data (dr(2,2,n,10,32),idph(2,2,n,10,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,180,0,  0,0,  0,1,180,0,  0,1,180,
     *  1,  0,1,180,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,0,  0,1,  0,
     *  1,180,1,  0,0,  0,1,180,0,  0,0,  0,1,180,0,  0/
      data (dr(3,2,n,10,32),idph(3,2,n,10,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,  0,1,180,
     *  1,180,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,180,1,  0,
     *  1,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180/
      data (dr(1,3,n,10,32),idph(1,3,n,10,32),n=1,48)
     * /0,  0,0,  0,0,  0,0,  0,0,  0,1,  0,1,180,1,  0,
     *  1,180,0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,0,  0,1,180,1,  0,1,180,
     *  1,  0,0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180,0,  0/
      data (dr(2,3,n,10,32),idph(2,3,n,10,32),n=1,48)
     * /0,  0,1,  0,1,  0,1,180,1,180,0,  0,0,  0,0,  0,
     *  0,  0,1,180,0,  0,0,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,  0,
     *  0,  0,1,180,1,180,1,  0,1,  0,0,  0,0,  0,0,  0,
     *  0,  0,1,  0,0,  0,0,  0,1,180,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,180/
      data (dr(3,3,n,10,32),idph(3,3,n,10,32),n=1,48)
     * /1,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,180,0,  0,0,  0,1,180,1,180,
     *  1,180,1,  0,1,  0,0,  0,0,  0,1,  0,0,  0,0,  0,
     *  1,180,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,0,  0,
     *  0,  0,0,  0,0,  0,1,  0,0,  0,0,  0,1,  0,1,  0,
     *  1,  0,1,180,1,180,0,  0,0,  0,1,180,0,  0,0,  0/

c --- Punktgruppe 6b 2 m (D_3h):

c      data maxid(33) /6/
c      data (ldim(k,33),k=1,6) /1,1,2,1,1,2/
c      data (cid(k,33),k=1,6) 
c     * /'A1'' ','A2'' ','E''  ','A1''''','A2''''','E'''' '/

c      data (dr(1,1,n,1,33),idph(1,1,n,1,33),n=1,12)
c     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,
c     *  1,  0,1,  0,1,  0/
c      data (dr(1,1,n,2,33),idph(1,1,n,2,33),n=1,12)
c     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,  0,1,  0,1,  0,
c     *  1,180,1,180,1,180/

c      data (dr(1,1,n,3,33),idph(1,1,n,3,33),n=1,12)
c     * /1,  0,c,180,c,180,1,  0,c,180,c,180,c,180,c,180,1,  0,
c     *  c,180,1,  0,c,180/
c      data (dr(2,1,n,3,33),idph(2,1,n,3,33),n=1,12)
c     * /0,  0,s,180,s,  0,0,  0,s,180,s,  0,s,  0,s,180,0,  0,
c     *  s,  0,0,  0,s,180/
c      data (dr(1,2,n,3,33),idph(1,2,n,3,33),n=1,12)
c     * /0,  0,s,  0,s,180,0,  0,s,180,s,  0,s,180,s,  0,0,  0,
c     *  s,  0,0,  0,s,180/
c      data (dr(2,2,n,3,33),idph(2,2,n,3,33),n=1,12)
c     * /1,  0,c,180,c,180,1,180,c,  0,c,  0,c,180,c,180,1,  0,
c     *  c,  0,1,180,c,  0/

c      data (dr(1,1,n,4,33),idph(1,1,n,4,33),n=1,12)
c     * /1,  0,1,  0,1,  0,1,  0,1,  0,1,  0,1,180,1,180,1,180,
c     *  1,180,1,180,1,180/
c      data (dr(1,1,n,5,33),idph(1,1,n,5,33),n=1,12)
c     * /1,  0,1,  0,1,  0,1,180,1,180,1,180,1,180,1,180,1,180,
c     *  1,  0,1,  0,1,  0/

c      data (dr(1,1,n,6,33),idph(1,1,n,6,33),n=1,12)
c     * /1,  0,c,180,c,180,1,  0,c,180,c,180,c,  0,c,  0,1,180,
c     *  c,  0,1,180,c,  0/
c      data (dr(2,1,n,6,33),idph(2,1,n,6,33),n=1,12)
c     * /0,  0,s,180,s,  0,0,  0,s,180,s,  0,s,180,s,  0,0,  0,
c     *  s,180,0,  0,s,  0/
c      data (dr(1,2,n,6,33),idph(1,2,n,6,33),n=1,12)
c     * /0,  0,s,  0,s,180,0,  0,s,180,s,  0,s,  0,s,180,0,  0,
c     *  s,180,0,  0,s,  0/
c      data (dr(2,2,n,6,33),idph(2,2,n,6,33),n=1,12)
c     * /1,  0,c,180,c,180,1,180,c,  0,c,  0,c,  0,c,  0,1,180,
c     *  c,180,1,  0,c,180/

c --- Zuordnung der Symmetrieelemente zu den Matrixelementen bestimmen:

      call pktmat(npgr,mpt,numsy,nsym,NSY)

c --- irred. Darstellungen zuordnen:

      idar = maxid(npgr)
      do 100 m=1,idar
         cmod(m) = cid(m,npgr)
         ndim(m) = ldim(m,npgr)
         do 50 i=1,ndim(m)
            do 40 j=1,ndim(m)
               do 30 n=1,nsym
                  ns  = mpt(n)
                  phs = PI/180. * idph(j,i,ns,m,npgr)
                  xr  = dr(j,i,ns,m,npgr) * cos(phs)
                  xi  = dr(j,i,ns,m,npgr) * sin(phs)
                  tau(n,j,i,m) = cmplx(xr,xi)
30             continue
40          continue
50       continue
100   continue
     

      return
      end


c******************************************************************************
c***   Nummer der Lauegruppe aus der Nummer der Punktgruppe bestimmen       ***
c******************************************************************************

      integer function laugrp(nc,npg,symor,NSY)

      character symor(NSY)*9 

c --- 1b (1, 1b):

      if(npg.eq.1.or.npg.eq.2) then
        laugrp = 1

c --- 2/m (2, m, 2/m):

      else if(npg.eq.3.or.npg.eq.4.or.npg.eq.5) then
        if(symor(2).eq.'  [0,1,0]') then
          laugrp = 2
        else
          laugrp = 12
        end if

c --- m m m (2 2 2, m m 2, m m m):

      else if(npg.eq.6.or.npg.eq.7.or.npg.eq.8) then
        laugrp = 3

c --- 4/m (4, 4b, 4/m):

      else if(npg.eq.9.or.npg.eq.10.or.npg.eq.11) then
        laugrp = 4

c --- 4/m m m (4 2 2, 4 m m, 4b 2 m, 4/m m m):

      else if(npg.eq.12.or.npg.eq.13.or.npg.eq.14.or.npg.eq.15) then
        laugrp = 5

c --- 3b (3, 3b): ,  hexagonal = 6, rhomboedrisch = 13

      else if(npg.eq.16.or.npg.eq.17) then
        if(nc.eq.2) then
          laugrp = 6
        else
          laugrp = 13
        end if

c --- 3b m (3 2, 3 m):  hexagonal = 7, rhomboedrisch = 14

      else if(npg.eq.18.or.npg.eq.19) then
        if(nc.eq.2) then
          laugrp = 7
        else
          laugrp = 14
        end if

c --- 3b m (3b m): 3b m 1 (hexagonal) = 7, 3b 1 m (hexagonal) = 15, 
c     rhomboedrisch = 14

      else if(npg.eq.20) then
        if(nc.eq.2) then
          if(symor(4).eq.'  [1,0,0]') then
            laugrp = 7
          else
            laugrp = 15
          end if
        else
          laugrp = 14
        end if

c --- 6/m (6, 6b, 6/m):

      else if(npg.eq.21.or.npg.eq.22.or.npg.eq.23) then
        laugrp = 8

c --- 6/m m m (6 2 2, 6 m m, 6b m 2, 6/m m m):

      else if(npg.eq.24.or.npg.eq.25.or.npg.eq.26.or.npg.eq.27.or.
     *        npg.eq.33) then
        laugrp = 9

c --- m 3 (2 3, m 3):

      else if(npg.eq.28.or.npg.eq.29) then
        laugrp = 10

c --- m 3b m (4b 3 m, 4 3 2, m 3b m):

      else if(npg.eq.30.or.npg.eq.31.or.npg.eq.32) then
        laugrp = 11

      end if


      return
      end
c******************************************************************************
c***      Reduzible Matrizen und deren Charakteren berechnen                ***
c******************************************************************************

      subroutine redmat(ipm,ism,mt,nchi_red,natom,nsym,NATO,NSY)

      integer ism(3,3,NSY), ipm(NATO,NATO), mt(NATO,NATO,NSY)
      integer nchi_red(NSY)

      do 50 n=1,nsym
         nchi_red(n) = 0
         do 45 k=1,natom
            do 40 l=1,natom
               if(ipm(l,n).eq.k) then
                 mt(l,k,n) = 1
                 if(l.eq.k) then
                   do 35 i=1,3
                      nchi_red(n) = nchi_red(n) + ism(i,i,n)
35                 continue
                 end if
               else
                 mt(l,k,n) = 0
               end if
40          continue
45       continue
50    continue


      return
      end
c******************************************************************************
c***    Alle Symmetrieelemente der Raumgruppe generieren                    ***
c******************************************************************************

      subroutine spggen(csym,ism,tran,nsym,NSY,tau,idar,ndim,IDM,IRD)

      parameter(EPSY = 0.00001, PI = 3.14159265359)

      character csym(NSY)*3, cgen(48)*60, cs*3, so*9
      real t(3), tran(3,NSY)
      integer ma(3,3), ism(3,3,NSY), ndim(IRD)
      complex tau(NSY,IDM,IDM,IRD)


      write(*,'(//2x,a)') 'Generation of the space group elements:'
      write(*,'(/2x,a,/)')'No. Symm. element   product of generators'

      do 10 ns=1,nsym
         cgen(ns)(1:3) = csym(ns)(1:3)
         call symchr(ism(1,1,ns),tran(1,ns),cs,so)
         write(*,'(2x,i2,2a,1x,3a)') ns,'. ',cs,so,' : ',cgen(ns)
 10   continue

      ng = nsym
      n1 = 1
      n2 = ng

 20   do 500 ns=1,ng
         do 400 n=n1,n2

c --- Symmetrieelemente miteinander multiplizieren:

            do 60 i=1,3
               tt = tran(i,ns)
               do 45 j=1,3
                  tt = tt + ism(j,i,ns) * tran(j,n)
 45            continue
               nt = int(tt + EPSY)
               if(tt.lt.-EPSY) nt = nt - 1
               t(i) = tt - nt
               do 55 j=1,3
                  ma(j,i) = 0
                  do 50 k=1,3
                     ma(j,i) = ma(j,i) + ism(k,i,ns) * ism(j,k,n)
 50               continue
 55            continue
 60         continue

c --- Pruefen, ob ein neues Symmetrieelement erzeugt wurde:

            neu = 1
            do 80 k=1,nsym
               do 75 i=1,3
                  if(abs(t(i)-tran(i,k)).gt.EPSY) goto 80
                  do 70 j=1,3
                     if(ma(j,i).ne.ism(j,i,k)) goto 80
 70               continue
 75            continue
               neu = 0
               goto 100
 80         continue

c --- Falls ein neues Symmetrieelement gefunden wurde,
c     neues Symmetrieelement abspeichern:

 100        if(neu.eq.1) then

              nsym = nsym + 1
              if(nsym.gt.NSY) then
                write(*,'(2x,a)') 
     *     'ERROR: the number of symmetry elements is greater than 48' 
                stop
              end if

              do 160 i=1,3
                 tran(i,nsym) = t(i)
                 do 150 j=1,3
                    ism(j,i,nsym) = ma(j,i)
 150             continue
 160          continue

              cgen(nsym)(1:3) = csym(ns)
              cgen(nsym)(4:6) = ' * '
              cgen(nsym)(7:60) = cgen(n)(1:54)
              call symchr(ism(1,1,nsym),tran(1,nsym),cs,so)
              write(*,'(2x,i2,2a,1x,3a)') 
     *             nsym,'. ',cs,so,' : ',cgen(nsym)

              do 200 m=1,idar
                 do 190 i=1,ndim(m)
                    do 180 j=1,ndim(m)
                       tau(nsym,j,i,m) = 0.
                       do 170 l=1,ndim(m)
                          tau(nsym,j,i,m) = tau(nsym,j,i,m) +
     *                                      tau(ns,l,i,m) * tau(n,j,l,m)
 170                   continue
 180                continue
 190             continue
 200          continue   

            end if

 400     continue
 500  continue

      n1 = n2
      n2 = nsym
      if(n2-n1.gt.0) goto 20

      return
      end


c******************************************************************************
c***        Symbol des Symmetrieelementes bestimmen                         ***
c******************************************************************************

      subroutine symchr(ism,tran,csym,symor)

      parameter(MAX = 48)
      real trp(3), tran(3)
      character csym*3, symor*9, cm*4, so*9
      integer irot(3,3), ism(3,3)


c --- Nummer des Symmetrieelementes num bestimmen:

      do 100 nc=1,2
         do 50 k=1,MAX
            call rotmat(k,nc,cm,so,irot)
            ng = 0
            do 30 i=1,3
               do 20 j=1,3
                  if(ism(j,i).eq.irot(j,i)) ng = ng + 1
 20            continue
 30         continue
            if(ng.eq.9) goto 120
 50      continue
 100  continue

 120  if(ng.eq.9) then

        csym  = cm
        symor = so

c --- Pruefen, ob ein Symmetrieelement {ism|tran} phasenschiebend ist 
c     und den phasenschiebenden Translationsanteil trp bestimmen
c     (npt gibt an, wie oft man das Symmetrieelement anwenden muss,
c     um wieder die Einheitsmatrix zu erhalten):

        call tranphs(trp,npt,ism,tran)

c --- Test auf Gleitspiegelebene/Schraubenachse:

        call transbl(csym,npt,trp)

      else

        write(*,'(/2x,a)') 'ERROR: Symmetry element not identified'
        stop

      end if


      return
      end

c******************************************************************************
c***          Die Gitterkomplexe der Position r0 finden                     ***
c******************************************************************************

      subroutine latcmp(xs,nlc,neat,r0,natom_0,NATO,intaus,ism_q,
     *                  tran_q,nsym_q,NSY)

      parameter(EPSY = 0.00001)
      logical neu
      character intaus*1
      integer ism_q(3,3,NSY), neat(NATO)
      real xs(3,NATO), x(3), xp(3), xc(3), r0(3,NATO), tran_q(3,NSY)


c --- Initialisierung:

      do 5 n=1,natom_0
         neat(n) = 0
 5    continue

c --- Nach unmarkierter Position in r0 suchen
c     und Positon des Gitterkomplexes in xs abspeichern:

      nlc = 0
 2    do 10 n=1,natom_0
         if(neat(n).eq.0) then
           neu = .true.
           nlc = nlc + 1
           do 9 i=1,3
              xs(i,nlc) = r0(i,n)
 9         continue
           goto 20
         end if
 10   continue
      neu = .false.

 20   if(neu) then

        do 50 n=1,nsym_q

c --- Symmetrieelement der little group auf xs anwenden:

           do 41 i=1,3
              x(i) = tran_q(i,n)
              do 40 j=1,3
                 x(i) = x(i) + ism_q(i,j,n) * xs(j,nlc)
 40           continue
 41        continue

c --- Erzeugte Position in der prim. Basis darstellen (itr = 0):

           itr = 0
           call transf(intaus,x,xp,itr)

c --- Erzeugte Position in die prim. Zelle schieben:

           do 44 i=1,3
              nt = int(xp(i)+EPSY)
              if(xp(i).lt.-EPSY) nt = nt - 1
              xp(i) = xp(i) - nt
 44        continue

c --- Erzeugte Position ruecktransformieren (itr = 1):

           itr = 1
           call transf(intaus,xc,xp,itr)

c --- Erzeugte Position in r0 markieren:

           do 45 k=1,natom_0
              if(abs(xc(1)-r0(1,k)).lt.EPSY.and.
     *           abs(xc(2)-r0(2,k)).lt.EPSY.and.
     *           abs(xc(3)-r0(3,k)).lt.EPSY) then
                neat(k) = nlc
                goto 50
              end if
 45        continue

 50     continue

        goto 2

      end if


      return
      end
c******************************************************************************
c***        Das Feld mpt sorgt dafuer, dass die Matrixelemente der ID       ***
c***          den Symmetrieelementen richtig zugeordnet werden              ***
c******************************************************************************

      subroutine pktmat(npgr,mpt,numsy,nsym,NSY)

      integer mpt(NSY), numsy(NSY)

      do 10 n=1,nsym
         mpt(n) = n
 10   continue

      if(npgr.eq.12.or.npgr.eq.14) then
        do 20 n=1,nsym
           if(numsy(n).eq.10.or.numsy(n).eq.34) then
             mpt(4) = 6
             mpt(5) = 4
             mpt(6) = 5
             goto 200
           else if(numsy(n).eq.11.or.numsy(n).eq.35) then
             mpt(4) = 5
             mpt(5) = 6
             mpt(6) = 4
             goto 200
           end if
 20     continue
      else if(npgr.eq.15) then
        do 30 n=1,nsym
           if(numsy(n).eq.10.or.numsy(n).eq.34) then
             mpt(4)  =  6
             mpt(5)  =  4
             mpt(6)  =  5
             mpt(12) = 14
             mpt(13) = 12
             mpt(14) = 13      
             goto 200
           else if(numsy(n).eq.11.or.numsy(n).eq.35) then
             mpt(4)  =  5
             mpt(5)  =  6
             mpt(6)  =  4
             mpt(12) = 13
             mpt(13) = 14
             mpt(14) = 12
             goto 200
           end if
 30     continue
      else if(npgr.eq.26) then
        if(numsy(nsym).eq.24) then
          mpt(10) = 11 
          mpt(11) = 12
          mpt(12) = 10
        end if
      end if

 200  continue
      return
      end
C******************************************************************************
C***               Symmetrie-Karten einlesen                                ***
C******************************************************************************

      subroutine syminp(ke,symfile,imo,ismat,spgrp,intaus,laue,icent,
     *                  ism,tran,nprim,nsym,q)

      parameter(maxline = 132, PI = 3.14159265359, max = 6)
      real tran(3,48), trp(3,48), trs(3,48), q(3)
      character rgsymb(max)*10, char(max)*10
      character spgrp*60, symfile*80, intaus*1, line*132, mcell*2 
      character csym(48)*3, symor(48)*9, clau(15)*8, vollsymb*60
      integer ism(3,3,48), numsy(48), num(48), nprim(48)

c --- Lauegruppen:

      data clau(1),clau(2),clau(3),clau(4),clau(5),clau(6),clau(7),
     *     clau(8),clau(9),clau(10),clau(11),clau(12),clau(13),
     *     clau(14),clau(15)
     * /'1b','2/m (C)','m m m','4/m','4/m m m','3b','3b m 1',
     *  '6/m','6/m m m','m 3b','m 3b m','2/m (B)','3b (R)','3b m (R)',
     *  '3b 1 m'/


      if(spgrp(1:3).eq.'   ') then

c --- Datei mit Symmetriekarten einlesen:

        nfile = 1
 
        write(*,*)
        line = ' '
        line = '  Space group file: '//symfile
        call writline(6,line,maxline)
        call symcar(ke,symfile,imo,intaus,mcell,laue,icent,ism,tran,
     *              nprim,nsym)

      else

        nfile = 0
        write(*,'(/2x,a,a)') 'Space group: ',spgrp

c --- Raumgruppensymbol "spgrp" in seine Teilsymbole "rgsymb" und
c     "char" zerlegen:

        call spgsym(imo,spgrp,char,rgsymb,nsbl,max,mcell)

c --- Raumgruppensymbol rgsymb testen und vollst. Symbol, Nummer und 
c     Aufstellung bestimmen:

        call rgtest(rgsymb,nsbl,max,vollsymb,nspg,nauf)

c --- Symmetriekarten bestimmen:

        call symint(vollsymb,intaus,laue,icent,ism,tran,nsym)

      end if

c --- Bei modulierten/magnetischen Strukturen und Zentrosymmetrie
c     die Inversion auf alle Symmetrieelemente anwenden:

      if(icent.eq.2) then
        if(imo.eq.1.or.(imo.eq.2.and.nfile.eq.0)) then
          ns = nsym
          do 700 n=1,nsym
             ns = ns + 1
             do 660 i=1,3
                tran(i,ns) = tran(i,n)
                do 650 j=1,3
                   ism(i,j,ns) = -ism(i,j,n)
650             continue
660          continue
700       continue
          nsym = ns
          icent = 1
        end if
      end if

c --- Kristallsystem bestimmen:

      nc = 1
      if(laue.eq.6.or.laue.eq.7.or.laue.eq.8.or.
     *   laue.eq.9.or.laue.eq.15) nc = 2

c --- Symmetrieelemente sortieren:

      call symsort(nc,csym,symor,ism,tran,trp,trs,num,numsy,nsym,48)

c --- Bei magnetischen Raumgruppen gestrichene Symmetrieelemente und
c     Wellenvektor bestimmen:

      if(imo.eq.2) then
        if(nfile.eq.0) then
          call magspg(char,nsbl+1,max,nc,nspg,csym,symor,ism,nprim,nsym)
        end if
        call wavvek(intaus,mcell,q)
      end if

c --- Symmetriekarten rausschreiben, falls erwuenscht:

      if(nfile.eq.0) then
        write(*,'(2x,a,a40,/2x,a,i3,a,i1)') 
     *       'Complete space group symbol: ',vollsymb,
     *       'Space group no. ',nspg,',   cell choice ',nauf
      end if
      write(*,'(2x,a,i4)') 'Number of symmetry cards = ',nsym
      if(icent.eq.2) then
        write(*,'(2x,a)') 'This space group is centrosymmetric'
      else
        write(*,'(2x,a)') 'This space group is non-centrosymmetric'
      end if
      write(*,'(2x,a,a,4x,a,a)')
     *'Centring type : ',intaus,'    Laue-group : ',clau(laue)
      if(imo.eq.2) then
        if(mcell.eq.'1 '.and.magfer(nprim,ism,nsym).eq.1) then
          write(*,'(2x,a)') 'This space group is ferromagnetic.'
        else
          write(*,'(2x,a)') 'This space group is antiferromagnetic.'
        end if
        write(*,'(2x,a,a)') 'Magnetic cell : ',mcell
        write(*,'(2x,a,3f8.5)') 
     *  'Wave vector of the magnetic structure: ',(q(i),i=1,3)
      end if
      if(ismat.eq.1) then
        write(*,'(/2x,a/)') 'S y m m e t r y   c a r d s'
        if(imo.eq.2) then
          do 135 n=1,nsym
             write(*,132) csym(n),symor(n),
     *            (real(ism(1,j,n)),j=1,3),tran(1,n),trp(1,n),
     *            (real(ism(2,j,n)),j=1,3),tran(2,n),trp(2,n),nprim(n),
     *            (real(ism(3,j,n)),j=1,3),tran(3,n),trp(3,n)
132          format(2x,a,1x,a,' :',8x,3f5.1,8x,f5.2,8x,f5.2,/,
     *              21x,'R=  ',3f5.1,'    T=  ',f5.2,'    Tp= ',f5.2,
     *              '   E= ',i2,/,25x,3f5.1,8x,f5.2,8x,f5.2/)
135       continue
        else
          do 145 n=1,nsym
             write(*,142) csym(n),symor(n),
     *            (real(ism(1,j,n)),j=1,3),tran(1,n),trp(1,n),
     *            ((real(ism(i,j,n)),j=1,3),tran(j,n),trp(j,n),i=2,3)
142          format(2x,a,1x,a,' :',8x,3f5.1,8x,f5.2,8x,f5.2,/,
     *              21x,'R=  ',3f5.1,'    T=  ',f5.2,'    Tp= ',f5.2,/,
     *              25x,3f5.1,8x,f5.2,8x,f5.2/)
145       continue
        end if
      end if

      return
      end


c******************************************************************************
c***   Symmetriekarten und Auslenkungsfelder aus einer Datei einlesen       ***
c******************************************************************************

      subroutine symcar(ke,symfile,imo,intaus,mcell,laue,icent,ism,
     *                  tran,nprim,nsym)

      parameter(max = 30, PI = 3.14159265359)
      integer ism(3,3,48), nprim(48)
      real werte(max), tran(3,48)
      character symfile*80, intaus*1, mcell*2, char(max)*132, ecod*3

      ecod = 'eee'

c --- Formate:

 31   format(/2x,'Error in space group file: Line "',a3,
     *  '> ..." is missing')

c --- Symmetriekarten-File oeffnen:

      open(ke,file=symfile,status='old',err=1000)

c --- Int. Ausl., Lauegruppe, Zentrosymmetrie einlesen:

      ist = 1
      call getinp(ke,'COD',ecod,werte,char,max,num,nn,ist)
      if(ist.eq.11) then
        write(*,31) 'COD'
        stop
      else
        if(imo.eq.2) then
          if(num.ne.4) then
            write(*,'(/2x,a,/2x,a)')
     *      'Error in space group file: wrong number of parameters,',
     *      'it should be 4, line "COD> ..."'
            stop
          end if
          intaus = char(1)(1:1)
          mcell  = char(2)(1:2)
          laue   = nint(werte(3))
          icent  = nint(werte(4))
        else
          if(num.ne.3) then
            write(*,'(/2x,a,/2x,a)')
     *      'Error in space group file: wrong number of parameters,',
     *      'it should be 3, line "COD> ..."'
            stop
          end if
          intaus = char(1)(1:1)
          laue   = nint(werte(2))
          icent  = nint(werte(3))
        end if
      end if

c --- Symmetriekarten einlesen:

      n = 0
100   ist = 1
      call getinp(ke,'SYM',ecod,werte,char,max,num,nn,ist)
      if(ist.eq.11.or.ist.eq.13) then
        if(n.eq.0) then
          write(*,31) 'SYM'
          stop
        else
          nsym = n
          goto 400
        end if
      else
        n = n + 1
        if(imo.eq.2) then
          if(num.ne.14) then
            write(*,'(/2x,a,/2x,a,i2)') 
     *      'Error in space group file: wrong number of parameters,',
     *      'it should be 14, line "SYM> ...", symmetry card ',n
            stop
          end if
          nprim(n) = nint(werte(14))
        else
          if(num.ne.13) then
            write(*,'(/2x,a,/2x,a,i2)') 
     *      'Error in space group file: wrong number of parameters,',
     *      'it should be 14, line "SYM> ...", symmetry card ',n
            stop
          end if
        end if
        do 120 i=1,3
           tran(i,n) = werte(10+i)
           do 130 j=1,3
              ism(i,j,n) = nint(werte(3*(i-1)+j+1))
130        continue
120     continue
      end if
      goto 100


c --- Datei schliessen:

400   close(ke)


c --- Falls die Symmetriekarte (x,y,z) fehlt, wird sie ergaenzt:

      iall = 0
      do 500 n=1,nsym
         idi = 0
	 do 499 i=1,3
	    if(ism(i,i,n).eq.1.and.tran(i,n).eq.0.) idi = idi + 1
499      continue
	 if(idi.eq.3) iall = 1
500   continue

      if(iall.ne.1) then
        nsym = nsym + 1
        n = nsym
        do 600 i=1,3
           tran(i,n) = 0.
           do 599 j=1,3
              ism(i,j,n) = 0
599        continue
           ism(i,i,n) = 1
600     continue
        if(imo.eq.2) nprim(n) = 1
      end if

      return


c --- Fehlermeldungen:

1000  stop ' Error: Cannot open space group file' 
  
      end


c******************************************************************************
c***    Bestimmung der umfaerbenden Symmetrieelemente                       ***
c******************************************************************************

      subroutine magspg(char,nzahl,max,nc,nspg,csym,symor,
     *                  ism,nprim,nsym)

      external symchg
      character symchg*3
      character char(max)*10, syor(10)*9
      character csym(48)*3, psym(48)*3, symor(48)*9, sybl(10)*3
      integer ism(3,3,48), nprim(48), nstrich(10)


c --- Generatoren der magnetischen Raumgruppe bestimmen:

      call genbest(char,nzahl,max,nspg,nc,sybl,syor,nstrich,ngen)

c --- Generatoren den nukl. Raumgruppenelementen zuordnen:

      do 49 ns=1,nsym
         psym(ns) = symchg(csym(ns))
         nprim(ns) = 0
 49   continue

      do 65 i=1,ngen
         do 60 ns=1,nsym 
            if(sybl(i).eq.psym(ns).and.syor(i).eq.symor(ns)) then
              nprim(ns) = nstrich(i)
              goto 65
            end if
 60      continue
         write(*,'(2x,2a,2x,2a)') 
     *   'Generator ',sybl(i),syor(i),' not identified.'
         stop
 65   continue

c --- Symmetrieelemente der nukl. Raumgruppe aus den Generatoren bestimmen
c     und testen, ob das Element umfaerbt order nicht:

      nprim(1) = 1
      call umfaerb(nprim,ism,nsym)
      ipr = 0
      do 85 ns=1,nsym
         if(nprim(ns).eq.-1) ipr = ipr + 1
 85   continue
      if(ipr.ne.nsym/2.and.ipr.ne.0) then
        write(*,'(2x,a)') 'Bad number of primed elements.'
        stop
      end if

      return
      end


c******************************************************************************
c***         Symbol und Orientierung der Generatoren bestimmen              ***
c******************************************************************************

      subroutine genbest(char,nzahl,max,nspg,nc,sybl,syor,
     *                   nstrich,ngen)

      integer nstrich(10), nst(2)
      character char(max)*10, sybl(10)*3, syor(10)*9, sym(2)*3


c --- Triklin:

      if(nspg.le.2) then
         call symbol(char(3),sym,nst,nsb)
         ngen = 1
         sybl(1) = sym(1)
         syor(1) = '         '
         nstrich(1) = nst(1)

c --- Monoklin:

      else if(nspg.ge.3.and.nspg.le.15) then

         call symbol(char(3),sym,nst,nsb)
         do 51 i=1,nsb
            sybl(i) = sym(i)
            syor(i) = '  [0,1,0]'
            nstrich(i) = nst(i)
 51      continue
         ngen = nsb

c --- Orthorhombisch:

      else if(nspg.ge.16.and.nspg.le.74) then
         call symbol(char(3),sym,nst,nsb)
         sybl(1) = sym(1)
         syor(1) = '  [1,0,0]'
         nstrich(1) = nst(1)
         call symbol(char(4),sym,nst,nsb)
         sybl(2) = sym(1)
         syor(2) = '  [0,1,0]'
         nstrich(2) = nst(1)
         call symbol(char(5),sym,nst,nsb)
         sybl(3) = sym(1)
         syor(3) = '  [0,0,1]'
         nstrich(3) = nst(1)
         ngen = 3

c --- Tetragonal:

      else if(nspg.ge.75.and.nspg.le.142) then
         call symbol(char(3),sym,nst,nsb)
         do 52 i=1,nsb
            sybl(i) = sym(i)
            syor(i) = '  [0,0,1]'
            nstrich(i) = nst(i)
 52      continue
         ngen = nsb
         if(nzahl.ge.4) then
            call symbol(char(4),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            syor(ngen) = '  [1,0,0]'
            nstrich(ngen) = nst(1)
         end if
         if(nzahl.eq.5) then
            call symbol(char(5),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            syor(ngen) = ' [1,-1,0]'
            nstrich(ngen) = nst(1)
         end if

c --- Trigonal:

      else if(nspg.ge.143.and.nspg.le.167) then
         if(nc.eq.1) then
            call symbol(char(3),sym,nst,nsb)
            sybl(1) = sym(1)
            syor(1) = '  [1,1,1]'
            nstrich(i) = nst(1)
            ngen = 1
            if(nzahl.eq.4) then
               call symbol(char(4),sym,nst,nsb)
               ngen = ngen + 1
               sybl(ngen) = sym(1)
               syor(ngen) = ' [1,-1,0]'
               nstrich(ngen) = nst(1)
            end if
         else if(nc.eq.2) then
            call symbol(char(3),sym,nst,nsb)
            sybl(1) = sym(1)
            syor(1) = '  [0,0,1]'
            nstrich(i) = nst(1)
            ngen = 1
            if(nzahl.ge.4) then
               call symbol(char(4),sym,nst,nsb)
               ngen = ngen + 1
               sybl(ngen) = sym(1)
               if(sym(1).eq.'  1') then
                 syor(ngen) = '         '
               else
                 syor(ngen) = '  [1,0,0]'
               end if
               nstrich(ngen) = nst(1)
            end if
            if(nzahl.eq.5) then
               call symbol(char(5),sym,nst,nsb)
               ngen = ngen + 1
               sybl(ngen) = sym(1)
               if(sym(1).eq.'  1') then
                 syor(ngen) = '         '
               else
                 syor(ngen) = ' [1,-1,0]'
               end if
               nstrich(ngen) = nst(1)
            end if
         end if

c --- Hexagonal:

      else if(nspg.ge.168.and.nspg.le.194) then
         call symbol(char(3),sym,nst,nsb)
         do 53 i=1,nsb
            sybl(i) = sym(i)
            syor(i) = '  [0,0,1]'
            nstrich(i) = nst(i)
 53      continue
         ngen = nsb
         if(nzahl.ge.4) then
            call symbol(char(4),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            syor(ngen) = '  [1,0,0]'
            nstrich(ngen) = nst(1)
         end if
         if(nzahl.eq.5) then
            call symbol(char(5),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            syor(ngen) = ' [1,-1,0]'
            nstrich(ngen) = nst(1)
         end if

c --- Kubisch:            

      else if(nspg.ge.195.and.nspg.le.230) then
         call symbol(char(3),sym,nst,nsb)
         do 54 i=1,nsb
            sybl(i) = sym(i)
            syor(i) = '  [0,0,1]'
            nstrich(i) = nst(i)
 54      continue
         ngen = nsb
         call symbol(char(4),sym,nst,nsb)
         ngen = ngen + 1
         if(sym(1).eq.'3b+') then
           sybl(ngen) = ' 3+'
         else
           sybl(ngen) = sym(1)
         end if
         syor(ngen) = '  [1,1,1]'
         nstrich(ngen) = nst(1)
         if(nzahl.ge.5.and.nspg.ge.207) then
            call symbol(char(5),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            if(nspg.ge.207.and.nspg.le.220) then
               syor(ngen) = '  [1,1,0]'
            else
               syor(ngen) = ' [1,-1,0]'
            end if
            nstrich(ngen) = nst(1)
         end if

      end if

      return
      end

c******************************************************************************
c***                Erkennung des Symbols                                   ***
c******************************************************************************

      subroutine symbol(char,sym,nst,nsy)

      integer nst(2)
      character char*10, sym(2)*3

c --- Initialisierung:

      nsy = 0
      sym(1) = '   '
      sym(2) = '   '
      nst(1) = 0
      nst(2) = 0

c --- Testen, ob "/" im String chr enthalten ist:

      ibksl = index(char,'/')
      if(ibksl.gt.0) then
        nsy = 2
        sym(1) = char(1:ibksl-1)
        sym(2) = char(ibksl+1:ibksl+3)
      else
        nsy = 1
        sym(1) = char(1:3)
      end if

c --- Symmetrieelement zuordnen:

      do 100 i=1,nsy
         if(index(sym(i),'''').gt.0) then
           nst(i) = -1
         else
           nst(i) = 1
         end if
         if(index(sym(i),'1b').gt.0) then
            sym(i) = ' 1b'
         else if(index(sym(i),'3b').gt.0) then
            sym(i) = '3b+'
         else if(index(sym(i),'4b').gt.0) then
            sym(i) = '4b+'
         else if(index(sym(i),'6b').gt.0) then
            sym(i) = '6b+'
         else if(index(sym(i),'21').gt.0) then
            sym(i) = '  2'
         else if(index(sym(i),'31').gt.0) then
            sym(i) = ' 3+'
         else if(index(sym(i),'32').gt.0) then
            sym(i) = ' 3+'
         else if(index(sym(i),'41').gt.0) then
            sym(i) = ' 4+'
         else if(index(sym(i),'42').gt.0) then
            sym(i) = ' 4+'
         else if(index(sym(i),'43').gt.0) then
            sym(i) = ' 4+'
         else if(index(sym(i),'61').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'62').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'63').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'64').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'65').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'2').gt.0) then
            sym(i) = '  2'
         else if(index(sym(i),'3').gt.0) then
            sym(i) = ' 3+'
         else if(index(sym(i),'4').gt.0) then
            sym(i) = ' 4+'
         else if(index(sym(i),'6').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'m').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'a').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'b').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'c').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'d').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'n').gt.0) then
            sym(i) = '  m'
         else
            sym(i) = '  1'
         end if
 100  continue

      end

c******************************************************************************
c***  Name der Symmetrieelemente der Punktgruppe bzgl. der Raumgruppe       ***
c******************************************************************************

      character*3 function symchg(csym)

      character csym*3

      if(csym.eq.'  m'.or.csym.eq.'  a'.or.csym.eq.'  b'.or.
     *   csym.eq.'  c'.or.csym.eq.'  n'.or.csym.eq.'  d') then
         symchg = '  m'
      else if(csym.eq.' 21') then
         symchg = '  2'
      else if(csym(1:2).eq.'31'.or.csym(1:2).eq.'32') then
         symchg(1:2) = ' 3'
         symchg(3:3) = csym(3:3)
      else if(csym(1:2).eq.'41'.or.csym(1:2).eq.'42'.or.
     *        csym(1:2).eq.'43') then
        symchg(1:2) = ' 4'
        symchg(3:3) = csym(3:3)
      else if(csym(1:2).eq.'61'.or.csym(1:2).eq.'62'.or.
     *        csym(1:2).eq.'63'.or.csym(1:2).eq.'64'.or.
     *        csym(1:2).eq.'65') then
        symchg(1:2) = ' 6'
        symchg(3:3) = csym(3:3)
      else
        symchg = csym
      end if

      return
      end

c******************************************************************************
c***                 In Grossbuchstaben umwandeln                           ***
c******************************************************************************

      character*1 function caplet(string)

      character string*1
     
      k1 = ichar('A')
      k2 = ichar('Z')
      n1 = ichar('a')
      n2 = ichar('z')

      iz = ichar(string)
      if(iz.ge.n1.and.iz.le.n2) then 
        caplet = char(iz-n2+k2)
      else
        caplet = string 
      end if

      return
      end

c******************************************************************************
c***         Testen, ob Symmetrieelement umfaerbt oder nicht                ***
c******************************************************************************

      subroutine umfaerb(nprim,ism,nsym)

      integer nprim(48), ism(3,3,48), matrix(3,3)

      ipr = 0
      do 10 n=1,nsym
         if(nprim(n).ne.0) ipr = ipr + 1
 10   continue
      if(ipr.eq.nsym) goto 999

 20   do 300 n=1,nsym
         if(nprim(n).ne.0) then
           do 200 k=1,nsym
              if(nprim(k).ne.0) then
                 call matprd(matrix,ism(1,1,n),ism(1,1,k))
                 do 50 l=1,nsym
                    if(nprim(l).eq.0) then
                       is = 0
                       do 45 i=1,3
                          do 44 j=1,3
                             if(matrix(i,j).eq.ism(i,j,l)) is = is + 1
 44                       continue
 45                    continue
                       if(is.eq.9) then
                          nprim(l) = nprim(n) * nprim(k)
                          ipr = ipr + 1
                          if(ipr.eq.nsym) then
                            goto 999
                          else
                            goto 200
                          end if
                       end if
                    end if
 50              continue   ! Schleife l
              end if
 200       continue   ! Schleife k
         end if
 300  continue   ! Schleife n
      goto 20


 999  return
      end

c******************************************************************************
c***                       Matrixmultiplikation                             ***
c******************************************************************************

      subroutine matprd(mc,ma,mb)

      integer ma(3,3), mb(3,3), mc(3,3)

      do 60 i=1,3
         do 55 j=1,3
            mc(i,j) = 0
            do 50 k=1,3
               mc(i,j) = mc(i,j) + ma(i,k) * mb(k,j)
 50         continue
 55      continue
 60   continue

      return
      end

c******************************************************************************
c***            Test auf ferromagnetische Raumgruppe                        ***
c******************************************************************************

      integer function magfer(nprim,ism,nsym)

      external ndet3
      integer nprim(48), ism(3,3,48), ma(3,3)

      do 20 i=1,3
         do 10 j=1,3
            ma(i,j) = 0
 10      continue
 20   continue

      do 50 n=1,nsym
         nd = ndet3(ism(1,1,n))
         do 40 i=1,3
            do 30 j=1,3
               ma(i,j) = ma(i,j) + ism(i,j,n) * nd * nprim(n)
 30         continue
 40      continue
 50   continue

      magfer = 0
      do 70 i=1,3
         do 60 j=1,3
            if(ma(i,j).ne.0) then
               magfer = 1
               goto 100
            end if
 60      continue
 70   continue

 100  return
      end

c******************************************************************************
c***           Berechnung der Determinante einer 3x3-Matrix                 ***
c***                  nach der Sarrusschen Regel                            ***
c******************************************************************************

      integer function ndet3(matrix)

      integer matrix(3,3)

      ndet3 = matrix(1,1) * matrix(2,2) * matrix(3,3) +
     *        matrix(1,2) * matrix(2,3) * matrix(3,1) +
     *        matrix(1,3) * matrix(2,1) * matrix(3,2) -
     *        matrix(1,3) * matrix(2,2) * matrix(3,1) -
     *        matrix(1,1) * matrix(2,3) * matrix(3,2) -
     *        matrix(1,2) * matrix(2,1) * matrix(3,3)

c      ndet3 = matrix(1,1) * matrix(2,2) * matrix(3,3) +
c     *        matrix(2,1) * matrix(3,2) * matrix(1,3) +
c     *        matrix(3,1) * matrix(1,2) * matrix(2,3) -
c     *        matrix(3,1) * matrix(2,2) * matrix(1,3) -
c     *        matrix(1,1) * matrix(3,2) * matrix(2,3) -
c     *        matrix(2,1) * matrix(1,2) * matrix(3,3)

      return
      end

c***************************************************************************
c***                  Wellenvektor bestimmen
c***************************************************************************

      subroutine wavvek(intaus,mcell,q)

      parameter(NV = 14)
      character intaus*1, mcell*2
      real qvec(3,NV), q(3)
      data (qvec(j,1),j=1,3)  / 0.,  0.,  0. /
      data (qvec(j,2),j=1,3)  / 0.5, 0.,  0. /
      data (qvec(j,3),j=1,3)  / 0.,  0.5, 0. /
      data (qvec(j,4),j=1,3)  / 0.,  0.,  0.5/
      data (qvec(j,5),j=1,3)  / 0.5, 0.5, 0. /
      data (qvec(j,6),j=1,3)  / 0.5, 0.,  0.5/
      data (qvec(j,7),j=1,3)  / 0.,  0.5, 0.5/
      data (qvec(j,8),j=1,3)  / 0.5, 0.5, 0.5/
      data (qvec(j,9),j=1,3)  / 1.,  0.,  0. /
      data (qvec(j,10),j=1,3) / 0.,  1.,  0. /
      data (qvec(j,11),j=1,3) / 0.,  0.,  1. / 
      data (qvec(j,12),j=1,3) / 0.5, 1.,  0. /
      data (qvec(j,13),j=1,3) / 1.,  0.5, 0. /
      data (qvec(j,14),j=1,3) / 1.,  0.,  0.5/

      if(intaus.eq.'P') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'2a') then
          nq = 2
        else if(mcell.eq.'2b') then
          nq = 3
        else if(mcell.eq.'2c') then
          nq = 4
        else if(mcell.eq.'C ') then
          nq = 5
        else if(mcell.eq.'B ') then
          nq = 6
        else if(mcell.eq.'A ') then
          nq = 7
        else if(mcell.eq.'F ') then
          nq = 8
        else if(mcell.eq.'P ') then
          nq = 5
        else if(mcell.eq.'I ') then
          nq = 8
        end if
      else if(intaus.eq.'A') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'2a') then
          nq = 2
        else if(mcell.eq.'P ') then
          nq = 10
        else if(mcell.eq.'I ') then
          nq = 12
        end if
      else if(intaus.eq.'B') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'2b') then
          nq = 3
        else if(mcell.eq.'P ') then
          nq = 9
        else if(mcell.eq.'I ') then
          nq = 13
        end if
      else if(intaus.eq.'C') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'2c') then
          nq = 4
        else if(mcell.eq.'P ') then
          nq = 9
        else if(mcell.eq.'I ') then
          nq = 14
        end if
      else if(intaus.eq.'I') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'P ') then
          nq = 9
        end if
      else if(intaus.eq.'F') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'A ') then
          nq = 9
        else if(mcell.eq.'B ') then
          nq = 10
        else if(mcell.eq.'C ') then
          nq = 11
        end if
      else if(intaus.eq.'R') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'R ') then
          nq = 8
        end if
      end if

c --- Wellenvektor q abspeichern:

      do 100 j=1,3
         q(j) = qvec(j,nq)
100   continue

      return
      end

c***********************************************************************
c***      Teilsymbole des Raumgruppensymbols bestimmen               ***
c***********************************************************************

      subroutine getsym(spgrp,char,num,max)

      character spgrp*60, char(max)*10


c --- Initialisierung:

      do 90 i=1,max
         char(i) = ' '
 90   continue

c --- Anfang suchen:

 110  do 120 i=1,60
         if(spgrp(i:i).ne.' ') then
           ibeg = i
           goto 130
         end if
 120  continue    
      return 

c --- Teilsymbole abspeichern:

 130  ic    = 0
      n     = 0
      num   = 1
      ibl   = 0
      komma = 0

      do 250 i=ibeg,60
         if(spgrp(i:i).eq.' ') then
           ibl = ibl + 1
         else if(spgrp(i:i).eq.',') then
           komma = 1
           num = num + 1
           n = 0
         else
           if(ibl.gt.0.and.komma.eq.0) then
             num = num + 1
             n = 0
           end if
           ic = ic + 1
           n = n + 1
           char(num)(n:n) = spgrp(i:i)
           ibl = 0
           komma = 0
         end if
 250  continue
      if(num.gt.max) then
        write(*,'(2x,a)') 'Bad space group symbol'
        stop
      end if

      return
      end


c******************************************************************************
c***                   Raumgruppen-Eingabe testen                           ***
c******************************************************************************

      subroutine rgtest(symb,num,max,spgrp,nspg,nc)

      parameter(NRG = 306)
      integer numrg(NRG), ncell(NRG)
      character kurz(NRG)*12, lang(NRG)*17, symb(max)*10,
     *          string*60, spgrp*60

      data numrg(  1), ncell(  1), kurz(  1), lang(  1)
     *  /  1,1,'P,1         ','P,1,,            '/
      data numrg(  2), ncell(  2), kurz(  2), lang(  2)
     *  /  2,1,'P,1B        ','P,1B,,           '/
      data numrg(  3), ncell(  3), kurz(  3), lang(  3)
     *  /  3,1,'P,2,1       ','P,1,2,1          '/
      data numrg(  4), ncell(  4), kurz(  4), lang(  4)
     *  /  3,2,'P,2,2        ','P,1,1,2          '/
      data numrg(  5), ncell(  5), kurz(  5), lang(  5)
     *  /  4,1,'P,21,1      ','P,1,21,1         '/
      data numrg(  6), ncell(  6), kurz(  6), lang(  6)
     *  /  4,2,'P,21,2      ','P,1,1,21         '/
      data numrg(  7), ncell(  7), kurz(  7), lang(  7)
     *  /  5,1,'C,2,1       ','C,1,2,1          '/
      data numrg(  8), ncell(  8), kurz(  8), lang(  8)
     *  /  5,2,'A,2,1       ','A,1,2,1          '/
      data numrg(  9), ncell(  9), kurz(  9), lang(  9)
     *  /  5,3,'I,2,1       ','I,1,2,1          '/
      data numrg( 10), ncell( 10), kurz( 10), lang( 10)
     *  /  5,4,'A,2,2       ','A,1,1,2          '/
      data numrg( 11), ncell( 11), kurz( 11), lang( 11)
     *  /  5,5,'B,2,2       ','B,1,1,2          '/
      data numrg( 12), ncell( 12), kurz( 12), lang( 12)
     *  /  5,6,'I,2,2       ','I,1,1,2          '/
      data numrg( 13), ncell( 13), kurz( 13), lang( 13)
     *  /  6,1,'P,M,1       ','P,1,M,1          '/
      data numrg( 14), ncell( 14), kurz( 14), lang( 14)
     *  /  6,2,'P,M,2       ','P,1,1,M          '/
      data numrg( 15), ncell( 15), kurz( 15), lang( 15)
     *  /  7,1,'P,C,1       ','P,1,C,1          '/
      data numrg( 16), ncell( 16), kurz( 16), lang( 16)
     *  /  7,2,'P,N,1       ','P,1,N,1          '/
      data numrg( 17), ncell( 17), kurz( 17), lang( 17)
     *  /  7,3,'P,A,1       ','P,1,A,1          '/
      data numrg( 18), ncell( 18), kurz( 18), lang( 18)
     *  /  7,4,'P,A,2       ','P,1,1,A          '/
      data numrg( 19), ncell( 19), kurz( 19), lang( 19)
     *  /  7,5,'P,N,2       ','P,1,1,N          '/
      data numrg( 20), ncell( 20), kurz( 20), lang( 20)
     *  /  7,6,'P,B,2       ','P,1,1,B          '/
      data numrg( 21), ncell( 21), kurz( 21), lang( 21)
     *  /  8,1,'C,M,1       ','C,1,M,1          '/
      data numrg( 22), ncell( 22), kurz( 22), lang( 22)
     *  /  8,2,'A,M,1       ','A,1,M,1          '/
      data numrg( 23), ncell( 23), kurz( 23), lang( 23)
     *  /  8,3,'I,M,1       ','I,1,M,1          '/
      data numrg( 24), ncell( 24), kurz( 24), lang( 24)
     *  /  8,4,'A,M,2       ','A,1,1,M          '/
      data numrg( 25), ncell( 25), kurz( 25), lang( 25)
     *  /  8,5,'B,M,2       ','B,1,1,M          '/
      data numrg( 26), ncell( 26), kurz( 26), lang( 26)
     *  /  8,6,'I,M,2       ','I,1,1,M          '/
      data numrg( 27), ncell( 27), kurz( 27), lang( 27)
     *  /  9,1,'C,C,1       ','C,1,C,1          '/
      data numrg( 28), ncell( 28), kurz( 28), lang( 28)
     *  /  9,2,'A,N,1       ','A,1,N,1          '/
      data numrg( 29), ncell( 29), kurz( 29), lang( 29)
     *  /  9,3,'I,A,1       ','I,1,A,1          '/
      data numrg( 30), ncell( 30), kurz( 30), lang( 30)
     *  /  9,4,'A,A,2       ','A,1,1,A          '/
      data numrg( 31), ncell( 31), kurz( 31), lang( 31)
     *  /  9,5,'B,N,2       ','B,1,1,N          '/
      data numrg( 32), ncell( 32), kurz( 32), lang( 32)
     *  /  9,6,'I,B,2       ','I,1,1,B          '/
      data numrg( 33), ncell( 33), kurz( 33), lang( 33)
     *  / 10,1,'P,2/M,1     ','P,1,2/M,1        '/
      data numrg( 34), ncell( 34), kurz( 34), lang( 34)
     *  / 10,2,'P,2/M,2     ','P,1,1,2/M        '/
      data numrg( 35), ncell( 35), kurz( 35), lang( 35)
     *  / 11,1,'P,21/M,1    ','P,1,21/M,1       '/
      data numrg( 36), ncell( 36), kurz( 36), lang( 36)
     *  / 11,2,'P,21/M,2    ','P,1,1,21/M       '/
      data numrg( 37), ncell( 37), kurz( 37), lang( 37)
     *  / 12,1,'C,2/M,1     ','C,1,2/M,1        '/
      data numrg( 38), ncell( 38), kurz( 38), lang( 38)
     *  / 12,2,'A,2/M,1     ','A,1,2/M,1        '/
      data numrg( 39), ncell( 39), kurz( 39), lang( 39)
     *  / 12,3,'I,2/M,1     ','I,1,2/M,1        '/
      data numrg( 40), ncell( 40), kurz( 40), lang( 40)
     *  / 12,4,'A,2/M,2     ','A,1,1,2/M        '/
      data numrg( 41), ncell( 41), kurz( 41), lang( 41)
     *  / 12,5,'B,2/M,2     ','B,1,1,2/M        '/
      data numrg( 42), ncell( 42), kurz( 42), lang( 42)
     *  / 12,6,'I,2/M,2     ','I,1,1,2/M        '/
      data numrg( 43), ncell( 43), kurz( 43), lang( 43)
     *  / 13,1,'P,2/C,1     ','P,1,2/C,1        '/
      data numrg( 44), ncell( 44), kurz( 44), lang( 44)
     *  / 13,2,'P,2/N,1     ','P,1,2/N,1        '/
      data numrg( 45), ncell( 45), kurz( 45), lang( 45)
     *  / 13,3,'P,2/A,1     ','P,1,2/A,1        '/
      data numrg( 46), ncell( 46), kurz( 46), lang( 46)
     *  / 13,4,'P,2/A,2     ','P,1,1,2/A        '/
      data numrg( 47), ncell( 47), kurz( 47), lang( 47)
     *  / 13,5,'P,2/N,2     ','P,1,1,2/N        '/
      data numrg( 48), ncell( 48), kurz( 48), lang( 48)
     *  / 13,6,'P,2/B,2     ','P,1,1,2/B        '/
      data numrg( 49), ncell( 49), kurz( 49), lang( 49)
     *  / 14,1,'P,21/C,1    ','P,1,21/C,1       '/
      data numrg( 50), ncell( 50), kurz( 50), lang( 50)
     *  / 14,2,'P,21/N,1    ','P,1,21/N,1       '/
      data numrg( 51), ncell( 51), kurz( 51), lang( 51)
     *  / 14,3,'P,21/A,1    ','P,1,21/A,1       '/
      data numrg( 52), ncell( 52), kurz( 52), lang( 52)
     *  / 14,4,'P,21/A,2    ','P,1,1,21/A       '/
      data numrg( 53), ncell( 53), kurz( 53), lang( 53)
     *  / 14,5,'P,21/N,2    ','P,1,1,21/N       '/
      data numrg( 54), ncell( 54), kurz( 54), lang( 54)
     *  / 14,6,'P,21/B,2    ','P,1,1,21/B       '/
      data numrg( 55), ncell( 55), kurz( 55), lang( 55)
     *  / 15,1,'C,2/C,1     ','C,1,2/C,1        '/
      data numrg( 56), ncell( 56), kurz( 56), lang( 56)
     *  / 15,2,'A,2/N,1     ','A,1,2/N,1        '/
      data numrg( 57), ncell( 57), kurz( 57), lang( 57)
     *  / 15,3,'I,2/A,1     ','I,1,2/A,1        '/
      data numrg( 58), ncell( 58), kurz( 58), lang( 58)
     *  / 15,4,'A,2/A,2     ','A,1,1,2/A        '/
      data numrg( 59), ncell( 59), kurz( 59), lang( 59)
     *  / 15,5,'B,2/N,2     ','B,1,1,2/N        '/
      data numrg( 60), ncell( 60), kurz( 60), lang( 60)
     *  / 15,6,'I,2/B,2     ','I,1,1,2/B        '/
      data numrg( 61), ncell( 61), kurz( 61), lang( 61)
     *  / 16,1,'P,2,2,2     ','P,2,2,2          '/
      data numrg( 62), ncell( 62), kurz( 62), lang( 62)
     *  / 17,1,'P,2,2,21    ','P,2,2,21         '/
      data numrg( 63), ncell( 63), kurz( 63), lang( 63)
     *  / 18,1,'P,21,21,2   ','P,21,21,2        '/
      data numrg( 64), ncell( 64), kurz( 64), lang( 64)
     *  / 19,1,'P,21,21,21  ','P,21,21,21       '/
      data numrg( 65), ncell( 65), kurz( 65), lang( 65)
     *  / 20,1,'C,2,2,21    ','C,2,2,21         '/
      data numrg( 66), ncell( 66), kurz( 66), lang( 66)
     *  / 21,1,'C,2,2,2     ','C,2,2,2          '/
      data numrg( 67), ncell( 67), kurz( 67), lang( 67)
     *  / 22,1,'F,2,2,2     ','F,2,2,2          '/
      data numrg( 68), ncell( 68), kurz( 68), lang( 68)
     *  / 23,1,'I,2,2,2     ','I,2,2,2          '/
      data numrg( 69), ncell( 69), kurz( 69), lang( 69)
     *  / 24,1,'I,21,21,21  ','I,21,21,21       '/
      data numrg( 70), ncell( 70), kurz( 70), lang( 70)
     *  / 25,1,'P,M,M,2     ','P,M,M,2          '/
      data numrg( 71), ncell( 71), kurz( 71), lang( 71)
     *  / 26,1,'P,M,C,21    ','P,M,C,21         '/
      data numrg( 72), ncell( 72), kurz( 72), lang( 72)
     *  / 27,1,'P,C,C,2     ','P,C,C,2          '/
      data numrg( 73), ncell( 73), kurz( 73), lang( 73)
     *  / 28,1,'P,M,A,2     ','P,M,A,2          '/
      data numrg( 74), ncell( 74), kurz( 74), lang( 74)
     *  / 29,1,'P,C,A,21    ','P,C,A,21         '/
      data numrg( 75), ncell( 75), kurz( 75), lang( 75)
     *  / 30,1,'P,N,C,2     ','P,N,C,2          '/
      data numrg( 76), ncell( 76), kurz( 76), lang( 76)
     *  / 31,1,'P,M,N,21    ','P,M,N,21         '/
      data numrg( 77), ncell( 77), kurz( 77), lang( 77)
     *  / 32,1,'P,B,A,2     ','P,B,A,2          '/
      data numrg( 78), ncell( 78), kurz( 78), lang( 78)
     *  / 33,1,'P,N,A,21    ','P,N,A,21         '/
      data numrg( 79), ncell( 79), kurz( 79), lang( 79)
     *  / 34,1,'P,N,N,2     ','P,N,N,2          '/
      data numrg( 80), ncell( 80), kurz( 80), lang( 80)
     *  / 35,1,'C,M,M,2     ','C,M,M,2          '/
      data numrg( 81), ncell( 81), kurz( 81), lang( 81)
     *  / 36,1,'C,M,C,21    ','C,M,C,21         '/
      data numrg( 82), ncell( 82), kurz( 82), lang( 82)
     *  / 37,1,'C,C,C,2     ','C,C,C,2          '/
      data numrg( 83), ncell( 83), kurz( 83), lang( 83)
     *  / 38,1,'A,M,M,2     ','A,M,M,2          '/
      data numrg( 84), ncell( 84), kurz( 84), lang( 84)
     *  / 39,1,'A,B,M,2     ','A,B,M,2          '/
      data numrg( 85), ncell( 85), kurz( 85), lang( 85)
     *  / 40,1,'A,M,A,2     ','A,M,A,2          '/
      data numrg( 86), ncell( 86), kurz( 86), lang( 86)
     *  / 41,1,'A,B,A,2     ','A,B,A,2          '/
      data numrg( 87), ncell( 87), kurz( 87), lang( 87)
     *  / 42,1,'F,M,M,2     ','F,M,M,2          '/
      data numrg( 88), ncell( 88), kurz( 88), lang( 88)
     *  / 43,1,'F,D,D,2     ','F,D,D,2          '/
      data numrg( 89), ncell( 89), kurz( 89), lang( 89)
     *  / 44,1,'I,M,M,2     ','I,M,M,2          '/
      data numrg( 90), ncell( 90), kurz( 90), lang( 90)
     *  / 45,1,'I,B,A,2     ','I,B,A,2          '/
      data numrg( 91), ncell( 91), kurz( 91), lang( 91)
     *  / 46,1,'I,M,A,2     ','I,M,A,2          '/
      data numrg( 92), ncell( 92), kurz( 92), lang( 92)
     *  / 47,1,'P,M,M,M     ','P,2/M,2/M,2/M    '/
      data numrg( 93), ncell( 93), kurz( 93), lang( 93)
     *  / 48,1,'P,N,N,N,1   ','P,2/N,2/N,2/N,1  '/
      data numrg( 94), ncell( 94), kurz( 94), lang( 94)
     *  / 48,2,'P,N,N,N,2   ','P,2/N,2/N,2/N,2  '/
      data numrg( 95), ncell( 95), kurz( 95), lang( 95)
     *  / 49,1,'P,C,C,M     ','P,2/C,2/C,2/M    '/
      data numrg( 96), ncell( 96), kurz( 96), lang( 96)
     *  / 50,1,'P,B,A,N,1   ','P,2/B,2/A,2/N,1  '/
      data numrg( 97), ncell( 97), kurz( 97), lang( 97)
     *  / 50,2,'P,B,A,N,2   ','P,2/B,2/A,2/N,2  '/
      data numrg( 98), ncell( 98), kurz( 98), lang( 98)
     *  / 51,1,'P,M,M,A     ','P,21/M,2/M,2/A   '/
      data numrg( 99), ncell( 99), kurz( 99), lang( 99)
     *  / 52,1,'P,N,N,A     ','P,2/N,21/N,2/A   '/
      data numrg(100), ncell(100), kurz(100), lang(100)
     *  / 53,1,'P,M,N,A     ','P,2/M,2/N,21/A   '/
      data numrg(101), ncell(101), kurz(101), lang(101)
     *  / 54,1,'P,C,C,A     ','P,21/C,2/C,2/A   '/
      data numrg(102), ncell(102), kurz(102), lang(102)
     *  / 55,1,'P,B,A,M     ','P,21/B,21/A,2/M  '/
      data numrg(103), ncell(103), kurz(103), lang(103)
     *  / 56,1,'P,C,C,N     ','P,21/C,21/C,2/N  '/
      data numrg(104), ncell(104), kurz(104), lang(104)
     *  / 57,1,'P,B,C,M     ','P,2/B,21/C,21/M  '/
      data numrg(105), ncell(105), kurz(105), lang(105)
     *  / 58,1,'P,N,N,M     ','P,21/N,21/N,2/M  '/
      data numrg(106), ncell(106), kurz(106), lang(106)
     *  / 59,1,'P,M,M,N,1   ','P,21/M,21/M,2/N,1'/
      data numrg(107), ncell(107), kurz(107), lang(107)
     *  / 59,2,'P,M,M,N,2   ','P,21/M,21/M,2/N,2'/
      data numrg(108), ncell(108), kurz(108), lang(108)
     *  / 60,1,'P,B,C,N     ','P,21/B,2/C,21/N  '/
      data numrg(109), ncell(109), kurz(109), lang(109)
     *  / 61,1,'P,B,C,A     ','P,21/B,21/C,21/A '/
      data numrg(110), ncell(110), kurz(110), lang(110)
     *  / 62,1,'P,N,M,A     ','P,21/N,21/M,21/A '/
      data numrg(111), ncell(111), kurz(111), lang(111)
     *  / 63,1,'C,M,C,M     ','C,2/M,2/C,21/M   '/
      data numrg(112), ncell(112), kurz(112), lang(112)
     *  / 64,1,'C,M,C,A     ','C,2/M,2/C,21/A   '/
      data numrg(113), ncell(113), kurz(113), lang(113)
     *  / 65,1,'C,M,M,M     ','C,2/M,2/M,2/M    '/
      data numrg(114), ncell(114), kurz(114), lang(114)
     *  / 66,1,'C,C,C,M     ','C,2/C,2/C,2/M    '/
      data numrg(115), ncell(115), kurz(115), lang(115)
     *  / 67,1,'C,M,M,A     ','C,2/M,2/M,2/A    '/
      data numrg(116), ncell(116), kurz(116), lang(116)
     *  / 68,1,'C,C,C,A,1   ','C,2/C,2/C,2/A,1  '/
      data numrg(117), ncell(117), kurz(117), lang(117)
     *  / 68,2,'C,C,C,A,2   ','C,2/C,2/C,2/A,2  '/
      data numrg(118), ncell(118), kurz(118), lang(118)
     *  / 69,1,'F,M,M,M     ','F,2/M,2/M,2/M    '/
      data numrg(119), ncell(119), kurz(119), lang(119)
     *  / 70,1,'F,D,D,D,1   ','F,2/D,2/D,2/D,1  '/
      data numrg(120), ncell(120), kurz(120), lang(120)
     *  / 70,2,'F,D,D,D,2   ','F,2/D,2/D,2/D,2  '/
      data numrg(121), ncell(121), kurz(121), lang(121)
     *  / 71,1,'I,M,M,M     ','I,2/M,2/M,2/M    '/
      data numrg(122), ncell(122), kurz(122), lang(122)
     *  / 72,1,'I,B,A,M     ','I,2/B,2/A,2/M    '/
      data numrg(123), ncell(123), kurz(123), lang(123)
     *  / 73,1,'I,B,C,A     ','I,21/B,21/C,21/A '/
      data numrg(124), ncell(124), kurz(124), lang(124)
     *  / 74,1,'I,M,M,A     ','I,21/M,21/M,21/A '/
      data numrg(125), ncell(125), kurz(125), lang(125)
     *  / 75,1,'P,4         ','P,4,,            '/
      data numrg(126), ncell(126), kurz(126), lang(126)
     *  / 76,1,'P,41        ','P,41,,           '/
      data numrg(127), ncell(127), kurz(127), lang(127)
     *  / 77,1,'P,42        ','P,42,,           '/
      data numrg(128), ncell(128), kurz(128), lang(128)
     *  / 78,1,'P,43        ','P,43,,           '/
      data numrg(129), ncell(129), kurz(129), lang(129)
     *  / 79,1,'I,4         ','I,4,,            '/
      data numrg(130), ncell(130), kurz(130), lang(130)
     *  / 80,1,'I,41        ','I,41,,           '/
      data numrg(131), ncell(131), kurz(131), lang(131)
     *  / 81,1,'P,4B        ','P,4B,,           '/
      data numrg(132), ncell(132), kurz(132), lang(132)
     *  / 82,1,'I,4B        ','I,4B,,           '/
      data numrg(133), ncell(133), kurz(133), lang(133)
     *  / 83,1,'P,4/M       ','P,4/M,,          '/
      data numrg(134), ncell(134), kurz(134), lang(134)
     *  / 84,1,'P,42/M      ','P,42/M,,         '/
      data numrg(135), ncell(135), kurz(135), lang(135)
     *  / 85,1,'P,4/N,1     ','P,4/N,,,1        '/
      data numrg(136), ncell(136), kurz(136), lang(136)
     *  / 85,2,'P,4/N,2     ','P,4/N,,,2        '/
      data numrg(137), ncell(137), kurz(137), lang(137)
     *  / 86,1,'P,42/N,1    ','P,42/N,,,1       '/
      data numrg(138), ncell(138), kurz(138), lang(138)
     *  / 86,2,'P,42/N,2    ','P,42/N,,,2       '/
      data numrg(139), ncell(139), kurz(139), lang(139)
     *  / 87,1,'I,4/M       ','I,4/M,,          '/
      data numrg(140), ncell(140), kurz(140), lang(140)
     *  / 88,1,'I,41/A,1    ','I,41/A,,,1       '/
      data numrg(141), ncell(141), kurz(141), lang(141)
     *  / 88,2,'I,41/A,2    ','I,41/A,,,2       '/
      data numrg(142), ncell(142), kurz(142), lang(142)
     *  / 89,1,'P,4,2,2     ','P,4,2,2          '/
      data numrg(143), ncell(143), kurz(143), lang(143)
     *  / 90,1,'P,4,21,2    ','P,4,21,2         '/
      data numrg(144), ncell(144), kurz(144), lang(144)
     *  / 91,1,'P,41,2,2    ','P,41,2,2         '/
      data numrg(145), ncell(145), kurz(145), lang(145)
     *  / 92,1,'P,41,21,2   ','P,41,21,2        '/
      data numrg(146), ncell(146), kurz(146), lang(146)
     *  / 93,1,'P,42,2,2    ','P,42,2,2         '/
      data numrg(147), ncell(147), kurz(147), lang(147)
     *  / 94,1,'P,42,21,2   ','P,42,21,2        '/
      data numrg(148), ncell(148), kurz(148), lang(148)
     *  / 95,1,'P,43,2,2    ','P,43,2,2         '/
      data numrg(149), ncell(149), kurz(149), lang(149)
     *  / 96,1,'P,43,21,2   ','P,43,21,2        '/
      data numrg(150), ncell(150), kurz(150), lang(150)
     *  / 97,1,'I,4,2,2     ','I,4,2,2          '/
      data numrg(151), ncell(151), kurz(151), lang(151)
     *  / 98,1,'I,41,2,2    ','I,41,2,2         '/
      data numrg(152), ncell(152), kurz(152), lang(152)
     *  / 99,1,'P,4,M,M     ','P,4,M,M          '/
      data numrg(153), ncell(153), kurz(153), lang(153)
     *  /100,1,'P,4,B,N     ','P,4,B,M          '/
      data numrg(154), ncell(154), kurz(154), lang(154)
     *  /101,1,'P,42,C,M    ','P,42,C,M         '/
      data numrg(155), ncell(155), kurz(155), lang(155)
     *  /102,1,'P,42,N,M    ','P,42,N,M         '/
      data numrg(156), ncell(156), kurz(156), lang(156)
     *  /103,1,'P,4,C,C     ','P,4,C,C          '/
      data numrg(157), ncell(157), kurz(157), lang(157)
     *  /104,1,'P,4,N,C     ','P,4,N,C          '/
      data numrg(158), ncell(158), kurz(158), lang(158)
     *  /105,1,'P,42,M,C    ','P,42,M,C         '/
      data numrg(159), ncell(159), kurz(159), lang(159)
     *  /106,1,'P,42,B,C    ','P,42,B,C         '/
      data numrg(160), ncell(160), kurz(160), lang(160)
     *  /107,1,'I,4,M,M     ','I,4,M,M          '/
      data numrg(161), ncell(161), kurz(161), lang(161)
     *  /108,1,'I,4,C,M     ','I,4,C,M          '/
      data numrg(162), ncell(162), kurz(162), lang(162)
     *  /109,1,'I,41,M,D    ','I,41,M,D         '/
      data numrg(163), ncell(163), kurz(163), lang(163)
     *  /110,1,'I,41,C,D    ','I,41,C,D         '/
      data numrg(164), ncell(164), kurz(164), lang(164)
     *  /111,1,'P,4B,2,M    ','P,4B,2,M         '/
      data numrg(165), ncell(165), kurz(165), lang(165)
     *  /112,1,'P,4B,2,C    ','P,4B,2,C         '/
      data numrg(166), ncell(166), kurz(166), lang(166)
     *  /113,1,'P,4B,21,M   ','P,4B,21,M        '/
      data numrg(167), ncell(167), kurz(167), lang(167)
     *  /114,1,'P,4B,21,C   ','P,4B,21,C        '/
      data numrg(168), ncell(168), kurz(168), lang(168)
     *  /115,1,'P,4B,M,2    ','P,4B,M,2         '/
      data numrg(169), ncell(169), kurz(169), lang(169)
     *  /116,1,'P,4B,C,2    ','P,4B,C,2         '/
      data numrg(170), ncell(170), kurz(170), lang(170)
     *  /117,1,'P,4B,B,2    ','P,4B,B,2         '/
      data numrg(171), ncell(171), kurz(171), lang(171)
     *  /118,1,'P,4B,N,2    ','P,4B,N,2         '/
      data numrg(172), ncell(172), kurz(172), lang(172)
     *  /119,1,'I,4B,M,2    ','I,4B,M,2         '/
      data numrg(173), ncell(173), kurz(173), lang(173)
     *  /120,1,'I,4B,C,2    ','I,4B,C,2         '/
      data numrg(174), ncell(174), kurz(174), lang(174)
     *  /121,1,'I,4B,2,M    ','I,4B,2,M         '/
      data numrg(175), ncell(175), kurz(175), lang(175)
     *  /122,1,'I,4B,2,D    ','I,4B,2,D         '/
      data numrg(176), ncell(176), kurz(176), lang(176)
     *  /123,1,'P,4/M,M,M   ','P,4/M,2/M,2/M    '/
      data numrg(177), ncell(177), kurz(177), lang(177)
     *  /124,1,'P,4/M,C,C   ','P,4/M,2/C,2/C    '/
      data numrg(178), ncell(178), kurz(178), lang(178)
     *  /125,1,'P,4/N,B,M,1 ','P,4/N,2/B,2/M,1  '/
      data numrg(179), ncell(179), kurz(179), lang(179)
     *  /125,2,'P,4/N,B,M,2 ','P,4/N,2/B,2/M,2  '/
      data numrg(180), ncell(180), kurz(180), lang(180)
     *  /126,1,'P,4/N,N,C,1 ','P,4/N,2/N,2/C,1  '/
      data numrg(181), ncell(181), kurz(181), lang(181)
     *  /126,2,'P,4/N,N,C,2 ','P,4/N,2/N,2/C,2  '/
      data numrg(182), ncell(182), kurz(182), lang(182)
     *  /127,1,'P,4/M,B,M   ','P,4/M,21/B,2/M   '/
      data numrg(183), ncell(183), kurz(183), lang(183)
     *  /128,1,'P,4/M,N,C   ','P,4/M,21/N,2/C   '/
      data numrg(184), ncell(184), kurz(184), lang(184)
     *  /129,1,'P,4/N,M,M,1 ','P,4/N,21/M,2/M,1 '/
      data numrg(185), ncell(185), kurz(185), lang(185)
     *  /129,2,'P,4/N,M,M,2 ','P,4/N,21/M,2/M,2 '/
      data numrg(186), ncell(186), kurz(186), lang(186)
     *  /130,1,'P,4/N,C,C,1 ','P,4/N,21/C,2/C,1 '/
      data numrg(187), ncell(187), kurz(187), lang(187)
     *  /130,2,'P,4/N,C,C,2 ','P,4/N,21/C,2/C,2 '/
      data numrg(188), ncell(188), kurz(188), lang(188)
     *  /131,1,'P,42/M,M,C  ','P,42/M,2/M,2/C   '/
      data numrg(189), ncell(189), kurz(189), lang(189)
     *  /132,1,'P,42/M,C,M  ','P,42/M,2/C,2/M   '/
      data numrg(190), ncell(190), kurz(190), lang(190)
     *  /133,1,'P,42/N,B,C,1','P,42/N,2/B,2/C,1 '/
      data numrg(191), ncell(191), kurz(191), lang(191)
     *  /133,2,'P,42/N,B,C,2','P,42/N,2/B,2/C,2 '/
      data numrg(192), ncell(192), kurz(192), lang(192)
     *  /134,1,'P,42/N,N,M,1','P,42/N,2/N,2/M,1 '/
      data numrg(193), ncell(193), kurz(193), lang(193)
     *  /134,2,'P,42/N,N,M,2','P,42/N,2/N,2/M,2 '/
      data numrg(194), ncell(194), kurz(194), lang(194)
     *  /135,1,'P,42/M,B,C  ','P,42/M,21/B,2/C  '/
      data numrg(195), ncell(195), kurz(195), lang(195)
     *  /136,1,'P,42/M,N,M  ','P,42/M,21/N,2/M  '/
      data numrg(196), ncell(196), kurz(196), lang(196)
     *  /137,1,'P,42/N,M,C,1','P,42/N,21/M,2/C,1'/
      data numrg(197), ncell(197), kurz(197), lang(197)
     *  /137,2,'P,42/N,M,C,2','P,42/N,21/M,2/C,2'/
      data numrg(198), ncell(198), kurz(198), lang(198)
     *  /138,1,'P,42/N,C,M,1','P,42/N,21/C,2/M,1'/
      data numrg(199), ncell(199), kurz(199), lang(199)
     *  /138,2,'P,42/N,C,M,2','P,42/N,21/C,2/M,2'/
      data numrg(200), ncell(200), kurz(200), lang(200)
     *  /139,1,'I,4/M,M,M   ','I,4/M,2/M,2/M    '/
      data numrg(201), ncell(201), kurz(201), lang(201)
     *  /140,1,'I,4/M,C,M   ','I,4/M,2/C,2/M    '/
      data numrg(202), ncell(202), kurz(202), lang(202)
     *  /141,1,'I,41/A,M,D,1','I,41/A,2/M,2/D,1 '/
      data numrg(203), ncell(203), kurz(203), lang(203)
     *  /141,2,'I,41/A,M,D,2','I,41/A,2/M,2/D,2 '/
      data numrg(204), ncell(204), kurz(204), lang(204)
     *  /142,1,'I,41/A,C,D,1','I,41/A,2/C,2/D,1 '/
      data numrg(205), ncell(205), kurz(205), lang(205)
     *  /142,2,'I,41/A,C,D,2','I,41/A,2/C,2/D,2 '/
      data numrg(206), ncell(206), kurz(206), lang(206)
     *  /143,1,'P,3         ','P,3,,            '/
      data numrg(207), ncell(207), kurz(207), lang(207)
     *  /144,1,'P,31        ','P,31,,           '/
      data numrg(208), ncell(208), kurz(208), lang(208)
     *  /145,1,'P,32        ','P,32,,           '/
      data numrg(209), ncell(209), kurz(209), lang(209)
     *  /146,1,'R,3,H       ','R,3,,,H          '/
      data numrg(210), ncell(210), kurz(210), lang(210)
     *  /146,2,'R,3,R       ','R,3,,,R          '/
      data numrg(211), ncell(211), kurz(211), lang(211)
     *  /147,1,'P,3B        ','P,3B,,           '/
      data numrg(212), ncell(212), kurz(212), lang(212)
     *  /148,1,'R,3B,H      ','R,3B,,,H         '/
      data numrg(213), ncell(213), kurz(213), lang(213)
     *  /148,2,'R,3B,R      ','R,3B,,,R         '/
      data numrg(214), ncell(214), kurz(214), lang(214)
     *  /149,1,'P,3,1,2     ','P,3,1,2          '/
      data numrg(215), ncell(215), kurz(215), lang(215)
     *  /150,1,'P,3,2,1     ','P,3,2,1          '/
      data numrg(216), ncell(216), kurz(216), lang(216)
     *  /151,1,'P,31,1,2    ','P,31,1,2         '/
      data numrg(217), ncell(217), kurz(217), lang(217)
     *  /152,1,'P,31,2,1    ','P,31,2,1         '/
      data numrg(218), ncell(218), kurz(218), lang(218)
     *  /153,1,'P,32,1,2    ','P,32,1,2         '/
      data numrg(219), ncell(219), kurz(219), lang(219)
     *  /154,1,'P,32,2,1    ','P,32,2,1         '/
      data numrg(220), ncell(220), kurz(220), lang(220)
     *  /155,1,'R,3,2,H     ','R,3,2,,H         '/
      data numrg(221), ncell(221), kurz(221), lang(221)
     *  /155,2,'R,3,2,R     ','R,3,2,,R         '/
      data numrg(222), ncell(222), kurz(222), lang(222)
     *  /156,1,'P,3,M,1     ','P,3,M,1          '/
      data numrg(223), ncell(223), kurz(223), lang(223)
     *  /157,1,'P,3,1,M     ','P,3,1,M          '/
      data numrg(224), ncell(224), kurz(224), lang(224)
     *  /158,1,'P,3,C,1     ','P,3,C,1          '/
      data numrg(225), ncell(225), kurz(225), lang(225)
     *  /159,1,'P,3,1,C     ','P,3,1,C          '/
      data numrg(226), ncell(226), kurz(226), lang(226)
     *  /160,1,'R,3,M,H     ','R,3,M,,H         '/
      data numrg(227), ncell(227), kurz(227), lang(227)
     *  /160,2,'R,3,M,R     ','R,3,M,,R         '/
      data numrg(228), ncell(228), kurz(228), lang(228)
     *  /161,1,'R,3,C,H     ','R,3,C,,H         '/
      data numrg(229), ncell(229), kurz(229), lang(229)
     *  /161,2,'R,3,C,R     ','R,3,C,,R         '/
      data numrg(230), ncell(230), kurz(230), lang(230)
     *  /162,1,'P,3B,1,M    ','P,3B,1,2/M       '/
      data numrg(231), ncell(231), kurz(231), lang(231)
     *  /163,1,'P,3B,1,C    ','P,3B,1,2/C       '/
      data numrg(232), ncell(232), kurz(232), lang(232)
     *  /164,1,'P,3B,M,1    ','P,3B,2/M,1       '/
      data numrg(233), ncell(233), kurz(233), lang(233)
     *  /165,1,'P,3B,C,1    ','P,3B,2/C,1       '/
      data numrg(234), ncell(234), kurz(234), lang(234)
     *  /166,1,'R,3B,M,H    ','R,3B,2/M,,H      '/
      data numrg(235), ncell(235), kurz(235), lang(235)
     *  /166,2,'R,3B,M,R    ','R,3B,2/M,,R      '/
      data numrg(236), ncell(236), kurz(236), lang(236)
     *  /167,1,'R,3B,C,H    ','R,3B,2/C,,H      '/
      data numrg(237), ncell(237), kurz(237), lang(237)
     *  /167,2,'R,3B,C,R    ','R,3B,2/C,,R      '/
      data numrg(238), ncell(238), kurz(238), lang(238)
     *  /168,1,'P,6         ','P,6,,            '/
      data numrg(239), ncell(239), kurz(239), lang(239)
     *  /169,1,'P,61        ','P,61,,           '/
      data numrg(240), ncell(240), kurz(240), lang(240)
     *  /170,1,'P,65        ','P,65,,           '/
      data numrg(241), ncell(241), kurz(241), lang(241)
     *  /171,1,'P,62        ','P,62,,           '/
      data numrg(242), ncell(242), kurz(242), lang(242)
     *  /172,1,'P,64        ','P,64,,           '/
      data numrg(243), ncell(243), kurz(243), lang(243)
     *  /173,1,'P,63        ','P,63,,           '/
      data numrg(244), ncell(244), kurz(244), lang(244)
     *  /174,1,'P,6B        ','P,6B,,           '/
      data numrg(245), ncell(245), kurz(245), lang(245)
     *  /175,1,'P,6/M       ','P,6/M,,          '/
      data numrg(246), ncell(246), kurz(246), lang(246)
     *  /176,1,'P,63/M      ','P,63/M,,         '/
      data numrg(247), ncell(247), kurz(247), lang(247)
     *  /177,1,'P,6,2,2     ','P,6,2,2          '/
      data numrg(248), ncell(248), kurz(248), lang(248)
     *  /178,1,'P,61,2,2    ','P,61,2,2         '/
      data numrg(249), ncell(249), kurz(249), lang(249)
     *  /179,1,'P,65,2,2    ','P,65,2,2         '/
      data numrg(250), ncell(250), kurz(250), lang(250)
     *  /180,1,'P,62,2,2    ','P,62,2,2         '/
      data numrg(251), ncell(251), kurz(251), lang(251)
     *  /181,1,'P,64,2,2    ','P,64,2,2         '/
      data numrg(252), ncell(252), kurz(252), lang(252)
     *  /182,1,'P,63,2,2    ','P,63,2,2         '/
      data numrg(253), ncell(253), kurz(253), lang(253)
     *  /183,1,'P,6,M,M     ','P,6,M,M          '/
      data numrg(254), ncell(254), kurz(254), lang(254)
     *  /184,1,'P,6,C,C     ','P,6,C,C          '/
      data numrg(255), ncell(255), kurz(255), lang(255)
     *  /185,1,'P,63,C,M    ','P,63,C,M         '/
      data numrg(256), ncell(256), kurz(256), lang(256)
     *  /186,1,'P,63,M,C    ','P,63,M,C         '/
      data numrg(257), ncell(257), kurz(257), lang(257)
     *  /187,1,'P,6B,M,2    ','P,6B,M,2         '/
      data numrg(258), ncell(258), kurz(258), lang(258)
     *  /188,1,'P,6B,C,2    ','P,6B,C,2         '/
      data numrg(259), ncell(259), kurz(259), lang(259)
     *  /189,1,'P,6B,2,M    ','P,6B,2,M         '/
      data numrg(260), ncell(260), kurz(260), lang(260)
     *  /190,1,'P,6B,2,C    ','P,6B,2,C         '/
      data numrg(261), ncell(261), kurz(261), lang(261)
     *  /191,1,'P,6/M,M,M   ','P,6/M,2/M,2/M    '/
      data numrg(262), ncell(262), kurz(262), lang(262)
     *  /192,1,'P,6/M,C,C   ','P,6/M,2/C,2/C    '/
      data numrg(263), ncell(263), kurz(263), lang(263)
     *  /193,1,'P,63/M,C,M  ','P,63/M,2/C,2/M   '/
      data numrg(264), ncell(264), kurz(264), lang(264)
     *  /194,1,'P,63/M,M,C  ','P,63/M,2/M,2/C   '/
      data numrg(265), ncell(265), kurz(265), lang(265)
     *  /195,1,'P,2,3       ','P,2,3,           '/
      data numrg(266), ncell(266), kurz(266), lang(266)
     *  /196,1,'F,2,3       ','F,2,3,           '/
      data numrg(267), ncell(267), kurz(267), lang(267)
     *  /197,1,'I,2,3       ','I,2,3,           '/
      data numrg(268), ncell(268), kurz(268), lang(268)
     *  /198,1,'P,21,3      ','P,21,3,          '/
      data numrg(269), ncell(269), kurz(269), lang(269)
     *  /199,1,'I,21,3      ','I,21,3,          '/
      data numrg(270), ncell(270), kurz(270), lang(270)
     *  /200,1,'P,M,3B      ','P,2/M,3B,        '/
      data numrg(271), ncell(271), kurz(271), lang(271)
     *  /201,1,'P,N,3B,1    ','P,2/N,3B,,1      '/
      data numrg(272), ncell(272), kurz(272), lang(272)
     *  /201,2,'P,N,3B,2    ','P,2/N,3B,,2      '/
      data numrg(273), ncell(273), kurz(273), lang(273)
     *  /202,1,'F,M,3B      ','F,2/M,3B,        '/
      data numrg(274), ncell(274), kurz(274), lang(274)
     *  /203,1,'F,D,3B,1    ','F,2/D,3B,,1      '/
      data numrg(275), ncell(275), kurz(275), lang(275)
     *  /203,2,'F,D,3B,2    ','F,2/D,3B,,2      '/
      data numrg(276), ncell(276), kurz(276), lang(276)
     *  /204,1,'I,M,3B      ','I,2/M,3B,        '/
      data numrg(277), ncell(277), kurz(277), lang(277)
     *  /205,1,'P,A,3B      ','P,21/A,3B,       '/
      data numrg(278), ncell(278), kurz(278), lang(278)
     *  /206,1,'I,A,3B      ','I,21/A,3B,       '/
      data numrg(279), ncell(279), kurz(279), lang(279)
     *  /207,1,'P,4,3,2     ','P,4,3,2          '/
      data numrg(280), ncell(280), kurz(280), lang(280)
     *  /208,1,'P,42,3,2    ','P,42,3,2         '/
      data numrg(281), ncell(281), kurz(281), lang(281)
     *  /209,1,'F,4,3,2     ','F,4,3,2          '/
      data numrg(282), ncell(282), kurz(282), lang(282)
     *  /210,1,'F,41,3,2    ','F,41,3,2         '/
      data numrg(283), ncell(283), kurz(283), lang(283)
     *  /211,1,'I,4,3,2     ','I,4,3,2          '/
      data numrg(284), ncell(284), kurz(284), lang(284)
     *  /212,1,'P,43,3,2    ','P,43,3,2         '/
      data numrg(285), ncell(285), kurz(285), lang(285)
     *  /213,1,'P,41,3,2    ','P,41,3,2         '/
      data numrg(286), ncell(286), kurz(286), lang(286)
     *  /214,1,'I,41,3,2    ','I,41,3,2         '/
      data numrg(287), ncell(287), kurz(287), lang(287)
     *  /215,1,'P,4B,3,M    ','P,4B,3,M         '/
      data numrg(288), ncell(288), kurz(288), lang(288)
     *  /216,1,'F,4B,3,M    ','F,4B,3,M         '/
      data numrg(289), ncell(289), kurz(289), lang(289)
     *  /217,1,'I,4B,3,M    ','I,4B,3,M         '/
      data numrg(290), ncell(290), kurz(290), lang(290)
     *  /218,1,'P,4B,3,N    ','P,4B,3,N         '/
      data numrg(291), ncell(291), kurz(291), lang(291)
     *  /219,1,'F,4B,3,C    ','F,4B,3,C         '/
      data numrg(292), ncell(292), kurz(292), lang(292)
     *  /220,1,'I,4B,3,D    ','I,4B,3,D         '/
      data numrg(293), ncell(293), kurz(293), lang(293)
     *  /221,1,'P,M,3B,M    ','P,4/M,3B,2/M     '/
      data numrg(294), ncell(294), kurz(294), lang(294)
     *  /222,1,'P,N,3B,N,1  ','P,4/N,3B,2/N,1   '/
      data numrg(295), ncell(295), kurz(295), lang(295)
     *  /222,2,'P,N,3B,N,2  ','P,4/N,3B,2/N,2   '/
      data numrg(296), ncell(296), kurz(296), lang(296)
     *  /223,1,'P,M,3B,N    ','P,42/M,3B,2/N    '/
      data numrg(297), ncell(297), kurz(297), lang(297)
     *  /224,1,'P,N,3B,M,1  ','P,42/N,3B,2/M,1  '/
      data numrg(298), ncell(298), kurz(298), lang(298)
     *  /224,2,'P,N,3B,M,2  ','P,42/N,3B,2/M,2  '/
      data numrg(299), ncell(299), kurz(299), lang(299)
     *  /225,1,'F,M,3B,M    ','F,4/M,3B,2/M     '/
      data numrg(300), ncell(300), kurz(300), lang(300)
     *  /226,1,'F,M,3B,C    ','F,4/M,3B,2/C     '/
      data numrg(301), ncell(301), kurz(301), lang(301)
     *  /227,1,'F,D,3B,M,1  ','F,41/D,3B,2/M,1  '/
      data numrg(302), ncell(302), kurz(302), lang(302)
     *  /227,2,'F,D,3B,M,2  ','F,41/D,3B,2/M,2  '/
      data numrg(303), ncell(303), kurz(303), lang(303)
     *  /228,1,'F,D,3B,C,1  ','F,41/D,3B,2/C,1  '/
      data numrg(304), ncell(304), kurz(304), lang(304)
     *  /228,2,'F,D,3B,C,2  ','F,41/D,3B,2/C,2  '/
      data numrg(305), ncell(305), kurz(305), lang(305)
     *  /229,1,'I,M,3B,M    ','I,4/M,3B,2/M     '/
      data numrg(306), ncell(306), kurz(306), lang(306)
     *  /230,1,'I,A,3B,D    ','I,41/A,3B,2/D    '/

      spgrp = ' '

c --- Test auf Raumgruppen-Nummer:

      read(symb(1)(1:3),'(i3)',iostat=ios) nspg
      if(ios.ne.0) nspg = 0

      if(nspg.ge.1.and.nspg.le.230) then
        if(num.le.2) then
          if(num.eq.2) then
            read(symb(2)(1:1),'(i1)',iostat=ios) nc
            if(ios.ne.0) then
              write(*,'(/2x,a,i3)') 
     *        'Error: bad cell choice for space group no. ',nspg
              stop
            end if
          else
            nc = 1
          end if
          do 50 i=1,NRG
             if(numrg(i).eq.nspg.and.nc.eq.ncell(i)) then
               spgrp(1:17) = lang(i)(1:17)
               goto 120
             end if
 50       continue
          write(*,'(/2x,a,i3)') 
     *    'Error: bad cell choice for space group no. ',nspg
          stop
        else
          write(*,'(/2x,a,/23x,a)') 
     *    'Error in INPUT file: wrong number of values,',
     *    ' it should be 1 or 2,','line "SPG> ..."'
          stop          
        end if
      else if(nspg.lt.0.or.nspg.gt.230) then
        write(*,'(/2x,a,i3,a)') 
     *  'Error: bad space group number (No. ',nspg,')'
        stop
      else

c --- Eingelesenes Raumgruppensymbol ohne Blanks zusammensetzen:

 90     ls = 0
        string = ' '
        do 95 k=1,num
           do 93 l=1,10
              ls = ls + 1
              if(symb(k)(l:l).eq.' ') then
                string(ls:ls) = ','
                goto 95
              else
                string(ls:ls) = symb(k)(l:l)
              end if
 93        continue
 95     continue
        string(ls:ls) = ' '

c --- Test auf Raumgruppen-Symbol:
      
        do 100 i=1,NRG
           if(string(1:12).eq.kurz(i)(1:12).or.
     *        string(1:17).eq.lang(i)(1:17)) then
             spgrp(1:17) = lang(i)(1:17)
             nspg = numrg(i)
             nc   = ncell(i)
             goto 120
           end if
 100    continue
        write(*,'(/2x,a,a40)') 
     *  'Error: bad space group symbol: ',string
        stop

      end if

 120  return
      end


c******************************************************************************
c*** Bei magn. Raumgruppen nukleares Raumgruppensymbol bestimmen            ***
c******************************************************************************

      subroutine spgsym(imo,spgrp,char,rgsymb,nsbl,max,mcell)

      external caplet
      character char(max)*10, rgsymb(max)*10, caplet*1
      character spgrp*60, mcell*2 

      call getsym(spgrp,char,nzahl,max)
      if(nzahl.gt.max) then
         write(*,'(2x,2a)') 'Error: Bad space group symbol: ',spgrp
         stop
      end if

      if(imo.eq.2) then
        nsbl = nzahl - 1
        rgsymb(1) = caplet(char(1))
        mcell = char(2)(1:2)
        do 12 i=3,nzahl
           ll = 0
           do 10 k=1,10
              if(char(i)(k:k).ne.'''') then
                ll = ll + 1 
                rgsymb(i-1)(ll:ll) = caplet(char(i)(k:k))
              end if
 10        continue
 12     continue
        if(mcell.eq.'2c'.and.rgsymb(1)(1:1).eq.'P') then
          if(nzahl.eq.2) then
            if(rgsymb(2).eq.'31        ') then
              rgsymb(2)(1:2) = '32'
            else if(rgsymb(2).eq.'32        ') then
              rgsymb(2)(1:2) = '31'
            end if
          else if(nzahl.eq.4) then
            if(rgsymb(2).eq.'31        '.and.
     *         rgsymb(3).eq.'1         '.and.
     *         rgsymb(4).eq.'2         ') then
              rgsymb(2)(1:2) = '32'
            else if(rgsymb(2).eq.'31        '.and.
     *              rgsymb(3).eq.'2         '.and.
     *              rgsymb(4).eq.'1         ') then
              rgsymb(2)(1:2) = '32'
            else if(rgsymb(2).eq.'32        '.and.
     *              rgsymb(3).eq.'2         '.and.
     *              rgsymb(4).eq.'1         ') then
              rgsymb(2)(1:2) = '31'
            else if(rgsymb(2).eq.'32        '.and.
     *              rgsymb(3).eq.'2         '.and.
     *              rgsymb(4).eq.'1         ') then
              rgsymb(2)(1:2) = '31'
            end if
          end if
        end if
      else
        nsbl = nzahl
        do 15 i=1,nsbl
           do 14 k=1,10
              rgsymb(i)(k:k) = caplet(char(i)(k:k))
 14        continue
 15     continue
      end if

      return
      end



