
c   *****************************
c   *        S I M R E F        *
c   *****************************

c   Version 2.6 vom 25.03.1999

c   H. Ritter, J.P. Ihringer, J.K. Maichle
c   Institut fuer Kristallographie
c   der Universitaet Tuebingen
c   Charlottenstrasse 33
c   D-72070 Tuebingen
c   Tel: +49(0)7071/2976389 or
c        +49(0)7071/2975242
c   E-mail: harald.ritter@uni-tuebingen.de
c           joerg.ihringer@uni-tuebingen.de


c Verwendete Variablen und Parameter:
c ===================================
c
c Allgemeine Parameter: 
c
c IDAT:   Max. Zahl der Datensaetze
c LPHASE: Maximale Zahl der Phasen
c NATS:   Maximale Zahl der Atome 
c jobtyp: Modus 
c idges:  Zahl der Datensaetze 
c ip:     Nummer der Phase 
c id:     Nummer des Datensatzes
c
c Ein- und Ausgabe:
c
c EPSY:     Genauigkeitsschranke bei Real-Zahlen
c ka:       Nummer des Ausgabekanals
c ke:       Nummer des Eingabekanals
c inpfile:  Name der Eingabe-Datei
c outfile:  Name des Ausgabedatei
c newin:    0 = es wird keine neue Eingabe-Datei angelegt
c           1 = es wird eine neue Eingabe-Datei angelegt
c maxline   Laenge der Ausgabezeile in der Subroutine writline
c line:     Ausgabezeile der Subroutine writline 
c date:     aktuelles Datum
c time:     aktuelle Uhrzeit
c
c Fit-Parameter (mrqmin):
c
c MSZ:     Max. Zahl der Parameter
c icorr:   0 = keine Ausgabe der Korrelationsmatrix
c          1 = Ausgabe der Korrelationsmatrix
c mcycle:  Anzahl der Rechenzyklen
c maxs:    Anzahl der Parameter
c maxp:    Anzahl der Parameter inclusive gebundener Parameter
c eps:     Faktor zur Steuerung der Abbruchbedingung
c relax:   Relaxationsfaktoren der Parameterverschiebungen
c parfile: Name der Parameter-Datei
c ifpar:   Eingelesene Nummern der Parameter in der Subroutine code
c ipar:    Nummer des Parameters
c covar:   Covarianz-Matrix
c alpha:   Hilfsmatrix
c dz:      Loesungsvektor des LGS
c dl:      Hilfsvektor
c ds:      Verschiebungen der Parameter
c dp:      Standardabweichungen der Parameter
c dersto:  Ableitungen nach den Parametern pro Reflex 
c deriv:   Ableitungen nach den Parametern
c indxc, indxr, ipiv: Hilfsfelder der Subroutine gaussj

c
c Zyklus-File:
c
c MAPA:    Max. Zahl der Ausgaben in der Zyklus-Datei
c kcyc:    Nummer des Ausgabekanals fuer die Zyklus-Datei
c cycfile: Name der Zyklus-Datei
c isp:     Spaltenzahl der Zyklus-Datei: 0 = 80, 1 = 132
c iout:    Anzahl der Ausgaben in der Zyklus-Datei
c param:   Namen der Ausgaben in der Zyklus-Datei
c out:     Ausgaben in der Zyklus-Datei
c
c R-Werte:
c
c numm:  Anzahl der an der Verfeinerung beteiligten Messwerte pro Datensatz
c chisq: Summe der 1./gew(i)*(yobs(i)-ycalc(i))**2 ("Chi square") pro Datensatz
c rpm:   Profil-R-Wert pro Datensatz pro Datensatz
c rwpm:  gewichteter Profil-R-Wert pro Datensatz
c swobs: Summe der  1./gew(i)*(yobs(i)-bac(i))**2  pro Datensatz
c num:   Anzahl der an der Verfeinerung beteiligten Messwerte
c rp:    Profil-R-Wert
c rwp:   gewichteter Profil-R-Wert
c sb1:   Summe der  |ycalc(i) - bac(i)|
c sb2:   Summe der  |yobs(i) - bac(i)|
c sb3:   Summe der  |yobs(i) - ycalc(i)|
c sw1:   Summe der  1./gew(i)*(yobs(i)-bac(i))**2
c sw2    Summe der  1./gew(i)*(yobs(i)-ycalc(i))**2 (Chisq.)
c
c === Parameter des Datensatzes:
c
c titel:  Titel der Einzelnen Datensaetze in der Eingabedatei
c sb:     Standard-Untergrund fuer die Profilberechnung
c tmv:    mu * r fuer die Absorptionkorrektur bei Neutronendaten
c rau:    konst. Rauschen bei Guinier-Filmdaten
c cthm:   Monochromatorwinkel
c ctha:   Analysatorwinkel
c rlim:   Grenzwinkel der Asymmetriekorrektur
c wdt:    Bereich [2theta - wdt*hwb, 2theta + wdt*hwb] 
c         um den Reflex, der angefittet wird
c wf:     Winkelfehler in 2-Theta (nach Rebmann et al.)
c ratio:  Intensitaetsverhaeltnis der Wellenlaengen
c rlamda: Wellenlaengen
c thmin:  Blockanfang in 2-Theta
c thmax:  Blockende in 2-Theta
c step:   Schrittweite in 2-Theta
c temp:   Messtemperatur im Kelvin
c igeo:   Nummer der Beugungsgeometrie
c ixn:    1 = Roentgendaten, 2 = Neutronendaten 
c nphase: Anzahl der Phasen des Datensatzes
c nprof:  Nummer der Profilfunktion
c
c --- Steuerparameter des Datensatzes:
c
c ihkl:  0 = keine nach Phasen getrennte hkl-Liste ausgeben
c        1 = nach Phasen getrennte hkl-Liste ausgeben
c lhkl:  0 = keine gemischte hkl-Liste ausgeben
c        1 = gemischte hkl-Liste ausgeben
c istru: 0 = keine Liste mit Iobs und Icalc ausgeben
c        1 = Liste mit Iobs und Icalc ausgeben
c nref:  0 = keine Datei mit einer hkl-Liste ausgeben,
c        1 = Datei mit einer hkl-Liste ausgeben
c ifou:  0 = keine Datei mit Strukturfaktoren anlegen
c        1 = Datei mit Strukturfaktoren anlegen
c ibac:  0 = Untergrund fitten
c        1 = Untergrund aus dem Untergrund-File verwenden
c        2 = Untergrund aus dem Datenfile verwenden
c ieich: 0 = kein File mit Eichreflexe einlesen 
c        1 = File mit Eichreflexen einlesen
c
c *** Messdaten:
c
c IDZ:     Max. Zahl der Messdaten
c datfile: Name der Datei mit den Messdaten
c izahl:   Anzahl der Daten
c the:     2-Theta-Werte der Messdaten
c obs:     Messwerte
c calc:    berechneter Profilwert
c gew:     Gewichtung des Messwerts
c bac:     Untergrund zu den einzelnen Messwerten
c mit:     0 = Reflex traegt nicht zum Messpunkt bei
c          1 = Reflex traegt zum Messpunkt bei
c
c *** Datei mit den Untergrund-Stuetzpunkten:
c 
c IBZ:     Max. Anzahl von Untergrund-Stuetzstellen pro Datensatz
c bacfile: Datei mit den Untergrund-Stuetzpunkten
c ibck:    Anzahl der Untergrund-Stuetzpunkte
c pos:     2-Theta-Wert des Untergrundwerts
c bck:     Untergrund-Wert
c
c *** Datei mit den Eichreflexen:
c
c MEC:       Maximale Zahl der Eichreflexe
c eichfile:  Name der Datei mit den Eichreflexen
c eichtitel: Titelzeile der Datei mit den Eichreflexen
c neich:     Anzahl der Eichreflexe
c th_cal:    berechnete Lage des Eichreflexes
c th_obs:    beobachtete Lage des Eichreflexes
c silat(T):  Function, die die Silizium-Gitterkonstante 
c            bei der Temperatur T berechnet 
c
c *** Excluded Regions:
c
c IEZ:   Maximale Zahl der "excluded regions"
c iexcl: Anzahl der "excluded regions"
c excl:  "excluded regions"
c
c *** Streudaten:
c
c istreu: 0 = keine Ausgabe der Streudaten,
c         1 = Ausgabe der Streudaten
c nam:    Chem. Symbol incl. der Ladung bei Ionen des externen Streuers
c ptr:    Parameter fuer die Zuordnung der Streudaten zu den einzelnen Atomen
c ac:     Streukoeffizienten zur Berechnung des Formfaktors
c dfp:    anomale Streudaten df'
c dfpp:   anomale Streudaten df''
c nscatm: Anzahl der extern eingegebenen Streudaten
c
c *** Parameter der Phase:
c
c phsnm:   Name der Phase 
c dmax:    maximaler Atomabstand
c imo:     1 = modulierte Struktur, 2 = magnetische Struktur, 0 = sonst
c ibeta:   anisotrope Temperaturfaktoren: 0 = beta's, 1 = B's, 2 = u's
c iord:    Ordnung der Satelliten, die beruecksichtigt werden
c lauq:    Nummer der Lauegruppe der modulierten Struktur

c *** Raumgruppe:
c
c spgrp:   Raumgruppensymbol
c symfile: Name der Symmetriekarten-Datei
c intaus:  Integrale Ausloeschungsregel
c laue:    Nummer der Lauegruppe (1...15)
c icent:   Zentrosymmetrie: 1 = nicht zentrosymmetrisch, 2 = zentrosymmetrisch
c ism:     Rotationsanteil des Symmetrieelements
c tran:    Translationsanteil des Symmetrieelements
c nsym:    Anzahl der Symmetrieelemente
c ismat:   0 = keine Ausgabe der Symmetriekarten
c          1 = Ausgabe der Symmetriekarten
c nprim:   Umfaerbeoperatoren bei magnetischen Strukturen
c
c *** Parameter:
c
c glb:  1-3 = peak-shift parameter
c       4-9 = Untergrund -Parameter
c       10 = Wellenlaenge 1
c aglb: Steuerparameter feur die Verschiebung des Parameters  
c lglb: Kodierung der Parameter
c par:  1,2 = Parameter der Profilfunktion
c       3 = Asymmetrieparameter  
c       4 = Skalierung
c       2 = Allgemeiner Temperaturfaktor
c       6,7 = Parameter der Vorzugsorientierungs-Korrektur,
c       8-10 = Hwb-Parameter
c       11-13 = Wellenvektor 
c       14-19 = rez. Gitterkonstanten
c apar: Steuerparameter feur die Verschiebung des Parameters  
c lpar: Kodierung der Parameter 
c
c *** Atomparameter:
c
c natom: Anzahl der Atome pro Phase und Datensatz
c atext: Name des Atoms
c ntyp:  Chemisches Symbol incl. Ladung bei Ionen
c rmult: Wykoff-Multiplizitaet der Atomlage
c atom:  Atomparameter
c aato:  Steuerparameter feur die Verschiebung des Parameters  
c lato:  Kodierung der Parameter 
c
c *** Modulationsparameter:
c
c IBV:    Maximale Zahl der Basisvektoren
c npos:   Anzahl der symmetrieaequivalenten Positionen
c apos:   Symmetrieaequivalente Positionen eines Gitterkomplexes
c nbas:   Anzahl der Basisvektoren pro Atom
c cbas:   Name des Basisvektors
c numbsv: Nummer des Basisvektors
c pa:     Auslenkungsvektoren der einzelnen Atome
c tp:     Translationsanteil fuer die Phasen-Berechnung 
c ampl:   Amplituden und Phasen der Basisvektoren
c aampl:  Steuerparameter feur die Verschiebung des Parameters   
c lampl:  Kodierung der Parameter 
c
c *** Parameter der Reflexe:
c
c IRS:    Max. Zahl der Reflexe
c NORD:   Maximale Ordnung der Satelliten
c iref:   Indizierung des Reflexes, Multiplizitaet, Nr. der Wellenlaenge
c ic:     Anzahl der Reflexe pro Phase und Datensatz
c theta:  Reflexlagen in Theta (Bogenmass)
c indx:   Reihenfolge der Reflexe nach aufsteigendem 2-Theta-Winkel
c ffreal: Realteil des Strukturfaktors
c ffimag: Imaginaerteil des Strukturfaktors
c rint:   berechnete Reflexintensitaeten/ integrale Intensitaet bei
c         Profilverfeinerung
c riobs:  beobachtete Reflexintensitaeten
c hwb:    Halbwertsbreite des Reflexes
c rlopo:  Lorentz-Polarisations-Faktor
c nuq:    Ordnungen der Satellitenbeitraege eines Reflexes
c f_000:  Strukturfaktor des Reflexes (0,0,0)
c
c *** Profilverfeinerung:
c
c rsig:    Standardabweichungen der integralen Intensitaeten
c defint:  Startwert der integralen Intensitaet
c intfile: Datei mit den integralen Intensitaeten
c aint:    Steuerparameter feur die Verschiebung des Parameters
c lint:    Kodierung der Parameter
c cint:    1 = integrale Intensitaet verfeinern
c          0 = integrale Intensitaet nicht verfeinern
c ifis:    0 = hkl-Generator wird nicht gestartet
c          1 = hkl-Generator wird gestartet
c filex:   .false. = Intensitaetsfile existiert nicht
c          .true.  = Intensitaetsfile existiert 
c
c *** Sonstige:
c
c pref:   Richtungsvektor der Vorzugsorientierung (Pref. Orientation Vector)
c ipref:  0 = Preferred orientation Funktion nach Rietveld (1969)
c         1 = Preferred oreintation Funktion nach Dollase (1986) 
c gitter: Gitterkonstanten
c celld:  Gitterkonstanten
c cellr:  reziproke Gitterkonstanten 
c rtemp, ref, itemp, ltemp: Hilfsfelder der Subroutine fis8
c ass:    Steuert den Aufruf der Subroutine assign (mrqmin)
c sumdr, sumdi, sumxr, sumxi: Hilfsfelder der Subroutine strucalc
c sumda,sumdb: Hilfsfelder der Subroutine strucalc
c nummer: Eingelsene Parameternummern (numcode)
c werte:  Eingelesene Zahlen-Werte (getinp)
c char:   Eingelesene Character-Werte (getinp)


      program simref
c      subroutine simref26(inpfile)

      implicit none
      external metchk, silat, pkshift
      integer IDAT, LPHASE, NATS, MSZ, MAPA, IDZ, IBZ,
     *        MEC, IEZ, IBV, NORD, IRS, MCL

      real q, d, sdw1, sdw2, sdw1ges, sdw2ges, xmin, xmax, as, epsy
      integer ke, ka, kcyc
      integer i, id, ip, iof, ibeg, iend, imoges, iod, ix, 
     *        ierr, irf, idzges, k, laenge, maxline, 
     *        mc, n, na, neu, nr, np, nhkl, nl, ibck, neich

c --- Felddimensionen:

      parameter(IDAT   = 4)        ! Datensaetze
      parameter(LPHASE = 3)        ! Phasen
      parameter(NATS   = 64)       ! Atome
      parameter(MSZ    = 200)      ! verfeinerbare Parameter
      parameter(MCL    = 200)      ! Zahl der Zyklen
      parameter(MAPA   = 1000)     ! Spalten im Zyklus-File
      parameter(IDZ    = 20000)    ! Messdaten 
      parameter(IBZ    = 30)       ! Untergrundstuetzpunkte
      parameter(MEC    = 30)       ! Eichreflexe
      parameter(IEZ    = 40)       ! Excluded Regions
      parameter(IBV    = 48)       ! Basisvektoren
      parameter(NORD   = 6)        ! Ordnung der Satelliten
      parameter(IRS    = 2500)     ! Reflexe

      complex f000(LPHASE,IDAT)

      logical filex(LPHASE)
      logical ass(IDAT)

      character date*9, time*8, line*132, char(MSZ)*132
      character*80 inpfile, outfile, savefile, parfile, cycfile,
     *             datfile(IDAT), bacfile(IDAT), eichfile(IDAT),
     *             symfile(LPHASE), intfile(LPHASE)*80
      character param(MAPA)*7, titel(IDAT)*70, eichtitle*70
      character*4 atext(NATS), ntyp(NATS,IDAT), nam(NATS,IDAT)
      character phsnm(LPHASE)*4, spgrp(LPHASE)*60, intaus(LPHASE)*1
      character cbas(IBV,NATS)*10, cint(IRS)*5

      integer ipar(MSZ,IDAT), ifpar(MSZ), ifbas(IBV,NATS),
     *        nummer(MSZ)
      integer lglb(10,IDAT), lpar(23,LPHASE,IDAT), lato(14,NATS),
     *        lampl(2,IBV,NATS), lint(IRS)
      integer igeo(IDAT), ixn(IDAT), nphase(IDAT), nprof(IDAT), 
     *        ihkl(IDAT), lhkl(IDAT), istru(IDAT), nref(IDAT), 
     *        ifou(IDAT), ifis(IDAT)
      integer izahl(IDAT), mit(IDZ), krr(2,LPHASE,IDZ),
     *        iexcl(IDAT), ibac(IDAT), ieich(IDAT), numm(IDAT)
      integer nscatm(IDAT), ptr(NATS,IDAT)
      integer imo(LPHASE), ibeta(LPHASE), iord(LPHASE),
     *        lauq(LPHASE), natom(LPHASE)
      integer laue(LPHASE), icent(LPHASE), nsym(LPHASE)
      integer ism(3,3,48,LPHASE), nprim(48,LPHASE)
      integer nbas(NATS), numbsv(IBV,NATS), npos(NATS)
      integer iref(7,IRS,IDAT), ic(LPHASE,IDAT), indx(IRS,IDAT),
     *        nuq(2*NORD+1,IRS,IDAT)
      integer itemp(IRS), ltemp(IRS)
      integer jobtyp, idges, ismat, icorr, istreu, newin, 
     *        isp, ishelx, ipref
      integer iout, maxs, maxp, mcycle, num, metchk
      integer indxc(MSZ), indxr(MSZ), ipiv(MSZ)

      real pkshift, silat
      real sb1, sb2, sb3, sw1, sw2
      real covar(MSZ,MSZ), alpha(MSZ,MSZ), dz(MSZ), dl(MSZ),
     *     ds(MSZ), dp(MSZ), out(MAPA,MCL), dersto(MSZ,IRS), 
     *     deriv(MSZ), werte(MSZ), eps
      real swobs(IDAT), chisq(IDAT), rpm(IDAT), rwpm(IDAT)
      real sb(IDAT), tmv(IDAT), rau(IDAT), step(IDAT), temp(IDAT),
     *     cthm(IDAT), ctha(IDAT), rlamda(2,IDAT), ratio(2,IDAT),
     *     thmin(IDAT), thmax(IDAT), wdt(IDAT), rlim(IDAT), wf(IDAT)
      real the(IDZ), obs(IDZ), calc(IDZ), gew(IDZ), bac(IDZ)
      real pos(IBZ), bck(IBZ), th_cal(MEC), th_obs(MEC),
     *     excl(2,IEZ,IDAT)
      real ac(9,NATS,IDAT), dfp(NATS,IDAT), dfpp(NATS,IDAT)
      real dmax(LPHASE), tran(3,48,LPHASE), defint(LPHASE) 
      real pref(3,LPHASE,IDAT), gitter(6,LPHASE,IDAT)
      real relax(8), celld(6), cellr(13)
      real aglb(10,IDAT), apar(23,LPHASE,IDAT), aato(14,NATS),
     *     aampl(2,IBV,NATS), aint(IRS)
      real glb(10,IDAT), par(23,LPHASE,IDAT), atom(14,NATS),
     *     ampl(2,IBV,NATS) 
      real rmult(NATS), pa(6,48,IBV,NATS), tp(3,48,NATS), 
     *     apos(3,48,NATS)
      real riobs(IRS,IDAT), theta(IRS,IDAT), rlopo(IRS,IDAT), 
     *     hwb(IRS,IDAT), ffreal(IRS,IDAT), ffimag(IRS,IDAT), 
     *     rint(IRS,IDAT), rsig(IRS), ref(3,IRS), 
     *     rtemp(IRS)
      real sumdr(2,IBV,NATS), sumdi(2,IBV,NATS), sumxr(11,NATS), 
     *     sumxi(11,NATS), sumda(3,14,NATS), sumdb(3,14,NATS)

      data epsy /0.00001/
      data maxline, ke, ka ,kcyc / 132, 1, 2, 3 / 


c --- Titel des Programms:

      write(*,'(13(/2x,a))')
     *'+-----------------------------------------------------------+',
     *'|                                                           |',
     *'|                        S I M R E F                        |',
     *'|                                                           |',
     *'|                   V e r s i o n   2 . 6                   |',
     *'|                                                           |',
     *'|                       25 March 1999                       |',
     *'|                                                           |',
     *'|          H. Ritter, J. P. Ihringer, J. K. Maichle         |',
     *'| Institut fuer Kristallographie der Universitaet Tuebingen |',
     *'|       Charlottenstr. 33, D-72070 Tuebingen, Germany       |',
     *'|                                                           |',
     *'+-----------------------------------------------------------+'

c --- Datum und Uhrzeit ausgeben:

      call datum(date,time)
      write(*,'(/2x,a,4x,a)') date,time

c --- Felddimensionen ausgeben;

      write(*,'(11(/2x,a,i5))') 
     * 'Maximum number of data sets                    : ',IDAT,
     * 'Maximum number of phases per data set          : ',LPHASE,
     * 'Maximum number of reflections per data set     : ',IRS,
     * 'Maximum number of atoms in the asymm. unit     : ',NATS,
     * 'Maximum number of measured data points         : ',IDZ,
     * 'Maximum number of background data per data set : ',IBZ,
     * 'Maximum number of excl. regions per data set   : ',IEZ,
     * 'Maximum number of parameters                   : ',MSZ,
     * 'Maximum number of parameters in CYCLE file     : ',MAPA,
     * 'Maximum number of base vectors                 : ',IBV,
     * 'Maximum order of the satellites                : ',NORD

c --- Lesen des Namens des INPUT-Files und Einlesen des INPUT-Files:

      write(*,'(/2x,$a)') 'Please type the input filename: '
      read(*,'(a)') inpfile
      write(*,*)
      write(*,*)
      call input(ke, inpfile, jobtyp, idges, ismat, icorr, 
     *           istreu, newin, isp, ishelx, ibeta, mcycle, eps, 
     *           relax, dmax, parfile, ixn, igeo, nphase, nscatm, 
     *           nprof, ihkl, lhkl, ibac, ieich, istru, nref, ifou, 
     *           ifis, titel, datfile, bacfile, eichfile, sb, wdt, 
     *           cthm, ctha, tmv, rlim, rau, wf, thmin, thmax, step, 
     *           rlamda, ratio, excl, iexcl, phsnm, spgrp, symfile, 
     *           imo, lauq, iord, defint, intfile, pref, ipref, 
     *           gitter, atext, ntyp, nam, natom, ac, dfp, dfpp, rmult,
     *           glb, par, atom, LPHASE, NORD, NATS, IDAT, IEZ, MCL)
      if(jobtyp.ge.2) idges = 1


c --- Reflexe und Strukturfaktoren berechnen -----------------------

      iod = 1
      idzges = 0
      do 20 id=1,idges

         if(id.gt.1) iod = iod + izahl(id-1)

         write(*,'(//2x,a,i2,a,/2x,a)') 'Data set ',id,': ',
     *                                  '============'

         if(jobtyp.eq.1.or.jobtyp.eq.5) then

c --- Daten einlesen:

           write(*,*)
           line = ' '
           line = '  Data file: '//datfile(id)
           call writline(6,line,maxline)
           call datinp(ke,IDZ,datfile(id),thmin(id),thmax(id),
     *                 step(id),temp(id),izahl(id),the(iod),
     *                 obs(iod),gew(iod),bac(iod))
           idzges = idzges + izahl(id)
           if(idzges.gt.IDZ) then
             write(*,'(/2x,a,i6,a)')
     *       'To many data points. Only ',IDZ,' are allowed.'
             stop
           end if

c --- Untergrund-Werte einlesen und Untergrund berechnen:

           if(ibac(id).eq.1) then
             write(*,*)
             line = ' '
             line = '  Background data file: '//bacfile(id)
             call writline(6,line,maxline)
             call bacinp(ke,bacfile(id),pos,bck,ibck,IBZ,
     *                   excl(1,1,id),iexcl(id),IEZ)
             call bacalc(pos,bck,ibck,IBZ,the(iod),bac(iod),
     *                   izahl(id),IDZ)
           end if

c --- Bei Filmaufnahmen konstanter Untergrund abziehen:

           if(igeo(id).eq.2.and.rau(id).gt.EPSY)
     *       call uab(rau(id),obs(iod),bac(iod),izahl(id),IDZ)

c --- Absorptionskorrektur bei Debye-Scherrer-Geometrie:

           if(tmv(id).gt.EPSY) 
     *       call absor(igeo(id),tmv(id),the(iod),obs(iod),
     *                  izahl(id),IDZ)

c --- Eichreflexe einlesen und Eichung durchfuehren:

           if(ieich(id).eq.1) then
             write(*,*)
             line = ' '
             line = '  Scale file: '//eichfile(id)
             call writline(6,line,maxline)
             call eichinp(ke,eichfile(id),eichtitle,th_obs,th_cal,
     *                    neich,MEC)
             call eich(th_obs,th_cal,neich,IRS,excl(1,1,id),iexcl(id),
     *                 IEZ,the(iod),izahl(id),IDZ)
           else if(ieich(id).eq.2) then
             write(*,'(/2x,a)') 'Scaling on Silicon pattern:'
             as = silat(temp(id))
             celld(1) = as
             celld(2) = as
             celld(3) = as
             celld(4) = 90.
             celld(5) = 90.
             celld(6) = 90.
             write(*,'(2x,a,f6.1,a,f9.5)')
     *       'Lattice constant at ',temp(id),' K : ',as
             call rezi(celld,cellr)
             do 11 k=1,6
                gitter(k,nphase(id),id) = celld(k)
                par(13+k,nphase(id),id) = cellr(k)
11           continue
           end if

c --- Winkelfehler beruecksichtigen:

           if(abs(wf(id)).gt.EPSY) then
             call gewichte(IDZ,izahl(id),obs(iod),step(id),wf(id),
     *                     gew(iod))
           end if

         end if

c --- Streukoeffizienten/Streulaengen einlesen:

         if(jobtyp.le.3) then
           call scatf(rlamda(1,id),istreu,ixn(id),nscatm(id),nphase(id),
     *                ntyp(1,id),natom,imo,ptr(1,id),nam(1,id),
     *                ac(1,1,id),dfp(1,id),dfpp(1,id),NATS,LPHASE)
         end if

c --- Schleife ueber die Phasen:

         iof = 1
         iend = 0
         imoges = 0
         do 15 ip=1,nphase(id)

            ibeg = iend + 1
            if(ip.gt.1) iof = iof + natom(ip-1) 
            if(imo(ip).ge.1) imoges = imo(ip)

            write(*,'(/2x,a,i2,2a,/2x,a)') 
     *           'Phase ',ip,': ',phsnm(ip),'--------------'

c --- Symmetriekarten einlesen:

            if(id.eq.1) then 
              call syminp(ke,symfile(ip),imo(ip),ismat,spgrp(ip),
     *                    intaus(ip),laue(ip),icent(ip),ism(1,1,1,ip),
     *                    tran(1,1,ip),nprim(1,ip),nsym(ip),
     *                    par(11,ip,id))
            end if

c --- Nummer der Lauegruppe:

            if(imo(ip).eq.1) then
               nl = lauq(ip)
            else
               nl = laue(ip)
            end if

c --- Gitterkonstanten auf ihre Vertraeglichkeit
c      mit der Laue-Gruppe ueberpruefen:

            if(metchk(gitter(1,ip,id),nl).gt.0) stop

c --- Reflexliste berechnen:

            if((jobtyp.eq.5.and.ifis(id).eq.1).or.jobtyp.lt.5) then

              if(jobtyp.eq.1.or.jobtyp.eq.5) then
                xmin = the(iod) + pkshift(the(iod),glb(1,id),3)
                xmax = the(iod-1+izahl(id)) + 
     *                 pkshift(the(iod-1+izahl(id)),glb(1,id),3)
              else
                xmin = thmin(id) + pkshift(thmin(id),glb(1,id),3)
                xmax = thmax(id) + pkshift(thmax(id),glb(1,id),3)
              end if
              call fis8(imo(ip),intaus(ip),nl,icent(ip),
     *                  ism(1,1,1,ip),tran(1,1,ip),nsym(ip),igeo(id),
     *                  iord(ip),par(8,ip,id),par(9,ip,id),
     *                  par(10,ip,id),par(11,ip,id),gitter(1,ip,id),
     *                  rlamda(1,id),cthm(id),ctha(id),xmin,xmax,
     *                  theta(ibeg,id),iref(1,ibeg,id),ref(1,ibeg),
     *                  hwb(ibeg,id),rlopo(ibeg,id),nuq(1,ibeg,id),
     *                  ic(ip,id),NORD,IRS,itemp,ltemp,rtemp)

c --- Reflexliste ausgeben:

              if(ihkl(id).gt.0.or.jobtyp.eq.4) then
                write(*,*)
                call rezi(gitter(1,ip,id),cellr)
                if(ihkl(id).gt.1) then
                  call satwrite(6,cellr,par(11,ip,id),iref(1,ibeg,id),
     *                          theta(ibeg,id),ic(ip,id),IRS)
                else
                  call hklwrite(6,imo(ip),cellr,par(11,ip,id),
     *                          iref(1,ibeg,id),theta(ibeg,id),
     *                          hwb(ibeg,id),rlopo(ibeg,id),
     *                          ic(ip,id),IRS)
                end if
              end if

            end if

c --- Intensitaeten bei Profilverfeinerung voreinstellen:

            if(jobtyp.eq.5.and.ifis(id).eq.1) then
              call intsort(nl,defint(ip),ratio(1,id),
     *                     iref(1,ibeg,id),rint(ibeg,id),cint(ibeg),
     *                     ic(ip,id),IRS)
            end if

c --- Bei Profilverfeinerung pruefen, ob die Datei mit den 
c     Intensitaets-Werten vorhanden ist. Fall ja, Reflexliste 
c     einlesen und sortieren:

            if(jobtyp.eq.5) then
              inquire(file=intfile(ip),exist=filex(ip))
              if(filex(ip)) then
                call intinp(ke,intfile(ip),imo(ip),iref(1,ibeg,id),
     *                      theta(ibeg,id),rlopo(ibeg,id),rint(ibeg,id),
     *                      rsig(ibeg),cint(ibeg),ic(ip,id),
     *                      irf,IRS,ierr)
                if(ierr.eq.1) then
                  stop 'Error: Cannot open intensity file.'
                else if(ierr.eq.2) then
                  stop 'Error reading intensity file.'
                end if
                write(*,'(/2x,a)') 'Reflection data:'
                line = ' '
                line = '  Data file: '//intfile(ip)
                call writline(6,line,maxline)
                write(*,'(2x,a,i4/)')
     *          'Number of reflections = ',irf
                call intwrite(6,IRS,imo(ip),iref(1,ibeg,id),
     *                        theta(ibeg,id),rlopo(ibeg,id),
     *                        rint(ibeg,id),rsig(ibeg),cint(ibeg),
     *                        ic(ip,id))
              end if
            end if

            iend = iend + ic(ip,id)

c --- Auslenkungsvektoren der einzelnen Atome einlesen:

            if(jobtyp.le.3.and.id.eq.1.and.imo(ip).eq.1) then
              write(*,'(//2x,a)') 
     *'Symmetry equivalent positions, phases and displacement vectors:'
              do 12 k=1,natom(ip)
                 na = iof - 1 + k
                 call dispgen(ke,atext(na),atom(1,na),
     *                        gitter(1,ip,id),apos(1,1,na),npos(na),
     *                        pa(1,1,1,na),tp(1,1,na),cbas(1,na),
     *                        ampl(1,1,na),numbsv(1,na),nbas(na),
     *                        ifbas(1,na),IBV)
 12           continue
            end if

c --- Multiplizitaeten der Atome berechnen: 

            if(jobtyp.le.3.and.id.eq.1) then
              do 13 k=1,natom(ip)
                 na = iof - 1 + k
                 rmult(na) = rmult(na)/nsym(ip)
 13           continue
            end if

c --- Strukturfaktoren berechnen:

            if(jobtyp.eq.2.or.jobtyp.eq.3) then
               maxs = 0
               call strucalc(ixn(id),rlamda(1,id),ratio(1,id),
     *              pref(1,ip,id),ipref,gitter(1,ip,id),
     *              ac(1,1,id),dfp(1,id),dfpp(1,id),ptr(iof,id),
     *              imo(ip),ibeta(ip),icent(ip),ism(1,1,1,ip),
     *              tran(1,1,ip),nprim(1,ip),nsym(ip),
     *              aglb(1,id),lglb(1,id),par(1,ip,id),apar(1,ip,id),
     *              lpar(1,ip,id),
     *              atom(1,iof),aato(1,iof),lato(1,iof),rmult(iof),
     *              natom(ip),NATS,
     *              ampl(1,1,iof),aampl(1,1,iof),lampl(1,1,iof),
     *              pa(1,1,1,iof),apos(1,1,iof),tp(1,1,iof),npos(iof),
     *              nbas(iof),IBV,
     *              iref(1,ibeg,id),theta(ibeg,id),hwb(ibeg,id),
     *              rlopo(ibeg,id),ffreal(ibeg,id),ffimag(ibeg,id),
     *              rint(ibeg,id),nuq(1,ibeg,id),ic(ip,id),NORD,IRS,
     *              dersto(1,ibeg),deriv,maxs,MSZ,
     *              sumxr,sumxi,sumdr,sumdi,sumda,sumdb)
            end if

c --- Strukturfaktoren ausgeben:

            if(jobtyp.eq.3) then
              call fnull(f000(ip,id),ixn(id),natom(ip),
     *                   atom(1,iof),ac(1,1,id),dfp(1,id),
     *                   dfpp(1,id),ptr(iof,id),rmult(iof),
     *                   nsym(ip),NATS)
              write(*,*)
              call struout(6,imo(ip),par(11,ip,id),iref(1,ibeg,id),
     *                     theta(ibeg,id),rlopo(ibeg,id),f000(ip,id),
     *                     ffreal(ibeg,id),ffimag(ibeg,id),
     *                     rint(ibeg,id),par(4,ip,id),ic(ip,id),IRS)
            end if

15       continue

c --- Gemischte hkl-Liste ausgeben:

         ibeg = 1
         nhkl = 0
         do 16 ip=1,nphase(id)
            nhkl = nhkl + ic(ip,id)
            do 14 i=ibeg,nhkl
               iref(7,i,id) = ip
14          continue
            ibeg = nhkl + 1
16       continue  
         call indexx(IRS,theta(1,id),indx(1,id),nhkl)
         if(lhkl(id).eq.1) then
           write(*,*)
           call hklout(6,outfile,imoges,par(1,1,id),titel(id),
     *                 iref(1,1,id),indx(1,id),theta(1,id),
     *                 hwb(1,id),rlopo(1,id),nhkl,IRS,LPHASE)
         end if

c --- Bei jobtyp = 4 gemischte hkl-Liste in der Datei 
c     <filename>.hkl abspeichern:

         if(jobtyp.eq.4) then
            call outfil(outfile,inpfile,'hkl',laenge,0)
            write(*,*)
            line = ' '
            line = '  Writing on file '//outfile
            call writline(6,line,maxline)
            call hklout(ka,outfile,imoges,par(1,1,id),titel(id),
     *                  iref(1,1,id),indx(1,id),theta(1,id),hwb(1,id),
     *                  rlopo(1,id),nhkl,IRS,LPHASE)
         end if
           

20    continue

c --- Profilberechnung (pattern.dat) -----------------------------------

      if(jobtyp.eq.2) then

	id = 1
        outfile = 'pattern.dat'
        call procalc(krr,ibac(id),nprof(id),nphase(id),
     *               LPHASE,natom,sb(id),rlim(id),wdt(id),
     *               thmin(id),thmax(id),step(id),izahl(id),
     *               excl(1,1,id),iexcl(id),IEZ,glb(1,id),lglb(1,id),
     *               par(1,1,id),lpar(1,1,id),lato,lampl,nbas,
     *               IBV,NATS,dersto,deriv,MSZ,obs,calc,the,bac,mit,
     *               IDZ,theta(1,id),hwb(1,id),rint(1,id),
     *               lint,ic(1,id),IRS)
        write(*,*)
        line = ' '
        line = '  Writing on file '//outfile
        call writline(6,line,maxline)
        call plotout(ka,outfile,titel(id),step(id),temp(id),
     *               glb(1,id),obs,calc,bac,the,mit,izahl(id),IDZ)
        if(nref(id).eq.1) then
          outfile = ' '
          outfile = 'pattern.hkl'
          write(*,*)
          line = ' '
          line = '  Writing on file '//outfile
          call writline(6,line,maxline)
          call hklout(ka,outfile,imoges,par(1,1,id),titel(id),
     *                iref(1,1,id),indx(1,id),theta(1,id),
     *                hwb(1,id),rlopo(1,id),nhkl,IRS,LPHASE)
        end if


C --- Profilverfeinerung -----------------------------------------------

      else if(jobtyp.eq.1.or.jobtyp.eq.5) then

c --- Einlesen des PARAMETER-Files, 
c     Kodierung und Ausgabe der Parameter-Kodierung:

        call code(ke,parfile,jobtyp,idges,nphase,glb,excl,iexcl,IEZ,
     *            imo,atext,natom,aglb,apar,aato,aampl,aint,cint,
     *            iref,theta,ic,ratio,nbas,IRS,IBV,IDAT,LPHASE,NATS,
     *            ifpar,ifbas,char,werte,nummer,maxs,maxp,MSZ)
        call pcode(jobtyp,idges,nphase,natom,nprof,imo,atext,numbsv,
     *             cbas,nbas,aglb,lglb,apar,lpar,aato,lato,aampl,lampl,
     *             aint,lint,ic,ipar,IRS,IBV,IDAT,LPHASE,NATS,MSZ)

c --- Feldueberlauf in OUTCYC abfangen:

        mc = 3 * maxp + (idges + 1) * 5
        if(mc.gt.MAPA) then
          write(*,'(2(/2x,a))')
     *    'WARNING: Arrays will exceed in module OUTCYC.',
     *    'Please decrease the number of parameters.'
          stop
        end if

c --- Profil anfitten:

        call mrqmin(jobtyp,idges,ass,ixn,nphase,nprof,ibac,
     *              rlamda,ratio,wdt,rlim,pref,ipref,gitter,
     *              ibeta,imo,ism,tran,nprim,nsym,icent,
     *              ac,dfp,dfpp,ptr,excl,iexcl,IEZ,
     *              glb,aglb,lglb,par,apar,lpar,IDAT,LPHASE,
     *              atext,atom,aato,lato,rmult,natom,NATS,
     *              ampl,aampl,lampl,apos,tp,pa,npos,nbas,numbsv,IBV,
     *              the,obs,calc,gew,bac,mit,krr,izahl,IDZ,
     *              iref,theta,hwb,rlopo,ffreal,ffimag,rint,
     *              nuq,aint,lint,ic,IRS,NORD,
     *              covar,alpha,dl,dz,dp,ds,deriv,dersto,ipar,
     *              indxc,indxr,ipiv,maxs,MSZ,
     *              ix,mcycle,icorr,relax,eps,param,out,iout,MAPA,MCL,
     *              sb1,sb2,sb3,sw1,sw2,num,swobs,chisq,rpm,rwpm,numm,
     *              sumxi,sumxr,sumdr,sumdi,sumda,sumdb)

c --- Ergebnisse auf dem Bildschirm ausgeben:         

        call outlas(6,jobtyp,idges,imo,maxs,phsnm,nphase,
     *              nprof,ibac,atext,ntyp,rmult,natom,glb,aglb,
     *              lglb,par,apar,lpar,atom,aato,lato,ampl,aampl,lampl,
     *              dp,intaus,icent,ism,tran,nprim,nsym,pa,apos,tp,npos,
     *              cbas,numbsv,nbas,dmax,num,sb1,sb2,sb3,sw1,sw2,
     *              LPHASE,NATS,IDAT,MSZ,IBV)

c --- Durbin-Watson-Statistik:

        iod = 1
        sdw1ges = 0.
        sdw2ges = 0.
        do 555 id=1,idges
           if(id.gt.1) iod = iod + izahl(id-1)
           call dwstat(d,q,sdw1,sdw2,obs(iod),calc(iod),gew(iod),
     *                 mit(iod),izahl(id),maxs,ibac(id),IDZ)
           sdw1ges = sdw1ges + sdw1
           sdw2ges = sdw2ges + sdw2
 555    continue
        d = sdw1ges/sdw2ges
        q = 2. * ( (num-1)/(num-maxs) - 3.0902/sqrt(real(num+2)) ) 

        write(*,'(/2x,a,/2x,3(a,f10.4))') 
     *  'Durbin-Watson statistic:',
     *  'd = ',d,'    Q = ',q,'    4-Q = ',4. - q

c --- Datum, Titel, und verfeinerte Parameter ins Zyklus-File schreiben:
c     Oeffnen der Datei: VMS: status = 'new', Unix: status = 'unknown'

        call outfil(cycfile,inpfile,'cyc',laenge,0)
        write(*,*)
        line = ' '
        line = '  Writing on file '//cycfile
        call writline(6,line,maxline)
	open(kcyc,file=cycfile,status='unknown',err=99986)
	call outmat(kcyc,ix,mcycle,isp,titel,idges,IDAT,
     *              param,out,iout,MCL,MAPA)

c ---Weitere Ausgabedateien:

        iod = 1
	do 700 id=1,idges

           nr = id
           if(idges.eq.1) nr = 0
           if(id.gt.1) iod = iod + izahl(id-1)

c --- Plot-File <filename.plt> anlegen:

           call outfil(outfile,inpfile,'plt',laenge,nr)
           write(*,*)
           line = ' '
           line = '  Writing on file '//outfile
           call writline(6,line,maxline)
           call plotout(ka,outfile,titel(id),step(id),temp(id),
     *                  glb(1,id),obs(iod),calc(iod),bac(iod),
     *                  the(iod),mit(iod),izahl(id),IDZ)


c --- Reflexlagen-File <filename.hkl> anlegen:

           nhkl = 0
           imoges = 0
           do 30 ip=1,nphase(id)
              nhkl = nhkl + ic(ip,id)
              if(imo(ip).ge.1) imoges = imo(ip)
 30        continue
           call outfil(outfile,inpfile,'hkl',laenge,nr)
           if(nref(id).eq.1) then
             write(*,*)
             line = ' '
             line = '  Writing on file '//outfile
             call writline(6,line,maxline)
             call hklout(ka,outfile,imoges,par(1,1,id),titel(id),
     *                   iref(1,1,id),indx(1,id),theta(1,id),hwb(1,id),
     *                   rlopo(1,id),nhkl,IRS,LPHASE)
           end if

c --- F_000 berechnen:

           if(jobtyp.eq.1) then
             iof = 1
             do 35 ip=1,nphase(id)
                if(ip.gt.1) iof = natom(ip-1)
                call fnull(f000(ip,id),ixn(id),natom(ip),
     *                     atom(1,iof),ac(1,1,id),dfp(1,id),
     *                     dfpp(1,id),ptr(iof,id),rmult(iof),
     *                     nsym(ip),NATS)
35           continue
           end if


           iof = 1
           iend = 0
           do 40 ip=1,nphase(id)
              ibeg = iend + 1
              iend = iend + ic(ip,id)
              if(ip.gt.1) iof = iof + natom(ip-1)

c --- Iobs-Werte berechnen:

              call iobscalc(nprof(id),rlim(id),wdt(id),
     *                      glb(1,id),par(1,ip,id),par(20,ip,id),
     *                      obs(iod),calc(iod),bac(iod),the(iod),
     *                      mit(iod),izahl(id),IDZ,
     *                      excl(1,1,id),iexcl(id),IEZ,
     *                      theta(ibeg,id),hwb(ibeg,id),rint(ibeg,id),
     *                      riobs(ibeg,id),ic(ip,id),IRS)

c --- Verfeinerte integrale Intensitaeten ausgeben:

              if(jobtyp.eq.5) then
                write(*,*)
                line = ' '
                line = '  Writing on file '//intfile(ip)
                call writline(6,line,maxline)
                call intout(ka,intfile(ip),filex(ip),imo(ip),
     *                      iref(1,ibeg,id),theta(ibeg,id),
     *                      rlopo(ibeg,id),rint(ibeg,id),cint(ibeg),
     *                      lint(ibeg),aint(ibeg),ic(ip,id),IRS,
     *                      dp,MSZ,ierr)
              end if

c --- SHELX-Inputfile <filename.ins> anlegen:

              if(ishelx.eq.1.and.id.eq.1) then
                np = ip
                if(nphase(id).eq.1) np = 0
                call outfil(outfile,inpfile,'res',laenge,np)
                write(*,*)
                line = ' '
                line = '  Writing on file '//outfile
                call writline(6,line,maxline)
                call shelxout(ka,outfile,titel(id),ixn(id),temp(id),
     *                        rlamda(1,id),par(1,ip,id),apar(1,ip,id),
     *                        lpar(1,ip,id),ibeta(ip),intaus(ip),
     *                        icent(ip),ism(1,1,1,ip),tran(1,1,ip),
     *                        nsym(ip),atext(iof),ntyp(iof,id),
     *                        ptr(iof,id),rmult(iof),atom(1,iof),
     *                        ac(1,1,id),dfp(1,id),dfpp(1,id),
     *                        natom(ip),NATS,dp,MSZ)
              end if

40         continue


c --- Datei <filename.kof> mit Iobs und Icalc anlegen:

           if(jobtyp.eq.1.and.ifou(id).eq.1) then
             call outfil(outfile,inpfile,'kof',laenge,nr)
             write(*,*)
             line = ' '
             line = '  Writing on file '//outfile
             call writline(6,line,maxline)
             call kofout(ka,outfile,LPHASE,IRS,nphase(id),titel(id),
     *                   iref(1,1,id),theta(1,id),f000(1,id),
     *                   ffreal(1,id),ffimag(1,id),rint(1,id),
     *                   riobs(1,id),ic(1,id))
           end if


700     continue

c --- Berechnete und beobachtete Intensitaeten ausgeben:

        do 800 id=1,idges
           if(istru(id).eq.1) then
             write(*,'(//2x,a,i2,a)') 'Data set ',id,':'
             iend = 0
             do 750 ip=1,nphase(id)
                if(jobtyp.eq.1) then
                  if(nphase(id).gt.1) then 
                    write(*,'(/2x,a,i2,a,a,5x,2(a,f10.3))') 
     *              'Phase: ',ip,': ',phsnm(ip),
     *              'A(000) = ',real(f000(ip,id)),
     *              '    B(000) = ',aimag(f000(ip,id))
                  else
                    write(*,'(/2x,2(a,f10.3))') 
     *              'A(000) = ',real(f000(ip,id)),
     *              '    B(000) = ',aimag(f000(ip,id))
                  end if
                end if
                ibeg = iend + 1
                iend = iend + ic(ip,id)
                call iobswrite(6,imo(ip),par(11,ip,id),
     *                         iref(1,ibeg,id),theta(ibeg,id),
     *                         rint(ibeg,id),riobs(ibeg,id),
     *                         ic(ip,id),IRS)
750          continue
           end if
800     continue

c --- Bragg-R-Werte berechnen, auf dem Bildschirm ausgeben und
c     ins Zyklusfile schreiben:

        call rbragg(kcyc,idges,nphase,LPHASE,IRS,IDAT,riobs,rint,ic)

c --- Zyklusfile schliesen:

        close(kcyc)

c --- Multiplizitaeten zuruecksetzen:

        if(jobtyp.eq.1) then
          id = 1
          iof = 0
          do 115 ip=1,nphase(id)
             if(ip.gt.1) iof = iof + natom(ip-1)
             do 114 k=1,natom(ip)
                na = iof + k
                rmult(na) = rmult(na) * nsym(ip)
114          continue
115       continue
        end if

c --- Neues INPUT-File anlegen:

	if(newin.eq.1) then
          call savinp(ke,ka,inpfile,savefile)
          call rewneu(ke,ka,savefile,inpfile,jobtyp,idges,thmin,
     *                thmax,step,rlamda,ratio,imo,nphase,ibac,atext,
     *                ntyp,natom,rmult,pref,gitter,glb,par,atom,
     *                LPHASE,NATS,IDAT)
        end if

c --- Modulationsparameter in die Dateien "atext".mod schreiben:

        id = 1
        iof = 0
        do 200 ip=1,nphase(id)
           if(ip.gt.1) iof = iof + natom(ip-1)
           if(imo(ip).eq.1) then
             do 150 n=1,natom(ip)
                na = iof + n
                neu = 1
c                do 140 k=1,n-1
c                   kn = k + iof
c                   if(atext(kn).eq.atext(na)) then
c                     neu = 0
c                     goto 144
c                   end if
c 140            continue
 144            if(neu.eq.1.and.nbas(na).gt.0) then 
                  call dispneu(ke,ka,atext(na),cbas(1,na),
     *                         numbsv(1,na),ampl(1,1,na),nbas(na),IBV)
                end if
 150         continue
           end if
 200    continue

      end if


c --- Datum und Uhrzeit:

      call datum(date,time)
      write(*,'(/2x,a,4x,a/)') date,time
      goto 99999


C --- Fehlermeldungen:

99986 stop 'Cannot open CYCLE file'


99999 continue
      end



c******************************************************************************
c***               Berechnung der Absorption                                ***
c******************************************************************************

	subroutine absor(igeo,ur,the,obs,izahl,IDZ)

	real the(IDZ), obs(IDZ)

c       data gamma / 45. /
	data a1, b1, a2, b2 / 1.7133, -0.0368, -0.0927, -0.375 /
	data pi / 3.141593 /

c --- Neutronen-Diffraktometer (Debye-Scherrer-Geometrie mit zylindrischer
c     Probe) nach Hewat, Acta Cryst. A35, p. 248-249 (1979)
c     ur:          u * R, linearer Absorptionskoeffizient u
c                  Radius der zylindrischen Probe in mm
c     c1,c2,c3,c4: Konstanten

	if(igeo.eq.5) then
	  do 100 i=1,izahl
	     sinth2 = (sin(pi/360.*the(i)))**2
	     t = exp(-1.*(a1+b1*sinth2)*ur - (a2+b2*sinth2)*ur**2)
	     obs(i) = obs(i) * t
 100	  continue

c --- Guinier-Geometrie (transmission):
c     Nach M. Oswald, Diplomarbeit:
c     gamma: Winkel zwischen einfallendem Primaerstrahl 
c            und der Probenoberflaeche
c     d:     effektive Dicke der Probe
c     ur:    linearer Apsorptionskoeffizient * d

c	else if(igeo.eq.1.or.igeo.eq.2) then
c	  sigam = sin(pi/180.*gamma)
c	  do 200 i=1,izahl
c	     sigate = sin(pi/180.*(the(i)-gamma))
c	     up = ur/d * (1./sigate - 1./sigam)
c	     t = exp(-1.*ur/sigam) * (1. - exp(-1.*up*d))/up
c            obs(i) = obs(i) * t	  
c 200	  continue

	end if

	return
	end
c******************************************************************************
c*** Bereiche bestimmen, die bei der Verfeinerung beruecksichtigt werden    ***
c******************************************************************************

      subroutine assign(the,krr,mit,izahl,IDZ,LPHASE,glb,wdt,nphase,
     *                  xmin,xmax,excl,iexcl,IEZ,theta,hwb,ic,IRS)

      external pkshift
      parameter(PI = 3.14159265359)
      integer krr(2,LPHASE,IDZ), mit(IDZ), ic(LPHASE) 
      integer ibeg(10), iend(10)
      real theta(IRS), hwb(IRS), the(IDZ), glb(10), excl(2,IEZ)


      ia = 0
      do 20 ip=1,nphase
         iend(ip) = ia + ic(ip)
         ibeg(ip) = ia + 1
         ia = iend(ip)
20    continue      

      do 500 n=1,izahl

         mit(n) = 0
         do 40 ip=1,nphase
            krr(1,ip,n) = 0
            krr(2,ip,n) = 0
40       continue

c --- "Excluded regions" beruecksichtigen:

         do 60 ie=1,iexcl
            if((the(n).gt.excl(1,ie)).and.
     *         (the(n).lt.excl(2,ie))) then
              mit(n) = -1    
              goto 500
            end if
60       continue

         thecor = the(n) + pkshift(the(n),glb(1),3)

         do 200 ip=1,nphase

c            write(*,'(1x,a,f8.3,3(a,i5))') 
c     *      'thecor=',thecor,'   ibeg=',ibeg(ip),
c     *      '   iend=',iend(ip),'   ip=',ip

            k = 0
            do 100 i=ibeg(ip),iend(ip)
               thobs = 360./PI * theta(i)
               rmin  = thobs - wdt * hwb(i)
               rmax  = thobs + wdt * hwb(i)  
               rmin  = amax1(rmin,xmin)
               rmax  = amin1(rmax,xmax)
               if(rmin.le.thecor.and.rmax.ge.thecor) then
                 k = k + 1
                 if(k.eq.1) krr(1,ip,n) = i
               else if(rmin.gt.thecor) then
                 goto 150
               end if
100         continue

150         if(k.gt.0) then
              mit(n) = 1
              ibeg(ip) = krr(1,ip,n)
              krr(2,ip,n) = krr(1,ip,n) + k - 1
            end if

c            if(krr(1,ip,n).gt.0) 
c     *        write(*,'(1x,2(a,i5),a)') 
c     *        '***** krr1=',krr(1,ip,n),'   krr2=',krr(2,ip,n),' *****'

200      continue

500   continue

      return
      end


c******************************************************************************
c***                Atommasse in Gramm bestimmen                            ***
c******************************************************************************

      real function atomass(ntyp)

      external smalet
      parameter(IATOM = 112)
      real Atommasse(IATOM)
      character ntyp*4, Element(IATOM)*4, text*4, smalet*1

      data Element(  1), Atommasse(  1) / 'H   ',   1.00794   /
      data Element(  2), Atommasse(  2) / 'He  ',   4.002602  /
      data Element(  3), Atommasse(  3) / 'Li  ',   6.941     /
      data Element(  4), Atommasse(  4) / 'Be  ',   9.012182  /
      data Element(  5), Atommasse(  5) / 'B   ',  10.811     /
      data Element(  6), Atommasse(  6) / 'C   ',  12.011     /
      data Element(  7), Atommasse(  7) / 'N   ',  14.00674   /
      data Element(  8), Atommasse(  8) / 'O   ',  15.9994    /
      data Element(  9), Atommasse(  9) / 'F   ',  18.9984032 /
      data Element( 10), Atommasse( 10) / 'Ne  ',  20.1797    /
      data Element( 11), Atommasse( 11) / 'Na  ',  22.989768  /
      data Element( 12), Atommasse( 12) / 'Mg  ',  24.305     /
      data Element( 13), Atommasse( 13) / 'Al  ',  26.981539  /
      data Element( 14), Atommasse( 14) / 'Si  ',  28.0855    /
      data Element( 15), Atommasse( 15) / 'P   ',  30.973762  /
      data Element( 16), Atommasse( 16) / 'S   ',  32.066     /
      data Element( 17), Atommasse( 17) / 'Cl  ',  35.4527    /
      data Element( 18), Atommasse( 18) / 'Ar  ',  39.948     /
      data Element( 19), Atommasse( 19) / 'K   ',  39.0983    /
      data Element( 20), Atommasse( 20) / 'Ca  ',  40.078     /
      data Element( 21), Atommasse( 21) / 'Sc  ',  44.95591   /
      data Element( 22), Atommasse( 22) / 'Ti  ',  47.88      /
      data Element( 23), Atommasse( 23) / 'V   ',  50.9415    /
      data Element( 24), Atommasse( 24) / 'Cr  ',  51.9961    /
      data Element( 25), Atommasse( 25) / 'Mn  ',  54.93805   /
      data Element( 26), Atommasse( 26) / 'Fe  ',  55.847     /
      data Element( 27), Atommasse( 27) / 'Co  ',  58.9332    /
      data Element( 28), Atommasse( 28) / 'Ni  ',  58.69      /
      data Element( 29), Atommasse( 29) / 'Cu  ',  63.546     /
      data Element( 30), Atommasse( 30) / 'Zn  ',  65.39      /
      data Element( 31), Atommasse( 31) / 'Ga  ',  69.723     /
      data Element( 32), Atommasse( 32) / 'Ge  ',  72.61      /
      data Element( 33), Atommasse( 33) / 'As  ',  74.92159   /
      data Element( 34), Atommasse( 34) / 'Se  ',  78.96      /
      data Element( 35), Atommasse( 35) / 'Br  ',  79.904     /
      data Element( 36), Atommasse( 36) / 'Kr  ',  83.8       /
      data Element( 37), Atommasse( 37) / 'Rb  ',  85.4678    /
      data Element( 38), Atommasse( 38) / 'Sr  ',  87.62      /
      data Element( 39), Atommasse( 39) / 'Y   ',  88.90585   /
      data Element( 40), Atommasse( 40) / 'Zr  ',  91.224     /
      data Element( 41), Atommasse( 41) / 'Nb  ',  92.90638   /
      data Element( 42), Atommasse( 42) / 'Mo  ',  95.94      /
      data Element( 43), Atommasse( 43) / 'Tc  ',  98.9063    /
      data Element( 44), Atommasse( 44) / 'Ru  ', 101.07      /
      data Element( 45), Atommasse( 45) / 'Rh  ', 102.9055    /
      data Element( 46), Atommasse( 46) / 'Pd  ', 106.42      /
      data Element( 47), Atommasse( 47) / 'Ag  ', 107.8682    /
      data Element( 48), Atommasse( 48) / 'Cd  ', 112.411     /
      data Element( 49), Atommasse( 49) / 'In  ', 114.82      /
      data Element( 50), Atommasse( 50) / 'Sn  ', 118.71      /
      data Element( 51), Atommasse( 51) / 'Sb  ', 121.75      /
      data Element( 52), Atommasse( 52) / 'Te  ', 127.6       /
      data Element( 53), Atommasse( 53) / 'I   ', 126.90447   /
      data Element( 54), Atommasse( 54) / 'Xe  ', 131.29      /
      data Element( 55), Atommasse( 55) / 'Cs  ', 132.90543   /
      data Element( 56), Atommasse( 56) / 'Ba  ', 137.327     /
      data Element( 57), Atommasse( 57) / 'La  ', 138.9055    /
      data Element( 58), Atommasse( 58) / 'Ce  ', 140.115     /
      data Element( 59), Atommasse( 59) / 'Pr  ', 140.90765   /
      data Element( 60), Atommasse( 60) / 'Nd  ', 144.24      /
      data Element( 61), Atommasse( 61) / 'Pm  ', 146.9151    /
      data Element( 62), Atommasse( 62) / 'Sm  ', 150.36      / 
      data Element( 63), Atommasse( 63) / 'Eu  ', 151.965     /
      data Element( 64), Atommasse( 64) / 'Gd  ', 157.25      /
      data Element( 65), Atommasse( 65) / 'Tb  ', 158.92534   /
      data Element( 66), Atommasse( 66) / 'Dy  ', 162.5       /
      data Element( 67), Atommasse( 67) / 'Ho  ', 164.93032   /
      data Element( 68), Atommasse( 68) / 'Er  ', 167.26      /
      data Element( 69), Atommasse( 69) / 'Tm  ', 168.93421   /
      data Element( 70), Atommasse( 70) / 'Yb  ', 173.04      /
      data Element( 71), Atommasse( 71) / 'Lu  ', 174.967     /
      data Element( 72), Atommasse( 72) / 'Hf  ', 178.49      /
      data Element( 73), Atommasse( 73) / 'Ta  ', 180.9479    /
      data Element( 74), Atommasse( 74) / 'W   ', 183.85      /
      data Element( 75), Atommasse( 75) / 'Re  ', 186.207     /
      data Element( 76), Atommasse( 76) / 'Os  ', 190.2       /
      data Element( 77), Atommasse( 77) / 'Ir  ', 192.22      /
      data Element( 78), Atommasse( 78) / 'Pt  ', 195.08      /
      data Element( 79), Atommasse( 79) / 'Au  ', 196.96654   /
      data Element( 80), Atommasse( 80) / 'Hg  ', 200.59      /
      data Element( 81), Atommasse( 81) / 'Tl  ', 204.3833    /
      data Element( 82), Atommasse( 82) / 'Pb  ', 207.2       /
      data Element( 83), Atommasse( 83) / 'Bi  ', 208.98037   / 
      data Element( 84), Atommasse( 84) / 'Po  ', 208.9824    /
      data Element( 85), Atommasse( 85) / 'At  ', 209.9871    /
      data Element( 86), Atommasse( 86) / 'Rn  ', 222.0176    /
      data Element( 87), Atommasse( 87) / 'Fr  ', 223.0197    /
      data Element( 88), Atommasse( 88) / 'Ra  ', 226.0254    /
      data Element( 89), Atommasse( 89) / 'Ac  ', 227.0278    / 
      data Element( 90), Atommasse( 90) / 'Th  ', 232.0381    /
      data Element( 91), Atommasse( 91) / 'Pa  ', 231.0359    /
      data Element( 92), Atommasse( 92) / 'U   ', 238.0289    /
      data Element( 93), Atommasse( 93) / 'Np  ', 237.0482    /
      data Element( 94), Atommasse( 94) / 'Pu  ', 244.0642    /
      data Element( 95), Atommasse( 95) / 'Am  ', 243.0614    /
      data Element( 96), Atommasse( 96) / 'Cm  ', 247.0703    /
      data Element( 97), Atommasse( 97) / 'Bk  ', 247.0703    /
      data Element( 98), Atommasse( 98) / 'Cf  ', 251.0796    /
      data Element( 99), Atommasse( 99) / 'Es  ', 252.0829    /
      data Element(100), Atommasse(100) / 'Fm  ', 257.0951    /
      data Element(101), Atommasse(101) / 'Md  ', 258.0986    /
      data Element(102), Atommasse(102) / 'No  ', 259.1009    /
      data Element(103), Atommasse(103) / 'Lr  ', 260.1053    /
      data Element(104), Atommasse(104) / 'Rf  ', 261.1087    /
      data Element(105), Atommasse(105) / 'Db  ', 262.1138    /
      data Element(106), Atommasse(106) / 'Sg  ', 263.1182    /
      data Element(107), Atommasse(107) / 'Bh  ', 262.1229    /
      data Element(108), Atommasse(108) / 'Hs  ', 265.        /
      data Element(109), Atommasse(109) / 'Mt  ', 266.        /
      data Element(110), Atommasse(110) / 'E110', 270.        /
      data Element(111), Atommasse(111) / 'E111', 272.        /
      data Element(112), Atommasse(112) / 'D   ',   2.0140    /
     

c -- Eventuelle Ladungen entfernen:

      text = '    '
      ip = index(ntyp,'+')
      im = index(ntyp,'-')
      if(ip.gt.0) then
         text(1:ip-1) = ntyp(1:ip-1)
      else if(im.gt.0) then
         text(1:im-1) = ntyp(1:im-1)
      else
         text = ntyp
      end if

c --- Grossgeschriebene Buchstaben in kleine umwandeln 
c     (Ausgenommen ist der Anfangsbuchstaben):

      do 50 i=2,4
         text(i:i) = smalet(text(i:i))
 50   continue

c --- Atommasse bestimmen:

      do 100 i=1,IATOM
         if(text.eq.Element(i)) then
           atomass = Atommasse(i)
           goto 150
         end if
 100  continue
      write(*,'(/2x,3a)') 'Element ',text,' not found.'
      stop

 150  return
      end


c******************************************************************************
c***                 In Grossbuchstaben umwandeln                           ***
c******************************************************************************

      character*1 function smalet(string)

      character string*1
     
      k1 = ichar('A')
      k2 = ichar('Z')
      n1 = ichar('a')
      n2 = ichar('z')

      iz = ichar(string)
      if(iz.ge.k1.and.iz.le.k2) then 
        smalet = char(n1+iz-k1)
      else
        smalet = string 
      end if

      return
      end




c******************************************************************************
c***   Bestimmung des Untergrunds aus Untergrund-Stuetzpunkten              ***
c******************************************************************************

      subroutine bacalc(pos,bck,ibck,IBZ,the,bac,izahl,IDZ)

      real the(IDZ), bac(IDZ), pos(IBZ), bck(IBZ)


c --- Ersten Stuetzpunkt bestimmen, der unterhalb the(1) liegt, und falls 
c     keiner unterhalb liegt, den Stuetzpunkt bck(1) an der Stelle pos(1) 
c     nehmen:

      do 10 k=ibck,1,-1
         if(pos(k).lt.the(1)) then
           ib1 = k
           ibeg = 1
           goto 15
         end if
10    continue

c --- Untergrund bestimmen, falls kleinster Untergrundstuetzpunkt
c     groesser als the(1) ist:

      do 12 n=1,izahl
         if(the(n).lt.pos(1)) then
           bac(n) = bck(1)
         else
           ib1 = 1
           ibeg = n
           goto 15
         end if
12    continue


c --- Letzten Stuetzpunkt bestimmen, der oberhalb the(izahl) liegt, und
c     falls keiner oberhalb liegt, den Stuetzpunkt bck(ibck) an der 
c     Stelle pos(ibck) nehmen:

15    do 20 k=1,ibck
         if(pos(k).gt.the(izahl)) then
           ib2 = k
           iend = izahl
           goto 25
         end if
20    continue


c --- Untergrund bestimmen, falls groesster Untergrundstuetzpunkt
c     kleiner als the(izahl) ist:

      do 22 n=izahl,ibeg,-1
         if(the(n).gt.pos(ibck)) then
           bac(n) = bck(ibck)
         else
           ib2 = ibck
           iend = n
           goto 25
         end if
22    continue


c --- Untergrund-Berechnung im Bereich der Untergrund-Werte:

25    if(ib1.lt.ib2) then
        do 100 k=ib1,ib2
           diff = pos(k+1) - pos(k)
           if(diff.eq.0.) goto 100
           stg  = (bck(k+1) - bck(k))/diff
           do 50 n=ibeg,iend
              if(the(n).gt.pos(k+1)) then
                ibeg = n
                goto 100
              else
                bac(n) = stg * (the(n) - pos(k)) + bck(k)
              end if
50         continue 
100     continue
      end if


      return
      end


C******************************************************************************
C***            Einlesen der Untergrund-Daten                               ***
C******************************************************************************

      subroutine bacinp(ke,bacfile,pos,bck,ibck,IBZ,excl,iexcl,IEZ)

      character string*20, title*70, bacfile*80
      real pos(IBZ), bck(IBZ), excl(2,IEZ)

      ix = 0


c --- Oeffnen des Files:

      open(ke,file=bacfile,status='old',err=1200)


c --- Titel lesen:

      read(ke,'(a)') title
      write(*,'(2x,a,a)') 'Title: ',title

c --- Untergrund-Werte einlesen:

      i=0
10    read(ke,'(a)',end=1000,err=1300) string

        do 20 k=1,18
           if(string(k:k+2).eq.'EXC'.or.
     *        string(k:k+2).eq.'exc') then
             ix = 1
             goto 1000
           end if
20      continue

        i=i+1
        if(i.gt.IBZ) then
          write(*,'(2x,a,i3,/2x,a,i3,a)')
     *    'Number of background data > ',IBZ, 
     *	  'Only ',IBZ,' background data are allowed'
          stop
        end if
        backspace ke
        read(ke,*,err=1300) pos(i), bck(i)

      goto 10


1000  ibck = i 
      write(*,'(2x,a,i3)')
     *'Number of background data = ',ibck


c --- Excluded regions einlesen, falls ix=1:

      if(ix.eq.1) then
        i=0
50      read(ke,*,end=60,err=1400) x,y 
	  i=i+1
          if(i.gt.IEZ) then
	    write(*,'(2x,a,i3,/2x,a,i3,a)') 
     *      'Number of excluded regions > ',IEZ,
     *      'Only ',IEZ,' are allowed'
            stop
          end if
          excl(1,i) = x
          excl(2,i) = y 
	goto 50
60      iexcl = i
      end if

      write(*,'(2x,a,i2)') 
     *'Number of excluded regions = ',iexcl


C --- File schliessen:

      close(ke)
      return


1200  stop ' Cannot open background data file'
1300  close(ke)
      stop ' Error reading background data'
1400  close(ke)
      stop ' Error reading excluded regions' 

      end


c******************************************************************************
c***    Gefitteter Untergrund berechnen, incl. der Ableitungen              ***
c******************************************************************************

      subroutine backgr(back,the,xmin,xmax,glb,lglb,aglb,deriv,
     *           maxs,MSZ)

      integer lglb(10)
      real glb(10), aglb(10), deriv(MSZ), dc(6)

      dc(1) = 1.
      back = glb(4)
      arg = (2.*the - xmax - xmin)/(xmax - xmin)
      do 100 i=2,6
         dc(i) = arg**(i-1)  
         back = back + glb(3+i) * dc(i)
 100  continue

      do 200 k=1,maxs
         do 170 i=4,9
            if(lglb(i).eq.k) deriv(k) = deriv(k) + aglb(i) * dc(i-3)
 170     continue
 200  continue

      return
      end
c******************************************************************************
c***                   n-te Besselfunktion berechnen                        ***
c******************************************************************************

      real function bessfkt(nu,w)

      external bessj, bessj0, bessj1

      n = iabs(nu)
      if(n.eq.0) then
        bs = bessj0(w)
      else if(n.eq.1) then
        bs = bessj1(w)
      else
        if(w.le.0.) then
          bs = 0.
        else
          bs = bessj(n,w)
        end if
      end if
      if(nu.lt.0) bs = (-1.)**n * bs

      bessfkt = bs
      return
      end


C******************************************************************************
C*              BERECHNEN DER N-TEN BESSELFUNKTION                            *
C******************************************************************************

      REAL FUNCTION BESSJ(N,X)

      PARAMETER (IACC=40,BIGNO=1.E10,BIGNI=1.E-10)
      IF(N.LT.2)PAUSE 'bad argument N in BESSJ'
      TOX=2./X
      IF(X.GT.FLOAT(N))THEN
	BJM=BESSJ0(X)
	BJ=BESSJ1(X)
	DO 11 J=1,N-1
	  BJP=J*TOX*BJ-BJM
	  BJM=BJ
	  BJ=BJP
11      CONTINUE
	BESSJ=BJ
      ELSE
	M=2*((N+INT(SQRT(FLOAT(IACC*N))))/2)
	BESSJ=0.
	JSUM=0
	SUM=0.
	BJP=0.
	BJ=1.
	DO 12 J=M,1,-1
	  BJM=J*TOX*BJ-BJP
	  BJP=BJ
	  BJ=BJM
	  IF(ABS(BJ).GT.BIGNO)THEN
	    BJ=BJ*BIGNI
	    BJP=BJP*BIGNI
	    BESSJ=BESSJ*BIGNI
	    SUM=SUM*BIGNI
	  ENDIF
	  IF(JSUM.NE.0)SUM=SUM+BJ
	  JSUM=1-JSUM
	  IF(J.EQ.N)BESSJ=BJP
12      CONTINUE
	SUM=2.*SUM-BJ
	BESSJ=BESSJ/SUM
      ENDIF
      RETURN
      END


C******************************************************************************
C*              BERECHNEN DER 0. BESSELFUNKTION                               *
C******************************************************************************

      REAL FUNCTION BESSJ0(X)

      REAL*8 Y,P1,P2,P3,P4,P5,Q1,Q2,Q3,Q4,Q5,R1,R2,R3,R4,R5,R6,
     *    S1,S2,S3,S4,S5,S6
      DATA P1,P2,P3,P4,P5/1.D0,-.1098628627D-2,.2734510407D-4,
     *    -.2073370639D-5,.2093887211D-6/, Q1,Q2,Q3,Q4,Q5/-.1562499995D-
     *1,
     *    .1430488765D-3,-.6911147651D-5,.7621095161D-6,-.934945152D-7/
      DATA R1,R2,R3,R4,R5,R6/57568490574.D0,-13362590354.D0,651619640.7D
     *0,
     *    -11214424.18D0,77392.33017D0,-184.9052456D0/,
     *    S1,S2,S3,S4,S5,S6/57568490411.D0,1029532985.D0,
     *    9494680.718D0,59272.64853D0,267.8532712D0,1.D0/
      IF(ABS(X).LT.8.)THEN
	Y=X**2
	BESSJ0=(R1+Y*(R2+Y*(R3+Y*(R4+Y*(R5+Y*R6)))))
     *      /(S1+Y*(S2+Y*(S3+Y*(S4+Y*(S5+Y*S6)))))
      ELSE
	AX=ABS(X)
	Z=8./AX
	Y=Z**2
	XX=AX-.785398164
	BESSJ0=SQRT(.636619772/AX)*(COS(XX)*(P1+Y*(P2+Y*(P3+Y*(P4+Y
     *      *P5))))-Z*SIN(XX)*(Q1+Y*(Q2+Y*(Q3+Y*(Q4+Y*Q5)))))
      ENDIF
      RETURN
      END


C******************************************************************************
C*              BERECHNEN DER 1. BESSELFUNKTION                               *
C******************************************************************************

      REAL FUNCTION BESSJ1(X)

      REAL*8 Y,P1,P2,P3,P4,P5,Q1,Q2,Q3,Q4,Q5,R1,R2,R3,R4,R5,R6,
     *    S1,S2,S3,S4,S5,S6
      DATA R1,R2,R3,R4,R5,R6/72362614232.D0,-7895059235.D0,242396853.1D0
     *,
     *    -2972611.439D0,15704.48260D0,-30.16036606D0/,
     *    S1,S2,S3,S4,S5,S6/144725228442.D0,2300535178.D0,
     *    18583304.74D0,99447.43394D0,376.9991397D0,1.D0/
      DATA P1,P2,P3,P4,P5/1.D0,.183105D-2,-.3516396496D-4,.2457520174D-5
     *,
     *    -.240337019D-6/, Q1,Q2,Q3,Q4,Q5/.04687499995D0,-.2002690873D-3
     *,
     *    .8449199096D-5,-.88228987D-6,.105787412D-6/
      IF(ABS(X).LT.8.)THEN
	Y=X**2
	BESSJ1=X*(R1+Y*(R2+Y*(R3+Y*(R4+Y*(R5+Y*R6)))))
     *      /(S1+Y*(S2+Y*(S3+Y*(S4+Y*(S5+Y*S6)))))
      ELSE
	AX=ABS(X)
	Z=8./AX
	Y=Z**2
	XX=AX-2.356194491
	BESSJ1=SQRT(.636619772/AX)*(COS(XX)*(P1+Y*(P2+Y*(P3+Y*(P4+Y
     *      *P5))))-Z*SIN(XX)*(Q1+Y*(Q2+Y*(Q3+Y*(Q4+Y*Q5)))))
     *      *SIGN(1.,X)
      ENDIF
      RETURN
      END


c******************************************************************************
c***               Berechnung der Ableitungen und ycalc                     ***
c******************************************************************************

      subroutine calcxn(ycalc,the,thecor,krr,
     *                  nprof,nphase,LPHASE,natom,rlim,
     *                  lglb,par,lpar,lato,lampl,nbas,IBV,NATS,
     *                  dersto,deriv,maxs,MSZ,
     *                  theta,hwb,rint,lint,IRS)

      implicit none

      external profil

      integer IRS, LPHASE, MSZ, IBV, NATS
      integer natom(LPHASE), lglb(10), krr(2,LPHASE), nbas(NATS)
      integer lpar(23,LPHASE), lato(14,NATS), lampl(2,IBV,NATS)
      integer lint(IRS), nprof, maxs
      real deriv(MSZ), dersto(MSZ,IRS), ycalc, the, thecor, rlim
      real par(23,LPHASE), theta(IRS), hwb(IRS), rint(IRS)

      integer iof, ip, i, j, k, m, n, na, nphase
      real profil, prof, pi, eta1, eta2, thobs
      real dPVdT, dPVdG, dPVdE1, dPVdE2, dPVdA(4)
      data pi / 3.14159265359 /

      ycalc = 0.0

      iof = 0
      do 600 ip=1,nphase

         if(ip.gt.1) iof = iof + natom(ip-1)

         if(krr(1,ip).gt.0) then
           do 500 i=krr(1,ip),krr(2,ip)

              thobs = 360./pi * theta(i)

c --- Winkelabhaengigkeit des Parameters Eta/Pearson-M:


              if(nprof.eq.5) then
                 eta1 = par(1,ip)
                 eta2 = par(2,ip)
              else if(nprof.eq.6.or.nprof.eq.9) then
                 eta1 = par(1,ip) + par(2,ip) * thobs + 
     *                  par(3,ip) * thobs**2
                 eta2 = 0.
              else if(nprof.eq.7) then
                 eta1 = par(1,ip) + par(2,ip) / thobs +
     *                  par(3,ip) / thobs**2
                 eta2 = 0.
              else 
                 eta1 = 0.
                 eta2 = 0.
              end if

c --- Profilberechnung:

              prof = profil(nprof,thecor,thobs,eta1,eta2,
     *                      par(20,ip),hwb(i),rlim,
     *                      dPVdT,dPVdG,dPVdE1,dPVdE2,dPVdA)

              ycalc = ycalc + rint(i) * prof

c --- Schleife ueber alle Parameter: Berechnung der Ableitungen ---------
 
              do 300 k=1,maxs

c --- Nullpunkt, peak shift parameters und Wellenlaenge:

                 if(lglb(1).eq.k)
     *             deriv(k) = deriv(k) - dersto(k,i) * dPVdT 
                 if(lglb(2).eq.k)
     *             deriv(k) = deriv(k) - dersto(k,i) * dPVdT * the 
                 if(lglb(3).eq.k)
     *             deriv(k) = deriv(k) - dersto(k,i) * dPVdT * the**2
                 if(lglb(10).eq.k)
     *             deriv(k) = deriv(k) + dersto(k,i) * dPVdT

c --- Parameter der Profilfunktionen Edgeworth Reihe, 
c     Pseudo-Voigt-Funktion und Pearson-VII-Funktion:

                 if(nprof.eq.5) then
                    if(lpar(1,ip).eq.k) 
     *                   deriv(k) = deriv(k) + dersto(k,i) * dPVdE1
                    if(lpar(2,ip).eq.k) 
     *                   deriv(k) = deriv(k) + dersto(k,i) * dPVdE2
                 else if(nprof.eq.6.or.nprof.eq.9) then
                    if(lpar(1,ip).eq.k) 
     *                   deriv(k) = deriv(k) + 
     *                              dersto(k,i) * dPVdE1
                    if(lpar(2,ip).eq.k) 
     *                   deriv(k) = deriv(k) + 
     *                              dersto(k,i) * dPVdE1 * thobs
                    if(lpar(3,ip).eq.k) 
     *                   deriv(k) = deriv(k) + 
     *                              dersto(k,i) * dPVdE1 * thobs**2
                 else if(nprof.eq.7) then
                    if(lpar(1,ip).eq.k) 
     *                   deriv(k) = deriv(k) + 
     *                              dersto(k,i) * dPVdE1
                    if(lpar(2,ip).eq.k) 
     *                   deriv(k) = deriv(k) + 
     *                              dersto(k,i) * dPVdE1 / thobs
                    if(lpar(3,ip).eq.k) 
     *                   deriv(k) = deriv(k) + 
     *                              dersto(k,i) * dPVdE1 / thobs**2
                 end if

c --- Overall Scale Factor, Overall Temerature Factor,
c     Preferred Orientation Correction Parameters:

                 do 105 m=4,7
                    if(lpar(m,ip).eq.k)
     *                deriv(k) = deriv(k) + dersto(k,i) * prof
 105             continue

C --- Halbwertsbreite-Parameter U, V, W:

                 do 110 m=8,10
                    if(lpar(m,ip).eq.k)
     *                deriv(k) = deriv(k) + dersto(k,i) * dPVdG 
 110             continue

c --- Wellenvektor und reziproke Gitterkonstanten:

	         do 120 m=11,19
                    if(lpar(m,ip).eq.k)
     *                deriv(k) = deriv(k) + dersto(k,i) * dPVdT
 120             continue

c --- Asymmetry correction parameters:

                 do 130 m=20,23
                    if(lpar(m,ip).eq.k) 
     *                deriv(k) = deriv(k) + dersto(k,i) * dPVdA(m-19)
 130             continue

c --- Atomparameter:

                 do 190 n=1,natom(ip)
                    na = iof + n
                    do 140 j=1,14
                       if(lato(j,na).eq.k)
     *                   deriv(k) = deriv(k) + dersto(k,i) * prof
 140                continue
                    do 160 m=1,nbas(na)
                       do 155 j=1,2
                          if(lampl(j,m,na).eq.k)
     *                      deriv(k) = deriv(k) + dersto(k,i) * prof
155                    continue
160                 continue
190              continue

c --- Integrale Intensitaeten (Profil-Verfeinerung):

                 if(lint(i).eq.k)
     *             deriv(k) = deriv(k) + dersto(k,i) * prof

300           continue
 
500        continue    
         end if
600   continue


      return
      end


c******************************************************************************
c***      Parameterdatei einlesen, Kodierung, Ausgabe de Kodierung          ***
c******************************************************************************

      subroutine code(ke,parfile,jobtyp,idges,nphase,glb,excl,iexcl,
     *                IEZ,imo,atext,natom,aglb,apar,aato,aampl,aint,
     *                cint,iref,theta,ic,ratio,nbas,IRS,IBV,IDAT,
     *                LPHASE,NATS,ifpar,ifbas,char,werte,nummer,
     *                maxs,maxp,MSZ)

      parameter(LMAX = 132, NC = 20)
      parameter(PI = 3.14159265359)

      external pkshift
      character parfile*80, atext(NATS)*4, tcod(NC)*1
      character titel*80, ecod*3, char(MSZ)*(LMAX), cint(IRS)*5
      integer nummer(MSZ), lcod(NC), iref(7,IRS,IDAT), ic(LPHASE,IDAT)
      integer nphase(IDAT), natom(LPHASE), ifpar(MSZ), iexcl(IDAT)
      integer nbas(NATS), imo(LPHASE), ifbas(IBV,NATS)
      real werte(MSZ), ucod(NC), ratio(2,IDAT)
      real aglb(10,IDAT), apar(23,LPHASE,IDAT), aato(14,NATS),
     *     aampl(2,IBV,NATS), aint(IRS)
      real excl(2,IEZ,IDAT), theta(IRS,IDAT), glb(10,IDAT)

c --- Parameter: 

c     glb(1,id)     : zeropoint t0
c     glb(2,id)     : peak-shift correction parameter t1
c     glb(3,id)     : peak-shift correction parameter t2
c     glb(4,id)     : background parameter c0
c     glb(5,id)     : background parameter c1
c     glb(6,id)     : background parameter c2
c     glb(7,id)     : background parameter c3
c     glb(8,id)     : background parameter c4
c     glb(9,id)     : background parameter c5
c     glb(10,id)    : 1. Wellenlaenge
c     par( 1,ip,id) : peak shape function parameter e0 
c     par( 2,ip,id) : peak shape function parameter e1 
c     par( 3,ip,id) : peak shape function parameter e2 
c     par( 4,ip,id) : overall scale factor s
c     par( 5,ip,id) : overall temperature factor B0
c     par( 6,ip,id) : preferred orientation correction parameter G1
c     par( 7,ip,id) : preferred orientation correction parameter G2
c     par( 8,ip,id) : Halfwidth parameter U
c     par( 9,ip,id) : Halfwidth parameter V 
c     par(10,ip,id) : Halfwidth parameter W
c     par(11,ip,id) : the wave vector q1
c     par(12,ip,id) : the wave vector q2
c     par(13,ip,id) : the wave vector q3
c     par(14,ip,id) : lattice parameter a 
c     par(15,ip,id) : lattice parameter b 
c     par(16,ip,id) : lattice parameter c 
c     par(17,ip,id) : lattice parameter alpha 
c     par(18,ip,id) : lattice parameter beta
c     par(19,ip,id) : lattice parameter gamma
c     par(20,ip,id) : asymmetry correction parameter a0
c     par(21,ip,id) : asymmetry correction parameter b0
c     par(22,ip,id) : asymmetry correction parameter a1
c     par(23,ip,id) : asymmetry correction parameter b1

      idu = 0
      ecod = 'eee'
      maxp = 0

c --- Formate:

 1001 format(/2x,'Error in PARAMETER file: Line "',a3,
     *       '> ..." is missing, data set ',i2)
 1002 format(/2x,'Error in PARAMETER file: Line "',a3,
     *       '> ..." is missing,',/2x,'phase ',i2,', data set ',i2)
 1003 format(/2x,'Error in PARAMETER file: Line "',a3,
     *       '> ..." is missing,',/2x,'atom ',i2,', phase ',i2,
     *       ', data set ',i2)
 1011 format(/2x,'Error in PARAMETER file: wrong number of parameters,',
     *' it should be ',i2,',',/23x,'line: "',a3,'>...", data set ',i2)
 1012 format(/2x,'Error in PARAMETER file: wrong number of parameters,',
     *' it should be ',i2,',',/23x,'line: "',a3,'> ...", phase ',i2,
     *', data set ',i2)
 1013 format(/2x,'Error in PARAMETER file: wrong number of parameters,',
     *' it should be ',i2,',',/23x,'line "',a3,'> ..." , atom ',i3,
     *', phase ',i2,', data set ',i2)


      write(*,'(/2(/2x,a))') 'Coding of variables:',
     *                       '===================='
      open(ke,file=parfile,status='old',err=99987)

      do 300 id=1,idges

         iof = 0
         iend = 0
         do 200 ip=1,nphase(id)

            ibeg = iend + 1
            iend = iend + ic(ip,id)
            if(ip.gt.1) iof = iof + natom(ip-1)

c --- Titel einlesen:
  
            ist = 0
            call getinp(ke,'TIT',ecod,werte,char,MSZ,num,nn,ist)
            if(ist.eq.11) then
               write(*,1002) 'TIT',ip,id
               stop 
            end if
            titel(1:80) = char(1)(1:80)
c            write(*,'(/2x,a75)') titel

c --- Nummern der zu verfeinernden Parameter:

            do 15 j=1,MSZ
               ifpar(j) = 0 
 15         continue

            ist = 1
            call getinp(ke,'PAR',ecod,werte,char,MSZ,num,nn,ist)
            if(ist.eq.11.or.ist.eq.13) then
               write(*,1002) 'PAR',ip,id
               stop
            end if
            nzp = 40
            if(jobtyp.eq.1) nzp = nzp + (20 * natom(ip)) - 6
            imaxsm = 0
            call numcode(char,werte,num,nummer,nzahl,MSZ) 
            do 20 j=1,nzahl
               nr = nummer(j)
               if((nr.ge.26.and.nr.le.30).or.
     *              nr.eq.40.or.nr.gt.nzp) then
                  write(*,'(/2x,a,a,i2,a,i2,a,i4)') 
     *            'Unknown parameter number in line "PAR> ...":',
     *            ' phase ',ip,', data set ',id,': no. ',nr
                  stop
               else   
                  imaxsm = imaxsm + 1
                  ifpar(imaxsm) = nr
               end if
 20         continue
            maxp = maxp + nzahl

c --- Koordinaten und anisotrope Temperaturfaktoren koennen bei
c     modulierten Strukturen (imo=1) nicht verfeinert werden:

            if(imo(ip).eq.1.and.id.eq.1) then
               do 26 k=1,natom(ip)
                  np = 21 + k * 20
                  do 25 i=1,imaxsm
                     if(ifpar(i).eq.np.or.ifpar(i).eq.np+1.or.
     *                  ifpar(i).eq.np+2) then
                        write(*,'(/2x,a,/2x,a,i2,a,i2,a,i4)') 
     *          'Parameter cannot be refined for modulated structures:',
     *          'line "PAR> ...", phase ',ip,', data set ',id,
     *          ', parameter no. ',ifpar(i)
                        stop
                     end if
 25               continue
 26            continue
            end if

c --- Peak-shift correction Parameter, Untergrund-Parameter
c     und Wellenlaenge:

            if(ip.eq.1) then
              ist = 1
              call getinp(ke,'GLB',ecod,werte,char,MSZ,num,nn,ist)
              if(ist.eq.11.or.ist.eq.13) then
                write(*,1001) 'GLB',id
                stop 
              else
                iz = 10
                if(num.ne.iz) then
                  write(*,1011) iz,'GLB',id
                  stop
                end if
                call chkcod(char,lcod,ucod,tcod,num,NC)
                do 32 l=1,imaxsm
                   do 30 j=1,10
                      if(ifpar(l).eq.j) then
                        if(tcod(j).eq.'%') then
                          sig = sign(1.,ucod(j))
                          aglb(j,id) = aglb(j,lcod(j)) * sig
                        else if(tcod(j).eq.'=') then
                          sig = sign(1.,ucod(j))
                          xx  = 10. * int(aglb(lcod(j),id)/10.)
                          aglb(j,id) = (xx + abs(ucod(j))) * sig
                        else if(tcod(j).eq.'1') then
                          idu = idu + 1
                          aglb(j,id) = 10.*idu + werte(j)
                        end if
                      end if
 30                continue
 32             continue
              end if
 
            end if

c --- Parameter der Profilfunktion, Skalierung s, Temperaturfaktor B0,
c     preferred orientation parameters G1, G2, Halbwertsbreite-Parameter
c     U, V, W und Asymmetriekorrektur-Parameter:

            ist = 1
            call getinp(ke,'PRO',ecod,werte,char,MSZ,num,nn,ist)
            if(ist.eq.11.or.ist.eq.13) then
              write(*,1002) 'PRO',ip,id
              stop
            end if
            iz = 15
            if(num.ne.iz) then
               write(*,1012) iz,'PRO',ip,id
               stop
            end if
            call chkcod(char,lcod,ucod,tcod,num,NC)
            do 41 l=1,imaxsm
               do 40 j=11,24
                  if(ifpar(l).eq.j) then
                     k = j - 10
                     if(j.le.20) then
                        m = 0
                     else
                        m = 9                   
                     end if
                     if(tcod(k).eq.'%') then
                        sig = sign(1.,ucod(k))
                        apar(k+m,ip,id) = apar(k+m,ip,lcod(k)) * sig
                     else if(tcod(k).eq.'&') then
                        sig = sign(1.,ucod(k))
                        xx  = 10. * int(apar(k+m,lcod(k),id)/10.)
                        apar(k+m,ip,id) = (xx + abs(ucod(k))) * sig
                     else if(tcod(k).eq.'=') then
                        sig = sign(1.,ucod(k))
                        xx  = 10. * int(apar(lcod(k)-10,ip,id)/10.)
                        apar(k+m,ip,id) = (xx + abs(ucod(k))) * sig
                     else if(tcod(k).eq.'1') then
                        idu = idu + 1
                        apar(k+m,ip,id) = 10*idu + werte(k)
                     end if
                  end if
 40            continue
 41         continue

c --- Gitterkonstanten und Wellenvektor:

            ist = 1
            call getinp(ke,'MET',ecod,werte,char,MSZ,num,nn,ist)
            if(ist.eq.11.or.ist.eq.13) then
              write(*,1002) 'MET',ip,id
              stop 
            end if
            iz = 9
            if(num.ne.iz) then
               write(*,1012) iz,'MET',ip,id
               stop
            end if
            call chkcod(char,lcod,ucod,tcod,num,NC)
            do 71 l=1,imaxsm
               do 70 j=31,39
                  if(ifpar(l).eq.j) then
                     k = j - 30
                     if(j.le.36) then
                        m = 17
                     else
                        m = 26                   
                     end if
                     if(tcod(k).eq.'%') then
                        sig = sign(1.,ucod(k))
                        apar(j-m,ip,id) = apar(j-m,ip,lcod(k)) * sig
                     else if(tcod(k).eq.'&') then
                        sig = sign(1.,ucod(k))
                        xx  = 10. * int(apar(j-m,lcod(k),id)/10.)
                        apar(j-m,ip,id) = (xx + abs(ucod(k))) * sig
                     else if(tcod(k).eq.'=') then
                        sig = sign(1.,ucod(k))
                        xx  = 10. * int(apar(lcod(k)-m,ip,id)/10.)
                        apar(j-m,ip,id) = (xx + abs(ucod(k))) * sig
                     else if(tcod(k).eq.'1') then
                        idu = idu + 1
                        apar(j-m,ip,id) = 10.*idu + werte(k)
                     end if
                  end if
 70            continue
 71         continue

c --- Atomparameter:

            if(jobtyp.eq.1.and.id.eq.1) then
               do 100 n=1,natom(ip)
                  ist = 1
                  call getinp(ke,'ATO','TIT',werte,char,MSZ,num,nn,ist)
                  if(ist.eq.11.or.ist.eq.13) then
                     write(*,1003) 'ATO',n,ip,id
                     stop
                  end if
                  if(imo(ip).eq.2) then
                     iz = 14
                  else
                     iz = 11
                  end if
                  if(num.ne.iz) then
                     write(*,1013) iz,'ATO',n,ip,id
                     stop
                  end if
                  call chkcod(char,lcod,ucod,tcod,num,MSZ)
                  m1 = 41 + (n-1) * 20
                  m2 = m1 + iz - 1
                  do 91 l=1,imaxsm
                     do 90 j=m1,m2
                        if(ifpar(l).eq.j) then
                           k = j - m1 + 1
                           if(tcod(k).eq.'=') then
                              sig = sign(1.,ucod(k))
                              nx = lcod(k) - 40
                              ia = nx/20 + 1
                              ib = nx - 20*(ia-1)
                              xx = 10. * int(aato(ib,iof+ia)/10.)
                              aato(k,iof+n) = (xx + abs(ucod(k))) * sig
                           else if(tcod(k).eq.'&') then
                              nph = lcod(k)
                              n1 = 1
                              do 87 nn=1,nph-1
                                 n1 = n1 + natom(nn)
 87                           continue
                              do 88 na=n1,natom(nph)
                                 if(atext(na).eq.atext(iof+n)) then
                                    npa = na
                                    goto 89
                                 endif
 88                           continue
 89                           sig = sign(1.,ucod(k))
                              aato(k,iof+n) = aato(k,npa) * sig
                           else if(tcod(k).eq.'1') then
                              idu = idu + 1
                              aato(k,iof+n) = 10.*idu + werte(k)
                           end if
                        end if
 90                  continue
 91               continue
 100           continue
            end if

c --- Modulationsparameter:

            if(jobtyp.eq.1.and.id.eq.1.and.imo(ip).eq.1) then
               do 150 n=1,natom(ip)
                  na = iof + n
                  neu = 1  
                  do 130 k=1,n-1
                     ka = k + iof
                     if(atext(na).eq.atext(ka)) then
                        neu = 0
                        goto 133
                     end if
 130              continue
 133              if(neu.eq.1) then
                     do 140 m=1,nbas(na)
                        if(ifbas(m,na).ne.0) then
                           idu = idu + 1
                           j = 1
                           if(ifbas(m,na).lt.0) j = 2
                           aampl(j,m,na) = 10.*idu + 1.   
                        end if
 140                 continue
                  else
                     do 145 m=1,nbas(na)
                        do 144 j=1,2
                           aampl(j,m,na) = aampl(j,m,ka)
 144                    continue
 145                 continue
                  end if
 150           continue
            end if

c --- Integrale Intensitaeten (nur bei Profilverfeinerung):

            if(jobtyp.eq.5.and.id.eq.1) then
               do 170 l=1,imaxsm
                  if(ifpar(l).eq.25) then
                     do 160 n=ibeg,iend

                        aint(n) = 0.
                        do 155 j=ibeg,n-1
                           if(iref(1,n,id).eq.iref(1,j,id).and.
     *                        iref(2,n,id).eq.iref(1,j,id).and.
     *                        iref(3,n,id).eq.iref(1,j,id).and.
     *                        iref(4,n,id).eq.iref(4,j,id)) then
                              aint(n) = aint(j) - 
     *                             1. + ratio(iref(5,j,id),id)
                              goto 160
                           end if
 155                    continue
                        if(cint(n)(1:1).eq.'1') then
                           idu = idu + 1
                           aint(n) = 10. * idu + 1
                        else if(cint(n)(1:1).eq.'=') then
                           read(cint(n)(2:5),'(i4)') j 
                           aint(n) = aint(j)
                        end if

c --- Excluded Regions beruecksichtigen:

                        th0 = 360./PI * theta(n,id)
                        ths = th0 - pkshift(th0,glb(1,id),3)
                        do 159 k=1,iexcl(id)
                           if(ths.ge.excl(1,k,id).and.
     *                        ths.le.excl(2,k,id)) then
                              aint(n) = 0.
                              if(cint(n)(1:1).eq.'1') idu = idu - 1
                              goto 160
                           end if
 159                    continue

 160                 continue
                  end if
 170           continue
            end if
            
 200     continue

 300  continue

c --- Zahl der Parameter:

      maxs = idu
      if(maxs.gt.MSZ) then
         write(*,'(/2x,a,i4,a)')'Only ',MSZ,' parameters are allowed'
         write(*,'(2x,i4,a)') maxs,' parameters are used'
         stop
      end if
      if(maxs.eq.1) then
         write(*,'(/2x,i4,a)') maxs,' parameter is adjusted'
      else
         write(*,'(/2x,i4,a)') maxs,' parameters are adjusted'
      end if
            
c --- Parameter-File schliessen:

      close(ke) 
      return

c --- Fehlermeldung:

99987 stop ' Cannot open PARAMETER file'

      end


c******************************************************************************
c***              Code uberpruefen und interpretieren                       ***
c******************************************************************************

      subroutine chkcod(char,lcod,ucod,tcod,num,MSZ)

      parameter(EPSY = 0.00001)
      integer lcod(MSZ)
      real ucod(MSZ)
      character char(MSZ)*132, tcod(MSZ)*1, cc*1

      do 100 k=1,num
         lcod(k) = 0
         ucod(k) = 0.
         cc = char(k)(1:1)
         if(cc.eq.'%'.or.cc.eq.'='.or.cc.eq.'&') then
           tcod(k) = cc
           read(char(k)(2:10),'(f8.0)',err=500) xx
           if(abs(xx).gt.EPSY) then
             lcod(k) = int(abs(xx))
             z = amod(xx,1.)
             if(abs(z).lt.EPSY) then
               ucod(k) = sign(1.,xx)
             else
               ucod(k) = z
             end if 
           else
             goto 500
           end if
         else
           read(char(k)(1:10),'(i4)',err=500) mc
           if(mc.ge.0) then
             tcod(k) = '1'
             ucod(k) = mc
           else
             goto 500
           end if
         end if
100   continue

      return

500   stop 'Bad coding of variables'  
      end



c******************************************************************************
c***                Nummerneingabe interpretieren                           ***
c******************************************************************************

      subroutine numcode(char,werte,num,nummer,nzahl,max)
      
      integer nummer(max)
      character char(max)*132
      real werte(max)

      k = 0
      do 100 i=1,num
         do 20 l=1,132
            if(char(i)(l:l).eq.'_') then
              read(char(i)(1:l-1),'(i4)') n1
              read(char(i)(l+1:l+4),'(i4)') n2
              do 15 j=n1,n2
                 k = k + 1
                 nummer(k) = j 
 15           continue
              goto 100
            end if 
 20      continue
         k = k + 1
         nummer(k) = nint(werte(i))
 100  continue
      nzahl = k

      return
      end
c******************************************************************************
c***                       Datenfile einlesen                               ***
c******************************************************************************

      subroutine datinp(ke,IDZ,datfile,thmin,thmax,step,temp,izahl,
     *                  the,obs,gew,bac)

      parameter(max=8)

      character datfile*80, title*70, filspe*3, char(max)*132
      real the(IDZ), obs(IDZ), gew(IDZ), bac(IDZ), werte(max)


c --- Datenfile oeffnen:

      open(ke,file=datfile,status='old',err=1100)

c --- Titel einlesen:

10    ist = 1
      call getinp(ke,'   ','   ',werte,char,max,num,nn,ist)
      if(ist.eq.11.or.ist.eq.13) goto 1000
      if(char(1)(1:3).ne.'DAT'.and.char(1)(1:3).ne.'dat'.and.
     *   char(1)(1:3).ne.'DUO'.and.char(1)(1:3).ne.'duo'.and.
     *   char(1)(1:3).ne.'NEU'.and.char(1)(1:3).ne.'neu') then
        backspace(ke)
        read(ke,'(a)',end=1000,err=1200) title
        write(*,'(2x,2a)') 'Title: ',title
        goto 10
      end if
      if(num.ne.8) goto 10

c --- Kopfzeile einlesen:

c      ist = 1
c      call getinp(ke,'   ','   ',werte,char,max,num,nn,ist)
      filspe = char(1)(1:3)
c      zeit   = werte(2)
      bla    = werte(3)
      step   = werte(4)
      ble    = werte(5)
      temp   = werte(6)
c      iblock = nint(werte(7))
      num    = nint(werte(8)) 

c --- Daten einlesen:

      izahl = nint((ble-bla)/step) + 1

      if(filspe.eq.'DUO'.or.filspe.eq.'duo') then
        call duoein(ke,IDZ,thmin,thmax,bla,step,izahl,obs,gew,the)
      else if(filspe.eq.'NEU'.or.filspe.eq.'neu') then
        call greein(ke,IDZ,thmin,thmax,bla,step,izahl,obs,gew,the)
      else
        call datein(ke,IDZ,thmin,thmax,izahl,obs,gew,bac,the)
      end if

c --- Datei schliessen:

      close(ke)

c --- Parameter des Datensatzes ausgeben:

      thmin = the(1)
      thmax = the(izahl)
      write(*,'(2x,a,f12.6,a,f12.4,/2x,a,f12.5,a,f12.5,/2x,a,i5)')
     *'stepwidth = ',step,'    temperature = ',temp,
     *'2theta minimum = ',thmin,'    2theta maximum = ',thmax,
     *'Number of values = ',izahl

      return

c -- Fehlermeldungen:

1000  stop ' Data file is empty' 
1100  stop ' Cannot open data file'  
1200  stop ' Error reading data file'

      end


C******************************************************************************
C***                       Datenfile einlesen                               ***
C******************************************************************************

      subroutine datein(ke,IDZ,thmin,thmax,izahl,obs,gew,bac,the)

      real obs(IDZ), gew(IDZ), bac(IDZ), the(IDZ)


c --- Thetabereich nicht eingeschraenkt:

      if(thmax.le.0.) then

        n = 0
10      read(ke,*,end=1000,err=1200) x, y, sigma, back
        n = n + 1
        the(n) = x
        obs(n) = y
        if(sigma.le.0.) sigma = 1.
        gew(n) = sigma * sigma
        bac(n) = back
        if(n.eq.IDZ) goto 1000
        goto 10

c --- Thetabereich eingeschraenkt:

      else

        n = 0
20      read(ke,*,end=1000,err=1200) x, y, sigma, back
        if(x.gt.thmax) goto 1000
        if(x.ge.thmin) then
          n = n + 1
          the(n) = x
          obs(n) = y
          if(sigma.le.0.) sigma = 1.
          gew(n) = sigma * sigma
          bac(n) = back
          if(n.eq.IDZ) goto 1000
        end if
        goto 20

      end if

c --- Zahl der eingelesenen Werte:

1000  izahl = n
      return

c -- Fehlermeldung:

1200  stop ' Error reading data file'

      end


C******************************************************************************
C***            Einlesen von Roentgen-Daten im DUO-Format                   ***
C******************************************************************************

      subroutine duoein(ke,IDZ,thmin,thmax,bla,step,izahl,obs,gew,the)

      real y(10)
      real obs(IDZ), gew(IDZ), the(IDZ)


200   format(8(f7.0,1x))

      iacht = izahl/8
      irest = izahl - 8*iacht

      n = 0

c --- Thetabereich nicht eingeschraenkt:

      if(thmax.le.0.) then

	do 300 i=1,iacht
	   read(ke,200,err=1200) (y(j),j=1,8)
	   do 250 j=1,8
	      n = n + 1
	      obs(n) = y(j)
	      the(n) = bla + (n-1)*step
	      if(y(j).eq.0.) then 
                gew(n) = 1.
	      else
		gew(n) = y(j)
	      end if
              if(n.eq.IDZ) goto 600
250        continue
300     continue

	if(irest.ge.1) then
	  read(ke,200,err=1200) (y(j),j=1,irest)
	  do 350 j=1,irest
	     n = n + 1
	     obs(n) = y(j)
	     the(n) = bla + (n-1)*step
	     if(y(j).eq.0.) then
	       gew(n) = 1.
	     else
	       gew(n) = y(j)
	     end if
             if(n.eq.IDZ) goto 600
350       continue      
	end if

c --- Thetabereich eingeschraenkt:
     
      else

        k = 0
	do 500 i=1,iacht
	   read(ke,200,err=1200) (y(j),j=1,8)
	   do 450 j=1,8
	      k = k + 1
	      winkel = bla + (k-1)*step
	      if(winkel.ge.thmin) then
		if(winkel.gt.thmax) goto 600
		n = n + 1
		obs(n) = y(j)
		the(n) = winkel
		if(y(j).eq.0.) then
		  gew(n) = 1.
		else
		  gew(n) = y(j)
		end if
                if(n.eq.IDZ) goto 600
	      end if
450        continue     
500     continue

	if(irest.ge.1) then
	  read(ke,200,err=1200) (y(j),j=1,irest)
	  do 550 j=1,irest
	     k = k + 1
	     winkel = bla + (k-1)*step
	     if(winkel.ge.thmin) then 
	       if(winkel.gt.thmax) goto 600
	       n = n + 1
	       obs(n) = y(j)
	       the(n) = winkel
	       if(y(j).eq.0.) then
		 gew(n) = 1.
	       else
		 gew(n) = y(j)
	       end if
               if(n.eq.IDZ) goto 600
	     end if
550       continue
	end if

      end if

c --- Zahl der eingelesenen Werte:

600   izahl = n
      return

c --- Fehlermeldung:

1200  stop ' Error reading data file'

      end


C******************************************************************************
C***            Einlesen von Grenobler Neutronen-Daten                      ***
C******************************************************************************

      subroutine greein(ke,IDZ,thmin,thmax,bla,step,izahl,obs,gew,the)

      real y(10)
      integer igew(10)
      real obs(IDZ), gew(IDZ), the(IDZ)


200   format(10(i2,f6.0))

      izehn = izahl/10
      irest = izahl - 10*izehn

      n = 0

c --- Thetabereich nicht eingeschraenkt:

      if(thmax.le.0.) then

	do 300 i=1,izehn
	   read(ke,200,err=1200) (igew(j),y(j),j=1,10)
	   do 250 j=1,10
	      n = n + 1
	      obs(n) = y(j)
              if(y(j).eq.0.) then
                gew(n) = 1.
              else
	        if(igew(j).eq.0) then
		  gew(n) = y(j)
	        else
		  gew(n) = y(j)/real(igew(j))
	        end if
              end if
	      the(n) = bla + (n-1)*step
              if(n.eq.IDZ) goto 600
250        continue 
300     continue

	if(irest.ge.1) then
	  read(ke,200,err=1200) (igew(j),y(j),j=1,irest)
	  do 350 j=1,irest
	     n = n + 1
	     obs(n) = y(j)
             if(y(j).eq.0.) then
               gew(n) = 1.
             else
	       if(igew(j).eq.0) then
	         gew(n) = y(j)
	       else
	         gew(n) = y(j)/real(igew(j))
	       end if
             end if
	     the(n) = bla + (n-1)*step
             if(n.eq.IDZ) goto 600
350       continue      
	end if

c --- Thetabereich eingeschraenkt:
     
      else

	k = 0
	do 500 i=1,izehn
	   read(ke,200,err=1200) (igew(j),y(j),j=1,10)
	   do 450 j=1,10
	      k = k + 1
	      winkel = bla + (k-1)*step
	      if(winkel.ge.thmin) then
		if(winkel.gt.thmax) goto 600
		n = n + 1
		obs(n) = y(j)
                if(y(j).eq.0.) then
                  gew(n) = 1.
                else
		  if(igew(j).eq.0) then
		    gew(n) = y(j)
		  else
		    gew(n) = y(j)/real(igew(j))
		  end if
                end if
		the(n) = winkel
                if(n.eq.IDZ) goto 600
	      end if
450        continue     
500     continue

	if(irest.ge.1) then
	  read(ke,200,err=1200) (igew(j),y(j),j=1,irest)
	  do 550 j=1,irest
	     k = k + 1
	     winkel = bla + (k-1)*step
	     if(winkel.ge.thmin) then 
	       if(winkel.gt.thmax) goto 600
	       n = n + 1
	       obs(n) = y(j)
               if(y(j).eq.0.) then
                 gew(n) = 1.
               else
	         if(igew(j).eq.0) then
		   gew(n) = y(j)
	         else
		   gew(n) = y(j)/real(igew(j))
	         end if
               end if
	       the(n) = winkel
               if(n.eq.IDZ) goto 600
	     end if
550       continue
	end if

      end if

c --- Zahl der eingelesenen Werte
600   izahl = n
      return

c --- Fehlermeldung:

1200  stop ' Error reading data file'

      end


c******************************************************************************
c***      Berechnet direkte Gitterkonstanten aus den reziproken             ***
c***           einschliesslich der Standardabweichungen                     ***
C******************************************************************************

      subroutine direct(cellr,stdd,celld,delta,vol,dvol)

      parameter (PI = 3.14159265359)
      real celld(6), delta(6), cellr(6), stdd(6)
      real d(6,6), t(3), r(3), dr(3,3), dt(3,3)


c --- Parameter und Variablen:

c     cellr(1) :  A*                celld(1)  :  A
c     cellr(2) :  B*                celld(2)  :  B
c     cellr(3) :  C*                celld(3)  :  C
c     cellr(4) :  cos(Alpha*)       celld(4)  :  Alpha
c     cellr(5) :  cos(Beta*)        celld(5)  :  Beta
c     cellr(6) :  cos(Gamma*)       celld(6)  :  Gamma
c     stdd(6)  :  Standardabweichungen der reziproken Gitterkonstanten
c     delta(6) :  Standardabweichungen der direkten Gitterkonstanten 
c     vol      :  Zellvolumen
c     dvol     :  Standardabweichung des Zellvolumens

c --- Sinuse der rez. Winkel:

      sinar = sqrt(1.-cellr(4)*cellr(4))
      sinbr = sqrt(1.-cellr(5)*cellr(5))
      singr = sqrt(1.-cellr(6)*cellr(6))

c --- Abkuerzungen:

      r(1) = cellr(5)*cellr(6) - cellr(4)
      r(2) = cellr(6)*cellr(4) - cellr(5)
      r(3) = cellr(4)*cellr(5) - cellr(6)
      t(1) = sinbr * singr
      t(2) = singr * sinar
      t(3) = sinar * sinbr

c --- Ableitungen von r nach den Cosinusen der rez. Winkel:

      dr(1,1) = -1.
      dr(1,2) =  cellr(6)
      dr(1,3) =  cellr(5)
      dr(2,1) =  cellr(6)
      dr(2,2) = -1.
      dr(2,3) =  cellr(4)
      dr(3,1) =  cellr(5)
      dr(3,2) =  cellr(4)
      dr(3,3) = -1.

c --- Ableitungen von t nach den Cosinusen der rez. Winkel:

      dt(1,1) =  0.
      dt(1,2) = -cellr(5)/sinbr * singr
      dt(1,3) = -cellr(6)/singr * sinbr
      dt(2,1) = -cellr(4)/sinar * singr
      dt(2,2) =  0.
      dt(2,3) = -cellr(6)/singr * sinar
      dt(3,1) = -cellr(4)/sinar * sinbr
      dt(3,2) = -cellr(5)/sinbr * sinar
      dt(3,3) =  0.

c --- Winkelanteil des reziproken Volumens und Ableitungen:

      u = sqrt(1. + 2.*cellr(4)*cellr(5)*cellr(6) -
     *    cellr(4)*cellr(4) - cellr(5)*cellr(5) - cellr(6)*cellr(6))

      dua = r(1)/u
      dub = r(2)/u
      dug = r(3)/u

c --- direkte Gitterkonstanten:

      celld(1) = sinar/cellr(1)/u
      celld(2) = sinbr/cellr(2)/u
      celld(3) = singr/cellr(3)/u

c --- Ableitungen von a, b und c nach den gesternten Groessen:

      d(1,1) = -celld(1)/cellr(1)
      d(1,2) =  0.
      d(1,3) =  0.
      d(1,4) = -celld(1)*dua/u - cellr(4)/sinar/cellr(1)/u
      d(1,5) = -celld(1)*dub/u
      d(1,6) = -celld(1)*dug/u

      d(2,1) =  0.
      d(2,2) = -celld(2)/cellr(2)
      d(2,3) =  0.
      d(2,4) = -celld(2)*dua/u
      d(2,5) = -celld(2)*dub/u - cellr(5)/sinbr/cellr(2)/u 
      d(2,6) = -celld(2)*dug/u

      d(3,1) =  0.
      d(3,2) =  0.
      d(3,3) = -celld(3)/cellr(3)
      d(3,4) = -celld(3)*dua/u
      d(3,5) = -celld(3)*dub/u
      d(3,6) = -celld(3)*dug/u - cellr(6)/singr/cellr(3)/u 

c --- Cosinuse und Sinuse der direkten Winkel:

      cosad = r(1)/t(1)
      cosbd = r(2)/t(2)
      cosgd = r(3)/t(3)

      sinad = sqrt(1.-cosad*cosad)
      sinbd = sqrt(1.-cosbd*cosbd)
      singd = sqrt(1.-cosgd*cosgd)

c --- Direkte Winkel in Grad:

      celld(4) = 180./PI * acos(cosad)
      celld(5) = 180./PI * acos(cosbd)
      celld(6) = 180./PI * acos(cosgd)

c --- Ableitungen von Alpha, Beta und Gamma nach den gesternten Groessen:

      d(4,1) =  0.
      d(4,2) =  0.
      d(4,3) =  0.
      d(4,4) = (-dr(1,1)*t(1) + r(1)*dt(1,1)) /sinad/t(1)/t(1)
      d(4,5) = (-dr(1,2)*t(1) + r(1)*dt(1,2)) /sinad/t(1)/t(1)
      d(4,6) = (-dr(1,3)*t(1) + r(1)*dt(1,3)) /sinad/t(1)/t(1)

      d(5,1) =  0.
      d(5,2) =  0.
      d(5,3) =  0.
      d(5,4) = (-dr(2,1)*t(2) + r(2)*dt(2,1)) /sinbd/t(2)/t(2)
      d(5,5) = (-dr(2,2)*t(2) + r(2)*dt(2,2)) /sinbd/t(2)/t(2)
      d(5,6) = (-dr(2,3)*t(2) + r(2)*dt(2,3)) /sinbd/t(2)/t(2)

      d(6,1) =  0.
      d(6,2) =  0.
      d(6,3) =  0.
      d(6,4) = (-dr(3,1)*t(3) + r(3)*dt(3,1)) /singd/t(3)/t(3)
      d(6,5) = (-dr(3,2)*t(3) + r(3)*dt(3,2)) /singd/t(3)/t(3)
      d(6,6) = (-dr(3,3)*t(3) + r(3)*dt(3,3)) /singd/t(3)/t(3)

c --- Fehler der direkten Gitterkonstanten:

      do 60 i=1,6
	 delta(i) = 0.
	 do 50 j=1,6
	    delta(i) = delta(i) + d(i,j)*d(i,j)*stdd(j)*stdd(j)
50       continue
	 delta(i) = sqrt(delta(i))
60    continue

c --- Zellvolumen vol und Standardabweichung dvol berechnen:

      vs = 1. + 2.*cosad*cosbd*cosgd 
     *        - cosad*cosad - cosbd*cosbd - cosgd*cosgd 
      vol = celld(1) * celld(2) * celld(3) * sqrt(vs)

      dvol = (delta(1)/celld(1))**2 + (delta(2)/celld(2))**2 +
     *       (delta(3)/celld(3))**2 + 
     *       (delta(4)/vs * sinad * (cosad - cosbd*cosgd))**2 +
     *       (delta(5)/vs * sinbd * (cosbd - cosad*cosgd))**2 +
     *       (delta(6)/vs * singd * (cosgd - cosad*cosbd))**2 
      dvol = vol * sqrt(dvol)

c --- Stdd. der Winkel in Grad umrechnen:

      delta(4) = delta(4) * 180./PI
      delta(5) = delta(5) * 180./PI
      delta(6) = delta(6) * 180./PI

      return
      end


c******************************************************************************
c***            Auslenkungsvektoren der einzelnen Atome einlesen            ***
c******************************************************************************

      subroutine dispgen(ke,atext,x0,celld,apos,npos,pa,tp,
     *                   cbas,ampl,numbsv,nbas,ifbas,IBV)

      parameter(max = 200, PI = 3.14159265359, EPSY = 0.00001)

      external absval
      logical filex
      integer numbsv(IBV), nbsv(max), npar(max), ifbas(IBV)
      character modfile*80, atext*4, char(max)*132, ecod*3
      character cbas(IBV)*10, modtit*70
      real pa(6,48,IBV), apos(3,48), tp(3,48), werte(max), absval
      real x0(3), celld(6), pn(6), ampl(2,IBV)

      ecod = 'eee'

c --- Name der Datei "atext".mod generieren:

      l = 0
      modfile = ' '
      do 20 i=1,4
         if(atext(i:i).ne.' ') then
           l = l + 1
           modfile(l:l) = atext(i:i)
         end if
 20   continue
      modfile(l+1:l+4) = '.mod'
      laenge = l + 4

c --- Testen, of  das File "atext".mod existiert:

      inquire(file=modfile,exist=filex)

      nbas = 0 
      if(filex) then

c --- Atomposition ausgeben:

        write(*,'(//2x,a,a,3x,a,3f8.4)') 
     *       'Atom: ',atext,'x =  ',(x0(i),i=1,3)

c --- Datei "atext".mod oeffnen:

        open(ke,file=modfile,status='old',err=1000)

c --- Titelzeile einlesen:

        ist = 0
        call getinp(ke,'TIT',ecod,werte,char,max,num,nn,ist)
        if(ist.eq.11) then
          write(*,110) modfile(1:laenge)
110       format(/2x,'Error in file ',a,': title is missing')
          stop
        else
          modtit = char(1)(1:70)
c          write(*,'(/2x,a,a)') 'Title: ',modtit
        end if

c --- Zeile mit den Nummern der Basisvektoren einlesen, die
c     beruecksichtigt werden sollen:

        ist = 1
        call getinp(ke,'BAS',ecod,werte,char,max,num,nn,ist)
        if(ist.eq.11) then
          write(*,120) modfile(1:laenge)
120       format(/2x,'Error in file ',a,': line "BAS> ..." is missing')
          stop
        else
          call numcode(char,werte,num,nbsv,nzahl,max)
          if(nzahl.gt.IBV) then
            write(*,130) modfile(1:laenge)
130         format(/2x,'Error in file ',a,
     *                 ': to many base vectors selected')
            stop
          end if
        end if

c --- Zeile mit den Nummern der Parameter der Basisvektoren, die
c     verfeinert werden sollen:

        ist = 1
        call getinp(ke,'PAR',ecod,werte,char,max,num,nn,ist)
        if(ist.eq.11) then
          write(*,140) modfile(1:laenge)
140       format(/2x,'Error in file ',a,': line "PAR> ..." is missing')
          stop
        else
          call numcode(char,werte,num,npar,npz,max)
          do 155 i=1,npz
             do 150 j=1,nzahl
                if(iabs(npar(i)).eq.nbsv(j)) goto 155
150          continue
             if(npar(i).gt.0) then
               write(*,151) modfile(1:laenge),iabs(npar(i))
151            format(/2x,'Error in file ',a,
     *       ': Amplitude of base vector ',i3,' could not be refined,',
     *         /2x,'base vector not selected.')
               stop
             else
               write(*,152) modfile(1:laenge),iabs(npar(i))
152            format(/2x,'Error in file ',a,
     *         ': Phase of base vector ',i3,' could not be refined,',
     *         /2x,'base vector not selected.')
               stop
             end if
155       continue
        end if


c --- Symmetrieaequivalente Positionen und 
c     Translationsanteil der Phasen einlesen:

        np = 0
        ist = 1
 160    call getinp(ke,'POS','MOD',werte,char,max,num,nn,ist)
        if(ist.eq.13) then
           npos = np
        else if(ist.eq.11) then
           write(*,'(/2x,3a)')
     *     'Error in file ',modfile(1:laenge),'. Data missing.'
           stop
        else
           np = np + 1 
           if(num.ne.7) then
              write(*,161) modfile(1:laenge), np
 161          format(/2x,'Error in file ',a,':',
     *        ' wrong number of parameters, it should be 7,',
     *        /24x,'line "POS> ...", position ',i3)
              stop
           else
              do 163 i=1,3
                 apos(i,np) = werte(i+1)
                 tp(i,np) = werte(i+4)
 163          continue
              goto 160
           end if
        end if

        write(*,'(/2x,2a)') 'Pos      x/a       y/b       z/c',
     *                      '       t_x       t_y       t_z'
        do 170 k=1,npos
           write(*,'(2x,i3,2x,6f10.5)') k,
     *          (apos(i,k),i=1,3),(tp(i,k),i=1,3)
 170    continue

c --- Auslenkungsvektoren incl. Phasen einlesen:

        if(nzahl.gt.0) then
 
          write(*,'(/2x,a,a)') 
     *    'Base vector  No.       u        phi',
     *    '     pos         p_abs          p_phase'

          nb = 0
200       ist = 1
          call getinp(ke,'MOD',ecod,werte,char,max,num,nn,ist)

          if(ist.ne.11) then

            nb = nb + 1
            if(num.ne.6) then
              write(*,210) modfile(1:laenge), nb
210           format(/2x,'Error in file ',a,':',
     *        ' wrong number of parameters, it should be 6,',
     *        /24x,'line "MOD> ...", base vector ',i3)
              stop 
            end if

c --- Nur Basisvektoren einlesen, die in der Liste "nbsv" stehen:

            do 300 k=1,nzahl
               if(nbsv(k).eq.nb) then
  
                 nbas = nbas + 1
                 cbas(nbas)(1:4) = char(1)(1:4)
                 cbas(nbas)(5:5) = ' '
                 cbas(nbas)(6:7) = char(2)(1:2)
                 write(cbas(nbas)(8:10),'(i3)') nint(werte(3))
                 numbsv(nbas) = nint(werte(4))
                 ampl(1,nbas) = werte(5)
                 ampl(2,nbas) = werte(6)

                 ist = 1
                 nvec = 0
220              call getinp(ke,'VEC','MOD',werte,char,max,num,nn,ist)

                 if(ist.ne.11.and.ist.ne.13) then

                   nvec = nvec + 1
                   if(num.ne.7) then
                     write(*,230) modfile(1:laenge),nvec,nbas
230                  format(/2x,'Error in file ',a,':',
     *               ' wrong number of parameters, it should be 7,',
     *               /21x,'line "VEC> ...", position ',i2,
     *               ', base vector ',i3)
                     stop
                   end if
                   na = nint(werte(1))
                   do 240 i=1,3
                      pn(i) = werte(i+1)
                      pn(3+i) = PI/180. * werte(4+i)
240                continue

c --- Betrag des Vektors pn bestimmen und 
c     auf die Laenge 1 Angstom normieren:

                   pp = absval(pn(1),celld,0)
                   do 245 i=1,3
                      if(pp.gt.0.) then
                         pn(i) = pn(i)/pp
                      else
                         pn(i) = 0.
                      end if
 245               continue

c --- Auslenkungsvektoren ausgeben:

                   if(nvec.eq.1) then
                     write(*,'(2x,a10,i6,1x,2f10.5,i6,2x,3f6.2,1x,3i4)') 
     *               cbas(nbas),numbsv(nbas),ampl(1,nbas),ampl(2,nbas),
     *               na,(pn(i),i=1,3),(nint(180./PI*pn(i)),i=4,6)
                   else
                     write(*,'(39x,i6,2x,3f6.2,1x,3i4)') na,
     *               (pn(i),i=1,3),(nint(180./PI*pn(i)),i=4,6)
                   end if

c -- Basisvektoren im Feld pa abspeichern:

                   do 280 i=1,6
                      pa(i,na,nbas) = pn(i)
 280               continue

                   goto 220

                 end if

               end if
 300        continue
            goto 200

          end if

        end if

c --- Testen, ob in der Liste "nbsv" eine unsinniger Eintrag steht:

        do 320 i=1,nzahl
           if(nbsv(i).lt.1.or.nbsv(i).gt.nb) then
             write(*,315) modfile(1:laenge),nbsv(i)
315          format(/2x,'Error in file ',a,
     *       ': base vector ',i3,' in line "BAS> ..." is unknown')
             stop
           end if
320     continue

c --- Datei schliessen:

        close(ke)

c --- Kodierung der Parameter abspeichern:

        do 500 i=1,npz
           do 450 m=1,nbas
              if(iabs(npar(i)).eq.numbsv(m)) then
                ifbas(m) = npar(i)
                goto 500
              end if
 450       continue
           write(*,460) modfile(1:laenge),iabs(npar(i))
 460       format(/2x,'Error in file ',a,
     *     ': base vector ',i3,' in line "PAR> ..." is unknown')
           stop
 500    continue

      end if

      return

c --- Fehlermeldungen:

 1000 write(*,'(a,a)') ' Error: Cannot open file ',modfile(1:laenge) 
      stop

      end


c******************************************************************************
c***        Betrag des Vektors v bezgl. der Gittermetrik a bestimmen        ***
c******************************************************************************

c --- ist = 0 : Koordinaten des Vektors v in Einheiten der Gitterkonstanten
c     ist = 1 : Koordinaten des Vektors v in Angstrom
c     a(6)    : Gitterkonstanten a, b, c, alpha, beta, gamma
c     v(3)    : Vektor, dessen Laenge zu bestimmen ist.

      real function absval(v,a,ist)

      real v(3), a(6), s(3)
      data bm /0.017453293/  ! Pi dividiert durch 180 Grad

      if(ist.eq.0) then
         s(1) = v(1) * a(1)
         s(2) = v(2) * a(2)
         s(3) = v(3) * a(3)
      else if(ist.eq.1) then
         s(1) = v(1)
         s(2) = v(2)
         s(3) = v(3)
      else
         write(*,'(/2x,a)') 
     *        'Error in function ABSVAL: unknown parameter.' 
         stop
      end if

      vv = s(1)**2 + s(2)**2 + s(3)**2 +
     *     2. * ( s(1) * s(2) * cos(bm * a(6)) +
     *            s(1) * s(3) * cos(bm * a(5)) +
     *            s(2) * s(3) * cos(bm * a(4)) )

      if(vv.gt.0.) then
         absval = sqrt(vv)
      else
         absval = 0.
      end if

      return
      end

c******************************************************************************
c***        Neue Datei mit den Modulationsparametern schreiben              ***
c******************************************************************************

      subroutine dispneu(ke,ka,atext,cbas,numbsv,ampl,nbas,IBV)

      parameter(max = 6, maxline = 132, EPSY = 0.00001)
      integer numbsv(IBV)
      real ampl(2,IBV)
      character line*132, atext*4, modfile*80, savefile*80
      character cbas(IBV)*10


c --- Name der Datei "atext".mod generieren:

      k = 0
      modfile = ' '
      do 10 i=1,4
         if(atext(i:i).ne.' ') then
           k = k + 1
           modfile(k:k) = atext(i:i)
         end if
 10   continue
      modfile(k+1:k+4) = '.mod'
      laenge = k + 4

c --- Alte Datei sichern:

      call savinp(ke,ka,modfile,savefile)

c --- Alte und neue Datei oeffnen:

      write(*,'(/2x,a,a)') 'Writing on file ',modfile(1:laenge)   
      open(ke,file=savefile,status='old',err=1000)
      open(ka,file=modfile,status='unknown',err=1100)

c --- Zeilen, die mit "MOD" beginnen, suchen, einlesen und 
c     neue Parameter hineinschreiben:

      nb = 0
 12   iwrt = 0
      read(ke,'(a)',end=500) line
      if(line(1:3).eq.'MOD') then
        nb = nb + 1
        do 20 m=1,nbas
           if(nb.eq.numbsv(m)) then
             write(ka,'(a,a10,i7,7x,2f10.5)') 
     *       'MOD> ',cbas(m),numbsv(m),ampl(1,m),ampl(2,m) 
 13          ibs = ibstes(line)
             if(ibs.eq.1) then
               line = ' '
               read(ke,'(a)') line
               goto 13
             end if
             iwrt = 1
             goto 25
           end if
 20     continue
      end if
 25   if(iwrt.eq.0) call writline(ka,line,maxline)
      goto 12

c --- Dateien schliessen:

 500  close(ke)
      close(ka)
      return

c --- Fehlermeldungen:

1000  write(*,'(/2x,a,a)') 'Cannot open file ',savefile(1:laenge)
      stop 
1100  write(*,'(/2x,a,a)') 'Cannot open file ',modfile(1:laenge)
      stop

      end

c******************************************************************************
c***                 Berechnen der Abstaende                                ***
c******************************************************************************

      subroutine dist(ka,dmax,atext,atom,aato,lato,natom,NATS,dp,
     *                MSZ,celld,intaus,icent,ism,tran,nsym)

      external absval

      parameter(PI = 3.141592654, EPSY = 0.00001, MAXD = 1000)

      character intaus*1, atext(NATS)*4
      integer ism(3,3,48), lato(14,NATS)
      real atom(14,NATS), aato(14,NATS), dp(MSZ), celld(6), tran(3,48)

      integer mlt(MAXD), min(3), max(3), iat(48)
      real r(3,48), d(MAXD), sd(MAXD), rd(3,MAXD)
      real x1(3), x2(3), xp(3), cellr(13)
      real cellp(6), b(3), g(3), gp(3), deriv(3), absval
      double precision sbb

c --- Cosinuse der Gitterwinkel:

      cosg  = cos(PI/180.*celld(6))
      cosb  = cos(PI/180.*celld(5))
      cosa  = cos(PI/180.*celld(4))

c --- Primitive Gitterkonstanten bestimmen:

      call primcell(intaus,celld,cellp)

c --- Reziproke primitive Gitterkonstanten bestimmen:

      call rezi(cellp,cellr)

c --- Schleife ueber alle Atome der asymmetrischen Einheit:

      izl = 0     
      itit = 0
      do 900 n=1,natom

c --- Atom n in Position 1:

         do 50 i=1,3
            x1(i) = atom(i,n)
 50      continue

c --- Laufindices fuer die Schleife ueber die Zellen, die von der
c     Kugel mit dem Radius dmax am Ort xp geschnitten werden:

         call transf(intaus,x1,xp,0)
         do 60 i=1,3
            rmin = xp(i) - dmax * cellr(i)
            rmax = xp(i) + dmax * cellr(i)
            min(i) = int(rmin)
            max(i) = int(rmax)
            if(rmin.lt.-EPSY) min(i) = min(i) - 1
            if(rmax.lt.-EPSY) max(i) = max(i) - 1
 60      continue

c --- Abstaende zu den Nachbaratomen berechnen:

         do 800 l=n,natom

c --- Alle zu atom(1,l) symmetrieaequivalenten Lagen r bestimmen:

            call atogen(atom(1,l),r,iat,num,intaus,icent,ism,
     *                  tran,nsym)

c --- Schleifen ueber die dem Atom 1 benachbarten prim. Elementarzellen:

            izahl = 0
            do 500 ix = max(1),min(1),-1
               gp(1) = ix
               do 400 iy = max(2),min(2),-1
                  gp(2) = iy
                  do 300 iz = max(3),min(3),-1
                     gp(3) = iz

c --- Gittervektor gp in Einheiten der Gitterkonstanten der primitiven Zelle
c     in den Gittervektor g in Einheiten der Gitterkonstanten der zentrierten 
c     Zelle transformieren:

                     call transf(intaus,g,gp,1)

                     do 200 k=1,num

c --- Ort x2 des Atoms 2 berechnen:

                        do 75 i=1,3
                           x2(i) = g(i) + r(i,k)
 75                     continue

c --- Abstandsvektor b berechnen:

                        do 80 i=1,3
                           b(i) = (x1(i) - x2(i)) * celld(i)
 80                     continue

c --- Abstand bb berechnen:

                        bb = absval(b,celld,1)

c --- Ableitungen des Abstands nach den Komponenten des Abstandvektors:
 
                        deriv(1) = b(1) + b(2) * cosg + b(3) * cosb 
                        deriv(2) = b(2) + b(1) * cosg + b(3) * cosa 
                        deriv(3) = b(3) + b(1) * cosb + b(2) * cosa 

c --- Ableitungen und Stdd. der Koordinaten von Atom 1:

                        sbb = 0.

                        do 85 i=1,3
                           km = lato(i,n)
                           if(km.ne.0) then
                             std = dp(km) * abs(aato(i,n)) * celld(i) 
                             sbb = sbb + (std * deriv(i))**2
                           end if
 85                     continue

c --- Ableitungen und Stdd. der Koordinaten von Atom 2:

                        do 87 i=1,3
                           do 86 j=1,3
                              km = lato(j,l)
                              if(km.ne.0) then
                                 std = dp(km) * abs(aato(j,l)) * 
     *                                 celld(j) * ism(iat(k),i,j)
                                 sbb = sbb + (std * deriv(i))**2
                              end if
 86                        continue
 87                     continue

c --- Testen, ob der Abstand bb im vorgegebenen Bereich liegt:

                        if(bb.le.dmax.and.bb.gt.EPSY) then
                          do 100 i=1,izahl
                             if(abs(d(i)-bb).lt.EPSY) then
                               mlt(i) = mlt(i) + 1
                               goto 200
                             end if
 100                      continue
                          izahl = izahl + 1
                          mlt(izahl) = 1
                          d(izahl)   = bb
                          sd(izahl)  = sqrt(sbb)/bb
                          do 105 i=1,3
                             rd(i,izahl) = x2(i)
 105                      continue
                        end if

 200                 continue
 300              continue
 400           continue
 500        continue

c --- Abstaende rausschreiben:

            izl = izl + izahl
            if(izahl.gt.0.and.itit.eq.0) then
              write(ka,'(//2x,a,f8.4)') 
     *        'Atomic distances:                  d_max = ',dmax
              write(ka,'(/2x,a,a)') 
     *             'atom 1    atom 2        d        sd',
     *             '       position of atom 2     mult'
              itit = 1
            end if
 649        format(3x,a4,a,a4,5x,f8.4,3x,f7.4,1x,3(1x,f7.4),2x,i4) 
            do 650 j=1,izahl
               write(ka,649) atext(n), '  -   ', atext(l), d(j), sd(j),
     *         (rd(i,j),i=1,3), mlt(j)
 650        continue

 800     continue
 900  continue

c --- Fehlermeldung, falls keine Abstaende d im Bereich d < dmax 
c     gefunden wurden:

      if(izl.eq.0) then
        write(ka,'(/2x,a,f8.4,a,/2x,a,/)')
     *  'WARNING: No distances smaller than ',dmax,' found.',
     *  'Please select a larger max. distance.'
      end if

c --- Primitive Gitterkonstanten ausgeben:

      if(intaus.ne.'P')
     *  write(*,'(/2x,a,6(/2x,a,f11.6))') 
     *  'Primitive cell parameters: ',
     *  'A     = ',cellp(1),
     *  'B     = ',cellp(2),
     *  'C     = ',cellp(3),
     *  'Alpha = ',cellp(4),
     *  'Beta  = ',cellp(5),
     *  'Gamma = ',cellp(6)

      return
      end


c******************************************************************************
c***     Primitive Gitterkonstanten einer zentrierten Zelle bestimmen       ***
c******************************************************************************

      subroutine primcell(intaus,celld,cellp)

      external absval
      parameter(PI = 3.141592654)
      character intaus*1
      real celld(6), cellp(6), xc(3,3), vp(3,3), absval
      data vp /1.,0.,0.,  0.,1.,0.,  0.,0.,1./


c --- Primitive Achsen:

      do 24 j=1,3
         call transf(intaus,xc(1,j),vp(1,j),1)
         cellp(j) = absval(xc(1,j),celld,0)
         do 23 l=1,3
            xc(l,j) = xc(l,j) * celld(l)
 23      continue
 24   continue

c --- Cosinuse der Gitterwinkel:

      cosg  = cos(PI/180.*celld(6))
      cosb  = cos(PI/180.*celld(5))
      cosa  = cos(PI/180.*celld(4))

c --- Primitive Winkel:

      cosgp = (xc(1,1)*xc(1,2) + xc(2,1)*xc(2,2) + xc(3,1)*xc(3,2) +
     *        xc(1,1)*xc(2,2)*cosg + xc(1,1)*xc(3,2)*cosb + 
     *        xc(2,1)*xc(3,2)*cosa)/cellp(1)/cellp(2)
      cosbp = (xc(1,1)*xc(1,3) + xc(2,1)*xc(2,3) + xc(3,1)*xc(3,3) +
     *        xc(1,1)*xc(2,3)*cosg + xc(1,1)*xc(3,3)*cosb + 
     *        xc(2,1)*xc(3,3)*cosa)/cellp(1)/cellp(3)
      cosap = (xc(1,2)*xc(1,3) + xc(2,2)*xc(2,3) + xc(3,2)*xc(3,3) +
     *        xc(1,2)*xc(2,3)*cosg + xc(1,2)*xc(3,3)*cosb + 
     *        xc(2,2)*xc(3,3)*cosa)/cellp(2)/cellp(3)
      cellp(4) = 180./PI * acos(cosap)
      cellp(5) = 180./PI * acos(cosbp)
      cellp(6) = 180./PI * acos(cosgp)

      return
      end


c******************************************************************************
c***          Symmetrieaequivalente Positionen bestimmen                    ***
c******************************************************************************

      subroutine atogen(x0,r,iat,num,intaus,icent,ism,tran,nsym)

c --- Verwendete Variablen und Felder:
c     x0:     Position in der asymetrischen Einheit
c     r:      Symmetrieaequivalente Positionen zu x0
c     iat:    Nummer des Symmetrieelementes, das die Position r erzeugte
c     num:    Anzahl der symmetrieaequivalenten Positionen
c     intaus: Symbol der integralen Ausloeschung: P, I, F, ...
c     icent:  Bei zentrosymmetrischen Strukturen kann bei nicht-modulierten
c             und nicht-magnetischen mit einem verkuerztem Satz von
c             Symmetrieelementen gearbeiten werden:
c             icent=1: vollstaendig; icent=2: verkuerzt
c             Zur Berechnung der symmetrieaequivalenten Positionen muessen
c             im Falle des verkuerzten Satzes die fehlenden Symmetrieelemente
c             erzeugt werden. 
c     ism:    Rotationsanteile der Symmetrieelemente
c     tran:   Translationsanteile der Symmetrieelemente
c     nsym:   Anzahl der Symmetrieelemente

c --- Deklaration der Felder und Variablen:

      parameter(EPSY = 0.00001)

      character intaus*1
      integer ism(3,3,48), iat(48)
      real x0(3), tran(3,48), r(3,48)

      real x(3), xp(3), xc(3)
      integer nc(2)
      data nc /1,-1/

c --- Schleife ueber die Symmetrieelemente:

      num = 0
      do 55 l=1,icent
         do 50 n=1,nsym

c --- Symmetrieelement auf x0 anwenden, 
c     x ist das transformierte Atom:

            do 41 i=1,3
               x(i) = tran(i,n)
               do 40 j=1,3
                  x(i) = x(i) + nc(l) * ism(i,j,n) * x0(j)
40             continue
41          continue

c --- Erzeugte Position in der prim. Basis darstellen:

            call transf(intaus,x,xp,0)

c --- Erzeugte Position in die prim. Zelle schieben:

            do 43 i=1,3
               nt = int(xp(i)+EPSY)
               if(xp(i).lt.-EPSY) nt = nt - 1
               xp(i) = xp(i) - nt
43          continue

c --- Erzeugte Positionen xp ruecktransformieren:

            call transf(intaus,xc,xp,1)

c --- Testen, ob erzeugte Position schon vorhanden ist:

            do 45 k=1,num
               if(abs(xc(1)-r(1,k)).lt.EPSY.and.
     *            abs(xc(2)-r(2,k)).lt.EPSY.and.
     *            abs(xc(3)-r(3,k)).lt.EPSY) goto 50
45          continue

c --- Erzeugte Position xc abspeichern:

            num = num + 1
            iat(num) = n
            do 46 i=1,3
               r(i,num)  = xc(i)
46          continue

50       continue
55    continue
      
      return
      end



c******************************************************************************
c***                 Berechnen der Abstaende                                ***
c******************************************************************************

      subroutine distm(ka,dmax,atext,atom,aato,lato,natom,NATS,dp,
     *                 MSZ,celld,q,intaus,icent,ism,tran,nsym,pa,
     *                 apos,tp,npos,ampl,aampl,lampl,nbas,IBV)

      external absval

      parameter(PI = 3.141592654, EPSY = 0.00001, MAXD = 1000)

      character intaus*1, atext(NATS)*4
      integer ism(3,3,48), lato(14,NATS), lampl(2,IBV,NATS)
      integer npos(NATS), nbas(NATS)
      real atom(14,NATS), aato(14,NATS), dp(MSZ), celld(6), q(3)
      real pa(6,48,IBV,NATS), tp(3,48,NATS), apos(3,48,NATS)
      real tran(3,48), ampl(2,IBV,NATS), aampl(2,IBV,NATS)

      integer mlt(MAXD), min(3), max(3), iat(48)
      real r(3,48), d(MAXD), sd(MAXD), rd(3,MAXD)
      real x1(3), x2(3), xp(3), cellr(13)
      real cellp(6), b(3), g(3), gp(3), deriv(3), absval
      double precision sbb, sbb1, sbb2

c --- Cosinuse der Gitterwinkel:

      cosg  = cos(PI/180.*celld(6))
      cosb  = cos(PI/180.*celld(5))
      cosa  = cos(PI/180.*celld(4))

c --- Primitive Gitterkonstanten bestimmen:

      call primcell(intaus,celld,cellp)

c --- Reziproke primitive Gitterkonstanten bestimmen:

      call rezi(cellp,cellr)

c --- Schleife ueber alle Atome der asymmetrischen Einheit:

      izl = 0     
      itit = 0
      do 900 n=1,natom

c --- Bei modulierten Gitterkomplexen ist das Atom in Position 1 
c     das Auf-Atom, bei nicht-modulierten Gitterkomplexen
c     Koordinaten aus dem Inputfile:

         if(npos(n).gt.0) then

c --- Ort des Atoms 1:

            phase = 0.
            do 30 i=1,3
               phase = phase + q(i) * tp(i,1,n)
 30         continue
            do 35 i=1,3
               u = 0.
               do 34 m=1,nbas(n)
                  arg = pa(3+i,1,m,n) + ampl(2,m,n) + phase
                  u = u + ampl(1,m,n) * pa(i,1,m,n) * cos(arg)
 34            continue
               x1(i) = apos(i,1,n) + u
 35         continue

c --- Ableitungen und Stdd. der Modulationsparameter von Atom 1:

            sbb1 = 0.
            do 90 m=1,nbas(n)
               yy = 0.
               zz = 0.
               do 89 i=1,3
                  arg = pa(3+i,1,m,n) + ampl(2,m,n) + phase
                  yy  = yy + pa(i,1,m,n) * cos(arg) * 
     *                 deriv(i)
                  zz  = zz - ampl(1,m,n) * celld(i) * 
     *                 pa(i,1,m,n) * sin(arg) * deriv(i)
 89            continue
               km = lampl(1,m,n)
               if(km.ne.0) then
                  std = dp(km) * abs(aampl(1,m,n)) * celld(i)  
                  sbb1 = sbb1 + (yy * std)**2
               end if
               km = lampl(2,m,n)
               if(km.ne.0) then
                  std = dp(km) * abs(aampl(2,m,n))
                  sbb1 = sbb1 + (zz * std)**2
               end if
 90         continue

         else

c --- Ort des Atoms 1:

            do 40 i=1,3
               x1(i) = atom(i,n)
 40         continue

c --- Ableitungen und Stdd. der Koordinaten von Atom 1:

            sbb1 = 0.
            do 85 i=1,3
               km = lato(i,n)
               if(km.ne.0) then
                  std = dp(km) * abs(aato(i,n)) * celld(i) 
                  sbb1 = sbb1 + (std * deriv(i))**2
               end if
 85         continue

         end if

c --- Laufindices fuer die Schleife ueber die Zellen, die von der
c     Kugel mit dem Radius dmax am Ort xp geschnitten werden:

         call transf(intaus,x1,xp,0)
         do 60 i=1,3
            rmin = xp(i) - dmax * cellr(i)
            rmax = xp(i) + dmax * cellr(i)
            min(i) = int(rmin)
            max(i) = int(rmax)
            if(rmin.lt.-EPSY) min(i) = min(i) - 1
            if(rmax.lt.-EPSY) max(i) = max(i) - 1
 60      continue

c --- Abstaende zu den Nachbaratomen berechnen:

         do 800 l=n,natom

c --- Bei nicht-modulierten Gitterkomplexen alle zu atom(1,l) 
c     symmetrieaequivalenten Lagen r bestimmen:

            if(npos(l).eq.0) then
               call atogen(atom(1,l),r,iat,num,intaus,icent,ism,
     *                     tran,nsym)
            else
               num = npos(l)
            end if

c --- Schleifen ueber die dem Atom 1 benachbarten prim. Elementarzellen:

            izahl = 0
            do 500 ix = max(1),min(1),-1
               gp(1) = ix
               do 400 iy = max(2),min(2),-1
                  gp(2) = iy
                  do 300 iz = max(3),min(3),-1
                     gp(3) = iz

c --- Gittervektor gp in Einheiten der Gitterkonstanten der primitiven Zelle
c     in den Gittervektor g in Einheiten der Gitterkonstanten der zentrierten 
c     Zelle transformieren:

                     call transf(intaus,g,gp,1)

                     do 200 k=1,num

                        if(npos(l).gt.0) then

c --- Phase, Amplitude und Ort des Atoms 2 bei modulierten Gitterkomplexen:

                           phase = 0.
                           do 65 i=1,3
                              phase = phase + q(i) * tp(i,k,l)
 65                        continue   
                           do 75 i=1,3
                              u = 0.
                              do 70 m=1,nbas(l)
                                 arg = pa(3+i,k,m,l) + 
     *                                ampl(2,m,l) + phase
                                 u = u + ampl(1,m,l) * 
     *                                pa(i,k,m,l) * cos(arg)
 70                           continue
                              x2(i) = g(i) + apos(i,k,l) + u
 75                        continue

c --- Ableitungen und Stdd. der Modulationsparameter von Atom 2:

                           sbb2 = 0.
                           do 95 m=1,nbas(l)
                              yy = 0.
                              zz = 0.
                              do 92 i=1,3
                                 arg = pa(3+i,k,m,l) + 
     *                                ampl(2,m,l) + phase
                                 yy  = yy + pa(i,k,m,l) * cos(arg) *
     *                                deriv(i)
                                 zz  = zz - ampl(1,m,l) * celld(i) * 
     *                                pa(i,k,m,l) * sin(arg) * deriv(i)
 92                           continue
                              km = lampl(1,m,l)
                              if(km.ne.0) then
                                 std = dp(km)*abs(aampl(1,m,l))*celld(i)
                                 sbb = sbb + (yy * std)**2
                              end if
                              km = lampl(2,m,l)
                              if(km.ne.0) then
                                 std = dp(km) * abs(aampl(2,m,l)) 
                                 sbb2 = sbb2 + (zz * std)**2
                              end if
 95                        continue

                        else

c --- Ort des Atoms 2 bei nicht-modulierten Gitterkomplexen:

                           do 76 i=1,3
                              x2(i) = g(i) + r(i,k)
 76                        continue

c --- Ableitungen und Stdd. der Koordinaten von Atom 2:

                           sbb2 = 0
                           do 87 i=1,3
                              do 86 j=1,3
                                 km = lato(j,l)
                                 if(km.ne.0) then
                                    std = dp(km) * abs(aato(j,l)) * 
     *                                   celld(j) * ism(iat(k),i,j)
                                    sbb2 = sbb2 + (std * deriv(i))**2
                                 end if
 86                           continue
 87                        continue

                        end if

c --- Abstandsvektor b berechnen:

                        do 80 i=1,3
                           b(i) = (x1(i) - x2(i)) * celld(i)
 80                     continue

c --- Abstand bb berechnen:

                        bb = absval(b,celld,1)

c --- Ableitungen des Abstands nach den Komponenten des Abstandvektors:
 
                        deriv(1) = b(1) + b(2) * cosg + b(3) * cosb 
                        deriv(2) = b(2) + b(1) * cosg + b(3) * cosa 
                        deriv(3) = b(3) + b(1) * cosb + b(2) * cosa 

c --- Testen, ob der Abstand bb im vorgegebenen Bereich liegt:

                        if(bb.le.dmax.and.bb.gt.EPSY) then
                          do 100 i=1,izahl
                             if(abs(d(i)-bb).lt.EPSY) then
                               mlt(i) = mlt(i) + 1
                               goto 200
                             end if
 100                      continue
                          izahl = izahl + 1
                          mlt(izahl) = 1
                          d(izahl)   = bb
                          sd(izahl)  = sqrt(sbb1 + sbb2)/bb
                          do 105 i=1,3
                             rd(i,izahl) = x2(i)
 105                      continue
                        end if

 200                 continue
 300              continue
 400           continue
 500        continue

c --- Abstaende rausschreiben:

            izl = izl + izahl
            if(izahl.gt.0.and.itit.eq.0) then
              write(ka,'(//2x,a,f8.4)') 
     *        'Atomic distances:                  d_max = ',dmax
              write(ka,'(/2x,a,a)') 
     *             'atom 1    atom 2        d        sd',
     *             '       position of atom 2     mult'
              itit = 1
            end if
 649        format(3x,a4,a,a4,5x,f8.4,3x,f7.4,1x,3(1x,f7.4),2x,i4)
            do 650 j=1,izahl
               write(ka,649) atext(n), '  -   ', atext(l), d(j), sd(j),
     *         (rd(i,j),i=1,3), mlt(j)
 650        continue

 800     continue
 900  continue

c --- Fehlermeldung, falls keine Abstaende d im Bereich d < dmax 
c     gefunden wurden:

      if(izl.eq.0) then
        write(ka,'(/2x,a,f8.4,a,/2x,a,/)')
     *  'WARNING: No distances smaller than ',dmax,' found.',
     *  'Please select a larger max. distance.'
      end if

c --- Primitive Gitterkonstanten ausgeben:

      if(intaus.ne.'P')
     *  write(*,'(/2x,a,6(/2x,a,f11.6))') 
     *  'Primitive cell parameters: ',
     *  'A     = ',cellp(1),
     *  'B     = ',cellp(2),
     *  'C     = ',cellp(3),
     *  'Alpha = ',cellp(4),
     *  'Beta  = ',cellp(5),
     *  'Gamma = ',cellp(6)

      return
      end
c*****************************************************************************
c***            Gewichtete Durbin-Watson-Statistik                          ***
c*****************************************************************************

      subroutine dwstat(d,q,sum1,sum2,obs,calc,gew,mit,izahl,
     *                  maxs,ibac,IDZ)

      integer mit(IDZ)
      real obs(IDZ), calc(IDZ), gew(IDZ), p(3)

c --- Significance level: p(1) => 0.1%, p(2) => 1%, p(3) => 5%

      data p / 3.0902, 2.32635, 1.64485 /
      level = 1

      num = 0
      sum1 = 0.
      sum2 = 0.
      if(ibac.eq.0) then
         do 100 i=2,izahl
            if(mit(i).ge.0.and.mit(i-1).ge.0) then
               dif = (obs(i) - calc(i)) / sqrt(gew(i)) - 
     *               (obs(i-1) - calc(i-1)) / sqrt(gew(i-1))
               sum1 = sum1 + dif * dif
            end if
 100     continue
         do 200 i=1,izahl
            if(mit(i).ge.0) then
               num = num + 1
               dif = (obs(i) - calc(i)) / sqrt(gew(i))
               sum2 = sum2 + dif * dif
            end if 
 200     continue
      else
         do 300 i=2,izahl
            if(mit(i).eq.1.and.mit(i-1).eq.1) then
               dif = (obs(i) - calc(i)) / sqrt(gew(i)) - 
     *               (obs(i-1) - calc(i-1)) / sqrt(gew(i-1))
               sum1 = sum1 + dif * dif
            end if
 300     continue
         do 400 i=1,izahl
            if(mit(i).eq.1) then
               num = num + 1
               dif = (obs(i) - calc(i)) / sqrt(gew(i))
               sum2 = sum2 + dif * dif
            end if 
 400     continue
      end if

      d = sum1/sum2

      q = 2. * ( (num-1)/(num-maxs) - p(level)/sqrt(real(num+2)) ) 

      return
      end

c******************************************************************************
c***                  Eichen der 2-Theta-Skala                              ***
c******************************************************************************

      subroutine eich(th_obs,th_cal,neich,IRS,excl,iexcl,IEZ,
     *                the,izahl,IDZ)

      real th_cal(IRS), th_obs(IRS), the(IDZ), excl(2,IEZ)

      
      if(neich.eq.0) then
      
        write(*,'(2x,a,/2x,a)') 
     *       'WARNING: Number of scale data equal zero',
     *       '         Scaling on standards not possible'

      else if(neich.eq.1) then

c --- Falls es nur eine Eichlinie gibt:
       
        diff = th_cal(1) - th_obs(1)
        do 80 l=1,iexcl
           do 70 m=1,2
              excl(m,l) = excl(m,l) + diff
 70        continue
 80     continue
        do 100 k=1,izahl
           the(k) = the(k) + diff
 100    continue

      else

c --- Eichung der 2-Theta-Skala the:

        ibeg = 1
        do 500 i=1,neich-1
           stg = (th_cal(i+1)-th_cal(i))/(th_obs(i+1)-th_obs(i))
           shift = th_cal(i+1) - stg*th_obs(i+1)
           do 300 k=ibeg,izahl
              if(the(k).gt.th_obs(i+1)) then
                ibeg = k
                goto 500
              end if
              the(k) = stg*the(k) + shift
300        continue
500     continue

        do 200 k=ibeg,izahl
           the(k) = stg*the(k) + shift
200     continue

c --- Eichung der excluded regions:

        if(iexcl.gt.0) then
          do 750 l=1,iexcl
             do 700 m=1,2
                if(excl(m,l).lt.th_obs(1)) then
                  nc = 2
                else if(excl(m,l).gt.th_obs(neich)) then
                  nc = neich
                else
                  do 600 i=2,neich
                     if(th_obs(i).gt.excl(m,l)) then
                       nc = i
                     end if
 600              continue
                end if
                stg = (th_cal(nc)-th_cal(nc-1))/
     *                (th_obs(nc)-th_obs(nc-1))
                shift = th_cal(nc) - stg*th_obs(nc)
                excl(m,l) = stg * excl(m,l) + shift
 700         continue
 750      continue
        end if

      end if

      return
      end


c******************************************************************************
c***                    Eichreflexe einlesen                                ***
c******************************************************************************

      subroutine eichinp(ke,eichfile,title,th_obs,th_cal,neich,IRS)

      real th_cal(IRS), th_obs(IRS)
      character eichfile*80, title*70


      open(ke,file=eichfile,status='old',err=1000)
      read(ke,'(a)',err=1200) title

      n = 0
10    read(ke,*,err=1200,end=100) cal,obs 
      n = n + 1
      th_cal(n) = cal
      th_obs(n) = obs 
      if(n.gt.IRS) goto 100
      goto 10

100   neich = n
      write(*,'(2x,a,i4)') 'Number of reflections = ',neich
      return

1000  stop 'Cannot open SCALE file'
1200  stop 'Error reading SCALE file'

      end


c******************************************************************************
c***                     h k l - G e n e r a t o r                          ***
c******************************************************************************

c  fis8 erzeugt eine Reflexliste (iref) und berechnet die
c  dazugehoerigen Theta-Werte (theta) aller Reflexe im Winkelbereich (xmin)
c  bis (xmax) fuer die Lauegruppe (laue) unter Beruecksichtigung der
c  integralen Ausloeschung (intaus), sowie der serialen und zonalen
c  Ausloeschungen der Raumgruppe. Satellitenreflexe werden bis zur
c  Ordnung (iord) erzeugt. Der Wellenvektor (q) in Richtung der 
c  Modulationswelle ist in Einheiten der rez. Gitterkonstanten anzugeben.
c  Der hkl-Generator in fis8 ist vollstaendig neu entwickelt worden und
c  erzeugt sowohl die Basis- und die Satelliten-/Ueberstruktur-Reflexe,
c  als auch magnetische Reflexe. 

      subroutine fis8(imo,intaus,laue,icent,ism,tran,nsym,igeo,iord,
     *                uu,vv,ww,q,celld,rlamda,cthm,ctha,xmin,xmax,
     *                theta,iref,ref,hwb,rlopo,nuq,nhkl,NORD,IRS,
     *                itemp,ltemp,rtemp)

      parameter (PI = 3.14159265359)
      external lorpol, fwhm      
      character intaus*1, clau(15)*8
      real q(3), rtemp(IRS), lorpol, fwhm
      real tran(3,48), celld(6), cellr(13), rlamda(2)
      real theta(IRS), ref(3,IRS), hwb(IRS), rlopo(IRS)
      integer iref(7,IRS), nuq(2*NORD+1,IRS)
      integer itemp(IRS), ltemp(IRS), ism(3,3,48)

c --- Eingabe-Daten:

c     IRS:         Maximale Zahl der Reflexe
c     NORD:        Max. Ordnung der Satelliten
c     intaus:      Integrale Ausloeschung (P, F, I, ...)
c     laue:        Nummer der Lauegruppe
c     icent:       Zentrosymmetrie (1 = nein, 2 = ja)
c     ism:         Rotationsmatrizen der Symmetrieelemente der Raumgruppe
c     tran:        Translationsvektoren  der Symmetrieelemente der Raumgruppe
c     nsym:        Zahl der Symmetrieelemente
c     iord:        maximale Ordnung der Satelliten
c     igeo:        Diffraktometer-Typ
c     q(3):        Wellenvektor
c     rlamda(2):   Wellenlaengen
c     xmin,xmax:   kleinster/groesster 2-Theta-Wert
c     celld(6):    direkte Gitterkonstanten
c     uu, vv, ww:  Halbwertsbreite-Parameter
c     cthm         Monochromatorwinkel
c     ctha         Analysatorwinkel

c --- Variablen und Parameter in FIS8:

c     icr:         Nummer des Kristallsystems
c     tmi,tma:     Theta_min, Theta_max 
c     nz,iz:       Parameter aus dem Modul EXTIN 
c     hmax,kmax,lmax: Maximale hkl-Werte
c     ref(3,IRS):  Ort der Reflexe im rez. Raum
c     cellr(13):   rez. Gitterkonstanten (a', b', c', cos(alpha'),
c                  cos(beta'), cos(gamma'), ...)
c     clau(15):    Namen der Lauegruppen
c     itemp,ltemp,temp: Hilfs-Felder in der Subroutine sort
c     lorpol:      Function, die den LoPo zur Geometrie igeo berechnet:

c --- Ausgabe-Daten:

c     nhkl:        Zahl der erzeugten Reflexe
c     iref(7,IRS): 1-3 -> Miller'sche Indizes
c                  4   -> Ordnung des Satelliten
c                  5   -> Numer der Wellenlaenge
c                  6   -> Multiplizitaet
c                  7   -> Nummer der Phase
c     theta(IRS):  Theta-Werte der Reflexe
c     hwb(IRS):    Halbwertsbreite
c     rlopo(IRS):  Lorentz-Polarisationsfaktor
c     nuq(2*NORD+1,IRS): Ordnung der Reflexe, die zum Reflex i beitragen


c --- Lauegruppen:

      data clau(1),clau(2),clau(3),clau(4),clau(5),clau(6),clau(7),
     *     clau(8),clau(9),clau(10),clau(11),clau(12),clau(13),
     *     clau(14),clau(15)
     * /'1b','2/m (C)','m m m','4/m','4/m m m','3b','3b m 1',
     *  '6/m','6/m m m','m 3b','m 3b m','2/m (B)','3b (R)','3b m (R)',
     *  '3b 1 m'/

c --- Lauegruppe ueberpruefen:

      if(laue.ge.1.and.laue.le.15) then
        write(*,'(/2x,a,a,a)') 'Laue symmetry ',clau(laue),
     *  ' will be used to generate indices.'
        if(imo.ge.1) write(*,'(2x,a,3f9.5)') 
     *    'Wave vector = ',(q(i),i=1,3)
      else
        write(*,'(/2x,a,i2/)') 'Unknown Laue group : ',laue
        stop
      end if

c --- Anzahl der Wellenlaengen bestimmen:

      lxn = 2
      if(rlamda(2).le.0.001.or.rlamda(2).eq.rlamda(1)) lxn = 1

c --- Faktor fuer h_max, k_max, l_max bestimmen:

      rai = 2. * sin(xmax * PI/360.) / rlamda(1)
      if(rlamda(2).gt.0.001.and.rlamda(2).lt.rlamda(1)) then
        rai = rai * rlamda(1) / rlamda(2)
      end if

c --- Winkelbereich:

      tmi = xmin
      tma = xmax
      if(rlamda(2).gt.rlamda(1)) then
        tmi = xmin * (rlamda(1) / rlamda(2))**2
      end if

c --- reziproke Gitterkonstanten berechnen:

      call rezi(celld,cellr)

c --- Integrale Ausloeschungsregel ausgeben:

      if(intaus.eq.'P') then
        write(*,'(2x,a)') 'No extinction'
      else if(intaus.eq.'I') then
        write(*,'(2x,a)') 'h, k and l only with h + k + l = 2n'
      else if(intaus.eq.'A') then
        write(*,'(2x,a)') 'h, k and l only with k + l = 2n'
      else if(intaus.eq.'B') then
        write(*,'(2x,a)') 'h, k and l only with h + l = 2n'
      else if(intaus.eq.'C') then
        write(*,'(2x,a)') 'h, k and l only with h + k = 2n'
      else if(intaus.eq.'F') then
        write(*,'(2x,a)') 
     *  'h, k and l only with h + k = 2n, h + l = 2n and k + l = 2n'
      else if(intaus.eq.'R') then
        write(*,'(2x,a)') 'h, k and l only with -h + k + l = 3n'
      end if

c --- Reflexe generieren:

      if(imo.eq.1) then
        call hklgen(intaus,laue,icent,ism,tran,nsym,iord,tmi,tma,rai,
     *              lxn,celld,cellr,rlamda,q,iref,ref,theta,nhkl,IRS)
      else if(imo.eq.2) then
        call maghkl(intaus,laue,iord,tmi,tma,rai,lxn,celld,cellr,
     *              rlamda,q,iref,ref,theta,nhkl,IRS)
      else
        call fisgen(intaus,laue,icent,ism,tran,nsym,tmi,tma,rai,lxn,
     *              celld,cellr,rlamda,iref,theta,nhkl,IRS)
      end if

c --- Zahl der Reflexe ausgeben:

      if(nhkl.gt.0) then
        write(*,'(/2x,a,i6)') 'Number of reflections : ',nhkl
      else
        write(*,'(/2x,a,f8.3,a,f8.3/)')
     *  'No reflections in the 2theta region ',xmin,' - ',xmax
        stop
      end if

c --- Reflexe nach Theta sortieren:

      call sort(theta,iref,itemp,ltemp,rtemp,nhkl,IRS)

c --- Voreinstellungen zur LoPo-Berechnung:
c     Guinier-Geometrie: 2-Theta-Monochromator = 27.2 (Germanium-(111))
c     Bragg-Brentano:    2-Theta-Monochromator = 28.06 (Silizium-(111))
c     Synchrotron:       d-Wert-Analysator     = 0.15373 (Germanium-(111))

      if(igeo.eq.1.or.igeo.eq.2.or.igeo.eq.6) then
        if(cthm.le.10.) cthm = 27.2
      else if(igeo.eq.3) then
        if(cthm.le.10.) cthm = 28.06
      else if(igeo.eq.4) then
        if(ctha.lt.0.01) ctha = 360./PI * asin(0.15373 * rlamda(1))
      end if

c --- Halbwertsbreiten, LoPo und Multiplizitaeten berechnen und
c     Satellitenbeitraege bestimmen:

      do 300 k=1,nhkl
         if(imo.eq.1) call satfis(iref(1,k),iord,q,ism,tran,nsym,
     *                            intaus,icent,nuq(1,k),NORD)
         rlopo(k) = lorpol(igeo,cthm,ctha,theta(k))
         hwb(k)   = fwhm(uu,vv,ww,theta(k))
300   continue


      return
      end


c******************************************************************************
c***       hkl-Generator fuer Ueberstrukturreflexe und Satelliten           ***
c******************************************************************************

      subroutine hklgen(intaus,laue,icent,ism,tran,nsym,iord,tmi,
     *                  tma,rai,lxn,celld,cellr,rlamda,q,iref,ref,
     *                  theta,nhkl,IRS)

      external loesch, extin2, hhcalc
      parameter (PI = 3.14159265359, EPSY = 0.00001)

      character intaus*1
      real tran(3,48), hs(3,48), h(3)
      real cellr(13), q(3), ref(3,IRS), rlamda(2), celld(6)
      real theta(IRS)
      integer isml(3,3,48), ism(3,3,48)
      integer iref(7,IRS), h_min, h_max, extin2


c --- Symmetrieelemente der Lauegruppe ermitteln:

      call symlaue(laue,isml,lsym)

c --- hkl's generieren:

      nhkl = 0
      do 500 is=1,-1,-2
         ia = 0
         if(is.lt.0) ia = -1
      do 400 in=ia,is*iord

         h_max = int(rai*celld(1) - in*q(1))
         k_max = int(rai*celld(2) - in*q(2)) 
         l_max = int(rai*celld(3) - in*q(3))
        
         h_min = int(-rai*celld(1) - in*q(1))
         k_min = int(-rai*celld(2) - in*q(2)) 
         l_min = int(-rai*celld(3) - in*q(3))

         do 300 ih=h_max,h_min,-1
            do 200 ik=k_max,k_min,-1
               do 100 il=l_max,l_min,-1
                  call hvec(ih,ik,il,in,q,h)
                  call multp(isml,lsym,h,hs,mlt)
                  do 95 k=1,nhkl
                     do 75 n=1,mlt
                        if(abs(hs(1,n)-ref(1,k)).lt.EPSY.and.
     *                     abs(hs(2,n)-ref(2,k)).lt.EPSY.and.
     *                     abs(hs(3,n)-ref(3,k)).lt.EPSY) goto 100
 75                  continue
 95               continue
                  if(extin2(ih,ik,il,intaus).eq.1) goto 100
                  if(loesch(ih,ik,il,icent,ism,tran,nsym).eq.1) goto 100
                  hh = hhcalc(hs(1,1),cellr)
                  do 99 iq=1,lxn
                     sqt = hh * rlamda(iq)*rlamda(iq)/4.
                     if(sqt.lt.1.) then
                       sinth  = sqrt(sqt)
                       th = asin(sinth)
                       thobs = th * 360./PI
                       if(thobs.ge.tmi.and.thobs.le.tma) then
                         nhkl = nhkl + 1
                         if(nhkl.gt.IRS) then
                           write(*,'(/2x,a,/2x,a)')
     *                     'Too many reflections!!!',
     *                     'Please, select a smaller theta region.'
                           stop
                         end if
                         iref(1,nhkl) = ih
                         iref(2,nhkl) = ik
                         iref(3,nhkl) = il
                         iref(4,nhkl) = in
                         iref(5,nhkl) = iq
                         iref(6,nhkl) = mlt
                         theta(nhkl)  = th
                         do 97 i=1,3
                            ref(i,nhkl) = hs(i,1)
 97                      continue
                       end if
                     end if
 99               continue
 100           continue
 200        continue
 300     continue
 400  continue
 500  continue

      return
      end


c******************************************************************************
c***              hkl-Generator fuer magnetische Reflexe                    ***
c******************************************************************************

      subroutine maghkl(intaus,laue,iord,tmi,tma,rai,lxn,celld,cellr,
     *                  rlamda,q,iref,ref,theta,nhkl,IRS)

      external extin2, hhcalc
      parameter (PI = 3.14159265359, EPSY = 0.00001)

      character intaus*1
      real cellr(13), q(3), ref(3,IRS), rlamda(2), celld(6)
      real hs(3,48), h(3), theta(IRS)
      integer isml(3,3,48)
      integer iref(7,IRS), h_min, h_max, extin2


c --- Symmetrieelemente der Lauegruppe ermitteln:

      call symlaue(laue,isml,lsym)

c --- hkl's generieren:

      nhkl = 0

      h_max = int(rai*celld(1) - q(1))
      k_max = int(rai*celld(2) - q(2)) 
      l_max = int(rai*celld(3) - q(3))
        
      h_min = int(-rai*celld(1) - q(1))
      k_min = int(-rai*celld(2) - q(2)) 
      l_min = int(-rai*celld(3) - q(3))

      do 300 ih=h_max,h_min,-1
         do 200 ik=k_max,k_min,-1
            do 100 il=l_max,l_min,-1
               call hvec(ih,ik,il,1,q,h)
               call multp(isml,lsym,h,hs,mlt)
               do 95 k=1,nhkl
                  do 75 n=1,mlt
                     if(abs(hs(1,n)-ref(1,k)).lt.EPSY.and.
     *                  abs(hs(2,n)-ref(2,k)).lt.EPSY.and.
     *                  abs(hs(3,n)-ref(3,k)).lt.EPSY) goto 100
 75               continue
 95            continue
               if(extin2(ih,ik,il,intaus).eq.1) goto 100
               hh = hhcalc(hs(1,1),cellr)
               do 99 iq=1,lxn
                  sqt = hh * rlamda(iq)*rlamda(iq)/4.
                  if(sqt.lt.1.) then
                    sinth  = sqrt(sqt)
                    th = asin(sinth)
                    thobs = th * 360./PI
                    if(thobs.ge.tmi.and.thobs.le.tma) then
                      nhkl = nhkl + 1
                      if(nhkl.gt.IRS) then
                        write(*,'(/2x,a,/2x,a)')
     *                  'Too many reflections!!!',
     *                  'Please, select a smaller theta region.'
                        stop
                      end if
                      iref(1,nhkl) = ih
                      iref(2,nhkl) = ik
                      iref(3,nhkl) = il
                      iref(4,nhkl) = 1
                      iref(5,nhkl) = iq
                      iref(6,nhkl) = mlt
                      theta(nhkl)  = th
                      do 97 i=1,3
                         ref(i,nhkl) = hs(i,1)
 97                   continue
                    end if
                  end if
 99            continue
 100        continue
 200     continue
 300  continue

      return
      end


c******************************************************************************
c***             Reziproken Gittervektor h berechnen                        ***
c******************************************************************************

      subroutine hvec(ih,ik,il,in,q,h)

      real q(3), h(3)

      h(1) = ih + in * q(1)
      h(2) = ik + in * q(2)
      h(3) = il + in * q(3)
 
      return
      end


c******************************************************************************
c***           Betrag des rez. Gittervektors h berechnen                    ***
c******************************************************************************

      real function hhcalc(h,cellr)

      real cellr(13), h(3)
 
      hh = h(1) * h(1) * cellr(7)  + h(2) * h(2) * cellr(8) + 
     *     h(3) * h(3) * cellr(9)  + h(1) * h(2) * cellr(12) + 
     *     h(1) * h(3) * cellr(11) + h(2) * h(3) * cellr(10)

      hhcalc = hh
      return
      end


c******************************************************************************
c***                Multiplizitaeten bestimmen                              ***
c******************************************************************************

      subroutine multp(ism,nsym,h,hs,mlt)

      parameter(EPSY = 0.00001)
      real h(3), z(3), hs(3,48)
      integer ism(3,3,48)

      mlt = 0
      do 100 n=1,nsym
         do 50 i=1,3
            z(i) = 0.
            do 40 j=1,3
               z(i) = z(i) + ism(j,i,n) * h(j)
 40         continue
 50      continue 
         do 60 k=1,mlt  
            if(abs(hs(1,k)-z(1)).lt.EPSY.and.
     *         abs(hs(2,k)-z(2)).lt.EPSY.and.
     *         abs(hs(3,k)-z(3)).lt.EPSY) goto 100
 60      continue
         mlt = mlt + 1
         do 70 i=1,3
            hs(i,k) = z(i)
 70      continue
 100  continue

      return
      end


c******************************************************************************
c***           Symmetrieelemente der Lauegruppe bestimmen                   ***
c******************************************************************************

      subroutine symlaue(laue,ism,nsym)

      character laugrp*60, intaus*1
      real tran(3,48)
      integer ism(3,3,48)

      if(laue.eq.1) then
        laugrp = 'P, 1B,,'
      else if(laue.eq.2) then
        laugrp = 'P,1,1,2/M'
      else if(laue.eq.3) then
        laugrp = 'P,2/M,2/M,2/M'
      else if(laue.eq.4) then
        laugrp = 'P,4/M,,'
      else if(laue.eq.5) then
        laugrp = 'P,4/M,2/M,2/M'
      else if(laue.eq.6) then
        laugrp = 'P,3B,,'
      else if(laue.eq.7) then
        laugrp = 'P,3B,2/M,1'
      else if(laue.eq.8) then
        laugrp = 'P,6/M,,'
      else if(laue.eq.9) then
        laugrp = 'P,6/M,2/M,2/M'
      else if(laue.eq.10) then
        laugrp = 'P,2/M,3B,'
      else if(laue.eq.11) then
        laugrp = 'P,4/M,3B,2/M'
      else if(laue.eq.12) then
        laugrp = 'P,1,2/M,1'
      else if(laue.eq.13) then
        laugrp = 'R,3B,,,R' 
      else if(laue.eq.14) then
        laugrp = 'R,3B,,,R'
      else if(laue.eq.15) then
        laugrp = 'P,3B,1,2/M'
      end if
      call symint(laugrp,intaus,nl,icent,ism,tran,ns)
      do 25 n=1,ns
         do 15 i=1,3
            do 10 j=1,3
               ism(j,i,ns+n) = -ism(j,i,n)
 10         continue
 15      continue
 25   continue
      nsym = 2 * ns

      return
      end


c******************************************************************************
c***          Sortieren der Reflexe nach steigendem Theta                   ***
c******************************************************************************

      subroutine sort(theta,iref,itemp,l,temp,nhkl,IRS)

      real theta(IRS), temp(IRS)
      integer iref(7,IRS), itemp(IRS), l(IRS)


      do 2 i=1,nhkl
         l(i) = i
2     continue
      m = nhkl - 1

      do 15 ll=1,IRS
         k = 0
         do 10 j=1,2
            do 10 i=j,m,2
               inp  = l(i)
               inp1 = l(i+1)
               r = theta(inp1) - theta(inp)   
               if(r.le.0) then
                 iz     = l(i)
                 l(i)   = l(i+1)
                 l(i+1) = iz
                 k = k + 1
               end if
10          continue
         if(k.eq.0) goto 20
15    continue

20    do 31 j=1,7
         do 29 i=1,nhkl
            itemp(i) = iref(j,i)
29       continue
         do 30 i=1,nhkl
            iref(j,i) = itemp(l(i))
30       continue
31    continue

      do 32 i=1,nhkl
         temp(i)  = theta(i)
32    continue
      do 33 i=1,nhkl
         theta(i) = temp(l(i))
33    continue
      

      return
      end


c******************************************************************************
c***                 Integrale Ausloeschung ermitteln                       ***
c******************************************************************************

      integer function extin2(ih,ik,il,intaus)

      parameter(PII = 6.28318530718, EPSY = 0.00001)
      integer nt(7)
      real tr(3,4,7)
      character*1 intaus

      data nt /1,2,2,2,2,4,3/
      data (tr(i,1,1),i=1,3)         / 0.0 , 0.0, 0.0 /
      data ((tr(i,n,2),i=1,3),n=1,2) / 0.0, 0.0, 0.0,  0.5, 0.5, 0.5 /
      data ((tr(i,n,3),i=1,3),n=1,2) / 0.0, 0.0, 0.0,  0.0, 0.5, 0.5 /
      data ((tr(i,n,4),i=1,3),n=1,2) / 0.0, 0.0, 0.0,  0.5, 0.0, 0.5 /
      data ((tr(i,n,5),i=1,3),n=1,2) / 0.0, 0.0, 0.0,  0.5, 0.5, 0.0 /
      data ((tr(i,n,6),i=1,3),n=1,4) / 0.0, 0.0, 0.0,  0.5, 0.5, 0.0,
     *                                 0.5, 0.0, 0.5,  0.0, 0.5, 0.5 /
      data ((tr(i,n,7),i=1,3),n=1,3) 
     *                           / 0.00000000, 0.00000000, 0.00000000,  
     *                             0.33333333, 0.66666667, 0.66666667,
     *                             0.66666667, 0.33333333, 0.33333333/


c --- Nummer der integralen Ausloeschung bestimmen: 

      if(intaus.eq.'P') then
        ma = 1
      else if(intaus.eq.'I') then
        ma = 2
      else if(intaus.eq.'A') then
        ma = 3
      else if(intaus.eq.'B') then
        ma = 4
      else if(intaus.eq.'C') then
        ma = 5
      else if(intaus.eq.'F') then
        ma = 6
      else if(intaus.eq.'R') then
        ma = 7
      end if

c --- Auf int. Ausloeschung testen:

      s1 = 0.
      s2 = 0.
      do 100 k=1,nt(ma)
         arg = ih * tr(1,k,ma) + ik * tr(2,k,ma) + il * tr(3,k,ma)
         arg = arg * PII
         s1 = s1 + cos(arg)
         s2 = s2 + sin(arg)
 100  continue

      iaus = 0
      if(abs(s1).lt.EPSY.and.abs(s2).lt.EPSY) iaus = 1
      extin2 = iaus

      return
      end


c******************************************************************************
c***         Beitraege der Satelliten zu einem Reflex finden                ***
c******************************************************************************

      subroutine satfis(iref,iord,q,ism,tran,nsym,intaus,icent,nuq,NORD)

      parameter(PI = 3.14159265359, EPSY = 0.00001)

      external loesch, extin2
      integer iref(7), h0(3), nuq(2*NORD+1), ism(3,3,48), extin2
      real q(3), h(3), tran(3,48)
      character intaus*1


      do 10 k=1,2*NORD+1
         nuq(k) = 999
 10   continue

      call hvec(iref(1),iref(2),iref(3),iref(4),q,h)

      j = 0 
      do 15 k=-iord,iord
         do 14 l=1,3
            hk = h(l) + k*q(l)
            h0(l) = nint(hk)
            if(abs(h0(l)-hk).gt.EPSY) goto 15
 14      continue
         if(extin2(h0(1),h0(2),h0(3),intaus).eq.1) goto 15
         if(loesch(h0(1),h0(2),h0(3),icent,ism,tran,nsym).eq.1) goto 15
         j = j + 1
         nuq(j) = -k
 15   continue

      return
      end
       

C******************************************************************************
C***                   Integrale Ausloeschungen                             ***
C******************************************************************************

      SUBROUTINE EXTIN(ICR,IH,IK,IL,KO,IRWC,NA,M1,M2,IZ,INTAUS)
       
C --- J. Ihringer, 31-MAI-77
C     J. Maichle, VAX-Version vom DEC-86
C     H. Ritter, ueberarbeitet fuer SIMREF 2.0 im JUL-1991

       
C --- Eingabeparameter sind:
C     ICR       :  Nr. des Kristallsystems
C     IH,IK,IL  :  Miller'sche Indizes des Reflexes (IH,IK,IL)
C     IRWC=2    :  Subr. schreibt Ausloeschungsregel aus
C     IRWC=3    :  Subr. prueft (IH,IK,IL) auf Ausloeschung
C
C --- Ausgabeparameter sind:
C     NA        :  Nummer der integralen Ausloeschungsregel
C     KO=0      :  Reflex ist ausgeloescht
C     KO=1      :  Reflex erscheint
C     Groessen, die von Aufruf zu Aufruf erhalten bleiben:
C     M1,M2     :  Charakterisieren die Ausloeschung:
C     M2=1      :  Keine Ausloeschung
C     M2=2      :  Nur integrale Ausloeschung.
C     IZ        :  Matrix der Koeffizienten der Ausloeschungsregel


      CHARACTER*1 INTAUS, AINT(9)
      DIMENSION IZ(5,6), LKH(3), ISS(6) 

      DATA AINT /'F','I','C','B','A','R','H','X','P'/

C --- Voreinstellung: Alle Reflexe erscheinen:

      KO = 1

C --- Voreinstellung der Parameter der Ausloeschungsregel:
C     K nummeriert die Regel, I die Koeffizienten A, B, C, D, F
C     von A*H + B*K + C*L = D*N + F


C === Ausloeschungsregel wird bestimmt und am Bildschirm ausgegeben:

      IF(IRWC.EQ.2) THEN

         DO 8 K=1,6
            DO 7 I=1,3
               IZ(I,K) = 1
               IZ(4,1) = 2
               IZ(4,K) = 1
               IZ(5,K) = 0
 7          continue
 8       continue
  
         M1 = 1
         M2 = 1
 10      FORMAT(1A1)

C --- Einlesen der integralen Ausloeschungsregel:

         NA = 0
         DO 310 I=1,8
            IF(AINT(I).EQ.INTAUS) NA = I
 310     CONTINUE
         IF(NA.ne.0) then
            M1 = 1
            IF(NA.eq.1) then
               IZ(3,1) = 0
               IZ(1,2) = 0
               IZ(4,2) = 2
            end if
            IF(NA.EQ.3) IZ(3,1) = 0
            IF(NA.EQ.4) IZ(2,1) = 0
            IF(NA.EQ.5) IZ(1,1) = 0
            IF(NA.EQ.6.OR.NA.EQ.7) IZ(4,1) = 3
            IF(NA.EQ.6) IZ(1,1) = -1
            M2 = 2
         end if

C --- Ausgabe der Ausloeschungsregel:

        IF(M2.EQ.1) THEN
	  write(*,205)
 205      format(2x,'No extinction',/)
        else
           write(*,'(/2x,a)') 'HKL only with'
           write(*,216)(iz(i,1),i=1,5)
 216       format(2x,i2,'*H+',i2,'*K+',i2,'*L=',i2,'*N+',i2)
           write(*,'(2x,a)') 'and'
           write(*,216) (iz(i,2),i=1,5)
        end if

C === Reflex (ih,ik,il) auf Ausloeschung ueberpruefen:

      else IF(IRWC.eq.3) then

         IF(M2.ne.1) then

            LKH(1) = IH
            LKH(2) = IK
            LKH(3) = IL
            KO  = 0
            IOK = 0
            IV  = 0

            DO 200 K=M1,M2

               IZS = 1
               IF(ICR.EQ.7.OR.ICR.EQ.5.) IZS = 3
               IF(ICR.EQ.4) IZS = 2
               IF(ICR.EQ.6.) IZS = 3

               MIZ = MOD(IZS,2)

               DO 190 I=1,IZS

                  IV = IV + 1

C --- Kubisches und tetragonales System:

                  if(icr.ne.6) then
                     KH1 = (1+MOD((I+2),3))*MIZ+(1-MIZ)*I
                     KH2 = (1+MOD(I,3))*MIZ+(1-MIZ)*(3-I)
                     KH3 = (1+MOD((I+1),3))*MIZ+(1-MIZ)*3
                  end if

C --- Hexagonales System:

                  if(icr.eq.6.and.i.ge.2) then

                     KH1 = 1
                     KH2 = 2
                     KH3 = 3

C --- erste Permutation: -(H+K),H,L

                     if(i.eq.2) then
                        KME1   = LKH(1)
                        KME2   = LKH(2)
                        LKH(1) = -(KME1+KME2)
                        LKH(2) = KME1
                     end if

C --- zweite Permutation:  K,-(H+K),L

                     if(i.eq.3) then
                        LKH(2) = LKH(1)
                        LKH(1) = KME2
                     end if

                  end if

C --- Ausloeschung, falls KO = 0 : 

                  ISS(K) = IZ(1,K)*LKH(KH1)+IZ(2,K)*LKH(KH2)+
     1                     IZ(3,K)*LKH(KH3)-IZ(5,K)
                  IDS    = MOD(ISS(K),IZ(4,K))
                  IF(IDS.eq.0) IOK = IOK + 1

 190           CONTINUE

 200        CONTINUE

            IF(IOK.EQ.IV) KO = 1
      
         end if

      end if

      RETURN
      END


c******************************************************************************
c***         Index-Generator fuer nicht-modulierte Strukturen               ***
c******************************************************************************

      subroutine fisgen(intaus,laue,icent,ism,tran,nsym,tmi,tma,rai,
     *                  lxn,celld,cellr,rlamda,iref,theta,nhkl,IRS)

      implicit none
      external loesch, hhcalc, multi, extin2

      integer IRS, lxn
      real tmi, tma, rai
      real rlamda(2), theta(IRS), celld(6), cellr(13), tran(3,48)
      integer laue, icent, nsym, nhkl, iref(7,IRS), ism(3,3,48)
      character intaus*1

      real hs(3), hhcalc, hh, pi, sinth2, th, thobs
      integer irep, iperm, iterm, jj, mm, nn, kmerk, ipermi
      integer kmin, kmax, lmin, lmax, ik, il, iq, multi, mo
      integer extin2, loesch, h, k, l, hmin, hmax, icr
      logical lnull, knull
      data pi /3.14159265359/


c --- Kristallklasse ermitteln:

      if(laue.eq.1.or.laue.eq.2) icr = 1
      if(laue.eq.3) icr = 3
      if(laue.eq.12) icr = 2
      if(laue.eq.4.or.laue.eq.5) icr = 4
      if(laue.ge.6.and.laue.le.9) icr = 6
      if(laue.eq.15) icr = 6
      if(laue.eq.10.or.laue.eq.11) icr = 7
      if(laue.eq.13.or.laue.eq.14) icr = 5

c --- Maxima fuer h, k und l bestimmen und Grundreflexe erzeugen:

      hmax = int(rai*celld(1)) 
      kmax = int(rai*celld(2)) 
      lmax = int(rai*celld(3)) 

c --- Parameterbestimmung fuer den Indexgenerator:

      nhkl  = 0
      ik    = 0
      il    = 0
      hmin  = 0
      knull = .true.
      lnull = .true.

      if(laue.eq.4.or.laue.eq.5.or.laue.eq.7.or.laue.eq.9.or.
     *   laue.eq.15) then
         ik = 1
      else if(laue.eq.6.or.laue.eq.8) then
         knull = .false.
      else if(laue.eq.10.or.laue.eq.11.or.laue.eq.13.or.laue.eq.14) then
         ik = 1
         il = 1
      end if

c --- Indexgenerator:

      do 4042 jj=hmin,hmax
	 kmin = ik * jj
	 do 4041 mm=kmin,kmax
	    if(.not.knull.and.mm.eq.0.and.jj.ne.0) goto 4041
	    lmin = il * mm
	    do 4040 nn=lmin,lmax
	       if(.not.lnull.and.nn.eq.0) goto 4040

	       irep  = 0
	       iperm = 0
	       iterm = 0
	       h = jj    
	       k = mm
	       l = nn

c --- Wahl der Reflexe einer Schicht:

 28            iperm = 0

c --- hkl-Auswahl gemaess der Ausloeschungsregeln:

 2828          if(extin2(h,k,l,intaus).eq.0.and.
     *            loesch(h,k,l,icent,ism,tran,nsym).eq.0) then

c --- Theta-Werte berechnen und aussortieren:

                  hs(1) = h
                  hs(2) = k
                  hs(3) = l
                  hh = hhcalc(hs,cellr)
                  do 3119 iq=1,lxn
                     sinth2 = hh * rlamda(iq)*rlamda(iq)/4.
                     if(sinth2.lt.1.) then
                        th = asin(sqrt(sinth2))
                        thobs = th * 360./pi
                        if(thobs.ge.tmi.and.thobs.le.tma) then
                           nhkl = nhkl + 1
                           if(nhkl.gt.IRS) then
                              write(*,'(/2x,a)') 
     *                             'Too many reflections!!!'
                              stop
                           end if
                           theta(nhkl)  = th
                           iref(1,nhkl) = h
                           iref(2,nhkl) = k
                           iref(3,nhkl) = l
                           iref(4,nhkl) = 0
                           iref(5,nhkl) = iq
                           iref(6,nhkl) = multi(h,k,l,laue)
                        end if
                     end if
 3119             continue
                  
               end if
       
c --- Vorzeichenwahl fuer h, k, und l:

c --- Triklin:

	       if(laue.eq.1) then
                  if(k.eq.0.and.l.eq.0) goto 4040
 1105             irep = irep + 1
                  if(irep.gt.3.or.(l.eq.0.and.irep.gt.1).or.
     *                 (h*k.eq.0.and.irep.gt.2)) goto 4040
                  mo = 2 - mod(irep,2)
                  if(mo.eq.1) then
                     if(h.eq.0) goto 1105
                     h = -h
                  else if(mo.eq.2) then
                     if(k.eq.0) goto 1105
                     k = -k
                  else
                     if(l.eq.0) goto 1105
                     l = -l
                  end if
                  goto 28
               end if

c --- Monoklin, c-Achse bevorzugt, (laue=2); b-Achse bevorzugt, (laue=12):

               if(laue.eq.2.or.laue.eq.12) then
                  if(irep.eq.1) goto 4040
                  if(laue.eq.2.and.h*k.eq.0) goto 4040
                  if(laue.eq.12.and.h*l.eq.0) goto 4040
                  irep = 1
                  h = -h
                  goto 2828
               end if

c --- Tetragonal, 4/m, (laue=4):

               if(laue.eq.4) then
                  if(irep.eq.1.or.(h.eq.k.or.h.eq.0)) goto 4040
                  irep  = 1
                  kmerk = k
                  k     = h
                  h     = kmerk
                  goto 2828
               end if

c --- Kubisch, m3, (laue=10):

               if(laue.eq.10) then
                  if(irep.ge.1.or.(h.eq.k.or.k.eq.l)) goto 4040
                  irep  = 1
                  kmerk = k
                  k     = l
                  l     = kmerk
                  goto 2828
               end if

c --- Rhomboedrisch, h<k<l, 3b m (laue=14);
c                    h<k,l, 3b   (laue=13) (+-l wird erzeugt):

               if(laue.eq.13.or.laue.eq.14) then

c --- Rueckpermutation fuer Lauegruppe 13:

                  if(ipermi.ne.0) then
                     kmerk = k
                     k = l
                     l = kmerk
                  end if

                  if(iterm.eq.1.and.laue.eq.13) goto 104
                  if(iterm.eq.2) ipermi = 0
                  if(iterm.eq.2) goto 4040
                  if(irep.ge.3) goto 4040
                  if(k.eq.0) goto 4040
                  if(k.eq.l) iperm = 1
                  if(h.eq.k) iperm = 2
                  goto 105

c --- Permutation fuer Lauegruppe 13:

 104              iterm = 2
 105              continue
                  if(ipermi.eq.1) goto 101
                  if (laue.eq.14) goto 101
                  if(h.eq.0.and.k.eq.iabs(l)) goto 101
                  if(h.eq.k.or.h.eq.l.or.k.eq.l) goto 101
                  kmerk = k
                  k = l
                  l = kmerk
                  ipermi = 1
                  goto 2828
 101              continue

c --- l wird negativ gesetzt:

                  if(irep.ne.1) then
                     if(irep.eq.2) goto 200
                     l = -l
                     irep = 1
                     ipermi = 0
                     goto 2828
                  end if

c --- k wird negativ gesetzt:

                  l = -l
                  ipermi = 0
                  if(h.eq.k.and.k.eq.l) goto 4040

c --- Naechste 2 Zeilen eingefuegt:

                  if(h.eq.0) goto 4040
                  if(iperm.eq.1) goto 201
                  if(iperm.eq.1.or.irep.eq.2) goto 200
                  k = -k
                  irep = 2
                  goto 2828

c --- h wird negativ gesetzt:

 200              k = -k
                  ipermi = 0
                  if(iperm.eq.2) goto 4040
 201              h = -h
                  iterm = 1
                  irep = 3
                  goto 2828

               end if

c --- Monoklin c- oder b-Achse bevorzugt, 
c     hkl's pos. generiert, laue=2, laue=12;
c     Hexagonal, 6 m m (laue=9), R 3b m (laue=7), h<k,l generiert;
c     Hexagonal, 6/m (laue=8), R 3b (laue=6) , hkl's generiert:

               if(icr.eq.2.or.icr.eq.6) then
                  if(irep.ge.3) goto 4040
                  irep = 3
                  if(l.eq.0) goto 4040
                  if(laue.eq.6.and..not.(h.eq.0.and.k.eq.0)) goto 1818
                  if(laue.eq.7.and.h.ne.k) goto 1818
                  if(laue.eq.15.and.h.ne.0.and.k.ne.0) goto 1818
                  goto 4040
 1818             l = -l
                  goto 2828
               end if

c --- h k -(h+k) permutation fuer die Reflexe einer Schicht im hex. Syst.:
       
               if(icr.eq.6.and.iperm.ne.2) then
                  iperm =  iperm + 1
                  kmerk =  h
                  h     = -(h + k)
                  k     =  kmerk
                  goto 2828
               end if

 4040       continue
 4041    continue
 4042 continue

      return
      end

c******************************************************************************
c***     Die Function multi bestimmt die Multiplizitaet der Reflexe         ***
c***     (h, k, l) fuer die durch "laue" spezifizierte Lauegruppe.          ***
c***     nach Cooper/Rouse, J. Appl. Cryst. (1977), 10, 134-135.            ***
c******************************************************************************

c     J. P. Ihringer, 26-Mai-77, 12-Jan-81
c     J. K. Maichle, 11-Mai-85 (VAX-Version)
c     H. Ritter, 29-Mai-96 

      integer function multi(h,k,l,laue)

      integer iav(14,14), h, k, l

      data iav/16*2,4,2,4,2,2*4,2,5*4,2*2,4,2,2*4,8,2*4,8,4,3*8,
     1         2,4,8,4,8,4,2*8,4,5*8,2,4,8,4,2*8,16,8,4,2*8,2*16,8,
     2         2,13*6,2,4*6,2*12,2*6,12,6,3*12,2,6,12,6,12,6,2*12,
     3         6,5*12,2,6,12,6,2*12,24,12,6,2*12,2*24,12,2*6,12,6,
     4         2*12,24,2*12,24,12,2*24,8,2*6,24,6,2*24,48,2*12,24,
     5         12,2*24,8,2*2,4,2*2,5*4,2,3*4,13*6,2,2*6,12,6,3*12,
     6         6*6,2/

      j = 100
      if((h*h+k*k).eq.0) j = 1
      if((h*h+l*l).eq.0) j = 2
      if(h.eq.0.and.k.ne.l.and.k*l.ne.0) j = 3
      if((k*k+l*l).eq.0) j = 4
      if(k.eq.0.and.h.ne.l.and.h*l.ne.0) j = 5
      if(l.eq.0.and.h.ne.k.and.h*k.ne.0) j = 6
      if(h.ne.k.and.k.ne.l.and.h.ne.l.and.(h*k*l).ne.0) j = 7
      if(h.eq.0.and.k.eq.l) j = 8
      if(l.eq.0.and.h.eq.k) j = 9
      if((h*k*l).ne.0.and.h.eq.k.and.k.ne.l) j = 10
      if(h.eq.l.and.k.eq.0) j = 11
      if((h*k*l).ne.0.and.k.eq.l.and.h.ne.k) j = 12
      if((h*k*l).ne.0.and.h.eq.l.and.k.ne.h) j = 13
      if(h.eq.k.and.k.eq.l) j = 14

      if(j.eq.100) then
        write(*,'(/2x,a)') 'Unknown Multiplicity!'
        stop
      end if

c --- Unterscheidung -3 m 1 (laue = 7) von -3 1 m (laue = 15):

      nl = laue
      if(laue.eq.15) then
        nl = 7
        jme = j
        if(j.eq.3.or.j.eq.5.or.j.eq.8) j = 10
        if(jme.eq.10.or.j.eq.14) j = 3
      end if

      multi = iav(j,nl)

c --- Test auf ( 0 h -h) bei laue = 14 (R3-quer (rhomboedrisch)):

      if(laue.eq.14) then
        if((j.eq.3.or.j.eq.5.or.j.eq.6).AND.
     *  (iabs(h).eq.iabs(k).or.iabs(k).eq.iabs(l).or.
     *   iabs(h).eq.iabs(l))) multi = 6
      end if

      return
      end


c******************************************************************************
c*             Berechnung des Lorentz-Polarisationsfaktors                    *
c******************************************************************************

      real function lorpol(igeo,thm,tha,theta)

      parameter (PI = 3.14159265359)


c --- Falls theta < 1 Grad, dann wird lorpol = 0 gesetzt:

      IF(180./PI*THETA.LE.1.0) THEN
	LORPOL = 0.
	RETURN
      END IF

C --- Guinier-Geometrie (Diffraktometer, Transmission):

      IF(IGEO.EQ.1) THEN
	SIGATE = SIN(2.*THETA + 45.*PI/180.)
	C2TMO  = COS(THM*PI/180.)**2
	LORPOL = (1.+C2TMO*COS(2.*THETA)**2.)*EXP(-.0633*SIGATE)/
     1           ((1.+C2TMO)*2.*SIN(THETA)*SIN(2.*THETA)*SIGATE*SIGATE)

C --- 2. Guinier-Geometrie (Film):

      ELSE IF(IGEO.EQ.2) THEN
	SIGATE = SIN(2.*THETA + 45.*PI/180.)
        C2TMO  = COS(THM*PI/180.)**2
	LORPOL = (1.+C2TMO*COS(2.*THETA)**2.)*EXP(-.0633*SIGATE)/
     1           ((1.+C2TMO)*2.*SIN(THETA)*SIN(2.*THETA)*SIGATE)

C --- 3. Bragg-Brentano-Geometrie:

      ELSE IF(IGEO.EQ.3) THEN
        C2TMO = COS(THM*PI/180.)**2
	LORPOL = (1.+C2TMO*COS(2.*THETA)**2.)/
     1           ((1.+C2TMO)*2.*SIN(THETA)*SIN(2.*THETA))

C --- 4. Diffraktometer am Synchrotron (Cox-Geometrie):

      ELSE IF(IGEO.EQ.4) THEN
        costha = abs(cos(PI/180.*tha)) 
	lorpol = ((1-thm) + thm * cos(2.*theta)**2 * costha)
     *           /(2. * sin(theta) * sin(2.*theta))

C --- 5. Neutronen-Pulverdiffraktometer mit Debye-Scherrer-Geometrie:

      ELSE IF(IGEO.EQ.5)  THEN
	LORPOL = 1./(2.*SIN(THETA)*SIN(2.*THETA))
      
C --- 6. Guinier-Geometrie (Diffraktometer, Reflexion):

      ELSE IF(IGEO.EQ.6) THEN
	SIGATE = SIN(PI - 2.*THETA + 45.*PI/180.)
        C2TMO = COS(THM*PI/180.)**2
        LORPOL = (1.+COS(2.*THETA)**2.*C2TMO)*EXP(-.0633*SIGATE)/
     1           ((1+C2TMO)*2.*SIN(THETA)*SIN(2.*THETA)*SIGATE*SIGATE)

      else
         
        lorpol = 1.

      END IF


      RETURN
      END

c******************************************************************************
c***               Zonale und seriale Ausloeschungen                        ***
c******************************************************************************

      integer function loesch(h,k,l,icent,ism,tran,nsym)

c     K. Burger, H. Ritter, 08-Jul-1997 
c
c     to each given reflection F_H all symmetry equivalents are calculated
c     using the known symmetry operators {R;T}. The new (h'k'l')
c     is got by use of the rotation matrix R alone: H' = HR.
c     The absolute values |F_HR| are constant since no anomalous dispersion
c     is considered. The phases of F_HR change if the symmetry operator has
c     a translatory component T not zero:
c     
c                 | R11  R12  R13 |
c       ( h k l ) | R21  R22  R23 |  = (h' k' l') = H R = (rotated hkl)
c                 | R31  R32  R33 |
c      
c       | F_HR |  =  | F_H |
c     
c       phi_HR = phi_H - 2 pi H*T ,  H=(h,k,l), T=(tx,ty,tz).
c     
c     see Giacovazzo, Fundamentals of Crystallography (1992) p. 156.
c     
c     ism(i,j,nsym) contains the rotation-matrix R, tran(i,nsym) the
c     corresponding translation T.
c
c     loesch = 0  -> Reflex nicht ausgeloescht
c     loesch = 1  -> Reflex ausgeloescht


      parameter(EPSY = 0.00001)
      integer ism(3,3,48), h, k, l
      real tran(3,48)

      ls  = 0
      do 100 n=1,nsym
         ix = h * ism(1,1,n) + k * ism(2,1,n) + l * ism(3,1,n)
         iy = h * ism(1,2,n) + k * ism(2,2,n) + l * ism(3,2,n)
         iz = h * ism(1,3,n) + k * ism(2,3,n) + l * ism(3,3,n)
         q  = h * tran(1,n) + k * tran(2,n) + l * tran(3,n)
         if(h.eq.ix.and.k.eq.iy.and.l.eq.iz.and.
     *      abs(q-nint(q)).gt.EPSY) then
           ls = 1
           goto 200
         end if
         if(icent.eq.2) then
           if(h.eq.-ix.and.k.eq.-iy.and.l.eq.-iz.and.
     *        abs(-q-nint(-q)).gt.EPSY) then
             ls = 1
             goto 200
           end if
         end if
 100  continue

 200  loesch = ls
      return
      end


c******************************************************************************
c***    Halbwertsbreite nach der Caglioti-Formel berechnen                  ***
c******************************************************************************

      real function fwhm(U,V,W,theta)

      implicit none
      real U, V, W, theta, tanth, hwb2, pi
      data pi /3.14159265359/

      tanth = sin(theta) / cos(theta)
      hwb2  = U * tanth**2 + V * tanth + W
      if(hwb2.le.0.) then
         write(*,'(/2x,a,f7.3,a)') 
     *        'ERROR: Fwhm at 2theta = ',360./pi * theta,
     *        ' degree lower or equal zero.'
         stop
      else
         fwhm = sqrt(hwb2)
      end if

      return
      end



c******************************************************************************
c***                      F(000) berechnen                                  ***
c******************************************************************************

      subroutine fnull(f000,ixn,natom,atom,ac,dfp,dfpp,ptr,rmult,
     *                 nsym,NATS)

      integer ptr(NATS)
      real rmult(NATS), atom(14,NATS)
      real ac(9,NATS), dfp(NATS), dfpp(NATS)
      complex f000

      freal = 0.
      fimag = 0.

      do 100 k=1,natom

c --- Formfaktor ff0 fuer das k-te Atom berechnen:

         if(ixn.eq.1) then
           ff0 = ac(9,ptr(k))
           do 35 j=1,4
              ff0 = ff0 + ac(2*j-1,ptr(k))
35         continue
         else
           ff0 = 0.
         end if

         bwst = atom(5,k) * rmult(k) * nsym   
         freal = freal + bwst * (ff0 + dfp(ptr(k)))
         fimag = fimag + bwst * dfpp(ptr(k))

100   continue

c --- Strukturfaktor F(000):

      f000 = cmplx(freal,fimag)


      return
      end


c******************************************************************************
c***          Eine Zeile einer Datei mit oder ohne Code einlesen            ***
c******************************************************************************

      subroutine getinp(ke,code,ecod,werte,char,max,num,nn,ist)

c     ke:      Kanalnummer
c     code:    Zeilencode, z. B. 'TIT'
c              code = '   ': Zeile ohne Code einlesen
c     ecod:    Code, bei dem das Einlesen abgebrochen wird
c     char:    Daten als Character-String abgespeichert
c     werte:   Daten als Realwerte abgespeichert
c     ist:     Eingabe: = 0 : Zeichenkette einlesen --> char
c                       = 1 : Daten einer Zeile als Realwerte (werte) und
c                             zusaetzlich als character (char) abspeichern 
c              Ausgabe: = 11 : Codewort nicht gefunden!
c                       = 13 : Ende beim Einlesen gefunden!
c                       = 15 : Zu viele Werte in der Eingabezeile (num > max)!
c     nn:      Zeilennummer
c     line:    Eingelesene Eingabe-Zeile
c              (einschliesslich aller Fortsetzungszeilen)
c     LMAX     Maximale Anzahl der Zeichen pro Zeile 
c     MAXLINE: Maximale Anzahl der Zeichen pro Eingabe-Zeile 
c              (einschliesslich der Fortsetzungszeilen)
c     max:     Maximale Anzahl der Werte pro Eingabe-Zeile
c     num:     Anzahl der gefundenen Werte pro Eingabe-Zeile


      parameter(MAXLINE = 500, LMAX = 250)
      real werte(max)
      character zeile*(LMAX), code*3, ecod*3, char(max)*132
      character line*(MAXLINE)


c -- Initialisierung:

      ibeg = 1

      do 5 k=1,max
         char(k)  = ' '
         werte(k) = 0.
5     continue

 10   k = 0
      line = ' '
      num = 0

c --- Zeile einlesen:

 11   zeile = ' '
      read(ke,'(a)',end=1100) zeile
c      write(*,'(a,a)') 'Z = ',zeile(1:70)
      nn = nn + 1

c --- Nach Fortsetzungszeilen suchen.

      do 20 i=1,LMAX
         if(zeile(i:i).eq.'\\') then
           if(i.eq.1) then
             goto 11
           else
             if(zeile(i-1:i-1).eq.' ') goto 11
           end if
         end if
         k = k + 1
         if(k.gt.MAXLINE) goto 30
         line(k:k) = zeile(i:i)
 20   continue

c --- Nach der Kennung suchen:

 30   if(code.ne.'   ') then

        if(ecod.ne.'eee'.and.line(1:3).eq.ecod) then
          backspace(ke)
          goto 1300
        end if
        if(line(1:3).ne.code(1:3)) goto 10

c --- ">" suchen:

        do 100 i=1,MAXLINE
           if(line(i:i).eq.'>') then
             ibeg = i + 1
             goto 110
           end if
100     continue
        stop 'ERROR: ">" is missing'

      end if

c --- Anfang suchen:

110   do 120 i=ibeg,MAXLINE
         if(line(i:i).ne.' ') then
           ibeg = i
           goto 130
         end if
120   continue    
      return 

c --- Daten der Zeile abspeichern:

130   if(ist.eq.0) then
        char(1)(1:132) = line(ibeg:ibeg+132)
      else
        ic    = 0
        n     = 0
        num   = 1
        ibl   = 0
        komma = 0
        do 250 i=ibeg,MAXLINE
           if(line(i:i).eq.' ') then
             ibl = ibl + 1
           else if(line(i:i).eq.',') then
             komma = 1
             num = num + 1
             n = 0
           else
             if(ibl.gt.0.and.komma.eq.0) then
               num = num + 1
               n = 0
             end if
             ic = ic + 1
             n = n + 1
             char(num)(n:n) = line(i:i)
             ibl = 0
             komma = 0
           end if
250     continue
        if(num.gt.max) goto 1500
        do 300 l=1,num
           read(char(l)(1:15),'(bn,f15.0)',err=300) werte(l)
300     continue
      end if

      return

c --- Fehlermeldungen und Warnungen zurueckgeben:
 
1100  ist = 11
      return
1300  ist = 13
      return
1500  ist = 15
      return

      end


c******************************************************************************
c***  Berechnung der Gewichte unter Beruecksichtigung des Winkelfehlers     ***
c******************************************************************************

      subroutine gewichte(IDZ,izahl,obs,step,wf,gew)

      implicit none
      integer IDZ, izahl, i
      real obs(IDZ), wf, step, wf2, steig, gew(IDZ), quoti, sig2
     
      wf2 = wf * wf
      quoti = step * 12.
      
c --- Berechnung der Gewichte:

      do 10 i=3,izahl-2
         steig = (obs(i-2) - 8.*obs(i-1) + 8.*obs(i+1) - obs(i+2))/quoti
         gew(i) = 1.
         sig2 = obs(i) + steig * steig * wf2 
         if(sig2.ne.0.) gew(i) = sig2
  10  continue
      gew(1) = 1.
      if(obs(1).ne.0.) gew(1) = obs(1)
      gew(2) = 1.      
      if(obs(2).ne.0.) gew(2) = obs(2)
      gew(izahl-1) = 1.
      if(obs(izahl-1).ne.0.) gew(izahl-1) = obs(izahl-1)
      gew(izahl) = 1.
      if(obs(izahl).ne.0.) gew(izahl) = obs(izahl) 

      return
      end

c******************************************************************************
c***          gemischte hkl-Liste aller Phasen ausgeben                     ***
c******************************************************************************

      subroutine hklout(ka,hklfile,imo,par,titel,iref,indx,
     *                  theta,hwb,rlopo,nhkl,IRS,LPHASE)

      parameter (PI = 3.14159265359)
      integer iref(7,IRS), indx(IRS)
      character hklfile*80, titel*70
      real theta(IRS), rlopo(IRS), hwb(IRS), par(23,LPHASE), hm(3)

     
      if(ka.ne.6) then
        open(ka,file=hklfile,status='unknown',err=1000)
        write(ka,'(a)') titel
      end if

      if(imo.eq.1) then

        write(ka,'(2x,a,a)')' No. Ph. Co.     2-Theta       Lopo',
     *'        Fwhm        h   k   l   n    Mult'
        do 100 i=1,nhkl
           ii = indx(i)
           th_obs = 360./PI * theta(ii)
           write(ka,'(2x,3i4,3f12.4,4x,4i4,i8)') 
     *     i, iref(7,ii), iref(5,ii), th_obs, rlopo(ii), hwb(ii), 
     *     iref(1,ii), iref(2,ii), iref(3,ii), iref(4,ii), iref(6,ii)
100     continue

      else if(imo.eq.2) then

        write(ka,'(2x,a,a)')' No. Ph. Co.     2-Theta       Lopo',
     *'        Fwhm         H     K     L     Mult'
        do 150 i=1,nhkl
           ii = indx(i)
           call hvec(iref(1,ii),iref(2,ii),iref(3,ii),iref(4,ii),
     *               par(11,iref(7,ii)),hm)
           th_obs = 360./PI * theta(ii)
           write(ka,'(2x,3i4,3f12.4,4x,3f6.1,i8)') 
     *     i, iref(7,ii), iref(5,ii), th_obs, rlopo(ii), hwb(ii),
     *     hm(1), hm(2), hm(3), iref(6,ii)
150     continue

      else

        write(ka,'(2x,a,a)')' No. Ph. Co.     2-Theta       Lopo',
     *'        Fwhm        h   k   l    Mult'
        do 200 i=1,nhkl
           ii = indx(i)
           th_obs = 360./PI * theta(ii)
           write(ka,'(2x,3i4,3f12.4,4x,3i4,i8)') 
     *     i, iref(7,ii), iref(5,ii), th_obs, rlopo(ii), hwb(ii), 
     *     iref(1,ii), iref(2,ii), iref(3,ii), iref(6,ii)
200     continue

      end if

      if(ka.ne.6) close(ka)
      return

1000  stop ' Cannot open hkl file'
      end


c******************************************************************************
c***                       hkl-Liste ausgeben                               ***
c******************************************************************************

      subroutine hklwrite(ka,imo,cellr,q,iref,theta,hwb,rlopo,
     *                    nhkl,IRS)

      parameter (PI = 3.14159265359)
      external hhcalc
      integer iref(7,IRS)
      real theta(IRS), rlopo(IRS), hwb(IRS), h(3), cellr(13), q(3)


      if(imo.eq.1) then

        write(ka,'(2x,a,a)') ' No. Co.   2-theta      d        Lopo',
     *'      Fwhm        h   k   l   n    Mult'
        do 100 i=1,nhkl
           call hvec(iref(1,i),iref(2,i),iref(3,i),iref(4,i),q,h)
           dwert = 0.
           hh = hhcalc(h,cellr)
           if(hh.gt.0.) dwert = 1./sqrt(hh)
           th_obs = 360./PI * theta(i)
           write(ka,'(2x,2i4,4f10.4,4x,4i4,i8)') 
     *     i,iref(5,i),th_obs,dwert,rlopo(i),hwb(i),
     *     iref(1,i),iref(2,i),iref(3,i),iref(4,i),iref(6,i)
100     continue

      else if(imo.eq.2) then

        write(ka,'(2x,a,a)') ' No.   2-theta      d        Lopo',
     *'      Fwhm       H     K     L     Mult'
        do 150 i=1,nhkl
           call hvec(iref(1,i),iref(2,i),iref(3,i),iref(4,i),q,h)
           dwert = 0.
           hh = hhcalc(h,cellr)
           if(hh.gt.0.) dwert = 1./sqrt(hh)
           th_obs = 360./PI * theta(i)
           write(ka,'(2x,i4,4f10.4,2x,3f6.1,i8)') 
     *     i,th_obs,dwert,rlopo(i),hwb(i),h(1),h(2),h(3),iref(6,i)
150     continue

      else

        write(ka,'(2x,a,a)') ' No. Co.   2-theta      d        Lopo',
     *'      Fwhm        h   k   l    Mult'
        do 200 i=1,nhkl
           call hvec(iref(1,i),iref(2,i),iref(3,i),iref(4,i),q,h)
           dwert = 0.
           hh = hhcalc(h,cellr)
           if(hh.gt.0.) dwert = 1./sqrt(hh)
           th_obs = 360./PI * theta(i)
           write(ka,'(2x,2i4,4f10.4,4x,3i4,i8)') 
     *     i,iref(5,i),th_obs,dwert,rlopo(i),hwb(i),
     *     iref(1,i),iref(2,i),iref(3,i),iref(6,i)
200     continue

      end if

    
      return
      end





c******************************************************************************
c***      Sortieren der 2-Theta-Werte nach steigenden Winkel                ***
c******************************************************************************

      subroutine indexx(IRS,theta,indx,nhkl)

      integer indx(IRS)
      real theta(IRS)


      do 11 j=1,nhkl
	 indx(j) = j
11    continue

      if(nhkl.gt.1) then

        l  = nhkl/2 + 1
        ir = nhkl

10      continue

          if(l.gt.1) then
 	    l = l - 1
	    indxt = indx(l)
	    q = theta(indxt)
	  else
	    indxt = indx(ir)
	    q = theta(indxt)
	    indx(ir) = indx(1)
	    ir = ir - 1
	    if(ir.eq.1) then
	      indx(1) = indxt
	      return
	    end if
	  end if

	  i = l
	  j = l + l

20        if(j.le.ir) then
	    if(j.lt.ir) then
	      if(theta(indx(j)).lt.theta(indx(j+1))) j = j + 1
	    end if
	    if(q.lt.theta(indx(j))) then
	      indx(i) = indx(j)
	      i = j
	      j = j + j
	    else
	      j = ir + 1
	    end if
	    goto 20
	  end if

	  indx(i) = indxt

        goto 10

      end if

      end


c******************************************************************************
c***                     INPUT-File einlesen                                ***
c******************************************************************************

      subroutine input(ke, inpfile, jobtyp, idges, ismat, icorr, 
     *                 istreu, newin, isp, ishelx, ibeta, mcycle, eps,
     *                 relax, dmax, parfile, ixn, igeo, nphase, nscatm,
     *                 nprof, ihkl, lhkl, ibac, ieich, istru, nref, 
     *                 ifou, ifis, titel, datfile, bacfile, eichfile, 
     *                 sb, wdt, cthm, ctha, tmv, rlim, rau, wf, thmin,
     *                 thmax, step, rlamda, ratio, excl, iexcl, phsnm,
     *                 spgrp, symfile, imo, lauq, iord, defint, intfile,
     *                 pref, ipref, gitter, atext, ntyp, nam, natom, 
     *                 ac, dfp, dfpp, rmult, glb, par, atom, 
     *                 LPHASE, NORD, NATS, IDAT, IEZ, MCL)

c --- Parameter:

c     glb(1,id)     : zeropoint t0
c     glb(2,id)     : peak-shift correction parameter t1
c     glb(3,id)     : peak-shift correction parameter t2
c     glb(4,id)     : background parameter c0
c     glb(5,id)     : background parameter c1
c     glb(6,id)     : background parameter c2
c     glb(7,id)     : background parameter c3
c     glb(8,id)     : background parameter c4
c     glb(9,id)     : background parameter c5
c     glb(10,id)    : unbelegt!
c     par( 1,ip,id) : peak shape function parameter e0
c     par( 2,ip,id) : peak shape function parameter e1
c     par( 3,ip,id) : peak shape function parameter e2
c     par( 4,ip,id) : overall scale factor s
c     par( 5,ip,id) : overall temperature factor B0
c     par( 6,ip,id) : preferred orientation correction parameter G1
c     par( 7,ip,id) : preferred orientation correction parameter G2
c     par( 8,ip,id) : Halfwidth parameter U
c     par( 9,ip,id) : Halfwidth parameter V
c     par(10,ip,id) : Halfwidth parameter W
c     par(11,ip,id) : the wave vector q1
c     par(12,ip,id) : the wave vector q2
c     par(13,ip,id) : the wave vector q3
c     par(14,ip,id) : lattice parameter a
c     par(15,ip,id) : lattice parameter b
c     par(16,ip,id) : lattice parameter c
c     par(17,ip,id) : lattice parameter alpha
c     par(18,ip,id) : lattice parameter beta
c     par(19,ip,id) : lattice parameter gamma
c     par(20,ip,id) : asymmetry correction parameter a0
c     par(21,ip,id) : asymmetry correction parameter b0
c     par(22,ip,id) : asymmetry correction parameter a1
c     par(23,ip,id) : asymmetry correction parameter b1

      parameter(maxline = 132, max = 23, PI = 3.14159265359,
     *          EPSY = 0.00001)

      real werte(max), celld(6), cellr(13)
      character char(max)*132, text*132, ecod*3, acod*3, profkt(9)*23

      integer ixn(IDAT), igeo(IDAT), nphase(IDAT), nscatm(IDAT),
     *        nprof(IDAT), ihkl(IDAT), lhkl(IDAT), ibac(IDAT), 
     *        ieich(IDAT), istru(IDAT), nref(IDAT), ifou(IDAT), 
     *        iexcl(IDAT), ifis(IDAT)
      integer imo(LPHASE), iord(LPHASE), ibeta(LPHASE), 
     *        natom(LPHASE), lauq(LPHASE)
      real relax(8), dmax(LPHASE), defint(LPHASE), excl(2,IEZ,IDAT), 
     *     rmult(NATS)
      real sb(IDAT), wdt(IDAT), cthm(IDAT), ctha(IDAT), tmv(IDAT), 
     *     rlim(IDAT), rau(IDAT), thmin(IDAT), thmax(IDAT),
     *     step(IDAT), rlamda(2,IDAT), ratio(2,IDAT), wf(IDAT) 
      real pref(3,LPHASE,IDAT), gitter(6,LPHASE,IDAT) 
      real ac(9,NATS,IDAT), dfp(NATS,IDAT), dfpp(NATS,IDAT)
      real glb(10,IDAT), par(23,LPHASE,IDAT), atom(14,NATS)

      character titel(IDAT)*70
      character spgrp(LPHASE)*60, phsnm(LPHASE)*4 
      character atext(NATS)*4, ntyp(NATS,IDAT)*4, nam(NATS,IDAT)*4
      character*80 inpfile, parfile, symfile(LPHASE), intfile(LPHASE),
     *             datfile(IDAT), bacfile(IDAT), eichfile(IDAT)

      data profkt /'Gaussian               ',
     *             'Lorentzian             ',
     *             'intermediate Lorentzian',
     *             'modified Lorentzian    ',
     *             'Edgeworth serie        ',
     *             'pseudo-Voigt           ',
     *             'Pearson VII            ',
     *             'double Gaussian        ',
     *             'asymmetric pseudo-Voigt'/

c --- Formate fuer Fehlermeldungen:

 11   format(/2x,'Error in INPUT file: Line "',a3,
     *  '> ..." is missing, data set ',i2)
 21   format(/2x,'Error in INPUT file: Line "',a3,
     *  '> ..." is missing, phase ',i2,', data set ',i2)
 31   format(/2x,'Error in INPUT file: Line "',a3,
     *  '> ..." is missing, phase ',i2)
 41   format(/2x,'Error in INPUT file: Line "',a3,'> ..." is missing')
 111  format(/2x,'Error in INPUT file: wrong number of parameters,',
     *' it should be ',i2,',',/23x,'line: "',a3,'>...", data set ',i2)
 1111 format(/2x,'Error in INPUT file: wrong number of parameters,',
     *' it should be ',i2,' or ',i2,',',
     */23x,'line: "',a3,'>...", data set ',i2)
 121  format(/2x,'Error in INPUT file: wrong number of parameters,',
     *' it should be ',i2,',',/23x,'line: "',a3,'> ...", phase ',i2,
     *', data set ',i2)
 131  format(/2x,'Error in INPUT file: wrong number of parameters,',
     *' it should be ',i2,',',/23x,'line "',a3,'> ..." , phase ',i2)
 141  format(/2x,'Error in INPUT file: wrong number of parameters,',
     *' it should be ',i2,',',/23x,'line "',a3,'> ..."')


c --- Initialisierung:

      nn = 0
      ecod = 'eee'

c --- Datei oeffnen:

      open(ke,file=inpfile,status='old',err=1000)


c --- Daten einlesen -----------------------------------------------

c --- Allgemeine Kodierung:

      ist = 1
      call getinp(ke,'INI',ecod,werte,char,max,num,nn,ist)
      if(ist.eq.11) then
        write(*,41) 'INI'
        stop
      else
        iz = 9
        if(num.ne.iz) then
          write(*,141) iz,'INI'
          stop
        end if
        jobtyp = nint(werte(1))
        idges  = nint(werte(2))
        ismat  = nint(werte(3))
        icorr  = nint(werte(4))
        istreu = nint(werte(5))
        newin  = nint(werte(6))
        isp    = nint(werte(7))  
        ishelx = nint(werte(8))
        ipref  = nint(werte(9))
      end if
      if(jobtyp.eq.1) then
        write(*,'(/2x,a/)')
     *  '====>  RIETVELD analysis for multiple data sets <===='
      else if(jobtyp.eq.2) then
        write(*,'(/2x,a/)') '===> Pattern calculation <==='
      else if(jobtyp.eq.3) then
        write(*,'(/2x,a/)') '===> Structure factors <==='
      else if(jobtyp.eq.4) then
        write(*,'(/2x,a/)') '===> hkl-list <==='
      else if(jobtyp.eq.5) then
        write(*,'(/2x,a/)') '===> Profile refinement <==='
      else
        write(*,'(/2x,a)')  'Undefined JOBTYP'
        stop
      end if

      if(jobtyp.eq.1.or.jobtyp.eq.5) then
        if(idges.le.IDAT) then
          write(*,'(2x,a,i4)') 'Number of data sets = ',idges
        else
          write(*,'(/2x,a,i3)') 
     *    'The number of data sets must be lower or equal ',IDAT
          stop
        end if
        if(icorr.eq.1) write(*,'(2x,a)') 'Output correlation matrix'
        if(newin.eq.1) write(*,'(2x,a)') 'Generate new input file'
        if(isp.eq.0) then
          write(*,'(2x,a)') 'Number of columns in CYCLE file = 80'
        else
          write(*,'(2x,a)') 'Number of columns in CYCLE file = 132'
        end if
      end if
      if(jobtyp.le.3) then
        if(istreu.eq.1) write(*,'(2x,a)') 
     *    'Output scattering lengths and coefficients'
      end if
      if(ismat.eq.1) write(*,'(2x,a)') 'Output symmetry cards'
      if(ishelx.eq.1) 
     *  write(*,'(2x,a)') 'Generate SHELX instruction file'
      if(ipref.eq.0) then
         write(*,'(2x,a)') 'Pref. orientation function: Rietveld (1969)'
      else 
         write(*,'(2x,a)') 'Pref. orientation function: Dollase (1986)'
      end if


c --- Zahl der Rechenzyklen, Epsylon, Relaxations-Parameter:

      if(jobtyp.eq.1.or.jobtyp.eq.5) then
        ist = 1 
        call getinp(ke,'FIT',ecod,werte,char,max,num,nn,ist)
        if(ist.eq.11) then
          write(*,41) 'FIT'
          stop
        else
          iz = 10
          if(num.ne.iz) then
            write(*,141) iz,'FIT'
            stop
          end if
          mcycle = nint(werte(1))
          if(mcycle.gt.MCL) then
             write(*,'(/2x,a,i4)') 
     *       'The number of cycles must be lower or equal to ',MCL
             stop
          end if
          eps    = werte(2)
          relax(1) = werte(3)   ! Atomparameter x, y, z, B, N
          relax(2) = werte(4)   ! Anisotrope Temp., Magn. Momente, mod. Par.
          relax(3) = werte(5)   ! Caglioti: U, V, W
          relax(4) = werte(6)   ! Skalierung, Ov. Temp., G1, G2, int. Intens.
          relax(5) = werte(7)   ! Gitterkonstanten, Wellenvektor
          relax(6) = werte(8)   ! Profilfunktion, Asymmetriekorrektur
          relax(7) = werte(9)   ! Peak-shift parameter t0, t1, t2, Lambda 1
          relax(8) = werte(10)  ! Untergrund-Parameter
        end if
        write(*,'(/2x,a,i4)') 'Number of cycles = ',mcycle
        write(*,'(2x,a,f6.1)') 'EPS-value = ',eps
        write(*,20) relax(1),relax(2),relax(3),relax(4),relax(5),
     *              relax(6),relax(7),relax(8)
 20     format(/2x,'Relaxation factors  - ',
     *  /2x,'for coordinates, iso.temp. factors, occupancies  : ',f5.2
     *  /2x,'for anisotropic temperature factors,               ',
     *  /2x,'    magnetic moments and modulation parameters   : ',f5.2,
     *  /2x,'for full-width-at-half-maximum parameters        : ',f5.2,
     *  /2x,'for scale factors, overall temperature factors,    ',
     *  /2x,'    preferred orientation correction parameters    ',
     *  /2x,'    and integrated intensities                   : ',f5.2,
     *  /2x,'for lattice constants and wave vector components : ',f5.2,
     *  /2x,'for peak shape function and asymmetry parameters : ',f5.2,
     *  /2x,'for zeropoint, peak shift corr. and wavelength   : ',f5.2,
     *  /2x,'for background function parameters               : ',f5.2)
      end if

c --- Parameter-Datei:

      if(jobtyp.eq.1.or.jobtyp.eq.5) then
        ist = 0
        call getinp(ke,'PAR',ecod,werte,char,max,num,nn,ist)
        if(ist.eq.11) then
          write(*,41) 'PAR'
          stop
        else
          parfile(1:80) = char(1)(1:80)
          write(*,*)
          text = ' '
          text = '  PARAMETER file: '//parfile
          call writline(6,text,maxline)
        end if
      end if

c -------------------------------------------------------------------

      do 200 id=1,idges

c --- Titel:

         ist = 0
         call getinp(ke,'TIT',ecod,werte,char,max,num,nn,ist)
         if(ist.eq.11) then
           write(*,11) 'TIT',id
           stop
         else
           titel(id)(1:70) = char(1)(1:70)
         end if

c --- Code:

         ist = 1
         call getinp(ke,'COD','TIT',werte,char,max,num,nn,ist)
         if(ist.eq.11.or.ist.eq.13) then
           write(*,11) 'COD',id
           stop
         else
           iz = 11
           if(num.ne.iz) then
             write(*,111) iz,'COD',id
             stop
           end if
           igeo(id)   = nint(werte(1))
           nphase(id) = nint(werte(2))
           nprof(id)  = nint(werte(3))
           ihkl(id)   = nint(werte(4))
           lhkl(id)   = nint(werte(5))
           ibac(id)   = nint(werte(6))
           ieich(id)  = nint(werte(7))
           istru(id)  = nint(werte(8))
           nref(id)   = nint(werte(9))
           ifou(id)   = nint(werte(10))
           ifis(id)   = nint(werte(11))
         end if

         if((igeo(id).ge.1.and.igeo(id).le.4).or.
     *       igeo(id).eq.6) then
           write(*,'(//2x,a,i2,a,/2x,a)')
     *     'Data set ',id,': X-ray data',
     *     '======================='
           ixn(id) = 1
         else if(igeo(id).eq.5) then
           write(*,'(//2x,a,i2,a,/2x,a)')
     *     'Data set ',id,': neutron data',
     *     '========================='
           ixn(id) = 2
         else
           write(*,'(/2x,a,i2)') 'Unknown data typ for data set ',id
           stop
         end if  

         write(*,'(/2x,a,a,/)') 'Title: ',titel(id)

         if(igeo(id).eq.1) then
           write(*,'(2x,a)') 'Guinier geometry (transmission)'
         else if(igeo(id).eq.2) then
           write(*,'(2x,a)') 'Guinier geometry (film)'
         else if(igeo(id).eq.3) then
           write(*,'(2x,a)') 'Bragg-Brentano for X-ray'
         else if(igeo(id).eq.4) then
           write(*,'(2x,a)') 'Bragg-Brentano for synchrotron'
         else if(igeo(id).eq.5) then
           write(*,'(2x,a)') 'Debye-Scherrer for neutron'
         else if(igeo(id).eq.6) then
           write(*,'(2x,a)') 'Guinier geometry (reflection)'
         else
           write(*,'(2x,a)') 'Undefined diffraction geometry!'
           stop
         end if

         if(nphase(id).gt.LPHASE) then
           write(*,'(/2x,a,i2)')
     *     'Number of phases must be < ',LPHASE + 1
           stop
         else
           write(*,'(2x,a,i2)') 'Number of phases =  ',nphase(id)
         end if
  
         if(jobtyp.le.2.or.jobtyp.eq.5) then
           if(nprof(id).gt.9.or.nprof(id).lt.1) then
             write(*,'(2x,a,i2)')
     *       'Unknown peak shape function no. ',nprof(id)
             stop
           else
             write(*,'(2x,a,a)')
     *       'Peak shape function: ',profkt(nprof(id))
           end if
         end if

         if(ihkl(id).ne.0.and.(jobtyp.le.3.or.jobtyp.eq.5))
     *     write(*,'(2x,a)') 'Print reflection list'
         if(lhkl(id).ne.0)
     *     write(*,'(2x,a)') 'Print merged reflection list'
         if(jobtyp.eq.1.or.jobtyp.eq.5) then
           if(ibac(id).eq.0) then
             write(*,'(2x,a)') 'Background fitting'
           else if(ibac(id).eq.1) then
             write(*,'(2x,a)') 
     *            'Background data from background data file'
           else if(ibac(id).eq.2) then
             write(*,'(2x,a)') 'Background data from data file'
           else
             write(*,'(2x,a)') 'Unknown background code'
             stop
           end if
         end if
         if(istru(id).ne.0.and.(jobtyp.eq.1.or.jobtyp.eq.5))
     *     write(*,'(2x,a)') 'Output structure factors'
         if(nref(id).ne.0.and.(jobtyp.le.2.or.jobtyp.eq.5)) 
     *     write(*,'(2x,a)') 'Generate file with reflection list'
         if(ifou(id).ne.0.and.jobtyp.eq.1) 
     *     write(*,'(2x,a)') 'Generate file with structure factors'
         if(jobtyp.eq.5.and.ifis(id).eq.1) 
     *     write(*,'(2x,a)') 'Generate reflections'

c --- Datei mit den gemessene Daten:

         if(jobtyp.eq.1.or.jobtyp.eq.5) then
           ist = 0
           call getinp(ke,'DAT','TIT',werte,char,max,num,nn,ist)
           if(ist.eq.11.or.ist.eq.13) then
             write(*,11) 'DAT',id
             stop
           else
             datfile(id)(1:80) = char(1)(1:80)
             write(*,*)
             text = ' '
             text = '  Data file: '//datfile(id)
             call writline(6,text,maxline)
           end if
         end if

c --- Untergrund und excluded regions:

         if(ibac(id).eq.0) then

c --- Excluded regions:

           if(jobtyp.eq.1.or.jobtyp.eq.5) then
             ix = 0
38           ist = 1
             call getinp(ke,'EXC','BAC',werte,char,max,num,nn,ist)
             if(ist.eq.11) then
               write(*,11) 'EXC',id
               stop
             else if(ist.eq.13) then
               if(ix.eq.0) write(*,'(2x,a)') 'No excluded regions'
               goto 39
             else
               iz = 2
               if(num.ne.iz) then
                 write(*,111) iz,'EXC',id
                 stop
               end if
               ix = ix + 1
               if(ix.le.IEZ) then
                 excl(1,ix,id) = werte(1)
                 excl(2,ix,id) = werte(2)
                 write(*,'(2x,a,2f10.4)') 
     *           'Excluded region = ',excl(1,ix,id),excl(2,ix,id)
               else
                 write(*,'(2x,a,i3,/2x,a,i3,a)')
     *           'Number of excluded regions > ',IEZ,
     *           'Only ',IEZ,' are allowed'
                 stop
               end if
             end if
             goto 38
39           iexcl(id) = ix
           end if 

c --- Untergrund-Parameter einlesen:

           ist = 1
           call getinp(ke,'BAC','TIT',werte,char,max,num,nn,ist)
           if(ist.eq.11.or.ist.eq.13) then
             write(*,11) 'BAC',id
             stop
           else
             iz = 6
             if(num.ne.iz) then
               write(*,111) iz,'BAC',id
               stop
             end if
             do 12 i=1,6 
                glb(3+i,id) = werte(i)
12           continue
           end if
           write(*,'(2x,a,6f8.2)')
     *          'Background parameters = ',(glb(3+i,id),i=1,6)

         else

c --- Datei mit den Untergrund-Stuetzpunkten:

           if(jobtyp.eq.1.or.jobtyp.eq.5) then
             ist = 0
             call getinp(ke,'BAC','TIT',werte,char,max,num,nn,ist)
             if(ist.eq.11.or.ist.eq.13) then
               write(*,11) 'BAC',id
               stop
             else
               bacfile(id)(1:80) = char(1)(1:80)
               text = ' '
               text = '  Background data file: '//bacfile(id)
               call writline(6,text,maxline)
             end if
           end if

         end if

c --- Datei mit den Eichdaten:

         if((jobtyp.eq.1.or.jobtyp.eq.5).and.ieich(id).eq.1) then
           ist = 0
           call getinp(ke,'SCL','TIT',werte,char,max,num,nn,ist)
           if(ist.eq.11.or.ist.eq.13) then
             write(*,11) 'SCL',id
             stop
           else
             eichfile(id)(1:80) = char(1)(1:80)
             text = ' '
             text = '  Scaling file: '//eichfile(id)
             call writline(6,text,maxline)
           end if
         end if

C --- Wellenlaengen:

         ist = 1
         call getinp(ke,'WVL','TIT',werte,char,max,num,nn,ist)
         if(ist.eq.11.or.ist.eq.13) then
           write(*,11) 'WVL',id
           stop
         else
            iz = 3
           if(num.ne.iz) then
             write(*,111) iz,'WVL',id
             stop
           end if
           rlamda(1,id) = werte(1)
           rlamda(2,id) = werte(2)
           glb(10,id)   = rlamda(1,id)
           ratio(1,id)  = 1.
           ratio(2,id)  = werte(3)
         end if
         if(rlamda(1,id).lt.0..or.rlamda(2,id).lt.0.) then
           write(*,'(/2x,a,i3,a,/2x,a)')
     *     'Error in INPUT file at line ',nn,':',
     *     'Wavelengths must be greater or equal zero'
           stop
         end if
         write(*,'(/2x,a,f10.6,a,f10.6,a,f10.6)')
     *   'The wavelengths are ',rlamda(1,id),' and ',rlamda(2,id),
     *   ' with a ratio of ',ratio(2,id)

c --- 2-Thetabereich, Schrittweite, 
c     Nullpunkt t0, Peak-Shift-Parameter t1 und t2:

         ist = 1
         call getinp(ke,'THE','TIT',werte,char,max,num,nn,ist)
         if(ist.eq.11.or.ist.eq.13) then
           write(*,11) 'THE',id
           stop
         else
           iz = 6
           if(num.ne.iz) then
             write(*,111) iz,'THE',id
             stop
           end if
           thmin(id) = werte(1)
           thmax(id) = werte(2)
           step(id)  = werte(3)
           glb(1,id) = werte(4)
           glb(2,id) = werte(5)
           glb(3,id) = werte(6)
         end if
         if(jobtyp.ge.2.and.jobtyp.le.4) then
           if(thmin(id).lt.0.) then
             write(*,'(/2x,a,i3,a,/2x,a)')
     *       'Error in INPUT file at line ',nn,':',
     *       '2-theta minimum must be greater or equal zero'
             stop
           end if
           if(thmax(id).le.thmin(id)) then
             write(*,'(/2x,a,i3,a,/2x,a)')
     *       'Error in INPUT file at line ',nn,':',
     *       '2-theta maximum must be greater than 2-theta minimum'
             stop
           end if
           if(step(id).le.0.) then
             write(*,'(/2x,a,i3,a,/2x,a)')
     *       'Error in INPUT file at line ',nn,':',
     *       'Stepwidth must be greater or equal zero'
             stop
           end if
           write(*,'(2x,a,2f10.5,2x,a,f10.5)')
     *     '2-theta region : ',thmin(id),thmax(id),
     *     ', with a stepwidth of ',step(id)  
         end if
         write(*,'(/2x,a,f10.5)') 'Zeropoint = ',glb(1,id)
         write(*,'(2x,a,1x,g10.3,2x,g10.3)')
     *        'Peak-shift parameters = ',glb(2,id),glb(3,id)

c --- Externe Streudaten:

         if(jobtyp.le.3) then

           n = 0
           ist = 1
5          call getinp(ke,'SCA','COR',werte,char,max,num,nn,ist)
           if(ist.eq.11.or.ist.eq.13) then
             nscatm(id) = n
             goto 7
           else
             if(ixn(id).eq.1) then
               iza = 3
               izb = 12
               if(num.ne.iza.and.num.ne.izb) then
                 write(*,1111) iza,izb,'SCA',id
                 stop
               else
                 n = n + 1
                 nam(n,id) = char(1)(1:4)
                 dfp(n,id) = werte(2)
                 dfpp(n,id) = werte(3)
                 if(num.eq.12) then
                   do 6 j=1,9
                      ac(j,n,id) = werte(3+j)
6                  continue
                 end if
               end if               
             else
               iza = 2
               izb = 11
               if(num.ne.iza.and.num.ne.izb) then
                 write(*,1111) iza,izb,'SCA',id
                 stop
               else
                 n = n + 1
                 nam(n,id) = char(1)(1:4)
                 dfp(n,id) = werte(2)
                 if(num.eq.11) then
                   do 61 j=1,8
                      ac(j,n,id) = werte(2+j)
 61                continue
                 end if
               end if
             end if
           end if
           goto 5

7          if(nscatm(id).gt.0) then
             write(*,*)
             if(ixn(id).eq.1) then
               do 8 n=1,nscatm(id)
                  write(*,'(2x,2a,2(a,f10.6))')      
     *            'Formfactors for ',nam(n,id),':   df'' = ',dfp(n,id),
     *            '  df'''' = ',dfpp(n,id)
                  if(num.eq.12) then
                    write(*,'(2x,a)') 'Coefficients:'
                    write(*,'(10x,3(a,f10.6),a)')
     *              'a1 = ',ac(1,n,id),',  b1 = ',ac(2,n,id),
     *              ',  a2 = ',ac(3,n,id),','
                    write(*,'(10x,3(a,f10.6),a)')
     *              'b2 = ',ac(4,n,id),',  a3 = ',ac(5,n,id),
     *              ',  b3 = ',ac(6,n,id),','
                    write(*,'(10x,3(a,f10.6))')
     *              'a4 = ',ac(7,n,id),',  b4 = ',ac(8,n,id),
     *              ',  c  = ',ac(9,n,id)
                  end if
8              continue 
             else if(ixn(id).eq.2) then
               do 9 n=1,nscatm(id)
                  write(*,'(2x,a)') 'Scattering lengths for ',
     *            nam(n,id),': ',dfp(n,id)
                  if(num.eq.12) then
                    write(*,'(10x,a)') 'Magnetic formfactor:'
                    write(*,'(10x,3(a,f10.6),a)')
     *              'A = ',ac(1,n,id),',  a = ',ac(2,n,id),
     *              ',  B = ',ac(3,n,id),','
                    write(*,'(10x,3(a,f10.6),a)')
     *              'b = ',ac(4,n,id),',  C = ',ac(5,n,id),
     *              ',  c = ',ac(6,n,id),','
                    write(*,'(10x,2(a,f10.6))')
     *              'D = ',ac(7,n,id),',  e = ',ac(8,n,id)
                  end if
9              continue
             end if
           end if

         end if 

c --- Korrektur-Parameter: Std.-Untergrund, Reflexweite, 
c     Monochromatorwinkel, Absorptionskorrektur, 
c     Grenzwinkel fuer Asymmetriekorrektur, Film-Rauschen:
 
         ist = 1
         call getinp(ke,'COR','TIT',werte,char,max,num,nn,ist)
         if(ist.eq.11.or.ist.eq.13) then
           write(*,11) 'COR',id
           stop
         else
           iz = 8
           if(num.ne.iz) then
             write(*,111) iz,'COR',id
             stop
           end if
           sb(id)   = werte(1) 
           wdt(id)  = werte(2) 
           cthm(id) = werte(3) 
           tmv(id)  = werte(4) 
           rlim(id) = werte(5) 
           rau(id)  = werte(6)
           ctha(id) = werte(7)
           wf(id)   = werte(8)
         end if
         write(*,*) 
         if(jobtyp.le.2.or.jobtyp.eq.5) write(*,626) wdt(id)
626      format(2x,'Base of peak = 2 * Fwhm *',f5.2)
         if(igeo(id).eq.4) then
           write(*,627) cthm(id)
627        format(2x,'Polarisation parameter K = ',f8.4)
           write(*,628) ctha(id)
628        format(2x,'Analysator angle = ',f8.4,' degrees')
         else
           write(*,629) cthm(id)
629        format(2x,'Monochromator angle = ',f8.4,' degrees')
         end if
         if(jobtyp.le.2.or.jobtyp.eq.5) write(*,630) tmv(id)
630      format(2x,'Absorption correction (mu*R) = ',f8.4)
         if(jobtyp.le.2.or.jobtyp.eq.5) write(*,631) rlim(id)
631      format(2x,'Asymmetry correction for angles less than ',f8.3,
     *   ' degrees')
         if(jobtyp.eq.1.or.jobtyp.eq.5) write(*,632) wf(id)
632      format(2x,'Standard deviation of angular positions = ',
     *          f8.5,' degrees')


         iof = 0
         do 100 ip=1,nphase(id)

c --- Integrale Ausloeschung, Lauegruppe, Zentrosymmetrie:

            if(id.eq.1) then
              ist = 1
              call getinp(ke,'PHS','TIT',werte,char,max,num,nn,ist)
              if(ist.eq.11.or.ist.eq.13) then
                write(*,31) 'PHS',ip
                stop
              else
                iz = 7
                if(num.ne.iz) then
                  write(*,131) iz,'PHS',ip
                  stop
                end if
                phsnm(ip)  = char(1)(1:4)
                imo(ip)    = nint(werte(2))
                lauq(ip)   = nint(werte(3))
                iord(ip)   = nint(werte(4))
                ibeta(ip)  = nint(werte(5))
                dmax(ip)   = werte(6)
                defint(ip) = werte(7)
              end if
            end if

            write(*,'(/2x,a,i2,a,a,/2x,a)') 
     *      'Phase ',ip,': ',phsnm(ip),'--------------'
            if(id.eq.1) then
              if(imo(ip).eq.1) then
                write(*,'(2x,a)') '==> Modulated structure'
                write(*,'(2x,a,i2)')
     *          'Laue group of the modulated structure = ',lauq(ip)
                if(iord(ip).gt.NORD.or.iord(ip).lt.0) then
                  write(*,'(/2x,a,i2)')
     *      'Max. order of the satellites must be between 0 and ',NORD
                  stop
                else
                  write(*,'(2x,a,i4)')
     *            'Max. order of the satellite peaks: ',iord(ip)
                end if
              else if(imo(ip).eq.2) then
                write(*,'(2x,a)') '==> Magnetic structure'
                write(*,'(2x,a,i2)')
     *          'Laue group of the magnetic structure = ',lauq(ip)
                if(iord(ip).gt.NORD.or.iord(ip).lt.0) then
                  write(*,'(/2x,a,a,i2)')
     *            'Max. order of the magnetic reflections must be ',
     *            'between 0 and ',NORD
                  stop
                else
                  write(*,'(2x,a,i4)')
     *            'Max. order of the magnetic reflections: ',iord(ip)
                end if
              end if
              if(jobtyp.le.3) then
                if(ibeta(ip).eq.0) then
                  write(*,'(2x,a)') 
     *                 'Anisotropic temperature factors: beta_ij'
                else if(ibeta(ip).eq.1) then
                  write(*,'(2x,a)') 
     *                 'Anisotropic temperature factors: B_ij'
                else if(ibeta(ip).eq.2) then
                  write(*,'(2x,a)') 
     *                 'Anisotropic temperature factors: u_ij'
                end if
                if(dmax(ip).gt.EPSY) then
                  write(*,'(2x,a,f8.4)') 
     *            'Calculate atomic distances lower than',dmax(ip)
                end if
              end if
              if(jobtyp.eq.5) then
                write(*,'(2x,a,f12.2)') 
     *          'Default Bragg intensity = ',defint(ip)
              end if 

c --- Raumgruppe:

              ist = 0
              call getinp(ke,'SPG','SYM',werte,char,max,num,nn,ist)
              if(ist.eq.11) then
                write(*,31) 'SPG',ip
                stop
              else if(ist.eq.13) then
                ist = 0
                if(jobtyp.eq.5) then
                  ecod = 'INT'
                else
                  ecod = 'MET'
                end if
                call getinp(ke,'SYM',ecod,werte,char,max,num,nn,ist) 
                if(ist.eq.11.or.ist.eq.13) then
                  write(*,31) 'SYM',ip
                  stop
                end if
                spgrp(ip) = ' ' 
                symfile(ip)(1:80) = char(1)(1:80)
                text = ' '
                text = '  Space group file: '//symfile(ip)
                call writline(6,text,maxline)
              else
                spgrp(ip)(1:60) = char(1)(1:60)
                text = ' '
                text = '  Space group: '//spgrp(ip)
                call writline(6,text,maxline)
              end if

            end if

            ecod = 'eee'

c --- Datei mit den Intensitaeten:

            if(jobtyp.eq.5) then
               ist = 0
               call getinp(ke,'INT',ecod,werte,char,max,num,nn,ist)
               if(ist.eq.11) then
                  write(*,31) 'INT',ip
                  stop
               end if
               intfile(ip)(1:80) = char(1)(1:80)
               text = ' '
               text = '  Intensity data file: '//intfile(ip)
               call writline(6,text,maxline)
            end if

c --- Gitterkonstanten:

            ist = 1
            call getinp(ke,'MET','TIT',werte,char,max,num,nn,ist)
            if(ist.eq.11.or.ist.eq.13) then
               write(*,21) 'MET',ip,id
               stop
            end if
            iz = 6
            if(num.ne.iz) then
               write(*,121) iz,'MET',ip,id
               stop
            end if
            do 70 k=1,6
               celld(k) = werte(k)
 70         continue
            write(*,'(2x,a,6f9.5)') 
     *           'Direct cell param. = ',(celld(k),k=1,6)
            call rezi(celld,cellr)
            do 80 k=1,6
               gitter(k,ip,id) = celld(k)
               par(13+k,ip,id) = cellr(k)
 80         continue

c --- Wellenvektor:

            if(imo(ip).eq.1) then
              ist = 1
              call getinp(ke,'WAV','TIT',werte,char,max,num,nn,ist)
              if(ist.eq.11.or.ist.eq.13) then
                write(*,21) 'WAV',ip,id
                stop
              end if
              iz = 3
              if(num.ne.iz) then
                 write(*,121) iz,'WAV',ip,id
                 stop
              end if
              do 60 k=1,3
                 par(10+k,ip,id) = werte(k)
 60           continue
              write(*,'(2x,a,3f9.5)')
     *             'Wave vector = ',(par(k,ip,id),k=11,13)
            end if

c --- Skalierung s, allgemeiner Temperaturfaktor B0 und 
c     Halbwertsbreite-Parameter nach Caglioti et al.:

            ist = 1
            call getinp(ke,'PRO','TIT',werte,char,max,num,nn,ist)
            if(ist.eq.11.or.ist.eq.13) then
               write(*,21) 'PRO',ip,id
               stop
            end if
            iz = 5
            if(num.ne.iz) then
               write(*,121) iz,'PRO',ip,id
               stop
            end if
            par( 4,ip,id) = werte(1)
            par( 5,ip,id) = werte(2)
            par( 8,ip,id) = werte(3) 
            par( 9,ip,id) = werte(4)
            par(10,ip,id) = werte(5)
            write(*,'(2x,a,g8.2)') 
     *           'Scale factor = ',par(4,ip,id)
            write(*,'(2x,a,f12.5)') 
     *           'Overall-Temp. factor = ',par(5,ip,id)
            write(*,'(2x,a,3f12.5)') 
     *           'Halfwidth Parameters (Caglioti) = ',
     *           (par(k,ip,id),k=8,10)
            
c --- Parameter der Profilfunktionen:

            if(jobtyp.le.2.or.jobtyp.eq.5) then
               ist = 1
               call getinp(ke,'ETA','TIT',werte,char,max,num,nn,ist)
               if(ist.eq.11.or.ist.eq.13) then
                  write(*,21) 'ETA',ip,id
                  stop
               end if
               iz = 3
               if(num.ne.iz) then
                  write(*,121) iz,'ETA',ip,id
                  stop
               end if
               do 50 k=1,3
                  par(k,ip,id) = werte(k)
 50            continue
               if(nprof(id).eq.5) then 
                  write(*,6461) par(1,ip,id),par(2,ip,id)
 6461             format(2x,'Skewness = ',f8.3,/2x,'Excess = ',f8.3)
               else if(nprof(id).eq.6.or.nprof(id).eq.9) then
                  write(*,6462) (par(j,ip,id),j=1,3)
 6462             format(2x,'Eta (pseudo-Voigt) = ',3f8.3)
               else if(nprof(id).eq.7) then
                  write(*,6463) (par(j,ip,id),j=1,3)
 6463             format(2x,'Pearson-M = ',3f8.3)
               end if
            end if

c --- Asymmetry correction parameters:

            if(jobtyp.le.2.or.jobtyp.eq.5) then
               ist = 1
               call getinp(ke,'ASY','TIT',werte,char,max,num,nn,ist)
               if(ist.eq.11.or.ist.eq.13) then
                  write(*,21) 'ASY',ip,id
                  stop
               end if
               iz = 4
               if(num.ne.iz) then
                  write(*,121) iz,'ASY',ip,id
                  stop
               end if
               do 51 k=20,23
                  par(k,ip,id) = werte(k-19)
 51            continue
               write(*,'(2x,a,4f10.5)') 
     *              'Asymmetry correction parameters = ',
     *              (par(j,ip,id),j=20,23)
            end if

c --- Preferred orientation parameter and vector:

            if(jobtyp.le.2) then
               ist = 1
               call getinp(ke,'PRE','TIT',werte,char,max,num,nn,ist)
               if(ist.eq.11.or.ist.eq.13) then
                  write(*,21) 'PRE',ip,id
                  stop
               end if
               iz = 5
               if(num.ne.iz) then
                  write(*,121) iz,'PRE',ip,id
                  stop
               end if
               par(6,ip,id) = werte(1)
               par(7,ip,id) = werte(2)
               do 30 k=1,3
                  pref(k,ip,id) = werte(2+k)
 30            continue
               write(*,'(2x,a,2f8.5)') 
     *              'Pref. orientation parameters = ',
     *              par(6,ip,id),par(7,ip,id)
              write(*,'(2x,a,3f8.4)') 
     *              'Preferred orientation vector = ',
     *              (pref(k,ip,id),k=1,3)
           end if

c --- Atom-Koordinaten x/a, y/b z/c, isotroper Temperaturfaktor B und 
c     Besetzungszahl N einlesen:

 109       if(jobtyp.le.3) then

             if(ip.gt.1) iof = iof + natom(ip-1)

             if(id.eq.1) then

                n = 0
                ist = 1
                if(ip.lt.nphase(id)) then
                  acod = 'PHS'
                else
                  acod = 'TIT'
                end if
220             call getinp(ke,'ATO',acod,werte,char,max,num,nn,ist)

                if(ist.eq.11.or.ist.eq.13) then
                  if(n.eq.0) then
                    write(*,31) 'ATO',ip
                    stop
                  else
                    natom(ip) = n
                  end if
                  goto 299
                else
                  n = n + 1
                  if(imo(ip).eq.2) then
                    iz = 17
                  else
                    iz = 14
                  end if
                  if(num.ne.iz) then
                    write(*,131) iz,'ATO',ip
                    stop
                  end if
                  atext(iof+n)(1:4) = char(1)(1:4)
                  ntyp(iof+n,id)(1:4) = char(2)(1:4)
                  rmult(iof+n) = werte(3)
                  do 240 j=1,num-3
                     atom(j,iof+n) = werte(3+j)
240               continue
                end if

                goto 220

c --- Atomparameter rausschreiben:

299             ibij = 0
                do 710 j=6,11
                   do 709 k=1,natom(ip)
                      if(atom(j,k+iof).ne.0.) ibij = ibij + 1
709                continue
710             continue
165             if(ibij.eq.0) then
                  write(*,58)
58                format(/2x,'Atom',9x,'Mult.'
     *            ,6x,'x',9x,'y',9x,'z',9x,'B',9x,'N')
                  write(*,171)
     *            (atext(k+iof),ntyp(k+iof,id),nint(rmult(k+iof)),
     *            (atom(j,k+iof),j=1,5),k=1,natom(ip))
171               format(2x,a4,1x,a4,i7,2x,5f10.5)
                else
                  write(*,57)
57                format(/2x,'Atom',9x,'Mult.'
     *            ,6x,'x',9x,'y',9x,'z',9x,'B',9x,'N',/,25x,'B11'
     *            ,7x,'B22',7x,'B33',7x,'B12',7x,'B13',7x,'B23')
                  write(*,170)
     *            (atext(k+iof),ntyp(k+iof,id),nint(rmult(k+iof)),
     *            (atom(j,k+iof),j=1,11),k=1,natom(ip))
170               format(2x,a4,1x,a4,i7,2x,5f10.5 / 20x,6f10.5)
                end if
                if(imo(ip).eq.2) then
                  write(*,59)
 59               format(/2x,'Atom',7x,'Mx',8x,'My',8x,'Mz')
                  write(*,172)
     *            (atext(k+iof),(atom(11+j,k+iof),j=1,3),k=1,natom(ip))
 172              format(2x,a4,2x,3f10.5)
                end if

              else

c --- Symbol des Streuers ntyp bestimmen:

                do 245 n=1,natom(ip)
                   ntyp(iof+n,id) = ntyp(iof+n,1)
245             continue

              end if
            end if


100      continue

200   continue

      close(ke)
      return


c --- Fehlermeldungen:

1000  stop ' Error: Cannot open INPUT file'

      end


c******************************************************************************
c*  Intensitaeten aus der Int.-Datei den von FIS erzeugten Reflexen zuordnen  *
c******************************************************************************

      subroutine intinp(ke,intfile,imo,iref,theta,rlopo,rint,
     *                  rsig,cint,nhkl,irf,IRS,ierr)

      parameter(EPSY = 0.00001, PI = 3.14159265359)

      character intfile*80, cint(IRS)*5, titel*80
      integer h, k, l, n, iq, iref(7,IRS)
      real rint(IRS), theta(IRS), rlopo(IRS), rsig(IRS)


c --- Datei mit den Intensitaets-Werten oeffnen:

      open(ke,file=intfile,status='old',err=1000)

c --- Titelzeile lesen:

      read(ke,'(a)') titel

c --- Daten einlesen:

      if(nhkl.eq.0) then

        if(imo.eq.1) then
          i = 0
10        i = i + 1
          read(ke,*,err=1100,end=12) 
     *    num, iref(5,i), iref(1,i), iref(2,i), iref(3,i), iref(4,i), 
     *    iref(6,i), ths, rlopo(i), rint(i), rsig(i), icod
          write(cint(i)(1:1),'(i1)') icod
          theta(i) = PI/360. * ths
          goto 10
        else
          i = 0
11        i = i + 1
          read(ke,*,err=1100,end=12) 
     *    num, iref(5,i), iref(1,i), iref(2,i), iref(3,i), iref(6,i), 
     *    ths, rlopo(i), rint(i), rsig(i), icod
          write(cint(i)(1:1),'(i1)') icod
          iref(4,i) = 0
          theta(i) = PI/360. * ths
          goto 11
        end if
12      irf = i - 1
        nhkl = irf

      else

c --- Daten einlesen und sortieren:

        if(imo.eq.1) then
          i = 0
20        i = i + 1
          read(ke,*,err=1100,end=220) 
     *        num, iq, h, k, l, n, mlt, ths, rlp, xint, stdd, icod
          do 30 j=1,nhkl
             if(iref(1,j).eq.h.and.iref(2,j).eq.k.and.
     *          iref(3,j).eq.l.and.iref(4,j).eq.n.and.
     *          iref(5,j).eq.iq) then
               rint(j) = xint
               rsig(j) = stdd
               write(cint(j)(1:1),'(i1)') icod
               goto 20
             end if
30        continue
          goto 20
        else
          i = 0
40        i = i + 1
          read(ke,*,err=1100,end=220) 
     *        num, iq, h, k, l, mlt, ths, rlp, xint, stdd, icod
          do 50 j=1,nhkl
             if(iref(1,j).eq.h.and.iref(2,j).eq.k.and.
     *          iref(3,j).eq.l.and.iref(5,j).eq.iq) then
               rint(j) = xint
               rsig(j) = stdd
               write(cint(j)(1:1),'(i1)') icod
               goto 40
             end if
50        continue
          goto 40
        end if
220     irf = i - 1

      end if


      ierr = 0
      close(ke)
      return

1000  ierr = 1
      return

1100  ierr = 2
      close(ke)
      return

      end

c******************************************************************************
c***           Intensitaeten in der Int.-Datei abspeichern                  ***
c******************************************************************************

      subroutine intout(ka,intfile,filex,imo,iref,theta,rlopo,
     *                  rint,cint,lint,aint,nhkl,IRS,dp,MSZ,ierr)

      parameter (PI = 3.14159265359, EPSY = 0.00001, MR = 2000)

      logical filex
      character intfile*80, cint(IRS)*5, ci(MR)*5
      integer h, k, l, iref(7,IRS), lint(IRS), ir(7,MR)
      real rint(IRS), theta(IRS), rlopo(IRS), aint(IRS), dp(MSZ)
      real rs(MR), th(MR), rl(MR), ri(MR)

      character ctemp(MR)*5
      integer itemp(MR), ll(MR)
      real temp(MR)

c --- Initialisierung:

      m = nhkl
      do 5 j=1,nhkl
         do 4 i=1,7
            ir(i,j) = iref(i,j)
 4       continue
         th(j) = theta(j)
         rl(j) = rlopo(j)
         ri(j) = rint(j)
         km = lint(j)
         if(km.ne.0) rs(j) = dp(km) * abs(aint(j))
         ci(j) = cint(j)
5     continue

      if(filex) then

c --- Datei mit den Intensitaets-Werten oeffnen, 
c     Daten einlesen und sortieren:

        open(ka,file=intfile,status='old',err=1000)
        read(ka,*)

        if(imo.eq.1) then
10        read(ka,*,err=1100,end=220) 
     *        num, iq, h, k, l, n, mlt, ths, rlp, xint, stdd, icod
          do 20 j=1,nhkl
             if(iref(1,j).eq.h.and.iref(2,j).eq.k.and.
     *          iref(3,j).eq.l.and.iref(4,j).eq.n.and.
     *          iref(5,j).eq.iq) goto 10
20        continue
          m = m + 1
          ir(1,m) = h
          ir(2,m) = k
          ir(3,m) = l
          ir(4,m) = n
          ir(5,m) = iq 
          ir(6,m) = mlt
          th(m)   = PI/360. * ths
          rl(m)   = rlp
          ri(m)   = xint
          rs(m)   = stdd 
          write(ci(m)(1:1),'(i1)') icod
          goto 10
        else
30        read(ka,*,err=1100,end=220) 
     *        num, iq, h, k, l, mlt, ths, rlp, xint, stdd, icod
          do 40 j=1,nhkl
             if(iref(1,j).eq.h.and.iref(2,j).eq.k.and.
     *          iref(3,j).eq.l.and.iref(5,j).eq.iq) goto 30
40        continue
          m = m + 1
          ir(1,m) = h
          ir(2,m) = k
          ir(3,m) = l
          ir(4,m) = 0
          ir(5,m) = iq
          ir(6,m) = mlt
          th(m)   = PI/360. * ths
          rl(m)   = rlp
          ri(m)   = xint
          rs(m)   = stdd 
          write(ci(m)(1:1),'(i1)') icod
          goto 30
        end if
220     close(ka)

c --- Sortieren der Reflexe nach aufsteigendem Theta-Wert:

        if(m.gt.0) then
          call refsort(ir,th,rl,ri,rs,ci,m,ctemp,itemp,temp,ll,MR)
        end if

      end if

c --- Neue Datei anlegen:

      open(ka,file=intfile,status='unknown',err=1000)
      call intwrite(ka,MR,imo,ir,th,rl,ri,rs,ci,m)
      ierr = 0
      close(ka)
      return

1000  ierr = 1
      return

1100  ierr = 2
      close(ka)
      return

      end


c******************************************************************************
c***            Reflexe nach aufsteigendem Theta-Wert sortieren             ***
c******************************************************************************

      subroutine refsort(iref,theta,rlopo,rint,rsig,cint,nhkl,
     *                   ctemp,itemp,temp,l,IRS)

      character cint(IRS)*5
      integer iref(7,IRS)
      real theta(IRS), rlopo(IRS), rint(IRS), rsig(IRS)

      character ctemp(IRS)*5
      integer itemp(IRS), l(IRS)
      real temp(IRS)

      do 2 i=1,nhkl
         l(i) = i
2     continue
      m = nhkl - 1

      do 15 ll=1,IRS
         k = 0
         do 10 j=1,2
            do 10 i=j,m,2
               inp  = l(i)
               inp1 = l(i+1)
               r = theta(inp1) - theta(inp)   
               if(r.le.0) then
                 iz     = l(i)
                 l(i)   = l(i+1)
                 l(i+1) = iz
                 k = k + 1
               end if
10          continue
         if(k.eq.0) goto 20
15    continue

20    do 30 i=1,nhkl
         ctemp(i) = cint(i)
30    continue
      do 31 i=1,nhkl
         cint(i) = ctemp(l(i))
31    continue

      do 35 j=1,7
         do 32 i=1,nhkl
            itemp(i) = iref(j,i)
32       continue
         do 33 i=1,nhkl
            iref(j,i) = itemp(l(i))
33       continue
35    continue

      do 36 i=1,nhkl
         temp(i) = theta(i)
36    continue
      do 37 i=1,nhkl
         theta(i) = temp(l(i))
37    continue

      do 38 i=1,nhkl
         temp(i) = rlopo(i)
38    continue
      do 39 i=1,nhkl
         rlopo(i) = temp(l(i))
39    continue

      do 40 i=1,nhkl
         temp(i) = rint(i)
40    continue
      do 41 i=1,nhkl
         rint(i) = temp(l(i))
41    continue

      do 42 i=1,nhkl
         temp(i) = rsig(i)
42    continue
      do 43 i=1,nhkl
         rsig(i) = temp(l(i))
43    continue

      
      return
      end

 




c******************************************************************************
c***  Bei Reflexen, die am exakt gleichen Ort erscheinen, wird immer nur    ***
c***  einer verfeinert und die anderen gleich Null gesetzt                  ***
c******************************************************************************

      subroutine intsort(laue,defint,ratio,iref,rint,cint,nhkl,IRS)

      character cint(IRS)*5
      integer iref(7,IRS)
      real rint(IRS), ratio(2)


c --- Kubisch:

      if(laue.eq.10.or.laue.eq.11) then
        do 200 i=1,nhkl
           rint(i) = defint * ratio(iref(5,i))
           cint(i)(1:1) = '1'
           ms1 = iref(1,i)**2 + iref(2,i)**2 + iref(3,i)**2 
           do 100 j=1,i-1
              if(iref(5,i).eq.iref(5,j).and.iref(4,i).eq.iref(4,j)) then
                ms2 = iref(1,j)**2 + iref(2,j)**2 + iref(3,j)**2 
                if(ms1.eq.ms2) then
                  cint(i)(1:1) = '0'
                  rint(i) = 0.
                  goto 200
                end if
              end if 
 100       continue
 200    continue

c --- Tetragonal:

      else if(laue.eq.4.or.laue.eq.5) then
        do 400 i=1,nhkl
           rint(i) = defint * ratio(iref(5,i))
           cint(i)(1:1) = '1'
           ms1 = iref(1,i)**2 + iref(2,i)**2 
           do 300 j=1,i-1
              if(iref(5,i).eq.iref(5,j).and.iref(4,i).eq.iref(4,j).and.
     *           iref(3,i).eq.iref(3,j)) then
                ms2 = iref(1,j)**2 + iref(2,j)**2 
                if(ms1.eq.ms2) then
                  cint(i)(1:1) = '0'
                  rint(i) = 0.
                  goto 400
                end if
              end if 
 300       continue
 400    continue

c --- Hexagonal:

      else if(laue.eq.6.or.laue.eq.7.or.laue.eq.8.or.
     *        laue.eq.9.or.laue.eq.15) then
        do 600 i=1,nhkl
           rint(i) = defint * ratio(iref(5,i))
           cint(i)(1:1) = '1'
           ms1 = iref(1,i)**2 + iref(2,i)**2 + iref(1,i)*iref(2,i) 
           do 500 j=1,i-1
              if(iref(5,i).eq.iref(5,j).and.iref(4,i).eq.iref(4,j).and.
     *           iref(3,i)**2.eq.iref(3,j)**2) then
                ms2 = iref(1,j)**2 + iref(2,j)**2 + iref(1,j)*iref(2,j) 
                if(ms1.eq.ms2) then
                  cint(i)(1:1) = '0'
                  rint(i) = 0.
                  goto 600
                end if
              end if 
 500       continue
 600    continue

c --- Rhomboedrisch:

      else if(laue.eq.13.or.laue.eq.14) then
        do 800 i=1,nhkl
           rint(i) = defint * ratio(iref(5,i))
           cint(i)(1:1) = '1'
           ms1 = iref(1,i)**2 + iref(2,i)**2 + iref(3,i)**2
           ks1 = iref(1,i)*iref(2,i) + iref(1,i)*iref(3,i) + 
     *           iref(2,i)*iref(3,i)
           do 700 j=1,i-1
              if(iref(5,i).eq.iref(5,j).and.iref(4,i).eq.iref(4,j)) then
                ms2 = iref(1,j)**2 + iref(2,j)**2 + iref(3,j)**2
                ks2 = iref(1,j)*iref(2,j) + iref(1,j)*iref(3,j) + 
     *                iref(2,j)*iref(3,j)
                if(ms1.eq.ms2.and.ks1.eq.ks2) then
                  cint(i)(1:1) = '0'
                  rint(i) = 0.
                  goto 800
                end if
              end if 
 700       continue
 800    continue

c --- Orthorhombisch, monoklin, triklin:

      else
        do 900 i=1,nhkl
           rint(i) = defint * ratio(iref(5,i))
           cint(i)(1:1) = '1'
 900    continue

      end if

      return
      end


c******************************************************************************
c***                    Intensitaeten ausgeben                              ***
c******************************************************************************

      subroutine intwrite(ka,IRS,imo,iref,theta,rlopo,rint,rsig,
     *                    cint,nhkl)

      parameter (PI = 3.14159265359)
      integer iref(7,IRS)
      real theta(IRS), rlopo(IRS), rint(IRS), rsig(IRS)
      character cint(IRS)*5

      if(imo.eq.1) then
        write(ka,'(a,a)') ' No. Co.     h   k   l   n  Mult   2-Theta',
     *'     Lopo     Intensity      sigma   R'
        do 100 i=1,nhkl
           th_obs = 360./PI * theta(i)
           read(cint(i)(1:1),'(i1)') icod
           write(ka,'(2i4,2x,4i4,i6,2f10.4,f13.4,f12.4,i3)') 
     *     i, iref(5,i), iref(1,i), iref(2,i), iref(3,i), iref(4,i),
     *     iref(6,i), th_obs, rlopo(i), rint(i), rsig(i), icod
100     continue
      else
        write(ka,'(a,a)') ' No. Co.     h   k   l  Mult   2-Theta',
     *'     Lopo     Intensity      sigma   R'
        do 200 i=1,nhkl
           th_obs = 360./PI * theta(i)
           read(cint(i)(1:1),'(i1)') icod
           write(ka,'(2i4,2x,3i4,i6,2f10.4,f13.4,f12.4,i3)') 
     *     i, iref(5,i), iref(1,i), iref(2,i), iref(3,i), iref(4,i),
     *     th_obs, rlopo(i), rint(i), rsig(i), icod
200     continue
      end if

      return
      end


c******************************************************************************
c***             beobachtete Intensitaeten ausgeben berechnen               ***
c******************************************************************************

      subroutine iobscalc(nprof,rlim,wdt,t,eta,asym,obs,
     *                    calc,bac,the,mit,izahl,IDZ,excl,iexcl,IEZ,
     *                    theta,hwb,rint,riobs,nhkl,IRS)

      implicit none
  
      external profil, pkshift

      integer IDZ, IEZ, IRS
      integer mit(IDZ), nprof, i, k, m, iexcl, nhkl, izahl
      real eta(3), asym(4), t(3), rlim, wdt
      real obs(IDZ), calc(IDZ), bac(IDZ), the(IDZ)
      real theta(IRS), hwb(IRS), riobs(IRS), rint(IRS), excl(2,IEZ)
      real profil, pi, ex1, ex2, eta1, eta2
      real thobs, thecor, thecor1, pkshift, prof
      real b1, rmin, rmax, xmin, xmax, xiobs
      real dPVdT, dPVdG, dPVdE1, dPVdE2, dPVdA(4)
      data pi / 3.14159265359 /

      xmin = the(1) + pkshift(the(1),t,3)
      xmax = the(izahl) + pkshift(the(izahl),t,3)

      do 100 i=1,nhkl

c --- Reflexlage berechnen:

	 thobs  = 360./pi * theta(i)

c --- Winkelabhaengigkeit des Parameters Eta/Pearson-M:

         if(nprof.eq.5) then
            eta1 = eta(1)
            eta2 = eta(2)
         else if(nprof.eq.6.or.nprof.eq.9) then
            eta1 = eta(1) + eta(2) * thobs + eta(3) * thobs**2
            eta2 = 0.
         else if(nprof.eq.7) then
            eta1 = eta(1) + eta(2) / thobs + eta(3) / thobs**2
            eta2 = 0.
         else
            eta1 = 0.
            eta2 = 0.
         end if

c --- Excluded Regions beruecksichtigen:

	 do 75 m=1,iexcl
            ex1 = excl(1,m) + pkshift(excl(1,m),t,3)
            ex2 = excl(2,m) + pkshift(excl(2,m),t,3)
	    if((thobs.gt.ex1).and.(thobs.lt.ex2)) then
              riobs(i) = -1.
              goto 100
            end if
75       continue

c --- Bereich um den Reflex berechnen:

         b1   = wdt * hwb(i)
	 rmin = thobs - b1
	 rmax = thobs + b1
	 rmin = amax1(rmin,xmin)
	 rmax = amin1(rmax,xmax)

c --- beobachtete Intensitaet berechnen:

	 xiobs = 0.
	 do 80 k=1,izahl-1

            thecor = the(k) + pkshift(the(k),t,3)
            thecor1 = the(k+1) + pkshift(the(k+1),t,3)
            if(thecor.ge.rmin.and.thecor1.le.rmax.and.mit(k).eq.1) then

c --- Profilberechnung:

              prof = profil(nprof,thecor,thobs,eta1,eta2,asym,hwb(i),
     *                      rlim,dPVdT,dPVdG,dPVdE1,dPVdE2,dPVdA)

c --- Iobs berechnen:

              xiobs = xiobs + rint(i) * (thecor1 - thecor) * prof * 
     *                (obs(k)-bac(k))/amax1(calc(k)-bac(k),1.)

            else if(thecor.gt.rmax) then

              goto 90

            end if

80       continue 

90       if(xiobs.lt.0.) xiobs = 0. 
         riobs(i) = xiobs

100   continue


      return 
      end



c******************************************************************************
c***       Beobachtete u. berechnete Intensitaeten ausgeben                 ***
c******************************************************************************

      subroutine iobswrite(ka,imo,q,iref,theta,rint,riobs,nhkl,IRS)

      parameter (PI = 3.14159265359)
      integer iref(7,IRS)
      real theta(IRS), riobs(IRS), rint(IRS), q(3), h(3)


      if(imo.eq.1) then

        write(ka,70)
70      format(/,'   No. Co.     h   k   l   n  Mult    Pos.',
     *'       Icalc        Iobs       Diff.')
        do 100 i=1,nhkl
           if(riobs(i).ge.0.) then
             thobs = 360./PI * theta(i)
             write(ka,82) i, iref(5,i), iref(1,i), iref(2,i), 
     *                    iref(3,i), iref(4,i), iref(6,i), thobs,
     *                    rint(i), riobs(i), riobs(i)-rint(i)
82           format(2x,2i4,2x,4i4,i6,f8.3,3f12.2)
           end if
100     continue

      else if(imo.eq.2) then

        write(ka,120)
120     format(/,'   No.      H     K     L   Mult    Pos.',
     *'       Icalc        Iobs       Diff.')
        do 150 i=1,nhkl
           if(riobs(i).ge.0.) then
             call hvec(iref(1,i),iref(2,i),iref(3,i),iref(4,i),q,h)
             thobs = 360./PI * theta(i)
             write(ka,132) i, h(1), h(2), h(3), iref(6,i), 
     *                     thobs, rint(i), riobs(i), riobs(i)-rint(i)
132          format(2x,i4,2x,3f6.1,i6,f8.3,3f12.2)
           end if
150     continue

      else

        write(ka,170)
170     format(/,'   No. Co.     h   k   l  Mult    Pos.',
     *'       Icalc        Iobs       Diff.')
        do 200 i=1,nhkl
           if(riobs(i).ge.0.) then
             thobs = 360./PI * theta(i)
             write(ka,182) i, iref(5,i), iref(1,i), iref(2,i),
     *                     iref(3,i), iref(6,i), thobs,
     *                     rint(i), riobs(i), riobs(i)-rint(i)
182          format(2x,2i4,2x,3i4,i6,f8.3,3f12.2)
           end if
200     continue

      end if


      return
      end


c******************************************************************************
c***       Beobachtete u. berechnete Intensitaeten ausgeben                 ***
c******************************************************************************

      subroutine kofout(ka,koffile,LPHASE,IRS,nphase,titel,iref,
     *                  theta,f000,ffreal,ffimag,rint,riobs,ic)

      parameter (PI = 3.14159265359)
      character koffile*80, titel*70
      integer iref(7,IRS), ic(LPHASE)
      real theta(IRS), riobs(IRS), ffreal(IRS), ffimag(IRS), rint(IRS) 
      complex f000(LPHASE)


c --- Datenformat:

81    format(1x,2i4,3x,4i4,f8.3,4f10.2)

c --- Datei oeffnen und Titelzeile schreiben:

      if(ka.ne.6) then 
        open(ka,file=koffile,status='unknown',err=1000)
	write(ka,'(a)') titel
      end if

c --- Intensitaeten ausgeben:

      iend = 0
      do 120 ip=1,nphase

   	 write(ka,'(2x,a,a)')
     *   'No. Co.      h   k   l   n    Pos.       A',
     *   '         B    FF(calc)   FF(obs)'

         fre = real(f000(ip))
         fim = aimag(f000(ip))
         ff  = fre**2 + fim**2
	 write(ka,81) 0, 1, 0, 0, 0, 0, 0., fre, fim, ff, 0.

	 ibeg = iend + 1
	 iend = iend + ic(ip)
	 do 100 i=ibeg,iend
            if(riobs(i).ge.0.) then
	      ix = i - ibeg + 1
	      thobs = 360./PI * theta(i)
              ff = ffreal(i)**2 + ffimag(i)**2
              ffobs = ff/rint(i) * riobs(i)
	      write(ka,81) ix, iref(5,i), iref(1,i), iref(2,i), 
     *                     iref(3,i), iref(4,i), thobs, ffreal(i),
     *                     ffimag(i), ff, ffobs
            end if
100      continue

120   continue

      if(ka.ne.6) close(ka)
      return

1000  stop ' Cannot open KOF file'
      end


C******************************************************************************
C***              Ausgabe der Korrelationsmatrix                            ***
C******************************************************************************

      SUBROUTINE KORREL(MSZ,MAXS,RJAC)

      INTEGER*4 KS(50)
      REAL RJAC(MSZ,MSZ)


      WRITE(*,'(//2X,A)') 'Correlation matrix: '

      IA  =  1
      LIM = 19

35    LIM = MIN0(MAXS,LIM)

      WRITE(*,'(/5x,19I6)') (I,I=IA,LIM)

      DO 40 I=1,MAXS
	 L = 0
	 X = RJAC(I,I)
	 DO 37 J=IA,LIM
	    L  = L+1
	    YY = RJAC(J,J)*X
	    IF(ABS(YY).EQ.0.0) THEN
	      WRITE(*,'(/2X,A,I3)')
     1        'Divided by zero in module KORREL with parameter ',J
	      GOTO 37
	    END IF
	    KS(L) = 100.*RJAC(I,J)/(SQRT(ABS(YY))*SIGN(1.,YY))+0.5
37       CONTINUE
	 WRITE(*,'(I5,19I6)') I,(KS(J),J=1,L)
40    CONTINUE

      IF(LIM.GE.MAXS) GOTO 45

      IA  = LIM+1
      LIM = LIM+19
      GOTO 35

45    RETURN
      END


c******************************************************************************
c***       Berechnung der Strukturfaktoren magnetischer Reflexe             ***
c******************************************************************************

      subroutine magstruc(ibeta,rmult,atom,aato,lato,natom,NATS,
     *                    nprim,ism,tran,nsym,
     *                    iref,theta,hwb,rlopo,rint,
     *                    ffreal,ffimag,nhkl,IRS,
     *                    aglb,lglb,par,apar,lpar,rlamda,ratio,celld,
     *                    pref,ipref,ac,dfp,ptr,
     *                    sumda,sumdb,dersto,deriv,maxs,MSZ)

      parameter(PIQ = 9.869604401, PI = 3.14159265359,
     *          PII = 6.28318530718, EPSY = 0.00001)

      external ndet3, fwhm, precor
      logical pac
      integer lglb(10), lpar(23), lato(14,NATS), ptr(NATS)
      integer ism(3,3,48), nprim(48), iref(7,IRS)
      real tran(3,48), atom(14,NATS), par(23), rlamda(2), ratio(2)
      real theta(IRS), hwb(IRS), rint(IRS)
      real rlopo(IRS), ac(9,NATS), dfp(NATS), rmult(NATS), celld(6)
      real ffreal(IRS), ffimag(IRS), dersto(MSZ,IRS), pref(3)
      real aglb(10), apar(23), aato(14,NATS), deriv(MSZ)

      real sumr(3), sumi(3), precor
      real sumda(3,14,NATS), sumdb(3,14,NATS), df(14), fwhm
      real am(3), bm(3), h(3), hs(3), al(3,3), fac(6), atd(6)

c      data cm /-0.27/
      data cm2 /0.0729/

c --- Cosinuse der Gitter-Winkel:

      cosa = cos(PI/180.*celld(4))
      cosb = cos(PI/180.*celld(5))
      cosg = cos(PI/180.*celld(6))

c --- reziproke Gitterkonstanten :

      al(1,1) = par(14) * par(14)
      al(2,2) = par(15) * par(15)
      al(3,3) = par(16) * par(16)
      al(1,2) = par(14) * par(15) * par(19)
      al(1,3) = par(14) * par(16) * par(18)
      al(2,3) = par(15) * par(16) * par(17)
      al(2,1) = al(1,2)
      al(3,1) = al(1,3)
      al(3,2) = al(2,3)

c --- Faktoren der anisotropen Temperaturfaktoren:

      if(ibeta.eq.0) then
        fac(1) = 1.
        fac(2) = 1.
        fac(3) = 1.
        fac(4) = 2.
        fac(5) = 2.
        fac(6) = 2.
      else if(ibeta.eq.1) then
        fac(1) = 0.25 * al(1,1)
        fac(2) = 0.25 * al(2,2)
        fac(3) = 0.25 * al(3,3)
        fac(4) = 0.50 * par(14) * par(15)
        fac(5) = 0.50 * par(14) * par(16)
        fac(6) = 0.50 * par(15) * par(16)
      else if(ibeta.eq.2) then
        fac(1) = PIQ * al(1,1)
        fac(2) = PIQ * al(2,2)
        fac(3) = PIQ * al(3,3)
        fac(4) = 2.*PIQ * par(14) * par(15)
        fac(5) = 2.*PIQ * par(14) * par(16)
        fac(6) = 2.*PIQ * par(15) * par(16)
      end if

c --- Vorzugsorientierung-Korrektur (ja oder nein):

      pac = par(6).ne.0..or.lpar(6).ne.0.or.
     *      par(7).ne.0..or.lpar(7).ne.0

      do 300 i=1,nhkl

         do 11 l=1,3
            am(l) = 0.
            bm(l) = 0.
 11      continue

c --- Reziproker Gittervektor h:

         h(1) = iref(1,i) + iref(4,i) * par(11)
         h(2) = iref(2,i) + iref(4,i) * par(12)
         h(3) = iref(3,i) + iref(4,i) * par(13)

c --- Betragsquadrat hh des reziproken Gittervektor h berechnen:

         hh = 0.
         do 13 l=1,3
            do 12 j=1,3
               hh = hh + h(l) * h(j) * al(l,j)
 12         continue
 13      continue

c --- Reflexlage und Halbwertsbreite berechnen:

         sinth       = 0.5 * rlamda(iref(5,i)) * sqrt(hh)
         sinlam      = hh/4.
         if(sinlam.eq.0.) sinlam = 0.0001
         if(abs(sinth).gt.1.) sinth = 0.
         th          = asin(sinth)
         tanth       = sinth/sqrt(1.-sinth*sinth)
         if(tanth.eq.0.) tanth = 0.0001
         theta(i)    = th
         hwb(i)      = fwhm(par(8),par(9),par(10),theta(i))

         do 200 k=1,natom

            do 21 l=1,3
               sumr(l) = 0.
               sumi(l) = 0.
               do 20 j=1,14
                  sumda(l,j,k) = 0.
                  sumdb(l,j,k) = 0.
 20            continue
 21         continue

            do 100 ns=1,nsym

c --- Symmetrieaequivalente Reflexe erzeugen:

               tr = 0.
               do 32 l=1,3
                  hs(l) = 0.
                  tr = tr + tran(l,ns) * h(l)
                  do 31 j=1,3
                     hs(l) = hs(l) + h(j) * ism(j,l,ns)
31                continue
32             continue

c --- Skalarprodukt hs * atom berechnen:

               scala = tr 
               do 33 j=1,3
                  scala = scala + hs(j) * atom(j,k)
33             continue 
               scala = PII * scala

c --- Anisotroper Temperaturfaktor des k-ten Atoms:

               atd(1) = hs(1) * hs(1) * fac(1)
               atd(2) = hs(2) * hs(2) * fac(2)
               atd(3) = hs(3) * hs(3) * fac(3)
               atd(4) = hs(1) * hs(2) * fac(4)
               atd(5) = hs(1) * hs(3) * fac(5)
               atd(6) = hs(2) * hs(3) * fac(6)

               arg = 0.
               do 35 j=1,6
                  arg = arg + atd(j) * atom(5+j,k)
 35            continue
     
               aniso = exp(-arg)

c --- Cosinus und Sinus der Phase mult. mit dem anisotropen Temp.fak.:

               fcos = cos(scala) * aniso
               fsin = sin(scala) * aniso

c --- Magnetischer Strukturfaktor P = A + i*B berechnen:

               do 60 l=1,3
                  sp = 0.
                  do 50 j=1,3
                     np = 11 + j
                     nspr = ism(l,j,ns) * nprim(ns) * ndet3(ism(1,1,ns))
                     sp = sp + nspr * atom(np,k)
                     sumda(l,np,k) = sumda(l,np,k) + nspr * fcos 
                     sumdb(l,np,k) = sumdb(l,np,k) + nspr * fsin
 50               continue
                  spcos = sp * fcos
                  spsin = sp * fsin
                  sumr(l) = sumr(l) + spcos
                  sumi(l) = sumi(l) + spsin
                  do 55 j=1,3
                     sumda(l,j,k) = sumda(l,j,k) - spsin * hs(j) 
                     sumdb(l,j,k) = sumdb(l,j,k) + spcos * hs(j)
 55               continue
                  do 57 j=6,11
                     sumda(l,j,k) = sumda(l,j,k) - atd(j-5) * spcos
                     sumdb(l,j,k) = sumdb(l,j,k) - atd(j-5) * spsin
 57               continue
 60            continue

 100        continue            ! Schleife ueber alle Symmetrieelemente

c --- Magnetischer Formfaktor berechnen:

            fm = ac(1,ptr(k)) * exp(-ac(2,ptr(k))*sinlam) +
     *           ac(3,ptr(k)) * exp(-ac(4,ptr(k))*sinlam) +
     *           ac(5,ptr(k)) * exp(-ac(6,ptr(k))*sinlam) +
     *           ac(7,ptr(k))

c --- Besetzungszahl atom(5,k), isotroper Temperaturfaktor atom(4,k),
c     auf die Zahl der Symm.-Karten norm. Wyckoff-Multiplizitaet rmult(k):

            tmf  = exp(-atom(4,k)*sinlam) * rmult(k) * fm
            facm = atom(5,k) * tmf

c --- Strukturfaktor und Ableitungen:

            do 110 l=1,3
               sumda(l,4,k) = sumr(l)
               sumdb(l,4,k) = sumi(l)
               sumda(l,5,k) = sumr(l) * tmf
               sumdb(l,5,k) = sumi(l) * tmf
               do 108 j=1,14
                  if(j.ne.5) then
                    sumda(l,j,k) = sumda(l,j,k) * facm
                    sumdb(l,j,k) = sumdb(l,j,k) * facm
                  end if
 108           continue
               am(l) = am(l) + sumr(l) * facm
               bm(l) = bm(l) + sumi(l) * facm
 110        continue

 200     continue      ! Schleife ueber alle Atome

c --- Betragsquadrat des magnetischen Strukturfaktors berechnen:

         pp = am(1)**2 + am(2)**2 + am(3)**2 +
     *        bm(1)**2 + bm(2)**2 + bm(3)**2 +
     *        2.*(am(1)*am(2) + bm(1)*bm(2))*cosg +
     *        2.*(am(1)*am(3) + bm(1)*bm(3))*cosb +
     *        2.*(am(2)*am(3) + bm(2)*bm(3))*cosa

         ha = h(1)*am(1)/celld(1) + h(2)*am(2)/celld(2) + 
     *        h(3)*am(3)/celld(3)
         hb = h(1)*bm(1)/celld(1) + h(2)*bm(2)/celld(2) + 
     *        h(3)*bm(3)/celld(3)
         hp = (ha**2 + hb**2) / hh

         sj = cm2 * (pp - hp)

         ffreal(i) = pp 
         ffimag(i) = sj

c --- Korrektur der Vorzugsorientierung:

         if(pac) then
           proc = precor(ipref,pref,par(6),par(7),al,h,sinlam,
     *                   dpref1,dpref2)
         else
           proc = 1.
         end if

c --- Intensitaet berechnen:

         xint = rlopo(i) * iref(6,i) * exp(-2.*par(5)*sinlam) * 
     *          ratio(iref(5,i)) * par(4)
         rcor = proc * xint
         rint(i) = sj * rcor 

c === Ableitungen von y_calc(i), die nicht von x(i) abhaengen:

         do 115 j=1,maxs
            deriv(j) = 0.
115      continue  

c --- Peak shape function parameters e0, e1 and e2:

         k = lpar(1)
         if(k.ne.0) deriv(k) = deriv(k) + apar(1) * rint(i)
         k = lpar(2)
         if(k.ne.0) deriv(k) = deriv(k) + apar(2) * rint(i)
         k = lpar(3)
         if(k.ne.0) deriv(k) = deriv(k) + apar(3) * rint(i)

c --- Skalierung:

         k = lpar(4)
         if(k.ne.0) deriv(k) = deriv(k) + apar(4) * rint(i)/par(4)

c --- Overall Temperature Factor:

         k = lpar(5)
         if(k.ne.0) deriv(k) = deriv(k) - 
     *                         2. * apar(5) * sinlam * rint(i) 

c --- Preferred Orientation Parameter G1, G2:

         k = lpar(6)
         if(k.ne.0) deriv(k) = deriv(k) + apar(6) * dpref1 * sj * xint
         k = lpar(7)
         if(k.ne.0) deriv(k) = deriv(k) + apar(7) * dpref2 * sj * xint

c --- Halbwertsbreite-Parameter U, V, W:

         rb = 0.5/hwb(i) * rint(i)

         k = lpar(8)
         if(k.ne.0) deriv(k) = deriv(k) + apar(8)*rb*tanth*tanth 
         k = lpar(9)
         if(k.ne.0) deriv(k) = deriv(k) + apar(9)*rb*tanth 
         k = lpar(10)
         if(k.ne.0) deriv(k) = deriv(k) + apar(10)*rb 

c --- reziproke Gitterkonstanten:

         rqa = 90./PI * tanth/sinlam * rint(i)

         k = lpar(14)
         if(k.ne.0) deriv(k) = deriv(k) + apar(14) * rqa *
     1              (h(1)*h(1)*par(14) + h(1)*h(2)*par(15)*par(19)
     2                                 + h(1)*h(3)*par(16)*par(18))
         k = lpar(15)  
         if(k.ne.0) deriv(k) = deriv(k) + apar(15) * rqa *
     1              (h(2)*h(2)*par(15) + h(2)*h(1)*par(14)*par(19)
     2                                 + h(2)*h(3)*par(16)*par(17))
         k = lpar(16)      
         if(k.ne.0) deriv(k) = deriv(k) + apar(16) * rqa *
     1              (h(3)*h(3)*par(16) + h(3)*h(1)*par(14)*par(18)
     2                                 + h(3)*h(2)*par(15)*par(17))
         k = lpar(17)
         if(k.ne.0) deriv(k) = deriv(k) + apar(17) * rqa *
     1                         h(2)*h(3)*par(15)*par(16)
         k = lpar(18)
         if(k.ne.0) deriv(k) = deriv(k) + apar(18) * rqa *
     1                         h(1)*h(3)*par(14)*par(16)
         k = lpar(19)
         if(k.ne.0) deriv(k) = deriv(k) + apar(19) * rqa *
     1                         h(1)*h(2)*par(14)*par(15)

c --- Asymmetriekorrektur-Parameter:

         k = lpar(20)
         if(k.ne.0) deriv(k) = deriv(k) + apar(20) * rint(i)
         k = lpar(21)
         if(k.ne.0) deriv(k) = deriv(k) + apar(21) * rint(i)
         k = lpar(22)
         if(k.ne.0) deriv(k) = deriv(k) + apar(22) * rint(i)
         k = lpar(23)
         if(k.ne.0) deriv(k) = deriv(k) + apar(23) * rint(i)

c --- Zeropoint, peak-shift correction:

         do 120 j=1,3
            k = lglb(j)         
            if(k.ne.0) deriv(k) = deriv(k) + aglb(j) * rint(i)
120      continue

c --- Wellenlaenge:

         k = lglb(10)
         if(k.ne.0) deriv(k) = deriv(k) +
     *              aglb(10) * rint(i) * tanth/rlamda(1) * 360./PI

c --- Atomparameter:

         c1 = am(1) +      am(2)*cosg + am(3)*cosb
         c2 = am(1)*cosg + am(2)      + am(3)*cosa
         c3 = am(1)*cosb + am(2)*cosa + am(3)
         s1 = bm(1) +      bm(2)*cosg + bm(3)*cosb
         s2 = bm(1)*cosg + bm(2)      + bm(3)*cosa
         s3 = bm(1)*cosb + bm(2)*cosa + bm(3)

         do 250 j=1,3
            df(j) = PII
 250     continue
         df(4) = -sinlam
         do 255 j=5,14
            df(j) = 1.
 255     continue

         do 270 k=1,natom
            do 260 j=1,14
               m = lato(j,k)
               if(m.ne.0) then
                 dp1 = c1 * sumda(1,j,k) + c2 * sumda(2,j,k) +
     *                 c3 * sumda(3,j,k) + s1 * sumdb(1,j,k) +
     *                 s2 * sumdb(2,j,k) + s3 * sumdb(3,j,k)
                 dp2 = ha * (h(1)*sumda(1,j,k)/celld(1) + 
     *                       h(2)*sumda(2,j,k)/celld(2) + 
     *                       h(3)*sumda(3,j,k)/celld(3)) +
     *                 hb * (h(1)*sumdb(1,j,k)/celld(1) + 
     *                       h(2)*sumdb(2,j,k)/celld(2) + 
     *                       h(3)*sumdb(3,j,k)/celld(3))
                 if(pp.ne.0.and.hp.ne.0.)
     *             deriv(m) = deriv(m) + aato(j,k) * rcor * 
     *                        2. * cm2 * df(j) * (dp1 - dp2)
              end if
 260        continue
 270     continue

c --- Ableitungen fuer jeden Reflex abspeichern:

         do 180 m=1,maxs
            dersto(m,i) = deriv(m)
 180     continue         

 300  continue    ! Schleife ueber alle Reflexe

      return
      end

c******************************************************************************
c***  Testen, ob Gitterkonstanten mit der Laue-Gruppe vertraeglich sind     ***
c******************************************************************************

      integer function metchk(a,laue)
      
      parameter(EPSY = 0.000001)
      real a(6)


      ierr = 0

c --- Alpha-Winkel:

      if((laue.ge.2.and.laue.le.12).or.laue.eq.15) then
        if(abs(a(4)-90.).gt.EPSY) then 
          write(*,'(/2x,a)') 'ERROR: Alpha not equal 90 degrees'
          ierr = ierr + 1
        end if
      end if

c --- Beta-Winkel:

      if((laue.ge.2.and.laue.le.11).or.laue.eq.15) then
        if(abs(a(5)-90.).gt.EPSY) then 
          write(*,'(/2x,a)') 'ERROR: Beta not equal 90 degrees'
          ierr = ierr + 1
        end if
      end if

c --- Gamma-Winkel:

      if((laue.ge.3.and.laue.le.5).or.
     *   (laue.ge.10.and.laue.le.12)) then
        if(abs(a(6)-90.).gt.EPSY) then 
          write(*,'(/2x,a)') 'ERROR: Gamma not equal 90 degrees'
          ierr = ierr + 1
        end if
      else if((laue.ge.6.and.laue.le.9).or.laue.eq.15) then 
        if(abs(a(6)-120.).gt.EPSY) then 
          write(*,'(/2x,a)') 'ERROR: Gamma not equal 120 degrees'
          ierr = ierr + 1
        end if
      end if

c --- a- und b-Achse bei tetragonalen und hexagonalen Zellen:

      if((laue.ge.4.and.laue.le.9).or.laue.eq.15) then
        if(abs(a(1)-a(2)).gt.EPSY) then 
          write(*,'(/2x,a)') 'ERROR: a not equal b '
          ierr = ierr + 1
        end if
      end if

c --- a-, b- und c-Achse bei kubischen und rhomboedrischen Zellen:

      if(laue.eq.10.or.laue.eq.11.or.
     *   laue.eq.13.or.laue.eq.14) then
        if(abs(a(1)-a(2)).gt.EPSY.or.abs(a(1)-a(3)).gt.EPSY.or.
     *     abs(a(2)-a(3)).gt.EPSY) then 
          write(*,'(/2x,a)') 'ERROR: a, b and c are not equal'
          ierr = ierr + 1
        end if
      end if

c --- Alpha-, Beta- und Gamma-Winkel bei rhomboedrischen Zellen:

      if(laue.eq.13.or.laue.eq.14) then
        if(abs(a(4)-a(5)).gt.EPSY.or.abs(a(4)-a(6)).gt.EPSY.or.
     *     abs(a(5)-a(6)).gt.EPSY) then 
          write(*,'(/2x,a)') 
     *    'ERROR: Alpha, beta and gamma are not equal'
          ierr = ierr + 1
        end if
      end if

      metchk = ierr
      return
      end

C******************************************************************************
C***            Berechnen der Intensitaeten und Ableitungen                 ***
C******************************************************************************

      subroutine modstruc(maxs,dersto,ibeta,ixn,
     *                    natom,atom,aato,lato,ampl,aampl,lampl,
     *                    rlamda,ratio,pref,ipref,par,apar,lpar,aglb,
     *                    lglb,ac,dfp,dfpp,ptr,rmult,pa,apos,tp,npos,
     *                    nbas,iref,theta,hwb,rlopo,ffreal,ffimag,
     *                    rint,nuq,nhkl,sumxr,sumxi,sumdr,sumdi,deriv,
     *                    NATS,IRS,MSZ,NORD,IBV)

      external fwhm, precor
      parameter(PIQ = 9.869604401, PI = 3.14159265359)

      logical pac
      real al(3,3), h(3), deriv(MSZ), fwhm, precor 
      real sumxi(11,NATS), sumxr(11,NATS), sumdi(2,IBV,NATS),
     *     sumdr(2,IBV,NATS)

      integer iref(7,IRS), nuq(2*NORD+1,IRS), ptr(NATS) 
      integer lglb(10), lpar(23), lato(14,NATS), lampl(2,IBV,NATS)
      integer nbas(NATS), npos(NATS)
      real aglb(10), apar(23), aato(14,NATS), aampl(2,IBV,NATS)
      real ac(9,NATS), dfp(NATS), dfpp(NATS), rlamda(2), ratio(2)  
      real pa(6,48,IBV,NATS), apos(3,48,NATS), tp(3,48,NATS)
      real atom(14,NATS), pref(3), rmult(NATS), par(23)
      real theta(IRS), rlopo(IRS), hwb(IRS), ffreal(IRS), ffimag(IRS) 
      real rint(IRS), dersto(MSZ,IRS)
      real ampl(2,IBV,NATS)

c --- reziproke Gitterkonstanten :

      al(1,1) = par(14) * par(14)
      al(2,2) = par(15) * par(15)
      al(3,3) = par(16) * par(16)
      al(1,2) = par(14) * par(15) * par(19)
      al(1,3) = par(14) * par(16) * par(18)
      al(2,3) = par(15) * par(16) * par(17)
      al(2,1) = al(1,2)
      al(3,1) = al(1,3)
      al(3,2) = al(2,3)

c --- Vorzugsorientierung-Korrektur (ja oder nein):

      pac = par(6).ne.0..or.lpar(6).ne.0.or.
     *      par(7).ne.0..or.lpar(7).ne.0


c ---- Reflexlagen, Halbwertsbreiten und Intensitaeten berechenen ---------

      do 200 i=1,nhkl

c --- Reziproker Gittervektor h und 
c     dessen Betragsquadrat hh berechnen:

         call hvec(iref(1,i),iref(2,i),iref(3,i),iref(4,i),par(11),h)

         hh = 0.
         do 12 l=1,3
            do 11 j=1,3
               hh = hh + h(l) * h(j) * al(l,j)
11          continue
12       continue

c --- Reflexlage, Lorentz-Polarisations-Faktor, Halbwertsbreite
c     und Overall Temperature Factor berechnen:

         sinth       = 0.5 * rlamda(iref(5,i)) * sqrt(hh)
         sinlam      = hh/4.
         if(sinlam.eq.0.) sinlam = 0.0001
         if(abs(sinth).gt.1.) sinth = 0.
         th          = asin(sinth)
         tanth       = sinth/sqrt(1.-sinth*sinth)
         if(tanth.eq.0.) tanth = 0.0001
         theta(i)    = th
         hwb(i)      = fwhm(par(8),par(9),par(10),theta(i))
                        
c --- Summen, die fuer die Ableitungen benoetigt werden, initialisieren:

         do 25 k=1,natom
            do 22 l=4,5
               sumxr(l,k) = 0.
               sumxi(l,k) = 0.
22         continue
           do 24 m=1,nbas(k)
              do 23 l=1,2
                 sumdr(l,m,k) = 0.
                 sumdi(l,m,k) = 0.
23            continue
24         continue
25       continue

c --- Strukturfaktoren berechnen:

c --- Satelliten-Beitraege aufaddieren:

         fre = 0.
         fim = 0.
         do 30 j=1,2*NORD+1
            ns = nuq(j,i)
            if(ns.eq.999) goto 31
            call struc(NATS,ixn,natom,atom,rmult,h,ns,sinlam,
     *                 par(11),ac,dfp,dfpp,ptr,pa,apos,tp,npos,
     *                 ampl,nbas,sf_real,sf_imag,sumxr,sumxi,sumdr,
     *                 sumdi,IBV)
            fre = fre + sf_real
            fim = fim + sf_imag
30       continue

31       ffreal(i) = fre
         ffimag(i) = fim
         ff = fre*fre + fim*fim

c --- Korrektur der Vorzugsorientierung:

         if(pac) then
           proc = precor(ipref,pref,par(6),par(7),al,h,sinlam,
     *                   dpref1,dpref2)
         else
           proc = 1.
         end if

c --- Intensitaet berechnen:

         xint = rlopo(i) * iref(6,i) * exp(-2.*par(5)*sinlam) * 
     *          ratio(iref(5,i)) * par(4)
         rcor = proc * xint
         rint(i) = ff * rcor 


c --- Ableitungen von y_calc(i), die nicht von x(i) abhaengen:

         do 50 j=1,maxs
            deriv(j) = 0.
50       continue  

c --- Profile function parameters e0, e2 and e2:

         k = lpar(1)
         if(k.ne.0) deriv(k) = deriv(k) + apar(1) * rint(i)
         k = lpar(2)
         if(k.ne.0) deriv(k) = deriv(k) + apar(2) * rint(i)
         k = lpar(3)
         if(k.ne.0) deriv(k) = deriv(k) + apar(3) * rint(i)

c --- Skalierung:

         k = lpar(4)
         if(k.ne.0) deriv(k) = deriv(k) + apar(4) * rint(i)/par(4)

c --- Overall temperature factor:

         k = lpar(5)
         if(k.ne.0) deriv(k) = deriv(k) - 
     *                         2. * apar(5) * sinlam * rint(i) 

c --- Preferred orientation parameters G1, G2:

         k = lpar(6)
         if(k.ne.0) deriv(k) = deriv(k) + apar(6) * dpref1 * ff * xint
         k = lpar(7)
         if(k.ne.0) deriv(k) = deriv(k) + apar(7) * dpref2 * ff * xint

c --- Halbwertsbreite-Parameter U, V, W:

         rb = 0.5/hwb(i) * rint(i)

         k = lpar(8)
         if(k.ne.0) deriv(k) = deriv(k) + apar(8)*rb*tanth*tanth 
         k = lpar(9)
         if(k.ne.0) deriv(k) = deriv(k) + apar(9)*rb*tanth 
         k = lpar(10)
         if(k.ne.0) deriv(k) = deriv(k) + apar(10)*rb 

c --- Wellenvektor:

         rqa = 90./PI * tanth/sinlam * rint(i)

         k = lpar(11)
         if(k.ne.0) deriv(k) = deriv(k) + apar(11) * rqa * iref(4,i) *
     1                         (h(1)*al(1,1)+h(2)*al(1,2)+h(3)*al(1,3))
         k = lpar(12)
         if(k.ne.0) deriv(k) = deriv(k) + apar(12) * rqa * iref(4,i) *
     1                         (h(2)*al(2,2)+h(1)*al(1,2)+h(3)*al(2,3))
         k = lpar(13)
         if(k.ne.0) deriv(k) = deriv(k) + apar(13) * rqa * iref(4,i) *
     1                         (h(3)*al(3,3)+h(1)*al(1,3)+h(2)*al(2,3))  

c --- reziproke Gitterkonstanten:

         k = lpar(14)
         if(k.ne.0) deriv(k) = deriv(k) + apar(14) * rqa *
     1              (h(1)*h(1)*par(14) + h(1)*h(2)*par(15)*par(19)
     2                                 + h(1)*h(3)*par(16)*par(18))
         k = lpar(15)  
         if(k.ne.0) deriv(k) = deriv(k) + apar(15) * rqa *
     1              (h(2)*h(2)*par(15) + h(2)*h(1)*par(14)*par(19)
     2                                 + h(2)*h(3)*par(16)*par(17))
         k = lpar(16)      
         if(k.ne.0) deriv(k) = deriv(k) + apar(16) * rqa *
     1              (h(3)*h(3)*par(16) + h(3)*h(1)*par(14)*par(18)
     2                                 + h(3)*h(2)*par(15)*par(17))
         k = lpar(17)
         if(k.ne.0) deriv(k) = deriv(k) + apar(17) * rqa *
     1                         h(2)*h(3)*par(15)*par(16)
         k = lpar(18)
         if(k.ne.0) deriv(k) = deriv(k) + apar(18) * rqa *
     1                         h(1)*h(3)*par(14)*par(16)
         k = lpar(19)
         if(k.ne.0) deriv(k) = deriv(k) + apar(19) * rqa *
     1                         h(1)*h(2)*par(14)*par(15)

c --- Asymmetriekorrektur-Parameter:

         k = lpar(20)
         if(k.ne.0) deriv(k) = deriv(k) + apar(20) * rint(i)
         k = lpar(21)
         if(k.ne.0) deriv(k) = deriv(k) + apar(21) * rint(i)
         k = lpar(22)
         if(k.ne.0) deriv(k) = deriv(k) + apar(22) * rint(i)
         k = lpar(23)
         if(k.ne.0) deriv(k) = deriv(k) + apar(23) * rint(i)

c --- Zeropoint, peak-shift correction:

         do 110 j=1,3
            k = lglb(j)         
            if(k.ne.0) deriv(k) = deriv(k) + aglb(j) * rint(i)
110      continue

c --- Wellenlaenge:

         k = lglb(10)
         if(k.ne.0) deriv(k) = deriv(k) + 
     *              aglb(10) * rint(i) * tanth/rlamda(1) * 360./PI

c --- Ableitungen nach den Atomparameter:

         do 150 k=1,natom

c --- isotroper Temperaturfaktor:

            m = lato(4,k)
            if(m.ne.0) 
     *        deriv(m) = deriv(m) - 2. * sinlam * rcor * aato(4,k) *
     *                             (fre*sumxr(4,k) + fim*sumxi(4,k))

c --- Besetzungszahl:

            m = lato(5,k)
            if(m.ne.0) 
     *        deriv(m) = deriv(m) + 2./atom(5,k) * rcor * aato(5,k) *
     *                              (fre*sumxr(5,k) + fim*sumxi(5,k))

c --- Auslenkungsamplituden:

            do 135 l=1,nbas(k)
               do 130 j=1,2
                  m = lampl(j,l,k)
                  if(m.ne.0) 
     *              deriv(m) = deriv(m) + 2. * rcor * aampl(j,l,k) * 
     *                         (fre*sumdr(j,l,k) + fim*sumdi(j,l,k))
130            continue
135         continue

150      continue 

c --- Ableitungen fuer jeden Reflex abspeichern:

         do 180 m=1,maxs
            dersto(m,i) = deriv(m)
180      continue


200   continue


      return
      end


c******************************************************************************
c***                   Strukturfaktor berechnen                             ***
c******************************************************************************

      subroutine struc(NATS,ixn,natom,atom,rmult,h,nu,sinlam,q,
     *                 ac,dfp,dfpp,ptr,pa,apos,tp,npos,ampl,nbas,
     *                 sf_real,sf_imag,sumxr,sumxi,sumdr,sumdi,IBV)

      parameter(PII = 6.28318530718, PI = 3.14159265359,
     *          PI2 = 1.570796326795, EPSY = 0.00001, MAX = 200)

      external bessfkt

      real dzr(2,MAX), dzi(2,MAX)
      real djcos(2,MAX), djsin(2,MAX), dpcos(2,MAX), dpsin(2,MAX)

      integer ptr(NATS), nbas(NATS), npos(NATS)
      real ac(9,NATS), dfp(NATS), dfpp(NATS), rmult(NATS)
      real pa(6,48,IBV,NATS), apos(3,48,NATS), tp(3,48,NATS)
      real sumxr(11,NATS), sumxi(11,NATS), h(3), atom(14,NATS), q(3)
      real sumdi(2,IBV,NATS), sumdr(2,IBV,NATS), ampl(2,IBV,NATS)


c --- Initialisierung:

      sa  = 0.
      sb  = 0.
      sc  = 0.
      sd  = 0.

C --- Schleife ueber die Atome der asymmetrischen Einheit ------------

      do 100 k=1,natom

c --- Summen auf Null setzen:
 
         sumcos = 0.
         sumsin = 0.
         do 22 m=1,nbas(k)
            do 21 j=1,2
               djcos(j,m) = 0.
               djsin(j,m) = 0.
               dpcos(j,m) = 0.
               dpsin(j,m) = 0.
21          continue
22       continue 

c --- Formfaktor ff0 fuer das k-te Atom berechnen:

         if(ixn.eq.1) then
           ff0 = ac(9,ptr(k))
           do 25 i=1,4
              ff0 = ff0 + ac(2*i-1,ptr(k)) * exp(-ac(2*i,ptr(k))*sinlam)
25         continue
         else
           ff0 = 0.
         end if

c --- Besetzungszahl atom(5,k), isotroper Temperaturfaktor atom(4,k),
c     auf die Zahl der Symm.-Karten norm. Wyckoff-Multiplizitaet rmult(k),
c     Formfaktor ff0 + df' + df'':

         bwst  = atom(5,k) * exp(-atom(4,k)*sinlam) !  * rmult(k)
         facs  = bwst * (ff0 + dfp(ptr(k)))
         facss = bwst * dfpp(ptr(k))


c --- Schleife ueber die symmetrie-aequivalenten Lagen zu Atom K -----

         do 60 ns=1,npos(k)

c --- Phasenverschiebungen q * tp bestimmen:

            phs = 0.
            do 27 i=1,3
               phs = phs + tp(i,ns,k) * q(i)
27          continue

c --- Skalarprodukt hs * atom berechnen:

            scala = 0.
            do 30 i=1,3
               scala = scala + h(i) * apos(i,ns,k)
30          continue
            scala = PII * scala

c --- Skalarprodukte der Auslenkungs-Amplituden mit dem reziproken 
c     Gittervektor und ueber alle Moden aufsummieren; Ableitungen
c     von zr und zi nach den Amplituden und Phasenverschiebungen:

            zr = 0.
            zi = 0.
            do 41 m=1,nbas(k)
               sumr = 0.
               sumi = 0.
               do 40 i=1,3
                  arg = pa(3+i,ns,m,k) + ampl(2,m,k)
                  sumr = sumr + h(i) * pa(i,ns,m,k) * cos(arg)
                  sumi = sumi + h(i) * pa(i,ns,m,k) * sin(arg)
40             continue
               sumr = PII * sumr
               sumi = PII * sumi
               zr = zr + ampl(1,m,k) * sumr 
               zi = zi + ampl(1,m,k) * sumi 
               dzr(1,m) = sumr
               dzi(1,m) = sumi
               dzr(2,m) = -sumi * ampl(1,m,k) 
               dzi(2,m) =  sumr * ampl(1,m,k) 
41          continue

c --- Argument w der Besselfunktion bestimmen:

            zz = zr*zr + zi*zi
            if(zz.le.0.) then
              w = 0.
            else
              w = sqrt(zz)
            end if
                              
c --- Phase phi bestimmen:

            if(zi.eq.0.) then
              if(zr.ge.0.) then
                phi = 0.
              else
                phi = PI
              end if
            else
              phi = atan2(zi,zr)
            end if      

c --- n-te Besselfunktion zum Argument w berechnen:

            bessel = bessfkt(-nu  ,w)
            bs1    = bessfkt(-nu-1,w)
            bs2    = bessfkt(-nu+1,w)

c --- Cosinus und Sinus der Phase des Strukturfaktors:

            arg = scala - nu * (phs + phi + PI2)
            fcos = cos(arg)
            fsin = sin(arg)

            bescos = bessel * fcos
            bessin = bessel * fsin

c --- Ableitungen von phi und bessel nach den Modulationsparametern:

            if(w.gt.0.) then
              do 45 m=1,nbas(k)
                 do 44 j=1,2
                    dbess = 0.5 * (bs1 - bs2) * 
     *                      (zr*dzr(j,m) + zi*dzi(j,m))/w
                    dphi  = -nu * (zr*dzi(j,m) - zi*dzr(j,m))/w/w
                    djcos(j,m) = djcos(j,m) + dbess * fcos
                    djsin(j,m) = djsin(j,m) + dbess * fsin 
                    dpcos(j,m) = dpcos(j,m) + dphi * bescos
                    dpsin(j,m) = dpsin(j,m) + dphi * bessin
44               continue
45            continue
            end if

c --- Cosinuse und Sinuse aufsummieren:

            sumcos = sumcos + bescos
            sumsin = sumsin + bessin

60       continue


c --- Ableitungen abspeichern:

         do 68 j=4,5
            sumxr(j,k) = sumxr(j,k) + sumcos*facs - sumsin*facss
            sumxi(j,k) = sumxi(j,k) + sumsin*facs + sumcos*facss
68       continue

         do 75 m=1,nbas(k)
            do 70 j=1,2
               sumdr(j,m,k) = sumdr(j,m,k) +
     *                        (djcos(j,m) - dpsin(j,m)) * facs -
     *                        (djsin(j,m) + dpcos(j,m)) * facss
               sumdi(j,m,k) = sumdi(j,m,k) +
     *                        (djsin(j,m) + dpcos(j,m)) * facs +
     *                        (djcos(j,m) - dpsin(j,m)) * facss
70          continue
75       continue


c --- Strukturfaktor berechnen:

         sa = sa + sumcos * facs
         sb = sb + sumsin * facs
         sc = sc + sumcos * facss
         sd = sd + sumsin * facss


100   continue 

c ----------------------------------------------------------------------


c --- Realteil, Imaginaerteil und Betragsquadrat des Strukturfaktors
c     berechnen:

      sf_real = sa - sd
      sf_imag = sb + sc


      return
      end
c******************************************************************************
c***            Levenberg-Marquardt-Fitroutine                              ***
c******************************************************************************

      subroutine mrqmin(jobtyp,idges,ass,ixn,nphase,nprof,ibac,
     *                  rlamda,ratio,wdt,rlim,pref,ipref,gitter,
     *                  ibeta,imo,ism,tran,nprim,nsym,icent,
     *                  ac,dfp,dfpp,ptr,excl,iexcl,IEZ,
     *                  glb,aglb,lglb,par,apar,lpar,IDAT,LPHASE,
     *                  atext,atom,aato,lato,rmult,natom,NATS,
     *                  ampl,aampl,lampl,apos,tp,pa,npos,nbas,
     *                  numbsv,IBV,
     *                  the,obs,calc,gew,bac,mit,krr,izahl,IDZ,
     *                  iref,theta,hwb,rlopo,ffreal,ffimag,rint,
     *                  nuq,aint,lint,ic,IRS,NORD,
     *                  covar,alpha,beta,da,dp,ds,deriv,dersto,ipar,
     *                  indxc,indxr,ipiv,maxs,MSZ,
     *                  ix,mcycle,icorr,relax,eps,
     *                  param,out,iout,MAPA,MCL,
     *                  sb1,sb2,sb3,sw1,sw2,num,
     *                  swobs,chisq,rpm,rwpm,numm,
     *                  sumxi,sumxr,sumdr,sumdi,sumda,sumdb)

      implicit none
      integer MAPA, MSZ, IDZ, IDAT, NORD, IBV, NATS, LPHASE, 
     *        IRS, IEZ, MCL
      integer jobtyp, idges, maxs, ipref, num, iout, mcycle, icorr
      integer mit(IDZ), krr(2,LPHASE,IDZ) 
      integer iexcl(IDAT), ixn(IDAT), nphase(IDAT), nprof(IDAT),
     *        ibac(IDAT), izahl(IDAT), numm(IDAT)
      integer ibeta(LPHASE), imo(LPHASE), ism(3,3,48,LPHASE), 
     *        nprim(48,LPHASE), nsym(LPHASE), icent(LPHASE)
      integer natom(LPHASE), ptr(NATS,IDAT), nbas(NATS),
     *        numbsv(IBV,NATS), npos(NATS)
      integer iref(7,IRS,IDAT), ic(LPHASE,IDAT),
     *        nuq(2*NORD+1,IRS,IDAT)
      integer lglb(10,IDAT), lpar(23,LPHASE,IDAT), lato(14,NATS),
     *        lampl(2,IBV,NATS), lint(IRS), ipar(MSZ,IDAT)
      integer indxc(MSZ), indxr(MSZ), ipiv(MSZ)
      real deriv(MSZ), dersto(MSZ,IRS), covar(MSZ,MSZ), da(MSZ), 
     *     relax(8), eps, out(MAPA,MCL),
     *     beta(MSZ), dp(MSZ), ds(MSZ), alpha(MSZ,MSZ)
      real pref(3,LPHASE,IDAT), gitter(6,LPHASE,IDAT) 
      real excl(2,IEZ,IDAT), rmult(NATS)
      real rlamda(2,IDAT), ratio(2,IDAT), wdt(IDAT), rlim(IDAT)
      real glb(10,IDAT), par(23,LPHASE,IDAT), atom(14,NATS),
     *     ampl(2,IBV,NATS)
      real aglb(10,IDAT), apar(23,LPHASE,IDAT), aato(14,NATS),
     *     aampl(2,IBV,NATS), aint(IRS)
      real apos(3,48,NATS), tp(3,48,NATS), pa(6,48,IBV,NATS),
     *     tran(3,48,LPHASE) 
      real ac(9,NATS,IDAT), dfp(NATS,IDAT), dfpp(NATS,IDAT) 
      real the(IDZ), obs(IDZ), calc(IDZ), gew(IDZ), bac(IDZ)
      real theta(IRS,IDAT), rlopo(IRS,IDAT), hwb(IRS,IDAT), 
     *     ffreal(IRS,IDAT), ffimag(IRS,IDAT), rint(IRS,IDAT)
      real sumxr(11,NATS), sumxi(11,NATS), sumdr(2,IBV,NATS),
     *     sumdi(2,IBV,NATS), sumda(3,14,NATS), sumdb(3,14,NATS)
      real swobs(IDAT), chisq(IDAT), rpm(IDAT), rwpm(IDAT)
      real sb1, sb2, sb3, sw1, sw2
      character atext(NATS)*4, param(MAPA)*7
      logical ass(IDAT)

      integer ix, j, k, l, ks(20), ia, lim
      real zl, alamda, yy


c --- Initialisierung:

      do 10 k=1,maxs
         ds(k) = 0.0
 10   continue

      do 11 j=1,idges
         ass(j) = .true.
 11   continue

      alamda = 0.001

      call mrqcof(jobtyp,idges,ass,ixn,nphase,nprof,ibac,
     *            rlamda,ratio,wdt,rlim,pref,ipref,gitter,
     *            ibeta,imo,ism,tran,nprim,nsym,icent,
     *            ac,dfp,dfpp,ptr,excl,iexcl,IEZ,
     *            glb,aglb,lglb,par,apar,lpar,IDAT,LPHASE,
     *            atom,aato,lato,rmult,natom,NATS,
     *            ampl,aampl,lampl,apos,tp,pa,npos,nbas,numbsv,IBV,
     *            the,obs,calc,gew,bac,mit,krr,izahl,IDZ,
     *            iref,theta,hwb,rlopo,ffreal,ffimag,rint,
     *            nuq,aint,lint,ic,IRS,NORD,
     *            alpha,beta,deriv,dersto,ipar,maxs,MSZ,
     *            sb1,sb2,sb3,sw1,sw2,num,
     *            swobs,chisq,rpm,rwpm,numm,
     *            sumxi,sumxr,sumdr,sumdi,sumda,sumdb)
      zl = sw2

c === Schleife ueber die Iterationszyklen:

      do 100 ix=1,mcycle

c --- Alter linearized fitting matrix, by augmentating diagonal elements:

         do 14 j=1,maxs
            do 13 k=1,maxs
               covar(j,k) = alpha(j,k)
 13         continue
            covar(j,j) = alpha(j,j) * (1. + alamda)
            da(j) = beta(j)
 14      continue

c --- Gleichungssystem loesen:

         call gaussj(covar,da,indxc,indxr,ipiv,maxs,MSZ)

c --- Kovarianz-Matrix "covar", Standardabweichungen "dp" und 
c     Verschiebungen "ds" berechnen:

         do 1211 j=1,maxs
            do 1210 k=1,maxs
               covar(j,k) = alpha(j,k)
 1210       continue
            ds(j) = da(j)
            da(j) = beta(j)
 1211    continue

         call gaussj(covar,da,indxc,indxr,ipiv,maxs,MSZ)

         do 15 j=1,maxs
            dp(j) = sqrt(abs(covar(j,j)))
 15      continue

c --- Parameter um "da" verschieben:

         call parshift(jobtyp,idges,ass,imo,relax,nphase,rlamda,
     *                 glb,aglb,lglb,par,apar,lpar,LPHASE,IDAT,
     *                 atom,aato,lato,natom,NATS,
     *                 ampl,aampl,lampl,nbas,IBV,
     *                 rint,aint,lint,ic,IRS,
     *                 ds,maxs,MSZ)

c --- Profil und R-Werte berechnen:

         call mrqcof(jobtyp,idges,ass,ixn,nphase,nprof,ibac,
     *               rlamda,ratio,wdt,rlim,pref,ipref,gitter,
     *               ibeta,imo,ism,tran,nprim,nsym,icent,
     *               ac,dfp,dfpp,ptr,excl,iexcl,IEZ,
     *               glb,aglb,lglb,par,apar,lpar,IDAT,LPHASE,
     *               atom,aato,lato,rmult,natom,NATS,
     *               ampl,aampl,lampl,apos,tp,pa,npos,nbas,numbsv,IBV,
     *               the,obs,calc,gew,bac,mit,krr,izahl,IDZ,
     *               iref,theta,hwb,rlopo,ffreal,ffimag,rint,
     *               nuq,aint,lint,ic,IRS,NORD,
     *               covar,da,deriv,dersto,ipar,maxs,MSZ,
     *               sb1,sb2,sb3,sw1,sw2,num,
     *               swobs,chisq,rpm,rwpm,numm,
     *               sumxi,sumxr,sumdr,sumdi,sumda,sumdb)

c --- Parameter und R-Werte ausgeben:

         call outcyc(ix,relax,ds,dp,maxs,MSZ,
     *               param,out,iout,MAPA,MCL,
     *               jobtyp,idges,imo,nphase,nprof,gitter,rlamda,
     *               glb,aglb,lglb,par,apar,lpar,IDAT,LPHASE,
     *               atext,atom,aato,lato,natom,NATS,
     *               ampl,aampl,lampl,nbas,numbsv,IBV,
     *               iref,rint,aint,lint,ic,IRS,
     *               chisq,swobs,rpm,rwpm,numm,
     *               sw1,sw2,sb2,sb3,num)     

c --- Test for convergence:

         if(sw2.lt.zl)then

            zl = sw2
            alamda = 0.1 * alamda

            do 17 j=1,maxs
               do 16 k=1,maxs
                  alpha(j,k) = covar(j,k)
 16            continue
               beta(j) = da(j)
 17         continue

         else

            alamda = 10. * alamda

c --- Parameter um "-da" zurueckschieben:
      
            do 18 k=1,maxs
               ds(k) = -ds(k)
 18         continue

            call parshift(jobtyp,idges,ass,imo,relax,nphase,rlamda,
     *                    glb,aglb,lglb,par,apar,lpar,LPHASE,IDAT,
     *                    atom,aato,lato,natom,NATS,
     *                    ampl,aampl,lampl,nbas,IBV,
     *                    rint,aint,lint,ic,IRS,
     *                    ds,maxs,MSZ)

         endif

c --- Rietveld'sche Abbruchbedingung:

         do 19 j=1,maxs
            if(abs(ds(j)).gt.dp(j)*eps) goto 100
 19      continue
         goto 200

 100  continue

c --- Ende der Iteration:

 200  if(ix.gt.mcycle) then
        ix = mcycle
      else if(ix.lt.mcycle) then
        write(*,'(/2x,a,/2x,a)') 'Forced termination',
     *                           '=================='
      end if
      write(*,'(/2x,a,i4)') 'Cycle number = ',ix

c --- Kovarianz-Matrix berechnen:

      do 211 j=1,maxs
         do 210 k=1,maxs
            covar(j,k) = alpha(j,k)
 210     continue
         da(j) = beta(j)
 211  continue

      call gaussj(covar,da,indxc,indxr,ipiv,maxs,MSZ)

c --- Standardabweichungen:

      do 220 j=1,maxs
         dp(j) = sqrt(abs(covar(j,j)))
 220  continue

c --- List correlations:

      if(icorr.eq.1) then
         ia = 1
         lim = 12
         write(*,'(//2x,a)') 'Correlations among the variables: '
 385     lim = min0(maxs,lim)
         write(*,'(/2x,a,12i6)') 'Va.',(k,k=ia,lim)
         do 395 k=1,maxs
            j = 0
            do 390 l=ia,lim
               j = j + 1
               yy = covar(l,l) * covar(k,k)
	       ks(j) = nint(100. * 
     *              covar(k,l) / (sqrt(abs(yy)) * sign(1.,yy)))
 390        continue
            write(*,'(i5,12i6)') k,(ks(l),l=1,j)
 395     continue
         if(lim.lt.maxs) then
            ia = lim + 1
            lim = lim + 12
            goto 385
         end if
      endif


      return
      end

c******************************************************************************
c***              Profil und R-Werte berechnen                              ***
c******************************************************************************

      subroutine mrqcof(jobtyp,idges,ass,ixn,nphase,nprof,ibac,
     *                  rlamda,ratio,wdt,rlim,pref,ipref,gitter,
     *                  ibeta,imo,ism,tran,nprim,nsym,icent,
     *                  ac,dfp,dfpp,ptr,excl,iexcl,IEZ,
     *                  glb,aglb,lglb,par,apar,lpar,IDAT,LPHASE,
     *                  atom,aato,lato,rmult,natom,NATS,
     *                  ampl,aampl,lampl,apos,tp,pa,npos,nbas,
     *                  numbsv,IBV,
     *                  the,obs,calc,gew,bac,mit,krr,izahl,IDZ,
     *                  iref,theta,hwb,rlopo,ffreal,ffimag,rint,
     *                  nuq,aint,lint,ic,IRS,NORD,
     *                  alpha,beta,deriv,dersto,ipar,maxs,MSZ,
     *                  sb1,sb2,sb3,sw1,sw2,num,
     *                  swobs,chisq,rpm,rwpm,numm,
     *                  sumxi,sumxr,sumdr,sumdi,sumda,sumdb)

      implicit none
      external pkshift
      integer i, ii, ip, id, iod, iof, ibeg, iend, j, k
      real dy, wt, ycalc, thecor, pkshift, xmin, xmax
      real sbcal, sbobs, sbdif

      integer MSZ, IDZ, IDAT, NORD, IBV, NATS, LPHASE, IRS, IEZ
      integer jobtyp, idges, maxs, ipref, numm(IDAT), num
      integer iexcl(IDAT), ixn(IDAT), nphase(IDAT), nprof(IDAT),
     *        ibac(IDAT), izahl(IDAT), mit(IDZ)
      integer ibeta(LPHASE), imo(LPHASE), ism(3,3,48,LPHASE), 
     *        nprim(48,LPHASE), nsym(LPHASE), icent(LPHASE)
      integer natom(LPHASE), ptr(NATS,IDAT), nbas(NATS),
     *        numbsv(IBV,NATS), npos(NATS)
      integer iref(7,IRS,IDAT), ic(LPHASE,IDAT),
     *        nuq(2*NORD+1,IRS,IDAT), krr(2,LPHASE,IDZ) 
      integer lglb(10,IDAT), lpar(23,LPHASE,IDAT), lato(14,NATS),
     *        lampl(2,IBV,NATS), lint(IRS), ipar(MSZ,IDAT)
      real alpha(MSZ,MSZ), beta(MSZ), dersto(MSZ,IRS), deriv(MSZ)
      real pref(3,LPHASE,IDAT), gitter(6,LPHASE,IDAT) 
      real excl(2,IEZ,IDAT), rmult(NATS)
      real rlamda(2,IDAT), ratio(2,IDAT), wdt(IDAT), rlim(IDAT)
      real glb(10,IDAT), par(23,LPHASE,IDAT), atom(14,NATS),
     *     ampl(2,IBV,NATS)
      real aglb(10,IDAT), apar(23,LPHASE,IDAT), aato(14,NATS),
     *     aampl(2,IBV,NATS), aint(IRS)
      real apos(3,48,NATS),tp(3,48,NATS), pa(6,48,IBV,NATS), 
     *     tran(3,48,LPHASE) 
      real ac(9,NATS,IDAT), dfp(NATS,IDAT), dfpp(NATS,IDAT) 
      real the(IDZ), obs(IDZ), calc(IDZ), gew(IDZ), bac(IDZ)
      real theta(IRS,IDAT), rlopo(IRS,IDAT), hwb(IRS,IDAT), 
     *     ffreal(IRS,IDAT), ffimag(IRS,IDAT), rint(IRS,IDAT)
      real sumxr(11,NATS), sumxi(11,NATS), sumdr(2,IBV,NATS),
     *     sumdi(2,IBV,NATS), sumda(3,14,NATS), sumdb(3,14,NATS)
      real swobs(IDAT), chisq(IDAT), rpm(IDAT), rwpm(IDAT)
      real sb1, sb2, sb3, sw1, sw2
      logical ass(IDAT)

c --- Initialisierung der Matrix "alpha" und des Vektors "beta":

      do 13 j=1,maxs
         do 12 k=1,j
            alpha(j,k) = 0.0
 12      continue
         beta(j) = 0.0
 13   continue

c --- Initialisierung der Gesamt-R-Werte:

      sb1 = 0.0
      sb2 = 0.0
      sb3 = 0.0
      sw1 = 0.0
      sw2 = 0.0
      num = 0

      iod = 0
      do 200 id=1,idges

         if(id.gt.1) iod = iod + izahl(id-1)

c --- Intensitaeten berechnen:

         iof = 1
         iend = 0
         do 150 ip=1,nphase(id)
            ibeg = iend + 1
            iend = iend + ic(ip,id)
            if(jobtyp.eq.1) then
               if(ip.gt.1) iof = iof + natom(ip-1)
               call strucalc(ixn(id),rlamda(1,id),ratio(1,id),
     *              pref(1,ip,id),ipref,gitter(1,ip,id),
     *              ac(1,1,id),dfp(1,id),dfpp(1,id),ptr(iof,id),
     *              imo(ip),ibeta(ip),icent(ip),ism(1,1,1,ip),
     *              tran(1,1,ip),nprim(1,ip),nsym(ip),
     *              aglb(1,id),lglb(1,id),par(1,ip,id),apar(1,ip,id),
     *              lpar(1,ip,id),
     *              atom(1,iof),aato(1,iof),lato(1,iof),rmult(iof),
     *              natom(ip),NATS,
     *              ampl(1,1,iof),aampl(1,1,iof),lampl(1,1,iof),
     *              pa(1,1,1,iof),apos(1,1,iof),tp(1,1,iof),npos(iof),
     *              nbas(iof),IBV,
     *              iref(1,ibeg,id),theta(ibeg,id),hwb(ibeg,id),
     *              rlopo(ibeg,id),ffreal(ibeg,id),ffimag(ibeg,id),
     *              rint(ibeg,id),nuq(1,ibeg,id),ic(ip,id),NORD,IRS,
     *              dersto(1,ibeg),deriv,maxs,MSZ,
     *              sumxr,sumxi,sumdr,sumdi,sumda,sumdb)
            else
               call refcalc(maxs,dersto(1,ibeg),deriv,rlamda(1,id),
     *              par(8,ip,id),par(9,ip,id),par(10,ip,id),
     *              par(11,ip,id),par(14,ip,id),apar(1,ip,id),
     *              lpar(1,ip,id),aglb(1,id),lglb(1,id),aint(ibeg),
     *              lint(ibeg),iref(1,ibeg,id),theta(ibeg,id),
     *              hwb(ibeg,id),rint(ibeg,id),ic(ip,id),IRS,MSZ)
            end if
 150     continue

c --- Minimum und Maximum in 2-Theta korrigieren:
            
         xmin = the(iod+1) + pkshift(the(iod+1),glb(1,id),3)
         xmax = the(iod+izahl(id)) + 
     *          pkshift(the(iod+izahl(id)),glb(1,id),3)

c --- Reflexe den Messpunkten zuordnen und "excluded regions"
c     beruecksichtigen:

         if(ass(id)) then 
            call assign(the(iod+1),krr(1,1,iod+1),mit(iod+1),
     *           izahl(id),IDZ,LPHASE,glb(1,id),wdt(id),
     *           nphase(id),xmin,xmax,excl(1,1,id),iexcl(id),
     *           IEZ,theta(1,id),hwb(1,id),ic(1,id),IRS)
         end if
            
c --- Profil berechnen:

         do 190 i=1,izahl(id)
            
            ii = iod + i

c --- Ableitungen auf Null setzen:

            do 160 k=1,maxs
               deriv(k) = 0.
 160        continue

c --- Peak-shift correction:

            thecor = the(ii) + pkshift(the(ii),glb(1,id),3)

c --- Nur Bereiche ausserhalb der "excluded regions" beruecksichtigen:

            if(mit(ii).ge.0) then

c --- Untergrund an der Stelle thecor und Ableitungen nach den
c     Untergrund-Parametern berechnen:

               if(ibac(id).eq.0) then
                  call backgr(bac(ii),thecor,xmin,xmax,glb(1,id),
     *                        lglb(1,id),aglb(1,id),deriv,maxs,MSZ)
               end if

c --- Profil ycalc und Ableitungen berechnen:

               if(mit(ii).eq.1) then
                  call calcxn(ycalc,the(ii),thecor,krr(1,1,ii),
     *                        nprof(id),nphase(id),LPHASE,natom,
     *                        rlim(id),lglb(1,id),par(1,1,id),
     *                        lpar(1,1,id),lato,lampl,nbas,
     *                        IBV,NATS,dersto,deriv,maxs,MSZ,
     *                        theta(1,id),hwb(1,id),rint(1,id),
     *                        lint,IRS)
               else
                  ycalc = 0.0
               end if

c --- Gesamt-Profil abspeichern:

               calc(ii) = ycalc + bac(ii)

c --- Matrix covar und den Vektor dz berechnen:

               if(ibac(id).eq.0.or.
     *              (ibac(id).gt.0.and.mit(ii).eq.1)) then
                  dy = obs(ii) - calc(ii)
                  do 180 j=1,maxs
                     if(ipar(j,id).eq.j) then
                        wt = deriv(j) / gew(ii)
                        beta(j) = beta(j) + dy * wt
                        do 170 k=1,j
                           if(ipar(k,id).eq.k) then
                              alpha(j,k) = 
     *                             alpha(j,k) + wt * deriv(k)
                           endif
 170                    continue
                     endif
 180              continue
               end if

c --- Excluded regions (mit = -1):

            else

               calc(ii) = 0.

            end if

 190     continue

c --- R-Werte berechnen:

         call rwert(ibac(id),obs(iod+1),calc(iod+1),gew(iod+1),
     *              bac(iod+1),mit(iod+1),izahl(id),IDZ,
     *              sbcal,sbobs,sbdif,swobs(id),chisq(id),
     *              rpm(id),rwpm(id),numm(id))
         num = num + numm(id)
         sb1 = sb1 + sbcal
         sb2 = sb2 + sbobs
         sb3 = sb3 + sbdif
         sw1 = sw1 + swobs(id)
         sw2 = sw2 + chisq(id)

 200  continue

c --- Symmetrischer Teil der Matrix "alpha" auffuellen:

      do 218 j=2,maxs
         do 217 k=1,j-1
            alpha(k,j) = alpha(j,k)
 217     continue
 218  continue

      return
      end

c******************************************************************************
c***    Parameter und R-Werte nach jedem Zyklus am Bildschirm ausgeben      ***
c***                 und im CYCLE-File abspeichern                          ***
c******************************************************************************

      subroutine outcyc(icycle,relax,ds,dp,maxs,MSZ,
     *                  param,out,iout,MAPA,MCL,
     *                  jobtyp,idges,imo,nphase,nprof,gitter,rlamda,
     *                  glb,aglb,lglb,par,apar,lpar,IDAT,LPHASE,
     *                  atext,atom,aato,lato,natom,NATS,
     *                  ampl,aampl,lampl,nbas,numbsv,IBV,
     *                  iref,rint,aint,lint,ic,IRS,
     *                  chisq,swobs,rpm,rwpm,numm,
     *                  sw1,sw2,sb2,sb3,num)     

      implicit none
      integer i, id, ip, ii, iof, ibeg, iend, j, km, m, na 
      real shift, stdd, vol, dvol, h(3)
      real celld(6), delta(6), sd(6), gof, rsig, rwp, rp, rexp
      character tglb(10)*10, cgl(10)*7, cpar(23)*10, cpa(23)*7
      character cato(14)*10, cats(14)*4, date*9, time*8, campl(2)*3
      character cmod(2)*1

      integer jobtyp, icycle, maxs, idges, num, iout
      integer IDAT, LPHASE, NATS, IBV, MAPA, MSZ, IRS, MCL
      integer numm(IDAT), numbsv(IBV,NATS), nbas(IBV)
      integer iref(7,IRS,IDAT), ic(LPHASE,IDAT)
      integer lglb(10,IDAT), lpar(23,LPHASE,IDAT), lato(14,NATS),
     *        lampl(2,IBV,NATS), lint(IRS)
      integer nphase(IDAT), nprof(IDAT), natom(LPHASE), imo(LPHASE)
      real ds(MSZ), dp(MSZ), out(MAPA,MCL), relax(8)
      real gitter(6,LPHASE,IDAT), rlamda(2,IDAT)
      real glb(10,IDAT), par(23,LPHASE,IDAT), atom(14,NATS),
     *     ampl(2,IBV,NATS), rint(IRS,IDAT)
      real aglb(10,IDAT), apar(23,LPHASE,IDAT), aato(14,NATS),
     *     aampl(2,IBV,NATS), aint(IRS)
      real swobs(IDAT), chisq(IDAT), rpm(IDAT), rwpm(IDAT)
      real sw1, sw2, sb2, sb3
      character atext(NATS)*4, param(MAPA)*7

c --- Texte fuer die Ausgabe:

      data tglb /'Zeropoint ','t1        ','t2        ',
     *           'c0        ','c1        ','c2        ',
     *           'c3        ','c4        ','c5        ',
     *           'Wavelength'/
      data cgl /' Zero  ',' t1    ',' t2    ',' c0    ',' c1    ',
     *          ' c2    ',' c3    ',' c4    ',' c5    ','Lambda '/
  
      data cato /'        x ','        y ','        z ',
     *           'Iso-Temp. ','Occupancy ',
     *           '      b11 ','      b22 ','      b33 ',
     *           '      b12 ','      b13 ','      b23 ',
     *           '       Mx ','       My ','       Mz '/
      data cats /',x  ',',y  ',',z  ',',B  ',',N  ',
     *           ',b11',',b22',',b33',',b12',',b13',',b23',
     *           ',Mx ',',My ',', Mz'/

      campl(1) = '  u'
      campl(2) = 'phi'
      cmod(1)  = 'u'
      cmod(2)  = 'p'

      cpar(3)  = 'Eta2      '
      cpar(4)  = 'Scale     '
      cpar(5)  = 'O-Temp.   '
      cpar(6)  = 'G1        '
      cpar(7)  = 'G2        '
      cpar(8)  = 'U         '
      cpar(9)  = 'V         '
      cpar(10) = 'W         '
      cpar(11) = 'q1        '
      cpar(12) = 'q2        '
      cpar(13) = 'q3        '
      cpar(14) = 'A         '
      cpar(15) = 'B         '
      cpar(16) = 'C         '
      cpar(17) = 'Alpha     '
      cpar(18) = 'Beta      '
      cpar(19) = 'Gamma     '
      cpar(22) = 'A1        '
      cpar(23) = 'B1        '

      cpa(3)   = ' Eta2  '
      cpa(4)   = 'Scale  '
      cpa(5)   = 'O-Temp '
      cpa(6)   = ' G1    '
      cpa(7)   = ' G2    '
      cpa(8)   = '  U    '
      cpa(9)   = '  V    '
      cpa(10)  = '  W    '
      cpa(11)  = ' q1    '
      cpa(12)  = ' q2    '
      cpa(13)  = ' q3    '
      cpa(14)  = '  A    '
      cpa(15)  = '  B    '
      cpa(16)  = '  C    '
      cpa(17)  = 'Alpha  '
      cpa(18)  = ' Beta  '
      cpa(19)  = 'Gamma  '
      cpa(22)  = ' A1    '
      cpa(23)  = ' B1    '

c --- Formate:

 157  format(2x,a10,13x,i2,6x,3f10.5)
 156  format(2x,a10,13x,i2,2x,i2,2x,3f10.5)
 153  format(2x,a4,1x,a3,i3,16x,i2,2x,3f10.5)
 154  format(2x,a4,1x,a10,12x,i2,2x,3f10.5)

 112  format(2x,a,4i4,a,8x,i2,2x,3f10.5)
 113  format(2x,a,3i4,a,12x,i2,2x,3f10.5)
 114  format(2x,a,3f6.1,a, 6x,i2,2x,3f10.5)

      iout = 0

      write(*,'(//2x,a,i3,a)') 'Fit in cycle',icycle,':'
      write(*,'(2x,a/)')       '================'

c --- Standardabweichung (stdd), Verschiebung (shift) und neuer Parameter
c     ausgeben (km: Codeword):

      if(maxs.gt.0) write(*,'(25x,a)') 
     *     'ds phs   parameter    shift      stdd.'

      do 200 id=1,idges

         if(nprof(id).eq.9) then
            cpar(20) = 'S/L       '
            cpar(21) = 'H/L       '
            cpa(20)  = ' S/L   '
            cpa(21)  = ' H/L   '
         else
            cpar(20) = 'A0        '
            cpar(21) = 'B0        '
            cpa(20)  = ' A0    '
            cpa(21)  = ' B0    '
         end if

c --- Peak-shift parameters, background coefficients, wavelength:

         if(nprof(id).eq.5) then
           cpar(1) = 'Skewness  '
           cpa(1)  = ' Skew  '
           cpar(2) = 'Excess    '
           cpa(2)  = ' Exce  '
         else if(nprof(id).eq.6.or.nprof(id).eq.9) then
           cpar(1) = 'Eta0      '
           cpa(1)  = ' Eta0  '
           cpar(2) = 'Eta1      '
           cpa(2)  = ' Eta1  '
         else if(nprof(id).eq.7) then
           cpar(1) = 'M0        '
           cpa(1)  = ' M0    '
           cpar(2) = 'M1        '
           cpa(2)  = ' M1    '
         end if

         do 10 j=1,3
            km = lglb(j,id)
            if(km.ne.0.and.km.le.maxs) then

               stdd = dp(km) * abs(aglb(j,id))
               shift = ds(km) * aglb(j,id) * relax(7)
               write(*,157) tglb(j), id, glb(j,id), shift, stdd

               iout = iout + 1
               param(iout) = cgl(j)
               out(iout,icycle) = glb(j,id)
               
               iout = iout + 1
               param(iout) = 'Delt   '
               out(iout,icycle) = shift
               
               iout = iout + 1
               param(iout) = 'Stdd   '
               out(iout,icycle) = stdd
               
            end if
 10      continue

c --- Background coefficients:

         do 15 j=4,9
            km = lglb(j,id)
            if(km.ne.0.and.km.le.maxs) then

               stdd = dp(km) * abs(aglb(j,id))
               shift = ds(km) * aglb(j,id) * relax(8)
               write(*,157) tglb(j), id, glb(j,id), shift, stdd

               iout = iout + 1
               param(iout) = cgl(j)
               out(iout,icycle) = glb(j,id)
               
               iout = iout + 1
               param(iout) = 'Delt   '
               out(iout,icycle) = shift
               
               iout = iout + 1
               param(iout) = 'Stdd   '
               out(iout,icycle) = stdd
               
            end if
 15      continue


c --- Wavelength lambda 1:

         km = lglb(10,id)
         if(km.ne.0.and.km.le.maxs) then

            stdd = dp(km) * abs(aglb(10,id))
            shift = ds(km) * aglb(10,id) * relax(7)
            write(*,157) tglb(10), id, glb(10,id), shift, stdd

            iout = iout + 1
            param(iout) = cgl(10)
            out(iout,icycle) = glb(10,id)
            
            iout = iout + 1
            param(iout) = 'Delt   '
            out(iout,icycle) = shift
               
            iout = iout + 1
            param(iout) = 'Stdd   '
            out(iout,icycle) = stdd
               
         end if

         rlamda(1,id) = glb(10,id)
 
c --- Parameter der einzelnen Phasen:

         iof = 0
         iend = 0                
         do 100 ip=1,nphase(id)

            ibeg = iend + 1
            iend = iend + ic(ip,id)

c --- Peak shape function parameters:

            do 20 j=1,3
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
               
                  stdd  = dp(km) * abs(apar(j,ip,id))
                  shift = ds(km) * apar(j,ip,id) * relax(6)
                  write(*,156) cpar(j),id,ip,par(j,ip,id),shift,stdd
                  
                  iout = iout + 1
                  param(iout) = cpa(j)
                  out(iout,icycle) = par(j,ip,id)
                  
                  iout = iout + 1
                  param(iout) = 'Delt   '
                  out(iout,icycle) = shift
                  
                  iout = iout + 1
                  param(iout) = 'Stdd   '
                  out(iout,icycle) = stdd
                  
               end if
 20         continue

c --- Scale factor s, overall temperature factor B0,
c     preferred orientation correction parameters G1 and G2:

            do 21 j=4,7
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
               
                  stdd  = dp(km) * abs(apar(j,ip,id))
                  shift = ds(km) * apar(j,ip,id) * relax(4)
                  write(*,156) cpar(j),id,ip,par(j,ip,id),shift,stdd
                  
                  iout = iout + 1
                  param(iout) = cpa(j)
                  out(iout,icycle) = par(j,ip,id)
                  
                  iout = iout + 1
                  param(iout) = 'Delt   '
                  out(iout,icycle) = shift
                  
                  iout = iout + 1
                  param(iout) = 'Stdd   '
                  out(iout,icycle) = stdd
                  
               end if
 21         continue

c --- Halfwidth parameters U, V and W (Caglioti):

            do 22 j=8,10
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
               
                  stdd  = dp(km) * abs(apar(j,ip,id))
                  shift = ds(km) * apar(j,ip,id) * relax(3)
                  write(*,156) cpar(j),id,ip,par(j,ip,id),shift,stdd
                  
                  iout = iout + 1
                  param(iout) = cpa(j)
                  out(iout,icycle) = par(j,ip,id)
                  
                  iout = iout + 1
                  param(iout) = 'Delt   '
                  out(iout,icycle) = shift
                  
                  iout = iout + 1
                  param(iout) = 'Stdd   '
                  out(iout,icycle) = stdd
                  
               end if
 22         continue

c --- Wave vector components:

            do 23 j=11,13
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
               
                  stdd  = dp(km) * abs(apar(j,ip,id))
                  shift = ds(km) * apar(j,ip,id) * relax(5)
                  write(*,156) cpar(j),id,ip,par(j,ip,id),shift,stdd
                  
                  iout = iout + 1
                  param(iout) = cpa(j)
                  out(iout,icycle) = par(j,ip,id)
                  
                  iout = iout + 1
                  param(iout) = 'Delt   '
                  out(iout,icycle) = shift
                  
                  iout = iout + 1
                  param(iout) = 'Stdd   '
                  out(iout,icycle) = stdd
                  
               end if
 23         continue

c --- Asymmetry correction parameters:

            do 24 j=20,23
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
                  
                  stdd  = dp(km) * abs(apar(j,ip,id))
                  shift = ds(km) * apar(j,ip,id) * relax(6)
                  write(*,156) cpar(j),id,ip,par(j,ip,id),shift,stdd
                  
                  iout = iout + 1
                  param(iout) = cpa(j)
                  out(iout,icycle) = par(j,ip,id)
                  
                  iout = iout + 1
                  param(iout) = 'Delt   '
                  out(iout,icycle) = shift
                  
                  iout = iout + 1
                  param(iout) = 'Stdd   '
                  out(iout,icycle) = stdd
                  
               end if
 24         continue

c --- Reziproke Gitterkonstanten (Standardabweichung der reziproken
c     Gitterkonstanten abspeichern):

            do 30 j=14,19
               km = lpar(j,ip,id)
               if(km.ne.0.and.km.le.maxs) then
                  sd(j-13) = dp(km) * abs(apar(j,ip,id))
               end if
 30         continue

c --- Berechnung der direkten Gitterkonstanten "celld" und deren
c     Standardabweichungen "delta":

            call direct(par(14,ip,id),sd,celld,delta,vol,dvol)

c --- Gitterkonstanten A, B, C, Alpha, Beta, Gamma:

            do 40 i=1,6
	       km = lpar(13+i,ip,id)
	       if(km.ne.0.and.km.le.maxs) then

                  shift = celld(i) - gitter(i,ip,id)
                  stdd  = delta(i)
                  gitter(i,ip,id) = celld(i)
                  write(*,156) cpar(13+i),id,ip,celld(i),shift,stdd
                  
                  iout = iout + 1
                  param(iout) = cpa(13+i)
                  out(iout,icycle) = celld(i)
                  
                  iout = iout + 1
                  param(iout) = 'Delt   '
                  out(iout,icycle) = shift
                  
                  iout = iout + 1
                  param(iout) = 'Stdd   '  
                  out(iout,icycle) = stdd
                  
	       end if
 40         continue

c === Atomparameter:

            if(id.eq.1) then
               if(jobtyp.eq.1) then

                  if(ip.gt.1) iof = iof + natom(ip-1)
               
                  do 60 i=1,natom(ip)
                     na = iof + i

c --- Koordinaten x, y, z, isotr. Temperaturfaktor, Besetzungszahl:

                     do 50 j=1,5
                        km = lato(j,na)
                        if(km.ne.0.and.km.le.maxs) then
                           
                           stdd  = dp(km) * abs(aato(j,na))
                           shift = ds(km) * aato(j,na) * relax(1)
                           write(*,154) atext(na), cato(j), ip,
     *                                  atom(j,na), shift, stdd

                           iout = iout + 1
                           param(iout)(1:4) = atext(na)
                           param(iout)(4:7) = cats(j)
                           out(iout,icycle) = atom(j,na)
                        
                           iout = iout + 1
                           param(iout) = 'Delt   '
                           out(iout,icycle) = shift
                        
                           iout = iout + 1
                           param(iout) = 'Stdd   '
                           out(iout,icycle) = stdd

                        end if
 50                  continue

c --- Anisotrope Temperaturfaktoren, magnetische Momente Mx, My, Mz:

                     do 51 j=6,14
                        km = lato(j,na)
                        if(km.ne.0.and.km.le.maxs) then

                           stdd  = dp(km) * abs(aato(j,na))
                           shift = ds(km) * aato(j,na) * relax(2)
                           write(*,154) atext(na), cato(j), ip,
     *                                  atom(j,na), shift, stdd

                           iout = iout + 1
                           param(iout)(1:4) = atext(na)
                           param(iout)(4:7) = cats(j)
                           out(iout,icycle) = atom(j,na)
                       
                           iout = iout + 1
                           param(iout) = 'Delt   '
                           out(iout,icycle) = shift

                           iout = iout + 1
                           param(iout) = 'Stdd   '
                           out(iout,icycle) = stdd

                        end if
 51                  continue

c --- Modulationsparameter:

                     if(imo(ip).eq.1) then
                        do 55 m=1,nbas(na)
                           do 54 j=1,2
                              km = lampl(j,m,na)
                              if(km.ne.0.and.km.le.maxs) then
                                 
                                 stdd = dp(km) * abs(aampl(j,m,na))
                                 shift = ds(km) * 
     *                                   aampl(j,m,na) * relax(2)
                                 write(*,153) atext(na), campl(j), 
     *                                numbsv(m,na), ip, 
     *                                ampl(j,m,na), shift, stdd

                                 iout = iout + 1
                                 param(iout)(1:4) = atext(na)
                                 param(iout)(4:4) = cmod(j)
                                 write(param(iout)(5:7),'(i3.3)') 
     *                                numbsv(m,na)
                                 out(iout,icycle) = ampl(j,m,na)

                                 iout = iout + 1
                                 param(iout) = 'Delt   '
                                 out(iout,icycle) = shift
                             
                                 iout = iout + 1
                                 param(iout) = 'Stdd   '
                                 out(iout,icycle) = stdd
                                 
                              end if
 54                        continue
 55                     continue
                     end if

 60               continue

               else

c --- Intensitaeten:

                  na = 0
                  do 70 j=ibeg,iend
                     na = na + 1
                     km = lint(j)
                     if(km.ne.0.and.km.le.maxs) then

                        stdd = dp(km) * abs(aint(j))
                        shift  = ds(km) * aint(j) * relax(4)
                        if(imo(ip).eq.1) then
                           write(*,112) 'I(',iref(1,j,id),iref(2,j,id),
     *                          iref(3,j,id),iref(4,j,id),')',ip,
     *                          rint(j,id),shift,stdd
                        else if(imo(ip).eq.2) then
                           call hvec(iref(1,j,id),iref(2,j,id),
     *                          iref(3,j,id),iref(4,j,id),
     *                          par(11,ip,id),h)
                           write(*,114) 'I(',h(1),h(2),h(3),')',
     *                          ip, rint(j,id), shift, stdd
                        else
                           write(*,113) 'I(',iref(1,j,id),iref(2,j,id),
     *                          iref(3,j,id),')',ip, 
     *                          rint(j,id), shift, stdd
                        end if

                        iout = iout + 1
                        param(iout) = 'I      '
                        write(param(iout)(2:5),'(i4)') na
                        out(iout,icycle) = rint(j,id)

                        iout = iout + 1
                        param(iout) = 'Delt   '
                        out(iout,icycle) = shift

                        iout = iout + 1
                        param(iout) = 'Stdd   '
                        out(iout,icycle) = stdd

                     end if
 70               continue

               end if
            end if

 100     continue
         
 200  continue
      

c === R-Werte:

      write(*,*)
      call datum(date,time)              
      write(*,'(2x,a,i4,27x,a9,4x,a8)') 'Fit in cycle ',icycle,date,time 
      write(*,*)' ---------------------------------------------------',
     *'--------------'

c --- R-Werte pro Datensatz:

      do 210 id=1,idges

c --- Profil-R-Wert Rpm(id):

  	 iout = iout + 1
	 param(iout) = 'Rp     '
	 write(param(iout)(3:4),'(i2)') id
	 out(iout,icycle) = rpm(id)

c --- Gewichteter Profil-R-Wert Rwpm(id):

	 iout = iout + 1
	 param(iout) = 'Rwp    '
	 write(param(iout)(3:4),'(i2)') id
	 out(iout,icycle) = rwpm(id)

c --- Zahl der Freiheitgrade N - P:

	 ii = numm(id) - maxs

c --- Expected R-Value Rexp:

	 iout = iout + 1 
	 rexp = 100. * sqrt(ii/swobs(id))
	 param(iout) = 'Ex     '
	 write(param(iout)(3:4),'(I2)') id
	 out(iout,icycle) = rexp

c --- Goodness-of-Fit Gof:

	 iout = iout + 1 
	 gof = sqrt(chisq(id)/real(ii))
	 param(iout) = 'Gof    '
	 write(param(iout)(3:4),'(I2)') id
	 out(iout,icycle) = gof

c --- N_sigma:

	 iout = iout + 1
         rsig = (chisq(id) - ii)/sqrt(2.*ii)
	 param(iout) = 'Ns     '
	 write(param(iout)(3:4),'(I2)') id
	 out(iout,icycle) = rsig

c --- R-Werte ausgeben:

	 write(*,'(3(a,i2,a,f11.4,2x))')'  Rp ',id,' = ',rpm(id),
     *   ' Rwp ',id,' = ',rwpm(id),' Rexp',id,' = ',rexp
	 write(*,'(a,i2,a,f11.4,2x,a,i5,7x,a,i2,a,f11.4)')
     *   '  Gof',id,' = ',gof,' N-P    = ',ii,'  Nsig',id,' = ',rsig 
	 write(*,*)' ------------------------------------------------',
     *'-----------------'

210   continue


c --- Gesamt-Profil-R-Wert rp:

      iout = iout + 1
      rp = 100. * sb3/sb2
      param(iout) = 'Rp     '
      out(iout,icycle) = rp

c --- Gesamter gewichteter Profil-R-Wert Rwp:

      iout = iout + 1
      rwp = 100. * sqrt(sw2/sw1)
      param(iout) = 'Rwp    '
      out(iout,icycle) = rwp

c --- Zahl der Freiheitsgrade:
 
      ii = num - maxs

c --- Erwarteter Gesamt-R-Wert Rexp:

      iout = iout + 1
      rexp = 100. * sqrt(real(ii)/sw1)
      param(iout) = 'Rexp   '
      out(iout,icycle) = rexp

c --- gesamtes Goodness-of-fit Gof:

      iout = iout + 1
      gof = sqrt(sw2/real(ii))
      param(iout) = 'Gof    '
      out(iout,icycle) = gof

c --- gesamtes N_sigma:

      iout = iout + 1
      rsig = (sw2 - ii)/sqrt(2.*ii)
      param(iout) = 'Nsig   '
      out(iout,icycle) = rsig

c --- Gesamt-Rwerte ausgeben:

      write(*,'(3(a,f11.4,2x))')'  Rp    = ',rp,' Rwp    = ',rwp,
     1' Rexp   = ',rexp
      write(*,'(a,f11.4,2x,a,i5,8x,a,f11.4)')'  Gof   = ',gof,
     1' N-P    = ',ii,' Nsig   = ',rsig


      return
      end


c******************************************************************************
c***              Parameter verschieben                                     ***
c******************************************************************************

      subroutine parshift(jobtyp,idges,ass,imo,relax,nphase,rlamda,
     *                    glb,aglb,lglb,par,apar,lpar,LPHASE,IDAT,
     *                    atom,aato,lato,natom,NATS,
     *                    ampl,aampl,lampl,nbas,IBV,
     *                    rint,aint,lint,ic,IRS,
     *                    ds,maxs,MSZ)

      implicit none 
      integer jobtyp, idges, maxs, IDAT, LPHASE, NATS, IBV, MSZ, IRS
      integer lglb(10,IDAT), lpar(23,LPHASE,IDAT), lato(14,NATS),
     *        lampl(2,IBV,NATS), lint(IRS)
      integer nphase(IDAT), natom(LPHASE), imo(LPHASE), nbas(IBV),
     *        ic(LPHASE,IDAT)
      real relax(8), ds(MSZ)
      real rlamda(2,IDAT)
      real glb(10,IDAT), par(23,LPHASE,IDAT), atom(14,NATS),
     *     ampl(2,IBV,NATS), rint(IRS,IDAT)
      real aglb(10,IDAT), apar(23,LPHASE,IDAT), aato(14,NATS),
     *     aampl(2,IBV,NATS), aint(IRS)
      logical ass(IDAT)

      real rsh, shift
      integer i, id, ip, iof, ibeg, iend, j, km, m, na

      data rsh /0.001/


      do 200 id=1,idges

         ass(id) = .false.
         
c --- Peak-shift parameters:

         do 10 j=1,3
            km = lglb(j,id)
            if(km.ne.0.and.km.le.maxs) then
               shift = ds(km) * aglb(j,id) * relax(7)
               glb(j,id) = glb(j,id) + shift
               if(glb(j,id).eq.0.) then
                  ass(id) = .true.
               else if(abs(shift/glb(j,id)).gt.rsh) then
                  ass(id) = .true.
               end if
            end if
 10      continue

c --- Background parameters:

         do 15 j=4,9
            km = lglb(j,id)
            if(km.ne.0.and.km.le.maxs) then
               shift = ds(km) * aglb(j,id) * relax(8)
               glb(j,id) = glb(j,id) + shift
            end if
 15      continue

c --- Wellenlaenge:

         km = lglb(10,id)
         if(km.ne.0.and.km.le.maxs) then
            shift = ds(km) * aglb(10,id) * relax(7)
            glb(10,id) = glb(10,id) + shift
            if(glb(10,id).eq.0.) then
               ass(id) = .true.
            else if(abs(shift/glb(10,id)).gt.rsh) then
               ass(id) = .true.
            end if
         end if

         rlamda(1,id) = glb(10,id)
 
c --- Parameter der einzelnen Phasen:
                     
         iof = 0
         iend = 0
         do 100 ip=1,nphase(id)

            ibeg = iend + 1
            iend = iend + ic(ip,id)

c --- Peak shape function parameters:

            do 20 j=1,3
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
                  shift = ds(km) * apar(j,ip,id) * relax(6)
                  par(j,ip,id) = par(j,ip,id) + shift
               end if
 20         continue

c --- Scale factor, overall temperature factor,
c     preferred orientation parameters:

            do 21 j=4,7
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
                  shift = ds(km) * apar(j,ip,id) * relax(4)
                  par(j,ip,id) = par(j,ip,id) + shift
               end if
 21         continue

c --- Halfwidth parameters (Caglioti):

            do 22 j=8,10
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
                  shift = ds(km) * apar(j,ip,id) * relax(3)
                  par(j,ip,id) = par(j,ip,id) + shift
                  if(par(j,ip,id).eq.0.) then
                     ass(id) = .true.
                  else if(abs(shift/par(j,ip,id)).gt.RSH) then
                     ass(id) = .true.
                  end if
               end if
 22         continue

c --- Wave vector components and reciprocal lattice parameters:

            do 23 j=11,19
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
                  shift = ds(km) * apar(j,ip,id) * relax(5)
                  par(j,ip,id) = par(j,ip,id) + shift
                  if(par(j,ip,id).eq.0.) then
                     ass(id) = .true.
                  else if(abs(shift/par(j,ip,id)).gt.rsh) then
                     ass(id) = .true.
                  end if
               end if
 23         continue

c --- Asymmetry correction parameters:

            do 24 j=20,23
	       km = lpar(j,ip,id)
	       if(km.ne.0.and.km.le.maxs) then
                  shift = ds(km) * apar(j,ip,id) * relax(6)
                  par(j,ip,id) = par(j,ip,id) + shift
               end if
 24         continue
            
c --- Atomparameter:

            if(id.eq.1) then
               if(jobtyp.eq.1) then

                  if(ip.gt.1) iof = iof + natom(ip-1)

                  do 60 i=1,natom(ip)
                     na = iof + i

c --- Koordinaten x, y, z, isotr. Temperaturfaktor, Besetzungszahl:

                     do 50 j=1,5
                        km = lato(j,na)
                        if(km.ne.0.and.km.le.maxs) then
                           shift = ds(km) * aato(j,na) * relax(1)
                           atom(j,na) = atom(j,na) + shift
                        end if
 50                  continue
                  
c --- Anisotrope Temperaturfaktoren, magnetische Momente Mx, My, Mz:

                     do 51 j=6,14
                        km = lato(j,na)
                        if(km.ne.0.and.km.le.maxs) then
                           shift = ds(km) * aato(j,na) * relax(2)
                           atom(j,na) = atom(j,na) + shift
                        end if
 51                  continue
                  
c --- Modulationsparameter:

                     if(imo(ip).eq.1) then
                        do 55 m=1,nbas(na)
                           do 54 j=1,2
                              km = lampl(j,m,na)
                              if(km.ne.0.and.km.le.maxs) then
                                 shift = ds(km) * 
     *                                   aampl(j,m,na) * relax(2)
                                 ampl(j,m,na) = ampl(j,m,na) + shift
                              end if
 54                        continue
 55                     continue
                     end if

 60               continue

               else

c --- Integrale Intensitaeten:

                  do 70 j=ibeg,iend
                     km = lint(j)
                     if(km.ne.0.and.km.le.maxs) then
                        shift = ds(km) * aint(j) * relax(4)
                        rint(j,id) = rint(j,id) + shift
                     end if
 70               continue

               end if
            end if
           
 100     continue
        
 200  continue


      return
      end

c******************************************************************************
c***               Gauss-Jordan elimination                                 ***
c******************************************************************************

      subroutine gaussj(covar,da,indxc,indxr,ipiv,maxs,MSZ)

      integer maxs, MSZ
      real covar(MSZ,MSZ), da(MSZ)
      integer i, icol, irow, j, k, l, ll
      integer indxc(MSZ), indxr(MSZ), ipiv(MSZ)
      real big, dum, pivinv

      do 11 j=1,maxs
         ipiv(j) = 0
 11   continue

      do 22 i=1,maxs
         big = 0.0
         do 13 j=1,maxs
            if(ipiv(j).ne.1)then
               do 12 k=1,maxs
                  if (ipiv(k).eq.0) then
                     if (abs(covar(j,k)).ge.big)then
                        big = abs(covar(j,k))
                        irow = j
                        icol = k
                     endif
                  else if (ipiv(k).gt.1) then
                     write(*,'(/2x,a/)') 
     *                    'ERROR: singular matrix in module gaussj'
                     stop
                  endif
 12            continue
            endif
 13      continue
         ipiv(icol) = ipiv(icol) + 1
         if (irow.ne.icol) then
            do 14 l=1,maxs
               dum = covar(irow,l)
               covar(irow,l) = covar(icol,l)
               covar(icol,l) = dum
 14         continue
            dum = da(irow)
            da(irow) = da(icol)
            da(icol) = dum
         endif
         indxr(i) = irow
         indxc(i) = icol
         if (covar(icol,icol).eq.0.) then
            write(*,'(/2x,a/)') 
     *           'ERROR: singular matrix in module gaussj'
            stop
         end if
         pivinv = 1. / covar(icol,icol)
         covar(icol,icol) = 1.
         do 16 l=1,maxs
            covar(icol,l) = covar(icol,l) * pivinv
 16      continue
         da(icol) = da(icol) * pivinv
         do 21 ll=1,maxs
            if(ll.ne.icol)then
               dum = covar(ll,icol)
               covar(ll,icol) = 0.
               do 18 l=1,maxs
                  covar(ll,l) = covar(ll,l) - covar(icol,l) * dum
 18            continue
               da(ll) = da(ll) - da(icol) * dum
            endif
 21      continue
 22   continue

      do 24 l=maxs,1,-1
         if(indxr(l).ne.indxc(l))then
            do 23 k=1,maxs
               dum = covar(k,indxr(l))
               covar(k,indxr(l)) = covar(k,indxc(l))
               covar(k,indxc(l)) = dum
 23         continue
         endif
 24   continue

      return
      end

c******************************************************************************
c***           Berechnen der Intensitaeten und Ableitungen                  ***
c***                bei nicht-modulierten Strukturen                        ***
c******************************************************************************

      subroutine nucstruc(maxs,dersto,ibeta,ixn,natom,atom,
     *                    aato,lato,rlamda,ratio,pref,ipref,par,apar,
     *                    lpar,aglb,lglb,ac,dfp,dfpp,ptr,rmult,icent,
     *                    ism,tran,nsym,iref,theta,hwb,rlopo,ffreal,
     *                    ffimag,rint,nhkl,sumxr,sumxi,deriv,
     *                    NATS,IRS,MSZ)

      external fwhm, precor

      parameter(PIQ = 9.869604401, PI = 3.14159265359,
     *          PII = 6.28318530718, EPSY = 0.00001)

      logical pac
      real precor, fwhm
      real sumxr(11,NATS), sumxi(11,NATS), dxcos(11), dxsin(11)
      real fac(6), hs(3), atd(6), al(3,3), deriv(MSZ), h(3)

      integer ptr(NATS), iref(7,IRS), ism(3,3,48)
      integer lglb(10), lpar(23), lato(14,NATS)
      real aglb(10), apar(23), aato(14,NATS), rmult(NATS)
      real ac(9,NATS), dfp(NATS), dfpp(NATS), rlamda(2), ratio(2)  
      real tran(3,48), atom(14,NATS), pref(3), par(23)
      real theta(IRS), rlopo(IRS), hwb(IRS), ffreal(IRS), ffimag(IRS)
      real rint(IRS), dersto(MSZ,IRS)


c --- reziproke Gitterkonstanten :

      al(1,1) = par(14) * par(14)
      al(2,2) = par(15) * par(15)
      al(3,3) = par(16) * par(16)
      al(1,2) = par(14) * par(15) * par(19)
      al(1,3) = par(14) * par(16) * par(18)
      al(2,3) = par(15) * par(16) * par(17)
      al(2,1) = al(1,2)
      al(3,1) = al(1,3)
      al(3,2) = al(2,3)

c --- Faktoren der anisotropen Temperaturfaktoren:

      if(ibeta.eq.0) then
        fac(1) = 1.
        fac(2) = 1.
        fac(3) = 1.
        fac(4) = 2.
        fac(5) = 2.
        fac(6) = 2.
      else if(ibeta.eq.1) then
        fac(1) = 0.25 * al(1,1)
        fac(2) = 0.25 * al(2,2)
        fac(3) = 0.25 * al(3,3)
        fac(4) = 0.50 * par(14) * par(15)
        fac(5) = 0.50 * par(14) * par(16)
        fac(6) = 0.50 * par(15) * par(16)
      else if(ibeta.eq.2) then
        fac(1) = PIQ * al(1,1)
        fac(2) = PIQ * al(2,2)
        fac(3) = PIQ * al(3,3)
        fac(4) = 2.*PIQ * par(14) * par(15)
        fac(5) = 2.*PIQ * par(14) * par(16)
        fac(6) = 2.*PIQ * par(15) * par(16)
      end if

c --- Vorzugsorientierung-Korrektur (ja oder nein):

      pac = par(6).ne.0..or.lpar(6).ne.0.or.
     *      par(7).ne.0..or.lpar(7).ne.0


c --- Reflexlagen, Halbwertsbreiten und Intensitaeten berechenen ---------

      do 200 i=1,nhkl

c --- Reziproker Gittervektor h:

         h(1) = iref(1,i)
         h(2) = iref(2,i)
         h(3) = iref(3,i)

c --- Betragsquadrat hh des reziproken Gittervektor h berechnen:

         hh = 0.
         do 22 l=1,3
            do 22 j=1,3
               hh = hh + h(l) * h(j) * al(l,j)
22       continue

c --- Reflexlage, Lorentz-Polarisations-Faktor, Halbwertsbreite
c     und Overall Temperature Factor berechnen:

         sinth       = 0.5 * rlamda(iref(5,i)) * sqrt(hh)
         sinlam      = hh/4.
         if(sinlam.eq.0.) sinlam = 0.0001
         if(abs(sinth).gt.1.) sinth = 0.
         th          = asin(sinth)
         tanth       = sinth/sqrt(1.-sinth*sinth)
         if(tanth.eq.0.) tanth = 0.0001
         theta(i)    = th
         hwb(i)      = fwhm(par(8),par(9),par(10),theta(i))


c --- Summen, die fuer die Ableitungen benoetigt werden, initialisieren:

         do 25 k=1,natom
            do 24 l=1,11
               sumxr(l,k) = 0.
               sumxi(l,k) = 0.
24          continue
25       continue

c --- Strukturfaktoren berechnen:

         sa = 0.
         sb = 0.
         sc = 0.
         sd = 0.

c --- Schleife ueber die Atome der asymmetrischen Einheit:

         do 100 k=1,natom

c --- Summen auf Null setzen:

            sumcos = 0.
            sumsin = 0.
            do 30 j=1,11
               dxcos(j) = 0.
               dxsin(j) = 0.
30          continue

c --- Formfaktor ff0 fuer das k-te Atom berechnen:

            if(ixn.eq.1) then
              ff0 = ac(9,ptr(k))
              do 35 j=1,4
                 ff0 = ff0 + 
     *                 ac(2*j-1,ptr(k)) * exp(-ac(2*j,ptr(k))*sinlam)
35            continue
            else
              ff0 = 0.
            end if

c --- Besetzungszahl atom(5,k), isotroper Temperaturfaktor atom(4,k),
c     auf die Zahl der Symm.-Karten norm. Wyckoff-Multiplizitaet rmult(k), 
c     Formfaktor ff0 + df' + df'':

            bwst  = atom(5,k) * exp(-atom(4,k)*sinlam) * rmult(k)   
            facs  = bwst * (ff0 + dfp(ptr(k)))
            facss = bwst * dfpp(ptr(k))


c --- Schleife ueber die symmetrie-aequivalenten Lagen zu Atom k:

            do 60 ns=1,nsym

c --- Symmetrieaequivalente Reflexe erzeugen:

               tr = 0.
               do 39 l=1,3
                  hs(l) = 0.
                  tr = tr + tran(l,ns) * h(l)
                  do 38 j=1,3
                     hs(l) = hs(l) + h(j) * ism(j,l,ns)
38                continue
39             continue

c --- Skalarprodukt hs * atom berechnen:

               scala = tr 
               do 40 j=1,3
                  scala = scala + hs(j) * atom(j,k)
40             continue 
               scala = PII * scala

c --- anisotroper Temperaturfaktor des k-ten Atoms:

               atd(1) = hs(1) * hs(1) * fac(1)
               atd(2) = hs(2) * hs(2) * fac(2)
               atd(3) = hs(3) * hs(3) * fac(3)
               atd(4) = hs(1) * hs(2) * fac(4)
               atd(5) = hs(1) * hs(3) * fac(5)
               atd(6) = hs(2) * hs(3) * fac(6)

               arg = 0.
               do 41 j=1,6
                  arg = arg + atd(j) * atom(5+j,k)
41             continue
     
               aniso = exp(-arg)

c --- Cosinus und Sinus der Phase mult. mit dem anisotropen Temp.fak.:

               fcos = cos(scala) * aniso
               fsin = sin(scala) * aniso

c --- Cosinuse und Sinuse aufsummieren:
            
               sumcos = sumcos + fcos
               if(icent.eq.1) sumsin = sumsin + fsin

c --- Ableitungen nach den Atomparametern x, y, z, b11, ... , b23:

               do 51 j=1,3
                  dxcos(j) = dxcos(j) - fsin * hs(j)
                  if(icent.eq.1) dxsin(j) = dxsin(j) + fcos * hs(j)
51             continue
               do 53 j=6,11
                  dxcos(j) = dxcos(j) - fcos * atd(j-5)
                  if(icent.eq.1) dxsin(j) = dxsin(j) - fsin * atd(j-5)
53             continue

60          continue

c --- Ableitungen abspeichern:

            do 67 j=1,3
               sumxr(j,k) = sumxr(j,k) + dxcos(j)*facs - dxsin(j)*facss
               sumxi(j,k) = sumxi(j,k) + dxsin(j)*facs + dxcos(j)*facss
67          continue
            do 68 j=4,5
               sumxr(j,k) = sumxr(j,k) + sumcos*facs - sumsin*facss 
               sumxi(j,k) = sumxi(j,k) + sumsin*facs + sumcos*facss
68          continue
            do 69 j=6,11
               sumxr(j,k) = sumxr(j,k) + dxcos(j)*facs - dxsin(j)*facss
               sumxi(j,k) = sumxi(j,k) + dxsin(j)*facs + dxcos(j)*facss
69          continue

c --- Strukturfaktoranteile A, B, C und D berechnen:

            sa = sa + sumcos * facs
            sb = sb + sumsin * facs
            sc = sc + sumcos * facss
            sd = sd + sumsin * facss

100      continue


c --- Realteil, Imaginaerteil und Betragsquadrat des Strukturfaktors
c     berechnen ( F = (A-D) + i*(B+C) ) :

         fre = sa - sd
         fim = sb + sc
         ffreal(i) = fre
         ffimag(i) = fim
         ff = fre**2 + fim**2


c --- Korrektur der Vorzugsorientierung:

         if(pac) then
           proc = precor(ipref,pref,par(6),par(7),al,h,sinlam,
     *                   dpref1,dpref2)
         else
           proc = 1.
         end if

c --- Intensitaet berechnen:

         xint = rlopo(i) * iref(6,i) * exp(-2.*par(5)*sinlam) * 
     *          ratio(iref(5,i)) * par(4)
         rcor = proc * xint
         rint(i) = ff * rcor


c --- Ableitungen von y_calc(i), die nicht von x(i) abhaengen:

         do 50 j=1,maxs
            deriv(j) = 0.
50       continue  

c --- Peak shape function parameters e0, e1 and e2:

         k = lpar(1)
         if(k.ne.0) deriv(k) = deriv(k) + apar(1) * rint(i)
         k = lpar(2)
         if(k.ne.0) deriv(k) = deriv(k) + apar(2) * rint(i)
         k = lpar(3)
         if(k.ne.0) deriv(k) = deriv(k) + apar(3) * rint(i)

c --- Skalierung:

         k = lpar(4)
         if(k.ne.0) deriv(k) = deriv(k) + apar(4) * rint(i)/par(4)

c --- Overall Temperature Factor:

         k = lpar(5)
         if(k.ne.0) deriv(k) = deriv(k) - 
     *                         2. * apar(5) * sinlam * rint(i) 

c --- Preferred Orientation Parameter G1, G2:

         k = lpar(6)
         if(k.ne.0) deriv(k) = deriv(k) + apar(6) * dpref1 * ff * xint
         k = lpar(7)
         if(k.ne.0) deriv(k) = deriv(k) + apar(7) * dpref2 * ff * xint

c --- Halbwertsbreite-Parameter U, V, W:

         rb = 0.5/hwb(i) * rint(i)

         k = lpar(8)
         if(k.ne.0) deriv(k) = deriv(k) + apar(8) * rb * tanth**2 
         k = lpar(9)
         if(k.ne.0) deriv(k) = deriv(k) + apar(9) * rb * tanth 
         k = lpar(10)
         if(k.ne.0) deriv(k) = deriv(k) + apar(10) * rb 

c --- reziproke Gitterkonstanten:

         rqa = 90./PI * tanth/sinlam * rint(i)

         k = lpar(14)
         if(k.ne.0) deriv(k) = deriv(k) + apar(14) * rqa *
     1              (h(1)*h(1)*par(14) + h(1)*h(2)*par(15)*par(19)
     2                                 + h(1)*h(3)*par(16)*par(18))
         k = lpar(15)  
         if(k.ne.0) deriv(k) = deriv(k) + apar(15) * rqa *
     1              (h(2)*h(2)*par(15) + h(2)*h(1)*par(14)*par(19)
     2                                 + h(2)*h(3)*par(16)*par(17))
         k = lpar(16)      
         if(k.ne.0) deriv(k) = deriv(k) + apar(16) * rqa *
     1              (h(3)*h(3)*par(16) + h(3)*h(1)*par(14)*par(18)
     2                                 + h(3)*h(2)*par(15)*par(17))
         k = lpar(17)
         if(k.ne.0) deriv(k) = deriv(k) + apar(17) * rqa *
     1                         h(2)*h(3)*par(15)*par(16)
         k = lpar(18)
         if(k.ne.0) deriv(k) = deriv(k) + apar(18) * rqa *
     1                         h(1)*h(3)*par(14)*par(16)
         k = lpar(19)
         if(k.ne.0) deriv(k) = deriv(k) + apar(19) * rqa *
     1                         h(1)*h(2)*par(14)*par(15)

c --- Asymmetriekorrektur-Parameter:

         k = lpar(20)
         if(k.ne.0) deriv(k) = deriv(k) + apar(20) * rint(i)
         k = lpar(21)
         if(k.ne.0) deriv(k) = deriv(k) + apar(21) * rint(i)
         k = lpar(22)
         if(k.ne.0) deriv(k) = deriv(k) + apar(22) * rint(i)
         k = lpar(23)
         if(k.ne.0) deriv(k) = deriv(k) + apar(23) * rint(i)

c --- Zeropoint, peak-shift correction:

         do 110 j=1,3
            k = lglb(j)         
            if(k.ne.0) deriv(k) = deriv(k) + aglb(j) * rint(i) 
110      continue

C --- Untergrund-Parameter:

         do 111 j=4,9
            k = lglb(j)
            if(k.ne.0) deriv(k) = deriv(k) + aglb(j)
111      continue

c --- Wellenlaenge:

         k = lglb(10)
         if(k.ne.0) deriv(k) = deriv(k) + aglb(10) * rint(i) * 
     *                                    tanth/rlamda(1) * 360./PI

c --- Ableitungen nach den Atomparameter:

         do 150 k=1,natom

c --- Koordinaten x, y und z:

            do 120 l=1,3
               m = lato(l,k)
               if(m.ne.0) 
     *           deriv(m) = deriv(m) + 4.*PI * rcor * aato(l,k) *
     *                          (fre*sumxr(l,k) + fim*sumxi(l,k)) 
120         continue

c --- isotroper Temperaturfaktor:

            m = lato(4,k)
            if(m.ne.0) 
     *        deriv(m) = deriv(m) - 2. * sinlam * rcor * aato(4,k) * 
     *                              (fre*sumxr(4,k) + fim*sumxi(4,k))

c --- Besetzungszahl:

            m = lato(5,k)
            if(m.ne.0) 
     *        deriv(m) = deriv(m) + 2./atom(5,k) * rcor * aato(5,k) *
     *                              (fre*sumxr(5,k) + fim*sumxi(5,k))

c --- Anisotrope Temperaturfaktoren beta(ij):

            do 125 j=6,11
               m = lato(j,k)
               if(m.ne.0) 
     *           deriv(m) = deriv(m) + 2. * rcor * aato(j,k) *  
     *                       (fre*sumxr(j,k) + fim*sumxi(j,k))
125         continue

150      continue 

c --- Ableitungen fuer jeden Reflex abspeichern:

         do 180 m=1,maxs
            dersto(m,i) = deriv(m)
180      continue


200   continue


      return
      end
C******************************************************************************
C***            Name der Ausgabe-Datei erzeugen                             ***
C******************************************************************************

      subroutine outfil(name,file,filspe,laenge,nr)

      parameter(MAX = 80)
      character filspe*3, file*80, name*80, form*4


c --- Initialisierung:

      name = ' '

c --- Zehnerpotenz der Zahl NR bestimmen:

      nz = 0
      if(nr.gt.0) nz = int(alog10(real(nr))) + 1

c --- Ende suchen:

      do 50 i = MAX,1,-1
         if(file(i:i).eq.'.') then
           iend = i - 1
           goto 60
         end if
50    continue

c --- Anfang suchen:

60    do 100 i = MAX,1,-1
         if(file(i:i).eq.'/'.or.file(i:i).eq.']'.or.
     *        file(i:i).eq.'\\') then
           ibeg = i + 1
           goto 120
         end if
100   continue
      ibeg = 1

c --- Basisnamen abspeichern:
      
120   n = 0
      do 200 i = ibeg,iend
         if(file(i:i).ne.' ') then
           n = n + 1
           name(n:n) = file(i:i)
         end if
200   continue     
    
c --- Laufende Nummer und Extension anhaengen:

      form = '(i )'
      write(form(3:3),'(i1)') nz
      if(nz.gt.0) write(name(n+1:n+nz),form) nr
      name(n+nz+1:n+nz+1) = '.'
      name(n+nz+2:n+nz+4) = filspe(1:3)
      laenge = n + nz + 4

      return
      end
C******************************************************************************
C***  Ausgabe der Fit-Ergebnisse auf dem Bildschirm nach der Verfeinerung   ***
C******************************************************************************

      subroutine outlas(ka,jobtyp,idges,imo,maxs,phsnm,
     *                  nphase,nprof,ibac,atext,ntyp,rmult,natom,
     *                  glb,aglb,lglb,par,apar,lpar,atom,aato,lato,ampl,
     *                  aampl,lampl,dp,intaus,icent,ism,tran,nprim,
     *                  nsym,pa,apos,tp,npos,cbas,numbsv,nbas,dmax,num,
     *                  sb1,sb2,sb3,sw1,sw2,LPHASE,NATS,IDAT,MSZ,IBV)

      parameter (NATO = 400)

      external atomass
      real std(23), zwth(3), hwb(3), shwb(3), delta(6)
      real u(3), rs(3), celld(6), atomass

      character cbas(IBV,NATS)*10, ntyp(NATS)*4
      character atext(NATS)*4, phsnm(LPHASE)*4, intaus(LPHASE)*1
      integer lglb(10,IDAT), lpar(23,LPHASE,IDAT), lato(14,NATS),
     *        lampl(2,IBV,NATS)
      integer ism(3,3,48,LPHASE), icent(LPHASE), nsym(LPHASE), 
     *        imo(LPHASE), nprim(48,LPHASE), npos(NATS)
      integer nbas(NATS), numbsv(IBV,NATS), ibac(IDAT)
      integer nphase(IDAT), nprof(IDAT), natom(LPHASE)
      real glb(10,IDAT), par(23,LPHASE,IDAT), atom(14,NATS),
     *     ampl(2,IBV,NATS)
      real aglb(10,IDAT), apar(23,LPHASE,IDAT), aato(14,NATS),
     *     aampl(2,IBV,NATS)
      real tran(3,48,LPHASE), pa(6,48,IBV,NATS), dp(MSZ)
      real dmax(LPHASE), gewant(20), dsq(20), rmult(NATS)
      real apos(3,48,NATS), tp(3,48,NATS)

      data PI, EPSY / 3.14159265359, 0.00001 /
      data zwth /20.,60.,100./

      write(ka,'(//2x,a,/2x,a)')
     *'New parameters and standard deviations:',
     *'======================================='

      do 200 id=1,idges

         write(ka,'(/2x,a,i2,a,/2x,a)') 'Data set ',id,':',
     *                                  '============'

         iof = 0
         sumgew = 0.
         sumder = 0.
         do 100 ip=1,nphase(id)

            if(ip.gt.1) iof = iof + natom(ip-1)

            write(ka,'(/2x,a,i2,a,a4,/2x,a)') 
     *           'Phase ',ip,': ',phsnm(ip),
     *           '--------------'

c --- Atomparameter:

            if(jobtyp.eq.1.and.id.eq.1) then

c --- Reziproke Gitterkonstanten berechnen:

              call direct(par(14,ip,id),std(14),celld,delta,vol,dvol) 
              cosa = cos(PI/180.*celld(4))
              cosb = cos(PI/180.*celld(5))
              cosg = cos(PI/180.*celld(6))

c --- Koordinaten x, y, z:

              write(ka,'(//2x,a,//2x,a)') 'Atom parameters:',
     *  'Atom     x       dx         y       dy         z       dz'
              do 21 k=1,natom(ip)
                 na = iof + k
                 do 20 j=1,3
                    std(j) = 0.
                    km = lato(j,na)
                    if(km.ne.0) 
     *                std(j) = dp(km) * abs(aato(j,na))
 20              continue
                 write(ka,'(2x,a4,3(1x,2f9.5))') 
     *           atext(na),(atom(j,na),std(j),j=1,3)
 21           continue

c --- Isotrope Temperaturfaktoren B und Besetzungszahlen N:

              write(ka,'(/2x,a)') 
     *        'Atom     B       dB         N       dN'
              do 23 k=1,natom(ip)
                 na = iof + k
                 do 22 j=4,5
                    std(j) = 0.
                    km = lato(j,na)
                    if(km.ne.0)
     *                std(j) = dp(km) * abs(aato(j,na))
 22              continue
                 write(ka,'(2x,a4,2(1x,2f9.5))') 
     *           atext(na),(atom(j,na),std(j),j=4,5)
 23           continue

c --- Anisotrope Temperaturfaktoren:

              ibij = 0
              do 25 k=1,natom(ip)
                 do 24 j=6,11
                    if(atom(j,iof+k).ne.0.) ibij = 1
 24              continue
 25           continue

              if(ibij.gt.0) then
                write(ka,'(//2x,a,//2x,a)') 
     *  'Anisotropic temperature factors:',
     *  'Atom    b11      db11      b22      db22      b33      db33'
                do 27 k=1,natom(ip)
                   na = iof + k
                   do 26 j=6,8
                      std(j) = 0.
                      km = lato(j,na)
                      if(km.ne.0) 
     *                  std(j) = dp(km) * abs(aato(j,na))
 26                continue
                   write(ka,'(2x,a4,3(1x,2f9.5))') 
     *             atext(na),(atom(j,na),std(j),j=6,8)
 27             continue
                write(ka,'(/2x,a)')
     *  'Atom    b12      db12      b13      db13      b23      db23'
                do 29 k=1,natom(ip)
                   na = iof + k
                   do 28 j=9,11
                      std(j) = 0.
                      km = lato(j,na)
                      if(km.ne.0)
     *                  std(j) = dp(km) * abs(aato(j,na))
 28                continue
                   write(ka,'(2x,a4,3(1x,2f9.5))')
     *             atext(na),(atom(j,na),std(j),j=9,11)
 29             continue
              end if

              if(imo(ip).eq.2) then

c --- Magnetische Momente ausgeben:

                write(ka,'(//2x,a,//2x,a,a)') 
     *  'Magnetic moment parameters:',
     *  'Atom    Mx      dMx      My      dMy      Mz      dMz',
     *  '      M       dM'
                do 31 k=1,natom(ip)
                   na = iof + k
                   do 30 j=12,14
                      std(j) = 0.
                      km = lato(j,na)
                      if(km.ne.0)
     *                  std(j) = dp(km) * abs(aato(j,na))
 30                continue
                   ss = sqrt(
     *             atom(12,na)**2 + atom(13,na)**2 + atom(14,na)**2 +
     *             2. * atom(12,na) * atom(13,na) * cosg +
     *             2. * atom(12,na) * atom(14,na) * cosb +
     *             2. * atom(13,na) * atom(14,na) * cosa )
                 dx = atom(12,na) + atom(13,na)*cosg + atom(14,na)*cosb
                 dy = atom(13,na) + atom(12,na)*cosg + atom(14,na)*cosa
                 dz = atom(14,na) + atom(12,na)*cosb + atom(13,na)*cosa
                   if(ss.gt.0.) then
                     dss = sqrt( (dx * std(12))**2 + 
     *                           (dy * std(13))**2 +
     *                           (dz * std(14))**2 )/ss
                   else
                     dss = 0.
                   end if
                   write(ka,'(2x,a4,4(1x,2f8.4))') 
     *             atext(na),(atom(j,na),std(j),j=12,14),ss,dss
 31             continue

c --- Magnetische Momente der symmetrieaequivalenten Positionen ausgeben:

                write(ka,'(//2x,a,//2x,a)') 
     *          'Positions and magnetic moments:', 
     *       'Atom  no     x       y       z        Mx      My      Mz'
                do 40 k=1,natom(ip)
                   na = iof + k
                   do 39 ns=1,nsym(ip)
                      do 38 i=1,3
                         rs(i) = tran(i,ns,ip)
                         u(i) = 0.
                         do 37 j=1,3
                            rs(i) = rs(i) + ism(i,j,ns,ip) * atom(j,na)
                            u(i) = u(i) + ism(i,j,ns,ip) * atom(11+j,na)
 37                      continue
                         u(i) = u(i) * 
     *                          nprim(ns,ip) * ndet3(ism(1,1,ns,ip))
 38                   continue
                      write(ka,'(2x,a,i4,1x,3f8.4,2x,3f8.4)') 
     *                atext(na),ns,(rs(i),i=1,3),(u(i),i=1,3)
 39                continue
 40             continue   

              else if(imo(ip).eq.1) then

c --- Modulationsparameter ausgeben:

                nbw = 0
                do 41 k=1,natom(ip)
                   nbw = nbw + nbas(iof+k)
 41             continue

                if(nbw.gt.0) then
                  write(ka,'(//2x,a,//2x,a,a)')
     *            'Displacement amplitudes and phases:',
     *            'Atom   Base vector  no.      u      du',
     *            '        phi     dphi'
                  do 44 k=1,natom(ip)
                     na = iof + k
                     if(nbas(na).gt.0) then
                       do 43 m=1,nbas(na)
                          do 42 j=1,2
                             std(j) = 0.
                             km = lampl(j,m,na) 
                             if(km.ne.0) 
     *                         std(14+j) = dp(km) * abs(aampl(j,m,na))
 42                       continue
                          write(ka,
     *                    '(2x,a4,3x,a10,i6,2x,2f8.4,2x,2f8.4)') 
     *                    atext(na),cbas(m,na),numbsv(m,na),
     *                    ampl(1,m,na),std(15),ampl(2,m,na),std(16)
 43                    continue
                     end if
 44               continue
                end if

c --- Alle symmetrieaequivalenten Positionen fuer alle Atome der
c     asymmetrischen Einheit bestimmen und Auslenkungen ausgeben:

                write(ka,'(//2x,a)') 
     *          'Positions and displacement amplitudes:'
                write(ka,'(/2x,a)') 
     *     'Atom  no    x/a     y/b     z/c       Ux/a    Uy/b    Uz/c'
                do 50 k=1,natom(ip)
                   na = iof + k
                   do 49 l=1,npos(na)
                      phase = 0.
                      do 45 i=1,3
                         phase = phase + par(10+i,ip,id) * tp(i,l,na)
 45                   continue
                      do 47 i=1,3
                         u(i) = 0.
                         do 46 m=1,nbas(na)
                            phs = pa(3+i,l,m,na) + 
     *                            ampl(2,m,na) + phase
                            u(i) = u(i) + 
     *                      ampl(1,m,na) * pa(i,l,m,na) * cos(phs)
 46                      continue
 47                   continue
                      write(ka,'(2x,a4,i4,1x,3f8.4,2x,3f8.4)') 
     *                atext(na),l,(apos(i,l,na),i=1,3),(u(i),i=1,3)
 49                continue
 50             continue

              end if

c --- Bindungslaengen berechnen und ausgeben:

              if(dmax(ip).gt.EPSY) then
                 if(imo(ip).eq.0) then
                    call dist(ka,dmax(ip),atext(iof+1),atom(1,iof+1),
     *                   aato(1,iof+1),lato(1,iof+1),natom(ip),
     *                   NATS,dp,MSZ,celld,intaus(ip),icent(ip),
     *                   ism(1,1,1,ip),tran(1,1,ip),nsym(ip))
                 else if(imo(ip).eq.1) then
                    call distm(ka,dmax(ip),atext(iof+1),atom(1,iof+1),
     *                   aato(1,iof+1),lato(1,iof+1),natom(ip),NATS,dp,
     *                   MSZ,celld,par(11,ip,id),intaus(ip),icent(ip),
     *                   ism(1,1,1,ip),tran(1,1,ip),nsym(ip),
     *                   pa(1,1,1,iof+1),apos(1,1,iof+1),tp(1,1,iof+1),
     *                   npos(iof+1),ampl(1,1,iof+1),aampl(1,1,iof+1),
     *                   lampl(1,1,iof+1),nbas(iof+1),IBV)
                 end if
              end if

              write(ka,*)


            end if

c --- Standardabweichungen der Parameter par ermitteln:

            do 60 j=1,23
   	       km = lpar(j,ip,id)
	       if(km.ne.0) then
                 std(j) = dp(km) * abs(apar(j,ip,id))
               else
	         std(j) = 0.
	       end if
 60         continue

C --- Direkte Gitterkonstanten berechnen:

            call direct(par(14,ip,id),std(14),celld,delta,vol,dvol)

c --- Reziproke Winkel berechnen:

            alrez = 180./PI * acos(par(17,ip,id))
            berez = 180./PI * acos(par(18,ip,id))
            garez = 180./PI * acos(par(19,ip,id))
            alstd = 180./PI * 1./sqrt(1-par(17,ip,id)**2) * std(17)
            bestd = 180./PI * 1./sqrt(1-par(18,ip,id)**2) * std(18)
            gastd = 180./PI * 1./sqrt(1-par(19,ip,id)**2) * std(19)

C --- Overall temperature factor und Skalierung ausgeben:

            if(jobtyp.eq.1) then
              write(ka,'(/2x,2(a,g9.3),/2x,2(a,f9.4))')
     *        'Overall scale factor = ',par(4,ip,id),' +- ',std(4),
     *        'Overall temp. factor = ',par(5,ip,id),' +- ',std(5)
            end if

C --- Preferred orientation correction parameters ausgeben:

            if(jobtyp.eq.1) then
              WRITE(ka,'(/2X,A,2(/2X,2(A,F8.5)))')
     *        'Preferred orientation parameters:',
     *        'G1 = ',PAR(6,ip,id),' +- ',std(6),
     *        'G2 = ',PAR(7,ip,id),' +- ',std(7)
            end if

c --- Parameter der Profilfunktion:

            if(nprof(id).eq.5) then
	      write(ka,'(/2x,a,2(/2x,2(a,f8.3)))')
     *             'Peak shape function parameters (Edgeworth serie):',
     *             'Skewness = ', par(1,ip,id),' +- ',std(1),
     *             'Excess   = ', par(2,ip,id),' +- ',std(2)
            else if(nprof(id).eq.6.or.nprof(id).eq.9) then
              write(ka,'(/2x,a,a)')
     *        'Pseudo-Voigt function parameters:',
     *        '       2-Theta    Eta     SEta'
              do 71 j=1,3
                 eta = par(1,ip,id) + par(2,ip,id) * zwth(j) + 
     *                 par(3,ip,id) * zwth(j)**2
                 seta = sqrt(std(1)**2 + zwth(j)**2 * std(2)**2 + 
     *                       zwth(j)**4 * std(3)**2)
                 write(ka,'(2x,a,i1,a,g11.4,a,g11.4,8x,f6.2,2f8.3)')
     *           'Eta',j,' = ',par(j,ip,id),' +- ',std(j),
     *           zwth(j),eta,seta
 71           continue
            else if(nprof(id).eq.7) then
              write(ka,'(/2x,a,a)')
     *        'Pearson-VII function parameters:',
     *        '      2-Theta     M      SM'
              do 72 j=1,3
                 eta = par(1,ip,id) + par(2,ip,id) / zwth(j) + 
     *                 par(3,ip,id) / zwth(j)**2
                 seta = sqrt( std(1)**2 + std(2)**2 / zwth(j)**2 + 
     *                        std(3)**2 / zwth(j)**4)
                 write(ka,'(2x,a,i1,a,g11.4,a,g11.4,8x,f6.2,2f8.3)')
     *           'M',j,' = ',par(j,ip,id),' +- ',std(j),
     *           zwth(j),eta,seta
 72           continue
            end if

C --- Asymmetrie-Parameter ausgeben:

            if(nprof(id).eq.9) then
               write(ka,'(/2x,a,2(/2x,a,f9.5,a,f8.5))') 
     *              'Asymmetry correction parameters:',
     *              'S/L = ', par(20,ip,id),' +- ',std(20),
     *              'H/L = ', par(21,ip,id),' +- ',std(21)
            else
               write(ka,'(/2x,a,4(/2x,a,f9.5,a,f8.5))') 
     *              'Asymmetry correction parameters:',
     *              'A0 = ', par(20,ip,id),' +- ',std(20),
     *              'B0 = ', par(21,ip,id),' +- ',std(21),
     *              'A1 = ', par(22,ip,id),' +- ',std(22),
     *              'B1 = ', par(23,ip,id),' +- ',std(23)
            end if

C --- Berechnen der Halbwertsbreiten sowie der dazugehoerigen
C     Standardabweichungen

            do 80 j=1,3
               th   = zwth(j) * PI/360.
               tgt  = sin(th)/cos(th)
               hwb2 = par(8,ip,id)*tgt*tgt + par(9,ip,id)*tgt 
     *                + par(10,ip,id)
               if(hwb2.ge.0.) then
                 hwb(j)  = sqrt(hwb2)
                 shwb(j) = sqrt(tgt**4 * std(8)*std(8) 
     *           + tgt*tgt*std(9)*std(9) + std(10)*std(10))/(2.*hwb(j))
               else
                 hwb(j) = -1.0
               end if
80          continue

C --- Halbwertsbreite-Parameter ausgeben:

            write(ka,
     *      '(/2x,a,13x,a,3(/2x,a,f10.6,a,f9.6,8x,f6.2,1x,2(1x,f9.6)))')
     *      'Halfwidth parameters:','2-Theta     Fwhm     SFwhm',
     *      'U = ',PAR( 8,ip,id),' +- ',std( 8),zwth(1),HWB(1),SHWB(1),
     *      'V = ',PAR( 9,ip,id),' +- ',std( 9),zwth(2),HWB(2),SHWB(2),
     *      'W = ',PAR(10,ip,id),' +- ',std(10),zwth(3),HWB(3),SHWB(3)

C --- Q-Vektor ausgeben:

            if(imo(ip).eq.1) then
	      write(ka,'(/2x,a,3(/2x,a,f9.5,a,f8.5))')
     *        'Wave vector of the modulation:',
     *        'q1 = ',par(11,ip,id),' +- ',std(11),
     *        'q2 = ',par(12,ip,id),' +- ',std(12),
     *        'q3 = ',par(13,ip,id),' +- ',std(13)
            end if

C --- Direkte und reziproke Gitterkonstanten ausgeben:

            write(ka,'(/2x,a,6(/2x,a,f11.6,a,f9.6))')
     *      'Reciprocal lattice parameters:',
     *      'A*     = ',par(14,ip,id),' +- ',std(14),
     *      'B*     = ',par(15,ip,id),' +- ',std(15),
     *      'C*     = ',par(16,ip,id),' +- ',std(16),
     *      'Alpha* = ',alrez,' +- ',alstd,
     *      'Beta*  = ',berez,' +- ',bestd,
     *      'Gamma* = ',garez,' +- ',gastd

            write(ka,'(/2x,a,6(/2x,a,f11.6,a,f9.6))')
     *      'Cell parameters:',
     *      'A     = ',celld(1),' +- ',delta(1),
     *      'B     = ',celld(2),' +- ',delta(2),
     *      'C     = ',celld(3),' +- ',delta(3),
     *      'Alpha = ',celld(4),' +- ',delta(4),
     *      'Beta  = ',celld(5),' +- ',delta(5),
     *      'Gamma = ',celld(6),' +- ',delta(6)

            write(ka,'(/2x,a,f9.3,a,f7.3))') 
     *      'Cell volume = ',vol,' +- ',dvol

c --- Quantitative Phasenanalyse ohne Brindley'scher 
c     Kontrastfaktor der Mikroabsorption (Teil 1):

            if(imo(ip).ne.2) then
               xm = 0.
               do 95 k=1,natom(ip)
                  xm = xm + rmult(iof+k) * atom(5,iof+k) * nsym(ip) *
     *                 atomass(ntyp(iof+k))
 95            continue
               if(par(4,ip,id).ne.0.) then
                  dsq(ip) = (std(4)/par(4,ip,id))**2
               else
                  dsq(ip) = 0.
               end if
               gewant(ip) = xm * vol * par(4,ip,id)
               sumgew = sumgew + gewant(ip)
               sumder = sumder + (xm * vol * std(4))**2
            end if

100      continue

c --- Quantitative Phasenanalyse (Teil 2):

         if(nphase(id).gt.1) then
            iqa = 0
            do 104 ip=1,nphase(id)
               if(imo(ip).ne.2) iqa = iqa + 1
 104        continue
            if(iqa.ge.2) then
               write(ka,'(/2x,a)') 'Relative weight fractions:' 
               do 105 ip=1,nphase(id)
                  if(imo(ip).ne.2) then
                     rga = gewant(ip)/sumgew
                     if(abs(rga-1.).gt.EPSY) then
                        drga = sqrt(sumder/sumgew**2 + 
     *                         (1.-2.*rga)*dsq(ip)) * rga
                     else
                        drga = 0.
                     end if
                     write(ka,'(2x,3a,f7.2,a,f7.2)') 
     *                    'Phase ',phsnm(ip),': ', rga,' +- ',drga
                  end if
 105           continue
            end if
         end if

c === Globale Parameter:

         do 190 j=1,10
            km = lglb(j,id)
	    if(km.ne.0) then
              std(j) = dp(km) * abs(aglb(j,id))
            else
              std(j) = 0.
            end if
190      continue

         write(ka,'(//2x,a)') 'Global parameters:'

c --- Peak-Shift-Parameter ausgeben:

         write(ka,'(/2x,a,3(/2x,2(a,g11.4)))')
     *        'Peak-shift correction:',
     *        'Zeropoint t0 = ', glb(1,id),' +- ',std(1),
     *        'Parameter t1 = ', glb(2,id),' +- ',std(2),
     *        'Parameter t2 = ', glb(3,id),' +- ',std(3)

c --- Wellenlaenge:

         write(ka,'(/2x,a,f10.6,a,f9.6)') 
     *        'Wavelength L1 = ',glb(10,id),' +- ',std(10)

c --- Untergrund-Parameter ausgeben:

         if(ibac(id).eq.0) then
           write(ka,'(/2x,a,6(/2x,a,f10.3,a,f8.3))')
     *          'Background parameters:',
     *          'c0 = ',glb(4,id),' +- ',std(4),
     *          'c1 = ',glb(5,id),' +- ',std(5),
     *          'c2 = ',glb(6,id),' +- ',std(6),
     *          'c3 = ',glb(7,id),' +- ',std(7),
     *          'c4 = ',glb(8,id),' +- ',std(8),
     *          'c5 = ',glb(9,id),' +- ',std(9)
         end if
         
200   continue


C --- Gesamt-R-Werte ausgeben:

      ii = num - maxs
      gof  = sqrt(sw2/real(ii))
      rsig = (sw2 - ii)/sqrt(2.*ii)
      rexp = 100. * sqrt(ii/sw1)
      rp   = 100. * sb3/sb2
      rwp  = 100. * sqrt(sw2/sw1)

      write(ka,'(//2x,a,/2x,a)')  'R-Factors:',
     *                            '=========='
      write(ka,'(/2x,3(a,f8.2))')
     *'Rp  = ',rp,'     Rwp = ',rwp,'     Rexp = ',rexp
      write(ka,'(/2x,a,i8,a,f8.2,a,f8.2)') 
     *'N-P = ',ii,'     Gof = ',gof,'     Nsig = ',rsig     
      write(ka,'(2(/2x,2(a,e14.4)),/2x,a,e14.4)') 
     *'SumYdif  = ',sb3,'     SumYobs    = ',sb2,
     *'SumYcalc = ',sb1,'     SumWYobsSQ = ',sw1,
     *'Residual = ',sw2


      return
      end


c******************************************************************************
c***               Schreiben des CYCLUS-Files                               ***
c******************************************************************************

      subroutine outmat(kcyc,icycle,mcycle,isp,titel,idges,IDAT,
     *                  param,out,iout,MCL,MAPA)

      character param(MAPA)*7, titel(IDAT)*70, date*9, time*8
      real out(MAPA,MCL)


c --- Datum, Uhrzeit und Titel schreiben:

      call datum(date,time)
      write(kcyc,'(1x,a,4x,a)') date, time
      do 15 id=1,idges
         write(kcyc,'(/1x,a)') titel(id)
15    continue
      write(kcyc,*)
      write(kcyc,*)


c --- Zahl der R-Werte:

      id4 = idges * 5 + 5 


c --- Ausgabe mit 80 Zeichen pro Zeile:

      if(isp.eq.0) then

         imod = 0
         ma   = (iout - id4) / 3
         imod = mod(ma,2)
         if(imod.ne.0) then
            ma = ma - 1
         end if
         ma = ma / 2

         do 310 k=1,ma
            write(kcyc,340) (param(ip),ip=(k-1)*6+1,(k-1)*6+6)
            do 300 i=1,icycle
               write(kcyc,330) i,(out(j,i),j=(k-1)*6+1,(k-1)*6+6)
 300        continue
            write(kcyc,*)
 310     continue

         if(imod.ne.0) then
            write(kcyc,340) (param(ip),ip=ma*6+1,ma*6+3)
            do 320 i=1,icycle
               write(kcyc,330) i,(out(j,i),j=ma*6+1,ma*6+3)
 320        continue
            write(kcyc,*)
         end if

 330     format(1x,i2,2(' |',f10.5,f9.5,f9.5))
 340     format(4x,2(4x,a7,3x,a7,2x,a7))


c --- Ausgabe mit 132 Zeichen pro Zeile:

      else

         imod = 0
         ma   = (iout - 5) / 3
         imod = mod(ma,4)
         if(imod.ne.0) then
            ma = ma - imod
         end if
         ma = ma / 4

         do 410 k=1,ma
            write(kcyc,440) (param(ip),ip=(k-1)*12+1,(k-1)*12+12)
            do 400 i=1,icycle
               write(kcyc,430) i,(out(j,i),j=(k-1)*12+1,(k-1)*12+12)
 400        continue
            write(kcyc,*)
 410     continue

         if(imod.ne.0) then
            iend = imod * 3
            write(kcyc,440) (param(ip),ip=ma*12+1,ma*12+iend)
            do 420 i=1,icycle
               write(kcyc,430) i,(out(j,i),j=ma*12+1,ma*12+iend)
 420        continue
            write(kcyc,*)
         end if

 430     format(1x,i2,4(' |',f10.5,f8.5,f8.5))       
 440     format(4x,4(4x,a7,2x,a7,1x,a7))

      end if

c --- R-Werte pro Zyklus ins CYCLUS-File schreiben :

      do 560 k=1,idges + 1
         
	 ir1 = (k-1)*5 + 1
	 ir2 = (k-1)*5 + 5
	 write(kcyc,540) (param(ip),ip=iout-id4+ir1,iout-id4+ir2)
	 do 520 i=1,icycle
	    write(kcyc,530) i,(out(j,i),j=iout-id4+ir1,iout-id4+ir2)
 520     continue

 530     format(1x,i2,5(' |',f12.5))
 540     format(5x,5(6x,a7,1x))

 560  continue

c --- Forced termination:

      if(icycle.lt.mcycle) then
         write(kcyc,'(//1x,a,/1x,a)') 'Forced termination',
     *                                '=================='
      end if


      return
      end

c******************************************************************************
c***  Kodierung der Parameter ausgeben und Parameter-Nummern zuruecksetzen  ***
c******************************************************************************

      subroutine pcode(jobtyp,idges,nphase,natom,nprof,imo,atext,
     *                 numbsv,cbas,nbas,aglb,lglb,apar,lpar,aato,lato,
     *                 aampl,lampl,aint,lint,ic,ipar,IRS,IBV,IDAT,
     *                 LPHASE,NATS,MSZ)

      character atext(NATS)*4, cbas(IBV,NATS)*10
      integer lglb(10,IDAT), lpar(23,LPHASE,IDAT), lato(14,NATS),
     *        lampl(2,IBV,NATS), lint(IRS), ic(LPHASE,IDAT)
      integer natom(LPHASE), nbas(NATS), numbsv(IBV,NATS)
      integer nphase(IDAT), nprof(IDAT), imo(LPHASE),
     *        ipar(MSZ,IDAT)
      real aglb(10,IDAT), apar(23,LPHASE,IDAT), aato(14,NATS),
     *     aampl(2,IBV,NATS), aint(IRS)

      do 100 id=1,idges

         write(*,*)
         if(idges.gt.1) then
           write(*,'(2x,a,i2,a)') 'Data set ',id,':'
           write(*,'(2x,a)')      '------------'
         end if

c --- Globale Parameter:

         if(aglb(1,id).gt.0.01.or.aglb(2,id).gt.0.01.or.
     *      aglb(3,id).gt.0.01) write(*,'(2x,a,3f8.2)')
     *     'Zeropoint, peak-shift param. = ',(aglb(j,id),j=1,3)
         if(aglb(4,id).gt.0.01.or.aglb(5,id).gt.0.01.or.
     *      aglb(6,id).gt.0.01.or.aglb(7,id).gt.0.01.or.
     *      aglb(8,id).gt.0.01.or.aglb(9,id).gt.0.01)
     *     write(*,'(2x,a,6f8.2)')
     *     'Background parameters = ',(aglb(j,id),j=4,9)
         if(aglb(10,id).gt.0.01) 
     *     write(*,'(2x,a,f8.2)') 'Wavelength = ',aglb(10,id)

         do 10 j=1,10
            x = aglb(j,id)
            iyy = int(abs(x)/10.)
            lglb(j,id) = iyy
            aglb(j,id) = (abs(x)-10.*iyy) * sign(1.,x)
            if(iyy.ne.0) ipar(iyy,id) = iyy
 10      continue

         iof = 0
         iend = 0
         do 100 ip=1,nphase(id)

            if(ip.gt.1) iof = iof + natom(ip-1)

            lz = 0
            do 20 i=1,23
               if(apar(i,ip,id).gt.0.01) lz = lz + 1
 20         continue
            if(id.eq.1) then
              do 25 k=1,natom(ip)
                 do 21 i=1,14
                    if(aato(i,iof+k).gt.0.01) lz = lz + 1
 21              continue
                 do 23 m=1,nbas(k)
                    do 22 i=1,2
                       if(aampl(i,m,iof+k).gt.0.01) lz = lz + 1
 22                 continue
 23              continue
 25           continue
              do 27 i=ibeg,iend
                 if(aint(i).gt.0.01) lz = lz + 1
 27           continue
            end if
            if(lz.gt.0.and.nphase(id).gt.1) 
     *        write(*,'(/2x,a,i2,a)') 'Phase ',ip,':'

c --- Parameter der Profilfunktionen:

            if(nprof(id).le.4.or.nprof(id).eq.8) then
              if(apar(1,ip,id).gt.0.01.or.apar(2,ip,id).gt.0.01.or.
     *           apar(3,ip,id).gt.0.01) then
                write(*,'(/2x,a,a)') 'ERROR: Profile parameter ',
     *          '"Skewness/Eta/Pearson-M" cannot be refined'
                stop
              end if
            else if(nprof(id).eq.5) then
              if(apar(1,ip,id).gt.0.01.or.apar(2,ip,id).gt.0.01) then
                write(*,'(2x,a,2f8.2)')
     *          'Skewness, Excess = ',apar(1,ip,id), apar(2,ip,id)
              end if
              if(apar(3,ip,id).gt.0.01) then
                 write(*,'(/2x,a)') 
     *           'ERROR: Profile parameter "Eta2/M2" cannot be refined'
                 stop
              end if
            else if(nprof(id).eq.6.or.nprof(id).eq.9) then
              if(apar(1,ip,id).gt.0.01.or.apar(2,ip,id).gt.0.01.or.
     *           apar(3,ip,id).gt.0.01) then
                 write(*,'(2x,a,3f8.2)') 
     *                'Eta (Pseudo-Voigt) = ',(apar(j,ip,id),j=1,3)
              end if
            else if(nprof(id).eq.7) then
              if(apar(1,ip,id).gt.0.01.or.apar(2,ip,id).gt.0.01.or.
     *           apar(3,ip,id).gt.0.01) then
                 write(*,'(2x,a,3f8.2)') 
     *                'Pearson-M = ',(apar(j,ip,id),j=1,3)
              end if
            end if

c --- Asymmetrie-Parameter:

            if(apar(20,ip,id).gt.0.01.or.apar(21,ip,id).gt.0.01.or.
     *         apar(22,ip,id).gt.0.01.or.apar(23,ip,id).gt.0.01)
     *        write(*,'(2x,a,4f8.2)')
     *        'Asymmetry parameters = ',(apar(j,ip,id),j=20,23)

c --- Skalierungsfaktor:

            if(apar(4,ip,id).gt.0.01) write(*,'(2x,a,f8.2)')
     *        'Overall scale factor = ',apar(4,ip,id)

c --- Allgemeiner Temperaturfaktor:

            if(apar(5,ip,id).gt.0.01) write(*,'(2x,a,f8.2)')
     *        'Overall temperature factor = ',apar(5,ip,id)

c --- Preferred orientation parameters:

            if(apar(6,ip,id).gt.0.01.or.apar(7,ip,id).gt.0.01)
     *        write(*,'(2x,a,2f8.2)') 
     *        'Preferred orientation parameters = ',
     *        (apar(j,ip,id),j=6,7)
                                                    
c --- Halbwertsbreite-Parameter:

            if(apar(8,ip,id).gt.0.01.or.apar(9,ip,id).gt.0.01.or.
     *         apar(10,ip,id).gt.0.01) write(*,'(2x,a,3f8.2)')
     *        'Halfwidth parameters = ',(apar(j,ip,id),j=8,10)    

c --- Kodierung der Gitterkonstanten ueberpruefen und ausgeben:

            if(apar(14,ip,id).gt.0.01.or.apar(15,ip,id).gt.0.01.or.
     *         apar(16,ip,id).gt.0.01.or.apar(17,ip,id).gt.0.01.or.
     *         apar(18,ip,id).gt.0.01.or.apar(19,ip,id).gt.0.01)
     *        write(*,'(2x,a,6f8.2)')
     *        'Cell constants = ',(apar(j,ip,id),j=14,19)

c --- Wellenvektor:

            if(apar(11,ip,id).gt.0.01.or.apar(12,ip,id).gt.0.01.or.
     *         apar(13,ip,id).gt.0.01) then
              if(imo(ip).eq.1) then
                write(*,'(2x,a,3f8.2)')
     *          'Wave vector of the modulation = ',
     *          (apar(j,ip,id),j=11,13)
              else
                write(*,'(/2x,2a)') 
     *          'ERROR: Wave vector cannot be refined'
                stop
              end if
            end if

c --- Zuruecksetzen der Parameter-Nummern:

            do 30 j=1,23
               x = apar(j,ip,id)
               iyy = int(abs(x)/10.)
               lpar(j,ip,id) = iyy
               apar(j,ip,id) = (abs(x)-10.*iyy) * sign(1.,x)
               if(iyy.ne.0) ipar(iyy,id) = iyy
 30         continue


c --- Atomparameter:

            if(jobtyp.eq.1.and.id.eq.1) then

              iatp = 0
              ibij = 0
              imag = 0
              do 50 n=1,natom(ip)
                 na = n + iof
                 do 41 j=1,5
                    if(aato(j,na).gt.0.01) iatp = iatp + 1
 41              continue
                 do 42 j=6,11
                    if(aato(j,na).gt.0.01) ibij = ibij + 1
 42              continue
                 do 43 j=12,14
                    if(aato(j,na).gt.0.01) imag = imag + 1
 43              continue                 
 50           continue

              if(iatp.gt.0) then
                write(*,60)
 60             format(/2x,'Atom',8x,'x',9x,'y',9x,'z',9x,'B',9x,'N')
                write(*,'(2x,a4,1x,5f10.2)') (atext(n+iof),
     *          (aato(j,n+iof),j=1,5),n=1,natom(ip))
              end if
              if(ibij.gt.0) then
                write(*,61)
 61             format(/2x,'Atom',7x,'b11',7x,'b22',7x,'b33',7x,
     *                 'b12',7x,'b13',7x,'b23')
                write(*,'(2x,a4,1x,6f10.2)') (atext(n+iof),
     *          (aato(j,n+iof),j=6,11),n=1,natom(ip))
              end if
              if(imag.gt.0) then
                write(*,62)
 62             format(/2x,'Atom',7x,'Mx',8x,'My',8x,'Mz')
                write(*,'(2x,a4,1x,3f10.2)') (atext(n+iof),
     *          (aato(j,n+iof),j=12,14),n=1,natom(ip))
              end if

              do 80 n=1,natom(ip)
                 na = n + iof
                 do 79 j=1,14
                    x = aato(j,na)
                    iyy = int(abs(x)/10.)
                    lato(j,na) = iyy
                    aato(j,na) = (abs(x)-10.*iyy) * sign(1.,x)
 79              continue
 80           continue

c --- Modulationsparameter:

              imod = 0
              do 85 n=1,natom(ip)
                 na = n + iof
                 do 84 m=1,nbas(na)
                    if(aampl(1,m,na).gt.0.01.or.
     *                 aampl(2,m,na).gt.0.01) then
                      imod = imod + 1
                    end if
 84              continue
 85           continue

              if(imod.gt.0) then
                write(*,'(/2x,a)') 
     *          'Atom   base vector    No.         u        phi'
                do 88 n=1,natom(ip)
                   na = n + iof
                   do 87 m=1,nbas(na)
                      write(*,'(2x,a4,3x,a,i8,2x,2f10.2)') 
     *                     atext(na),cbas(m,na),numbsv(m,na),
     *                     aampl(1,m,na),aampl(2,m,na)
 87                continue
 88             continue
              end if

              do 95 n=1,natom(ip)
                 na = n + iof
                 do 90 m=1,nbas(na)
                    do 89 j=1,2
                       x = aampl(j,m,na)
                       iyy = int(abs(x)/10.)
                       lampl(j,m,na) = iyy
                       aampl(j,m,na) = (abs(x)-10.*iyy) * sign(1.,x)
 89                 continue
 90              continue
 95           continue

c --- Integrale Intensitaeten (Profilverfeinerung);

            else if(jobtyp.eq.5.and.id.eq.1) then

              ibeg = iend + 1
              iend = iend + ic(ip,id)
              ipr = 0
              do 97 i=ibeg,iend
                 if(aint(i).gt.0.01) then
                   ipr = 1
                   goto 98
                 end if
 97           continue
 98           if(ipr.eq.1) then
                itop = nint(amin1(7.,real(ic(ip,id))))
                write(*,'(2x,a,7f8.2)') 'Intensities =   ',
     *          (aint(n),n=ibeg,ibeg+itop-1)
                write(*,'(2x,9f8.2)') (aint(n),n=ibeg+itop,iend)
              end if

            end if

 100     continue
 200  continue

c --- Nummern der atomaren Parameter bzw. integralen Intensitaeten
c     pro Datensatz abspeichern:

      do 500 id=1,idges
         iof = 0
         iend = 0
         do 400 ip=1,nphase(id)
            if(jobtyp.eq.1) then
              if(ip.gt.1) iof = iof + natom(ip-1)
              do 300 k=1,natom(ip)
                 na = k + iof
                 do 220 j=1,14
                    iyy = lato(j,na)
                    if(iyy.ne.0) ipar(iyy,id) = iyy
 220             continue
                 do 250 m=1,nbas(na)
                    do 240 j=1,2
                       iyy = lampl(j,m,na)
                       if(iyy.ne.0) ipar(iyy,id) = iyy
 240                continue
 250             continue
 300          continue
            else
              ibeg = iend + 1
              iend = iend + ic(ip,id)
              do 350 n=ibeg,iend
                 x = aint(n)
                 iyy = int(abs(x)/10.)
                 lint(n) = iyy
                 aint(n) = (abs(x)-10.*iyy)*sign(1.,x)
                 if(iyy.ne.0) ipar(iyy,id) = iyy
 350          continue
            end if
 400     continue
 500  continue


      return
      end
      

c******************************************************************************
c***                    Peak-shift correction                               ***
c******************************************************************************

      real function pkshift(theta,t,NC)

      parameter(PI=3.14159265359)
      real t(NC)

      d2t = t(1)
      do 100 m=2,NC
         d2t = d2t + t(m) * theta**(m-1)
100   continue

      pkshift = d2t
      return
      end
C******************************************************************************
C***                Plotfile anlegen (*.PLOT)                               ***
C******************************************************************************

      subroutine plotout(ka,plotfile,titel,step,temp,t,obs,calc,bac,
     *                   the,mit,izahl,IDZ)

      external pkshift
      integer mit(IDZ)
      character plotfile*80,titel*70
      real obs(IDZ), calc(IDZ), bac(IDZ), the(IDZ), t(3)


C --- Plotfile oeffnen:

      open(ka,file=plotfile,status='unknown',err=1000)

C --- Titel schreiben:

      write(ka,'(a)') titel

c --- Kopfzeile schreiben:

      xmin = the(1) + pkshift(the(1),t,3)
      xmax = the(izahl) + pkshift(the(izahl),t,3)
      write(ka,'(a,f15.4,f12.4,f15.4,f15.2,i10)') 
     *     'SIM',xmin,step,xmax,temp,izahl

c --- Datenformat:

 50   format(f12.4,4(1x,f12.0))
 
c --- Daten ins Plotfile schreiben:

      do 100 n=1,izahl
         ths = the(n) + pkshift(the(n),t,3)
         if(mit(n).ge.0) then
           write(ka,50) ths,obs(n),calc(n),obs(n)-calc(n),bac(n)
         else
           write(ka,50) ths,obs(n),0.,0.,0.
         end if
100   continue

      close(ka)
      return

1000  stop ' Cannot open plotfile'
      end


C******************************************************************************
C***             Korrektur der Vorzugsorientierung                          ***
C******************************************************************************

      real function precor(ipref,pref,g1,g2,al,h,sinlam,dpref1,dpref2)

      real pref(3), al(3,3), h(3)

      pp = 0.
      ph = 0.
      do 20 i=1,3
	 do 10 j=1,3
	    pp = pp + pref(i) * pref(j) * al(i,j)
	    ph = ph + pref(i) * h(j) * al(i,j)
10       continue
20    continue
      cosq   = ph**2 / (4.*sinlam) / pp

      if(ipref.eq.0) then

c --- Rietveld:

         gamma  = 1.570796327
         if(cosq.ne.0.) gamma = atan(sqrt(abs((1.-cosq)/cosq)))
         gam2   = gamma * gamma      
         prexp  = exp(-g1*gam2) 
         prf    = (1.-g2) * prexp
         precor = g2 + prf
         dpref1 = -gam2 * prf
         dpref2 = 1. - prexp

      else

c --- W.A. Dollase, J. Appl. Cryst. 19, 267-272 (1986):

         sinq = 1. - cosq
         u   = g1**2 * cosq + 1./g1 * sinq
         prf = 1./u**1.5
         precor = prf
         dpref1 = -1.5 * prf / u * (2. * g1 * cosq - 1./g1**2 * sinq)
         dpref2 = 0.
      end if

      return
      end


C******************************************************************************
C***                Profilberechnung (PATTERN.DAT)                          ***
C******************************************************************************

      subroutine procalc(krr,ibac,nprof,nphase,LPHASE,natom,sb,rlim,
     *                   wdt,thmin,thmax,step,izahl,excl,iexcl,IEZ,
     *                   glb,lglb,par,lpar,lato,lampl,nbas,IBV,NATS,
     *                   dersto,deriv,MSZ,obs,calc,the,bac,mit,IDZ,
     *                   theta,hwb,rint,lint,ic,IRS)

      external pkshift

      integer natom(LPHASE), mit(IDZ), lglb(10), lpar(19,LPHASE), 
     *        lato(17,NATS), krr(2,LPHASE,IDZ), ic(LPHASE),
     *        lampl(2,IBV,NATS), nbas(NATS), lint(IRS)
      real dersto(MSZ,IRS), glb(10), par(19,LPHASE), 
     *     deriv(MSZ), obs(IDZ), calc(IDZ), bac(IDZ), 
     *     the(IDZ), theta(IRS), hwb(IRS), rint(IRS), excl(2,IEZ)

      maxs  = 0
      izahl = nint((thmax - thmin)/step) + 1
      xmin  = thmin + pkshift(thmin,glb(1),3)  
      xmax  = thmax + pkshift(thmax,glb(1),3)
      do 30 n=1,izahl
         the(n) = thmin + (n-1) * step
 30   continue

      call assign(the,krr,mit,izahl,IDZ,LPHASE,glb,wdt,nphase,
     *            xmin,xmax,excl,iexcl,IEZ,theta,hwb,ic,IRS)

      do 100 n=1,izahl
         thecor  = the(n) + pkshift(the(n),glb(1),3)
         if(ibac.eq.0) then
           call backgr(bac(n),thecor,xmin,xmax,glb,lglb,aglb,
     *                 deriv,maxs,MSZ)
         else
           bac(n) = sb
         end if
         ycalc = 0.
         if(mit(n).gt.0) 
     *     call calcxn(ycalc,the(n),thecor,krr(1,1,n),nprof,nphase,
     *                 LPHASE,natom,rlim,lglb,par,lpar,lato,lampl,nbas,
     *                 IBV,NATS,dersto,deriv,maxs,MSZ,theta,hwb,
     *                 rint,lint,IRS)
         calc(n) = ycalc + bac(n)
         obs(n)  = calc(n)
100   continue

      return
      end

c******************************************************************************
c***        Berechnung der asymmetrischen Profilfunktion nach               ***
c***        J.-F. Berar and G. Baldinozzi                                   ***
c***        J. Appl. Cryst. (1993), 26, 128-129                             ***
c******************************************************************************

      real function profil(nprof,pos,pos0,eta1,eta2,a,hwb,rlim,
     *                     dPVdT,dPVdG,dPVdE1,dPVdE2,dPVdA)

      implicit none

      external Gauss, Lorentz, IntLor, ModLor, Edgeworth, PsVoigt,
     *         Pearson7, DbGauss, profval
      real Gauss, Lorentz, IntLor, ModLor, Edgeworth, PsVoigt,
     *         Pearson7, DbGauss, profval

      integer nprof
      logical Use_Asym
      real rlim, pos, pos0, hwb, a(4), eta1, eta2
      real x, dPRdX, dasdx, asym, prof, gss, theta
      real pi, fa, fb, dfa, dfb, as1, as2, tann, tann2
      real dPRdT, dPRdG, dPRdE1, dPRdE2
      real dPVdT, dPVdG, dPVdE1, dPVdE2, dPVdA(4)
      data pi / 3.14159265359 /


      if(nprof.eq.9) then

c --- Asymmetrische Pseudo-Voigt-Funktion nach Finger et al.:

         Use_Asym = .true.
         profil = profval(eta1,hwb,a(1),a(2),pos,pos0,
     *        dPVdT,dPVdG,dPVdE1,dPVdA(1),dPVdA(2),Use_Asym)
         dPVdE2   = 0.
         dPVdA(3) = 0.
         dPVdA(4) = 0.

      else

c --- Berechnung der Profilfunktionen und Ableitungen an der Stelle x:

         x = (pos - pos0) / hwb

         if(nprof.eq.1) then
            prof = Gauss(x,dPRdx) / hwb
            dPRdE1 = 0.
            dPRdE2 = 0.
         else if(nprof.eq.2) then
            prof = Lorentz(x,dPRdx) / hwb
            dPRdE1 = 0.
            dPRdE2 = 0.
         else if(nprof.eq.3) then
            prof = IntLor(x,dPRdx) / hwb
            dPRdE1 = 0.
            dPRdE2 = 0.
         else if(nprof.eq.4) then
            prof = ModLor(x,dPRdx) / hwb
            dPRdE1 = 0.
            dPRdE2 = 0.
         else if(nprof.eq.5) then
            prof = Edgeworth(x,eta1,eta2,dPRdx,dPRdE1,dPRdE2) / hwb
            dPRdE1 = dPRdE1 / hwb
            dPRdE2 = dPRdE2 / hwb
         else if(nprof.eq.6) then
            prof = PsVoigt(x,eta1,dPRdx,dPRdE1) / hwb
            dPRdE1 = dPRdE1 / hwb
            dPRdE2 = 0.
         else if(nprof.eq.7) then
            prof = Pearson7(x,eta1,dPRdx,dPRdE1) / hwb
            dPRdE1 = dPRdE1 / hwb
            dPRdE2 = 0.
         else if(nprof.eq.8) then
            prof = DbGauss(x,dPRdx) / hwb
            dPRdE1 = 0.
            dPRdE2 = 0.
         end if
         
         dPRdT  = -dPRdx / hwb**2
         dPRdG  = -prof / hwb - x * dPRdx / hwb**2
         
c --- Asymmetriekorrektur nur fuer pos0 <= rlim nach J.-F. Berar und 
c     G. Baldinozzi, J. Appl. Cryst., 26, 128-129 (1993)

         if(pos0.le.rlim) then
            theta = pi/360. * pos0
            tann = sin(theta) / cos(theta)
            tann2 = sin(2.*theta) / cos(2.*theta)
            gss = exp(-x**2)
            fa  = 2. * x * gss
            dfa = (2. - 4. * x**2) * gss
            fb  = (8. * x**3 - 12. * x) * gss
            dfb = (48. * x**2 - 14. * x**4 - 12.) * gss
            as1 = a(1) / tann + a(3) / tann2
            as2 = a(2) / tann + a(4) / tann2
            asym     = 1. + as1 * fa + as2 * fb
            profil   = prof * asym
            dasdx    = as1 * dfa + as2 * dfb
            dPVdT    = dPRdT * asym - prof * dasdx / hwb
            dPVdG    = dPRdG * asym - prof * dasdx * x / hwb
            dPVdE1   = dPRdE1 * asym
            dPVdE2   = dPRdE2 * asym
            dPVdA(1) = prof * fa / tann
            dPVdA(2) = prof * fb / tann
            dPVdA(3) = prof * fa / tann2
            dPVdA(4) = prof * fb / tann2
         else
            profil   = prof
            dPVdT    = dPRdT
            dPVdG    = dPRdG
            dPVdE1   = dPRdE1
            dPVdE2   = dPRdE2
            dPVdA(1) = 0.
            dPVdA(2) = 0.
            dPVdA(3) = 0.
            dPVdA(4) = 0.
         end if

      end if 

      return
      end

c******************************************************************************
c***                       Gaussfunktion                                    ***
c******************************************************************************

      real function Gauss(x, dPRdx)

      implicit none
      real g1, g2, x, y, temp, dPRdx
      data g1 /0.939437279/
      data g2 /2.772588722/

      y = g2 * x**2
      if(y.gt.23.) y = 23.
      temp  = g1 * exp(-y)
      dPRdx = -2.* g2 * x * temp
      Gauss = temp

      return
      end

c******************************************************************************
c***                      Lorentz-Funktion                                  ***
c******************************************************************************

      real function Lorentz(x, dPRdx)

      implicit none
      real c1, r1, x, y, dPRdx, temp
      data c1 /4.000000000/
      data r1 /0.636619772/

      y = 1. + c1 * x**2
      temp = r1 / y
      dPRdx = -2. * c1 * x * temp / y
      Lorentz = temp

      return
      end

c******************************************************************************
c***          Intermediate Lorentz-Funktion - Potenz 1.5                    ***
c******************************************************************************

      real function IntLor(x, dPRdx)

      implicit none
      real c15, r15, x, y, dPRdx, temp
      data c15 /2.349604208/
      data r15 /0.766420936/

      y = 1 + c15 * x**2
      temp = r15 / y**1.5
      dPRdx = -3. * c15 * x * temp / y
      IntLor = temp

      return
      end

c******************************************************************************
c***            Modifizierte Lorentz-Funktion - Potenz 2                    ***
c******************************************************************************

      real function ModLor(x, dPRdx)

      implicit none
      real c2, r2, x, y, dPRdx, temp
      data c2 /1.656854249/
      data r2 /0.819449653/

      y = 1. + c2 * x**2
      temp = r2 / y**2
      dPRdx = -4. * c2 * x * temp / y
      ModLor = temp

      return
      end

c******************************************************************************
c***                          Edgeworth Reihe                               ***
c******************************************************************************

      real function Edgeworth(x, eta1, eta2, dPRdx, dPRdE1, dPRdE2)

      implicit none
      real x, eta1, eta2, dPRdx, dPRdE1, dPRdE2
      real g1, g2, y, gauss, a1, a2, a3
      data g1 /0.939437279/
      data g2 /2.772588722/

      y = 2. * g2 * x**2.
      if(y.gt.46.) y = 46.
      Gauss = g1 * exp(-0.5*y)
      a1 = eta1/6.     * 4. * g2**2. * x * (3.-y)
      a2 = eta2/24.    * 4. * g2**2. * (y*(y-6.)+3.)
      a3 = eta1**2/72. * 8. * g2**3. * (y*(y*(y-15.)+45.)-15.)
      Edgeworth = Gauss * (1. - a1 + a2 + a3) 
      dPRdx  = Gauss * (-2. * g2 * x -
     *  eta1 / 6.      *  4. * g2**2. * (y*(y-6.)+3.) +
     *  eta2 / 24.     *  8. * g2**3. * x * (y*(10.-y)-15.) +
     *  eta1**2. / 72. * 16. * g2**4. * x * (y*(y*(84.-y)-210.)+105.) )
      dPRdE1 = Gauss * (a1 / eta1 + 2. * a3 / eta1)
      dPRdE2 = Gauss * a2 / eta2

      return
      end

c******************************************************************************
c***                      Pseudo-Voigt-Funktion                             ***
c******************************************************************************

      real function PsVoigt(x, eta, dPRdx, dPRdE)

      implicit none
      external Gauss, Lorentz
      real x, eta, dPRdx, dPRdE
      real Gauss, Lorentz, G, L, dGdx, dLdx

      G = Gauss(x,dGdx)
      L = Lorentz(x,dLdx)
      PsVoigt = eta * L + (1.-eta) * G
      dPRdx = eta * dLdx + (1.-eta) * dGdx
      dPRdE = L - G

      return
      end

c******************************************************************************
c***                       Pearson-VII-Funktion                             ***
c******************************************************************************

      real function Pearson7(x, eta, dPRdx, dPRdE)

      implicit none
      external gammln
      real x, eta, dPRdx, dPRdE
      real pi, cm, y, temp, gammln
      data pi /3.14159265359/

      cm = 4. * (2.**(1./eta) - 1.)
      y  = 1. + cm * x**2.
      temp  = sqrt(cm/pi) * exp(gammln(eta)-gammln(eta-0.5)) / y**eta
      dPRdx = -2. * eta * cm * x * temp / y
      dPRdE = -alog(y) * temp
      Pearson7 = temp

      return
      end

c******************************************************************************
c***                      Doppel-Gauss-Funktion                             ***
c******************************************************************************

      real function DbGauss(x0,dPRdx)

      implicit none
      external Gauss
      real Gauss, sc, x0, x1, x2, dPRdx, dGdx1, dGdx2 
      data sc /0.7786267/

      x1 = sc * x0
      x2 = 2. * sc * x0
      DBGauss = 0.4 * sc * (2. * Gauss(x1,dGdx1) + Gauss(x2,dGdx2))
      dPRdx = 0.8 * sc**2 * (dGdx1 + dGdx2)

      return
      end

c******************************************************************************
c***     Berechnung des nat. Logarithmus der Gamma-Funktion fuer xx>0       ***
c******************************************************************************

      real function gammln(xx)

      real*8 cof(6), stp, half, one, fpf, x, tmp, ser

      data cof, stp /76.18009173D0,-86.50532033D0,24.01409822D0,
     *     -1.231739516D0,.120858003D-2,-.536382D-5,2.50662827465D0/
      data half, one, fpf /0.5D0,1.0D0,5.5D0/

      x   = xx - one
      tmp = x + fpf
      tmp = (x + half) * log(tmp) - tmp
      ser = one

      do 11 j=1,6
	 x = x + one
	 ser = ser + cof(j) / x
 11   continue

      gammln = tmp + log(stp * ser)

      return
      end

c******************************************************************************
c***          Asymmetric Pseudo Voigt function                              ***
c******************************************************************************

c --- Version for Linux + f2c + gcc
c     H. Ritter (9-Sept-1998)

        real function Profval( Eta , Gamma , S_L , D_L , TwoTH , 
     1   TwoTH0 , dPRdT, dPRdG, dPRdE , dPRdS , dPRdD , Use_Asym )

c --- Returns value of Profile
c     Eta is the mixing coefficient between Gaussian and Lorentzian
c     Gamma is the FWHM
c     S_L is source width/detector distance
c     D_L is detector width/detector distance
c     TwoTH is point at which to evaluate the profile
c     TwoTH0 is two theta value for peak
c     dPRdT is derivative of profile wrt TwoTH0
c     dPRdG is derivative of profile wrt Gamma
c     dPRdE is derivative of profile wrt Eta
c     dPRdS is derivative of profile wrt S_L
c     dPRdD is derivative of profile wrt D_L
c     Use_Asym is true if asymmetry to be used

c --- Asymmetry due to axial divergence using the method of Finger, Cox and
c     Jephcoat, J. Appl. Cryst. 27, 892-900, 1994.

      implicit none
      real Eta , Gamma , S_L , D_L , TwoTH, RAD 
      real TwoTH0 , dPRdT, dPRdG, dPRdE , dPRdS , dPRdD
      logical Use_Asym
      integer NTERMS(14), Fstterm(14)

      integer ArrayNum , K , NGT, ngt2 , it, i
      real CsTH             	! cos(theta)
      real TTH                  ! tan(theta)
      real SnTwoTH		! sin(twoth)
      real CsTwoTH 		! cos(twoth)
      real ApB                  ! (S + H)/L 
      real AmB                  ! (S - H)/L 
      real ApB2 		! (ApB) **2
      real Einfl                ! 2phi value for inflection point 
      real Emin                 ! 2phi value for minimum 
      real dEmindA              ! derivative of Emin wrt A
      real tmp , tmp1 , tmp2    ! intermediate values 
      real WP(1883) , XP(1883)  ! Storage for Gauss-Legendre weights and intervals 
      real Delta                ! Angle of integration for comvolution 
      real dDELTAdA             ! derivative of DELTA wrt A (S/L)
      real sinDELTA             ! sine of DELTA 
      real cosDELTA             ! cosine of DELTA 
      real tanDELTA             ! tangent of DELTA 
      real RcosDELTA            ! 1/cos(DELTA) 
      real F , dFdA
      real G , dGdA , dGdB , PsVoigt2
      real sumWG , sumWRG , sumWdGdA , sumWRdGdA ,sumWdGdB , sumWRdGdB
      real sumWGdRdG , sumWGdRdE , sumWGdRdA , sumWGdRdB , sumWGdRd2t 

      data NTERMS /6,10,20,40,60,80,100,150,200,300,400,600,800,1000/
      data Fstterm /0,3,8,18,38,68,108,158,233,333,483,683,983,1383/
      data RAD /57.2957795/

c --- Values for the abscissas and weights of the Gauss-Legendre
c     N-point quadrature formula have been precomputed using routine
c     Gauleg from "Numerical Recipes" (Press, Flannery, Teukolsky
c     and Vetterling, 1986, Cambridge University Press,
c     ISBN 0 521 30811 9), and are stored in the DATA statements
c     for XP and WP below.

      data (xp(i),i=   1,  40)/
     *.2386192E+00,.6612094E+00,.9324695E+00,.1488743E+00,.4333954E+00,
     *.6794096E+00,.8650634E+00,.9739065E+00,.7652652E-01,.2277859E+00,
     *.3737061E+00,.5108670E+00,.6360537E+00,.7463319E+00,.8391170E+00,
     *.9122344E+00,.9639719E+00,.9931286E+00,.3877242E-01,.1160841E+00,
     *.1926976E+00,.2681522E+00,.3419941E+00,.4137792E+00,.4830758E+00,
     *.5494671E+00,.6125539E+00,.6719567E+00,.7273183E+00,.7783057E+00,
     *.8246122E+00,.8659595E+00,.9020988E+00,.9328128E+00,.9579168E+00,
     *.9772599E+00,.9907262E+00,.9982377E+00,.2595977E-01,.7780933E-01/
      data (xp(i),i=  41,  80)/
     *.1294491E+00,.1807400E+00,.2315436E+00,.2817229E+00,.3311428E+00,
     *.3796701E+00,.4271737E+00,.4735258E+00,.5186014E+00,.5622789E+00,
     *.6044406E+00,.6449728E+00,.6837663E+00,.7207165E+00,.7557238E+00,
     *.7886937E+00,.8195375E+00,.8481720E+00,.8745199E+00,.8985103E+00,
     *.9200785E+00,.9391663E+00,.9557223E+00,.9697018E+00,.9810672E+00,
     *.9897879E+00,.9958405E+00,.9992101E+00,.1951138E-01,.5850444E-01,
     *.9740840E-01,.1361640E+00,.1747123E+00,.2129945E+00,.2509524E+00,
     *.2885281E+00,.3256644E+00,.3623048E+00,.3983934E+00,.4338754E+00/
      data (xp(i),i=  81, 120)/
     *.4686966E+00,.5028041E+00,.5361459E+00,.5686713E+00,.6003306E+00,
     *.6310758E+00,.6608599E+00,.6896376E+00,.7173652E+00,.7440003E+00,
     *.7695024E+00,.7938327E+00,.8169541E+00,.8388315E+00,.8594314E+00,
     *.8787226E+00,.8966756E+00,.9132631E+00,.9284599E+00,.9422428E+00,
     *.9545908E+00,.9654851E+00,.9749091E+00,.9828486E+00,.9892913E+00,
     *.9942275E+00,.9976499E+00,.9995538E+00,.1562898E-01,.4687168E-01,
     *.7806858E-01,.1091892E+00,.1402031E+00,.1710801E+00,.2017899E+00,
     *.2323025E+00,.2625881E+00,.2926172E+00,.3223603E+00,.3517885E+00/
      data (xp(i),i= 121, 160)/
     *.3808730E+00,.4095853E+00,.4378974E+00,.4657816E+00,.4932108E+00,
     *.5201580E+00,.5465970E+00,.5725019E+00,.5978475E+00,.6226089E+00,
     *.6467619E+00,.6702830E+00,.6931492E+00,.7153381E+00,.7368281E+00,
     *.7575981E+00,.7776279E+00,.7968979E+00,.8153892E+00,.8330839E+00,
     *.8499645E+00,.8660147E+00,.8812187E+00,.8955616E+00,.9090296E+00,
     *.9216093E+00,.9332885E+00,.9440559E+00,.9539008E+00,.9628137E+00,
     *.9707858E+00,.9778094E+00,.9838775E+00,.9889844E+00,.9931249E+00,
     *.9962951E+00,.9984920E+00,.9997137E+00,.1043694E-01,.3130627E-01/
      data (xp(i),i= 161, 200)/
     *.5216195E-01,.7299491E-01,.9379607E-01,.1145563E+00,.1352667E+00,
     *.1559181E+00,.1765016E+00,.1970082E+00,.2174290E+00,.2377550E+00,
     *.2579774E+00,.2780874E+00,.2980762E+00,.3179352E+00,.3376556E+00,
     *.3572289E+00,.3766466E+00,.3959001E+00,.4149811E+00,.4338813E+00,
     *.4525925E+00,.4711065E+00,.4894151E+00,.5075106E+00,.5253849E+00,
     *.5430303E+00,.5604390E+00,.5776036E+00,.5945165E+00,.6111703E+00,
     *.6275579E+00,.6436720E+00,.6595056E+00,.6750519E+00,.6903041E+00,
     *.7052554E+00,.7198995E+00,.7342299E+00,.7482404E+00,.7619248E+00/
      data (xp(i),i= 201, 240)/
     *.7752773E+00,.7882919E+00,.8009631E+00,.8132853E+00,.8252531E+00,
     *.8368613E+00,.8481049E+00,.8589789E+00,.8694787E+00,.8795996E+00,
     *.8893372E+00,.8986874E+00,.9076460E+00,.9162090E+00,.9243729E+00,
     *.9321340E+00,.9394890E+00,.9464346E+00,.9529678E+00,.9590857E+00,
     *.9647858E+00,.9700655E+00,.9749225E+00,.9793548E+00,.9833603E+00,
     *.9869373E+00,.9900843E+00,.9927999E+00,.9950829E+00,.9969323E+00,
     *.9983473E+00,.9993274E+00,.9998723E+00,.7834291E-02,.2350095E-01,
     *.3916184E-01,.5481311E-01,.7045093E-01,.8607145E-01,.1016708E+00/
      data (xp(i),i= 241, 280)/
     *.1172453E+00,.1327909E+00,.1483040E+00,.1637806E+00,.1792170E+00,
     *.1946095E+00,.2099541E+00,.2252472E+00,.2404850E+00,.2556638E+00,
     *.2707798E+00,.2858293E+00,.3008086E+00,.3157141E+00,.3305421E+00,
     *.3452890E+00,.3599510E+00,.3745247E+00,.3890065E+00,.4033927E+00,
     *.4176799E+00,.4318646E+00,.4459432E+00,.4599124E+00,.4737686E+00,
     *.4875086E+00,.5011288E+00,.5146260E+00,.5279969E+00,.5412382E+00,
     *.5543465E+00,.5673188E+00,.5801518E+00,.5928424E+00,.6053874E+00,
     *.6177838E+00,.6300285E+00,.6421185E+00,.6540509E+00,.6658228E+00/
      data (xp(i),i= 281, 320)/
     *.6774311E+00,.6888732E+00,.7001461E+00,.7112472E+00,.7221736E+00,
     *.7329227E+00,.7434919E+00,.7538786E+00,.7640801E+00,.7740941E+00,
     *.7839181E+00,.7935496E+00,.8029862E+00,.8122257E+00,.8212659E+00,
     *.8301044E+00,.8387391E+00,.8471679E+00,.8553887E+00,.8633995E+00,
     *.8711983E+00,.8787832E+00,.8861524E+00,.8933041E+00,.9002364E+00,
     *.9069477E+00,.9134364E+00,.9197008E+00,.9257394E+00,.9315507E+00,
     *.9371333E+00,.9424859E+00,.9476071E+00,.9524956E+00,.9571503E+00,
     *.9615700E+00,.9657536E+00,.9697002E+00,.9734086E+00,.9768781E+00/
      data (xp(i),i= 321, 360)/
     *.9801078E+00,.9830968E+00,.9858445E+00,.9883502E+00,.9906132E+00,
     *.9926330E+00,.9944091E+00,.9959410E+00,.9972285E+00,.9982712E+00,
     *.9990687E+00,.9996210E+00,.9999281E+00,.5227245E-02,.1568116E-01,
     *.2613337E-01,.3658271E-01,.4702806E-01,.5746827E-01,.6790220E-01,
     *.7832871E-01,.8874665E-01,.9915490E-01,.1095523E+00,.1199377E+00,
     *.1303101E+00,.1406682E+00,.1510109E+00,.1613371E+00,.1716456E+00,
     *.1819354E+00,.1922054E+00,.2024543E+00,.2126811E+00,.2228846E+00,
     *.2330638E+00,.2432175E+00,.2533446E+00,.2634441E+00,.2735147E+00/
      data (xp(i),i= 361, 400)/
     *.2835555E+00,.2935652E+00,.3035429E+00,.3134874E+00,.3233976E+00,
     *.3332725E+00,.3431110E+00,.3529120E+00,.3626744E+00,.3723971E+00,
     *.3820792E+00,.3917194E+00,.4013169E+00,.4108705E+00,.4203792E+00,
     *.4298420E+00,.4392578E+00,.4486255E+00,.4579443E+00,.4672130E+00,
     *.4764306E+00,.4855961E+00,.4947086E+00,.5037670E+00,.5127704E+00,
     *.5217177E+00,.5306079E+00,.5394402E+00,.5482135E+00,.5569269E+00,
     *.5655795E+00,.5741702E+00,.5826982E+00,.5911624E+00,.5995621E+00,
     *.6078963E+00,.6161639E+00,.6243643E+00,.6324964E+00,.6405594E+00/
      data (xp(i),i= 401, 440)/
     *.6485524E+00,.6564744E+00,.6643248E+00,.6721025E+00,.6798068E+00,
     *.6874367E+00,.6949916E+00,.7024704E+00,.7098725E+00,.7171970E+00,
     *.7244432E+00,.7316101E+00,.7386971E+00,.7457033E+00,.7526281E+00,
     *.7594705E+00,.7662300E+00,.7729057E+00,.7794970E+00,.7860030E+00,
     *.7924232E+00,.7987567E+00,.8050030E+00,.8111612E+00,.8172308E+00,
     *.8232111E+00,.8291014E+00,.8349011E+00,.8406095E+00,.8462260E+00,
     *.8517501E+00,.8571811E+00,.8625184E+00,.8677614E+00,.8729095E+00,
     *.8779623E+00,.8829191E+00,.8877794E+00,.8925427E+00,.8972084E+00/
      data (xp(i),i= 441, 480)/
     *.9017761E+00,.9062452E+00,.9106152E+00,.9148857E+00,.9190563E+00,
     *.9231263E+00,.9270955E+00,.9309634E+00,.9347295E+00,.9383934E+00,
     *.9419548E+00,.9454132E+00,.9487683E+00,.9520197E+00,.9551671E+00,
     *.9582100E+00,.9611482E+00,.9639814E+00,.9667092E+00,.9693313E+00,
     *.9718476E+00,.9742575E+00,.9765610E+00,.9787578E+00,.9808476E+00,
     *.9828302E+00,.9847054E+00,.9864729E+00,.9881326E+00,.9896844E+00,
     *.9911279E+00,.9924632E+00,.9936899E+00,.9948081E+00,.9958175E+00,
     *.9967181E+00,.9975097E+00,.9981923E+00,.9987659E+00,.9992302E+00/
      data (xp(i),i= 481, 520)/
     *.9995854E+00,.9998313E+00,.9999680E+00,.3922075E-02,.1176598E-01,
     *.1960917E-01,.2745115E-01,.3529144E-01,.4312955E-01,.5096502E-01,
     *.5879735E-01,.6662606E-01,.7445067E-01,.8227070E-01,.9008566E-01,
     *.9789509E-01,.1056985E+00,.1134954E+00,.1212853E+00,.1290678E+00,
     *.1368423E+00,.1446083E+00,.1523655E+00,.1601134E+00,.1678513E+00,
     *.1755790E+00,.1832958E+00,.1910013E+00,.1986951E+00,.2063767E+00,
     *.2140456E+00,.2217013E+00,.2293434E+00,.2369713E+00,.2445847E+00,
     *.2521830E+00,.2597658E+00,.2673327E+00,.2748830E+00,.2824165E+00/
      data (xp(i),i= 521, 560)/
     *.2899326E+00,.2974308E+00,.3049108E+00,.3123719E+00,.3198139E+00,
     *.3272362E+00,.3346383E+00,.3420199E+00,.3493804E+00,.3567194E+00,
     *.3640365E+00,.3713311E+00,.3786029E+00,.3858515E+00,.3930762E+00,
     *.4002768E+00,.4074528E+00,.4146037E+00,.4217291E+00,.4288285E+00,
     *.4359016E+00,.4429478E+00,.4499667E+00,.4569580E+00,.4639212E+00,
     *.4708558E+00,.4777615E+00,.4846377E+00,.4914841E+00,.4983003E+00,
     *.5050859E+00,.5118403E+00,.5185633E+00,.5252543E+00,.5319131E+00,
     *.5385391E+00,.5451319E+00,.5516912E+00,.5582166E+00,.5647076E+00/
      data (xp(i),i= 561, 600)/
     *.5711639E+00,.5775851E+00,.5839707E+00,.5903203E+00,.5966337E+00,
     *.6029103E+00,.6091498E+00,.6153519E+00,.6215161E+00,.6276420E+00,
     *.6337293E+00,.6397777E+00,.6457866E+00,.6517559E+00,.6576850E+00,
     *.6635737E+00,.6694215E+00,.6752281E+00,.6809932E+00,.6867164E+00,
     *.6923974E+00,.6980357E+00,.7036311E+00,.7091832E+00,.7146916E+00,
     *.7201561E+00,.7255763E+00,.7309518E+00,.7362823E+00,.7415676E+00,
     *.7468072E+00,.7520008E+00,.7571482E+00,.7622490E+00,.7673029E+00,
     *.7723096E+00,.7772688E+00,.7821801E+00,.7870433E+00,.7918581E+00/
      data (xp(i),i= 601, 640)/
     *.7966241E+00,.8013412E+00,.8060089E+00,.8106271E+00,.8151953E+00,
     *.8197134E+00,.8241811E+00,.8285980E+00,.8329640E+00,.8372787E+00,
     *.8415419E+00,.8457533E+00,.8499127E+00,.8540198E+00,.8580743E+00,
     *.8620760E+00,.8660247E+00,.8699201E+00,.8737620E+00,.8775501E+00,
     *.8812842E+00,.8849641E+00,.8885896E+00,.8921603E+00,.8956762E+00,
     *.8991369E+00,.9025424E+00,.9058923E+00,.9091864E+00,.9124246E+00,
     *.9156067E+00,.9187324E+00,.9218016E+00,.9248141E+00,.9277697E+00,
     *.9306682E+00,.9335094E+00,.9362932E+00,.9390194E+00,.9416878E+00/
      data (xp(i),i= 641, 680)/
     *.9442982E+00,.9468506E+00,.9493447E+00,.9517803E+00,.9541574E+00,
     *.9564759E+00,.9587354E+00,.9609360E+00,.9630774E+00,.9651596E+00,
     *.9671823E+00,.9691456E+00,.9710493E+00,.9728932E+00,.9746772E+00,
     *.9764012E+00,.9780652E+00,.9796690E+00,.9812125E+00,.9826957E+00,
     *.9841183E+00,.9854805E+00,.9867820E+00,.9880227E+00,.9892027E+00,
     *.9903218E+00,.9913800E+00,.9923771E+00,.9933133E+00,.9941882E+00,
     *.9950021E+00,.9957547E+00,.9964460E+00,.9970760E+00,.9976447E+00,
     *.9981519E+00,.9985978E+00,.9989822E+00,.9993052E+00,.9995666E+00/
      data (xp(i),i= 681, 720)/
     *.9997666E+00,.9999050E+00,.9999820E+00,.2615810E-02,.7847359E-02,
     *.1307869E-01,.1830967E-01,.2354014E-01,.2876997E-01,.3399902E-01,
     *.3922713E-01,.4445417E-01,.4967999E-01,.5490445E-01,.6012741E-01,
     *.6534873E-01,.7056825E-01,.7578585E-01,.8100137E-01,.8621467E-01,
     *.9142561E-01,.9663405E-01,.1018398E+00,.1070429E+00,.1122429E+00,
     *.1174399E+00,.1226337E+00,.1278242E+00,.1330111E+00,.1381944E+00,
     *.1433739E+00,.1485495E+00,.1537210E+00,.1588884E+00,.1640513E+00,
     *.1692098E+00,.1743636E+00,.1795127E+00,.1846569E+00,.1897960E+00/
      data (xp(i),i= 721, 760)/
     *.1949299E+00,.2000585E+00,.2051816E+00,.2102991E+00,.2154108E+00,
     *.2205166E+00,.2256164E+00,.2307101E+00,.2357974E+00,.2408782E+00,
     *.2459525E+00,.2510200E+00,.2560807E+00,.2611343E+00,.2661808E+00,
     *.2712201E+00,.2762519E+00,.2812761E+00,.2862926E+00,.2913013E+00,
     *.2963021E+00,.3012947E+00,.3062790E+00,.3112550E+00,.3162225E+00,
     *.3211813E+00,.3261313E+00,.3310724E+00,.3360045E+00,.3409273E+00,
     *.3458408E+00,.3507449E+00,.3556393E+00,.3605240E+00,.3653989E+00,
     *.3702637E+00,.3751184E+00,.3799629E+00,.3847969E+00,.3896204E+00/
      data (xp(i),i= 761, 800)/
     *.3944333E+00,.3992353E+00,.4040264E+00,.4088065E+00,.4135754E+00,
     *.4183329E+00,.4230790E+00,.4278136E+00,.4325364E+00,.4372474E+00,
     *.4419464E+00,.4466333E+00,.4513080E+00,.4559703E+00,.4606202E+00,
     *.4652574E+00,.4698819E+00,.4744936E+00,.4790923E+00,.4836778E+00,
     *.4882502E+00,.4928091E+00,.4973546E+00,.5018864E+00,.5064046E+00,
     *.5109088E+00,.5153991E+00,.5198753E+00,.5243372E+00,.5287848E+00,
     *.5332179E+00,.5376364E+00,.5420402E+00,.5464292E+00,.5508032E+00,
     *.5551622E+00,.5595059E+00,.5638343E+00,.5681473E+00,.5724448E+00/
      data (xp(i),i= 801, 840)/
     *.5767266E+00,.5809926E+00,.5852427E+00,.5894768E+00,.5936947E+00,
     *.5978964E+00,.6020817E+00,.6062506E+00,.6104028E+00,.6145384E+00,
     *.6186571E+00,.6227589E+00,.6268437E+00,.6309113E+00,.6349616E+00,
     *.6389945E+00,.6430100E+00,.6470079E+00,.6509880E+00,.6549504E+00,
     *.6588948E+00,.6628211E+00,.6667294E+00,.6706194E+00,.6744910E+00,
     *.6783442E+00,.6821788E+00,.6859947E+00,.6897919E+00,.6935702E+00,
     *.6973295E+00,.7010697E+00,.7047907E+00,.7084924E+00,.7121748E+00,
     *.7158376E+00,.7194809E+00,.7231044E+00,.7267082E+00,.7302921E+00/
      data (xp(i),i= 841, 880)/
     *.7338560E+00,.7373998E+00,.7409234E+00,.7444268E+00,.7479097E+00,
     *.7513722E+00,.7548142E+00,.7582355E+00,.7616360E+00,.7650157E+00,
     *.7683744E+00,.7717121E+00,.7750287E+00,.7783241E+00,.7815982E+00,
     *.7848508E+00,.7880821E+00,.7912917E+00,.7944797E+00,.7976459E+00,
     *.8007903E+00,.8039128E+00,.8070132E+00,.8100916E+00,.8131479E+00,
     *.8161818E+00,.8191934E+00,.8221826E+00,.8251493E+00,.8280935E+00,
     *.8310149E+00,.8339136E+00,.8367895E+00,.8396425E+00,.8424725E+00,
     *.8452794E+00,.8480632E+00,.8508238E+00,.8535611E+00,.8562750E+00/
      data (xp(i),i= 881, 920)/
     *.8589656E+00,.8616325E+00,.8642760E+00,.8668957E+00,.8694918E+00,
     *.8720640E+00,.8746124E+00,.8771368E+00,.8796372E+00,.8821136E+00,
     *.8845658E+00,.8869937E+00,.8893975E+00,.8917768E+00,.8941318E+00,
     *.8964623E+00,.8987683E+00,.9010496E+00,.9033063E+00,.9055383E+00,
     *.9077455E+00,.9099278E+00,.9120852E+00,.9142177E+00,.9163252E+00,
     *.9184075E+00,.9204648E+00,.9224968E+00,.9245036E+00,.9264851E+00,
     *.9284412E+00,.9303720E+00,.9322772E+00,.9341570E+00,.9360111E+00,
     *.9378397E+00,.9396426E+00,.9414198E+00,.9431712E+00,.9448967E+00/
      data (xp(i),i= 921, 960)/
     *.9465965E+00,.9482703E+00,.9499181E+00,.9515400E+00,.9531358E+00,
     *.9547056E+00,.9562492E+00,.9577666E+00,.9592578E+00,.9607228E+00,
     *.9621615E+00,.9635738E+00,.9649597E+00,.9663193E+00,.9676524E+00,
     *.9689590E+00,.9702391E+00,.9714927E+00,.9727196E+00,.9739199E+00,
     *.9750936E+00,.9762406E+00,.9773609E+00,.9784544E+00,.9795211E+00,
     *.9805610E+00,.9815741E+00,.9825603E+00,.9835197E+00,.9844521E+00,
     *.9853575E+00,.9862360E+00,.9870876E+00,.9879120E+00,.9887095E+00,
     *.9894799E+00,.9902232E+00,.9909394E+00,.9916285E+00,.9922904E+00/
      data (xp(i),i= 961,1000)/
     *.9929252E+00,.9935328E+00,.9941132E+00,.9946664E+00,.9951924E+00,
     *.9956911E+00,.9961626E+00,.9966068E+00,.9970238E+00,.9974135E+00,
     *.9977758E+00,.9981109E+00,.9984186E+00,.9986990E+00,.9989521E+00,
     *.9991778E+00,.9993762E+00,.9995472E+00,.9996909E+00,.9998072E+00,
     *.9998962E+00,.9999577E+00,.9999920E+00,.1962267E-02,.5886772E-02,
     *.9811186E-02,.1373545E-01,.1765950E-01,.2158328E-01,.2550673E-01,
     *.2942978E-01,.3335238E-01,.3727447E-01,.4119598E-01,.4511686E-01,
     *.4903704E-01,.5295647E-01,.5687508E-01,.6079282E-01,.6470962E-01/
      data (xp(i),i=1001,1040)/
     *.6862542E-01,.7254017E-01,.7645380E-01,.8036625E-01,.8427746E-01,
     *.8818738E-01,.9209594E-01,.9600308E-01,.9990874E-01,.1038129E+00,
     *.1077154E+00,.1116162E+00,.1155154E+00,.1194128E+00,.1233083E+00,
     *.1272019E+00,.1310936E+00,.1349832E+00,.1388708E+00,.1427562E+00,
     *.1466395E+00,.1505204E+00,.1543991E+00,.1582754E+00,.1621492E+00,
     *.1660205E+00,.1698893E+00,.1737555E+00,.1776190E+00,.1814798E+00,
     *.1853377E+00,.1891928E+00,.1930450E+00,.1968942E+00,.2007404E+00,
     *.2045835E+00,.2084235E+00,.2122602E+00,.2160937E+00,.2199238E+00/
      data (xp(i),i=1041,1080)/
     *.2237505E+00,.2275738E+00,.2313936E+00,.2352099E+00,.2390225E+00,
     *.2428314E+00,.2466366E+00,.2504380E+00,.2542355E+00,.2580292E+00,
     *.2618188E+00,.2656044E+00,.2693859E+00,.2731633E+00,.2769365E+00,
     *.2807054E+00,.2844699E+00,.2882301E+00,.2919859E+00,.2957372E+00,
     *.2994839E+00,.3032259E+00,.3069634E+00,.3106961E+00,.3144240E+00,
     *.3181470E+00,.3218652E+00,.3255784E+00,.3292866E+00,.3329897E+00,
     *.3366877E+00,.3403805E+00,.3440681E+00,.3477503E+00,.3514272E+00,
     *.3550987E+00,.3587648E+00,.3624253E+00,.3660802E+00,.3697295E+00/
      data (xp(i),i=1081,1120)/
     *.3733731E+00,.3770109E+00,.3806429E+00,.3842691E+00,.3878893E+00,
     *.3915036E+00,.3951118E+00,.3987140E+00,.4023100E+00,.4058998E+00,
     *.4094834E+00,.4130607E+00,.4166316E+00,.4201960E+00,.4237541E+00,
     *.4273055E+00,.4308504E+00,.4343887E+00,.4379203E+00,.4414451E+00,
     *.4449632E+00,.4484743E+00,.4519786E+00,.4554759E+00,.4589662E+00,
     *.4624494E+00,.4659255E+00,.4693945E+00,.4728562E+00,.4763106E+00,
     *.4797577E+00,.4831973E+00,.4866296E+00,.4900543E+00,.4934715E+00,
     *.4968812E+00,.5002831E+00,.5036774E+00,.5070638E+00,.5104425E+00/
      data (xp(i),i=1121,1160)/
     *.5138133E+00,.5171762E+00,.5205312E+00,.5238781E+00,.5272169E+00,
     *.5305477E+00,.5338702E+00,.5371846E+00,.5404906E+00,.5437884E+00,
     *.5470777E+00,.5503587E+00,.5536311E+00,.5568951E+00,.5601504E+00,
     *.5633972E+00,.5666352E+00,.5698645E+00,.5730851E+00,.5762968E+00,
     *.5794996E+00,.5826936E+00,.5858785E+00,.5890544E+00,.5922213E+00,
     *.5953790E+00,.5985276E+00,.6016669E+00,.6047970E+00,.6079177E+00,
     *.6110291E+00,.6141311E+00,.6172236E+00,.6203066E+00,.6233801E+00,
     *.6264440E+00,.6294982E+00,.6325427E+00,.6355775E+00,.6386024E+00/
      data (xp(i),i=1161,1200)/
     *.6416176E+00,.6446229E+00,.6476182E+00,.6506036E+00,.6535789E+00,
     *.6565442E+00,.6594994E+00,.6624444E+00,.6653792E+00,.6683037E+00,
     *.6712180E+00,.6741219E+00,.6770155E+00,.6798986E+00,.6827712E+00,
     *.6856333E+00,.6884849E+00,.6913259E+00,.6941562E+00,.6969758E+00,
     *.6997847E+00,.7025828E+00,.7053701E+00,.7081465E+00,.7109120E+00,
     *.7136666E+00,.7164102E+00,.7191427E+00,.7218642E+00,.7245746E+00,
     *.7272737E+00,.7299617E+00,.7326385E+00,.7353039E+00,.7379581E+00,
     *.7406008E+00,.7432322E+00,.7458521E+00,.7484606E+00,.7510575E+00/
      data (xp(i),i=1201,1240)/
     *.7536428E+00,.7562165E+00,.7587786E+00,.7613290E+00,.7638676E+00,
     *.7663945E+00,.7689096E+00,.7714129E+00,.7739043E+00,.7763837E+00,
     *.7788512E+00,.7813067E+00,.7837502E+00,.7861816E+00,.7886009E+00,
     *.7910080E+00,.7934030E+00,.7957857E+00,.7981562E+00,.8005144E+00,
     *.8028602E+00,.8051937E+00,.8075148E+00,.8098234E+00,.8121196E+00,
     *.8144033E+00,.8166744E+00,.8189330E+00,.8211789E+00,.8234122E+00,
     *.8256328E+00,.8278407E+00,.8300358E+00,.8322182E+00,.8343877E+00,
     *.8365444E+00,.8386882E+00,.8408191E+00,.8429370E+00,.8450420E+00/
      data (xp(i),i=1241,1280)/
     *.8471339E+00,.8492128E+00,.8512786E+00,.8533313E+00,.8553709E+00,
     *.8573972E+00,.8594104E+00,.8614104E+00,.8633970E+00,.8653704E+00,
     *.8673304E+00,.8692771E+00,.8712104E+00,.8731303E+00,.8750367E+00,
     *.8769297E+00,.8788091E+00,.8806750E+00,.8825274E+00,.8843662E+00,
     *.8861913E+00,.8880028E+00,.8898006E+00,.8915847E+00,.8933550E+00,
     *.8951117E+00,.8968545E+00,.8985835E+00,.9002987E+00,.9020000E+00,
     *.9036874E+00,.9053609E+00,.9070204E+00,.9086660E+00,.9102976E+00,
     *.9119152E+00,.9135187E+00,.9151081E+00,.9166835E+00,.9182447E+00/
      data (xp(i),i=1281,1320)/
     *.9197918E+00,.9213247E+00,.9228435E+00,.9243480E+00,.9258383E+00,
     *.9273143E+00,.9287760E+00,.9302235E+00,.9316566E+00,.9330754E+00,
     *.9344797E+00,.9358697E+00,.9372453E+00,.9386065E+00,.9399532E+00,
     *.9412854E+00,.9426031E+00,.9439063E+00,.9451950E+00,.9464691E+00,
     *.9477286E+00,.9489735E+00,.9502038E+00,.9514195E+00,.9526205E+00,
     *.9538069E+00,.9549785E+00,.9561355E+00,.9572777E+00,.9584052E+00,
     *.9595179E+00,.9606159E+00,.9616990E+00,.9627673E+00,.9638208E+00,
     *.9648595E+00,.9658833E+00,.9668922E+00,.9678863E+00,.9688654E+00/
      data (xp(i),i=1321,1360)/
     *.9698296E+00,.9707788E+00,.9717132E+00,.9726325E+00,.9735369E+00,
     *.9744262E+00,.9753006E+00,.9761600E+00,.9770043E+00,.9778335E+00,
     *.9786477E+00,.9794468E+00,.9802309E+00,.9809998E+00,.9817537E+00,
     *.9824924E+00,.9832160E+00,.9839244E+00,.9846177E+00,.9852958E+00,
     *.9859587E+00,.9866065E+00,.9872390E+00,.9878564E+00,.9884585E+00,
     *.9890455E+00,.9896171E+00,.9901736E+00,.9907148E+00,.9912407E+00,
     *.9917514E+00,.9922468E+00,.9927269E+00,.9931917E+00,.9936412E+00,
     *.9940754E+00,.9944943E+00,.9948979E+00,.9952862E+00,.9956591E+00/
      data (xp(i),i=1361,1400)/
     *.9960167E+00,.9963590E+00,.9966859E+00,.9969974E+00,.9972936E+00,
     *.9975745E+00,.9978400E+00,.9980901E+00,.9983248E+00,.9985442E+00,
     *.9987482E+00,.9989368E+00,.9991100E+00,.9992678E+00,.9994103E+00,
     *.9995373E+00,.9996489E+00,.9997452E+00,.9998261E+00,.9998915E+00,
     *.9999416E+00,.9999762E+00,.9999955E+00,.1570010E-02,.4710016E-02,
     *.7849975E-02,.1098986E-01,.1412963E-01,.1726926E-01,.2040873E-01,
     *.2354799E-01,.2668702E-01,.2982579E-01,.3296426E-01,.3610241E-01,
     *.3924020E-01,.4237761E-01,.4551459E-01,.4865113E-01,.5178719E-01/
      data (xp(i),i=1401,1440)/
     *.5492274E-01,.5805775E-01,.6119218E-01,.6432601E-01,.6745921E-01,
     *.7059174E-01,.7372358E-01,.7685468E-01,.7998504E-01,.8311460E-01,
     *.8624334E-01,.8937123E-01,.9249824E-01,.9562434E-01,.9874950E-01,
     *.1018737E+00,.1049969E+00,.1081190E+00,.1112401E+00,.1143601E+00,
     *.1174789E+00,.1205966E+00,.1237131E+00,.1268284E+00,.1299424E+00,
     *.1330552E+00,.1361666E+00,.1392767E+00,.1423855E+00,.1454928E+00,
     *.1485987E+00,.1517031E+00,.1548060E+00,.1579074E+00,.1610073E+00,
     *.1641055E+00,.1672022E+00,.1702971E+00,.1733905E+00,.1764821E+00/
      data (xp(i),i=1441,1480)/
     *.1795719E+00,.1826600E+00,.1857463E+00,.1888308E+00,.1919134E+00,
     *.1949941E+00,.1980728E+00,.2011497E+00,.2042245E+00,.2072973E+00,
     *.2103681E+00,.2134368E+00,.2165035E+00,.2195679E+00,.2226302E+00,
     *.2256904E+00,.2287482E+00,.2318039E+00,.2348572E+00,.2379083E+00,
     *.2409569E+00,.2440033E+00,.2470472E+00,.2500886E+00,.2531276E+00,
     *.2561641E+00,.2591981E+00,.2622295E+00,.2652584E+00,.2682846E+00,
     *.2713082E+00,.2743291E+00,.2773473E+00,.2803628E+00,.2833755E+00,
     *.2863854E+00,.2893925E+00,.2923967E+00,.2953980E+00,.2983965E+00/
      data (xp(i),i=1481,1520)/
     *.3013920E+00,.3043845E+00,.3073740E+00,.3103605E+00,.3133439E+00,
     *.3163243E+00,.3193015E+00,.3222756E+00,.3252465E+00,.3282141E+00,
     *.3311786E+00,.3341398E+00,.3370977E+00,.3400522E+00,.3430035E+00,
     *.3459513E+00,.3488957E+00,.3518367E+00,.3547742E+00,.3577082E+00,
     *.3606387E+00,.3635657E+00,.3664890E+00,.3694087E+00,.3723248E+00,
     *.3752373E+00,.3781460E+00,.3810510E+00,.3839522E+00,.3868497E+00,
     *.3897433E+00,.3926331E+00,.3955190E+00,.3984010E+00,.4012791E+00,
     *.4041533E+00,.4070234E+00,.4098896E+00,.4127517E+00,.4156097E+00/
      data (xp(i),i=1521,1560)/
     *.4184636E+00,.4213134E+00,.4241591E+00,.4270006E+00,.4298378E+00,
     *.4326708E+00,.4354996E+00,.4383241E+00,.4411442E+00,.4439600E+00,
     *.4467714E+00,.4495784E+00,.4523810E+00,.4551791E+00,.4579727E+00,
     *.4607618E+00,.4635464E+00,.4663264E+00,.4691018E+00,.4718726E+00,
     *.4746387E+00,.4774001E+00,.4801569E+00,.4829089E+00,.4856561E+00,
     *.4883986E+00,.4911362E+00,.4938690E+00,.4965969E+00,.4993199E+00,
     *.5020381E+00,.5047512E+00,.5074594E+00,.5101626E+00,.5128607E+00,
     *.5155538E+00,.5182418E+00,.5209247E+00,.5236025E+00,.5262750E+00/
      data (xp(i),i=1561,1600)/
     *.5289425E+00,.5316046E+00,.5342616E+00,.5369133E+00,.5395597E+00,
     *.5422007E+00,.5448365E+00,.5474668E+00,.5500918E+00,.5527113E+00,
     *.5553254E+00,.5579340E+00,.5605371E+00,.5631347E+00,.5657267E+00,
     *.5683131E+00,.5708940E+00,.5734692E+00,.5760387E+00,.5786026E+00,
     *.5811608E+00,.5837132E+00,.5862599E+00,.5888008E+00,.5913360E+00,
     *.5938652E+00,.5963886E+00,.5989062E+00,.6014178E+00,.6039235E+00,
     *.6064233E+00,.6089170E+00,.6114048E+00,.6138865E+00,.6163622E+00,
     *.6188318E+00,.6212953E+00,.6237527E+00,.6262040E+00,.6286490E+00/
      data (xp(i),i=1601,1640)/
     *.6310879E+00,.6335205E+00,.6359469E+00,.6383670E+00,.6407808E+00,
     *.6431883E+00,.6455895E+00,.6479843E+00,.6503727E+00,.6527547E+00,
     *.6551303E+00,.6574994E+00,.6598620E+00,.6622181E+00,.6645677E+00,
     *.6669107E+00,.6692472E+00,.6715770E+00,.6739003E+00,.6762169E+00,
     *.6785268E+00,.6808300E+00,.6831266E+00,.6854163E+00,.6876994E+00,
     *.6899756E+00,.6922451E+00,.6945077E+00,.6967635E+00,.6990124E+00,
     *.7012544E+00,.7034895E+00,.7057176E+00,.7079388E+00,.7101530E+00,
     *.7123603E+00,.7145605E+00,.7167536E+00,.7189397E+00,.7211187E+00/
      data (xp(i),i=1641,1680)/
     *.7232906E+00,.7254553E+00,.7276129E+00,.7297634E+00,.7319066E+00,
     *.7340426E+00,.7361714E+00,.7382929E+00,.7404071E+00,.7425141E+00,
     *.7446137E+00,.7467060E+00,.7487909E+00,.7508684E+00,.7529385E+00,
     *.7550013E+00,.7570565E+00,.7591043E+00,.7611446E+00,.7631774E+00,
     *.7652027E+00,.7672204E+00,.7692306E+00,.7712332E+00,.7732282E+00,
     *.7752155E+00,.7771953E+00,.7791673E+00,.7811317E+00,.7830884E+00,
     *.7850373E+00,.7869785E+00,.7889120E+00,.7908376E+00,.7927555E+00,
     *.7946656E+00,.7965678E+00,.7984622E+00,.8003486E+00,.8022273E+00/
      data (xp(i),i=1681,1720)/
     *.8040979E+00,.8059607E+00,.8078155E+00,.8096624E+00,.8115013E+00,
     *.8133321E+00,.8151550E+00,.8169698E+00,.8187765E+00,.8205752E+00,
     *.8223658E+00,.8241483E+00,.8259227E+00,.8276889E+00,.8294470E+00,
     *.8311968E+00,.8329385E+00,.8346720E+00,.8363972E+00,.8381142E+00,
     *.8398229E+00,.8415234E+00,.8432156E+00,.8448994E+00,.8465749E+00,
     *.8482421E+00,.8499009E+00,.8515513E+00,.8531933E+00,.8548269E+00,
     *.8564521E+00,.8580688E+00,.8596771E+00,.8612769E+00,.8628682E+00,
     *.8644510E+00,.8660253E+00,.8675910E+00,.8691482E+00,.8706968E+00/
      data (xp(i),i=1721,1760)/
     *.8722369E+00,.8737683E+00,.8752911E+00,.8768053E+00,.8783108E+00,
     *.8798077E+00,.8812959E+00,.8827754E+00,.8842463E+00,.8857083E+00,
     *.8871617E+00,.8886063E+00,.8900422E+00,.8914692E+00,.8928875E+00,
     *.8942970E+00,.8956977E+00,.8970895E+00,.8984725E+00,.8998466E+00,
     *.9012119E+00,.9025683E+00,.9039157E+00,.9052543E+00,.9065839E+00,
     *.9079046E+00,.9092164E+00,.9105192E+00,.9118130E+00,.9130978E+00,
     *.9143736E+00,.9156404E+00,.9168982E+00,.9181469E+00,.9193866E+00,
     *.9206172E+00,.9218387E+00,.9230511E+00,.9242545E+00,.9254487E+00/
      data (xp(i),i=1761,1800)/
     *.9266338E+00,.9278098E+00,.9289766E+00,.9301343E+00,.9312828E+00,
     *.9324221E+00,.9335522E+00,.9346731E+00,.9357848E+00,.9368872E+00,
     *.9379805E+00,.9390645E+00,.9401392E+00,.9412046E+00,.9422608E+00,
     *.9433077E+00,.9443453E+00,.9453735E+00,.9463925E+00,.9474021E+00,
     *.9484024E+00,.9493933E+00,.9503749E+00,.9513471E+00,.9523099E+00,
     *.9532633E+00,.9542073E+00,.9551420E+00,.9560672E+00,.9569829E+00,
     *.9578893E+00,.9587862E+00,.9596736E+00,.9605516E+00,.9614201E+00,
     *.9622791E+00,.9631287E+00,.9639687E+00,.9647992E+00,.9656203E+00/
      data (xp(i),i=1801,1840)/
     *.9664318E+00,.9672338E+00,.9680262E+00,.9688091E+00,.9695824E+00,
     *.9703462E+00,.9711004E+00,.9718451E+00,.9725801E+00,.9733056E+00,
     *.9740215E+00,.9747278E+00,.9754244E+00,.9761115E+00,.9767889E+00,
     *.9774567E+00,.9781148E+00,.9787633E+00,.9794022E+00,.9800314E+00,
     *.9806509E+00,.9812608E+00,.9818610E+00,.9824515E+00,.9830323E+00,
     *.9836035E+00,.9841649E+00,.9847166E+00,.9852586E+00,.9857909E+00,
     *.9863135E+00,.9868264E+00,.9873295E+00,.9878229E+00,.9883066E+00,
     *.9887805E+00,.9892447E+00,.9896991E+00,.9901437E+00,.9905786E+00/
      data (xp(i),i=1841,1880)/
     *.9910037E+00,.9914191E+00,.9918247E+00,.9922205E+00,.9926065E+00,
     *.9929827E+00,.9933492E+00,.9937058E+00,.9940527E+00,.9943897E+00,
     *.9947169E+00,.9950344E+00,.9953420E+00,.9956398E+00,.9959278E+00,
     *.9962060E+00,.9964743E+00,.9967328E+00,.9969815E+00,.9972204E+00,
     *.9974494E+00,.9976686E+00,.9978780E+00,.9980775E+00,.9982672E+00,
     *.9984471E+00,.9986171E+00,.9987772E+00,.9989275E+00,.9990680E+00,
     *.9991986E+00,.9993193E+00,.9994302E+00,.9995313E+00,.9996225E+00,
     *.9997038E+00,.9997753E+00,.9998369E+00,.9998886E+00,.9999306E+00/
      data (xp(i),i=1881,1883)/
     *.9999626E+00,.9999848E+00,.9999971E+00/
      data (wp(i),i=   1,  40)/
     *.4679139E+00,.3607616E+00,.1713245E+00,.2955242E+00,.2692667E+00,
     *.2190864E+00,.1494513E+00,.6667134E-01,.1527534E+00,.1491730E+00,
     *.1420961E+00,.1316886E+00,.1181945E+00,.1019301E+00,.8327674E-01,
     *.6267205E-01,.4060143E-01,.1761401E-01,.7750595E-01,.7703982E-01,
     *.7611036E-01,.7472317E-01,.7288658E-01,.7061165E-01,.6791205E-01,
     *.6480401E-01,.6130624E-01,.5743977E-01,.5322785E-01,.4869581E-01,
     *.4387091E-01,.3878217E-01,.3346020E-01,.2793701E-01,.2224585E-01,
     *.1642106E-01,.1049828E-01,.4521277E-02,.5190788E-01,.5176794E-01/
      data (wp(i),i=  41,  80)/
     *.5148845E-01,.5107016E-01,.5051418E-01,.4982204E-01,.4899558E-01,
     *.4803703E-01,.4694899E-01,.4573438E-01,.4439648E-01,.4293889E-01,
     *.4136555E-01,.3968070E-01,.3788887E-01,.3599490E-01,.3400389E-01,
     *.3192122E-01,.2975249E-01,.2750356E-01,.2518048E-01,.2278952E-01,
     *.2033712E-01,.1782990E-01,.1527462E-01,.1267817E-01,.1004756E-01,
     *.7389931E-02,.4712730E-02,.2026812E-02,.3901781E-01,.3895840E-01,
     *.3883965E-01,.3866176E-01,.3842499E-01,.3812971E-01,.3777636E-01,
     *.3736549E-01,.3689771E-01,.3637375E-01,.3579439E-01,.3516053E-01/
      data (wp(i),i=  81, 120)/
     *.3447312E-01,.3373321E-01,.3294194E-01,.3210050E-01,.3121017E-01,
     *.3027232E-01,.2928837E-01,.2825982E-01,.2718823E-01,.2607524E-01,
     *.2492254E-01,.2373188E-01,.2250509E-01,.2124403E-01,.1995061E-01,
     *.1862681E-01,.1727465E-01,.1589618E-01,.1449351E-01,.1306876E-01,
     *.1162411E-01,.1016177E-01,.8683945E-02,.7192905E-02,.5690922E-02,
     *.4180313E-02,.2663534E-02,.1144950E-02,.3125542E-01,.3122488E-01,
     *.3116384E-01,.3107234E-01,.3095048E-01,.3079838E-01,.3061619E-01,
     *.3040408E-01,.3016227E-01,.2989098E-01,.2959049E-01,.2926108E-01/
      data (wp(i),i= 121, 160)/
     *.2890309E-01,.2851685E-01,.2810276E-01,.2766120E-01,.2719261E-01,
     *.2669746E-01,.2617622E-01,.2562940E-01,.2505754E-01,.2446120E-01,
     *.2384096E-01,.2319742E-01,.2253122E-01,.2184300E-01,.2113344E-01,
     *.2040323E-01,.1965309E-01,.1888374E-01,.1809594E-01,.1729046E-01,
     *.1646809E-01,.1562962E-01,.1477588E-01,.1390771E-01,.1302595E-01,
     *.1213146E-01,.1122511E-01,.1030780E-01,.9380420E-02,.8443871E-02,
     *.7499073E-02,.6546948E-02,.5588428E-02,.4624450E-02,.3655961E-02,
     *.2683925E-02,.1709393E-02,.7346345E-03,.2087312E-01,.2086402E-01/
      data (wp(i),i= 161, 200)/
     *.2084584E-01,.2081857E-01,.2078223E-01,.2073683E-01,.2068240E-01,
     *.2061896E-01,.2054653E-01,.2046515E-01,.2037486E-01,.2027568E-01,
     *.2016767E-01,.2005088E-01,.1992534E-01,.1979113E-01,.1964829E-01,
     *.1949689E-01,.1933700E-01,.1916867E-01,.1899200E-01,.1880705E-01,
     *.1861391E-01,.1841266E-01,.1820338E-01,.1798617E-01,.1776113E-01,
     *.1752835E-01,.1728792E-01,.1703997E-01,.1678459E-01,.1652190E-01,
     *.1625201E-01,.1597504E-01,.1569110E-01,.1540033E-01,.1510285E-01,
     *.1479879E-01,.1448828E-01,.1417146E-01,.1384846E-01,.1351943E-01/
      data (wp(i),i= 201, 240)/
     *.1318451E-01,.1284384E-01,.1249758E-01,.1214587E-01,.1178887E-01,
     *.1142673E-01,.1105962E-01,.1068768E-01,.1031109E-01,.9930004E-02,
     *.9544593E-02,.9155022E-02,.8761463E-02,.8364086E-02,.7963064E-02,
     *.7558573E-02,.7150788E-02,.6739888E-02,.6326051E-02,.5909457E-02,
     *.5490289E-02,.5068728E-02,.4644959E-02,.4219166E-02,.3791535E-02,
     *.3362252E-02,.2931504E-02,.2499479E-02,.2066366E-02,.1632357E-02,
     *.1197647E-02,.7624721E-03,.3276087E-03,.1566826E-01,.1566442E-01,
     *.1565672E-01,.1564519E-01,.1562981E-01,.1561059E-01,.1558755E-01/
      data (wp(i),i= 241, 280)/
     *.1556067E-01,.1552998E-01,.1549547E-01,.1545716E-01,.1541506E-01,
     *.1536917E-01,.1531950E-01,.1526608E-01,.1520891E-01,.1514800E-01,
     *.1508338E-01,.1501505E-01,.1494303E-01,.1486735E-01,.1478802E-01,
     *.1470505E-01,.1461848E-01,.1452832E-01,.1443459E-01,.1433731E-01,
     *.1423652E-01,.1413223E-01,.1402447E-01,.1391327E-01,.1379866E-01,
     *.1368065E-01,.1355929E-01,.1343460E-01,.1330661E-01,.1317535E-01,
     *.1304086E-01,.1290316E-01,.1276230E-01,.1261831E-01,.1247122E-01,
     *.1232106E-01,.1216788E-01,.1201172E-01,.1185260E-01,.1169058E-01/
      data (wp(i),i= 281, 320)/
     *.1152568E-01,.1135796E-01,.1118744E-01,.1101418E-01,.1083822E-01,
     *.1065959E-01,.1047835E-01,.1029454E-01,.1010820E-01,.9919373E-02,
     *.9728115E-02,.9534468E-02,.9338480E-02,.9140200E-02,.8939676E-02,
     *.8736957E-02,.8532093E-02,.8325134E-02,.8116132E-02,.7905137E-02,
     *.7692201E-02,.7477377E-02,.7260717E-02,.7042274E-02,.6822103E-02,
     *.6600256E-02,.6376790E-02,.6151757E-02,.5925215E-02,.5697218E-02,
     *.5467822E-02,.5237083E-02,.5005059E-02,.4771806E-02,.4537382E-02,
     *.4301844E-02,.4065249E-02,.3827657E-02,.3589125E-02,.3349711E-02/
      data (wp(i),i= 321, 360)/
     *.3109476E-02,.2868477E-02,.2626773E-02,.2384425E-02,.2141492E-02,
     *.1898033E-02,.1654108E-02,.1409777E-02,.1165101E-02,.9201405E-03,
     *.6749606E-03,.4296466E-03,.1845901E-03,.1045439E-01,.1045325E-01,
     *.1045097E-01,.1044754E-01,.1044297E-01,.1043726E-01,.1043041E-01,
     *.1042242E-01,.1041329E-01,.1040302E-01,.1039161E-01,.1037907E-01,
     *.1036539E-01,.1035058E-01,.1033464E-01,.1031758E-01,.1029938E-01,
     *.1028006E-01,.1025961E-01,.1023804E-01,.1021535E-01,.1019155E-01,
     *.1016663E-01,.1014060E-01,.1011347E-01,.1008523E-01,.1005588E-01/
      data (wp(i),i= 361, 400)/
     *.1002544E-01,.9993899E-02,.9961267E-02,.9927547E-02,.9892741E-02,
     *.9856855E-02,.9819891E-02,.9781854E-02,.9742747E-02,.9702576E-02,
     *.9661345E-02,.9619057E-02,.9575718E-02,.9531333E-02,.9485905E-02,
     *.9439441E-02,.9391946E-02,.9343424E-02,.9293880E-02,.9243321E-02,
     *.9191751E-02,.9139177E-02,.9085604E-02,.9031038E-02,.8975485E-02,
     *.8918951E-02,.8861442E-02,.8802965E-02,.8743525E-02,.8683130E-02,
     *.8621786E-02,.8559499E-02,.8496277E-02,.8432127E-02,.8367054E-02,
     *.8301068E-02,.8234174E-02,.8166380E-02,.8097693E-02,.8028121E-02/
      data (wp(i),i= 401, 440)/
     *.7957672E-02,.7886353E-02,.7814173E-02,.7741138E-02,.7667257E-02,
     *.7592538E-02,.7516989E-02,.7440619E-02,.7363435E-02,.7285447E-02,
     *.7206662E-02,.7127090E-02,.7046739E-02,.6965617E-02,.6883734E-02,
     *.6801099E-02,.6717721E-02,.6633608E-02,.6548770E-02,.6463217E-02,
     *.6376957E-02,.6290000E-02,.6202356E-02,.6114033E-02,.6025043E-02,
     *.5935394E-02,.5845096E-02,.5754159E-02,.5662594E-02,.5570409E-02,
     *.5477616E-02,.5384224E-02,.5290244E-02,.5195685E-02,.5100559E-02,
     *.5004875E-02,.4908644E-02,.4811876E-02,.4714583E-02,.4616774E-02/
      data (wp(i),i= 441, 480)/
     *.4518461E-02,.4419654E-02,.4320364E-02,.4220601E-02,.4120378E-02,
     *.4019704E-02,.3918590E-02,.3817049E-02,.3715090E-02,.3612725E-02,
     *.3509965E-02,.3406822E-02,.3303306E-02,.3199429E-02,.3095203E-02,
     *.2990638E-02,.2885746E-02,.2780539E-02,.2675029E-02,.2569225E-02,
     *.2463141E-02,.2356788E-02,.2250177E-02,.2143320E-02,.2036229E-02,
     *.1928915E-02,.1821391E-02,.1713667E-02,.1605756E-02,.1497670E-02,
     *.1389420E-02,.1281018E-02,.1172476E-02,.1063806E-02,.9550200E-03,
     *.8461294E-03,.7371464E-03,.6280830E-03,.5189512E-03,.4097636E-03/
      data (wp(i),i= 481, 520)/
     *.3005340E-03,.1912855E-03,.8217779E-04,.7844110E-02,.7843627E-02,
     *.7842662E-02,.7841214E-02,.7839284E-02,.7836871E-02,.7833976E-02,
     *.7830599E-02,.7826741E-02,.7822400E-02,.7817579E-02,.7812276E-02,
     *.7806493E-02,.7800229E-02,.7793485E-02,.7786262E-02,.7778560E-02,
     *.7770379E-02,.7761720E-02,.7752583E-02,.7742970E-02,.7732880E-02,
     *.7722314E-02,.7711273E-02,.7699757E-02,.7687768E-02,.7675306E-02,
     *.7662371E-02,.7648965E-02,.7635088E-02,.7620742E-02,.7605926E-02,
     *.7590643E-02,.7574892E-02,.7558676E-02,.7541994E-02,.7524848E-02/
      data (wp(i),i= 521, 560)/
     *.7507240E-02,.7489169E-02,.7470638E-02,.7451646E-02,.7432197E-02,
     *.7412290E-02,.7391927E-02,.7371109E-02,.7349838E-02,.7328114E-02,
     *.7305939E-02,.7283315E-02,.7260243E-02,.7236724E-02,.7212760E-02,
     *.7188352E-02,.7163501E-02,.7138210E-02,.7112480E-02,.7086312E-02,
     *.7059708E-02,.7032669E-02,.7005198E-02,.6977296E-02,.6948964E-02,
     *.6920205E-02,.6891020E-02,.6861412E-02,.6831380E-02,.6800929E-02,
     *.6770059E-02,.6738773E-02,.6707072E-02,.6674958E-02,.6642433E-02,
     *.6609500E-02,.6576160E-02,.6542416E-02,.6508269E-02,.6473721E-02/
      data (wp(i),i= 561, 600)/
     *.6438775E-02,.6403433E-02,.6367697E-02,.6331569E-02,.6295052E-02,
     *.6258147E-02,.6220857E-02,.6183184E-02,.6145131E-02,.6106700E-02,
     *.6067893E-02,.6028713E-02,.5989161E-02,.5949241E-02,.5908956E-02,
     *.5868306E-02,.5827296E-02,.5785926E-02,.5744201E-02,.5702123E-02,
     *.5659693E-02,.5616916E-02,.5573792E-02,.5530326E-02,.5486520E-02,
     *.5442376E-02,.5397897E-02,.5353085E-02,.5307945E-02,.5262478E-02,
     *.5216687E-02,.5170575E-02,.5124145E-02,.5077400E-02,.5030342E-02,
     *.4982975E-02,.4935301E-02,.4887323E-02,.4839045E-02,.4790469E-02/
      data (wp(i),i= 601, 640)/
     *.4741598E-02,.4692436E-02,.4642984E-02,.4593247E-02,.4543228E-02,
     *.4492929E-02,.4442353E-02,.4391504E-02,.4340385E-02,.4288999E-02,
     *.4237349E-02,.4185438E-02,.4133270E-02,.4080847E-02,.4028173E-02,
     *.3975251E-02,.3922085E-02,.3868678E-02,.3815032E-02,.3761152E-02,
     *.3707040E-02,.3652700E-02,.3598135E-02,.3543349E-02,.3488345E-02,
     *.3433126E-02,.3377697E-02,.3322059E-02,.3266217E-02,.3210173E-02,
     *.3153933E-02,.3097498E-02,.3040873E-02,.2984060E-02,.2927064E-02,
     *.2869888E-02,.2812535E-02,.2755009E-02,.2697314E-02,.2639453E-02/
      data (wp(i),i= 641, 680)/
     *.2581429E-02,.2523247E-02,.2464909E-02,.2406419E-02,.2347782E-02,
     *.2289000E-02,.2230077E-02,.2171017E-02,.2111823E-02,.2052500E-02,
     *.1993050E-02,.1933477E-02,.1873786E-02,.1813979E-02,.1754061E-02,
     *.1694034E-02,.1633904E-02,.1573673E-02,.1513345E-02,.1452924E-02,
     *.1392413E-02,.1331817E-02,.1271139E-02,.1210383E-02,.1149552E-02,
     *.1088651E-02,.1027682E-02,.9666507E-03,.9055595E-03,.8444126E-03,
     *.7832138E-03,.7219667E-03,.6606753E-03,.5993432E-03,.5379742E-03,
     *.4765722E-03,.4151409E-03,.3536841E-03,.2922057E-03,.2307099E-03/
      data (wp(i),i= 681, 720)/
     *.1692014E-03,.1076904E-03,.4626372E-04,.5231608E-02,.5231465E-02,
     *.5231179E-02,.5230749E-02,.5230177E-02,.5229461E-02,.5228602E-02,
     *.5227600E-02,.5226454E-02,.5225166E-02,.5223735E-02,.5222161E-02,
     *.5220444E-02,.5218584E-02,.5216581E-02,.5214435E-02,.5212147E-02,
     *.5209716E-02,.5207142E-02,.5204426E-02,.5201567E-02,.5198567E-02,
     *.5195423E-02,.5192138E-02,.5188710E-02,.5185141E-02,.5181429E-02,
     *.5177576E-02,.5173581E-02,.5169445E-02,.5165167E-02,.5160747E-02,
     *.5156186E-02,.5151485E-02,.5146642E-02,.5141658E-02,.5136534E-02/
      data (wp(i),i= 721, 760)/
     *.5131269E-02,.5125863E-02,.5120318E-02,.5114632E-02,.5108806E-02,
     *.5102840E-02,.5096735E-02,.5090490E-02,.5084106E-02,.5077583E-02,
     *.5070920E-02,.5064119E-02,.5057180E-02,.5050102E-02,.5042885E-02,
     *.5035531E-02,.5028039E-02,.5020409E-02,.5012642E-02,.5004738E-02,
     *.4996696E-02,.4988518E-02,.4980203E-02,.4971752E-02,.4963165E-02,
     *.4954443E-02,.4945584E-02,.4936590E-02,.4927461E-02,.4918197E-02,
     *.4908799E-02,.4899266E-02,.4889599E-02,.4879799E-02,.4869864E-02,
     *.4859797E-02,.4849596E-02,.4839263E-02,.4828797E-02,.4818199E-02/
      data (wp(i),i= 761, 800)/
     *.4807470E-02,.4796608E-02,.4785616E-02,.4774492E-02,.4763238E-02,
     *.4751853E-02,.4740338E-02,.4728694E-02,.4716920E-02,.4705017E-02,
     *.4692985E-02,.4680825E-02,.4668537E-02,.4656121E-02,.4643577E-02,
     *.4630907E-02,.4618109E-02,.4605185E-02,.4592136E-02,.4578960E-02,
     *.4565659E-02,.4552233E-02,.4538683E-02,.4525008E-02,.4511210E-02,
     *.4497288E-02,.4483243E-02,.4469075E-02,.4454785E-02,.4440373E-02,
     *.4425840E-02,.4411185E-02,.4396410E-02,.4381514E-02,.4366498E-02,
     *.4351363E-02,.4336109E-02,.4320736E-02,.4305245E-02,.4289636E-02/
      data (wp(i),i= 801, 840)/
     *.4273910E-02,.4258066E-02,.4242106E-02,.4226030E-02,.4209839E-02,
     *.4193532E-02,.4177110E-02,.4160574E-02,.4143924E-02,.4127161E-02,
     *.4110284E-02,.4093296E-02,.4076195E-02,.4058982E-02,.4041659E-02,
     *.4024225E-02,.4006681E-02,.3989027E-02,.3971264E-02,.3953392E-02,
     *.3935412E-02,.3917324E-02,.3899129E-02,.3880828E-02,.3862420E-02,
     *.3843906E-02,.3825288E-02,.3806564E-02,.3787737E-02,.3768805E-02,
     *.3749771E-02,.3730634E-02,.3711395E-02,.3692054E-02,.3672612E-02,
     *.3653070E-02,.3633427E-02,.3613685E-02,.3593845E-02,.3573906E-02/
      data (wp(i),i= 841, 880)/
     *.3553869E-02,.3533735E-02,.3513504E-02,.3493177E-02,.3472754E-02,
     *.3452237E-02,.3431624E-02,.3410918E-02,.3390119E-02,.3369227E-02,
     *.3348242E-02,.3327166E-02,.3305999E-02,.3284741E-02,.3263394E-02,
     *.3241957E-02,.3220431E-02,.3198818E-02,.3177116E-02,.3155328E-02,
     *.3133454E-02,.3111493E-02,.3089448E-02,.3067318E-02,.3045104E-02,
     *.3022806E-02,.3000426E-02,.2977964E-02,.2955420E-02,.2932796E-02,
     *.2910091E-02,.2887306E-02,.2864443E-02,.2841501E-02,.2818481E-02,
     *.2795384E-02,.2772211E-02,.2748961E-02,.2725637E-02,.2702238E-02/
      data (wp(i),i= 881, 920)/
     *.2678765E-02,.2655218E-02,.2631599E-02,.2607908E-02,.2584146E-02,
     *.2560312E-02,.2536409E-02,.2512437E-02,.2488395E-02,.2464286E-02,
     *.2440109E-02,.2415865E-02,.2391555E-02,.2367179E-02,.2342739E-02,
     *.2318235E-02,.2293667E-02,.2269037E-02,.2244344E-02,.2219590E-02,
     *.2194775E-02,.2169901E-02,.2144966E-02,.2119973E-02,.2094922E-02,
     *.2069814E-02,.2044649E-02,.2019428E-02,.1994152E-02,.1968821E-02,
     *.1943437E-02,.1917999E-02,.1892508E-02,.1866966E-02,.1841373E-02,
     *.1815729E-02,.1790036E-02,.1764294E-02,.1738503E-02,.1712665E-02/
      data (wp(i),i= 921, 960)/
     *.1686780E-02,.1660848E-02,.1634872E-02,.1608850E-02,.1582785E-02,
     *.1556676E-02,.1530525E-02,.1504331E-02,.1478097E-02,.1451822E-02,
     *.1425507E-02,.1399154E-02,.1372762E-02,.1346332E-02,.1319866E-02,
     *.1293363E-02,.1266825E-02,.1240253E-02,.1213646E-02,.1187006E-02,
     *.1160334E-02,.1133630E-02,.1106895E-02,.1080130E-02,.1053335E-02,
     *.1026511E-02,.9996593E-03,.9727801E-03,.9458743E-03,.9189426E-03,
     *.8919858E-03,.8650045E-03,.8379996E-03,.8109717E-03,.7839217E-03,
     *.7568502E-03,.7297579E-03,.7026457E-03,.6755143E-03,.6483644E-03/
      data (wp(i),i= 961,1000)/
     *.6211967E-03,.5940120E-03,.5668111E-03,.5395947E-03,.5123635E-03,
     *.4851182E-03,.4578597E-03,.4305887E-03,.4033058E-03,.3760120E-03,
     *.3487078E-03,.3213941E-03,.2940716E-03,.2667411E-03,.2394033E-03,
     *.2120589E-03,.1847087E-03,.1573535E-03,.1299941E-03,.1026314E-03,
     *.7526651E-04,.4790311E-04,.2057885E-04,.3924530E-02,.3924469E-02,
     *.3924348E-02,.3924167E-02,.3923925E-02,.3923623E-02,.3923260E-02,
     *.3922837E-02,.3922354E-02,.3921810E-02,.3921206E-02,.3920541E-02,
     *.3919816E-02,.3919030E-02,.3918185E-02,.3917278E-02,.3916312E-02/
      data (wp(i),i=1001,1040)/
     *.3915285E-02,.3914198E-02,.3913051E-02,.3911843E-02,.3910575E-02,
     *.3909247E-02,.3907858E-02,.3906410E-02,.3904901E-02,.3903332E-02,
     *.3901703E-02,.3900014E-02,.3898265E-02,.3896456E-02,.3894587E-02,
     *.3892658E-02,.3890668E-02,.3888619E-02,.3886510E-02,.3884342E-02,
     *.3882113E-02,.3879825E-02,.3877476E-02,.3875068E-02,.3872601E-02,
     *.3870074E-02,.3867487E-02,.3864840E-02,.3862134E-02,.3859369E-02,
     *.3856544E-02,.3853660E-02,.3850716E-02,.3847713E-02,.3844651E-02,
     *.3841530E-02,.3838349E-02,.3835109E-02,.3831811E-02,.3828453E-02/
      data (wp(i),i=1041,1080)/
     *.3825036E-02,.3821561E-02,.3818026E-02,.3814433E-02,.3810781E-02,
     *.3807070E-02,.3803300E-02,.3799473E-02,.3795586E-02,.3791641E-02,
     *.3787638E-02,.3783576E-02,.3779456E-02,.3775278E-02,.3771042E-02,
     *.3766747E-02,.3762395E-02,.3757984E-02,.3753516E-02,.3748990E-02,
     *.3744406E-02,.3739765E-02,.3735066E-02,.3730309E-02,.3725495E-02,
     *.3720624E-02,.3715695E-02,.3710709E-02,.3705666E-02,.3700566E-02,
     *.3695408E-02,.3690194E-02,.3684923E-02,.3679596E-02,.3674211E-02,
     *.3668770E-02,.3663273E-02,.3657719E-02,.3652109E-02,.3646442E-02/
      data (wp(i),i=1081,1120)/
     *.3640720E-02,.3634941E-02,.3629106E-02,.3623216E-02,.3617269E-02,
     *.3611267E-02,.3605209E-02,.3599096E-02,.3592927E-02,.3586703E-02,
     *.3580424E-02,.3574090E-02,.3567700E-02,.3561256E-02,.3554757E-02,
     *.3548203E-02,.3541594E-02,.3534931E-02,.3528213E-02,.3521441E-02,
     *.3514615E-02,.3507734E-02,.3500800E-02,.3493812E-02,.3486770E-02,
     *.3479674E-02,.3472524E-02,.3465321E-02,.3458065E-02,.3450756E-02,
     *.3443393E-02,.3435977E-02,.3428508E-02,.3420987E-02,.3413413E-02,
     *.3405786E-02,.3398107E-02,.3390375E-02,.3382592E-02,.3374756E-02/
      data (wp(i),i=1121,1160)/
     *.3366868E-02,.3358928E-02,.3350937E-02,.3342894E-02,.3334800E-02,
     *.3326654E-02,.3318457E-02,.3310208E-02,.3301909E-02,.3293559E-02,
     *.3285158E-02,.3276707E-02,.3268205E-02,.3259653E-02,.3251051E-02,
     *.3242398E-02,.3233696E-02,.3224944E-02,.3216142E-02,.3207290E-02,
     *.3198390E-02,.3189440E-02,.3180440E-02,.3171392E-02,.3162295E-02,
     *.3153149E-02,.3143955E-02,.3134712E-02,.3125421E-02,.3116082E-02,
     *.3106695E-02,.3097260E-02,.3087778E-02,.3078247E-02,.3068670E-02,
     *.3059045E-02,.3049373E-02,.3039654E-02,.3029888E-02,.3020075E-02/
      data (wp(i),i=1161,1200)/
     *.3010217E-02,.3000311E-02,.2990360E-02,.2980362E-02,.2970318E-02,
     *.2960229E-02,.2950094E-02,.2939914E-02,.2929688E-02,.2919418E-02,
     *.2909102E-02,.2898742E-02,.2888336E-02,.2877887E-02,.2867393E-02,
     *.2856855E-02,.2846273E-02,.2835647E-02,.2824977E-02,.2814264E-02,
     *.2803508E-02,.2792708E-02,.2781865E-02,.2770980E-02,.2760052E-02,
     *.2749081E-02,.2738068E-02,.2727013E-02,.2715915E-02,.2704776E-02,
     *.2693596E-02,.2682374E-02,.2671110E-02,.2659805E-02,.2648460E-02,
     *.2637073E-02,.2625646E-02,.2614179E-02,.2602671E-02,.2591123E-02/
      data (wp(i),i=1201,1240)/
     *.2579536E-02,.2567908E-02,.2556241E-02,.2544535E-02,.2532789E-02,
     *.2521005E-02,.2509181E-02,.2497319E-02,.2485419E-02,.2473480E-02,
     *.2461503E-02,.2449488E-02,.2437436E-02,.2425346E-02,.2413218E-02,
     *.2401054E-02,.2388852E-02,.2376614E-02,.2364339E-02,.2352028E-02,
     *.2339680E-02,.2327296E-02,.2314877E-02,.2302422E-02,.2289931E-02,
     *.2277405E-02,.2264844E-02,.2252249E-02,.2239618E-02,.2226953E-02,
     *.2214254E-02,.2201520E-02,.2188753E-02,.2175952E-02,.2163117E-02,
     *.2150249E-02,.2137349E-02,.2124415E-02,.2111448E-02,.2098449E-02/
      data (wp(i),i=1241,1280)/
     *.2085417E-02,.2072354E-02,.2059258E-02,.2046131E-02,.2032972E-02,
     *.2019782E-02,.2006561E-02,.1993309E-02,.1980026E-02,.1966713E-02,
     *.1953370E-02,.1939996E-02,.1926592E-02,.1913159E-02,.1899697E-02,
     *.1886205E-02,.1872684E-02,.1859134E-02,.1845555E-02,.1831949E-02,
     *.1818314E-02,.1804650E-02,.1790960E-02,.1777241E-02,.1763495E-02,
     *.1749722E-02,.1735922E-02,.1722096E-02,.1708242E-02,.1694363E-02,
     *.1680457E-02,.1666526E-02,.1652569E-02,.1638586E-02,.1624578E-02,
     *.1610545E-02,.1596488E-02,.1582405E-02,.1568299E-02,.1554168E-02/
      data (wp(i),i=1281,1320)/
     *.1540013E-02,.1525835E-02,.1511633E-02,.1497407E-02,.1483159E-02,
     *.1468888E-02,.1454594E-02,.1440278E-02,.1425940E-02,.1411579E-02,
     *.1397197E-02,.1382794E-02,.1368369E-02,.1353923E-02,.1339456E-02,
     *.1324969E-02,.1310461E-02,.1295933E-02,.1281385E-02,.1266817E-02,
     *.1252230E-02,.1237623E-02,.1222998E-02,.1208353E-02,.1193690E-02,
     *.1179009E-02,.1164309E-02,.1149592E-02,.1134857E-02,.1120104E-02,
     *.1105334E-02,.1090547E-02,.1075743E-02,.1060923E-02,.1046086E-02,
     *.1031234E-02,.1016365E-02,.1001481E-02,.9865808E-03,.9716658E-03/
      data (wp(i),i=1321,1360)/
     *.9567359E-03,.9417913E-03,.9268321E-03,.9118587E-03,.8968712E-03,
     *.8818700E-03,.8668551E-03,.8518269E-03,.8367855E-03,.8217313E-03,
     *.8066644E-03,.7915851E-03,.7764936E-03,.7613902E-03,.7462750E-03,
     *.7311483E-03,.7160104E-03,.7008614E-03,.6857017E-03,.6705313E-03,
     *.6553507E-03,.6401600E-03,.6249593E-03,.6097491E-03,.5945295E-03,
     *.5793007E-03,.5640630E-03,.5488166E-03,.5335618E-03,.5182987E-03,
     *.5030277E-03,.4877489E-03,.4724626E-03,.4571690E-03,.4418684E-03,
     *.4265610E-03,.4112470E-03,.3959267E-03,.3806003E-03,.3652680E-03/
      data (wp(i),i=1361,1400)/
     *.3499301E-03,.3345867E-03,.3192383E-03,.3038849E-03,.2885269E-03,
     *.2731644E-03,.2577977E-03,.2424270E-03,.2270526E-03,.2116747E-03,
     *.1962935E-03,.1809093E-03,.1655224E-03,.1501328E-03,.1347410E-03,
     *.1193471E-03,.1039514E-03,.8855408E-04,.7315545E-04,.5775582E-04,
     *.4235569E-04,.2695689E-04,.1158044E-04,.3140018E-02,.3139987E-02,
     *.3139926E-02,.3139833E-02,.3139709E-02,.3139554E-02,.3139368E-02,
     *.3139152E-02,.3138904E-02,.3138625E-02,.3138316E-02,.3137975E-02,
     *.3137604E-02,.3137201E-02,.3136768E-02,.3136304E-02,.3135809E-02/
      data (wp(i),i=1401,1440)/
     *.3135283E-02,.3134726E-02,.3134138E-02,.3133519E-02,.3132869E-02,
     *.3132189E-02,.3131477E-02,.3130735E-02,.3129962E-02,.3129158E-02,
     *.3128323E-02,.3127457E-02,.3126560E-02,.3125633E-02,.3124675E-02,
     *.3123686E-02,.3122666E-02,.3121615E-02,.3120534E-02,.3119422E-02,
     *.3118279E-02,.3117105E-02,.3115901E-02,.3114666E-02,.3113400E-02,
     *.3112103E-02,.3110776E-02,.3109418E-02,.3108029E-02,.3106610E-02,
     *.3105160E-02,.3103680E-02,.3102169E-02,.3100627E-02,.3099055E-02,
     *.3097452E-02,.3095819E-02,.3094155E-02,.3092461E-02,.3090736E-02/
      data (wp(i),i=1441,1480)/
     *.3088981E-02,.3087195E-02,.3085379E-02,.3083532E-02,.3081655E-02,
     *.3079748E-02,.3077810E-02,.3075842E-02,.3073843E-02,.3071815E-02,
     *.3069756E-02,.3067666E-02,.3065547E-02,.3063397E-02,.3061217E-02,
     *.3059007E-02,.3056766E-02,.3054496E-02,.3052195E-02,.3049865E-02,
     *.3047504E-02,.3045113E-02,.3042692E-02,.3040242E-02,.3037761E-02,
     *.3035250E-02,.3032709E-02,.3030139E-02,.3027538E-02,.3024908E-02,
     *.3022248E-02,.3019558E-02,.3016838E-02,.3014089E-02,.3011310E-02,
     *.3008501E-02,.3005662E-02,.3002794E-02,.2999896E-02,.2996969E-02/
      data (wp(i),i=1481,1520)/
     *.2994012E-02,.2991026E-02,.2988010E-02,.2984965E-02,.2981890E-02,
     *.2978786E-02,.2975652E-02,.2972489E-02,.2969297E-02,.2966075E-02,
     *.2962825E-02,.2959545E-02,.2956236E-02,.2952897E-02,.2949530E-02,
     *.2946134E-02,.2942708E-02,.2939254E-02,.2935770E-02,.2932258E-02,
     *.2928716E-02,.2925146E-02,.2921547E-02,.2917919E-02,.2914262E-02,
     *.2910577E-02,.2906863E-02,.2903120E-02,.2899349E-02,.2895549E-02,
     *.2891720E-02,.2887863E-02,.2883978E-02,.2880064E-02,.2876122E-02,
     *.2872151E-02,.2868152E-02,.2864125E-02,.2860069E-02,.2855985E-02/
      data (wp(i),i=1521,1560)/
     *.2851873E-02,.2847734E-02,.2843565E-02,.2839369E-02,.2835145E-02,
     *.2830893E-02,.2826613E-02,.2822305E-02,.2817970E-02,.2813606E-02,
     *.2809215E-02,.2804796E-02,.2800350E-02,.2795875E-02,.2791374E-02,
     *.2786844E-02,.2782288E-02,.2777704E-02,.2773092E-02,.2768453E-02,
     *.2763787E-02,.2759094E-02,.2754373E-02,.2749625E-02,.2744850E-02,
     *.2740048E-02,.2735219E-02,.2730363E-02,.2725480E-02,.2720571E-02,
     *.2715634E-02,.2710671E-02,.2705681E-02,.2700664E-02,.2695621E-02,
     *.2690551E-02,.2685454E-02,.2680331E-02,.2675182E-02,.2670006E-02/
      data (wp(i),i=1561,1600)/
     *.2664804E-02,.2659576E-02,.2654321E-02,.2649040E-02,.2643733E-02,
     *.2638400E-02,.2633041E-02,.2627657E-02,.2622246E-02,.2616809E-02,
     *.2611347E-02,.2605858E-02,.2600344E-02,.2594805E-02,.2589240E-02,
     *.2583649E-02,.2578033E-02,.2572391E-02,.2566724E-02,.2561032E-02,
     *.2555315E-02,.2549572E-02,.2543804E-02,.2538011E-02,.2532193E-02,
     *.2526350E-02,.2520483E-02,.2514590E-02,.2508672E-02,.2502730E-02,
     *.2496763E-02,.2490772E-02,.2484756E-02,.2478715E-02,.2472650E-02,
     *.2466561E-02,.2460447E-02,.2454309E-02,.2448147E-02,.2441961E-02/
      data (wp(i),i=1601,1640)/
     *.2435751E-02,.2429516E-02,.2423258E-02,.2416976E-02,.2410670E-02,
     *.2404340E-02,.2397986E-02,.2391609E-02,.2385209E-02,.2378784E-02,
     *.2372337E-02,.2365866E-02,.2359371E-02,.2352853E-02,.2346312E-02,
     *.2339748E-02,.2333161E-02,.2326551E-02,.2319918E-02,.2313262E-02,
     *.2306584E-02,.2299882E-02,.2293158E-02,.2286411E-02,.2279642E-02,
     *.2272850E-02,.2266036E-02,.2259200E-02,.2252341E-02,.2245460E-02,
     *.2238557E-02,.2231631E-02,.2224684E-02,.2217715E-02,.2210724E-02,
     *.2203711E-02,.2196677E-02,.2189620E-02,.2182543E-02,.2175443E-02/
      data (wp(i),i=1641,1680)/
     *.2168323E-02,.2161180E-02,.2154017E-02,.2146832E-02,.2139626E-02,
     *.2132400E-02,.2125152E-02,.2117883E-02,.2110593E-02,.2103282E-02,
     *.2095951E-02,.2088599E-02,.2081226E-02,.2073833E-02,.2066420E-02,
     *.2058986E-02,.2051531E-02,.2044057E-02,.2036562E-02,.2029047E-02,
     *.2021513E-02,.2013958E-02,.2006384E-02,.1998789E-02,.1991175E-02,
     *.1983542E-02,.1975888E-02,.1968216E-02,.1960524E-02,.1952812E-02,
     *.1945082E-02,.1937332E-02,.1929563E-02,.1921775E-02,.1913968E-02,
     *.1906142E-02,.1898298E-02,.1890434E-02,.1882552E-02,.1874652E-02/
      data (wp(i),i=1681,1720)/
     *.1866733E-02,.1858795E-02,.1850840E-02,.1842866E-02,.1834874E-02,
     *.1826863E-02,.1818835E-02,.1810789E-02,.1802725E-02,.1794643E-02,
     *.1786544E-02,.1778427E-02,.1770292E-02,.1762140E-02,.1753970E-02,
     *.1745784E-02,.1737580E-02,.1729359E-02,.1721120E-02,.1712865E-02,
     *.1704593E-02,.1696304E-02,.1687999E-02,.1679677E-02,.1671338E-02,
     *.1662983E-02,.1654611E-02,.1646223E-02,.1637819E-02,.1629399E-02,
     *.1620962E-02,.1612510E-02,.1604042E-02,.1595557E-02,.1587058E-02,
     *.1578542E-02,.1570011E-02,.1561465E-02,.1552903E-02,.1544325E-02/
      data (wp(i),i=1721,1760)/
     *.1535733E-02,.1527125E-02,.1518503E-02,.1509865E-02,.1501213E-02,
     *.1492545E-02,.1483863E-02,.1475167E-02,.1466456E-02,.1457730E-02,
     *.1448990E-02,.1440236E-02,.1431467E-02,.1422684E-02,.1413888E-02,
     *.1405077E-02,.1396253E-02,.1387414E-02,.1378563E-02,.1369697E-02,
     *.1360818E-02,.1351926E-02,.1343020E-02,.1334101E-02,.1325169E-02,
     *.1316224E-02,.1307265E-02,.1298294E-02,.1289310E-02,.1280314E-02,
     *.1271305E-02,.1262283E-02,.1253249E-02,.1244202E-02,.1235143E-02,
     *.1226072E-02,.1216989E-02,.1207894E-02,.1198787E-02,.1189668E-02/
      data (wp(i),i=1761,1800)/
     *.1180538E-02,.1171396E-02,.1162242E-02,.1153077E-02,.1143900E-02,
     *.1134712E-02,.1125513E-02,.1116303E-02,.1107082E-02,.1097850E-02,
     *.1088607E-02,.1079354E-02,.1070089E-02,.1060815E-02,.1051529E-02,
     *.1042234E-02,.1032928E-02,.1023612E-02,.1014286E-02,.1004950E-02,
     *.9956034E-03,.9862475E-03,.9768819E-03,.9675067E-03,.9581219E-03,
     *.9487276E-03,.9393240E-03,.9299112E-03,.9204892E-03,.9110581E-03,
     *.9016180E-03,.8921690E-03,.8827112E-03,.8732448E-03,.8637697E-03,
     *.8542861E-03,.8447941E-03,.8352937E-03,.8257851E-03,.8162684E-03/
      data (wp(i),i=1801,1840)/
     *.8067436E-03,.7972109E-03,.7876703E-03,.7781220E-03,.7685659E-03,
     *.7590023E-03,.7494312E-03,.7398528E-03,.7302670E-03,.7206740E-03,
     *.7110739E-03,.7014668E-03,.6918528E-03,.6822320E-03,.6726045E-03,
     *.6629703E-03,.6533295E-03,.6436824E-03,.6340289E-03,.6243691E-03,
     *.6147032E-03,.6050312E-03,.5953533E-03,.5856694E-03,.5759799E-03,
     *.5662846E-03,.5565837E-03,.5468774E-03,.5371657E-03,.5274486E-03,
     *.5177264E-03,.5079991E-03,.4982667E-03,.4885295E-03,.4787874E-03,
     *.4690406E-03,.4592892E-03,.4495332E-03,.4397729E-03,.4300082E-03/
      data (wp(i),i=1841,1880)/
     *.4202392E-03,.4104661E-03,.4006890E-03,.3909079E-03,.3811229E-03,
     *.3713342E-03,.3615418E-03,.3517459E-03,.3419465E-03,.3321437E-03,
     *.3223377E-03,.3125285E-03,.3027162E-03,.2929009E-03,.2830827E-03,
     *.2732617E-03,.2634380E-03,.2536118E-03,.2437830E-03,.2339519E-03,
     *.2241184E-03,.2142827E-03,.2044449E-03,.1946051E-03,.1847634E-03,
     *.1749198E-03,.1650745E-03,.1552276E-03,.1453792E-03,.1355293E-03,
     *.1256781E-03,.1158257E-03,.1059721E-03,.9611747E-04,.8626190E-04,
     *.7640548E-04,.6654832E-04,.5669051E-04,.4683217E-04,.3697344E-04/
      data (wp(i),i=1881,1883)/
     *.2711461E-04,.1725677E-04,.7413338E-05/

      CsTH = cos(TwoTH0 * 0.5/RAD)
      if (abs(CsTH) .lt. 1.0e-15) CsTH = 1.0e-15
      TTH = sin(TwoTH0 * 0.5/RAD)/CsTH
      CsTwoTH = cos(TwoTH0/RAD)
      SnTwoTH = sin(TwoTH0/RAD)
      ApB = S_L + D_L
      AmB = S_L - D_L
      ApB2 = ApB**2
      if (((S_L .ne. 0.0) .or. (D_L .ne. 0.0)) .and. Use_Asym) then
         tmp = sqrt(1.0 + AmB**2)*CsTwoTH
         if (abs(tmp) .gt. 1.0) then
            Einfl = acos(CsTwoTH)*RAD
         else
            Einfl = acos(tmp)*RAD
         endif
         tmp2 = 1.0 + ApB2
         tmp = sqrt(tmp2 ) * CsTwoTH

c --- If S_L or D_L are zero, set Einfl = 2theta 
    
         if ((S_L .eq. 0.0) .or. (D_L .eq. 0.0)) Einfl = TwoTH0
         if (abs(tmp) .le. 1.0) then
            Emin = acos(tmp) * RAD
            tmp1 = tmp2 * (1.0 - tmp2 * CsTwoTH**2)
         else
            tmp1 = 0.0
            if (tmp .gt. 0.0) then
               Emin = 0.0
            else
               Emin = 180.0
            endif
         endif
         if ((tmp1 .gt. 0.0) .and. (abs(tmp) .le. 1.0)) then
            dEmindA = -ApB * CsTwoTH/sqrt(tmp1)
         else
            dEmindA = 0.0
         endif
         ArrayNum = 1
         K = 400.0 * (TwoTH0 - Emin) ! Calculate number of terms needed 
         do while ((ArrayNum .lt. 14) .and. (K .gt. NTERMS(ArrayNum)))
            ArrayNum = ArrayNum + 1
         enddo
         NGT = nterms(ArrayNum) ! Save number of terms 
         ngt2 = ngt / 2
         
c --- Clear terms needed for summations 

         sumWG = 0.0
         sumWRG = 0.0
         sumWdGdA = 0.0
         sumWRdGdA = 0.0
         sumWdGdB = 0.0
         sumWRdGdB = 0.0
         sumWGdRd2t = 0.0
         sumWGdRdG = 0.0
         sumWGdRdE = 0.0
         sumWGdRdA = 0.0
         sumWGdRdB = 0.0
         
c --- Compute the convolution integral 

         it = fstterm(arraynum)-ngt2
         do K = ngt2 , NGT 
            delta = Emin + (TwoTH0 - Emin) * xp(K + it)
            dDeltadA = (1.0 - xp(k+it) ) * dEmindA
            sinDELTA = sin(Delta/RAD)
            cosDELTA = cos(Delta/RAD)
            if (abs(cosDELTA) .lt. 1.0e-15) cosDELTA = 1.0e-15
            RcosDELTA = 1.0 / cosDELTA
            tanDELTA = tan(Delta/RAD)
            tmp = cosDELTA**2 - CsTwoTH**2
            if (tmp .gt. 0.0) then
               tmp1 = sqrt(tmp)
               F = abs(CsTwoTH) / tmp1
               dFdA = cosDELTA * CsTwoTH * sinDELTA * dDELTAdA 
     1              / (tmp1 * tmp1 * tmp1)
            else
               F = 0.0
               dFdA = 0.0
            endif

c --- calculate G(Delta,2theta) , FCJ eq. 7a and 7b 

            if ( abs(Delta - Emin) .gt. abs(Einfl - Emin)) then
               if (S_L .gt. D_L) then

c --- N.B. this is the only place where d()/dA <> d()/dB

                  G = 2.0 * D_L * F * RcosDELTA
                  dGdA = 2.0 * D_L * RcosDELTA * (dFdA + 
     1                 F*tanDELTA*dDELTAdA)
                  dGdB = dGdA + 2.0 * F * RcosDELTA
               else
                  G = 2.0 * S_L * F * RcosDELTA
                  dGdB = 2.0 * S_L * RcosDELTA
     1                 *(dFdA + F * tanDELTA * dDELTAdA)
                  dGdA = dGdB + 2.0 * F * RcosDELTA
               endif
            else
               G = (-1.0 + ApB * F) * RcosDELTA
               dGdA = RcosDELTA * (F - tanDELTA * dDELTAdA + ApB * dFdA
     1              + ApB * F * tanDELTA * dDELTAdA)
               dGdB = dGdA
            endif
            tmp = PsVoigt2(TwoTh-DELTA+TwoTH0,TwoTH0,eta,Gamma,dPRdT
     1           ,dPRdG,dPRdE)
            sumWG = sumWG + wp(k+it) * G
            sumWRG = sumWRG + wp(k+it) * G * tmp
            sumWdGdA = sumWdGdA + wp(k+it) * dGdA
            sumWdGdB = sumWdGdB + wp(k+it) * dGdB
            sumWRdGdA = sumWRdGdA + wp(k+it) * dGdA * tmp
            sumWRdGdB = sumWRdGdB + wp(k+it) * dGdB * tmp 
            sumWGdRd2t = sumWGdRd2t + wp(k+it) * G * dPRdT
            sumWGdRdG = sumWGdRdG + wp(k+it) * G * dPRdG
            sumWGdRdE = sumWGdRdE + wp(k+it) * G * dPRdE
            sumWGdRdA = sumWGdRdA + wp(k+it) * G * dPRdT * 
     1           dDELTAdA * RAD
         enddo
         if (sumWG .eq. 0.0) sumWG = 1.0
         Profval = sumWRG / sumWG
         dPRdT = sumWGdRd2t/ sumWG
         dPRdG = sumWGdRdG / sumWG
         dPRdE = sumWGdRdE / sumWG
         dPRdS = (sumWRdGdA + sumWGdRdA/RAD) / sumWG - sumWRG * 
     1        sumWdGdA/RAD/sumWG**2
         dPRdD = (sumWRdGdB + sumWGdRdA/RAD) / sumWG - sumWRG * 
     1        sumWdGdB/RAD/sumWG**2

      else                      ! here for no asymmetry }

         tmp = PsVoigt2(TwoTH,TwoTH0,eta,Gamma,dPRdT,dPRdG,dPRdE)
         Profval = tmp
         dPRdS = 0.0
         dPRdD = 0.0

      endif

      return
      end

c******************************************************************************
c***             Pseudo Voigt function and its derivatives                  ***
c******************************************************************************

      real function PsVoigt2(TwoTH , TwoTH0 , Eta , Gamma,
     1     dPRdT , dPRdG , dPRdE ) 

c --- Returns value of Pseudo Voigt
c     Eta is the mixing coefficient between Gaussian and Lorentzian
c     Gamma is the FWHM
c     TwoTH is point at which to evaluate the profile
c     TwoTH0 is two theta value for peak
c     dPRdT is derivative of profile wrt TwoTH0
c     dPRdG is derivative of profile wrt Gamma
c     dPRdE is derivative of profile wrt Eta

      implicit none
      external Gauss, Lorentz
      real TwoTH, TwoTH0, Gamma, Eta, dPRdT, dPRdG, dPRdE
      real Gauss, Lorentz, G, L, x, dGdx, dLdx, dPRdx, temp

      x = (TwoTH - TwoTH0) / Gamma
      G = Gauss(x,dGdx)
      L = Lorentz(x,dLdx)
      temp = Eta * L + (1.-Eta) * G
      dPRdx = Eta * dLdx + (1.-Eta) * dGdx
      dPRdE = (L - G) / Gamma
      dPRdT  = -dPRdx / Gamma**2
      dPRdG  = -1. * (temp + x * dPRdx) / Gamma**2
      PsVoigt2 = temp / Gamma

      return
      end


C******************************************************************************
C***       Bragg-R-Wert (Nuklearer Intensitaets-R-Wert) berechnen           ***
C******************************************************************************

      subroutine rbragg(kcyc,idges,nphase,LPHASE,IRS,IDAT,riobs,rint,ic)

      character form1*20
      integer nphase(IDAT), ic(LPHASE,IDAT)
      real riobs(IRS,IDAT), rint(IRS,IDAT)
      real sumdd(10), sumdo(10), rnuc(10)

      nph = 1
      do 10 id=1,idges
         nph = max(nph,nphase(id))
 10   continue
     
      do 20 ip=1,nph
         sumdd(ip) = 0.
         sumdo(ip) = 0.
 20   continue

      write(*,'(//2x,a)') 'Derived BRAGG R-Factors:'
      write(kcyc,'(/2x,a)') 'Derived BRAGG R-Factors:'
      form1 = ' '
      form1 = '(/2x,a,  (a,i3))'
      write(form1(8:9),'(i2)') nph
      write(*,form1) 'Data set',('     phase',ip,ip=1,nph)
      write(kcyc,form1) 'Data set',('     phase',ip,ip=1,nph)

c --- Bragg-R-Werte der einzelnen Datensaetze und Phasen:

      do 150 id=1,idges
         iend = 0
         do 120 ip=1,nphase(id)
            ibeg = iend + 1
            iend = iend + ic(ip,id)
            sumdiff = 0
            sumiobs = 0
	    do 100 i=ibeg,iend
               if(riobs(i,id).ge.0.) then
                 sumiobs = sumiobs + riobs(i,id)
                 sumdiff = sumdiff + abs(riobs(i,id) - rint(i,id)) 
               end if
 100        continue
	    sumdd(ip) = sumdd(ip) + sumdiff
	    sumdo(ip) = sumdo(ip) + sumiobs
            if(sumiobs.ne.0.) then
               rnuc(ip) = 100. * sumdiff / sumiobs
            else
               rnuc(ip) = -1.
            end if
 120     continue
         form1 = ' '
         form1 = '(i7,3x,  f13.2)'
         write(form1(8:9),'(i2)') nphase(id)
         write(*,form1) id,(rnuc(ip),ip=1,nphase(id))
         write(kcyc,form1) id,(rnuc(ip),ip=1,nphase(id))
 150  continue

c ---Gesamt-Bragg-R-Werte bei mehreren Datensaetzen:

      if(idges.gt.1) then
         do 300 ip=1,nph
            if(sumdo(ip).ne.0.) then
               rnuc(ip) = 100. * sumdd(ip) / sumdo(ip)
            else
               rnuc(ip) = -1.
            end if
 300     continue
         form1 = ' '
         form1 = '(3x,a,2x,  f13.2)'
         write(form1(10:11),'(i2)') nph
         write(*,form1) 'total',(rnuc(ip),ip=1,nph)
         write(kcyc,form1) 'total',(rnuc(ip),ip=1,nph)
      end if
      write(*,*)
      write(kcyc,*)

      return
      end


C******************************************************************************
C***            Berechnen der Intensitaeten und Ableitungen                 ***
C******************************************************************************

      subroutine refcalc(maxs,dersto,deriv,rlamda,
     *                   uu,vv,ww,q,a,apar,lpar,aglb,lglb,aint,
     *                   lint,iref,theta,hwb,rint,
     *                   nhkl,IRS,MSZ)

      parameter(PIQ = 9.869604401, PI = 3.14159265359)

      integer iref(7,IRS), lglb(10), lpar(23), lint(IRS)
      real rlamda(2), a(6), q(3), al(3,3), h(3), deriv(MSZ) 
      real theta(IRS), hwb(IRS), rint(IRS)
      real dersto(MSZ,IRS), aglb(10), apar(23), aint(IRS)


C --- reziproke Gitterkonstanten :

      al(1,1) = a(1) * a(1)
      al(2,2) = a(2) * a(2)
      al(3,3) = a(3) * a(3)
      al(1,2) = a(1) * a(2) * a(6)
      al(1,3) = a(1) * a(3) * a(5)
      al(2,3) = a(2) * a(3) * a(4)
      al(2,1) = al(1,2)
      al(3,1) = al(1,3)
      al(3,2) = al(2,3)


      do 200 i=1,nhkl

c --- Reziproker Gittervektor h und 
c     dessen Betragsquadrat hh berechnen:

         call hvec(iref(1,i),iref(2,i),iref(3,i),iref(4,i),q,h)

         hh = 0.
         do 22 l=1,3
            do 22 j=1,3
               hh = hh + h(l) * h(j) * al(l,j)
22       continue


c --- Reflexlage, Lorentz-Polarisations-Faktor, Halbwertsbreite,
c     tan(Theta) und tan(2-Theta) berechnen:

         sinth       = 0.5 * rlamda(iref(5,i)) * sqrt(hh)
         sinlam      = hh/4.
         if(sinlam.eq.0.) sinlam = 0.0001
         if(abs(sinth).gt.1.) sinth = 0.
         th          = asin(sinth)
         tanth       = sinth/sqrt(1.-sinth*sinth)
         if(tanth.eq.0.) tanth = 0.0001
         theta(i)    = th
         hwb2        = uu*tanth*tanth + vv*tanth + ww
         if(hwb2.ge.0.) then
           hwb(i)    = sqrt(hwb2)
         else
           write(*,'(/2x,a,f9.4,a)') 
     *     'Error: fwhm < 0 at 2theta = ', 360./PI * th, ' degrees'
           stop
         end if 

c --- Ableitungen von y_calc(i), die nicht von x(i) abhaengen:

         do 110 m=1,maxs
            deriv(m) = 0.
110      continue  

c --- Parameter der Profilfunktion, Asymmetrieparameter:

         k = lpar(1)
         if(k.ne.0) deriv(k) = deriv(k) + apar(1) * rint(i)
         k = lpar(2)
         if(k.ne.0) deriv(k) = deriv(k) + apar(2) * rint(i)
         k = lpar(3)
         if(k.ne.0) deriv(k) = deriv(k) + apar(3) * rint(i)

c --- Halbwertsbreite-Parameter U, V, W:

         rb = 0.5/hwb(i) * rint(i)

         k = lpar(8)
         if(k.ne.0) deriv(k) = deriv(k) + apar(8) * rb * tanth**2 
         k = lpar(9)
         if(k.ne.0) deriv(k) = deriv(k) + apar(9) * rb * tanth 
         k = lpar(10)
         if(k.ne.0) deriv(k) = deriv(k) + apar(10) * rb 

c --- Wellenvektor:

         rqa = 90./PI * tanth/sinlam * rint(i)

         k = lpar(11)
         if(k.ne.0) deriv(k) = deriv(k) + apar(11) * rqa * iref(4,i) *
     *        (h(1) * al(1,1) + h(2) * al(1,2) + h(3) * al(1,3))
         k = lpar(12)
         if(k.ne.0) deriv(k) = deriv(k) + apar(12) * rqa * iref(4,i) *
     *        (h(2) * al(2,2) + h(1) * al(1,2) + h(3) * al(2,3))
         k = lpar(13)
         if(k.ne.0) deriv(k) = deriv(k) + apar(13) * rqa * iref(4,i) *
     *        (h(3) * al(3,3) + h(1) * al(1,3) + h(2) * al(2,3))  

c --- reziproke Gitterkonstanten:

         k = lpar(14)
         if(k.ne.0) deriv(k) = deriv(k) + apar(14) * rqa *
     *        (h(1)**2 * a(1) + h(1) * h(2) * a(2) * a(6) +
     *         h(1) * h(3) * a(3) * a(5))
         k = lpar(15)  
         if(k.ne.0) deriv(k) = deriv(k) + apar(15) * rqa *
     *        (h(2)**2 * a(2) + h(2) * h(1) * a(1) * a(6) +
     *        h(2) * h(3) * a(3) * a(4))
         k = lpar(16)      
         if(k.ne.0) deriv(k) = deriv(k) + apar(16) * rqa * 
     *        (h(3)**2 * a(3) + h(3) * h(1) * a(1) * a(5) +
     *        h(3) * h(2) * a(2) * a(4))
         k = lpar(17)
         if(k.ne.0) deriv(k) = deriv(k) + 
     *        apar(17) * rqa * h(2) * h(3) * a(2) * a(3)
         k = lpar(18)
         if(k.ne.0) deriv(k) = deriv(k) + 
     *        apar(18) * rqa * h(1) * h(3) * a(1) * a(3)
         k = lpar(19)
         if(k.ne.0) deriv(k) = deriv(k) + 
     *        apar(19) * rqa * h(1) * h(2) * a(1) * a(2)

c --- Asymmetriekorrektur-Parameter:

         k = lpar(20)
         if(k.ne.0) deriv(k) = deriv(k) + apar(20) * rint(i)
         k = lpar(21)
         if(k.ne.0) deriv(k) = deriv(k) + apar(21) * rint(i)
         k = lpar(22)
         if(k.ne.0) deriv(k) = deriv(k) + apar(22) * rint(i)
         k = lpar(23)
         if(k.ne.0) deriv(k) = deriv(k) + apar(23) * rint(i)

c --- Nullpunkt t0 und peak-shift parameter t1 und t2:

         k = lglb(1)         
         if(k.ne.0) deriv(k) = deriv(k) + aglb(1) * rint(i)
         k = lglb(2)         
         if(k.ne.0) deriv(k) = deriv(k) + aglb(2) * rint(i)
         k = lglb(3)         
         if(k.ne.0) deriv(k) = deriv(k) + aglb(3) * rint(i)

c --- Wellenlaenge:

         k = lglb(10) 
         if(k.ne.0) deriv(k) = deriv(k) +  
     *              aglb(10) * rint(i) * tanth/rlamda(1) * 360./PI

c --- Intensitaeten:

         k = lint(i)
         if(k.ne.0) deriv(k) = deriv(k) + aint(i)

c --- Ableitungen fuer jeden Reflex abspeichern:

         do 180 m=1,maxs
            dersto(m,i) = deriv(m)
180      continue


200   continue


      return
      end


c******************************************************************************
c***                   Neues Inputfile schreiben                            ***
c******************************************************************************
      subroutine rewneu(ke,ka,savefile,inpfile,jobtyp,idges,thmin,
     *                  thmax,step,rlamda,ratio,imo,nphase,ibac,
     *                  atext,ntyp,natom,rmult,pref,gitter,glb,par,
     *                  atom,LPHASE,NATS,IDAT)

      parameter(maxline=132, PI = 3.14159265359)
      real gitter(6,LPHASE,IDAT), glb(10,IDAT), par(23,LPHASE,IDAT),
     *     atom(14,NATS), pref(3,LPHASE,IDAT),
     *     thmin(IDAT), thmax(IDAT), step(IDAT), rlamda(2,IDAT),
     *     ratio(2,IDAT), rmult(NATS)
      integer nphase(IDAT), natom(LPHASE), ibac(IDAT), imo(LPHASE)
      character line*132, atext(NATS)*4, ntyp(NATS,IDAT)*4,
     *          inpfile*80, savefile*80


      write(*,'(/2x,a)') 'Creating a new INPUT file.'

c --- Datei oeffnen:
   
      open(ke,file=savefile,status='old',err=1000)
      open(ka,file=inpfile,status='unknown',err=1100)

      do 500 id=1,idges      

c --- Untergrund-Parameter:

         if(ibac(id).eq.0) then
           iwrt = 0
10         read(ke,'(a)') line
           if(line(1:3).eq.'BAC') then
             write(ka,'(a,6(f10.2,1x))') 
     *            'BAC> ',(glb(k,id),k=4,9)
             iwrt = 1
           else
             call writline(ka,line,maxline)
           end if
           if(iwrt.eq.0) goto 10
         end if
11       ibs = ibstes(line)
         if(ibs.eq.1) then
           line = ' '
           read(ke,'(a)') line
           goto 11
         end if

c --- Wellenlaengen:

         iwrt = 0
12       read(ke,'(a)') line
         if(line(1:3).eq.'WVL') then
           write(ka,'(a,2(f10.6,1x),f9.5)')
     *     'WVL> ',rlamda(1,id),rlamda(2,id),ratio(2,id) 
           iwrt = 1
         else
           call writline(ka,line,maxline)
         end if
         if(iwrt.eq.0) goto 12
13       ibs = ibstes(line)
         if(ibs.eq.1) then
           line = ' '
           read(ke,'(a)') line
           goto 13
         end if


c --- 2-Theta-Bereich, Schrittweite,
c     Nullpunkt, Peak-Shift-Parameter t1 und t2:

         iwrt = 0
15       read(ke,'(a)') line
         if(line(1:3).eq.'THE') then
           write(ka,'(a,4(1x,f9.5),1x,2(1x,g11.4))')
     *     'THE> ',thmin(id),thmax(id),step(id),(glb(i,id),i=1,3) 
           iwrt = 1
         else
           call writline(ka,line,maxline)
         end if
         if(iwrt.eq.0) goto 15
16       ibs = ibstes(line)
         if(ibs.eq.1) then
           line = ' '
           read(ke,'(a)') line
           goto 16
         end if

         iof = 0 
         do 300 ip=1,nphase(id)

c --- Gitterkonstanten:

            iwrt = 0
20          read(ke,'(a)') line
            if(line(1:3).eq.'MET') then
              write(ka,'(a,6(1x,f9.5))') 'MET> ',(gitter(k,ip,id),k=1,6)
              iwrt = 1
            else
              call writline(ka,line,maxline)
            end if
            if(iwrt.eq.0) goto 20
21          ibs = ibstes(line)
            if(ibs.eq.1) then
              line = ' '
              read(ke,'(a)') line
              goto 21
            end if

c --- Wellenvektor:

            if(imo(ip).eq.1) then
              iwrt = 0
30            read(ke,'(a)') line
              if(line(1:3).eq.'WAV') then
                write(ka,'(a,3(1x,f9.5))') 
     *               'WAV> ',(par(k,ip,id),k=11,13)
                iwrt = 1
              else
                call writline(ka,line,maxline)
              end if
              if(iwrt.eq.0) goto 30
31            ibs = ibstes(line)
              if(ibs.eq.1) then
                line = ' '
                read(ke,'(a)') line
                goto 31
              end if
            end if

c --- Scale factor, overall temperature factor, halfwidth parameters:

            iwrt = 0
40          read(ke,'(a)') line
            if(line(1:3).eq.'PRO') then
              write(ka,'(a,1x,g10.4,4(1x,f9.5))') 
     *        'PRO> ',par(4,ip,id),par(5,ip,id),(par(k,ip,id),k=8,10)
              iwrt = 1
            else
             call writline(ka,line,maxline)
            end if
            if(iwrt.eq.0) goto 40 
41          ibs = ibstes(line)
            if(ibs.eq.1) then
              line = ' '
              read(ke,'(a)') line
              goto 41
            end if

c --- Parameters of the peak shape function:
 
            iwrt = 0
50          read(ke,'(a)') line
            if(line(1:3).eq.'ETA') then
              write(ka,'(a,1x,f9.5,2(1x,g11.4))') 
     *             'ETA> ',(par(k,ip,id),k=1,3)
              iwrt = 1
            else
              call writline(ka,line,maxline)
            end if
            if(iwrt.eq.0) goto 50   
51          ibs = ibstes(line)
            if(ibs.eq.1) then
              line = ' '
              read(ke,'(a)') line
              goto 51
            end if

c --- Asymmetry correction parameters:

            iwrt = 0
60          read(ke,'(a)') line
            if(line(1:3).eq.'ASY') then
              write(ka,'(a,4(1x,f9.5))') 
     *             'ASY> ',(par(k,ip,id),k=20,23)
              iwrt = 1
            else
              call writline(ka,line,maxline)
            end if
            if(iwrt.eq.0) goto 60   
61          ibs = ibstes(line)
            if(ibs.eq.1) then
              line = ' '
              read(ke,'(a)') line
              goto 61
            end if

c --- Preferred orientation parameter and vector:

            if(jobtyp.eq.1) then
              iwrt = 0
70            read(ke,'(a)') line
              if(line(1:3).eq.'PRE') then
                write(ka,'(a,2(1x,f9.5),3(1x,f8.3))')
     *          'PRE> ',par(6,ip,id),par(7,ip,id),(pref(k,ip,id),k=1,3)
                iwrt = 1
              else
                call writline(ka,line,maxline)
              end if
              if(iwrt.eq.0) goto 70
71            ibs = ibstes(line)
              if(ibs.eq.1) then
                line = ' '
                read(ke,'(a)') line
                goto 71
              end if
            end if

c --- Atomparameter:

            if(jobtyp.eq.1.and.id.eq.1) then
              n = 0
              if(ip.gt.1) iof = iof + natom(ip-1)
150           read(ke,'(a)') line
              if(line(1:3).eq.'ATO') then
                n = n + 1
                write(ka,'(a,a4,1x,a4,1x,i7,5(1x,f8.5),a)') 
     *          'ATO> ',atext(iof+n),ntyp(iof+n,id),
     *          nint(rmult(iof+n)),(atom(k,iof+n),k=1,5),' \\'
                if(imo(ip).eq.2) then
                  write(ka,'(4x,6(1x,f8.5),a,/4x,3(1x,f8.5))') 
     *            (atom(k,iof+n),k=6,11),' \\',(atom(k,iof+n),k=12,14)
                else
                  write(ka,'(4x,6(1x,f8.5))') (atom(k,iof+n),k=6,11)
                end if
              else
                call writline(ka,line,maxline)
              end if
151           ibs = ibstes(line)
              if(ibs.eq.1) then
                line = ' '
                read(ke,'(a)') line
                goto 151
              end if
              if(n.lt.natom(ip)) goto 150
            end if

300      continue
500   continue


900   read(ke,'(a)',end=999) line
      call writline(ka,line,maxline)
      goto 900

999   close(ke)
      close(ka)
      return

1000  stop ' Cannot open INPUT file'
1100  stop ' Cannot open OUPUT file'

      end


c******************************************************************************
c***                Zeile nach Backslash durchsuchen                        ***
c******************************************************************************

      integer function ibstes(line)

      character line*132

      ibstes = 0
      do 20 i=1,132
         if(line(i:i).eq.'\\') then
           if(i.eq.1) then
             ibstes = 1
             return
           else
             if(line(i-1:i-1).eq.' ') then
               ibstes = 1
               return
             end if 
           end if
         end if
20    continue

      end
C******************************************************************************
C***           Berechnet dir Reziproken Gitterkonstanten                    ***
C******************************************************************************

      SUBROUTINE REZI(CELLD,CELLR)

      parameter (PI = 3.14159265359)
      REAL CELLD(6),CELLR(13)

C     CELLD(1) = A          CELLR(1)  = A*
C     CELLD(2) = B          CELLR(2)  = B*
C     CELLD(3) = C          CELLR(3)  = C*
C     CELLD(4) = Alpha      CELLR(4)  = cos(Alpha*)
C     CELLD(5) = Beta       CELLR(5)  = cos(Beta*)
C     CELLD(6) = Gamma      CELLR(6)  = cos(Gamma*)
C                           CELLR(7)  = (A*)**2
C                           CELLR(8)  = (B*)**2.
C                           CELLR(9)  = (C*)**2.
C                           CELLR(10) = 2.*(A*)*(B*)*COS(Gamma*)
C                           CELLR(11) = 2.*(A*)*(C*)*COS(Beta*)
C                           CELLR(12) = 2.*(B*)*(C*)*COS(Alpha*)
C                           CELLR(13) = 1./Volumen


C --- Achsen:

      A = CELLD(1)
      B = CELLD(2)
      C = CELLD(3)

C --- Cosinuse und Sinuse der Winkel:

      COSAA = COS(CELLD(4)*PI/180.)
      COSBB = COS(CELLD(5)*PI/180.)
      COSGG = COS(CELLD(6)*PI/180.)

      SINAA = sqrt(1. - cosaa * cosaa)
      SINBB = sqrt(1. - cosbb * cosbb)
      SINGG = sqrt(1. - cosgg * cosgg)

C --- Zellvolumen:

      VOL = A*B*C*SQRT(1. + 2.*COSAA*COSBB*COSGG -
     1                 COSAA*COSAA - COSBB*COSBB - COSGG*COSGG)

C --- reziproke Gitterkonstanten und reziprokes Volumen:

      AREZ = B*C*SINAA/VOL
      BREZ = C*A*SINBB/VOL
      CREZ = A*B*SINGG/VOL

      CELLR(1) = AREZ
      CELLR(2) = BREZ
      CELLR(3) = CREZ

      ALREZ = (COSBB*COSGG-COSAA)/(SINBB*SINGG)
      BEREZ = (COSGG*COSAA-COSBB)/(SINGG*SINAA)
      GAREZ = (COSAA*COSBB-COSGG)/(SINAA*SINBB)

      CELLR(4) = ALREZ
      CELLR(5) = BEREZ
      CELLR(6) = GAREZ

      ASTERN = AREZ*AREZ
      BSTERN = BREZ*BREZ
      CSTERN = CREZ*CREZ

      CELLR(7) = ASTERN
      CELLR(8) = BSTERN
      CELLR(9) = CSTERN

      ALSTE = 2.*BREZ*CREZ*ALREZ
      BESTE = 2.*CREZ*AREZ*BEREZ
      GASTE = 2.*AREZ*BREZ*GAREZ

      CELLR(10) = ALSTE
      CELLR(11) = BESTE
      CELLR(12) = GASTE

      CELLR(13) = 1./VOL


      RETURN
      END


c******************************************************************************
c***                     Berechnung der R-Werte                             ***
c******************************************************************************

      subroutine rwert(ibac,obs,calc,gew,bac,mit,izahl,IDZ,sbcal,
     *                 sbobs,sbdif,swobs,chisq,rp,rwp,num)

      implicit none
      integer IDZ
      integer j, ibac, izahl, num, mit(IDZ)
      real obs(IDZ), calc(IDZ), gew(IDZ), bac(IDZ)
      real sbcal, sbobs, sbdif, swobs, chisq, rp, rwp


      num = 0
      sbcal = 0.0
      sbobs = 0.0
      sbdif = 0.0
      swobs = 0.0
      chisq = 0.0

      if(ibac.eq.0) then
        do 10 j=1,izahl
           if(mit(j).ge.0) then
	     num = num + 1
             sbcal = sbcal + abs(calc(j))
	     sbobs = sbobs + abs(obs(j))
	     sbdif = sbdif + abs(obs(j)-calc(j))
	     swobs = swobs + 1./gew(j) * obs(j)**2.
	     chisq = chisq + 1./gew(j) * (obs(j)-calc(j))**2.
           end if
10      continue
      else
        do 20 j=1,izahl
	   if(mit(j).eq.1) then
	     num = num + 1
             sbcal = sbcal + abs(calc(j)-bac(j))
             sbobs = sbobs + abs(obs(j)-bac(j))
             sbdif = sbdif + abs(obs(j)-calc(j))
             swobs = swobs + 1./gew(j) * (obs(j)-bac(j))**2.
             chisq = chisq + 1./gew(j) * (obs(j)-calc(j))**2.
           end if
20      continue
      end if
	
      rp  = 100. * sbdif/sbobs 
      rwp = 100. * sqrt(chisq/swobs)

      return
      end


c******************************************************************************
c***                       hkl-Liste ausgeben                               ***
c******************************************************************************

      subroutine satwrite(ka,cellr,q,iref,theta,nhkl,IRS)

      parameter (PI = 3.14159265359)
      external hhcalc
      integer iref(7,IRS)
      real theta(IRS), cellr(13), h(3), q(3)

      write(ka,'(2x,2a)') ' No. Co.     h   k   l   n',
     *'     H       K       L    Mult   2-theta      d'
      do 300 i=1,nhkl
         thobs = 360./PI * theta(i)
         call hvec(iref(1,i),iref(2,i),iref(3,i),iref(4,i),q,h)
         dwert = 0.
         hh = hhcalc(h,cellr)
         if(hh.gt.0.) dwert = 1./sqrt(hh)
         write(ka,'(2x,2i4,2x,4i4,3(f8.3),i6,2x,f8.3,f10.5)')
     *         i,iref(5,i),iref(1,i),iref(2,i),iref(3,i),iref(4,i),
     *         (h(j),j=1,3),iref(6,i),thobs,dwert
300   continue

    
      return
      end





c******************************************************************************
c***                    Altes Inputfile sichern                             ***
c******************************************************************************

      subroutine savinp(ke,ka,inpfile,savefile)

      parameter(maxline=132)
      character line*132, inpfile*80, savefile*80, chr*1


c --- Name der Sicherungskopie bestimmen:

      iend = 81
      do 50 i=0,79
         chr = inpfile(80-i:80-i)
         if(chr.ne.' '.and.iend.eq.81) iend = i
         if(chr.eq.'.') then
           k = 80-i-1               
           goto 60         
         end if
50    continue
      k = iend
                             
60    savefile(1:k) = inpfile(1:k)
      savefile(k+1:k+4) = '.bak'


c --- Sichern:

      open(ke,file=inpfile,status='old',err=1000)
      open(ka,file=savefile,status='unknown',err=1100)

100   read(ke,'(a)',err=1200,end=500) line
      call writline(ka,line,maxline)
      goto 100

500   close(ke)
      close(ka)
      return


c --- Fehlermeldungen:

1000  stop ' Cannot open input file'
1100  stop ' Cannot open save file'
1200  stop ' ERROR: Error reading input file'

      end


c******************************************************************************
c***         Streukoeffizienten, anomale Streufaktoren df',df''             ***
c***            extern eingegebene Streudaten zuordnen                      ***
c******************************************************************************

      subroutine scatf(rlam,istreu,ixn,nscatm,nphase,ntyp,natom,imo,
     *                 ptr,nam,ac,dfp,dfpp,NATS,LPHASE)

      parameter(EPSY = 0.00001)
      integer ptr(NATS), natom(LPHASE), imo(LPHASE)
      real xryz(5), ac(9,NATS), dfp(NATS), dfpp(NATS)
      character*4 tabnc(128), tbxc(256), ntyp(NATS), nam(NATS), 
     *            tbmc(128), cneu

      common /tabchr/ tabnc, tbxc, tbmc
      common /tables/ tbx(256,10), tbd(128,10), tabn(128), tbm(128,8)

c --- Roentgen-Wellenlaengen (Cr, Fe, Cu, Mo, Ag):

      data xryz(1), xryz(2), xryz(3), xryz(4), xryz(5)
     *    /2.28962, 1.93597, 1.54051, 0.70926, 0.556363/


c --- Wellenlaenge der Roentgenstrahlung 
c     (ixray: 1=Cr, 2=Fe, 3=Cu, 4=Mo, 5=Ag):

      if(ixn.eq.1) then
        do 6 j=1,5
           if(rlam+0.1.gt.xryz(j)) goto 7
 6      continue
 7      ixray = j
      end if

c --- Wurden bei der externen Eingabe der Roentgen-Streudaten nur
c     df' und df'' eingegeben, muessen die Streukoeffizienten noch
c     bestimmt werden:

      if(ixn.eq.1) then
        do 16 i=1,nscatm
           nl = 0
           do 13 l=1,9
              if(abs(ac(l,i)).lt.EPSY) nl = nl + 1
 13        continue
           if(nl.eq.9) then
             do 15 j=1,213
	        if(nam(i).eq.tbxc(j)) then
                  do 14 l=1,9
                     ac(l,i) = tbx(j,l)
 14               continue
                  goto 16 
                end if
 15           continue
              write(*,'(/2x,a,a)') 
     *             'Scattering coefficients not found for ',nam(i)
	      stop ' Scattering data missing'
           end if
 16     continue
      end if


c --- Schleife ueber die Phasen:

      imoges = 0
      do 17 ip=1,nphase
         if(imo(ip).eq.2) imoges = 2
 17   continue

      iof = 0
      ns = nscatm      
      do 300 ip=1,nphase

         if(ip.gt.1) iof = iof + natom(ip-1)

c === Roentgen-Beugung:

         if(ixn.eq.1) then

           do 50 k=1,natom(ip)

c --- Kontrolle, ob Streukoeffizienten von aussen eingegeben wurden,
c     falls Eintrag vorhanden, wird 50-er Schleife erhoeht:

              do 20 j=1,ns
	         if(ntyp(iof+k).eq.nam(j)) then
                   ptr(iof+k) = j
                   goto 50
	         end if
20            continue

c --- Wenn Streukoeffizienten nicht von Aussen eingegeben wurden,
c     dann wird intern nachgeprueft. Falls Eintrag vorhanden, wird 
c     50-er Schleife erhoeht:

	      do 40 j=1,213
	         if(ntyp(iof+k).eq.tbxc(j)) then
		   ns = ns + 1
		   ptr(iof+k) = ns
		   do 30 l=1,9
		      ac(l,ns) = tbx(j,l)
30                 continue
		   nam(ns)  = tbxc(j)
		   itbxptr  = tbx(j,10) + 0.5
		   dfp(ns)  = tbd(itbxptr,ixray)
		   dfpp(ns) = tbd(itbxptr,ixray+5)
		   goto 50 
	         end if
40            continue

c --- Falls kein Eintrag gefunden, Fehlermeldung ausgeben und
c     Programm abbrechen:

              write(*,'(/2x,a,a)') 
     *             'Scattering coefficients not found for ',ntyp(iof+k)
	      stop ' Scattering data missing'

50         continue

c === Neutronen-Beugung:

         else

	   do 150 k=1,natom(ip)

c --- Kontrolle, ob Streulaengen von Aussen eingegeben wurden,
c     falls Eintrag vorhanden, wird 150-er Schleife erhoeht:

              do 90 j=1,ns
	         if(ntyp(iof+k).eq.nam(j)) then
		   ptr(iof+k) = j
		   goto 150
	         end if
90            continue

c --- Wenn Streulaengen nicht von Aussen eingegeben wurden, dann
c     wird intern nachgeprueft. Falls Eintrag vorhanden, wird 
c     150-er Schleife erhoeht:

              cneu = ' '
              do 100 l=1,4
                 if(ntyp(iof+k)(l:l).eq.'+'.or.
     *              ntyp(iof+k)(l:l).eq.'-') goto 105
                 cneu(l:l) = ntyp(iof+k)(l:l)
 100          continue

 105          do 110 j=1,91
	         if(cneu.eq.tabnc(j)) then
                    ns = ns + 1
                    ptr(iof+k) = ns
                    dfp(ns) = tabn(j)
                    nam(ns) = ntyp(iof+k)
                    goto 115
	         end if
 110          continue

c --- Falls kein Eintrag gefunden, Fehlermeldung ausgeben und
c     Programm abbrechen:

              write(*,'(/2x,a,a)')
     *             'Scattering length not found for ',ntyp(iof+k)
	      stop ' Scattering data missing'

c --- Koeffizienten zur Berechnung der magnetischen Formfaktoren:

115           if(imoges.eq.2) then
                do 130 i=1,95
                   if(ntyp(iof+k).eq.tbmc(i)) then
                     do 120 l=1,8
                        ac(l,ns) = tbm(i,l)
120                  continue
                     goto 150
                   end if
130             continue
              end if

150        continue

         end if

300   continue


c --- Streukoeffizienten/Streulaengen ausgeben:

      if(istreu.eq.1) then
        if(ixn.eq.1) then
          write(*,'(/2x,a)') 'X-ray scattering coefficients:'
          do 410 j=1,ns
             write(*,'(/2x,a4,2(a,f7.3))') 
     *       nam(j),':   df'' = ',dfp(j),',   df'''' = ',dfpp(j) 
             write(*,'(10x,3(a,f10.6),a)')
     *       'a1 = ',ac(1,j),',  b1 = ',ac(2,j),',  a2 = ',ac(3,j),','
             write(*,'(10x,3(a,f10.6),a)')
     *       'b2 = ',ac(4,j),',  a3 = ',ac(5,j),',  b3 = ',ac(6,j),','
             write(*,'(10x,3(a,f10.6))')
     *       'a4 = ',ac(7,j),',  b4 = ',ac(8,j),',  c  = ',ac(9,j) 
 410      continue
        else
          write(*,'(/2x,a/)') 'Scattering lengths for neutrons:'
          do 420 j=1,ns
             write(*,'(2x,a4,a,f10.6)') nam(j),':  ',dfp(j)
             if(imoges.eq.2) then
               write(*,'(10x,3(a,f10.6),a)')
     *         'A = ',ac(1,j),',  a = ',ac(2,j),',  B = ',ac(3,j),','
               write(*,'(10x,3(a,f10.6),a)')
     *         'b = ',ac(4,j),',  C = ',ac(5,j),',  c = ',ac(6,j),','
               write(*,'(10x,2(a,f10.6))')
     *         'D = ',ac(7,j),',  e = ',ac(8,j)
             end if
 420      continue
        end if
      end if

      return
      end


c******************************************************************************
c***                   SHELX-Eingabefile erzeugen                           ***
c******************************************************************************

      subroutine shelxout(ka,outfile,titel,ixn,temp,rlam,par,apar,lpar,
     *                    ibeta,intaus,icent,ism,tran,nsym,atext,ntyp,
     *                    ptr,rmult,atom,ac,dfp,dfpp,natom,NATS,dp,MSZ)

      parameter(PI2 = 9.8696044)
      character text*70, cs*60, titel*70, tranc*5
      character outfile*80, atext(NATS)*4, ntyp(NATS)*4, intaus*1
      integer ism(3,3,48), ptr(NATS), lpar(23)
      real rlam, par(23), std(23), tran(3,48), atom(14,NATS)
      real rmult(NATS), celld(6), delta(6), apar(23), u(6)
      real ac(9,NATS), dfp(NATS), dfpp(NATS), dp(MSZ)

c --- Open file:

      open(ka,file=outfile,status='unknown')

c --- Title:

      write(ka,'(a,1x,a70)') 'TITL',titel

c --- Wavelength and unit-cell dimensions:

      do 20 j=1,23
   	 km = lpar(j)
	 IF(km.ne.0) then
           std(j) = dp(km) * abs(apar(j))
         else
	   std(j) = 0.
	 end if
20    continue
      call direct(par(14),std(14),celld,delta,vol,dvol) 
      write(ka,'(a,1x,7f10.5)') 'CELL',rlam,(celld(i),i=1,6)

c --- Number of formula units per cell and 
c     estimated standard deviations in the unit-cell dimensions:

      nz = 1
      write(ka,'(a,1x,i3,2x,6f10.5)') 'ZERR',nz,(delta(i),i=1,6)

c --- Lattice type:

      if(intaus.eq.'P') then
        latt = 1
      else if(intaus.eq.'I') then
        latt = 2
      else if(intaus.eq.'R') then
        latt = 3
      else if(intaus.eq.'F') then
        latt = 4
      else if(intaus.eq.'A') then
        latt = 5
      else if(intaus.eq.'B') then
        latt = 6
      else if(intaus.eq.'C') then
        latt = 7
      end if
      if(icent.eq.1) latt = -latt
      write(ka,'(a,1x,i2)') 'LATT',latt

c --- Symmetry operations:

      do 50 n=1,nsym
         if(ism(1,1,n).eq.1.and.ism(2,2,n).eq.1.and.ism(3,3,n).eq.1.and.
     *      ism(1,2,n).eq.0.and.ism(1,3,n).eq.0.and.ism(2,3,n).eq.0.and.
     *      ism(2,1,n).eq.0.and.ism(3,1,n).eq.0.and.ism(3,2,n).eq.0) 
     *     goto 50
         cs = ' '
         m = 0
         do 45 j=1,3
            js = 0
            do 44 k=1,3
               if(ism(j,k,n).ne.0) then
                  js = js + 1
                  if(ism(j,k,n).eq.-1) then
                    m = m + 1
                    cs(m:m) = '-'
                  else 
                    if(js.gt.1) then
                      m = m + 1 
                      cs(m:m) = '+'
                    end if
                  end if
                  m = m + 1
                  if(k.eq.1) then
                    cs(m:m) = 'x'
                  else if(k.eq.2) then
                    cs(m:m) = 'y'
                  else if(k.eq.3) then
                    cs(m:m) = 'z'
                  end if 
               end if
 44         continue
            if(abs(tran(j,n)).gt.0.01) then
              m = m + 1
              if(tran(j,n).gt.0) then
                cs(m:m) = '+'
              else
                cs(m:m) = '-'
              end if
              call tranchar(tran(j,n),tranc,len)
              cs(m+1:m+len) = tranc(1:len)
              m = m + len
            end if
            if(j.lt.3) then
               m = m + 1
               cs(m:m) = ','
            end if
 45      continue
         write(ka,'(a,3(1x,a))') 'SYMM', cs
 50   continue

c --- Scattering coefficients, scattering length, ... :

      nscat = 0
      do 60 k=1,natom
         nscat = max0(nscat,ptr(k))
 60   continue

      if(ixn.eq.1) then
        text = ' '
        do 62 ns=1,nscat
           do 61 k=1,natom
              if(ns.eq.ptr(k)) then
                m = (k-1)*5 + 1
                text(m:m+3) = ntyp(k)
                if(ns.lt.nscat) text(m+4:m+4) = ' '
                goto 62
              end if
 61        continue
 62     continue
        write(ka,'(a,1x,a)') 'SFAC',text
      else
        do 65 ns=1,nscat
           do 64 k=1,natom
              if(ns.eq.ptr(k)) then
                write(ka,'(a,1x,a,a,f8.4,a)') 'SFAC',ntyp(k),
     *          ' 0. 0. 0. 0. 0. 0. 0. 0. ',10.*dfp(ns),
     *          ' 0. 0. 0. 1. 1.'
                goto 65
              end if
 64        continue
 65     continue
      end if

c --- Number of atoms of each type in the unit-cell, in SFAC order:

      text = ' '
      do 75 ns=1,nscat
         nu = 0
         do 74 k=1,natom
            if(ptr(k).eq.ns) then
              nu = nu + nint(rmult(k)*nsym)
            end if
 74      continue
         m = (ns-1)*4 + 1
         write(text(m:m+3),'(i4)') nu
 75   continue
      write(ka,'(a,1x,a)') 'UNIT',text

c --- Number of least-squares cycles:

      write(ka,'(/a)') 'L.S.  1'

c --- Crystallographic Information File for Acta Cryst.:
 
      write(ka,'(a)') 'ACTA'

c --- Temperature:

      write(ka,'(a,1x,f8.4)') 'TEMP',temp

c --- Atoms of the asymmetric unit:

      do 90 k=1,natom
         ianiso = 0
         do 80 j=6,11
            if(abs(atom(j,k)).gt.0.00001) ianiso = ianiso + 1
 80      continue
         if(ianiso.eq.0) then
            write(ka,'(a4,i4,5f9.5)') atext(k),ptr(k),
     *      (atom(j,k),j=1,3), rmult(k) * atom(5,k), atom(4,k)/8./PI2 
         else
            if(ibeta.eq.0) then
               u(1) = atom(6,k)/2./PI2/par(14)
               u(2) = atom(7,k)/2./PI2/par(15)
               u(3) = atom(8,k)/2./PI2/par(16)
               u(4) = atom(11,k)/2./PI2/par(15)/par(16)
               u(5) = atom(10,k)/2./PI2/par(14)/par(16)
               u(6) = atom(9,k)/2./PI2/par(14)/par(15)
            else if(ibeta.eq.1) then
               u(1) = atom(6,k)/8./PI2
               u(2) = atom(7,k)/8./PI2
               u(3) = atom(8,k)/8./PI2
               u(4) = atom(11,k)/8./PI2
               u(5) = atom(10,k)/8./PI2
               u(6) = atom(9,k)/8./PI2
            else if(ibeta.eq.2) then
               u(1) = atom(6,k)
               u(2) = atom(7,k)
               u(3) = atom(8,k)
               u(4) = atom(11,k)
               u(5) = atom(10,k)
               u(6) = atom(9,k)
            end if
            write(ka,'(a4,i4,5f9.5)') atext(k),ptr(k),
     *      (atom(j,k),j=1,3), rmult(k) * atom(5,k), (u(j),j=1,6) 
         end if
 90   continue

      write(ka,'(a)') 'HKLF 4'
      write(ka,'(a)') 'END'


      close(ka)

      return
      end


c******************************************************************************
c***           Translationsanteil des Symmetrieelementes                    ***
c******************************************************************************

      subroutine tranchar(tran,tranc,len)

      implicit none

      integer i, len
      character tranc*5
      real tran, tmp, epsylon
      data epsylon /0.00001/

      tmp = abs(tran)

      if(abs(tran).lt.epsylon) then
         tranc = '     '
      else if(abs(tmp-0.083333333).lt.epsylon) then
         tranc = '1/12 '
      else if(abs(tmp-0.125000000).lt.epsylon) then
         tranc = '1/8  '
      else if(abs(tmp-0.166666667).lt.epsylon) then
         tranc = '1/6  '
      else if(abs(tmp-0.250000000).lt.epsylon) then
         tranc = '1/4  '
      else if(abs(tmp-0.333333333).lt.epsylon) then
         tranc = '1/3  '
      else if(abs(tmp-0.375000000).lt.epsylon) then
         tranc = '3/8  '
      else if(abs(tmp-0.416666667).lt.epsylon) then
         tranc = '5/12 '
      else if(abs(tmp-0.500000000).lt.epsylon) then
         tranc = '1/2  '
      else if(abs(tmp-0.583333333).lt.epsylon) then
         tranc = '7/12 '
      else if(abs(tmp-0.625000000).lt.epsylon) then
         tranc = '5/8  '
      else if(abs(tmp-0.666666667).lt.epsylon) then
         tranc = '2/3  '
      else if(abs(tmp-0.750000000).lt.epsylon) then
         tranc = '3/4  '
      else if(abs(tmp-0.833333333).lt.epsylon) then
         tranc = '5/6  '
      else if(abs(tmp-0.875000000).lt.epsylon) then
         tranc = '7/8  '
      else if(abs(tmp-0.916666667).lt.epsylon) then
         tranc = '11/12'
      end if
      
      do 100 i=1,5
         if(tranc(i:i).eq.' ') then
            len = i - 1
            goto 120
         end if
 100  continue
      len = 5

 120  return
      end
c******************************************************************************
c***    Geeichte Gitterkonstanten von Silizium bei der Temperatur temp      ***
c******************************************************************************

      real function silat(temp)

c --- J. Ihringer, H. Ritter, Version vom 09.12.1996

      external yskal
      parameter(neich = 19)
      real t_eich(neich), a_eich(neich)

      data    c1,     c2,    c3,     c4,       a0,      t0 
     *    / 3.725, 5.88E-3, 124., 5.548E-4, 0.5430741, 273.2 /

c --- Eichtabelle fuer Silizium fuer 10 K < T < 190 K:

      data t_eich / 10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,
     *              90., 100., 110., 120., 130., 140., 150., 160.,
     *             170., 180., 190. /
      data a_eich / 0.5429833, 0.5429833, 0.5429832, 0.5429826,
     *              0.5429813, 0.5429794, 0.5429771, 0.5429745,
     *              0.5429720, 0.5429699, 0.5429684, 0.5429675,
     *              0.5429678, 0.5429689, 0.5429711, 0.5429743,
     *              0.5429785, 0.542983805, 0.542990051 /


c --- Silizium Temperaturgang nach der Eichtabelle fuer 0 < T < 180 K
c     nach K. G. Lyon, G. L. Salinger, C. A. Swenson and G. K. White,
c      J. Appl. Phys. 43(3), 865 (1977):

      if(temp.ge.0..or.temp.lt.180.) then

        if(temp.LE.t_eich(1)) then
          a = yskal(a_eich(1),a_eich(2),t_eich(1),t_eich(2),temp)
        else
          if(temp.ge.t_eich(neich)) then
             a = yskal(a_eich(neich-1),a_eich(neich),
     *                 t_eich(neich-1),t_eich(neich),temp)
          else
            do 30 i=1,neich-1
               if(temp.gt.t_eich(i).and.temp.le.t_eich(i+1)) then
                 a = yskal(a_eich(i),a_eich(i+1),
     *                     t_eich(i),t_eich(i+1),temp)
               end if
 30         continue
          end if
        end if

c --- Silizium-Temperaturgang fuer 180 K < T < 1500 K nach
c     Y. Okada and Y. Tokumaro, J. Appl. Phys. 56(2), 314 (1984) 

      else if(temp.ge.180..and.temp.le.1500.) then

        a = c1*(temp-t0+exp(c2*c3)*(exp(-c2*temp)-exp(-c2*t0))/c2)
        a = a + c4/2.*(temp**2.-t0**2.)
        a = (a*1E-6 + 1.)*a0

      else

        write(*,'(/2x,a)') 'Temperature out of range'
        a = 0.54309

      end if

      silat = 10. * a

      return
      end


c******************************************************************************
c***                       Function yskal                                   ***
c******************************************************************************

      real function yskal(y1,y2,x1,x2,x)

c --- J. Ihringer, Version vom 22.08.1982

c --- yskal plaziert einen zu x gehoerenden wert yskal auf eine
c     Gerade, die durch zwei Punkte (x1,y1) und (x2,y2) definiert ist
c     oder yskal skaliert x auf die y-Skala.

c     y1,y2: Minimum/Maximum der y-Skala
c     x1,x2: Minimum/Maximum der x-Skala

      yskal = (x-x1)*(y2-y1)/(x2-x1) + y1

      return
      end
c******************************************************************************
c***          Intensitaeten und Strukturfaktoren berechnen                  ***
c******************************************************************************

      subroutine strucalc(ixn,rlamda,ratio,pref,ipref,celld,
     *                    ac,dfp,dfpp,ptr,
     *                    imo,ibeta,icent,ism,tran,nprim,nsym,
     *                    aglb,lglb,par,apar,lpar,
     *                    atom,aato,lato,rmult,natom,NATS,
     *                    ampl,aampl,lampl,pa,apos,tp,npos,nbas,IBV,
     *                    iref,theta,hwb,rlopo,ffreal,ffimag,
     *                    rint,nuq,nhkl,NORD,IRS,
     *                    dersto,deriv,maxs,MSZ,
     *                    sumxr,sumxi,sumdr,sumdi,sumda,sumdb)

      implicit none
      integer IRS, NATS, NORD, MSZ, IBV
      integer ixn, ipref, imo, ibeta, natom, nhkl, maxs
      integer ptr(NATS), ism(3,3,48), nprim(48), iref(7,IRS)
      integer lglb(10), lpar(23), lato(14,NATS), lampl(2,IBV,NATS)
      integer nuq(2*NORD+1,IRS), icent, nsym, nbas(NATS), npos(NATS)
      real rlamda(2), ratio(2), pref(3), celld(6), tran(3,48)
      real pa(6,48,IBV,NATS), tp(3,48,NATS), apos(3,48,NATS)
      real dersto(MSZ,IRS), deriv(MSZ)
      real par(23), atom(14,NATS), ampl(2,IBV,NATS)
      real aglb(10), apar(23), aato(14,NATS), aampl(2,IBV,NATS)
      real ac(9,NATS), dfp(NATS), dfpp(NATS), rmult(NATS)
      real theta(IRS), hwb(IRS), rlopo(IRS), ffreal(IRS),
     *     ffimag(IRS), rint(IRS)
      real sumdr(2,IBV,NATS), sumdi(2,IBV,NATS), sumxr(11,NATS), 
     *     sumxi(11,NATS), sumda(3,14,NATS), sumdb(3,14,NATS)

      if(imo.eq.1) then
         call modstruc(maxs,dersto,ibeta,ixn,natom,atom,aato,
     *                 lato,ampl,aampl,lampl,rlamda,ratio,
     *                 pref,ipref,par,apar,lpar,aglb,lglb,
     *                 ac,dfp,dfpp,ptr,rmult,pa,apos,tp,npos,nbas,
     *                 iref,theta,hwb,rlopo,ffreal,ffimag,
     *                 rint,nuq,nhkl,sumxr,sumxi,sumdr,
     *                 sumdi,deriv,NATS,IRS,MSZ,NORD,IBV)
      else if(imo.eq.2) then
         call magstruc(ibeta,rmult,atom,aato,lato,natom,NATS,
     *                 nprim,ism,tran,nsym,
     *                 iref,theta,hwb,rlopo,rint,ffreal,ffimag,nhkl,IRS,
     *                 aglb,lglb,par,apar,lpar,
     *                 rlamda,ratio,celld,pref,ipref,ac,dfp,ptr,
     *                 sumda,sumdb,dersto,deriv,maxs,MSZ)
      else
         call nucstruc(maxs,dersto,ibeta,ixn,natom,atom,aato,
     *                 lato,rlamda,ratio,pref,ipref,par,apar,lpar,
     *                 aglb,lglb,ac,dfp,dfpp,ptr,rmult,icent,ism,
     *                 tran,nsym,iref,theta,hwb,rlopo,ffreal,ffimag,
     *                 rint,nhkl,sumxr,sumxi,deriv,NATS,IRS,MSZ)
      end if
      
      return
      end




c******************************************************************************
c***              Strukturfaktoren ausgeben                                 ***
c******************************************************************************

c Anmerkung: Bei magnetischen Strukturen ist im Feld "ffreal" |P|^2 und
c            im Feld "ffimag" |F|^2 = |P|^2 - |h*P|^2 abgespeichert.

      subroutine struout(ka,imo,q,iref,theta,rlopo,f000,ffreal,
     *                   ffimag,rint,scale,nhkl,IRS)

      parameter (PI = 3.14159265359)
      complex f000
      integer iref(7,IRS)
      real theta(IRS), rlopo(IRS), ffreal(IRS), ffimag(IRS), rint(IRS)
      real hm(3), q(3)

      fre = real(f000)
      fim = aimag(f000)
      ff  = fre**2 + fim**2
      
      if(imo.eq.1) then

        write(ka,'(2x,3a)')
     *  ' Nr. Co.     h   k   l   n    Mult',
     *  '        2-Theta       Lopo             A              B',
     *  '             F*F       Intensity'
        write(ka,'(2x,2i4,2x,4i4,i8,3x,2f12.4,4f15.3)')
     *       0, 1, 0, 0, 0, 0, 1, 0., 0., fre, fim, ff, 0.
        do 100 i=1,nhkl
           th_obs = 360./PI * theta(i)
           ff = ffreal(i)**2 + ffimag(i)**2 
           write(ka,'(2x,2i4,2x,4i4,i8,3x,2f12.4,4f15.3)')
     *          i, iref(5,i), iref(1,i), iref(2,i), iref(3,i), 
     *          iref(4,i), iref(6,i), th_obs, rlopo(i), 
     *          ffreal(i), ffimag(i), ff, rint(i)/scale
100     continue

      else if(imo.eq.2) then

        write(ka,'(2x,3a)')
     *  ' Nr. Co.      H     K     L     Mult',
     *  '        2-Theta       Lopo            P*P',
     *  '            F*F       Intensity'
        do 150 i=1,nhkl
           call hvec(iref(1,i),iref(2,i),iref(3,i),iref(4,i),q,hm)
           th_obs = 360./PI * theta(i) 
           write(ka,'(2x,2i4,2x,3f6.1,i8,3x,2f12.4,3f15.3)')
     *          i, iref(5,i), hm(1), hm(2), hm(3), iref(6,i), th_obs,
     *          rlopo(i), ffreal(i), ffimag(i), rint(i)/scale
150     continue

      else

        write(ka,'(2x,3a)')
     *  ' Nr. Co.     h   k   l    Mult',
     *  '        2-Theta       Lopo             A              B',
     *  '             F*F       Intensity'
        write(ka,'(2x,2i4,2x,3i4,i8,3x,2f12.4,4f15.3)')
     *       0, 1, 0, 0, 0, 1, 0., 0., fre, fim, ff, 0.
        do 200 i=1,nhkl
           th_obs = 360./PI * theta(i)
           ff = ffreal(i)**2 + ffimag(i)**2 
           write(ka,'(2x,2i4,2x,3i4,i8,3x,2f12.4,4f15.3)')
     *          i, iref(5,i), iref(1,i), iref(2,i), iref(3,i), 
     *          iref(5,i), th_obs, rlopo(i), 
     *          ffreal(i), ffimag(i), ff, rint(i)/scale
200     continue

      end if


      return
      end


C******************************************************************************
C***               Symmetrie-Karten einlesen                                ***
C******************************************************************************

      subroutine syminp(ke,symfile,imo,ismat,spgrp,intaus,laue,icent,
     *                  ism,tran,nprim,nsym,q)

      parameter(maxline = 132, PI = 3.14159265359, max = 6)
      real tran(3,48), trp(3,48), trs(3,48), q(3)
      character rgsymb(max)*10, char(max)*10
      character spgrp*60, symfile*80, intaus*1, line*132, mcell*2 
      character csym(48)*3, symor(48)*9, clau(15)*8, vollsymb*60
      integer ism(3,3,48), numsy(48), num(48), nprim(48)

c --- Lauegruppen:

      data clau(1),clau(2),clau(3),clau(4),clau(5),clau(6),clau(7),
     *     clau(8),clau(9),clau(10),clau(11),clau(12),clau(13),
     *     clau(14),clau(15)
     * /'1b','2/m (C)','m m m','4/m','4/m m m','3b','3b m 1',
     *  '6/m','6/m m m','m 3b','m 3b m','2/m (B)','3b (R)','3b m (R)',
     *  '3b 1 m'/


      if(spgrp(1:3).eq.'   ') then

c --- Datei mit Symmetriekarten einlesen:

        nfile = 1
 
        write(*,*)
        line = ' '
        line = '  Space group file: '//symfile
        call writline(6,line,maxline)
        call symcar(ke,symfile,imo,intaus,mcell,laue,icent,ism,tran,
     *              nprim,nsym)

      else

        nfile = 0
        write(*,'(/2x,a,a)') 'Space group: ',spgrp

c --- Raumgruppensymbol "spgrp" in seine Teilsymbole "rgsymb" und
c     "char" zerlegen:

        call spgsym(imo,spgrp,char,rgsymb,nsbl,max,mcell)

c --- Raumgruppensymbol rgsymb testen und vollst. Symbol, Nummer und 
c     Aufstellung bestimmen:

        call rgtest(rgsymb,nsbl,max,vollsymb,nspg,nauf)

c --- Symmetriekarten bestimmen:

        call symint(vollsymb,intaus,laue,icent,ism,tran,nsym)

      end if

c --- Bei modulierten/magnetischen Strukturen und Zentrosymmetrie
c     die Inversion auf alle Symmetrieelemente anwenden:

      if(icent.eq.2) then
        if(imo.eq.1.or.(imo.eq.2.and.nfile.eq.0)) then
          ns = nsym
          do 700 n=1,nsym
             ns = ns + 1
             do 660 i=1,3
                tran(i,ns) = tran(i,n)
                do 650 j=1,3
                   ism(i,j,ns) = -ism(i,j,n)
650             continue
660          continue
700       continue
          nsym = ns
          icent = 1
        end if
      end if

c --- Kristallsystem bestimmen:

      nc = 1
      if(laue.eq.6.or.laue.eq.7.or.laue.eq.8.or.
     *   laue.eq.9.or.laue.eq.15) nc = 2

c --- Symmetrieelemente sortieren:

      call symsort(nc,csym,symor,ism,tran,trp,trs,num,numsy,nsym,48)

c --- Bei magnetischen Raumgruppen gestrichene Symmetrieelemente und
c     Wellenvektor bestimmen:

      if(imo.eq.2) then
        if(nfile.eq.0) then
          call magspg(char,nsbl+1,max,nc,nspg,csym,symor,ism,nprim,nsym)
        end if
        call wavvek(intaus,mcell,q)
      end if

c --- Symmetriekarten rausschreiben, falls erwuenscht:

      if(nfile.eq.0) then
        write(*,'(2x,a,a40,/2x,a,i3,a,i1)') 
     *       'Complete space group symbol: ',vollsymb,
     *       'Space group no. ',nspg,',   cell choice ',nauf
      end if
      write(*,'(2x,a,i4)') 'Number of symmetry cards = ',nsym
      if(icent.eq.2) then
        write(*,'(2x,a)') 'This space group is centrosymmetric'
      else
        write(*,'(2x,a)') 'This space group is non-centrosymmetric'
      end if
      write(*,'(2x,a,a,4x,a,a)')
     *'Centring type : ',intaus,'    Laue-group : ',clau(laue)
      if(imo.eq.2) then
        if(mcell.eq.'1 '.and.magfer(nprim,ism,nsym).eq.1) then
          write(*,'(2x,a)') 'This space group is ferromagnetic.'
        else
          write(*,'(2x,a)') 'This space group is antiferromagnetic.'
        end if
        write(*,'(2x,a,a)') 'Magnetic cell : ',mcell
        write(*,'(2x,a,3f8.5)') 
     *  'Wave vector of the magnetic structure: ',(q(i),i=1,3)
      end if
      if(ismat.eq.1) then
        write(*,'(/2x,a/)') 'S y m m e t r y   c a r d s'
        if(imo.eq.2) then
          do 135 n=1,nsym
             write(*,132) csym(n),symor(n),
     *            (real(ism(1,j,n)),j=1,3),tran(1,n),trp(1,n),
     *            (real(ism(2,j,n)),j=1,3),tran(2,n),trp(2,n),nprim(n),
     *            (real(ism(3,j,n)),j=1,3),tran(3,n),trp(3,n)
132          format(2x,a,1x,a,' :',8x,3f5.1,8x,f5.2,8x,f5.2,/,
     *              21x,'R=  ',3f5.1,'    T=  ',f5.2,'    Tp= ',f5.2,
     *              '   E= ',i2,/,25x,3f5.1,8x,f5.2,8x,f5.2/)
135       continue
        else
          do 145 n=1,nsym
             write(*,142) csym(n),symor(n),
     *            (real(ism(1,j,n)),j=1,3),tran(1,n),trp(1,n),
     *            ((real(ism(i,j,n)),j=1,3),tran(j,n),trp(j,n),i=2,3)
142          format(2x,a,1x,a,' :',8x,3f5.1,8x,f5.2,8x,f5.2,/,
     *              21x,'R=  ',3f5.1,'    T=  ',f5.2,'    Tp= ',f5.2,/,
     *              25x,3f5.1,8x,f5.2,8x,f5.2/)
145       continue
        end if
      end if

      return
      end


c******************************************************************************
c***   Symmetriekarten und Auslenkungsfelder aus einer Datei einlesen       ***
c******************************************************************************

      subroutine symcar(ke,symfile,imo,intaus,mcell,laue,icent,ism,
     *                  tran,nprim,nsym)

      parameter(max = 30, PI = 3.14159265359)
      integer ism(3,3,48), nprim(48)
      real werte(max), tran(3,48)
      character symfile*80, intaus*1, mcell*2, char(max)*132, ecod*3

      ecod = 'eee'

c --- Formate:

 31   format(/2x,'Error in space group file: Line "',a3,
     *  '> ..." is missing')

c --- Symmetriekarten-File oeffnen:

      open(ke,file=symfile,status='old',err=1000)

c --- Int. Ausl., Lauegruppe, Zentrosymmetrie einlesen:

      ist = 1
      call getinp(ke,'COD',ecod,werte,char,max,num,nn,ist)
      if(ist.eq.11) then
        write(*,31) 'COD'
        stop
      else
        if(imo.eq.2) then
          if(num.ne.4) then
            write(*,'(/2x,a,/2x,a)')
     *      'Error in space group file: wrong number of parameters,',
     *      'it should be 4, line "COD> ..."'
            stop
          end if
          intaus = char(1)(1:1)
          mcell  = char(2)(1:2)
          laue   = nint(werte(3))
          icent  = nint(werte(4))
        else
          if(num.ne.3) then
            write(*,'(/2x,a,/2x,a)')
     *      'Error in space group file: wrong number of parameters,',
     *      'it should be 3, line "COD> ..."'
            stop
          end if
          intaus = char(1)(1:1)
          laue   = nint(werte(2))
          icent  = nint(werte(3))
        end if
      end if

c --- Symmetriekarten einlesen:

      n = 0
100   ist = 1
      call getinp(ke,'SYM',ecod,werte,char,max,num,nn,ist)
      if(ist.eq.11.or.ist.eq.13) then
        if(n.eq.0) then
          write(*,31) 'SYM'
          stop
        else
          nsym = n
          goto 400
        end if
      else
        n = n + 1
        if(imo.eq.2) then
          if(num.ne.14) then
            write(*,'(/2x,a,/2x,a,i2)') 
     *      'Error in space group file: wrong number of parameters,',
     *      'it should be 14, line "SYM> ...", symmetry card ',n
            stop
          end if
          nprim(n) = nint(werte(14))
        else
          if(num.ne.13) then
            write(*,'(/2x,a,/2x,a,i2)') 
     *      'Error in space group file: wrong number of parameters,',
     *      'it should be 14, line "SYM> ...", symmetry card ',n
            stop
          end if
        end if
        do 120 i=1,3
           tran(i,n) = werte(10+i)
           do 130 j=1,3
              ism(i,j,n) = nint(werte(3*(i-1)+j+1))
130        continue
120     continue
      end if
      goto 100


c --- Datei schliessen:

400   close(ke)


c --- Falls die Symmetriekarte (x,y,z) fehlt, wird sie ergaenzt:

      iall = 0
      do 500 n=1,nsym
         idi = 0
	 do 499 i=1,3
	    if(ism(i,i,n).eq.1.and.tran(i,n).eq.0.) idi = idi + 1
499      continue
	 if(idi.eq.3) iall = 1
500   continue

      if(iall.ne.1) then
        nsym = nsym + 1
        n = nsym
        do 600 i=1,3
           tran(i,n) = 0.
           do 599 j=1,3
              ism(i,j,n) = 0
599        continue
           ism(i,i,n) = 1
600     continue
        if(imo.eq.2) nprim(n) = 1
      end if

      return


c --- Fehlermeldungen:

1000  stop ' Error: Cannot open space group file' 
  
      end


c******************************************************************************
c***    Bestimmung der umfaerbenden Symmetrieelemente                       ***
c******************************************************************************

      subroutine magspg(char,nzahl,max,nc,nspg,csym,symor,
     *                  ism,nprim,nsym)

      external symchg
      character symchg*3
      character char(max)*10, syor(10)*9
      character csym(48)*3, psym(48)*3, symor(48)*9, sybl(10)*3
      integer ism(3,3,48), nprim(48), nstrich(10)


c --- Generatoren der magnetischen Raumgruppe bestimmen:

      call genbest(char,nzahl,max,nspg,nc,sybl,syor,nstrich,ngen)

c --- Generatoren den nukl. Raumgruppenelementen zuordnen:

      do 49 ns=1,nsym
         psym(ns) = symchg(csym(ns))
         nprim(ns) = 0
 49   continue

      do 65 i=1,ngen
         do 60 ns=1,nsym 
            if(sybl(i).eq.psym(ns).and.syor(i).eq.symor(ns)) then
              nprim(ns) = nstrich(i)
              goto 65
            end if
 60      continue
         write(*,'(2x,2a,2x,2a)') 
     *   'Generator ',sybl(i),syor(i),' not identified.'
         stop
 65   continue

c --- Symmetrieelemente der nukl. Raumgruppe aus den Generatoren bestimmen
c     und testen, ob das Element umfaerbt order nicht:

      nprim(1) = 1
      call umfaerb(nprim,ism,nsym)
      ipr = 0
      do 85 ns=1,nsym
         if(nprim(ns).eq.-1) ipr = ipr + 1
 85   continue
      if(ipr.ne.nsym/2.and.ipr.ne.0) then
        write(*,'(2x,a)') 'Bad number of primed elements.'
        stop
      end if

      return
      end


c******************************************************************************
c***         Symbol und Orientierung der Generatoren bestimmen              ***
c******************************************************************************

      subroutine genbest(char,nzahl,max,nspg,nc,sybl,syor,
     *                   nstrich,ngen)

      integer nstrich(10), nst(2)
      character char(max)*10, sybl(10)*3, syor(10)*9, sym(2)*3


c --- Triklin:

      if(nspg.le.2) then
         call symbol(char(3),sym,nst,nsb)
         ngen = 1
         sybl(1) = sym(1)
         syor(1) = '         '
         nstrich(1) = nst(1)

c --- Monoklin:

      else if(nspg.ge.3.and.nspg.le.15) then

         call symbol(char(3),sym,nst,nsb)
         do 51 i=1,nsb
            sybl(i) = sym(i)
            syor(i) = '  [0,1,0]'
            nstrich(i) = nst(i)
 51      continue
         ngen = nsb

c --- Orthorhombisch:

      else if(nspg.ge.16.and.nspg.le.74) then
         call symbol(char(3),sym,nst,nsb)
         sybl(1) = sym(1)
         syor(1) = '  [1,0,0]'
         nstrich(1) = nst(1)
         call symbol(char(4),sym,nst,nsb)
         sybl(2) = sym(1)
         syor(2) = '  [0,1,0]'
         nstrich(2) = nst(1)
         call symbol(char(5),sym,nst,nsb)
         sybl(3) = sym(1)
         syor(3) = '  [0,0,1]'
         nstrich(3) = nst(1)
         ngen = 3

c --- Tetragonal:

      else if(nspg.ge.75.and.nspg.le.142) then
         call symbol(char(3),sym,nst,nsb)
         do 52 i=1,nsb
            sybl(i) = sym(i)
            syor(i) = '  [0,0,1]'
            nstrich(i) = nst(i)
 52      continue
         ngen = nsb
         if(nzahl.ge.4) then
            call symbol(char(4),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            syor(ngen) = '  [1,0,0]'
            nstrich(ngen) = nst(1)
         end if
         if(nzahl.eq.5) then
            call symbol(char(5),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            syor(ngen) = ' [1,-1,0]'
            nstrich(ngen) = nst(1)
         end if

c --- Trigonal:

      else if(nspg.ge.143.and.nspg.le.167) then
         if(nc.eq.1) then
            call symbol(char(3),sym,nst,nsb)
            sybl(1) = sym(1)
            syor(1) = '  [1,1,1]'
            nstrich(i) = nst(1)
            ngen = 1
            if(nzahl.eq.4) then
               call symbol(char(4),sym,nst,nsb)
               ngen = ngen + 1
               sybl(ngen) = sym(1)
               syor(ngen) = ' [1,-1,0]'
               nstrich(ngen) = nst(1)
            end if
         else if(nc.eq.2) then
            call symbol(char(3),sym,nst,nsb)
            sybl(1) = sym(1)
            syor(1) = '  [0,0,1]'
            nstrich(i) = nst(1)
            ngen = 1
            if(nzahl.ge.4) then
               call symbol(char(4),sym,nst,nsb)
               ngen = ngen + 1
               sybl(ngen) = sym(1)
               if(sym(1).eq.'  1') then
                 syor(ngen) = '         '
               else
                 syor(ngen) = '  [1,0,0]'
               end if
               nstrich(ngen) = nst(1)
            end if
            if(nzahl.eq.5) then
               call symbol(char(5),sym,nst,nsb)
               ngen = ngen + 1
               sybl(ngen) = sym(1)
               if(sym(1).eq.'  1') then
                 syor(ngen) = '         '
               else
                 syor(ngen) = ' [1,-1,0]'
               end if
               nstrich(ngen) = nst(1)
            end if
         end if

c --- Hexagonal:

      else if(nspg.ge.168.and.nspg.le.194) then
         call symbol(char(3),sym,nst,nsb)
         do 53 i=1,nsb
            sybl(i) = sym(i)
            syor(i) = '  [0,0,1]'
            nstrich(i) = nst(i)
 53      continue
         ngen = nsb
         if(nzahl.ge.4) then
            call symbol(char(4),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            syor(ngen) = '  [1,0,0]'
            nstrich(ngen) = nst(1)
         end if
         if(nzahl.eq.5) then
            call symbol(char(5),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            syor(ngen) = ' [1,-1,0]'
            nstrich(ngen) = nst(1)
         end if

c --- Kubisch:            

      else if(nspg.ge.195.and.nspg.le.230) then
         call symbol(char(3),sym,nst,nsb)
         do 54 i=1,nsb
            sybl(i) = sym(i)
            syor(i) = '  [0,0,1]'
            nstrich(i) = nst(i)
 54      continue
         ngen = nsb
         call symbol(char(4),sym,nst,nsb)
         ngen = ngen + 1
         if(sym(1).eq.'3b+') then
           sybl(ngen) = ' 3+'
         else
           sybl(ngen) = sym(1)
         end if
         syor(ngen) = '  [1,1,1]'
         nstrich(ngen) = nst(1)
         if(nzahl.ge.5.and.nspg.ge.207) then
            call symbol(char(5),sym,nst,nsb)
            ngen = ngen + 1
            sybl(ngen) = sym(1)
            if(nspg.ge.207.and.nspg.le.220) then
               syor(ngen) = '  [1,1,0]'
            else
               syor(ngen) = ' [1,-1,0]'
            end if
            nstrich(ngen) = nst(1)
         end if

      end if

      return
      end

c******************************************************************************
c***                Erkennung des Symbols                                   ***
c******************************************************************************

      subroutine symbol(char,sym,nst,nsy)

      integer nst(2)
      character char*10, sym(2)*3

c --- Initialisierung:

      nsy = 0
      sym(1) = '   '
      sym(2) = '   '
      nst(1) = 0
      nst(2) = 0

c --- Testen, ob "/" im String chr enthalten ist:

      ibksl = index(char,'/')
      if(ibksl.gt.0) then
        nsy = 2
        sym(1) = char(1:ibksl-1)
        sym(2) = char(ibksl+1:ibksl+3)
      else
        nsy = 1
        sym(1) = char(1:3)
      end if

c --- Symmetrieelement zuordnen:

      do 100 i=1,nsy
         if(index(sym(i),'''').gt.0) then
           nst(i) = -1
         else
           nst(i) = 1
         end if
         if(index(sym(i),'1b').gt.0) then
            sym(i) = ' 1b'
         else if(index(sym(i),'3b').gt.0) then
            sym(i) = '3b+'
         else if(index(sym(i),'4b').gt.0) then
            sym(i) = '4b+'
         else if(index(sym(i),'6b').gt.0) then
            sym(i) = '6b+'
         else if(index(sym(i),'21').gt.0) then
            sym(i) = '  2'
         else if(index(sym(i),'31').gt.0) then
            sym(i) = ' 3+'
         else if(index(sym(i),'32').gt.0) then
            sym(i) = ' 3+'
         else if(index(sym(i),'41').gt.0) then
            sym(i) = ' 4+'
         else if(index(sym(i),'42').gt.0) then
            sym(i) = ' 4+'
         else if(index(sym(i),'43').gt.0) then
            sym(i) = ' 4+'
         else if(index(sym(i),'61').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'62').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'63').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'64').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'65').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'2').gt.0) then
            sym(i) = '  2'
         else if(index(sym(i),'3').gt.0) then
            sym(i) = ' 3+'
         else if(index(sym(i),'4').gt.0) then
            sym(i) = ' 4+'
         else if(index(sym(i),'6').gt.0) then
            sym(i) = ' 6+'
         else if(index(sym(i),'m').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'a').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'b').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'c').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'d').gt.0) then
            sym(i) = '  m'
         else if(index(sym(i),'n').gt.0) then
            sym(i) = '  m'
         else
            sym(i) = '  1'
         end if
 100  continue

      end

c******************************************************************************
c***  Name der Symmetrieelemente der Punktgruppe bzgl. der Raumgruppe       ***
c******************************************************************************

      character*3 function symchg(csym)

      character csym*3

      if(csym.eq.'  m'.or.csym.eq.'  a'.or.csym.eq.'  b'.or.
     *   csym.eq.'  c'.or.csym.eq.'  n'.or.csym.eq.'  d') then
         symchg = '  m'
      else if(csym.eq.' 21') then
         symchg = '  2'
      else if(csym(1:2).eq.'31'.or.csym(1:2).eq.'32') then
         symchg(1:2) = ' 3'
         symchg(3:3) = csym(3:3)
      else if(csym(1:2).eq.'41'.or.csym(1:2).eq.'42'.or.
     *        csym(1:2).eq.'43') then
        symchg(1:2) = ' 4'
        symchg(3:3) = csym(3:3)
      else if(csym(1:2).eq.'61'.or.csym(1:2).eq.'62'.or.
     *        csym(1:2).eq.'63'.or.csym(1:2).eq.'64'.or.
     *        csym(1:2).eq.'65') then
        symchg(1:2) = ' 6'
        symchg(3:3) = csym(3:3)
      else
        symchg = csym
      end if

      return
      end

c******************************************************************************
c***                 In Grossbuchstaben umwandeln                           ***
c******************************************************************************

      character*1 function caplet(string)

      character string*1
     
      k1 = ichar('A')
      k2 = ichar('Z')
      n1 = ichar('a')
      n2 = ichar('z')

      iz = ichar(string)
      if(iz.ge.n1.and.iz.le.n2) then 
        caplet = char(iz-n2+k2)
      else
        caplet = string 
      end if

      return
      end

c******************************************************************************
c***         Testen, ob Symmetrieelement umfaerbt oder nicht                ***
c******************************************************************************

      subroutine umfaerb(nprim,ism,nsym)

      integer nprim(48), ism(3,3,48), matrix(3,3)

      ipr = 0
      do 10 n=1,nsym
         if(nprim(n).ne.0) ipr = ipr + 1
 10   continue
      if(ipr.eq.nsym) goto 999

 20   do 300 n=1,nsym
         if(nprim(n).ne.0) then
           do 200 k=1,nsym
              if(nprim(k).ne.0) then
                 call matprd(matrix,ism(1,1,n),ism(1,1,k))
                 do 50 l=1,nsym
                    if(nprim(l).eq.0) then
                       is = 0
                       do 45 i=1,3
                          do 44 j=1,3
                             if(matrix(i,j).eq.ism(i,j,l)) is = is + 1
 44                       continue
 45                    continue
                       if(is.eq.9) then
                          nprim(l) = nprim(n) * nprim(k)
                          ipr = ipr + 1
                          if(ipr.eq.nsym) then
                            goto 999
                          else
                            goto 200
                          end if
                       end if
                    end if
 50              continue   ! Schleife l
              end if
 200       continue   ! Schleife k
         end if
 300  continue   ! Schleife n
      goto 20


 999  return
      end

c******************************************************************************
c***                       Matrixmultiplikation                             ***
c******************************************************************************

      subroutine matprd(mc,ma,mb)

      integer ma(3,3), mb(3,3), mc(3,3)

      do 60 i=1,3
         do 55 j=1,3
            mc(i,j) = 0
            do 50 k=1,3
               mc(i,j) = mc(i,j) + ma(i,k) * mb(k,j)
 50         continue
 55      continue
 60   continue

      return
      end

c******************************************************************************
c***            Test auf ferromagnetische Raumgruppe                        ***
c******************************************************************************

      integer function magfer(nprim,ism,nsym)

      external ndet3
      integer nprim(48), ism(3,3,48), ma(3,3)

      do 20 i=1,3
         do 10 j=1,3
            ma(i,j) = 0
 10      continue
 20   continue

      do 50 n=1,nsym
         nd = ndet3(ism(1,1,n))
         do 40 i=1,3
            do 30 j=1,3
               ma(i,j) = ma(i,j) + ism(i,j,n) * nd * nprim(n)
 30         continue
 40      continue
 50   continue

      magfer = 0
      do 70 i=1,3
         do 60 j=1,3
            if(ma(i,j).ne.0) then
               magfer = 1
               goto 100
            end if
 60      continue
 70   continue

 100  return
      end

c******************************************************************************
c***           Berechnung der Determinante einer 3x3-Matrix                 ***
c***                  nach der Sarrusschen Regel                            ***
c******************************************************************************

      integer function ndet3(matrix)

      integer matrix(3,3)

      ndet3 = matrix(1,1) * matrix(2,2) * matrix(3,3) +
     *        matrix(1,2) * matrix(2,3) * matrix(3,1) +
     *        matrix(1,3) * matrix(2,1) * matrix(3,2) -
     *        matrix(1,3) * matrix(2,2) * matrix(3,1) -
     *        matrix(1,1) * matrix(2,3) * matrix(3,2) -
     *        matrix(1,2) * matrix(2,1) * matrix(3,3)

c      ndet3 = matrix(1,1) * matrix(2,2) * matrix(3,3) +
c     *        matrix(2,1) * matrix(3,2) * matrix(1,3) +
c     *        matrix(3,1) * matrix(1,2) * matrix(2,3) -
c     *        matrix(3,1) * matrix(2,2) * matrix(1,3) -
c     *        matrix(1,1) * matrix(3,2) * matrix(2,3) -
c     *        matrix(2,1) * matrix(1,2) * matrix(3,3)

      return
      end

c***************************************************************************
c***                  Wellenvektor bestimmen
c***************************************************************************

      subroutine wavvek(intaus,mcell,q)

      parameter(NV = 14)
      character intaus*1, mcell*2
      real qvec(3,NV), q(3)
      data (qvec(j,1),j=1,3)  / 0.,  0.,  0. /
      data (qvec(j,2),j=1,3)  / 0.5, 0.,  0. /
      data (qvec(j,3),j=1,3)  / 0.,  0.5, 0. /
      data (qvec(j,4),j=1,3)  / 0.,  0.,  0.5/
      data (qvec(j,5),j=1,3)  / 0.5, 0.5, 0. /
      data (qvec(j,6),j=1,3)  / 0.5, 0.,  0.5/
      data (qvec(j,7),j=1,3)  / 0.,  0.5, 0.5/
      data (qvec(j,8),j=1,3)  / 0.5, 0.5, 0.5/
      data (qvec(j,9),j=1,3)  / 1.,  0.,  0. /
      data (qvec(j,10),j=1,3) / 0.,  1.,  0. /
      data (qvec(j,11),j=1,3) / 0.,  0.,  1. / 
      data (qvec(j,12),j=1,3) / 0.5, 1.,  0. /
      data (qvec(j,13),j=1,3) / 1.,  0.5, 0. /
      data (qvec(j,14),j=1,3) / 1.,  0.,  0.5/

      if(intaus.eq.'P') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'2a') then
          nq = 2
        else if(mcell.eq.'2b') then
          nq = 3
        else if(mcell.eq.'2c') then
          nq = 4
        else if(mcell.eq.'C ') then
          nq = 5
        else if(mcell.eq.'B ') then
          nq = 6
        else if(mcell.eq.'A ') then
          nq = 7
        else if(mcell.eq.'F ') then
          nq = 8
        else if(mcell.eq.'P ') then
          nq = 5
        else if(mcell.eq.'I ') then
          nq = 8
        end if
      else if(intaus.eq.'A') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'2a') then
          nq = 2
        else if(mcell.eq.'P ') then
          nq = 10
        else if(mcell.eq.'I ') then
          nq = 12
        end if
      else if(intaus.eq.'B') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'2b') then
          nq = 3
        else if(mcell.eq.'P ') then
          nq = 9
        else if(mcell.eq.'I ') then
          nq = 13
        end if
      else if(intaus.eq.'C') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'2c') then
          nq = 4
        else if(mcell.eq.'P ') then
          nq = 9
        else if(mcell.eq.'I ') then
          nq = 14
        end if
      else if(intaus.eq.'I') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'P ') then
          nq = 9
        end if
      else if(intaus.eq.'F') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'A ') then
          nq = 9
        else if(mcell.eq.'B ') then
          nq = 10
        else if(mcell.eq.'C ') then
          nq = 11
        end if
      else if(intaus.eq.'R') then
        if(mcell.eq.'1 ') then
          nq = 1
        else if(mcell.eq.'R ') then
          nq = 8
        end if
      end if

c --- Wellenvektor q abspeichern:

      do 100 j=1,3
         q(j) = qvec(j,nq)
100   continue

      return
      end

c***********************************************************************
c***      Teilsymbole des Raumgruppensymbols bestimmen               ***
c***********************************************************************

      subroutine getsym(spgrp,char,num,max)

      character spgrp*60, char(max)*10


c --- Initialisierung:

      do 90 i=1,max
         char(i) = ' '
 90   continue

c --- Anfang suchen:

 110  do 120 i=1,60
         if(spgrp(i:i).ne.' ') then
           ibeg = i
           goto 130
         end if
 120  continue    
      return 

c --- Teilsymbole abspeichern:

 130  ic    = 0
      n     = 0
      num   = 1
      ibl   = 0
      komma = 0

      do 250 i=ibeg,60
         if(spgrp(i:i).eq.' ') then
           ibl = ibl + 1
         else if(spgrp(i:i).eq.',') then
           komma = 1
           num = num + 1
           n = 0
         else
           if(ibl.gt.0.and.komma.eq.0) then
             num = num + 1
             n = 0
           end if
           ic = ic + 1
           n = n + 1
           char(num)(n:n) = spgrp(i:i)
           ibl = 0
           komma = 0
         end if
 250  continue
      if(num.gt.max) then
        write(*,'(2x,a)') 'Bad space group symbol'
        stop
      end if

      return
      end


c******************************************************************************
c***                   Raumgruppen-Eingabe testen                           ***
c******************************************************************************

      subroutine rgtest(symb,num,max,spgrp,nspg,nc)

      parameter(NRG = 306)
      integer numrg(NRG), ncell(NRG)
      character kurz(NRG)*12, lang(NRG)*17, symb(max)*10,
     *          string*60, spgrp*60

      data numrg(  1), ncell(  1), kurz(  1), lang(  1)
     *  /  1,1,'P,1         ','P,1,,            '/
      data numrg(  2), ncell(  2), kurz(  2), lang(  2)
     *  /  2,1,'P,1B        ','P,1B,,           '/
      data numrg(  3), ncell(  3), kurz(  3), lang(  3)
     *  /  3,1,'P,2,1       ','P,1,2,1          '/
      data numrg(  4), ncell(  4), kurz(  4), lang(  4)
     *  /  3,2,'P,2,2        ','P,1,1,2          '/
      data numrg(  5), ncell(  5), kurz(  5), lang(  5)
     *  /  4,1,'P,21,1      ','P,1,21,1         '/
      data numrg(  6), ncell(  6), kurz(  6), lang(  6)
     *  /  4,2,'P,21,2      ','P,1,1,21         '/
      data numrg(  7), ncell(  7), kurz(  7), lang(  7)
     *  /  5,1,'C,2,1       ','C,1,2,1          '/
      data numrg(  8), ncell(  8), kurz(  8), lang(  8)
     *  /  5,2,'A,2,1       ','A,1,2,1          '/
      data numrg(  9), ncell(  9), kurz(  9), lang(  9)
     *  /  5,3,'I,2,1       ','I,1,2,1          '/
      data numrg( 10), ncell( 10), kurz( 10), lang( 10)
     *  /  5,4,'A,2,2       ','A,1,1,2          '/
      data numrg( 11), ncell( 11), kurz( 11), lang( 11)
     *  /  5,5,'B,2,2       ','B,1,1,2          '/
      data numrg( 12), ncell( 12), kurz( 12), lang( 12)
     *  /  5,6,'I,2,2       ','I,1,1,2          '/
      data numrg( 13), ncell( 13), kurz( 13), lang( 13)
     *  /  6,1,'P,M,1       ','P,1,M,1          '/
      data numrg( 14), ncell( 14), kurz( 14), lang( 14)
     *  /  6,2,'P,M,2       ','P,1,1,M          '/
      data numrg( 15), ncell( 15), kurz( 15), lang( 15)
     *  /  7,1,'P,C,1       ','P,1,C,1          '/
      data numrg( 16), ncell( 16), kurz( 16), lang( 16)
     *  /  7,2,'P,N,1       ','P,1,N,1          '/
      data numrg( 17), ncell( 17), kurz( 17), lang( 17)
     *  /  7,3,'P,A,1       ','P,1,A,1          '/
      data numrg( 18), ncell( 18), kurz( 18), lang( 18)
     *  /  7,4,'P,A,2       ','P,1,1,A          '/
      data numrg( 19), ncell( 19), kurz( 19), lang( 19)
     *  /  7,5,'P,N,2       ','P,1,1,N          '/
      data numrg( 20), ncell( 20), kurz( 20), lang( 20)
     *  /  7,6,'P,B,2       ','P,1,1,B          '/
      data numrg( 21), ncell( 21), kurz( 21), lang( 21)
     *  /  8,1,'C,M,1       ','C,1,M,1          '/
      data numrg( 22), ncell( 22), kurz( 22), lang( 22)
     *  /  8,2,'A,M,1       ','A,1,M,1          '/
      data numrg( 23), ncell( 23), kurz( 23), lang( 23)
     *  /  8,3,'I,M,1       ','I,1,M,1          '/
      data numrg( 24), ncell( 24), kurz( 24), lang( 24)
     *  /  8,4,'A,M,2       ','A,1,1,M          '/
      data numrg( 25), ncell( 25), kurz( 25), lang( 25)
     *  /  8,5,'B,M,2       ','B,1,1,M          '/
      data numrg( 26), ncell( 26), kurz( 26), lang( 26)
     *  /  8,6,'I,M,2       ','I,1,1,M          '/
      data numrg( 27), ncell( 27), kurz( 27), lang( 27)
     *  /  9,1,'C,C,1       ','C,1,C,1          '/
      data numrg( 28), ncell( 28), kurz( 28), lang( 28)
     *  /  9,2,'A,N,1       ','A,1,N,1          '/
      data numrg( 29), ncell( 29), kurz( 29), lang( 29)
     *  /  9,3,'I,A,1       ','I,1,A,1          '/
      data numrg( 30), ncell( 30), kurz( 30), lang( 30)
     *  /  9,4,'A,A,2       ','A,1,1,A          '/
      data numrg( 31), ncell( 31), kurz( 31), lang( 31)
     *  /  9,5,'B,N,2       ','B,1,1,N          '/
      data numrg( 32), ncell( 32), kurz( 32), lang( 32)
     *  /  9,6,'I,B,2       ','I,1,1,B          '/
      data numrg( 33), ncell( 33), kurz( 33), lang( 33)
     *  / 10,1,'P,2/M,1     ','P,1,2/M,1        '/
      data numrg( 34), ncell( 34), kurz( 34), lang( 34)
     *  / 10,2,'P,2/M,2     ','P,1,1,2/M        '/
      data numrg( 35), ncell( 35), kurz( 35), lang( 35)
     *  / 11,1,'P,21/M,1    ','P,1,21/M,1       '/
      data numrg( 36), ncell( 36), kurz( 36), lang( 36)
     *  / 11,2,'P,21/M,2    ','P,1,1,21/M       '/
      data numrg( 37), ncell( 37), kurz( 37), lang( 37)
     *  / 12,1,'C,2/M,1     ','C,1,2/M,1        '/
      data numrg( 38), ncell( 38), kurz( 38), lang( 38)
     *  / 12,2,'A,2/M,1     ','A,1,2/M,1        '/
      data numrg( 39), ncell( 39), kurz( 39), lang( 39)
     *  / 12,3,'I,2/M,1     ','I,1,2/M,1        '/
      data numrg( 40), ncell( 40), kurz( 40), lang( 40)
     *  / 12,4,'A,2/M,2     ','A,1,1,2/M        '/
      data numrg( 41), ncell( 41), kurz( 41), lang( 41)
     *  / 12,5,'B,2/M,2     ','B,1,1,2/M        '/
      data numrg( 42), ncell( 42), kurz( 42), lang( 42)
     *  / 12,6,'I,2/M,2     ','I,1,1,2/M        '/
      data numrg( 43), ncell( 43), kurz( 43), lang( 43)
     *  / 13,1,'P,2/C,1     ','P,1,2/C,1        '/
      data numrg( 44), ncell( 44), kurz( 44), lang( 44)
     *  / 13,2,'P,2/N,1     ','P,1,2/N,1        '/
      data numrg( 45), ncell( 45), kurz( 45), lang( 45)
     *  / 13,3,'P,2/A,1     ','P,1,2/A,1        '/
      data numrg( 46), ncell( 46), kurz( 46), lang( 46)
     *  / 13,4,'P,2/A,2     ','P,1,1,2/A        '/
      data numrg( 47), ncell( 47), kurz( 47), lang( 47)
     *  / 13,5,'P,2/N,2     ','P,1,1,2/N        '/
      data numrg( 48), ncell( 48), kurz( 48), lang( 48)
     *  / 13,6,'P,2/B,2     ','P,1,1,2/B        '/
      data numrg( 49), ncell( 49), kurz( 49), lang( 49)
     *  / 14,1,'P,21/C,1    ','P,1,21/C,1       '/
      data numrg( 50), ncell( 50), kurz( 50), lang( 50)
     *  / 14,2,'P,21/N,1    ','P,1,21/N,1       '/
      data numrg( 51), ncell( 51), kurz( 51), lang( 51)
     *  / 14,3,'P,21/A,1    ','P,1,21/A,1       '/
      data numrg( 52), ncell( 52), kurz( 52), lang( 52)
     *  / 14,4,'P,21/A,2    ','P,1,1,21/A       '/
      data numrg( 53), ncell( 53), kurz( 53), lang( 53)
     *  / 14,5,'P,21/N,2    ','P,1,1,21/N       '/
      data numrg( 54), ncell( 54), kurz( 54), lang( 54)
     *  / 14,6,'P,21/B,2    ','P,1,1,21/B       '/
      data numrg( 55), ncell( 55), kurz( 55), lang( 55)
     *  / 15,1,'C,2/C,1     ','C,1,2/C,1        '/
      data numrg( 56), ncell( 56), kurz( 56), lang( 56)
     *  / 15,2,'A,2/N,1     ','A,1,2/N,1        '/
      data numrg( 57), ncell( 57), kurz( 57), lang( 57)
     *  / 15,3,'I,2/A,1     ','I,1,2/A,1        '/
      data numrg( 58), ncell( 58), kurz( 58), lang( 58)
     *  / 15,4,'A,2/A,2     ','A,1,1,2/A        '/
      data numrg( 59), ncell( 59), kurz( 59), lang( 59)
     *  / 15,5,'B,2/N,2     ','B,1,1,2/N        '/
      data numrg( 60), ncell( 60), kurz( 60), lang( 60)
     *  / 15,6,'I,2/B,2     ','I,1,1,2/B        '/
      data numrg( 61), ncell( 61), kurz( 61), lang( 61)
     *  / 16,1,'P,2,2,2     ','P,2,2,2          '/
      data numrg( 62), ncell( 62), kurz( 62), lang( 62)
     *  / 17,1,'P,2,2,21    ','P,2,2,21         '/
      data numrg( 63), ncell( 63), kurz( 63), lang( 63)
     *  / 18,1,'P,21,21,2   ','P,21,21,2        '/
      data numrg( 64), ncell( 64), kurz( 64), lang( 64)
     *  / 19,1,'P,21,21,21  ','P,21,21,21       '/
      data numrg( 65), ncell( 65), kurz( 65), lang( 65)
     *  / 20,1,'C,2,2,21    ','C,2,2,21         '/
      data numrg( 66), ncell( 66), kurz( 66), lang( 66)
     *  / 21,1,'C,2,2,2     ','C,2,2,2          '/
      data numrg( 67), ncell( 67), kurz( 67), lang( 67)
     *  / 22,1,'F,2,2,2     ','F,2,2,2          '/
      data numrg( 68), ncell( 68), kurz( 68), lang( 68)
     *  / 23,1,'I,2,2,2     ','I,2,2,2          '/
      data numrg( 69), ncell( 69), kurz( 69), lang( 69)
     *  / 24,1,'I,21,21,21  ','I,21,21,21       '/
      data numrg( 70), ncell( 70), kurz( 70), lang( 70)
     *  / 25,1,'P,M,M,2     ','P,M,M,2          '/
      data numrg( 71), ncell( 71), kurz( 71), lang( 71)
     *  / 26,1,'P,M,C,21    ','P,M,C,21         '/
      data numrg( 72), ncell( 72), kurz( 72), lang( 72)
     *  / 27,1,'P,C,C,2     ','P,C,C,2          '/
      data numrg( 73), ncell( 73), kurz( 73), lang( 73)
     *  / 28,1,'P,M,A,2     ','P,M,A,2          '/
      data numrg( 74), ncell( 74), kurz( 74), lang( 74)
     *  / 29,1,'P,C,A,21    ','P,C,A,21         '/
      data numrg( 75), ncell( 75), kurz( 75), lang( 75)
     *  / 30,1,'P,N,C,2     ','P,N,C,2          '/
      data numrg( 76), ncell( 76), kurz( 76), lang( 76)
     *  / 31,1,'P,M,N,21    ','P,M,N,21         '/
      data numrg( 77), ncell( 77), kurz( 77), lang( 77)
     *  / 32,1,'P,B,A,2     ','P,B,A,2          '/
      data numrg( 78), ncell( 78), kurz( 78), lang( 78)
     *  / 33,1,'P,N,A,21    ','P,N,A,21         '/
      data numrg( 79), ncell( 79), kurz( 79), lang( 79)
     *  / 34,1,'P,N,N,2     ','P,N,N,2          '/
      data numrg( 80), ncell( 80), kurz( 80), lang( 80)
     *  / 35,1,'C,M,M,2     ','C,M,M,2          '/
      data numrg( 81), ncell( 81), kurz( 81), lang( 81)
     *  / 36,1,'C,M,C,21    ','C,M,C,21         '/
      data numrg( 82), ncell( 82), kurz( 82), lang( 82)
     *  / 37,1,'C,C,C,2     ','C,C,C,2          '/
      data numrg( 83), ncell( 83), kurz( 83), lang( 83)
     *  / 38,1,'A,M,M,2     ','A,M,M,2          '/
      data numrg( 84), ncell( 84), kurz( 84), lang( 84)
     *  / 39,1,'A,B,M,2     ','A,B,M,2          '/
      data numrg( 85), ncell( 85), kurz( 85), lang( 85)
     *  / 40,1,'A,M,A,2     ','A,M,A,2          '/
      data numrg( 86), ncell( 86), kurz( 86), lang( 86)
     *  / 41,1,'A,B,A,2     ','A,B,A,2          '/
      data numrg( 87), ncell( 87), kurz( 87), lang( 87)
     *  / 42,1,'F,M,M,2     ','F,M,M,2          '/
      data numrg( 88), ncell( 88), kurz( 88), lang( 88)
     *  / 43,1,'F,D,D,2     ','F,D,D,2          '/
      data numrg( 89), ncell( 89), kurz( 89), lang( 89)
     *  / 44,1,'I,M,M,2     ','I,M,M,2          '/
      data numrg( 90), ncell( 90), kurz( 90), lang( 90)
     *  / 45,1,'I,B,A,2     ','I,B,A,2          '/
      data numrg( 91), ncell( 91), kurz( 91), lang( 91)
     *  / 46,1,'I,M,A,2     ','I,M,A,2          '/
      data numrg( 92), ncell( 92), kurz( 92), lang( 92)
     *  / 47,1,'P,M,M,M     ','P,2/M,2/M,2/M    '/
      data numrg( 93), ncell( 93), kurz( 93), lang( 93)
     *  / 48,1,'P,N,N,N,1   ','P,2/N,2/N,2/N,1  '/
      data numrg( 94), ncell( 94), kurz( 94), lang( 94)
     *  / 48,2,'P,N,N,N,2   ','P,2/N,2/N,2/N,2  '/
      data numrg( 95), ncell( 95), kurz( 95), lang( 95)
     *  / 49,1,'P,C,C,M     ','P,2/C,2/C,2/M    '/
      data numrg( 96), ncell( 96), kurz( 96), lang( 96)
     *  / 50,1,'P,B,A,N,1   ','P,2/B,2/A,2/N,1  '/
      data numrg( 97), ncell( 97), kurz( 97), lang( 97)
     *  / 50,2,'P,B,A,N,2   ','P,2/B,2/A,2/N,2  '/
      data numrg( 98), ncell( 98), kurz( 98), lang( 98)
     *  / 51,1,'P,M,M,A     ','P,21/M,2/M,2/A   '/
      data numrg( 99), ncell( 99), kurz( 99), lang( 99)
     *  / 52,1,'P,N,N,A     ','P,2/N,21/N,2/A   '/
      data numrg(100), ncell(100), kurz(100), lang(100)
     *  / 53,1,'P,M,N,A     ','P,2/M,2/N,21/A   '/
      data numrg(101), ncell(101), kurz(101), lang(101)
     *  / 54,1,'P,C,C,A     ','P,21/C,2/C,2/A   '/
      data numrg(102), ncell(102), kurz(102), lang(102)
     *  / 55,1,'P,B,A,M     ','P,21/B,21/A,2/M  '/
      data numrg(103), ncell(103), kurz(103), lang(103)
     *  / 56,1,'P,C,C,N     ','P,21/C,21/C,2/N  '/
      data numrg(104), ncell(104), kurz(104), lang(104)
     *  / 57,1,'P,B,C,M     ','P,2/B,21/C,21/M  '/
      data numrg(105), ncell(105), kurz(105), lang(105)
     *  / 58,1,'P,N,N,M     ','P,21/N,21/N,2/M  '/
      data numrg(106), ncell(106), kurz(106), lang(106)
     *  / 59,1,'P,M,M,N,1   ','P,21/M,21/M,2/N,1'/
      data numrg(107), ncell(107), kurz(107), lang(107)
     *  / 59,2,'P,M,M,N,2   ','P,21/M,21/M,2/N,2'/
      data numrg(108), ncell(108), kurz(108), lang(108)
     *  / 60,1,'P,B,C,N     ','P,21/B,2/C,21/N  '/
      data numrg(109), ncell(109), kurz(109), lang(109)
     *  / 61,1,'P,B,C,A     ','P,21/B,21/C,21/A '/
      data numrg(110), ncell(110), kurz(110), lang(110)
     *  / 62,1,'P,N,M,A     ','P,21/N,21/M,21/A '/
      data numrg(111), ncell(111), kurz(111), lang(111)
     *  / 63,1,'C,M,C,M     ','C,2/M,2/C,21/M   '/
      data numrg(112), ncell(112), kurz(112), lang(112)
     *  / 64,1,'C,M,C,A     ','C,2/M,2/C,21/A   '/
      data numrg(113), ncell(113), kurz(113), lang(113)
     *  / 65,1,'C,M,M,M     ','C,2/M,2/M,2/M    '/
      data numrg(114), ncell(114), kurz(114), lang(114)
     *  / 66,1,'C,C,C,M     ','C,2/C,2/C,2/M    '/
      data numrg(115), ncell(115), kurz(115), lang(115)
     *  / 67,1,'C,M,M,A     ','C,2/M,2/M,2/A    '/
      data numrg(116), ncell(116), kurz(116), lang(116)
     *  / 68,1,'C,C,C,A,1   ','C,2/C,2/C,2/A,1  '/
      data numrg(117), ncell(117), kurz(117), lang(117)
     *  / 68,2,'C,C,C,A,2   ','C,2/C,2/C,2/A,2  '/
      data numrg(118), ncell(118), kurz(118), lang(118)
     *  / 69,1,'F,M,M,M     ','F,2/M,2/M,2/M    '/
      data numrg(119), ncell(119), kurz(119), lang(119)
     *  / 70,1,'F,D,D,D,1   ','F,2/D,2/D,2/D,1  '/
      data numrg(120), ncell(120), kurz(120), lang(120)
     *  / 70,2,'F,D,D,D,2   ','F,2/D,2/D,2/D,2  '/
      data numrg(121), ncell(121), kurz(121), lang(121)
     *  / 71,1,'I,M,M,M     ','I,2/M,2/M,2/M    '/
      data numrg(122), ncell(122), kurz(122), lang(122)
     *  / 72,1,'I,B,A,M     ','I,2/B,2/A,2/M    '/
      data numrg(123), ncell(123), kurz(123), lang(123)
     *  / 73,1,'I,B,C,A     ','I,21/B,21/C,21/A '/
      data numrg(124), ncell(124), kurz(124), lang(124)
     *  / 74,1,'I,M,M,A     ','I,21/M,21/M,21/A '/
      data numrg(125), ncell(125), kurz(125), lang(125)
     *  / 75,1,'P,4         ','P,4,,            '/
      data numrg(126), ncell(126), kurz(126), lang(126)
     *  / 76,1,'P,41        ','P,41,,           '/
      data numrg(127), ncell(127), kurz(127), lang(127)
     *  / 77,1,'P,42        ','P,42,,           '/
      data numrg(128), ncell(128), kurz(128), lang(128)
     *  / 78,1,'P,43        ','P,43,,           '/
      data numrg(129), ncell(129), kurz(129), lang(129)
     *  / 79,1,'I,4         ','I,4,,            '/
      data numrg(130), ncell(130), kurz(130), lang(130)
     *  / 80,1,'I,41        ','I,41,,           '/
      data numrg(131), ncell(131), kurz(131), lang(131)
     *  / 81,1,'P,4B        ','P,4B,,           '/
      data numrg(132), ncell(132), kurz(132), lang(132)
     *  / 82,1,'I,4B        ','I,4B,,           '/
      data numrg(133), ncell(133), kurz(133), lang(133)
     *  / 83,1,'P,4/M       ','P,4/M,,          '/
      data numrg(134), ncell(134), kurz(134), lang(134)
     *  / 84,1,'P,42/M      ','P,42/M,,         '/
      data numrg(135), ncell(135), kurz(135), lang(135)
     *  / 85,1,'P,4/N,1     ','P,4/N,,,1        '/
      data numrg(136), ncell(136), kurz(136), lang(136)
     *  / 85,2,'P,4/N,2     ','P,4/N,,,2        '/
      data numrg(137), ncell(137), kurz(137), lang(137)
     *  / 86,1,'P,42/N,1    ','P,42/N,,,1       '/
      data numrg(138), ncell(138), kurz(138), lang(138)
     *  / 86,2,'P,42/N,2    ','P,42/N,,,2       '/
      data numrg(139), ncell(139), kurz(139), lang(139)
     *  / 87,1,'I,4/M       ','I,4/M,,          '/
      data numrg(140), ncell(140), kurz(140), lang(140)
     *  / 88,1,'I,41/A,1    ','I,41/A,,,1       '/
      data numrg(141), ncell(141), kurz(141), lang(141)
     *  / 88,2,'I,41/A,2    ','I,41/A,,,2       '/
      data numrg(142), ncell(142), kurz(142), lang(142)
     *  / 89,1,'P,4,2,2     ','P,4,2,2          '/
      data numrg(143), ncell(143), kurz(143), lang(143)
     *  / 90,1,'P,4,21,2    ','P,4,21,2         '/
      data numrg(144), ncell(144), kurz(144), lang(144)
     *  / 91,1,'P,41,2,2    ','P,41,2,2         '/
      data numrg(145), ncell(145), kurz(145), lang(145)
     *  / 92,1,'P,41,21,2   ','P,41,21,2        '/
      data numrg(146), ncell(146), kurz(146), lang(146)
     *  / 93,1,'P,42,2,2    ','P,42,2,2         '/
      data numrg(147), ncell(147), kurz(147), lang(147)
     *  / 94,1,'P,42,21,2   ','P,42,21,2        '/
      data numrg(148), ncell(148), kurz(148), lang(148)
     *  / 95,1,'P,43,2,2    ','P,43,2,2         '/
      data numrg(149), ncell(149), kurz(149), lang(149)
     *  / 96,1,'P,43,21,2   ','P,43,21,2        '/
      data numrg(150), ncell(150), kurz(150), lang(150)
     *  / 97,1,'I,4,2,2     ','I,4,2,2          '/
      data numrg(151), ncell(151), kurz(151), lang(151)
     *  / 98,1,'I,41,2,2    ','I,41,2,2         '/
      data numrg(152), ncell(152), kurz(152), lang(152)
     *  / 99,1,'P,4,M,M     ','P,4,M,M          '/
      data numrg(153), ncell(153), kurz(153), lang(153)
     *  /100,1,'P,4,B,N     ','P,4,B,M          '/
      data numrg(154), ncell(154), kurz(154), lang(154)
     *  /101,1,'P,42,C,M    ','P,42,C,M         '/
      data numrg(155), ncell(155), kurz(155), lang(155)
     *  /102,1,'P,42,N,M    ','P,42,N,M         '/
      data numrg(156), ncell(156), kurz(156), lang(156)
     *  /103,1,'P,4,C,C     ','P,4,C,C          '/
      data numrg(157), ncell(157), kurz(157), lang(157)
     *  /104,1,'P,4,N,C     ','P,4,N,C          '/
      data numrg(158), ncell(158), kurz(158), lang(158)
     *  /105,1,'P,42,M,C    ','P,42,M,C         '/
      data numrg(159), ncell(159), kurz(159), lang(159)
     *  /106,1,'P,42,B,C    ','P,42,B,C         '/
      data numrg(160), ncell(160), kurz(160), lang(160)
     *  /107,1,'I,4,M,M     ','I,4,M,M          '/
      data numrg(161), ncell(161), kurz(161), lang(161)
     *  /108,1,'I,4,C,M     ','I,4,C,M          '/
      data numrg(162), ncell(162), kurz(162), lang(162)
     *  /109,1,'I,41,M,D    ','I,41,M,D         '/
      data numrg(163), ncell(163), kurz(163), lang(163)
     *  /110,1,'I,41,C,D    ','I,41,C,D         '/
      data numrg(164), ncell(164), kurz(164), lang(164)
     *  /111,1,'P,4B,2,M    ','P,4B,2,M         '/
      data numrg(165), ncell(165), kurz(165), lang(165)
     *  /112,1,'P,4B,2,C    ','P,4B,2,C         '/
      data numrg(166), ncell(166), kurz(166), lang(166)
     *  /113,1,'P,4B,21,M   ','P,4B,21,M        '/
      data numrg(167), ncell(167), kurz(167), lang(167)
     *  /114,1,'P,4B,21,C   ','P,4B,21,C        '/
      data numrg(168), ncell(168), kurz(168), lang(168)
     *  /115,1,'P,4B,M,2    ','P,4B,M,2         '/
      data numrg(169), ncell(169), kurz(169), lang(169)
     *  /116,1,'P,4B,C,2    ','P,4B,C,2         '/
      data numrg(170), ncell(170), kurz(170), lang(170)
     *  /117,1,'P,4B,B,2    ','P,4B,B,2         '/
      data numrg(171), ncell(171), kurz(171), lang(171)
     *  /118,1,'P,4B,N,2    ','P,4B,N,2         '/
      data numrg(172), ncell(172), kurz(172), lang(172)
     *  /119,1,'I,4B,M,2    ','I,4B,M,2         '/
      data numrg(173), ncell(173), kurz(173), lang(173)
     *  /120,1,'I,4B,C,2    ','I,4B,C,2         '/
      data numrg(174), ncell(174), kurz(174), lang(174)
     *  /121,1,'I,4B,2,M    ','I,4B,2,M         '/
      data numrg(175), ncell(175), kurz(175), lang(175)
     *  /122,1,'I,4B,2,D    ','I,4B,2,D         '/
      data numrg(176), ncell(176), kurz(176), lang(176)
     *  /123,1,'P,4/M,M,M   ','P,4/M,2/M,2/M    '/
      data numrg(177), ncell(177), kurz(177), lang(177)
     *  /124,1,'P,4/M,C,C   ','P,4/M,2/C,2/C    '/
      data numrg(178), ncell(178), kurz(178), lang(178)
     *  /125,1,'P,4/N,B,M,1 ','P,4/N,2/B,2/M,1  '/
      data numrg(179), ncell(179), kurz(179), lang(179)
     *  /125,2,'P,4/N,B,M,2 ','P,4/N,2/B,2/M,2  '/
      data numrg(180), ncell(180), kurz(180), lang(180)
     *  /126,1,'P,4/N,N,C,1 ','P,4/N,2/N,2/C,1  '/
      data numrg(181), ncell(181), kurz(181), lang(181)
     *  /126,2,'P,4/N,N,C,2 ','P,4/N,2/N,2/C,2  '/
      data numrg(182), ncell(182), kurz(182), lang(182)
     *  /127,1,'P,4/M,B,M   ','P,4/M,21/B,2/M   '/
      data numrg(183), ncell(183), kurz(183), lang(183)
     *  /128,1,'P,4/M,N,C   ','P,4/M,21/N,2/C   '/
      data numrg(184), ncell(184), kurz(184), lang(184)
     *  /129,1,'P,4/N,M,M,1 ','P,4/N,21/M,2/M,1 '/
      data numrg(185), ncell(185), kurz(185), lang(185)
     *  /129,2,'P,4/N,M,M,2 ','P,4/N,21/M,2/M,2 '/
      data numrg(186), ncell(186), kurz(186), lang(186)
     *  /130,1,'P,4/N,C,C,1 ','P,4/N,21/C,2/C,1 '/
      data numrg(187), ncell(187), kurz(187), lang(187)
     *  /130,2,'P,4/N,C,C,2 ','P,4/N,21/C,2/C,2 '/
      data numrg(188), ncell(188), kurz(188), lang(188)
     *  /131,1,'P,42/M,M,C  ','P,42/M,2/M,2/C   '/
      data numrg(189), ncell(189), kurz(189), lang(189)
     *  /132,1,'P,42/M,C,M  ','P,42/M,2/C,2/M   '/
      data numrg(190), ncell(190), kurz(190), lang(190)
     *  /133,1,'P,42/N,B,C,1','P,42/N,2/B,2/C,1 '/
      data numrg(191), ncell(191), kurz(191), lang(191)
     *  /133,2,'P,42/N,B,C,2','P,42/N,2/B,2/C,2 '/
      data numrg(192), ncell(192), kurz(192), lang(192)
     *  /134,1,'P,42/N,N,M,1','P,42/N,2/N,2/M,1 '/
      data numrg(193), ncell(193), kurz(193), lang(193)
     *  /134,2,'P,42/N,N,M,2','P,42/N,2/N,2/M,2 '/
      data numrg(194), ncell(194), kurz(194), lang(194)
     *  /135,1,'P,42/M,B,C  ','P,42/M,21/B,2/C  '/
      data numrg(195), ncell(195), kurz(195), lang(195)
     *  /136,1,'P,42/M,N,M  ','P,42/M,21/N,2/M  '/
      data numrg(196), ncell(196), kurz(196), lang(196)
     *  /137,1,'P,42/N,M,C,1','P,42/N,21/M,2/C,1'/
      data numrg(197), ncell(197), kurz(197), lang(197)
     *  /137,2,'P,42/N,M,C,2','P,42/N,21/M,2/C,2'/
      data numrg(198), ncell(198), kurz(198), lang(198)
     *  /138,1,'P,42/N,C,M,1','P,42/N,21/C,2/M,1'/
      data numrg(199), ncell(199), kurz(199), lang(199)
     *  /138,2,'P,42/N,C,M,2','P,42/N,21/C,2/M,2'/
      data numrg(200), ncell(200), kurz(200), lang(200)
     *  /139,1,'I,4/M,M,M   ','I,4/M,2/M,2/M    '/
      data numrg(201), ncell(201), kurz(201), lang(201)
     *  /140,1,'I,4/M,C,M   ','I,4/M,2/C,2/M    '/
      data numrg(202), ncell(202), kurz(202), lang(202)
     *  /141,1,'I,41/A,M,D,1','I,41/A,2/M,2/D,1 '/
      data numrg(203), ncell(203), kurz(203), lang(203)
     *  /141,2,'I,41/A,M,D,2','I,41/A,2/M,2/D,2 '/
      data numrg(204), ncell(204), kurz(204), lang(204)
     *  /142,1,'I,41/A,C,D,1','I,41/A,2/C,2/D,1 '/
      data numrg(205), ncell(205), kurz(205), lang(205)
     *  /142,2,'I,41/A,C,D,2','I,41/A,2/C,2/D,2 '/
      data numrg(206), ncell(206), kurz(206), lang(206)
     *  /143,1,'P,3         ','P,3,,            '/
      data numrg(207), ncell(207), kurz(207), lang(207)
     *  /144,1,'P,31        ','P,31,,           '/
      data numrg(208), ncell(208), kurz(208), lang(208)
     *  /145,1,'P,32        ','P,32,,           '/
      data numrg(209), ncell(209), kurz(209), lang(209)
     *  /146,1,'R,3,H       ','R,3,,,H          '/
      data numrg(210), ncell(210), kurz(210), lang(210)
     *  /146,2,'R,3,R       ','R,3,,,R          '/
      data numrg(211), ncell(211), kurz(211), lang(211)
     *  /147,1,'P,3B        ','P,3B,,           '/
      data numrg(212), ncell(212), kurz(212), lang(212)
     *  /148,1,'R,3B,H      ','R,3B,,,H         '/
      data numrg(213), ncell(213), kurz(213), lang(213)
     *  /148,2,'R,3B,R      ','R,3B,,,R         '/
      data numrg(214), ncell(214), kurz(214), lang(214)
     *  /149,1,'P,3,1,2     ','P,3,1,2          '/
      data numrg(215), ncell(215), kurz(215), lang(215)
     *  /150,1,'P,3,2,1     ','P,3,2,1          '/
      data numrg(216), ncell(216), kurz(216), lang(216)
     *  /151,1,'P,31,1,2    ','P,31,1,2         '/
      data numrg(217), ncell(217), kurz(217), lang(217)
     *  /152,1,'P,31,2,1    ','P,31,2,1         '/
      data numrg(218), ncell(218), kurz(218), lang(218)
     *  /153,1,'P,32,1,2    ','P,32,1,2         '/
      data numrg(219), ncell(219), kurz(219), lang(219)
     *  /154,1,'P,32,2,1    ','P,32,2,1         '/
      data numrg(220), ncell(220), kurz(220), lang(220)
     *  /155,1,'R,3,2,H     ','R,3,2,,H         '/
      data numrg(221), ncell(221), kurz(221), lang(221)
     *  /155,2,'R,3,2,R     ','R,3,2,,R         '/
      data numrg(222), ncell(222), kurz(222), lang(222)
     *  /156,1,'P,3,M,1     ','P,3,M,1          '/
      data numrg(223), ncell(223), kurz(223), lang(223)
     *  /157,1,'P,3,1,M     ','P,3,1,M          '/
      data numrg(224), ncell(224), kurz(224), lang(224)
     *  /158,1,'P,3,C,1     ','P,3,C,1          '/
      data numrg(225), ncell(225), kurz(225), lang(225)
     *  /159,1,'P,3,1,C     ','P,3,1,C          '/
      data numrg(226), ncell(226), kurz(226), lang(226)
     *  /160,1,'R,3,M,H     ','R,3,M,,H         '/
      data numrg(227), ncell(227), kurz(227), lang(227)
     *  /160,2,'R,3,M,R     ','R,3,M,,R         '/
      data numrg(228), ncell(228), kurz(228), lang(228)
     *  /161,1,'R,3,C,H     ','R,3,C,,H         '/
      data numrg(229), ncell(229), kurz(229), lang(229)
     *  /161,2,'R,3,C,R     ','R,3,C,,R         '/
      data numrg(230), ncell(230), kurz(230), lang(230)
     *  /162,1,'P,3B,1,M    ','P,3B,1,2/M       '/
      data numrg(231), ncell(231), kurz(231), lang(231)
     *  /163,1,'P,3B,1,C    ','P,3B,1,2/C       '/
      data numrg(232), ncell(232), kurz(232), lang(232)
     *  /164,1,'P,3B,M,1    ','P,3B,2/M,1       '/
      data numrg(233), ncell(233), kurz(233), lang(233)
     *  /165,1,'P,3B,C,1    ','P,3B,2/C,1       '/
      data numrg(234), ncell(234), kurz(234), lang(234)
     *  /166,1,'R,3B,M,H    ','R,3B,2/M,,H      '/
      data numrg(235), ncell(235), kurz(235), lang(235)
     *  /166,2,'R,3B,M,R    ','R,3B,2/M,,R      '/
      data numrg(236), ncell(236), kurz(236), lang(236)
     *  /167,1,'R,3B,C,H    ','R,3B,2/C,,H      '/
      data numrg(237), ncell(237), kurz(237), lang(237)
     *  /167,2,'R,3B,C,R    ','R,3B,2/C,,R      '/
      data numrg(238), ncell(238), kurz(238), lang(238)
     *  /168,1,'P,6         ','P,6,,            '/
      data numrg(239), ncell(239), kurz(239), lang(239)
     *  /169,1,'P,61        ','P,61,,           '/
      data numrg(240), ncell(240), kurz(240), lang(240)
     *  /170,1,'P,65        ','P,65,,           '/
      data numrg(241), ncell(241), kurz(241), lang(241)
     *  /171,1,'P,62        ','P,62,,           '/
      data numrg(242), ncell(242), kurz(242), lang(242)
     *  /172,1,'P,64        ','P,64,,           '/
      data numrg(243), ncell(243), kurz(243), lang(243)
     *  /173,1,'P,63        ','P,63,,           '/
      data numrg(244), ncell(244), kurz(244), lang(244)
     *  /174,1,'P,6B        ','P,6B,,           '/
      data numrg(245), ncell(245), kurz(245), lang(245)
     *  /175,1,'P,6/M       ','P,6/M,,          '/
      data numrg(246), ncell(246), kurz(246), lang(246)
     *  /176,1,'P,63/M      ','P,63/M,,         '/
      data numrg(247), ncell(247), kurz(247), lang(247)
     *  /177,1,'P,6,2,2     ','P,6,2,2          '/
      data numrg(248), ncell(248), kurz(248), lang(248)
     *  /178,1,'P,61,2,2    ','P,61,2,2         '/
      data numrg(249), ncell(249), kurz(249), lang(249)
     *  /179,1,'P,65,2,2    ','P,65,2,2         '/
      data numrg(250), ncell(250), kurz(250), lang(250)
     *  /180,1,'P,62,2,2    ','P,62,2,2         '/
      data numrg(251), ncell(251), kurz(251), lang(251)
     *  /181,1,'P,64,2,2    ','P,64,2,2         '/
      data numrg(252), ncell(252), kurz(252), lang(252)
     *  /182,1,'P,63,2,2    ','P,63,2,2         '/
      data numrg(253), ncell(253), kurz(253), lang(253)
     *  /183,1,'P,6,M,M     ','P,6,M,M          '/
      data numrg(254), ncell(254), kurz(254), lang(254)
     *  /184,1,'P,6,C,C     ','P,6,C,C          '/
      data numrg(255), ncell(255), kurz(255), lang(255)
     *  /185,1,'P,63,C,M    ','P,63,C,M         '/
      data numrg(256), ncell(256), kurz(256), lang(256)
     *  /186,1,'P,63,M,C    ','P,63,M,C         '/
      data numrg(257), ncell(257), kurz(257), lang(257)
     *  /187,1,'P,6B,M,2    ','P,6B,M,2         '/
      data numrg(258), ncell(258), kurz(258), lang(258)
     *  /188,1,'P,6B,C,2    ','P,6B,C,2         '/
      data numrg(259), ncell(259), kurz(259), lang(259)
     *  /189,1,'P,6B,2,M    ','P,6B,2,M         '/
      data numrg(260), ncell(260), kurz(260), lang(260)
     *  /190,1,'P,6B,2,C    ','P,6B,2,C         '/
      data numrg(261), ncell(261), kurz(261), lang(261)
     *  /191,1,'P,6/M,M,M   ','P,6/M,2/M,2/M    '/
      data numrg(262), ncell(262), kurz(262), lang(262)
     *  /192,1,'P,6/M,C,C   ','P,6/M,2/C,2/C    '/
      data numrg(263), ncell(263), kurz(263), lang(263)
     *  /193,1,'P,63/M,C,M  ','P,63/M,2/C,2/M   '/
      data numrg(264), ncell(264), kurz(264), lang(264)
     *  /194,1,'P,63/M,M,C  ','P,63/M,2/M,2/C   '/
      data numrg(265), ncell(265), kurz(265), lang(265)
     *  /195,1,'P,2,3       ','P,2,3,           '/
      data numrg(266), ncell(266), kurz(266), lang(266)
     *  /196,1,'F,2,3       ','F,2,3,           '/
      data numrg(267), ncell(267), kurz(267), lang(267)
     *  /197,1,'I,2,3       ','I,2,3,           '/
      data numrg(268), ncell(268), kurz(268), lang(268)
     *  /198,1,'P,21,3      ','P,21,3,          '/
      data numrg(269), ncell(269), kurz(269), lang(269)
     *  /199,1,'I,21,3      ','I,21,3,          '/
      data numrg(270), ncell(270), kurz(270), lang(270)
     *  /200,1,'P,M,3B      ','P,2/M,3B,        '/
      data numrg(271), ncell(271), kurz(271), lang(271)
     *  /201,1,'P,N,3B,1    ','P,2/N,3B,,1      '/
      data numrg(272), ncell(272), kurz(272), lang(272)
     *  /201,2,'P,N,3B,2    ','P,2/N,3B,,2      '/
      data numrg(273), ncell(273), kurz(273), lang(273)
     *  /202,1,'F,M,3B      ','F,2/M,3B,        '/
      data numrg(274), ncell(274), kurz(274), lang(274)
     *  /203,1,'F,D,3B,1    ','F,2/D,3B,,1      '/
      data numrg(275), ncell(275), kurz(275), lang(275)
     *  /203,2,'F,D,3B,2    ','F,2/D,3B,,2      '/
      data numrg(276), ncell(276), kurz(276), lang(276)
     *  /204,1,'I,M,3B      ','I,2/M,3B,        '/
      data numrg(277), ncell(277), kurz(277), lang(277)
     *  /205,1,'P,A,3B      ','P,21/A,3B,       '/
      data numrg(278), ncell(278), kurz(278), lang(278)
     *  /206,1,'I,A,3B      ','I,21/A,3B,       '/
      data numrg(279), ncell(279), kurz(279), lang(279)
     *  /207,1,'P,4,3,2     ','P,4,3,2          '/
      data numrg(280), ncell(280), kurz(280), lang(280)
     *  /208,1,'P,42,3,2    ','P,42,3,2         '/
      data numrg(281), ncell(281), kurz(281), lang(281)
     *  /209,1,'F,4,3,2     ','F,4,3,2          '/
      data numrg(282), ncell(282), kurz(282), lang(282)
     *  /210,1,'F,41,3,2    ','F,41,3,2         '/
      data numrg(283), ncell(283), kurz(283), lang(283)
     *  /211,1,'I,4,3,2     ','I,4,3,2          '/
      data numrg(284), ncell(284), kurz(284), lang(284)
     *  /212,1,'P,43,3,2    ','P,43,3,2         '/
      data numrg(285), ncell(285), kurz(285), lang(285)
     *  /213,1,'P,41,3,2    ','P,41,3,2         '/
      data numrg(286), ncell(286), kurz(286), lang(286)
     *  /214,1,'I,41,3,2    ','I,41,3,2         '/
      data numrg(287), ncell(287), kurz(287), lang(287)
     *  /215,1,'P,4B,3,M    ','P,4B,3,M         '/
      data numrg(288), ncell(288), kurz(288), lang(288)
     *  /216,1,'F,4B,3,M    ','F,4B,3,M         '/
      data numrg(289), ncell(289), kurz(289), lang(289)
     *  /217,1,'I,4B,3,M    ','I,4B,3,M         '/
      data numrg(290), ncell(290), kurz(290), lang(290)
     *  /218,1,'P,4B,3,N    ','P,4B,3,N         '/
      data numrg(291), ncell(291), kurz(291), lang(291)
     *  /219,1,'F,4B,3,C    ','F,4B,3,C         '/
      data numrg(292), ncell(292), kurz(292), lang(292)
     *  /220,1,'I,4B,3,D    ','I,4B,3,D         '/
      data numrg(293), ncell(293), kurz(293), lang(293)
     *  /221,1,'P,M,3B,M    ','P,4/M,3B,2/M     '/
      data numrg(294), ncell(294), kurz(294), lang(294)
     *  /222,1,'P,N,3B,N,1  ','P,4/N,3B,2/N,1   '/
      data numrg(295), ncell(295), kurz(295), lang(295)
     *  /222,2,'P,N,3B,N,2  ','P,4/N,3B,2/N,2   '/
      data numrg(296), ncell(296), kurz(296), lang(296)
     *  /223,1,'P,M,3B,N    ','P,42/M,3B,2/N    '/
      data numrg(297), ncell(297), kurz(297), lang(297)
     *  /224,1,'P,N,3B,M,1  ','P,42/N,3B,2/M,1  '/
      data numrg(298), ncell(298), kurz(298), lang(298)
     *  /224,2,'P,N,3B,M,2  ','P,42/N,3B,2/M,2  '/
      data numrg(299), ncell(299), kurz(299), lang(299)
     *  /225,1,'F,M,3B,M    ','F,4/M,3B,2/M     '/
      data numrg(300), ncell(300), kurz(300), lang(300)
     *  /226,1,'F,M,3B,C    ','F,4/M,3B,2/C     '/
      data numrg(301), ncell(301), kurz(301), lang(301)
     *  /227,1,'F,D,3B,M,1  ','F,41/D,3B,2/M,1  '/
      data numrg(302), ncell(302), kurz(302), lang(302)
     *  /227,2,'F,D,3B,M,2  ','F,41/D,3B,2/M,2  '/
      data numrg(303), ncell(303), kurz(303), lang(303)
     *  /228,1,'F,D,3B,C,1  ','F,41/D,3B,2/C,1  '/
      data numrg(304), ncell(304), kurz(304), lang(304)
     *  /228,2,'F,D,3B,C,2  ','F,41/D,3B,2/C,2  '/
      data numrg(305), ncell(305), kurz(305), lang(305)
     *  /229,1,'I,M,3B,M    ','I,4/M,3B,2/M     '/
      data numrg(306), ncell(306), kurz(306), lang(306)
     *  /230,1,'I,A,3B,D    ','I,41/A,3B,2/D    '/

      spgrp = ' '

c --- Test auf Raumgruppen-Nummer:

      read(symb(1)(1:3),'(i3)',iostat=ios) nspg
      if(ios.ne.0) nspg = 0

      if(nspg.ge.1.and.nspg.le.230) then
        if(num.le.2) then
          if(num.eq.2) then
            read(symb(2)(1:1),'(i1)',iostat=ios) nc
            if(ios.ne.0) then
              write(*,'(/2x,a,i3)') 
     *        'Error: bad cell choice for space group no. ',nspg
              stop
            end if
          else
            nc = 1
          end if
          do 50 i=1,NRG
             if(numrg(i).eq.nspg.and.nc.eq.ncell(i)) then
               spgrp(1:17) = lang(i)(1:17)
               goto 120
             end if
 50       continue
          write(*,'(/2x,a,i3)') 
     *    'Error: bad cell choice for space group no. ',nspg
          stop
        else
          write(*,'(/2x,a,/23x,a)') 
     *    'Error in INPUT file: wrong number of values,',
     *    ' it should be 1 or 2,','line "SPG> ..."'
          stop          
        end if
      else if(nspg.lt.0.or.nspg.gt.230) then
        write(*,'(/2x,a,i3,a)') 
     *  'Error: bad space group number (No. ',nspg,')'
        stop
      else

c --- Eingelesenes Raumgruppensymbol ohne Blanks zusammensetzen:

 90     ls = 0
        string = ' '
        do 95 k=1,num
           do 93 l=1,10
              ls = ls + 1
              if(symb(k)(l:l).eq.' ') then
                string(ls:ls) = ','
                goto 95
              else
                string(ls:ls) = symb(k)(l:l)
              end if
 93        continue
 95     continue
        string(ls:ls) = ' '

c --- Test auf Raumgruppen-Symbol:
      
        do 100 i=1,NRG
           if(string(1:12).eq.kurz(i)(1:12).or.
     *        string(1:17).eq.lang(i)(1:17)) then
             spgrp(1:17) = lang(i)(1:17)
             nspg = numrg(i)
             nc   = ncell(i)
             goto 120
           end if
 100    continue
        write(*,'(/2x,a,a40)') 
     *  'Error: bad space group symbol: ',string
        stop

      end if

 120  return
      end


c******************************************************************************
c*** Bei magn. Raumgruppen nukleares Raumgruppensymbol bestimmen            ***
c******************************************************************************

      subroutine spgsym(imo,spgrp,char,rgsymb,nsbl,max,mcell)

      external caplet
      character char(max)*10, rgsymb(max)*10, caplet*1
      character spgrp*60, mcell*2 

      call getsym(spgrp,char,nzahl,max)
      if(nzahl.gt.max) then
         write(*,'(2x,2a)') 'Error: Bad space group symbol: ',spgrp
         stop
      end if

      if(imo.eq.2) then
        nsbl = nzahl - 1
        rgsymb(1) = caplet(char(1))
        mcell = char(2)(1:2)
        do 12 i=3,nzahl
           ll = 0
           do 10 k=1,10
              if(char(i)(k:k).ne.'''') then
                ll = ll + 1 
                rgsymb(i-1)(ll:ll) = caplet(char(i)(k:k))
              end if
 10        continue
 12     continue
        if(mcell.eq.'2c'.and.rgsymb(1)(1:1).eq.'P') then
          if(nzahl.eq.2) then
            if(rgsymb(2).eq.'31        ') then
              rgsymb(2)(1:2) = '32'
            else if(rgsymb(2).eq.'32        ') then
              rgsymb(2)(1:2) = '31'
            end if
          else if(nzahl.eq.4) then
            if(rgsymb(2).eq.'31        '.and.
     *         rgsymb(3).eq.'1         '.and.
     *         rgsymb(4).eq.'2         ') then
              rgsymb(2)(1:2) = '32'
            else if(rgsymb(2).eq.'31        '.and.
     *              rgsymb(3).eq.'2         '.and.
     *              rgsymb(4).eq.'1         ') then
              rgsymb(2)(1:2) = '32'
            else if(rgsymb(2).eq.'32        '.and.
     *              rgsymb(3).eq.'2         '.and.
     *              rgsymb(4).eq.'1         ') then
              rgsymb(2)(1:2) = '31'
            else if(rgsymb(2).eq.'32        '.and.
     *              rgsymb(3).eq.'2         '.and.
     *              rgsymb(4).eq.'1         ') then
              rgsymb(2)(1:2) = '31'
            end if
          end if
        end if
      else
        nsbl = nzahl
        do 15 i=1,nsbl
           do 14 k=1,10
              rgsymb(i)(k:k) = caplet(char(i)(k:k))
 14        continue
 15     continue
      end if

      return
      end



C******************************************************************************
C***               KODIERUNG DER SYMMETRIEKARTEN                            ***
C******************************************************************************

      SUBROUTINE SYMINT(STRING,CZENT,NL,ICENT,ISM,TRAN,NSYM)

      PARAMETER (MAXS = 100)

      INTEGER ISM(3,3,48)
      REAL TRAN(3,48)
 
      CHARACTER HEX(30)*13, TET(85)*14, KUB(42)*14, RHO(32)*14
      CHARACTER ORT(70)*14, MON(60)*13, TRI(2)*13

      COMMON /CHAHEX/ HEX
      COMMON /CHATET/ TET
      COMMON /CHARHO/ RHO
      COMMON /CHAKUB/ KUB
      COMMON /CHAORT/ ORT
      COMMON /CHAMON/ MON
      COMMON /CHATRI/ TRI

      COMMON /INHEX/ IHEX(30,24)
      COMMON /INTET/ ITET(85,16)
      COMMON /INRHO/ IRHO(32,12)
      COMMON /INKUB/ IKUB(42,48)
      COMMON /INORT/ IORT(70,8)
      COMMON /INMON/ IMON(60,4)
      COMMON /INTRI/ ITRI(2,2) 

      CHARACTER CH*3, CZENT*1, C1C*4, C2C*4, C3C*4, CUR*1, STRING*60
      INTEGER LL(0:3), LMAT(3,3), NITR(3), IDUM(MAXS)
      COMMON /CH/ CH, CUR
      COMMON /IENTE/ IH(9)


c --- Initialisierung:

      CZENT(1:1) = ' '
      C1C(1:4)   = '    '
      C2C(1:4)   = '    '
      C3C(1:4)   = '    '
      CUR(1:1)   = ' '

c --- Raumgruppen-Symbol:

      CALL SYMMK(STRING,CZENT,C1C,C2C,C3C,CUR)

C --- Interpretation des Symbols:

      IF(C2C(1:1).EQ.'3') THEN
C        WRITE(*,*) ' Kubische RG.'
	ICO = 43
      ELSE IF(C1C(1:1).EQ.'4') THEN
C        WRITE(*,*) ' Tetragonale RG.'
	ICO = 4
      ELSE IF(C1C(1:1).EQ.'6') THEN
C        WRITE(*,*) ' Hexagonale RG.'
	ICO = 6
      ELSE IF(C1C(1:1).EQ.'3') THEN
C        WRITE(*,*) ' Trigonale RG.'
	ICO = 3
      ELSE IF(C1C(1:1).EQ.'1') THEN
	  IF(C2C(1:1).EQ.' ') THEN
C            WRITE(*,*) ' Trikline RG.'
	    ICO = 1
	  ELSE
C            WRITE(*,*) ' Monokline RG.'
	    ICO = 2
	  END IF
      ELSE IF (C1C(1:1).EQ.'1'.OR.C2C(1:1).EQ.'1'.OR.
     *         C3C(1:1).EQ.'1') THEN
C        WRITE(*,*) ' Monokline RG.'
	ICO = 2
      ELSE
C        WRITE(*,*)' Orthorhombische RG.'
	ICO = 5
      END IF


c --- Tetragonale Raumgruppen:

      IF(ICO.EQ.4) THEN
C        WRITE(*,*)' Tetragonale Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 16
	LSYM  =  0 
	DO 477 K=1,85
	  IF(CUR(1:1).EQ.'2') THEN
	    IF(TET(K)(14:14).NE.'2') GOTO 477
	  END IF
	  IF(TET(K)(1:1).NE.CZENT(1:1)) GOTO 477
	  IF(TET(K)(2:5).NE.C1C(1:4)) GOTO 477
	  IF(TET(K)(6:9).NE.C2C(1:4)) GOTO 477
	  IF(TET(K)(10:13).NE.C3C(1:4)) GOTO 477
	  LSYM = K
	  GOTO 478
477     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (TET)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

478     CONTINUE
	DO 452 L=1,IMAXS
	  IDUM(L) = ITET(LSYM,L)
452     CONTINUE


c --- Hexagonale Raumgruppen:

      ELSE IF (ICO.EQ.6) THEN
C        WRITE(*,*)' Hexagonale Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 24
	LSYM  =  0
	DO 677 K=1,30
	  IF(HEX(K)(1:1).NE.CZENT(1:1)) GOTO 677
	  IF(HEX(K)(2:5).NE.C1C(1:4)) GOTO 677
	  IF(HEX(K)(6:9).NE.C2C(1:4)) GOTO 677
	  IF(HEX(K)(10:13).NE.C3C(1:4)) GOTO 677
	  LSYM = K
	  GOTO 678
677     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (HEX)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

678     CONTINUE
	DO 651 L=1,IMAXS
 	   IDUM(L) = IHEX(LSYM,L)
651     CONTINUE


c --- Trigonale Raumgruppen:

      ELSE IF (ICO.EQ.3) THEN
c        WRITE(*,*)' Rhomboedr. Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
c     *  ' ',CUR

	IMAXS = 12
	LSYM  =  0
	DO 377 K=1,32
	  IF(RHO(K)(1:1).NE.CZENT(1:1)) GOTO 377
	  IF(RHO(K)(2:5).NE.C1C(1:4)) GOTO 377
	  IF(RHO(K)(6:9).NE.C2C(1:4)) GOTO 377
	  IF(RHO(K)(10:13).NE.C3C(1:4)) GOTO 377
	  IF(RHO(K)(14:14).NE.CUR(1:1)) GOTO 377
	  LSYM = K
	  GOTO 378
377     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (RHO)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

378     CONTINUE

C --- BEI RHOMBOEDRISCHEN RAUMGRUPPEN STEHT EIN "R" AN ERSTER STELLE, DAS ALS 
C     INTEGRALE AUSLOESCHUNG INTERPRETIERT WIRD. DESHALB WIRD HIER IN 'CZENT' 
C     EIN "P" REINGESCHRIEBEN.

        if(cur.eq.'R') CZENT(1:1) = 'P'

	DO 351 L=1,IMAXS
	  IDUM(L) = IRHO(LSYM,L)
351     CONTINUE


c --- Kubische Raumgruppen:

      ELSE IF (ICO.EQ.43) THEN
C        WRITE(*,*)' Kubische Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 48
	LSYM  =  0
	DO 4377 K=1,42
	  IF(CUR(1:1).EQ.'2') THEN
	    IF(KUB(K)(14:14).NE.'2') GOTO 4377
	  END IF
	  IF(KUB(K)(1:1).NE.CZENT(1:1)) GOTO 4377
	  IF(KUB(K)(2:5).NE.C1C(1:4)) GOTO 4377
	  IF(KUB(K)(6:9).NE.C2C(1:4)) GOTO 4377
	  IF(KUB(K)(10:13).NE.C3C(1:4)) GOTO 4377
	  LSYM = K
	  GOTO 4378
4377    CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (KUB)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

4378    CONTINUE
	DO 4351 L=1,IMAXS
	  IDUM(L) = IKUB(LSYM,L)
4351    CONTINUE


c --- Trikline Raumgruppen:

      ELSE IF (ICO.EQ.1) THEN
C        WRITE(*,*)' Trikline Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 2
	LSYM  = 0
	DO 177 K=1,2
	  IF(TRI(K)(1:1).NE.CZENT(1:1)) GOTO 177
	  IF(TRI(K)(2:5).NE.C1C(1:4)) GOTO 177
	  IF(TRI(K)(6:9).NE.C2C(1:4)) GOTO 177
	  IF(TRI(K)(10:13).NE.C3C(1:4)) GOTO 177
	  LSYM = K
	  GOTO 178
177     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (TRI)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

178     CONTINUE
	DO 151 L=1,IMAXS
	   IDUM(L) = ITRI(LSYM,L)
151     CONTINUE


c --- Monokline Raumgruppen:

      ELSE IF (ICO.EQ.2) THEN
C        WRITE(*,*)' Monokline Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 4
	LSYM  = 0
	DO 277 K=1,60
	  IF(MON(K)(1:1).NE.CZENT(1:1)) GOTO 277
	  IF(MON(K)(2:5).NE.C1C(1:4)) GOTO 277
	  IF(MON(K)(6:9).NE.C2C(1:4)) GOTO 277
	  IF(MON(K)(10:13).NE.C3C(1:4)) GOTO 277
	  LSYM = K
	  GOTO 278
277     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (MON)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

278     CONTINUE
	DO 251 L=1,IMAXS
	   IDUM(L) = IMON(LSYM,L)
251     CONTINUE


c --- Orthorhombische Raumgruppen:

      ELSE IF (ICO.EQ.5) THEN
C        WRITE(*,*)' Orthor. Rg. ',CZENT,' ',C1C,' ',C2C,' ',C3C,
C     *  ' ',CUR

	IMAXS = 8
	LSYM  = 0
	DO 577 K=1,70
	  IF(CUR(1:1).EQ.'2') THEN
	    IF(ORT(K)(14:14).NE.'2') GOTO 577
	  END IF
	  IF(ORT(K)(1:1).NE.CZENT(1:1)) GOTO 577
	  IF(ORT(K)(2:5).NE.C1C(1:4)) GOTO 577
	  IF(ORT(K)(6:9).NE.C2C(1:4)) GOTO 577
	  IF(ORT(K)(10:13).NE.C3C(1:4)) GOTO 577
	  LSYM = K
	  GOTO 578
577     CONTINUE

	IF(LSYM.EQ.0) THEN
	  WRITE(*,*) ' This space group is unknown (ORT)'
	  WRITE(*,*) CZENT,' ',C1C,' ',C2C,' ',C3C,' ',CUR
	  STOP
	END IF

578     CONTINUE
	DO 551 L=1,IMAXS
	  IDUM(L) = IORT(LSYM,L)
551     CONTINUE

c --- Ungueltiges Raumgruppensymbol:
 
      ELSE

	WRITE(*,*)' This space group is unknown '

      END IF

c --- Schleife ueber die Matrizen:

      ISC = 0
900   CONTINUE

      DO 771 M=1,IMAXS

	 IF(IDUM(M).LE.0) GOTO 771

         ISC = ISC + 1
         ICODE = IDUM(M)
 
c --- Entcodierung:

         CALL ENTCOD(ISC,ICODE,LAUE,ICE,NNENN,LL,NITR)

         DO 500 M1=1,3
	    DO 500 M2=1,3
	       LMAT(M1,M2) = 0
500      CONTINUE

         CALL MATRIX(LL,LMAT)

         IF(ISC.EQ.1) THEN
	   NL = LAUE
	   ICENT = ICE
C           WRITE(*,*)' SYMINT LAUEGRUPPE = ',LAUE
C           WRITE(*,*)' SYMINT ZENTROSY. = ',ICE
         END IF

         IF(NNENN.EQ.0) THEN
	   DO 213 N1=1,3
	      DO 213 N2=1,3
	         ISM(N1,N2,ISC) = LMAT(N1,N2)
213        CONTINUE
C           DO 215 N1=1,3  
C              WRITE(*,*) (ISM(N1,N2,ISC),N2=1,3)
C215        CONTINUE
         ELSE
  	   DO 212 N1=1,3
	      TRANS = REAL(NITR(N1))/REAL(NNENN)
	      TRAN(N1,ISC) = TRANS    
	      DO 212 N2=1,3
	         ISM(N1,N2,ISC) = LMAT(N1,N2)
212        CONTINUE
C           DO 216 N1=1,3  
C              WRITE(*,*) (ISM(N1,N2,ISC),N2=1,3),TRAN(N1,ISC)
C216        CONTINUE
         END IF

C         WRITE(*,*)' DIE ',ISC,' TE MATRIX'

771   CONTINUE

      NSYM = ISC
C      WRITE(*,*)'ISC',ISC,'NSYM=',NSYM
   
      RETURN
      END


C******************************************************************************
C***            Einlesen von Symmetrie-Daten                                ***
C******************************************************************************

      SUBROUTINE SYMMK(STRING,CZENT,C1C,C2C,C3C,CUR)

      PARAMETER (ISTR = 60)

      CHARACTER STRING*60, STRANG*60
      CHARACTER CZENT*1, C1C*4, C2C*4, C3C*4, CUR*1

      INTEGER IKOM(0:20)


c --- Initialisierung:

      do 110 i=0,20
         ikom(i) = 0
 110  continue

C --- Die 'Blanks' werden rausgeschmissen:

      DO 140 ISP=1,ISTR
	 STRANG(ISP:ISP) = ' '
140   CONTINUE

      ISPEN = 0   
210   DO 150 ISP=1,ISTR
	 IF(STRING(ISP:ISP).NE.' ') THEN
	   ISPEN = ISPEN + 1
	   STRANG(ISPEN:ISPEN) = STRING(ISP:ISP)
	 END IF
150   CONTINUE

c      WRITE(*,*) ' SYMM - STRANG ',STRANG

C --- Kommata werden festgelegt:

      IKOMM = 0
      DO 160 JJ=1,ISPEN
	 IF(STRANG(JJ:JJ).EQ.',') THEN
	   IKOMM = IKOMM + 1
	   IKOM(IKOMM) = JJ
	 END IF
160   CONTINUE

      IF(IKOMM.GT.21) THEN
	WRITE(*,*) ' Too many commas in space group symbol'
	STOP
      END IF

      I1  = IKOM(1) - 1
      CZENT(1:1) = STRANG(1:I1)
      I1  = I1 + 2
      I2  = IKOM(2) - 1
      C1C = STRANG(I1:I2)
      I1  = I2 + 2
      I2  = IKOM(3) - 1
      IF(I1.LE.I2) C2C = STRANG(I1:I2)
      I1  = I2 + 2
      IF(IKOM(4).EQ.0) THEN
	C3C = STRANG(I1:I1+4)
	CUR = ' '
      ELSE
	I2  = IKOM(4) - 1
	IF(I1.LE.I2) C3C = STRANG(I1:I2)
	I1  = I2 + 2
	CUR = STRANG(I1:I1)
      END IF

c      WRITE(*,*) ' SYMM: ',CZENT,C1C,C2C,C3C,CUR

9999  RETURN
      END


C******************************************************************************
C***            Entkodierung der Raumgruppen                                ***
C******************************************************************************

      SUBROUTINE ENTCOD(ISC,ICODE,LAUE,ICE,NNENN,LL,NITR)

      INTEGER LL(0:*),NITR(*)

      COMMON/IENTE/IH(9)

      IF(ISC.EQ.1) THEN
	LAUE  = ICODE/IH(9)
	ICODE = ICODE-LAUE*IH(9)
	ICE   = ICODE/IH(8)
	ICODE = ICODE-ICE*IH(8)
      END IF

      LL(0)   = ICODE/IH(7)-4
      ICODE   = ICODE-(LL(0)+4)*IH(7)
      LL(1)   = ICODE/IH(6)-4
      ICODE   = ICODE-(LL(1)+4)*IH(6)
      LL(2)   = ICODE/IH(5)-4
      ICODE   = ICODE-(LL(2)+4)*IH(5)
      LL(3)   = ICODE/IH(4)-4
      ICODE   = ICODE-(LL(3)+4)*IH(4)
      NNENN   = ICODE/IH(3)
      ICODE   = ICODE-NNENN*IH(3)
      NITR(1) = ICODE/IH(2)
      ICODE   = ICODE-NITR(1)*IH(2)
      NITR(2) = ICODE/IH(1)
      ICODE   = ICODE-NITR(2)*IH(1)
      NITR(3) = ICODE

      RETURN
      END    


C******************************************************************************
C***                                                                        ***
C******************************************************************************

      SUBROUTINE MATRIX(KK,IMAT)

      INTEGER KK(0:3),IMAT(3,3)

      IF(KK(0).EQ.0) THEN
	IF(KK(1).EQ. 1) IMAT(1,1) =  1
	IF(KK(1).EQ. 2) IMAT(1,2) =  1
	IF(KK(1).EQ. 3) IMAT(1,3) =  1
	IF(KK(1).EQ.-1) IMAT(1,1) = -1
	IF(KK(1).EQ.-2) IMAT(1,2) = -1
	IF(KK(1).EQ.-3) IMAT(1,3) = -1
	IF(KK(2).EQ. 1) IMAT(2,1) =  1
	IF(KK(2).EQ. 2) IMAT(2,2) =  1
	IF(KK(2).EQ. 3) IMAT(2,3) =  1
	IF(KK(2).EQ.-1) IMAT(2,1) = -1
	IF(KK(2).EQ.-2) IMAT(2,2) = -1
	IF(KK(2).EQ.-3) IMAT(2,3) = -1
	IF(KK(3).EQ. 1) IMAT(3,1) =  1
	IF(KK(3).EQ. 2) IMAT(3,2) =  1
	IF(KK(3).EQ. 3) IMAT(3,3) =  1
	IF(KK(3).EQ.-1) IMAT(3,1) = -1
	IF(KK(3).EQ.-2) IMAT(3,2) = -1
	IF(KK(3).EQ.-3) IMAT(3,3) = -1
      ELSE
	IF(KK(3).EQ. 1) IMAT(3,1) =  1
	IF(KK(3).EQ. 2) IMAT(3,2) =  1
	IF(KK(3).EQ. 3) IMAT(3,3) =  1
	IF(KK(3).EQ.-1) IMAT(3,1) = -1
	IF(KK(3).EQ.-2) IMAT(3,2) = -1
	IF(KK(3).EQ.-3) IMAT(3,3) = -1
	IF(ABS(KK(0)).EQ.1) THEN
	  IF(KK(2).EQ. 1) IMAT(2,1) =  1
	  IF(KK(2).EQ. 2) IMAT(2,2) =  1
	  IF(KK(2).EQ. 3) IMAT(2,3) =  1
	  IF(KK(2).EQ.-1) IMAT(2,1) = -1
	  IF(KK(2).EQ.-2) IMAT(2,2) = -1
	  IF(KK(2).EQ.-3) IMAT(2,3) = -1
	  IF(KK(0).EQ.1) THEN
	    IMAT(1,1) =  1
	    IMAT(1,2) = -1
	  ELSE
	    IMAT(1,1) = -1
	    IMAT(1,2) =  1
	  END IF
	END IF
	IF(ABS(KK(0)).EQ.2) THEN
	  IF(KK(1).EQ. 1) IMAT(1,1) =  1
	  IF(KK(1).EQ. 2) IMAT(1,2) =  1
	  IF(KK(1).EQ. 3) IMAT(1,3) =  1
	  IF(KK(1).EQ.-1) IMAT(1,1) = -1
	  IF(KK(1).EQ.-2) IMAT(1,2) = -1
	  IF(KK(1).EQ.-3) IMAT(1,3) = -1
	  IF(KK(0).EQ.2) THEN
	    IMAT(2,1) =  1
	    IMAT(2,2) = -1
	  ELSE
	    IMAT(2,1) = -1
	    IMAT(2,2) =  1
	  END IF
	END IF
      END IF

      RETURN
      END

c******************************************************************************
c***                  Symmetrieelemente sortieren                           ***
c******************************************************************************

      subroutine symsort(nc,csym,symor,ism,tran,trp,trs,num,numsy,
     *                   nsym,NSY)

      parameter(MAX = 48, EPSY = 0.0001)
      real trp(3,NSY), tran(3,NSY), trs(3,NSY)
      character csym(NSY)*3, symor(NSY)*9, cm*4, so*9
      integer irot(3,3), ism(3,3,NSY)
      integer num(NSY), numsy(NSY)


c --- Nummer des Symmetrieelementes bestimmen:

      do 100 n=1,nsym
         do 50 k=1,MAX
            call rotmat(k,nc,cm,so,irot)
            ng = 0
            do 30 i=1,3
               do 20 j=1,3
                  if(ism(j,i,n).eq.irot(j,i)) ng = ng + 1
 20            continue
 30         continue
            if(ng.eq.9) goto 60
 50      continue
 60      if(ng.eq.9) then
           num(n) = k
           do 70 i=1,3
              trs(i,k) = tran(i,n)
 70        continue           
         else
           write(*,'(/2x,a,i2,a)') 
     *     'ERROR: Symmetry element ',n,' not identified'
           stop
         end if
 100  continue

c --- Symbol bestimmen und Rotationsmatrizen und Translationen nach oben 
c     gegebener Reihenfolge abspeichern:

      ns = 0
      do 200 k=1,MAX
         do 150 n=1,nsym
            if(num(n).eq.k) then

              ns = ns + 1
              numsy(ns) = k
              call rotmat(k,nc,cm,so,irot)
              do 130 i=1,3
                 tran(i,ns) = trs(i,k) 
                 do 120 j=1,3
                    ism(j,i,ns) = irot(j,i)
 120             continue
 130          continue
              csym(ns) = cm
              symor(ns) = so

c --- Pruefen, ob ein Symmetrieelement {ism|tran} phasenschiebend ist
c     und den phasenschiebenden Translationsanteil trp bestimmen
c     (npt gibt an, wie oft man das Symmetrieelement anwenden muss,
c     um wieder die Einheitsmatrix zu erhalten):

              call tranphs(trp(1,ns),npt,ism(1,1,ns),tran(1,ns))

c --- Test auf Gleitspiegelebene/Schraubenachse:

              if(abs(trp(1,ns)).gt.EPSY.or. 
     *           abs(trp(2,ns)).gt.EPSY.or.
     *           abs(trp(3,ns)).gt.EPSY) then
                call transbl(csym(ns),npt,trp(1,ns))
              end if

              goto 200

            end if
 150     continue
 200  continue


      return
      end


c******************************************************************************
c***                 Symmetrieelement bestimmen                             ***
c******************************************************************************

      subroutine rotmat(k,nc,csym,symor,ism)

      parameter(MAX = 48)
      character csym*3, symor*9, cm(MAX,2)*3, so(MAX,2)*9
      integer irot(3,3,MAX,2), ism(3,3)


c --- Rotationsmatrizen fuer das trikline, monokline, orthorhombische,
c     tetragonale, rhomboedrische und kubische System:

      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=1,8)
     *  / '  1',  '         ',   1, 0, 0,   0, 1, 0,   0, 0, 1,  
     *    ' 3+',  '  [1,1,1]',   0, 0, 1,   1, 0, 0,   0, 1, 0,
     *    ' 3+',  '[1,-1,-1]',   0, 0,-1,  -1, 0, 0,   0, 1, 0,
     *    ' 3+',  '[-1,1,-1]',   0, 0, 1,  -1, 0, 0,   0,-1, 0,
     *    ' 3+',  '[-1,-1,1]',   0, 0,-1,   1, 0, 0,   0,-1, 0,
     *    ' 3-',  '  [1,1,1]',   0, 1, 0,   0, 0, 1,   1, 0, 0,
     *    ' 3-',  '[1,-1,-1]',   0,-1, 0,   0, 0, 1,  -1, 0, 0,
     *    ' 3-',  '[-1,1,-1]',   0,-1, 0,   0, 0,-1,   1, 0, 0/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=9,16)
     *  / ' 3-',  '[-1,-1,1]',   0, 1, 0,   0, 0,-1,  -1, 0, 0,
     *    ' 4+',  '  [1,0,0]',   1, 0, 0,   0, 0,-1,   0, 1, 0,
     *    ' 4+',  '  [0,1,0]',   0, 0, 1,   0, 1, 0,  -1, 0, 0,
     *    ' 4+',  '  [0,0,1]',   0,-1, 0,   1, 0, 0,   0, 0, 1,
     *    ' 4-',  '  [1,0,0]',   1, 0, 0,   0, 0, 1,   0,-1, 0,
     *    ' 4-',  '  [0,1,0]',   0, 0,-1,   0, 1, 0,   1, 0, 0,
     *    ' 4-',  '  [0,0,1]',   0, 1, 0,  -1, 0, 0,   0, 0, 1,
     *    '  2',  '  [1,0,0]',   1, 0, 0,   0,-1, 0,   0, 0,-1/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=17,24)
     *  / '  2',  '  [0,1,0]',  -1, 0, 0,   0, 1, 0,   0, 0,-1,
     *    '  2',  '  [0,0,1]',  -1, 0, 0,   0,-1, 0,   0, 0, 1,
     *    '  2',  '  [1,1,0]',   0, 1, 0,   1, 0, 0,   0, 0,-1,
     *    '  2',  '  [1,0,1]',   0, 0, 1,   0,-1, 0,   1, 0, 0,
     *    '  2',  '  [0,1,1]',  -1, 0, 0,   0, 0, 1,   0, 1, 0,
     *    '  2',  ' [1,-1,0]',   0,-1, 0,  -1, 0, 0,   0, 0,-1,
     *    '  2',  ' [-1,1,0]',   0, 0,-1,   0,-1, 0,  -1, 0, 0,
     *    '  2',  ' [-1,0,1]',  -1, 0, 0,   0, 0,-1,   0,-1, 0/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=25,32)
     *  / ' 1b',  '         ',  -1, 0, 0,   0,-1, 0,   0, 0,-1,
     *    '3b+',  '  [1,1,1]',   0, 0,-1,  -1, 0, 0,   0,-1, 0,
     *    '3b+',  '[1,-1,-1]',   0, 0, 1,   1, 0, 0,   0,-1, 0,
     *    '3b+',  '[-1,1,-1]',   0, 0,-1,   1, 0, 0,   0, 1, 0,
     *    '3b+',  '[-1,-1,1]',   0, 0, 1,  -1, 0, 0,   0, 1, 0,
     *    '3b-',  '  [1,1,1]',   0,-1, 0,   0, 0,-1,  -1, 0, 0,
     *    '3b-',  '[1,-1,-1]',   0, 1, 0,   0, 0,-1,   1, 0, 0,
     *    '3b-',  '[-1,1,-1]',   0, 1, 0,   0, 0, 1,  -1, 0, 0/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=33,40)
     *  / '3b-',  '[-1,-1,1]',   0,-1, 0,   0, 0, 1,   1, 0, 0,
     *    '4b+',  '  [1,0,0]',  -1, 0, 0,   0, 0, 1,   0,-1, 0,
     *    '4b+',  '  [0,1,0]',   0, 0,-1,   0,-1, 0,   1, 0, 0,
     *    '4b+',  '  [0,0,1]',   0, 1, 0,  -1, 0, 0,   0, 0,-1,
     *    '4b-',  '  [1,0,0]',  -1, 0, 0,   0, 0,-1,   0, 1, 0,
     *    '4b-',  '  [0,1,0]',   0, 0, 1,   0,-1, 0,  -1, 0, 0,
     *    '4b-',  '  [0,0,1]',   0,-1, 0,   1, 0, 0,   0, 0,-1,
     *    '  m',  '  [1,0,0]',  -1, 0, 0,   0, 1, 0,   0, 0, 1/
      data (cm(n,1),so(n,1),((irot(j,i,n,1),i=1,3),j=1,3),n=41,48)
     *  / '  m',  '  [0,1,0]',   1, 0, 0,   0,-1, 0,   0, 0, 1,
     *    '  m',  '  [0,0,1]',   1, 0, 0,   0, 1, 0,   0, 0,-1,
     *    '  m',  '  [1,1,0]',   0,-1, 0,  -1, 0, 0,   0, 0, 1,
     *    '  m',  '  [1,0,1]',   0, 0,-1,   0, 1, 0,  -1, 0, 0,
     *    '  m',  '  [0,1,1]',   1, 0, 0,   0, 0,-1,   0,-1, 0,
     *    '  m',  ' [1,-1,0]',   0, 1, 0,   1, 0, 0,   0, 0, 1,
     *    '  m',  ' [-1,0,1]',   0, 0, 1,   0, 1, 0,   1, 0, 0,
     *    '  m',  ' [0,1,-1]',   1, 0, 0,   0, 0, 1,   0, 1, 0/


c --- Rotationmatrizen fuer das hexagonale System:

      data (cm(n,2),so(n,2),((irot(j,i,n,2),i=1,3),j=1,3),n=1,8)
     *  / '  1',  '         ',   1, 0, 0,   0, 1, 0,   0, 0, 1,
     *    ' 3+',  '  [0,0,1]',   0,-1, 0,   1,-1, 0,   0, 0, 1,
     *    ' 3-',  '  [0,0,1]',  -1, 1, 0,  -1, 0, 0,   0, 0, 1,
     *    ' 6+',  '  [0,0,1]',   1,-1, 0,   1, 0, 0,   0, 0, 1,
     *    ' 6-',  '  [0,0,1]',   0, 1, 0,  -1, 1, 0,   0, 0, 1, 
     *    '  2',  '  [0,0,1]',  -1, 0, 0,   0,-1, 0,   0, 0, 1,
     *    '  2',  '  [1,0,0]',   1,-1, 0,   0,-1, 0,   0, 0,-1,
     *    '  2',  '  [0,1,0]',  -1, 0, 0,  -1, 1, 0,   0, 0,-1/
      data (cm(n,2),so(n,2),((irot(j,i,n,2),i=1,3),j=1,3),n=9,16)
     *  / '  2',  '  [1,1,0]',   0, 1, 0,   1, 0, 0,   0, 0,-1,
     *    '  2',  ' [1,-1,0]',   0,-1, 0,  -1, 0, 0,   0, 0,-1,
     *    '  2',  '  [1,2,0]',  -1, 1, 0,   0, 1, 0,   0, 0,-1,
     *    '  2',  '  [2,1,0]',   1, 0, 0,   1,-1, 0,   0, 0,-1,
     *    ' 1b',  '         ',  -1, 0, 0,   0,-1, 0,   0, 0,-1,
     *    '3b+',  '  [0,0,1]',   0, 1, 0,  -1, 1, 0,   0, 0,-1,
     *    '3b-',  '  [0,0,1]',   1,-1, 0,   1, 0, 0,   0, 0,-1,
     *    '6b+',  '  [0,0,1]',  -1, 1, 0,  -1, 0, 0,   0, 0,-1/
      data (cm(n,2),so(n,2),((irot(j,i,n,2),i=1,3),j=1,3),n=17,24)
     *  / '6b-',  '  [0,0,1]',   0,-1, 0,   1,-1, 0,   0, 0,-1,
     *    '  m',  '  [0,0,1]',   1, 0, 0,   0, 1, 0,   0, 0,-1,
     *    '  m',  '  [1,0,0]',  -1, 1, 0,   0, 1, 0,   0, 0, 1,
     *    '  m',  '  [0,1,0]',   1, 0, 0,   1,-1, 0,   0, 0, 1,
     *    '  m',  '  [1,1,0]',   0,-1, 0,  -1, 0, 0,   0, 0, 1,
     *    '  m',  ' [1,-1,0]',   0, 1, 0,   1, 0, 0,   0, 0, 1,
     *    '  m',  '  [1,2,0]',   1,-1, 0,   0,-1, 0,   0, 0, 1,
     *    '  m',  '  [2,1,0]',  -1, 0, 0,  -1, 1, 0,   0, 0, 1/

      csym  = cm(k,nc)
      symor = so(k,nc)
      do 20 i=1,3
         do 10 j=1,3
            ism(j,i) = irot(j,i,k,nc)
 10      continue
 20   continue

      return
      end


c******************************************************************************
c***        Test auf Gleitspiegelebene/Schraubenachse                       ***
c******************************************************************************

      subroutine transbl(csym,npt,trp)

      parameter(NGP = 23, NSA = 16, EPSY = 0.0001)
      real trp(3), tgp(3,NGP), tsa(3,NSA)
      character csym*3, chr*1, gp(NGP)*3, sa(2,NSA)*3

c --- Gleitspiegelebenen:

      data (gp(k),(tgp(i,k),i=1,3),k=1,NGP)
     *   /'  a',  0.5,  0.,   0.,   '  b',  0.,   0.50, 0.,
     *    '  c',  0.,   0.,   0.50, '  n',  0.50, 0.50, 0.,
     *    '  n',  0.,   0.50, 0.50, '  n',  0.50, 0.  , 0.50,
     *    '  n',  0.50, 0.50, 0.50, '  n', -0.50, 0.50, 0.50,
     *    '  n',  0.50,-0.50, 0.50, '  n',  0.50, 0.50,-0.50,
     *    '  d',  0.25, 0.25, 0.,   '  d',  0.25,-0.25, 0.,
     *    '  d',  0.  , 0.25, 0.25, '  d',  0.  , 0.25,-0.25,
     *    '  d',  0.25, 0.  , 0.25, '  d', -0.25, 0.  , 0.25,
     *    '  d',  0.25, 0.25, 0.25, '  d', -0.25, 0.25, 0.25,
     *    '  d',  0.25,-0.25, 0.25, '  d',  0.25, 0.25,-0.25,
     *    '  d', -0.25,-0.25, 0.25, '  d', -0.25, 0.25,-0.25,
     *    '  d',  0.25,-0.25,-0.25/

c --- Schraubenachsen:

      data (sa(1,k),sa(2,k),(tsa(i,k),i=1,3),k=1,NSA)
     *   /'31+','32-', 0.,   0.,   0.33333333,
     *    '32+','31-', 0.,   0.,   0.66666667,
     *    '41+','43-', 0.,   0.,   0.25,
     *    '42+','42-', 0.,   0.,   0.50,
     *    '43+','41-', 0.,   0.,   0.75,
     *    '41+','43-', 0.,   0.25, 0.,
     *    '42+','42-', 0.,   0.50, 0.,
     *    '43+','41-', 0.,   0.75, 0.,
     *    '41+','43-', 0.25, 0.,   0.,
     *    '42+','42-', 0.50, 0.,   0.,
     *    '43+','41-', 0.75, 0.,   0.,
     *    '61+','65-', 0.,   0.,   0.16666667,
     *    '62+','64-', 0.,   0.,   0.33333333,
     *    '63+','63-', 0.,   0.,   0.50,
     *    '64+','62-', 0.,   0.,   0.66666667,
     *    '65+','61-', 0.,   0.,   0.83333333/     


c --- Test auf a,b,c,n,d-Gleitspiegelebene:

      if(index(csym,'m').gt.0) then

        do 245 l=1,NGP
           if(abs(trp(1)-tgp(1,l)).lt.EPSY.and.
     *        abs(trp(2)-tgp(2,l)).lt.EPSY.and.
     *        abs(trp(3)-tgp(3,l)).lt.EPSY) then
             csym = gp(l)
             goto 250
           end if
 245    continue
        do 247 j=1,3
           xx = npt * trp(j)
           if(abs(amod(xx,1.)).gt.EPSY) then
             csym = '  d'
             goto 250
           end if
 247    continue
 250    continue

c --- Test auf Schraubenachse: 

      else

        do 400 n=1,6
           write(chr,'(i1)') n
           na = index(csym,chr)
           if(na.gt.0) then
             if(n.eq.2) then
               csym = ' 21'
             else
               do 350 k=1,NSA
                  if(abs(trp(1)-tsa(1,k)).lt.EPSY.and.
     *               abs(trp(2)-tsa(2,k)).lt.EPSY.and.
     *               abs(trp(3)-tsa(3,k)).lt.EPSY.and.
     *               csym(2:2).eq.sa(1,k)(1:1)) then
                    if(csym(3:3).eq.'+') then
                      csym = sa(1,k)
                    else
                      csym = sa(2,k)
                    end if
                  end if
 350           continue
             end if
             goto 450
           end if
 400    continue
 450    continue

      end if

      return
      end
c******************************************************************************
c***  Koordinaten der Position x bezuegl. der prim. Basis berechnen         ***
c******************************************************************************

      subroutine transf(intaus,x,xp,itr)

      character intaus*1
      real x(3), xp(3), pm(3,3,7), qm(3,3,7) 

c --- P-Matrizen in der Reihenfolge: P, I, F, A, B, C, R
c     (Transformation von der zentrierten in die primitive Zelle)

      data pm / 1.0, 0.0, 0.0,    0.0, 1.0, 0.0,    0.0, 0.0, 1.0,
     *         -0.5, 0.5, 0.5,    0.5,-0.5, 0.5,    0.5, 0.5,-0.5,
     *          0.0, 0.5, 0.5,    0.5, 0.0, 0.5,    0.5, 0.5, 0.0,
     *          1.0, 0.0, 0.0,    0.0, 0.5, 0.5,    0.0,-0.5, 0.5,
     *          0.5, 0.0,-0.5,    0.0, 1.0, 0.0,    0.5, 0.0, 0.5,
     *          0.5, 0.5, 0.0,   -0.5, 0.5, 0.0,    0.0, 0.0, 1.0,
     *          0.6666666667,-0.3333333333,-0.3333333333,
     *          0.3333333333, 0.3333333333,-0.6666666667,
     *          0.3333333333, 0.3333333333, 0.3333333333 /

c --- Q-Matrizen (Q = P^(-1)) in der Reihenfolge: P, I, F, A, B, C, R
c     (Transformation von der primitiven in die zentrierte Zelle)

      data qm / 1.0, 0.0, 0.0,    0.0, 1.0, 0.0,    0.0, 0.0, 1.0,
     *          0.0, 1.0, 1.0,    1.0, 0.0, 1.0,    1.0, 1.0, 0.0,
     *         -1.0, 1.0, 1.0,    1.0,-1.0, 1.0,    1.0, 1.0,-1.0,
     *          1.0, 0.0, 0.0,    0.0, 1.0,-1.0,    0.0, 1.0, 1.0,
     *          1.0, 0.0, 1.0,    0.0, 1.0, 0.0,   -1.0, 0.0, 1.0,
     *          1.0,-1.0, 0.0,    1.0, 1.0, 0.0,    0.0, 0.0, 1.0,
     *          1.0, 0.0, 1.0,   -1.0, 1.0, 1.0,    0.0,-1.0, 1.0 /

      if(intaus.eq.'P') then
        nc = 1
      else if(intaus.eq.'I') then
        nc = 2
      else if(intaus.eq.'F') then
        nc = 3
      else if(intaus.eq.'A') then
        nc = 4
      else if(intaus.eq.'B') then
        nc = 5
      else if(intaus.eq.'C') then
        nc = 6
      else if(intaus.eq.'R'.or.intaus.eq.'H') then
        nc = 7
      end if


c --- Koordinaten bezuegl. der primitiven Basis erzeugen:

      if(itr.eq.0) then
        do 11 i=1,3
           xp(i) = 0.0
           do 10 j=1,3
              xp(i) = xp(i) + qm(j,i,nc) * x(j)    
10         continue
11      continue

c --- Koordinaten bezueglich der zentrierten Basis erzeugen:

      else
        do 21 i=1,3
           x(i) = 0.0
           do 20 j=1,3
              x(i) = x(i) + pm(j,i,nc) * xp(j)    
20         continue
21      continue
      end if

      return
      end
c******************************************************************************
c***  Bestimmung der Phasenschiebende Translation der Symmetrieelemente     ***
c******************************************************************************

      subroutine tranphs(trp,ns,ism,tran)

      parameter (EPSY = 0.0001)
      integer m(3,3), mp(3,3), me(3,3), ism(3,3)
      real t(3), tp(3), tran(3), trp(3)

      data me / 1,0,0,  0,1,0,  0,0,1 /


c --- Initialisierung:

      do 3 i=1,3
         trp(i) = 0.
 3    continue


      if(abs(tran(1)).gt.EPSY.or.abs(tran(2)).gt.EPSY.or.
     *   abs(tran(3)).gt.EPSY) then
         
c --- Die Matrix m und den Vektor t initialisieren:

        ns = 1
        do 5 i=1,3
           t(i) = tran(i)
           do 4 j=1,3
              m(i,j) = ism(i,j)
 4         continue
 5      continue

c --- Die Produktmatrix {mp|tp} = {ism|tran} * {m|t} berechnen:

 7      ns = ns + 1
        call symprd(ism,tran,m,t,mp,tp)

c --- Testen, ob erzeugte die Produktmatrix mp die Einheitsmatrix me ist
c     und ob die dazugehoerige Translation tp der Einheitsvektor ist.
c     Falls dies der Fall ist, ist das Symmetrieelement n nicht 
c     phasenschiebend: 

        iw = 0
        do 45 i=1,3
           do 44 j=1,3
              if(mp(i,j).ne.me(i,j)) iw = 1
44         continue
45      continue

        if(iw.eq.0) then

c --- Testen, ob nach ns-facher Anwendung des Symmetrieelementes {ism|tran}
c     der Translationsanteil tp von Null verschieden ist. Wenn ja, wird
c     der Translationsanteil pro Anwendung berechnet:

          if(abs(tp(1)).gt.EPSY.or.abs(tp(2)).gt.EPSY.or.
     *       abs(tp(3)).gt.EPSY) then
            do 46 i=1,3
               trp(i) = tp(i)/real(ns)
46          continue
          end if

        else

c --- Matrix m und den Vektor t auffuellen:

47        do 60 i=1,3
             t(i) = tp(i)
             do 50 j=1,3
                m(i,j) = mp(i,j)
50           continue
60        continue
          goto 7

        end if

      end if


      return
      end


c******************************************************************************
c***            Produkt zweier Symmetrieelemente                            ***
c******************************************************************************

c     {mc|tc} = {ma|ta} * {mb|tb} = {ma * mb| ma*tb + ta}

      subroutine symprd(ma,ta,mb,tb,mc,tc)

      integer ma(3,3), mb(3,3), mc(3,3)
      real ta(3), tb(3), tc(3)

      do 50 i=1,3
         tc(i) = ta(i)
         do 40 j=1,3
            tc(i) = tc(i) + ma(j,i) * tb(j)
            mc(j,i) = 0
            do 30 k=1,3
               mc(j,i) = mc(j,i) + ma(k,i) * mb(j,k)   
30          continue
40       continue
50    continue

      return
      end


c******************************************************************************
c***      Bei Filmaufnahmen konstanter Untergrund abziehen                  ***
c******************************************************************************

      subroutine uab(rau,obs,bac,izahl,IDZ)

      real obs(IDZ), bac(IDZ)

      do 100 n=1,izahl
         obs(n) = obs(n) - rau
         bac(n) = bac(n) - rau
100   continue

      return
      end



c******************************************************************************
c***                Eine Zeile ins INPUT-File schreiben                     ***
c******************************************************************************

      subroutine writline(ka,line,maxline)

      character line*132, form*6

      form = '(a000)'

      do 10 n=0,maxline-1
         if(line(maxline-n:maxline-n).ne.' ') then
           npos = n
           goto 12
         end if
10    continue
      npos = maxline - 1

12    write(form(3:5),'(i3)') maxline - npos
      write(ka,fmt=form) line

      return
      end


